using System;
using System.Security;
using System.Xml.Serialization;
using System.Net;
using System.Globalization;

namespace Microsoft.eResearch.GenePattern.Common
{
    /// <summary>
    /// Connection configuration properties for accessing a GenePattern server
    /// </summary>
    [Serializable]
    public class Connection : DomainBase
    {
        private string name;
        private string serverName;
        private string completeServerName;

        private string serverDnsName;
        private string username;
        private string password;
        private string serverVersion;
        private int port;
        private readonly string protocol = "http://";

        /// <summary>
        /// Initializes a new instance of the <see cref="Connection"/> class.
        /// </summary>
        public Connection() { }

        /// <summary>
        /// Initializes a new instance of the <see cref="Connection"/> class.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="serverName">Name of the server.</param>
        /// <param name="username">The username.</param>
        /// <param name="password">The password.</param>
        /// <param name="port">The port.</param>
        public Connection(string name, string serverName, string username,
            string password, int port)
        {
            this.name = name;
            this.serverName = serverName;
            this.serverDnsName = serverName;
            this.username = username;
            this.password = password;
            this.port = port;
            if (String.IsNullOrEmpty(completeServerName))
            {
                this.completeServerName = serverName;
            }
        }

        /// <summary>
        /// Constructs the URI.
        /// </summary>
        /// <returns></returns>
        private Uri ConstructURI()
        {
            GetDnsHostName();
            string uri = protocol + this.serverDnsName + (this.port > 0 ? ":" + this.port : string.Empty);
            return new Uri(uri);
        }

        /// <summary>
        /// Gets the name of the DNS host.
        /// </summary>
        private void GetDnsHostName()
        {
            if (string.Compare(this.serverDnsName, "localhost", true, CultureInfo.InvariantCulture) == 0)
            {
                this.serverDnsName = Dns.GetHostName();
            }
        }

        /// <summary>
        /// Constructs the short URI string.
        /// </summary>
        /// <returns></returns>
        private string ConstructShortURIString()
        {
            GetDnsHostName();
            return this.serverDnsName + (this.port > 0 ? ":" + this.port : string.Empty);
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        /// <summary>
        /// Gets or sets the name of the server.
        /// </summary>
        /// <value>The name of the server.</value>
        public string ServerName
        {
            get { return serverName; }
            set
            {
                serverName = value;
                serverDnsName = value;
            }
        }

        /// <summary>
        /// Gets or sets the name of the complete server.
        /// </summary>
        /// <value>The name of the complete server.</value>
        public string CompleteServerName
        {
            get { return completeServerName; }
            set { completeServerName = value; }
        }

        /// <summary>
        /// Gets or sets the username.
        /// </summary>
        /// <value>The username.</value>
        public string Username
        {
            get { return username; }
            set { username = value; }
        }

        /// <summary>
        /// Gets or sets the password.
        /// </summary>
        /// <value>The password.</value>
        public string Password
        {
            get { return password; }
            set { password = value; }
        }

        /// <summary>
        /// Gets or sets the port.
        /// </summary>
        /// <value>The port.</value>
        public int Port
        {
            get { return port; }
            set { port = value; }
        }

        /// <summary>
        /// Gets the server version.
        /// </summary>
        /// <value>The server version.</value>
        public string ServerVersion
        {
            get { return serverVersion; }
            set { serverVersion = value; }
        }

        /// <summary>
        /// Gets the server URI.
        /// </summary>
        /// <value>The server URI.</value>
        [DoNotClone]
        [DoNotCompare]
        [XmlIgnore]
        public Uri ServerUri
        {
            get { return ConstructURI(); }
        }

        /// <summary>
        /// Gets the server URI.
        /// </summary>
        /// <value>The server URI.</value>
        [DoNotCompare]
        [XmlIgnore]
        public string ShortServerInformation
        {
            get { return ConstructShortURIString(); }
        }

        /// <summary>
        /// Serves as a hash function for a particular type. <see cref="M:System.Object.GetHashCode"></see> is suitable for use in hashing algorithms and data structures like a hash table.
        /// </summary>
        /// <returns>
        /// A hash code for the current <see cref="T:System.Object"></see>.
        /// </returns>
        public override int GetHashCode()
        {
            int result = base.GetHashCode();
            result = 29*result + (serverName != null ? serverName.GetHashCode() : 0);
            result = 29*result + port;
            return result;
        }
    }
}
