using System;
using System.Collections.Generic;
using System.IO;

namespace Microsoft.eResearch.GenePattern.Common
{
   /// <summary>
   /// The services from GenePatter server accessible in the GenePattern package
   /// </summary>
    public interface IGenePatternServer
    {
        /// <summary>
        /// Gets the job by id.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <returns>the matching <c>Job</c></returns>
        Job GetJobById(int jobId);

        /// <summary>
        /// Gets or sets the connection.
        /// </summary>
        /// <value>The connection.</value>
        Connection Connection
        {
            get;
            set;
        }

        /// <summary>
        /// Gets the pipeline by lsid.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <returns>the matching <c>Pipeline</c> or null if no match</returns>
        Pipeline GetPipelineByLsid(string lsid);

        /// <summary>
        /// Gets the documentation.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <returns></returns>
        string[] GetDocumentation(string lsid);

        /// <summary>
        /// Downloads the pipeline.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <param name="recursive">Whether or not to download child modules as well</param>
        /// <returns>a <c>MemoryStream</c> hosting the pipeline zip file</returns>
        MemoryStream DownloadTask(string lsid, bool recursive);

        /// <summary>
        /// Determines whether the specified pipeline or module is installed.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <returns>
        /// 	<c>true</c> if a module or pipeline with the specified lsid is installed; otherwise, <c>false</c>.
        /// </returns>
        bool IsInstalled(string lsid);

        /// <summary>
        /// Runs the pipeline.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="parameters">The parameter list.</param>
        /// <param name="stopPoint">The stop point.</param>
        /// <returns></returns>
        int RunPipeline(Pipeline pipeline, IList<JobParameter> parameters, int stopPoint);

        /// <summary>
        /// Checks the job status.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <returns>a <c>JobStatus</c> if job existis otherwise null</returns>
        JobStatus CheckJobStatus(int jobId);

        /// <summary>
        /// Gets the job results.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <returns>an <c>ICollection</c> of <c>Result</c>s from the matching job</returns>
        IList<Result> GetJobResults(int jobId);


        /// <summary>
        /// Gets the job results.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <param name="specificFilename">The specific filename.</param>
        /// <returns></returns>
        IList<Result> GetJobResults(int jobId, string specificFilename);
        
        /// <summary>
        /// Gets the job results.
        /// </summary>
        /// <param name="results">The results.</param>
        /// <returns></returns>
        IList<Result> GetJobResults(IList<ShallowJobResult> results);

        /// <summary>
        /// Gets the job results.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <param name="pipelineConnection">The pipeline connection.</param>
        /// <returns></returns>
        IList<Result> GetJobResults(int jobId, Connection pipelineConnection);

        /// <summary>
        /// Gets the child job ids of job with the specified id.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <returns>an <c>int[]</c> with ids of the child jobs</returns>
        int[] GetChildren(int jobId);

        /// <summary>
        /// Registers for notification of job completion
        /// </summary>
        /// <param name="emailAddress">The email address.</param>
        /// <param name="jobId">The job id.</param>
        void RegisterForJobNotification(string emailAddress, int jobId);

        /// <summary>
        /// Tests the connection.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if connection was successful, otherwise false
        /// </returns>
        bool TestConnection();

        /// <summary>
        /// Gets the jobs by user of teh username of the current connection.
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        IList<Job> GetJobsByCurrentUser();

        /// <summary>
        /// Gets the name of the stop point parameter.
        /// </summary>
        /// <value>The name of the stop point parameter.</value>
        string StopPointParameterName
        {
            get;
        }

        /// <summary>
        /// Determines wheter the module with the specified lsid is a pipline
        /// </summary>
        /// <param name="lsid">the lsid of the module</param>
        bool IsPipeline(string lsid);


        /// <summary>
        /// Installs the pipeline.
        /// </summary>
        /// <param name="data">The data.</param>
        /// <returns>
        /// The lsid of the installed pipeline
        /// </returns>
        string InstallPipeline(byte[] data);


        /// <summary>
        /// Gets the input file.
        /// </summary>
        /// <param name="filename">The filename.</param>
        /// <returns>a <c>Stream</c> representing the file</returns>
        Stream GetInputFile(string filename);

        /// <summary>
        /// Gets the documentation file.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <param name="filename">The filename.</param>
        /// <returns>a <c>MemoryStream</c> representing the file</returns>
        MemoryStream GetDocumentationFile(string lsid, string filename);

        /// <summary>
        /// Gets the service info.
        /// </summary>
        /// <returns></returns>
        ServiceInfo GetServiceInfo {get;}

        /// <summary>
        /// Pings the server.
        /// </summary>
        /// <returns>returns true if connection and credentials are correct, otherwise false</returns>
        bool Ping();

        /// <summary>
        /// Gets all tasks.
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        List<Task> GetAllTasks{get;}

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources
        /// </summary>
        void Dispose();
    }
}
