using System;
using System.Collections.Generic;

namespace Microsoft.eResearch.GenePattern.Common
{

    /// <summary>
    /// A job represents a task that is sent to the server, identified by a pipeline Lsid an and a list of parameters
    /// The server returns an id to uniquely identify it, and then when it's done (checked by polling the server) a 
    /// list of result filenames
    /// </summary>
    public class Job : DomainBase
    {
        private int id;
        private string pipelineName;
        private string pipelineLsid;
        private int stopPoint;
        private JobStatus status;
        private List<JobParameter> parameters;
        private List<ShallowJobResult> results;
        private DateTime dateSubmitted;
        private DateTime dateCompleted;

        /// <summary>
        /// Initializes a new instance of the <see cref="Job"/> class.
        /// </summary>
        public Job()
        {
            parameters = new List<JobParameter>();
            results = new List<ShallowJobResult>();
            id = -1;
        }


        /// <summary>
        /// Gets or sets the id.
        /// </summary>
        /// <value>The id.</value>
        public int Id
        {
            get { return id; }
            set { id = value; }
        }

        /// <summary>
        /// Gets or sets the name of the pipeline.
        /// </summary>
        /// <value>The name of the pipeline.</value>
        public string PipelineName
        {
            get { return pipelineName; }
            set { pipelineName = value; }
        }

        /// <summary>
        /// Gets or sets the pipeline lsid.
        /// </summary>
        /// <value>The pipeline lsid.</value>
        public string PipelineLsid
        {
            get { return pipelineLsid; }
            set { pipelineLsid = value; }
        }


        /// <summary>
        /// Gets or sets the zero-index based stop point.
        /// </summary>
        /// <value>The stop point.</value>
        public int StopPoint
        {
            get { return stopPoint; }
            set { stopPoint = value; }
        }

        /// <summary>
        /// Gets or sets the status.
        /// </summary>
        /// <value>The status.</value>
        public JobStatus Status
        {
            get { return status; }
            set { status = value; }
        }

        /// <summary>
        /// Gets the parameters.
        /// </summary>
        /// <value>The parameters.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<JobParameter> Parameters
        {
            get { return parameters; }
            set { parameters = value; }
        }

        /// <summary>
        /// Gets the result filenames.
        /// </summary>
        /// <value>The result filenames.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<ShallowJobResult> ShallowResults
        {
            get { return results; }
            set { results = value; }
        }

        /// <summary>
        /// Gets or sets the date submitted.
        /// </summary>
        /// <value>The date submitted.</value>
        public DateTime DateSubmitted
        {
            get
            {
                return dateSubmitted;
            }
            set
            {
                dateSubmitted = value;
            }
        }

        /// <summary>
        /// Gets or sets the date completed.
        /// </summary>
        /// <value>The date completed.</value>
        public DateTime DateCompleted
        {
            get
            {
                return dateCompleted;
            }
            set
            {
                dateCompleted = value;
            }
        }

        /// <summary>
        /// Gets a parameter by name
        /// </summary>
        /// <param name="name">The name.</param>
        /// <returns></returns>
        public JobParameter GetParameter(string name)
        {
            foreach (JobParameter parameter in parameters)
            {
                if (parameter.Name == name)
                {
                    return parameter;
                }
            }
            return null;
        }

        public List<string> FileNames
        {
            get
            {
                List<string> result = new List<string>();
                foreach (ShallowJobResult res in ShallowResults)
                {
                    result.Add(res.Filename);
                }
                return result;
            }
        }

        /// <summary>
        /// Checks to see if a rerun actually made it to at least one Visualizer
        /// </summary>
        /// <param name="pipeline"></param>
        /// <returns></returns>
        public bool HasReachedVisualizers(Pipeline pipeline)
        {
            // see if any of the visualizer steps were reached on the run
            for (int i = 0; i <= stopPoint; i++)
            {
                if (pipeline.Steps[i].VisualizerData.Count > 0)
                {
                    return true;
                }
            }
            return false;
        }
    }
}
