using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Xml.Serialization;
using Microsoft.eResearch.ScientificAddIn.Common;

namespace Microsoft.eResearch.GenePattern.Common
{
    /// <summary>
    /// Represents the Data Model of the Gene Pattern Components.
    /// </summary>
    [XmlRoot("Model")]
    public class Model : DomainBase
    {
        private SerializableDictionary<int, PipelineAssociation> pipelineAssociations = new SerializableDictionary<int, PipelineAssociation>();
        private SerializableDictionary<string, Pipeline> pipelines = new SerializableDictionary<string, Pipeline>();
        private SerializableDictionary<string, Module> modules = new SerializableDictionary<string, Module>();
        private int autoNumberKey = 1;
        private bool transactionActive;
        private Model previousVersion;


        /// <summary>
        /// Gets or sets the _ auto number key. ONLY USED FOR XML SERIALIZATION!
        /// </summary>
        /// <value>The _ auto number key.</value>
        public int AutoNumberKey
        {
            get { return autoNumberKey; }
            set { autoNumberKey = value; }
        }

        /// <summary>
        /// Gets or sets the pipeline associations.
        /// </summary>
        /// <value>The pipeline associations.</value>
        [CompareAsDictionary]
        [CloneAsDictionary]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly")]
        public SerializableDictionary<int, PipelineAssociation> PipelineAssociations
        {
            get { return pipelineAssociations; }
            set { pipelineAssociations = value; }
        }

        /// <summary>
        /// Gets or sets the pipelines.
        /// </summary>
        /// <value>The pipelines.</value>
        [CompareAsDictionary]
        [CloneAsDictionary]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly")]
        public SerializableDictionary<string, Pipeline> Pipelines
        {
            get { return pipelines; }
            set { pipelines = value; }
        }

        /// <summary>
        /// Gets or sets the modules.
        /// </summary>
        /// <value>The modules.</value>
        [CompareAsDictionary]
        [CloneAsDictionary]
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly")]
        public SerializableDictionary<string, Module> Modules
        {
            get { return modules; }
            set { modules = value; }
        }
        /// <summary>
        /// Gets a value indicating whether there is an active transaction
        /// </summary>
        /// <value><c>true</c> if a transaction active; otherwise, <c>false</c>.</value>
        [XmlIgnore]
        [DoNotClone]
        [DoNotCompare]
        public bool TransactionActive
        {
            get { return transactionActive; }
        }
        /// <summary>
        /// Begin a transaction (changes to the the model) that can be rolled back
        /// </summary>
        public void BeginTransaction()
        {
            // copy yourself so you have something to rollback to
            previousVersion = (Model)this.Clone();
            transactionActive = true;
        }

        public SerializableDictionary<int, PipelineAssociation> GetPrevPipelineAssociations()
        {
            if (previousVersion != null)
                return previousVersion.PipelineAssociations;
            return null;
        }

        /// <summary>
        /// Ends the transaction a returns the previous model.  Assign this to your reference.  
        /// </summary>
        /// <returns></returns>
        public Model RollbackTransaction()
        {
            // we can't modify _this_ so we have to return what should be assigned to this.
            Debug.Assert(previousVersion != null);
            if (previousVersion != null)
            {
                previousVersion.transactionActive = false;
                return previousVersion;
            }
            else
            {
                return this;
            }
        }
        /// <summary>
        /// Finish a transaction and save changes
        /// </summary>
        public void CommitTransaction()
        {
            transactionActive = false;
            previousVersion = null;
        }


        /// <summary>
        /// Creates the pipeline.
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        public Pipeline CreatePipeline()
        {
            Pipeline p = new Pipeline();
            return p;
        }


        /// <summary>
        /// Creates the pipeline association.
        /// </summary>
        /// <returns></returns>
        public PipelineAssociation CreatePipelineAssociation()
        {
            PipelineAssociation p = new PipelineAssociation();
            p.LocalId = autoNumberKey;
            autoNumberKey++;
            return p;
        }

        /// <summary>
        /// Adds the pipeline.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        public void AddPipeline(Pipeline pipeline)
        {
            if (!pipelines.ContainsKey(pipeline.Lsid))
            {
                pipelines.Add(pipeline.Lsid, pipeline);
            }
        }

        /// <summary>
        /// Deletes the pipeline association.
        /// </summary>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        public void DeletePipelineAssociation(PipelineAssociation pipelineAssociation)
        {
            if (pipelineAssociations.ContainsKey(pipelineAssociation.LocalId))
            {
                if (pipelineAssociations.Remove(pipelineAssociation.LocalId))
                {
                    bool doDeletePipeline = true;
                    string lsid = pipelineAssociation.PipelineLsid;
                    foreach (PipelineAssociation association in pipelineAssociations.Values)
                    {
                        if (lsid == association.PipelineLsid)
                        {
                            doDeletePipeline = false;
                            break;
                        }
                    }
                    if (doDeletePipeline)
                    {
                        if (pipelines.ContainsKey(lsid))
                        {
                            pipelines.Remove(lsid);
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Adds the pipeline association.
        /// </summary>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        public void AddPipelineAssociation(PipelineAssociation pipelineAssociation)
        {
            if (pipelineAssociation.LocalId <= 0)
            {
                pipelineAssociation.LocalId = autoNumberKey;
                autoNumberKey++;
            }
            if (!pipelineAssociations.ContainsKey(pipelineAssociation.LocalId))
            {
                pipelineAssociations.Add(pipelineAssociation.LocalId, pipelineAssociation);
            }
        }

        /// <summary>
        /// Adds the module.
        /// </summary>
        /// <param name="module">The module.</param>
        public void AddModule(Module module)
        {
            if (!modules.ContainsKey(module.Lsid))
            {
                modules.Add(module.Lsid, module);
            }
        }
    }
}
