using System;
using System.Collections;
using System.Collections.Generic;
using System.Xml.Serialization;
using Microsoft.eResearch.ScientificAddIn.Common;

namespace Microsoft.eResearch.GenePattern.Common
{
    /// <summary>
    /// The document-side representation of a pipeline on the server.  contains datafiles, a manifest file, and a collection
    /// of child pipeline steps that can be other pipelines or modules.
    /// uniquely identified by an lsid.
    /// </summary>
    [XmlRoot("Pipeline")]
    public class Pipeline : DomainBase
    {
        private string lsid;
        private List<PipelineStep> steps;
        private List<string> documentationFilenames;
        private List<InternalFile> documentation;
        private InternalFile manifestFile;
        private bool isLinked;
        private List<InternalFile> datafiles;
        private string name;
        private List<PipelineParameter> parameters;
        private Connection linkConnection;
       

        /// <summary>
        /// Initializes a new instance of the <see cref="Pipeline"/> class.
        /// </summary>
        public Pipeline()
        {
            datafiles = new List<InternalFile>();
            parameters = new List<PipelineParameter>();
            steps = new List<PipelineStep>();
            documentationFilenames = new List<string>();
            documentation = new List<InternalFile>();
           
        }

        /// <summary>
        /// Gets or sets the lsid.
        /// </summary>
        /// <value>The lsid.</value>
        [XmlElement("Lsid")]
        public string Lsid
        {
            get { return lsid; }
            set { lsid = value; }
        }

        /// <summary>
        /// Gets the steps.
        /// </summary>
        /// <value>The steps.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<PipelineStep> Steps
        {
            get { return steps; }
            set { steps = value; }
        }

        /// <summary>
        /// Gets or sets the documentation.
        /// </summary>
        /// <value>The documentation.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists"), XmlElement("Documentation")]
        public List<InternalFile> Documentation
        {
            get { return documentation; }
            set { documentation = value; }
        }

        /// <summary>
        /// Gets or sets the manifest file.
        /// </summary>
        /// <value>The manifest file.</value>
        public InternalFile ManifestFile
        {
            get { return manifestFile; }
            set { manifestFile = value; }
        }

        /// <summary>
        /// Gets the datafiles.
        /// </summary>
        /// <value>The datafiles.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<InternalFile> Datafiles
        {
            get { return datafiles; }
            set { datafiles = value; }
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
        [XmlElement("Name")]
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        /// <summary>
        /// Gets the parameters.
        /// </summary>
        /// <value>The parameters.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<PipelineParameter> Parameters
        {
            get { return parameters; }
            set { parameters = value;  }
        }

        /// <summary>
        /// Gets the documentation filenames.
        /// </summary>
        /// <value>The documentation filenames.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<string> DocumentationFilenames
        {
            get { return documentationFilenames; }
            set { documentationFilenames = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is linked.
        /// </summary>
        /// <value><c>true</c> if this instance is linked; otherwise, <c>false</c>.</value>
        public bool IsLinked
        {
            get { return isLinked; }
            set { isLinked = value; }
        }

        /// <summary>
        /// Gets or sets the link connection.
        /// </summary>
        /// <value>The link connection.</value>
        public Connection LinkConnection
        {
            get { return linkConnection; }
            set { linkConnection = value; }
        }

        /// <summary>
        /// Gets the parameter.
        /// </summary>
        /// <param name="parameterName">Name of the parameter.</param>
        /// <returns>The pipeline parameter</returns>
        public PipelineParameter GetParameter(string parameterName)
        {
            foreach (PipelineParameter parameter in parameters)
            {
                if (parameter.Name == parameterName)
                {
                    return parameter;
                }
            }
            return null;
        }
       
        [DoNotClone]
        [XmlIgnore]
        public bool HasVisualizers
        {
            get
            {
                bool visualizers = false;
                foreach(PipelineStep step in steps)
                {
                    if(step.VisualizerData.Count > 0)
                    {
                        visualizers = true;
                        break;
                    }
                }
                return visualizers;
            }
        }
    }
}
