using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Xml.Serialization;
using Microsoft.eResearch.ScientificAddIn.Common;

namespace Microsoft.eResearch.GenePattern.Common
{
    /// <summary>
    /// The link between a document reference and the corresponding pipeline on the server.  Could be linked to an image
    /// or a table or link in the document.  Parameters indicate the parameters that were used to run the pipeline, along with
    /// a PrimaryResult.
    /// If the pipeline is rerun from this association, that run is stored in the rerun object.
    /// </summary>
    [XmlRoot("PipelineAssociation")]
    public class PipelineAssociation : DomainBase
    {
        private int localId;
        private string pipelineLsid;
        private DocumentImage image;
        private int userStopPoint;
        private Job job;
        private string primaryResultName;
        private string annotation;
        private string name;
        private Rerun rerun;
        private List<InternalFile> embeddedResults = new List<InternalFile>();
        private Image thumbnail;
        private List<JobParameter> originalRerunParameters = new List<JobParameter>();

        [XmlIgnore]
        [DoNotCompare]
        public List<JobParameter> OriginalRerunParameters
        {
            get { return originalRerunParameters; }
            set { originalRerunParameters = value; }
        }

        [XmlIgnore]
        [DoNotCompare]
        public Image ThumbnailImage
        {
            get { return thumbnail; }
            set { thumbnail = value; }
        }

        [DoNotCompare]
        public byte[] SerializedThumbnailImage
        {
            get
            {
                if (thumbnail == null)
                {
                    return null;
                }
                else
                {
                    using (MemoryStream stream = new MemoryStream())
                    {
                        BinaryFormatter bf = new BinaryFormatter();
                        bf.Serialize(stream, thumbnail);

                        bf = null;
                        byte[] serializedThumbnailImage = stream.ToArray();
                        return serializedThumbnailImage;
                    }
                }

            }
            set
            {
                if (value != null)
                {
                    using (MemoryStream imageStream = new MemoryStream(value))
                    {
                        BinaryFormatter bf = new BinaryFormatter();
                        thumbnail = (Image)bf.Deserialize(imageStream);
                        bf = null;
                    }
                }
                else
                {
                    thumbnail = null;
                }
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="PipelineAssociation"/> class.
        /// </summary>
        public PipelineAssociation()
        {
            job = new Job();
            userStopPoint = -1;
            rerun = new Rerun();
            localId = -1;
        }

        /// <summary>
        /// Gets or sets the pipeline.
        /// </summary>
        /// <value>The pipeline.</value>
        public string PipelineLsid
        {
            get { return pipelineLsid; }
            set { pipelineLsid = value; }
        }

        /// <summary>
        /// Gets or sets the image.
        /// </summary>
        /// <value>The image.</value>
        public DocumentImage PipelineDocumentImage
        {
            get { return image; }
            set { image = value; }
        }

        /// <summary>
        /// Gets or sets the stop point.
        /// </summary>
        /// <value>The stop point.</value>
        public int UserStopPoint
        {
            get { return userStopPoint; }
            set { userStopPoint = value; }
        }


        /// <summary>
        /// Gets or sets the primary result.
        /// </summary>
        /// <value>The primary result.</value>
        public string PrimaryResultName
        {
            get { return primaryResultName; }
            set { primaryResultName = value; }
        }

        /// <summary>
        /// Gets or sets the annotation.
        /// </summary>
        /// <value>The annotation.</value>
        public string Annotation
        {
            get { return annotation; }
            set { annotation = value; }
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        /// <summary>
        /// Gets or sets the local id.
        /// </summary>
        /// <value>The local id.</value>
        [DoNotCompare]
        public int LocalId
        {
            get { return localId; }
            set { localId = value; }
        }


        /// <summary>
        /// Gets or sets the job.
        /// </summary>
        /// <value>The job.</value>
        public Job Job
        {
            get { return job; }
            set { job = value; }
        }

        [DoNotCompare]
        [XmlIgnore]
        public Rerun Rerun
        {
            get { return rerun; }
            set { rerun = value; }
        }

        public List<InternalFile> EmbeddedResults
        {
            get { return embeddedResults; }
            set { embeddedResults = value; }
        }

        /// <summary>
        /// Sets some things that shouldn't have been cloned back to their base state.  Req'd because we also do a full clone sometimes
        /// </summary>
        public void MakeShallowClone()
        {
            localId = -1;
            rerun = new Rerun();
            ThumbnailImage = null;
        }
    }
}
