using System.Collections.Generic;
using System.Globalization;

namespace Microsoft.eResearch.GenePattern.Common
{
    /// <summary>
    /// Enum for Pipeline Step Type
    /// </summary>
    public enum PipelineStepType
    {
        /// <summary>
        /// Pipeline Step
        /// </summary>
        Pipeline,

        /// <summary>
        /// Module Step
        /// </summary>
        Module,

        /// <summary>
        /// Visualizer Step
        /// </summary>
        Visualizer
    }
    
    
    /// <summary>
    /// Class to Store Visualizer Data
    /// </summary>
    public class VisualizerData : DomainBase
    {
        private int sourceFileStep;
        private int sourceFileIndex;
        private string sourceFileExtension; 
        private bool useExtension;
        private string filenameVariable;

        /// <summary>
        /// Gets or sets the source file step.
        /// </summary>
        /// <value>The source file step.</value>
       public int SourceFileStep
       {
          get { return sourceFileStep; }
          set { sourceFileStep = value; }
       }

       /// <summary>
       /// Gets or sets the index of the source file.
       /// </summary>
       /// <value>The index of the source file.</value>
       public int SourceFileIndex
       {
          get { return sourceFileIndex; }
          set { sourceFileIndex = value; }
       }

       /// <summary>
       /// Gets or sets the source file extension.
       /// </summary>
       /// <value>The source file extension.</value>
       public string SourceFileExtension
       {
          get { return sourceFileExtension; }
          set { sourceFileExtension = value; }
       }

       /// <summary>
       /// Gets or sets a value indicating whether [use extension].
       /// </summary>
       /// <value><c>true</c> if [use extension]; otherwise, <c>false</c>.</value>
       public bool UseExtension
       {
          get { return useExtension; }
          set { useExtension = value; }
       }

       /// <summary>
       /// Gets or sets the filename variable.
       /// </summary>
       /// <value>The filename variable.</value>
       public string FilenameVariable
       {
          get { return filenameVariable; }
          set { filenameVariable = value; }
       }
    }
    /// <summary>
    /// A pipeline building block.  This can either be a module or another pipeline.  This is indicated by the isPipeline
    /// value.   Both pipelines and modules implement the ITask interface and can be stored as content.
    /// </summary>
    public class PipelineStep : DomainBase
    {
        private string lsid;
        private PipelineStepType type;
        private string name;
        private List<VisualizerData> visualizerData;

        /// <summary>
        /// Initializes a new instance of the <see cref="PipelineStep"/> class.
        /// </summary>
        public PipelineStep()
        {
            visualizerData = new List<VisualizerData>();
        }

        /// <summary>
        /// Gets or sets the lsid.
        /// </summary>
        /// <value>The lsid.</value>
        public string Lsid
        {
            get { return lsid; }
            set { lsid = value; }
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        /// <summary>
        /// Gets or sets the type
        /// </summary>
        public PipelineStepType Type
        {
            get { return type; }
            set { type = value; }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is pipeline.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is pipeline; otherwise, <c>false</c>.
        /// </value>
        [DoNotClone]
        public bool IsPipeline
        {
            get { return type == PipelineStepType.Pipeline; }   
        }

        /// <summary>
        /// Gets the visualizer data.
        /// </summary>
        /// <value>The visualizer data.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<VisualizerData> VisualizerData
        {
            get { return visualizerData; }
        }

       /// <summary>
        /// Returns results needed for the visualizer for the given result, or null if no visualizer exists.
        /// </summary>
        /// <param name="results"></param>
        /// <returns></returns>
        public IList<Result> GetVisualizerSourceResults(IList<Result> results)
        {
            IList<Result> resultsUsed = new List<Result>();
            foreach(VisualizerData vizdata in VisualizerData)
            {
                foreach (Result selResult in results)
                {
                    int period = selResult.Filename.LastIndexOf('.');
                    string extension = "";
                    if (period > 0)
                    {
                        extension = selResult.Filename.Substring(period + 1).ToLower(CultureInfo.InvariantCulture);
                    }
                    if (vizdata.SourceFileStep == selResult.SourceStep)
                    {
                       if (vizdata.UseExtension)
                        {
                            if (vizdata.SourceFileExtension.ToLower(CultureInfo.InvariantCulture) == extension)
                            {
                                resultsUsed.Add(selResult);
                            }
                        }
                        else if (vizdata.SourceFileIndex == selResult.OutputIndex)
                        {
                            resultsUsed.Add(selResult);
                        }
                    }
                }
            }
            return resultsUsed;
        }

        /// <summary>
        /// Gets the visualizer filenames.
        /// </summary>
        /// <param name="results">The results.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<ShallowJobResult> GetVisualizerFilenames(List<ShallowJobResult> results)
        {
           List<ShallowJobResult> resultsUsed = new List<ShallowJobResult>();
           foreach (VisualizerData vizdata in VisualizerData)
           {
              foreach (ShallowJobResult selResult in results)
              {
                 int period = selResult.Filename.LastIndexOf('.');
                 string extension = "";
                 if (period > 0)
                 {
                     extension = selResult.Filename.Substring(period + 1).ToLower(CultureInfo.InvariantCulture);
                 }
                 if (vizdata.SourceFileStep == selResult.SourceStep)
                 {
                    if (vizdata.UseExtension)
                    {
                        if (vizdata.SourceFileExtension.ToLower(CultureInfo.InvariantCulture) == extension)
                       {
                          resultsUsed.Add(selResult);
                       }
                    }
                    else if (vizdata.SourceFileIndex == selResult.SourceIndex)
                    {
                       resultsUsed.Add(selResult);
                    }
                 }
              }
           }
           return resultsUsed;
        }
     }
}
