using System;
using System.Collections.Generic;
using System.Globalization;
using System.Threading;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using System.Drawing;
using Microsoft.eResearch.ScientificAddIn.Common;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// Wizard for associating a PipelineName
    /// </summary>
    public partial class AssociatePipelineWizard : Form, IWizard
    {
        private int currentStep;
        private SelectJobFromServer selectJobFromServer;
        private IntroPage introPage;
        private ShowParameters showParams;
        private SelectStopPoint selectStopPoint;
        private SelectPrimaryResult selectPrimaryResult;
        private AddAnnotation addAnnotation;
        private ViewPipelines viewPipelines;
        private AssociatePipelineWizardModel model;
        private bool isEditing;
        private readonly IController controller;
        private const int MAX_STEPS = 7;
        private Bitmap[] stepImages;
        private bool workComplete;
        private bool abort;
        // flag for checking the existance of association name      
        private bool isNameExisting;
        private string EditPipelineName;

        /// <summary>
        /// Initializes a new instance of the <see cref="AssociatePipelineWizard"/> class.
        /// </summary>
        /// <param name="controller">The controller.</param>
        /// <param name="showLinkedImageMessage">if set to <c>true</c> show GP link image message.</param>
        public AssociatePipelineWizard(IController controller, bool showLinkedImageMessage)
        {
            if (controller == null)
                throw new ArgumentNullException("controller");

            InitializeComponent();

            bool showWizardIntroPage = controller.UserData.ShowAssociationWizardIntroPage;

            model = new AssociatePipelineWizardModel();
            model.AddEmptyData();
            model.PipelineAssociation.Annotation = Properties.Settings.Default.annotationTemplate;
            selectJobFromServer = new SelectJobFromServer(controller.GetGenePatternServer(), model, controller);
            introPage = new IntroPage();
            if (showLinkedImageMessage)
            {
                //TODO resource it
                introPage.Header = "This guide will help you in inserting a pipeline association into the document.";
            }
            else
            {
                //TODO resource it
                introPage.Header = "This guide will help you in associating a pipeline with the image.";
            }
            introPage.ShowIntroPage = showWizardIntroPage;
            introPage.ShowIntroPageChanged += new EventHandler<EventArgs>(introPage_ShowIntroPageChanged);
            showParams = new ShowParameters(model);
            selectStopPoint = new SelectStopPoint(model);
            selectPrimaryResult = new SelectPrimaryResult(model);
            addAnnotation = new AddAnnotation(model);
            viewPipelines = new ViewPipelines(model);
            this.controller = controller;

            stepImages = new Bitmap[] { Resources.Associate_Pipeline_Step1,
                                        Resources.Associate_Pipeline_Step2,
                                        Resources.Associate_Pipeline_Step3,
                                        Resources.Associate_Pipeline_Step4,
                                        Resources.Associate_Pipeline_Step5,
                                        Resources.Associate_Pipeline_Step6,
                                        Resources.Associate_Pipeline_Step7};
            SetImages();

            currentStep = showWizardIntroPage ? 0 : 1;
        }

        /// <summary>
        /// The get method for isEditing flag
        /// </summary>
        public bool IsEditing
        {
            get
            {
                return isEditing;
            }
        }

        /// <summary>
        /// The associations created by the pipeline wizard
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<PipelineAssociation> PipelineAssociations
        {
            get { return model.PipelineAssociations; }
        }

        void introPage_ShowIntroPageChanged(object sender, EventArgs e)
        {
            controller.UserData.ShowAssociationWizardIntroPage = introPage.ShowIntroPage;
            UserDataManager.Save(controller.UserData);
        }

        /// <summary>
        /// Sets the images.
        /// </summary>
        private void SetImages()
        {
            this.BackgroundImage = Resources.Wizard_background;
            this.BackgroundImageLayout = ImageLayout.Stretch;
            TitleImagePictureBox.Image = Resources.AssociateWizard_icon;
            TitlePictureBox.Image = Resources.Associate_Wizard_Title;

            SetCurrentStepIndicationImage(currentStep);
        }

        /// <summary>
        /// Sets the current step indication image.
        /// </summary>
        /// <param name="step">The step.</param>
        private void SetCurrentStepIndicationImage(int step)
        {
            CurrentStepIndicationPanel.BackgroundImageLayout = ImageLayout.Stretch;
            CurrentStepIndicationPanel.BackgroundImage = stepImages[step];
        }

        /// <summary>
        /// Edits the existing pipeline.
        /// </summary>
        /// <param name="pa">The pipeline association</param>
        public void EditExistingPipeline(PipelineAssociation pa)
        {
            model.Pipeline = controller.ActiveModel.Pipelines[pa.PipelineLsid];
            model.PipelineAssociation = pa;
            isEditing = true;
            viewPipelines.ShowEditingTitle();
            Text = "Edit Pipeline Association";
            this.Name = "Edit Pipeline Association";
            EditPipelineName = pa.Name;
            currentStep = 2; // skip intro
        }


        /// <summary>
        /// Updates the call.
        /// </summary>
        /// <param name="delta">The delta.</param>
        public void UpdateCall(int delta)
        {
            if (isEditing && delta == 0)
            {
                controller.BeginModelChanges();
            }

            WizardBaseControl oldControl = GetControlForStep(currentStep);
            if (oldControl.CanHide() || delta <= 0) // can always go backwards
            {
                currentStep += delta;
                WizardBaseControl newControl = GetControlForStep(currentStep);
                // note short circuit eval prevents onHide from being called sometimes
                bool successfullyHidden = delta == 0 || oldControl.OnHide(delta);
                if (successfullyHidden)
                {
                    ShowCurrentStep();
                    // In the ShowPipelineAssociations() function [ViewPipeline.cs]
                    // we need to check whether there is any existing association
                    // having the same name as the name given by the user for
                    // the pipeline that is currently trying to associate.
                    // for this we are using the IsAssociationNameExist(..)
                    // function of this class, so we are passing refernce to this
                    // object to the control boject.
                    newControl.SetParent(this);
                    newControl.OnShown();
                }
                else
                {
                    // rollback
                    newControl = oldControl;
                    currentStep -= delta;
                }


                if (newControl.CanHide())
                {
                    this.Next.Enabled = currentStep < MAX_STEPS - 1;
                }
                else
                {
                    Next.Enabled = false;
                }
                this.Previous.Enabled = currentStep > 0;
            }
        }

        /// <summary>
        /// Shows the current step.
        /// </summary>
        private void ShowCurrentStep()
        {
            UserControl control = GetControlForStep(currentStep);
            control.Dock = DockStyle.Fill;
            StepPanel.Controls.Clear();
            StepPanel.Controls.Add(control);
            control.Focus();
            SetCurrentStepIndicationImage(currentStep);
            UpdateFinishButton();
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
        private WizardBaseControl GetControlForStep(int step)
        {
            switch (step)
            {
                case 0:
                    return introPage;
                //case 1:
                //    return selectPipelineSource;
                //    break;
                case 1:
                    return selectJobFromServer;
                case 2:
                    return showParams;
                case 3:
                    return selectStopPoint;
                case 4:
                    return selectPrimaryResult;
                case 5:
                    return addAnnotation;
                case 6:
                    return viewPipelines;
                default:
                    throw new IndexOutOfRangeException();
            }
        }

        /// <summary>
        /// Handles the Load event of the AssociatePipelineWizard control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void AssociatePipelineWizard_Load(object sender, EventArgs e)
        {
            UpdateCall(0);
        }

        /// <summary>
        /// Determines whether [is association name exist] [the specified asso pipeline name].
        /// </summary>
        /// <param name="associationPipelineName">Name of the asso pipeline.</param>
        /// <returns>
        /// 	<c>true</c> if [is association name exist] [the specified asso pipeline name]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsAssociationNameExist(String associationPipelineName)
        {
            bool AlreadyExistingName = false;
            bool prev = false;
            int count = model.PipelineAssociations.Count;

            // User can associate multiple pipeline by clicking 'Add Another'
            // option in the final step od Association. Check whether the user selected
            // same name for multiple associations at this time.
            for (int index1 = 0; index1 < count; index1++)
            {
                for (int index2 = index1 + 1; index2 < count; index2++)
                {
                    if (model.PipelineAssociations[index1].Name.Equals(model.PipelineAssociations[index2].Name))
                    {
                        AlreadyExistingName = true;
                        return AlreadyExistingName;
                    }
                }
            }

            // Check whether the user given a name, which is already name
            // of an associated pipeline.
            foreach (PipelineAssociation pa in controller.ActiveModel.PipelineAssociations.Values)
            {
                if ((associationPipelineName != null) && (pa.Name.Equals(associationPipelineName)))
                    continue;

                for (int index = 0; index < model.PipelineAssociations.Count; index++)
                {
                    if (pa.Name.Equals(model.PipelineAssociations[index].Name))
                    {

                        if (isEditing == true)
                        {
                            if (model.PipelineAssociations[index].Name.Equals(this.EditPipelineName))
                                continue;

                            foreach (PipelineAssociation pa1 in controller.ActiveModel.GetPrevPipelineAssociations().Values)
                            {
                                prev = false;
                                if (pa1.Name.Equals(pa.Name))
                                {
                                    prev = true;
                                    break;
                                }
                            }
                        }


                        if (isEditing == false)
                        {
                            AlreadyExistingName = true;
                            break;
                        }
                        else
                        {
                            if (prev != false)
                            {
                                AlreadyExistingName = true;
                                break;
                            }
                        }
                    }
                }

                if (AlreadyExistingName)
                    break;
            }

            return AlreadyExistingName;
        }

        /// <summary>
        /// Handles the Click event of the Finish control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void Finish_Click(object sender, EventArgs e)
        {
            // Check whether the name given for the pipeline that we are trying to associate 
            // already exist.

            if (isEditing == true)
            {
                if (!model.PipelineAssociations[0].Name.Equals(this.EditPipelineName))
                {
                    if (IsAssociationNameExist(this.EditPipelineName))
                    {
                        MessageBox.Show(Resources.ASSOCIATION_ALREADY_EXISTS, Resources.GENEPATTREN_MSG_HEADER,
                            MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        UpdateCall(-1);
                        isNameExisting = true;
                        return;
                    }
                }
            }
            else
            {
                if (IsAssociationNameExist(null))
                {
                    MessageBox.Show(Resources.ASSOCIATION_ALREADY_EXISTS, Resources.GENEPATTREN_MSG_HEADER,
                        MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    isNameExisting = true;
                    return;
                }
            }
            try
            {
                // store data from last step
                GetControlForStep(currentStep).OnHide(0);

                DownloadPipelineFiles dialog = new DownloadPipelineFiles();
                dialog.SetWork(controller, model.Pipelines, model.PipelineAssociations, isEditing);
                DialogResult canceled = dialog.ShowDialog();
                MemoryManagement.FlushMemory();
                if (canceled != DialogResult.Cancel)
                {
                    workComplete = true;
                }
                else
                {
                    this.DialogResult = DialogResult.Cancel; // bubble the cancellation up
                }
            }
            catch (Exception ex)
            {
                AbortForm(ex);
            }
        }

        /// <summary>
        /// Adds another pipeline.
        /// </summary>
        public void AddAnotherPipeline()
        {
            // reset the add annotation user control
            addAnnotation = new AddAnnotation(model);

            // work with a copy of the data
            model = (AssociatePipelineWizardModel)model.Clone();
            model.PipelineAssociation.Name = "";
            model.PipelineAssociation.Annotation = Properties.Settings.Default.annotationTemplate;
            // pass the new (cloned) model to the usercontrols
            for (int i = 0; i < MAX_STEPS; i++)
            {
                WizardBaseControl c = GetControlForStep(i);
                c.SetModel(model);
            }


            // switch back to the start
            UpdateCall(-1 * (MAX_STEPS - 2));
        }

        /// <summary>
        /// Handles the Click event of the Next control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void Next_Click(object sender, EventArgs e)
        {
            try
            {
                PerformNextStep();
            }
            catch (Exception ex)
            {
                AbortForm(ex);
            }
        }

        /// <summary>
        /// Performs the next step.
        /// </summary>
        public void PerformNextStep()
        {
            Cursor.Current = Cursors.WaitCursor;
            if (currentStep < MAX_STEPS)
            {
                if (currentStep < MAX_STEPS)
                {
                    UpdateCall(1);
                }
            }
        }

        /// <summary>
        /// Aborts the form.
        /// </summary>
        /// <param name="ex">The ex.</param>
        internal void AbortForm(Exception ex)
        {
            MessageForm.ShowUnhandledExceptionMessage(ex);
            abort = true;
            this.DialogResult = DialogResult.Cancel;
            Close();
        }

        /// <summary>
        /// Handles the Click event of the Previous control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void Previous_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            if (currentStep > 0)
            {
                try
                {
                    if (currentStep > 0)
                    {
                        UpdateCall(-1);
                    }
                }
                catch (Exception ex)
                {
                    AbortForm(ex);
                }
            }
            Cursor.Current = Cursors.Default;
        }

        /// <summary>
        /// Handles the FormClosing event of the AssociatePipelineWizard control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void AssociatePipelineWizard_FormClosing(object sender, FormClosingEventArgs e)
        {

            if (isNameExisting)
            {
                e.Cancel = true;
                isNameExisting = false;
                return;
            }

            if (abort)
                return;
            if (model.PipelineAssociations.Count >= 1 && !workComplete)
            {
                string message = string.Empty;
                string caption = string.Empty;
                if (!isEditing)
                {
                    message = Resources.CANCEL_SEVERAL_PIPELINE_ASSOCIATIONS;
                    caption = Resources.CANCEL_SEVERAL_PIPELINE_ASSOCIATIONS_CAPTION;
                }
                else
                {
                    message = Resources.CANCEL_EDIT_PIPELINE_ASSOCIATIONS;
                    caption = Resources.CANCEL_EDIT_PIPELINE_ASSOCIATIONS_CAPTION;

                }
                DialogResult result =
                MessageBox.Show(String.Format(CultureInfo.CurrentUICulture, message, model.PipelineAssociations.Count),
                                caption,
                                MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                if (result == DialogResult.No)
                {

                    e.Cancel = true;
                }
                else
                {
                    if (isEditing)
                        controller.RollbackModelChanges();
                }
            }
        }


        #region IWizard Members

        /// <summary>
        /// Updates the finish button.
        /// </summary>
        public void UpdateFinishButton()
        {
            // check to see that we've filled out out data, then enable/disable the finish button
            bool canFinish = model.PipelineAssociation != null && model.Pipeline != null;
            // don't use &= so we can short circuit
            canFinish = canFinish && (model.PipelineAssociation.PrimaryResultName != null);
            canFinish = canFinish && (model.PipelineAssociation.Job != null);
            canFinish = canFinish && (!String.IsNullOrEmpty(model.PipelineAssociation.Annotation) && !String.IsNullOrEmpty(model.PipelineAssociation.Name));
            Finish.Enabled = canFinish;
        }

        /// <summary>
        /// Enables the next button.
        /// </summary>
        /// <param name="enabled">if set to <c>true</c> [enabled].</param>
        public void EnableNextButton(bool enabled)
        {
            if (currentStep < MAX_STEPS - 1)
            {
                Next.Enabled = enabled;
            }
        }

        #endregion

    }
}
