using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Drawing.Text;
using System.Globalization;
using System.IO;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Microsoft.eResearch.ScientificAddIn.Common;
using Ionic.Utils.Zip;
using Office = Microsoft.Office.Core;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.GenePattern.Server;
using System.Threading;
using System.Security.Cryptography;
using System.Text;
using System.Collections;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// The controller of the GenePattern Package
    /// </summary>
    public class Controller : IController
    {
        #region Fields

        private IUserControlCreator userControlCreator;
        private ITaskPaneManager taskPaneManager;
        private IRibbonInvalidator ribbonInvalidator;
        private IDataStore docStore;
        private readonly Dictionary<string, Model> models = new Dictionary<string, Model>();
        private IPackageHost host;
        private IGenePatternServer _genePatternServer;
        private UserData userData;
        private Dictionary<string, ControlState> _controlStates = new Dictionary<string, ControlState>();
        private Dictionary<string, DocumentJobProcessing> _documentJobProcessingStatus = new Dictionary<string, DocumentJobProcessing>();
        private bool updateAssociationList = true;
        private bool jobStatus;
        #endregion

        #region Constructor

        /// <summary>
        /// Initializes a new instance of the <see cref="Controller"/> class.
        /// </summary>
        /// <param name="userControlCreator">The user control creator.</param>
        public Controller(IUserControlCreator userControlCreator)
        {
            this.userControlCreator = userControlCreator;

        }

        #endregion

        #region Public Properties

        /// <summary>
        /// Gets the active model, which is the model associated with the current document.
        /// </summary>
        /// <returns>The Model of teh current document</returns>
        public Model ActiveModel
        {
            get
            {
                Model model = null;
                string docKey = host.ActiveDocumentKey;
                if (models.ContainsKey(docKey))
                {
                    model = models[docKey];
                }
                else
                {
                    //Check to see if we have model stored in the Document
                    model = docStore.DeserializeModel();
                    //If no model exists, we create a new one
                    if (model == null)
                    {
                        model = new Model();
                    }
                    models.Add(docKey, model);
                }
                return model;
            }
        }

        /// <summary>
        /// Gets or sets the task pane manager.
        /// </summary>
        /// <value>The task pane manager.</value>
        public ITaskPaneManager TaskPaneManager
        {
            get
            {
                return taskPaneManager;
            }
            set
            {
                taskPaneManager = value;
                taskPaneManager.TaskPaneVisibilityChanged += new EventHandler<VisibilityChangedEventArgs>(taskPaneManager_TaskPaneVisibilityChanged);
                taskPaneManager.TaskPaneDockPositionChanged += new EventHandler<DockPositionChangedEventArgs>(taskPaneManager_TaskPaneDockPositionChanged);
            }
        }

        /// <summary>
        /// Gets the current connection.
        /// </summary>
        /// <value>The connection.</value>
        public Connection CurrentConnection
        {
            get
            {
                string connectionName = this.ActiveConnectionForDocument;
                if (string.IsNullOrEmpty(connectionName))
                {
                    connectionName = UserData.CurrentConnectionName;
                }

                if (!string.IsNullOrEmpty(connectionName) && UserData.Connections.ContainsKey(connectionName))
                    return UserData.Connections[connectionName];
                else
                    return null;
            }
            set
            {
                if (value == null)
                {
                    UserData.CurrentConnectionName = null;
                }
                else
                {
                    UserData.CurrentConnectionName = value.Name;

                    if (UserData.Connections.ContainsKey(UserData.CurrentConnectionName))
                    {
                        UserData.Connections[UserData.CurrentConnectionName] = value;
                    }
                    else
                    {
                        UserData.Connections.Add(UserData.CurrentConnectionName, value);
                    }

                    this.ActiveConnectionForDocument = value.Name;

                    UpdateViewConnectionsTaskPane();

                }

                CurrentGenePatternServer.Connection = value;
                string controlID = this.UserControlCreator.GetControlId(ControlKey.ConnectionDropDown);
                this.RibbonInvalidator.InvalidateControl(controlID);
                this.SaveUserData();

                // Update Wizard Free UI Panes
                if (!IsWizardUI)
                {
                    UpdatePaneOnConnectionChanged(value);
                }
            }
        }

        /// <summary>
        /// Gets or sets the active connection for document.
        /// </summary>
        /// <value>The active connection for document.</value>
        public string ActiveConnectionForDocument
        {
            get
            {
                if (_controlStates.ContainsKey(host.ActiveDocumentKey))
                    return _controlStates[host.ActiveDocumentKey].DropDownActiveConnection;
                else
                    return string.Empty;
            }
            set
            {
                if (_controlStates.ContainsKey(host.ActiveDocumentKey))
                    _controlStates[host.ActiveDocumentKey].DropDownActiveConnection = value;
                else
                {
                    ControlState controlState = new ControlState();
                    controlState.DropDownActiveConnection = value;
                    _controlStates.Add(host.ActiveDocumentKey, controlState);
                }
            }
        }

        void taskPaneManager_TaskPaneDockPositionChanged(object sender, DockPositionChangedEventArgs e)
        {
            ITaskPaneControl control = taskPaneManager.GetTaskPaneUserControl(e.TaskPaneTitle) as ITaskPaneControl;
            if (control != null)
            {
                control.SetDockPosition(e.DockPosition);
            }
        }

        /// <summary>
        /// Gets or sets the ribbon invalidator.
        /// </summary>
        /// <value>The ribbon invalidator.</value>
        public IRibbonInvalidator RibbonInvalidator
        {
            get
            {
                return ribbonInvalidator;
            }
            set
            {
                ribbonInvalidator = value;
            }
        }

        /// <summary>
        /// Gets or sets the data store.
        /// </summary>
        /// <value>The data store.</value>
        public IDataStore DataStore
        {
            get { return docStore; }
            set { docStore = value; }
        }

        /// <summary>
        /// Gets the models.
        /// </summary>
        /// <value>The models.</value>
        public Dictionary<string, Model> Models
        {
            get { return models; }
        }

        /// <summary>
        /// Gets the user data.
        /// </summary>
        /// <value>The user data.</value>
        public UserData UserData
        {
            get
            {
                if (userData == null)
                {
                    userData = UserDataManager.Read();
                }
                return userData;
            }
        }

        /// <summary>
        /// Gets the document key.
        /// </summary>
        /// <value>The document key.</value>
        public string DocumentKey
        {
            get { return host.ActiveDocumentKey; }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is wizard free UI.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is wizard free UI; otherwise, <c>false</c>.
        /// </value>
        public bool IsWizardUI
        {
            get
            {
                if (_controlStates.ContainsKey(host.ActiveDocumentKey))
                    return _controlStates[host.ActiveDocumentKey].IsWizardFreeUI;
                else
                    return true;
            }
            set
            {
                if (_controlStates.ContainsKey(host.ActiveDocumentKey))
                    _controlStates[host.ActiveDocumentKey].IsWizardFreeUI = value;
                else
                {
                    ControlState controlState = new ControlState();
                    controlState.IsWizardFreeUI = value;
                    _controlStates.Add(host.ActiveDocumentKey, controlState);
                }
            }
        }

        /// <summary>
        /// Gets a value indicating whether the job is failed or not.
        /// </summary>
        /// <value>
        /// 	<c>false</c> if the job failed; otherwise, <c>true</c>.
        /// </value>
        public bool JobStatus
        {
            get { return jobStatus; }
        }

        /// <summary>
        /// Gets the get document image.
        /// </summary>
        /// <value>The get document image.</value>
        public DocumentImage GetDocumentImage
        {
            get { return host.GetSelectedImage(); }
        }

        /// <summary>
        /// Gets the range.
        /// </summary>
        /// <returns></returns>
        public ImageRange GetRange
        {
            get
            {
                return host.GetRange;
            }
        }

        /// <summary>
        /// Gets the document count.
        /// </summary>
        /// <value>The document count.</value>
        public int DocumentCount
        {
            get { return host.DocumentCount; }
        }

        #endregion

        #region Private Properties

        private IGenePatternServer CurrentGenePatternServer
        {
            get
            {
                if (_genePatternServer == null)
                {
                    _genePatternServer = GetGenePatternServer();
                }
                return _genePatternServer;
            }
        }

        #endregion

        #region IController Members

        /// <summary>
        /// Gets the user control creator.
        /// </summary>
        /// <value>The user control creator.</value>
        public IUserControlCreator UserControlCreator
        {
            get { return userControlCreator; }
        }

        /// <summary>
        /// Gets the server broker.
        /// </summary>
        /// <value>The server broker.</value>
        public IServerBroker ServerBroker
        {
            get { return new ServerBroker(userData); }
        }

        /// <summary>
        /// Shows the associate pipeline wizard.
        /// </summary>
        /// returns pipeline association, or null if canceled (actually embedded a pipeline)
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<PipelineAssociation> ShowAssociatePipelineWizard(bool isLinked)
        {
            CheckFirstTimeRunning();
            if (CheckDocumentProtection())
            {
                return new List<PipelineAssociation>();
            }
            if (!isLinked) // for linked pipelines, we'll insert an image later
            {
                DocumentImage documentImage = host.GetSelectedImage();
                if (documentImage == null)
                {
                    Trace.WriteLine("No image is selected for associating a pipeline", TraceLevel.Info.ToString());
                    MessageBox.Show(Resources.NO_IMAGE_SELECTED, Resources.NO_IMAGE_CAPTION,
                                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);

                    return null;
                }
                if (documentImage.AppendixImage)
                {
                    MessageBox.Show(Resources.ASSOCITAION_WITH_RERUN_RESULT_NOT_ALLOWED, Resources.GENEPATTREN_MSG_HEADER,
                                   MessageBoxButtons.OK, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1);

                    return null;
                }
            }
            try
            {
                using (AssociatePipelineWizard wizard = new AssociatePipelineWizard(this, isLinked))
                {
                    Trace.WriteLine("Starting to associate a pipeline to an image", TraceLevel.Info.ToString());
                    DialogResult result = wizard.ShowDialog();
                    if (result == DialogResult.Cancel)
                    {
                        return null;
                    }
                    else
                    {
                        return wizard.PipelineAssociations;
                    }
                }
            }
            catch (ConnectionFailedException e)
            {
                Trace.WriteLine("Exception: " + e, TraceLevel.Error.ToString());
                MessageBox.Show(Resources.CONNECTION_FAILED, Resources.CONNECTION_FAILED_CAPTION,
                                MessageBoxButtons.OK, MessageBoxIcon.Exclamation,
                                MessageBoxDefaultButton.Button1);
                return null;
            }
        }

        /// <summary>
        /// Determines whether [is job processing in progress] [the specified document key].
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <returns>
        /// 	<c>true</c> if [is job processing in progress] [the specified document key]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsJobProcessingInProgress(string documentKey)
        {
            if (_documentJobProcessingStatus.ContainsKey(documentKey))
                return _documentJobProcessingStatus[documentKey].IsJobProcessingInProgress;
            else
                return false;
        }

        /// <summary>
        /// Sets the job processing status.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <param name="status">if set to <c>true</c> [status].</param>
        public void SetJobProcessingStatus(string documentKey, bool status)
        {
            if (_documentJobProcessingStatus.ContainsKey(documentKey))
                _documentJobProcessingStatus[documentKey].IsJobProcessingInProgress = status;
            else
                _documentJobProcessingStatus.Add(documentKey, new DocumentJobProcessing(status));
        }

        /// <summary>
        /// Check to see if this is the first time to application has been run
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        public void CheckFirstTimeRunning()
        {
            if (!UserData.HasShownIntroMessage)
            {
                MessageBox.Show(Resources.ADDIN_FIRST_RUN_MSG, Resources.GENEPATTREN_MSG_HEADER,
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                UserData.HasShownIntroMessage = true;
            }
        }

        /// <summary>
        /// Shows the rerun pipeline wizard.
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        public bool ShowRerunPipelineWizard()
        {
            CheckFirstTimeRunning();
            DocumentImage documentImage = host.GetSelectedImage();

            if (documentImage == null)
            {
                Trace.WriteLine("No image is selected for pipeline rerun", TraceLevel.Info.ToString());
                MessageBox.Show(Resources.NO_IMAGE_SELECTED_FOR_RERUN, Resources.NO_IMAGE_CAPTION,
                                MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);

                return false;
            }

            if (documentImage.AppendixImage)
            {
                MessageBox.Show(Resources.PIPELINE_RERUN_FROM_RERUN_RESULT_NOT_ALLOWED, Resources.GENEPATTREN_MSG_HEADER,
                MessageBoxButtons.OK, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1);

                return false;
            }

            List<PipelineAssociation> pipelineAssociations = GetAssociatedPipelines(documentImage, ActiveModel);
            using (RerunPipelineWizard rerunPipelineWizard = new RerunPipelineWizard(this,
                                                                                     pipelineAssociations,
                                                                                     null))
            {
                rerunPipelineWizard.ShowDialog();
            }

            return true;
        }

        /// <summary>
        /// Sets the package host.
        /// </summary>
        /// <param name="packageHost">The package host.</param>
        public void SetPackageHost(IPackageHost packageHost)
        {
            this.host = packageHost;
            this.DataStore.SetPackageHost(packageHost);
            this.host.DocumentSave += new EventHandler<DocumentSaveEventArgs>(host_DocumentSaved);
            this.host.ImageSelect += new EventHandler<SelectedImageEventArgs>(host_ImageSelect);
            this.host.ImageDeselect += new EventHandler(host_ImageDeSelect);
            this.host.DocumentClose += new EventHandler<CancelCloseEventArgs>(host_DocumentClose);
            this.host.DocumentOpen += new EventHandler(host_DocumentOpen);
            this.host.DocumentActivated += new EventHandler(host_DocumentActivated);
            this.host.ImageDoubleClicked += new EventHandler<SelectedImageEventArgs>(host_ImageDoubleClicked);
        }

      
        /// <summary>
        /// Updates the pipeline associations.
        /// </summary>
        /// <param name="pipelines">The pipelines.</param>
        /// <param name="pipelineAssociations">The pipeline associations.</param>
        /// <returns></returns>
        public List<string> UpdatePipelineAssociations(List<Pipeline> pipelines, List<PipelineAssociation> pipelineAssociations)
        {
            return UpdatePipelineAssociations(pipelines, pipelineAssociations, null);
        }

        /// <summary>
        /// Updates the pipeline associations.
        /// </summary>
        /// <param name="pipelines">The pipelines.</param>
        /// <param name="pipelineAssociations">The pipeline associations.</param>
        /// <param name="downloadProgressCallback">The download progress callback.</param>
        public List<string> UpdatePipelineAssociations(List<Pipeline> pipelines,
            List<PipelineAssociation> pipelineAssociations, DownloadProgress downloadProgressCallback)
        {
            Debug.Assert(pipelines.Count == pipelineAssociations.Count);

            List<string> messages = new List<string>();
            for (int i = 0; i < pipelines.Count; i++)
            {
                Pipeline pipeline = pipelines[i];
                PipelineAssociation association = pipelineAssociations[i];

                //does the pipeline already exist?
                if (ActiveModel.Pipelines.ContainsKey(pipeline.Lsid))
                {
                    Pipeline alreadyStoredPipeline = ActiveModel.Pipelines[pipeline.Lsid];
                    // pipeline is already stored, update ONLY it's linked status (anything else would mean a different lsid)
                    // if new pipeline is linked, need to clean old files for it -- this will actually happen on save

                    // if pipeline is switching from linked to not-linked, download
                    if (alreadyStoredPipeline.IsLinked && !pipeline.IsLinked)
                    {
                        // make the changes to the already stored pipeline since we aren't actually adding a new one 
                        DownloadAndStorePipelineFiles(alreadyStoredPipeline, downloadProgressCallback, false);
                    }
                    else
                    {
                        // show fake download
                        if (downloadProgressCallback != null)
                        {
                            downloadProgressCallback(pipeline.Name, pipeline.Lsid, true);
                            downloadProgressCallback(pipeline.Name, pipeline.Lsid, false);
                        }
                    }

                    alreadyStoredPipeline.IsLinked = pipeline.IsLinked;
                }
                else // pipeline does not already exist
                {
                    if (!pipeline.IsLinked)
                    {
                        DownloadAndStorePipelineFiles(pipeline, downloadProgressCallback, false);
                    }
                    else
                    {
                        // fake download
                        if (downloadProgressCallback != null)
                        {
                            downloadProgressCallback(pipeline.Name, pipeline.Lsid, true);
                            downloadProgressCallback(pipeline.Name, pipeline.Lsid, false);
                        }
                    }
                }

                // if there are any images in the results, download them and make thumbnails
                GetImageResults(association, null);
                messages.AddRange(DownloadAndStoreInputFiles(association, null, null));

                #region Clear Previous Pipeline and add the new one

                ActiveModel.Pipelines.Remove(association.PipelineLsid);
                if (ActiveModel.Pipelines.ContainsKey(pipeline.Lsid))
                {
                    ActiveModel.Pipelines[pipeline.Lsid] = pipeline;
                }
                else
                {
                    ActiveModel.Pipelines.Add(pipeline.Lsid, pipeline);
                }

                #endregion

                association.PipelineLsid = pipeline.Lsid;
            }
            host.IsDocumentSaved = false;
            return messages;
        }

        /// <summary>
        /// Updates the pipeline associations.
        /// </summary>
        /// <param name="pipelines">The pipelines.</param>
        /// <param name="pipelineAssociations">The pipeline associations.</param>
        /// <param name="downloadProgressCallback">The download progress callback.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="genePatternServer">The gene pattern server.</param>
        /// <returns></returns>
        public List<string> UpdatePipelineAssociations(List<Pipeline> pipelines,
            List<PipelineAssociation> pipelineAssociations, DownloadProgress downloadProgressCallback,
            string documentKey, IGenePatternServer genePatternServer)
        {
            Debug.Assert(pipelines.Count == pipelineAssociations.Count);

            List<string> messages = new List<string>();
            Model currentModel = GetDocumentModel(documentKey);

            for (int i = 0; i < pipelines.Count; i++)
            {
                Pipeline pipeline = pipelines[i];
                PipelineAssociation association = pipelineAssociations[i];

                //does the pipeline already exist?
                if (currentModel.Pipelines.ContainsKey(pipeline.Lsid))
                {
                    Pipeline alreadyStoredPipeline = currentModel.Pipelines[pipeline.Lsid];
                    // pipeline is already stored, update ONLY it's linked status (anything else would mean a different lsid)
                    // if new pipeline is linked, need to clean old files for it -- this will actually happen on save

                    // if pipeline is switching from linked to not-linked, download
                    if (alreadyStoredPipeline.IsLinked && !pipeline.IsLinked)
                    {
                        // make the changes to the already stored pipeline since we aren't actually adding a new one 
                        DownloadAndStorePipelineFiles(alreadyStoredPipeline, downloadProgressCallback, false);
                    }
                    else
                    {
                        // show fake download
                        if (downloadProgressCallback != null)
                        {
                            downloadProgressCallback(pipeline.Name, pipeline.Lsid, true);
                            downloadProgressCallback(pipeline.Name, pipeline.Lsid, false);
                        }
                    }

                    alreadyStoredPipeline.IsLinked = pipeline.IsLinked;
                }
                else // pipeline does not already exist
                {
                    if (!pipeline.IsLinked)
                    {
                        DownloadAndStorePipelineFiles(pipeline, downloadProgressCallback, false);
                    }
                    else
                    {
                        // fake download
                        if (downloadProgressCallback != null)
                        {
                            downloadProgressCallback(pipeline.Name, pipeline.Lsid, true);
                            downloadProgressCallback(pipeline.Name, pipeline.Lsid, false);
                        }
                    }
                }

                // if there are any images in the results, download them and make thumbnails
                GetImageResults(association, genePatternServer);

                messages.AddRange(DownloadAndStoreInputFiles(association, documentKey, genePatternServer));

                #region Clear Previous Pipeline and add the new one

                currentModel.Pipelines.Remove(association.PipelineLsid);
                if (currentModel.Pipelines.ContainsKey(pipeline.Lsid))
                {
                    currentModel.Pipelines[pipeline.Lsid] = pipeline;
                }
                else
                {
                    currentModel.Pipelines.Add(pipeline.Lsid, pipeline);
                }

                #endregion

                association.PipelineLsid = pipeline.Lsid;
            }
            host.SetDocumentSaved(false, documentKey);
            return messages;
        }


        /// <summary>
        /// Adds pipelines and their associations.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        /// <returns></returns>
        public List<string> AddPipelineAssociations(List<Pipeline> pipeline, List<PipelineAssociation> pipelineAssociation)
        {
            return AddPipelineAssociations(pipeline, pipelineAssociation, null);
        }

        /// <summary>
        /// Adds the pipeline associations.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        /// <param name="downloadProgressCallback">The download progress callback.</param>
        /// <returns></returns>
        public List<string> AddPipelineAssociations(List<Pipeline> pipeline, List<PipelineAssociation> pipelineAssociation,
            DownloadProgress downloadProgressCallback)
        {
            Debug.Assert(pipeline.Count == pipelineAssociation.Count);
            List<string> messages = new List<string>();
            IGenePatternServer genePatternServer = GetGenePatternServer();

            // Get the Current Model based on DocumentKey
            Model currentModel = ActiveModel;

            for (int i = 0; i < pipeline.Count; i++)
            {
                Pipeline newPipeline = pipeline[i];
                PipelineAssociation association = pipelineAssociation[i];

                // does the pipeline already exist?
                if (currentModel.Pipelines.ContainsKey(newPipeline.Lsid))
                {
                    Pipeline alreadyStoredPipeline = currentModel.Pipelines[newPipeline.Lsid];

                    // pipeline is already stored, update ONLY it's linked status (anything else would mean a different lsid)
                    // if new pipeline is linked, need to clean old files for it -- this will actually happen on save
                    if (downloadProgressCallback != null)
                    {
                        downloadProgressCallback(newPipeline.Name, newPipeline.Lsid, true);
                    }
                    // if pipeline is switching from linked to not-linked, download
                    if (alreadyStoredPipeline.IsLinked && !newPipeline.IsLinked)
                    {
                        // make the changes to the already stored pipeline since we aren't actually adding a new one 
                        DownloadAndStorePipelineFiles(alreadyStoredPipeline, downloadProgressCallback, false);
                    }

                    alreadyStoredPipeline.IsLinked = newPipeline.IsLinked;
                    if (downloadProgressCallback != null)
                    {
                        downloadProgressCallback(newPipeline.Name, newPipeline.Lsid, false);
                    }


                    if (newPipeline.LinkConnection == null && 
                        alreadyStoredPipeline.LinkConnection.ServerUri != genePatternServer.Connection.ServerUri)
                    {
                        string message = string.Format(Resources.NOT_ORIGINAL_CONNECTION,
                            alreadyStoredPipeline.LinkConnection.ServerName);
                        MessageBox.Show(message, Resources.GENEPATTREN_MSG_HEADER,
                                        MessageBoxButtons.OK, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1);
                        break;
                    }

                    // use existing pipeline for any checks after this point
                    newPipeline = alreadyStoredPipeline;
                    genePatternServer = ServerBroker.GetTestedVerifiedServer(newPipeline.LinkConnection, string.Empty, true);
                }
                else
                {
                    if (!newPipeline.IsLinked)
                    {
                        DownloadAndStorePipelineFiles(newPipeline, downloadProgressCallback, false);
                    }
                    else
                    {
                        // fake download
                        if (downloadProgressCallback != null)
                        {
                            downloadProgressCallback(newPipeline.Name, newPipeline.Lsid, true);
                            downloadProgressCallback(newPipeline.Name, newPipeline.Lsid, false);
                        }
                    }
                }

                // if there are any images in the results, download them and make thumbnails
                GetImageResults(association, genePatternServer);

                // get image to associate with
                // these CAN be null, just means that the caller will need to set them later
                DocumentImage theImage = host.GetSelectedImage();

                if (association.PipelineDocumentImage != null)
                    theImage.IsGenePatternLink = association.PipelineDocumentImage.IsGenePatternLink;

                association.PipelineDocumentImage = theImage;
                association.ThumbnailImage = host.GetSelectedImageThumb;
                if (theImage != null)
                {
                    host.MakeDocumentDirty();
                }
                // store everything in the association
                newPipeline.Name = association.Job.PipelineName;

                currentModel.AddPipeline(newPipeline);
                association.PipelineLsid = newPipeline.Lsid;
                messages.AddRange(DownloadAndStoreInputFiles(association, null, genePatternServer));
                currentModel.AddPipelineAssociation(association);
            }
            if (jobStatus)
            {
                host.IsDocumentSaved = false;
            }
            return messages;
        }

        /// <summary>
        /// Adds the pipeline associations.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        /// <param name="downloadProgressCallback">The download progress callback.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="documentImage">The document image.</param>
        /// <param name="genePatternServer">The gene pattern server.</param>
        /// <returns></returns>
        public List<string> AddPipelineAssociations(List<Pipeline> pipeline, List<PipelineAssociation> pipelineAssociation,
            DownloadProgress downloadProgressCallback, string documentKey,
            DocumentImage documentImage, IGenePatternServer genePatternServer)
        {
            Debug.Assert(pipeline.Count == pipelineAssociation.Count);
            List<string> messages = new List<string>();
            jobStatus = true;
            // Get the Current Model based on DocumentKey
            Model currentModel = new Model();
            if (string.IsNullOrEmpty(documentKey))
            {
                currentModel = ActiveModel;
            }
            else
            {
                currentModel = GetDocumentModel(documentKey);
            }

            for (int i = 0; i < pipeline.Count; i++)
            {
                Pipeline newPipeline = pipeline[i];
                PipelineAssociation association = pipelineAssociation[i];
                
                // does the pipeline already exist?
                if (currentModel.Pipelines.ContainsKey(newPipeline.Lsid))
                {
                    Pipeline alreadyStoredPipeline = currentModel.Pipelines[newPipeline.Lsid];

                    // pipeline is already stored, update ONLY it's linked status (anything else would mean a different lsid)
                    // if new pipeline is linked, need to clean old files for it -- this will actually happen on save
                    if (downloadProgressCallback != null)
                    {
                        downloadProgressCallback(newPipeline.Name, newPipeline.Lsid, true);
                    }
                    // if pipeline is switching from linked to not-linked, download
                    if (alreadyStoredPipeline.IsLinked && !newPipeline.IsLinked)
                    {
                        // make the changes to the already stored pipeline since we aren't actually adding a new one 
                        DownloadAndStorePipelineFiles(alreadyStoredPipeline, downloadProgressCallback, false, documentKey, genePatternServer);
                    }

                    alreadyStoredPipeline.IsLinked = newPipeline.IsLinked;
                    if (downloadProgressCallback != null)
                    {
                        downloadProgressCallback(newPipeline.Name, newPipeline.Lsid, false);
                    }

                    if (newPipeline.LinkConnection == null &&
                        alreadyStoredPipeline.LinkConnection.ServerUri != genePatternServer.Connection.ServerUri)
                    {
                        string message = string.Format(Resources.NOT_ORIGINAL_CONNECTION,
                            alreadyStoredPipeline.LinkConnection.ServerName);
                        MessageBox.Show(message, Resources.GENEPATTREN_MSG_HEADER,
                                        MessageBoxButtons.OK, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1);
                        jobStatus = false;
                        break;
                    }

                    // use existing pipeline for any checks after this point
                    newPipeline = alreadyStoredPipeline;
                }
                else
                {
                    if (!newPipeline.IsLinked)
                    {
                        DownloadAndStorePipelineFiles(newPipeline, downloadProgressCallback, false, documentKey, genePatternServer);
                    }
                    else
                    {
                        // fake download
                        if (downloadProgressCallback != null)
                        {
                            downloadProgressCallback(newPipeline.Name, newPipeline.Lsid, true);
                            downloadProgressCallback(newPipeline.Name, newPipeline.Lsid, false);
                        }
                    }
                }

                // if there are any images in the results, download them and make thumbnails
                GetImageResults(association, genePatternServer);

                // get image to associate with
                // these CAN be null, just means that the caller will need to set them later
                #region Set Document Image & Create Thumbnail

                if (documentImage == null)
                {
                    association.ThumbnailImage = null;
                }
                else
                {
                    if (association.PipelineDocumentImage != null)
                        documentImage.IsGenePatternLink = association.PipelineDocumentImage.IsGenePatternLink;

                    association.PipelineDocumentImage = documentImage;
                    if (documentImage != null)
                    {
                        Image image = host.GetSelectedImage(documentKey, documentImage.ImageKey);
                        association.ThumbnailImage = GetSelectedImageThumb(image);
                    }
                }

                #endregion

                if (documentImage != null)
                {
                    host.MakeDocumentDirty();
                }
                // store everything in the association
                newPipeline.Name = association.Job.PipelineName;

                currentModel.AddPipeline(newPipeline);
                association.PipelineLsid = newPipeline.Lsid;
                messages.AddRange(DownloadAndStoreInputFiles(association, documentKey, genePatternServer));
                currentModel.AddPipelineAssociation(association);
            }
            if (jobStatus)
            {
                host.SetDocumentSaved(false, documentKey);
            }
            return messages;
        }

        /// <summary>
        /// Gets the image results.
        /// </summary>
        /// <param name="association">The association.</param>
        /// <param name="genePatternServer">The gene pattern server.</param>
        private void GetImageResults(PipelineAssociation association, IGenePatternServer genePatternServer)
        {
            Job theJob = association.Job;
            string[] img_extensions = { "bmp", "jpg", "gif", "png", "jpeg", "tiff", "tif", "eps" };
            List<string> enumerableImgExtensions = new List<string>(img_extensions);

            bool hasImages = false;
            // check for images so we can skip the download otherwise
            foreach (ShallowJobResult s in theJob.ShallowResults)
            {
                int ext_begin = s.Filename.LastIndexOf('.');
                string extension = ext_begin >= 0
                                       ? s.Filename.Substring(ext_begin + 1).ToLower(CultureInfo.InvariantCulture)
                                       : "";
                if (enumerableImgExtensions.Contains(extension))
                {
                    hasImages = true;
                    break;
                }
            }
            if (hasImages)
            {
                if (genePatternServer == null)
                    genePatternServer = CurrentGenePatternServer;

                IList<Result> results = genePatternServer.GetJobResults(theJob.Id);

                foreach (Result r in results)
                {
                    // is the result an image?
                    int ext_begin = r.Filename.LastIndexOf('.');
                    string extension = ext_begin >= 0
                                           ? r.Filename.Substring(ext_begin + 1).ToLower(CultureInfo.InvariantCulture)
                                           : "";
                    if (enumerableImgExtensions.Contains(extension))
                    {
                        foreach (ShallowJobResult result in theJob.ShallowResults)
                        {
                            if (result.Filename == r.Filename)
                            {
                                if (!string.Equals(Path.GetExtension(r.Filename), ".eps"))
                                {
                                    Image fullImage = Image.FromStream(new MemoryStream(r.Content));
                                    if (fullImage != null)
                                    {
                                        Image.GetThumbnailImageAbort dummyCallBack =
                                            new Image.GetThumbnailImageAbort(ThumbnailCallback);

                                        int thumbSize = 64;
                                        // we are doing this solely for the images viewer, so this dependency is only kind of ugly...
                                        result.ThumbnailImage = ImageResultsViewer.GetProportionedThumbnail(fullImage, thumbSize, dummyCallBack);
                                    }
                                }
                                else
                                {
                                    Image fullImage = Resources.NoPreviewAvailable;
                                    if (fullImage != null)
                                    {
                                        Image.GetThumbnailImageAbort dummyCallBack =
                                           new Image.GetThumbnailImageAbort(ThumbnailCallback);

                                        int thumbSize = 64;
                                        // we are doing this solely for the images viewer, so this dependency is only kind of ugly...
                                        result.ThumbnailImage = ImageResultsViewer.GetProportionedThumbnail(fullImage, thumbSize, dummyCallBack);
                                    }
                                }
                                break;
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Gets the image results.
        /// </summary>
        /// <param name="association">The association.</param>
        /// <param name="genePatternServer">The gene pattern server.</param>
        /// <param name="pipelineConnection">The pipeline connection.</param>
        private void GetImageResults(PipelineAssociation association, IGenePatternServer genePatternServer,
            Connection pipelineConnection)
        {
            Job theJob = association.Job;
            string[] img_extensions = { "bmp", "jpg", "gif", "png", "jpeg", "tiff", "tif", "eps" };
            List<string> enumerableImgExtensions = new List<string>(img_extensions);

            bool hasImages = false;
            // check for images so we can skip the download otherwise
            foreach (ShallowJobResult s in theJob.ShallowResults)
            {
                int ext_begin = s.Filename.LastIndexOf('.');
                string extension = ext_begin >= 0
                                       ? s.Filename.Substring(ext_begin + 1).ToLower(CultureInfo.InvariantCulture)
                                       : "";
                if (enumerableImgExtensions.Contains(extension))
                {
                    hasImages = true;
                    break;
                }
            }
            if (hasImages)
            {
                if (genePatternServer == null)
                    genePatternServer = CurrentGenePatternServer;

                IList<Result> results = genePatternServer.GetJobResults(theJob.Id, pipelineConnection);

                foreach (Result r in results)
                {
                    // is the result an image?
                    int ext_begin = r.Filename.LastIndexOf('.');
                    string extension = ext_begin >= 0
                                           ? r.Filename.Substring(ext_begin + 1).ToLower(CultureInfo.InvariantCulture)
                                           : "";
                    if (enumerableImgExtensions.Contains(extension))
                    {
                        foreach (ShallowJobResult result in theJob.ShallowResults)
                        {
                            if (result.Filename == r.Filename)
                            {
                                if (!string.Equals(Path.GetExtension(r.Filename), ".eps"))
                                {
                                    Image fullImage = Image.FromStream(new MemoryStream(r.Content));
                                    if (fullImage != null)
                                    {
                                        Image.GetThumbnailImageAbort dummyCallBack =
                                            new Image.GetThumbnailImageAbort(ThumbnailCallback);

                                        int thumbSize = 64;
                                        // we are doing this solely for the images viewer, so this dependency is only kind of ugly...
                                        result.ThumbnailImage = ImageResultsViewer.GetProportionedThumbnail(fullImage, thumbSize, dummyCallBack);
                                    }
                                }
                                else
                                {
                                    Image fullImage = Resources.NoPreviewAvailable;
                                    if (fullImage != null)
                                    {
                                        Image.GetThumbnailImageAbort dummyCallBack =
                                           new Image.GetThumbnailImageAbort(ThumbnailCallback);

                                        int thumbSize = 64;
                                        // we are doing this solely for the images viewer, so this dependency is only kind of ugly...
                                        result.ThumbnailImage = ImageResultsViewer.GetProportionedThumbnail(fullImage, thumbSize, dummyCallBack);
                                    }
                                }
                                break;
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Gets the pipelineID files.
        /// </summary>
        /// <param name="lsid">The LSID.</param>
        public MemoryStream GetPipelineZip(string lsid, string documentKey)
        {
            Model currenModel;
            if (string.IsNullOrEmpty(documentKey))
            {
                currenModel = ActiveModel;
            }
            else
            {
                currenModel = GetDocumentModel(documentKey);
            }

            // if it's actually a module, return the stored zip
            if (currenModel.Modules.ContainsKey(lsid))
            {
                return DataStore.GetFile(GetFileGuid(lsid));
            }
            else // otherwise build the big zip from component parts
            {
                MemoryStream outputStream = new MemoryStream();
                MemoryStream tempStream = new MemoryStream();
                ZipFile zfile = new ZipFile(outputStream);
                ZipFile ztemp = new ZipFile(tempStream);
                Pipeline pipeline = currenModel.Pipelines[lsid];

                if (pipeline.IsLinked)
                {
                    // download straight from server
                    Connection connection = pipeline.LinkConnection.Clone() as Connection;
                    IGenePatternServer genePatternServer = ServerBroker.GetTestedVerifiedServer(connection, string.Empty, true);
                   
                    if (genePatternServer == null)
                        return outputStream;

                    tempStream = genePatternServer.DownloadTask(pipeline.Lsid, false);
                    tempStream.Position = 0;
                    AddStreamToZip(zfile, tempStream, pipeline.Name + ".zip");
                    
                    foreach (PipelineStep step in pipeline.Steps)
                    {
                        using (MemoryStream stepzip = genePatternServer.DownloadTask(step.Lsid, false))
                        {
                            AddStreamToZip(zfile, stepzip, step.Name + ".zip");
                        }
                    }

                    zfile.Save();
                    RemoveSourcesOfZipFile(zfile);
                }
                else
                {
                    if (pipeline.ManifestFile != null)
                    {
                        AddFileToZip(ztemp, pipeline.ManifestFile); //Store manifest file in new zipfile.

                    }
                    foreach (InternalFile file in pipeline.Documentation) //Store documentation files into new zipfile.
                    {
                        AddFileToZip(ztemp, file);
                    }
                    foreach (InternalFile file in pipeline.Datafiles)   //Store Data files into new zipfile.
                    {
                        AddFileToZip(ztemp, file);
                    }
                    ztemp.Save();
                    tempStream.Position = 0;
                    AddStreamToZip(zfile, tempStream, pipeline.Name + ".zip");  //Add this new stream containing the manifest,datafiles and documentation files to the final Zipfile.
                    RemoveSourcesOfZipFile(ztemp);
                    // add steps in their own zips
                    foreach (PipelineStep step in pipeline.Steps)
                    {
                        using (MemoryStream stepZip = GetPipelineZip(step.Lsid, documentKey))
                        {
                            AddStreamToZip(zfile, stepZip, step.Name + ".zip");
                        }
                    }
                    zfile.Save();
                    RemoveSourcesOfZipFile(zfile);
                }

                #region Release Resources

                tempStream.Dispose();
                zfile.Dispose();
                ztemp.Dispose();

                tempStream = null;
                zfile = null;
                ztemp = null;

                #endregion

                return outputStream;
            }
        }

        /// <summary>
        /// Removes the sources of zip file.
        /// </summary>
        /// <param name="zfile">The zfile.</param>
        private static void RemoveSourcesOfZipFile(ZipFile zfile)
        {
            //cleanup temp files that were zipped
            foreach (ZipEntry ze in zfile)
            {
                try
                {
                    if (!ze.IsDirectory)
                        File.Delete(ze.FileName);
                }
                catch (IOException) { }
            }
        }

        /// <summary>
        /// Gets the pipeline zip.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <param name="includePipelineSteps">if set to <c>true</c> [include pipeline steps].</param>
        /// <returns></returns>
        public MemoryStream GetPipelineZip(string lsid, bool includePipelineSteps)
        {
            if (includePipelineSteps)
                return this.GetPipelineZip(lsid, string.Empty);
            else
            {
                Pipeline pipeline = ActiveModel.Pipelines[lsid];
                MemoryStream outputStream = new MemoryStream();
                using (ZipFile zf = new ZipFile(outputStream))
                {
                    AddPipelineFilesToZipStream(pipeline, zf);
                    zf.Save();
                    RemoveSourcesOfZipFile(zf);
                }
                return outputStream;
            }

        }

        /// <summary>
        /// Gets all pipeline associations in the model
        /// </summary>
        /// <returns></returns>
        public ReadOnlyCollection<PipelineAssociation> GetAllPipelineAssociations()
        {
            // prune dead associations
            ReadOnlyCollection<DocumentImage> docImages = host.GetAllImages();
            List<int> toRemove = new List<int>();
            foreach (int key in ActiveModel.PipelineAssociations.Keys)
            {
                PipelineAssociation pa = ActiveModel.PipelineAssociations[key];
                // note that a null pa.Image means it's just being added -- don't delete it!
                if (pa.PipelineDocumentImage != null && !docImages.Contains(pa.PipelineDocumentImage))
                {
                    toRemove.Add(key);
                }
            }
            foreach (int deleteKey in toRemove)
            {
                ActiveModel.PipelineAssociations.Remove(deleteKey);
            }
            return new ReadOnlyCollection<PipelineAssociation>(new List<PipelineAssociation>(ActiveModel.PipelineAssociations.Values));
        }

        /// <summary>
        /// Gets all pipeline associations in the model
        /// </summary>
        /// <returns></returns>
        public ReadOnlyCollection<PipelineAssociation> GetAllPipelineAssociations(string documentKey)
        {
            // prune dead associations
            ReadOnlyCollection<DocumentImage> docImages = host.GetAllImages(documentKey);
            Model currentModel = GetDocumentModel(documentKey);

            List<int> toRemove = new List<int>();
            foreach (int key in currentModel.PipelineAssociations.Keys)
            {
                PipelineAssociation pa = currentModel.PipelineAssociations[key];
                // note that a null pa.Image means it's just being added -- don't delete it!
                if (pa.PipelineDocumentImage != null && !docImages.Contains(pa.PipelineDocumentImage))
                {
                    toRemove.Add(key);
                }
            }
            foreach (int deleteKey in toRemove)
            {
                currentModel.PipelineAssociations.Remove(deleteKey);
            }
            return new ReadOnlyCollection<PipelineAssociation>(new List<PipelineAssociation>(currentModel.PipelineAssociations.Values));
        }

        /// <summary>
        /// Deletes all pipeline associations in the model
        /// </summary>
        /// <param name="pipelineassociations"></param>
        public void DeleteAllPipelineassociations(ReadOnlyCollection<PipelineAssociation> pipelineassociations)
        {
            foreach (PipelineAssociation pa in pipelineassociations)
            {
                DeletePipelineAssociation(pa);
            }
        }

        /// <summary>
        /// Shows the server connection form.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="editMode">if set to <c>true</c> [edit mode].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        public void ShowServerConnectionForm(Connection connection, bool editMode)
        {
            CheckFirstTimeRunning();
            using (ServerConnection form = new ServerConnection(connection, PasswordRequiredStatus.Unknown, false))
            {
                if (DialogResult.OK == form.ShowDialog())
                {
                    if (form.IsConnectionSuccessful)
                    {
                        if (editMode)
                        {
                            string connectionName = connection.Name;
                            //User edited a existing connection. Remove the old one and add a new one
                            RemoveConnection(connectionName);
                            userData.Connections.Add(form.Connection.Name, form.Connection);
                            string currentConnection = userData.CurrentConnectionName;
                            if ((!String.IsNullOrEmpty(currentConnection)) && currentConnection.Equals(connectionName))
                            {
                                userData.CurrentConnectionName = form.Connection.Name;
                                //Will also cause the View Connections user control to be refreshed
                                CurrentConnection = form.Connection;
                            }
                            else
                            {
                                this.UpdateViewConnectionsTaskPane(false);
                            }
                            UpdateConnectionTaskPaneDirtyStatus();
                        }
                        else //New Connetion. Set it as default
                        {
                            // test connection before setting it as the defeault
                            CurrentConnection = form.Connection;
                            UpdateConnectionTaskPaneDirtyStatus();
                        }
                        RerunPipelineTaskPane rerunPipelineTaskPane = this.GetActiveRerunPipelineTaskPane(DocumentKey);
                        if (rerunPipelineTaskPane != null)
                        {
                            rerunPipelineTaskPane.UpdateConnectionsComboBox();
                        }
                    }
                    else
                    {
                        MessageBox.Show(Resources.CONNECTION_FAILED, Resources.GENEPATTREN_MSG_HEADER,
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        /// <summary>
        /// Shows the server connection form with GenePattern Public Server Settings.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="editMode">if set to <c>true</c> [edit mode].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        public void ShowGenePatternServerConnectionForm(Connection connection, bool editMode)
        {
            CheckFirstTimeRunning();
            
            using (ServerConnection form = new ServerConnection())
            {
                if (DialogResult.OK == form.ShowDialog())
                {
                    if (form.IsConnectionSuccessful)
                    {
                        if (editMode)
                        {
                            string connectionName = connection.Name;
                            //User edited a existing connection. Remove the old one and add a new one
                            RemoveConnection(connectionName);
                            userData.Connections.Add(form.Connection.Name, form.Connection);
                            string currentConnection = userData.CurrentConnectionName;
                            if ((!String.IsNullOrEmpty(currentConnection)) && currentConnection.Equals(connectionName))
                            {
                                userData.CurrentConnectionName = form.Connection.Name;
                                //Will also cause the View Connections user control to be refreshed
                                CurrentConnection = form.Connection;
                            }
                            else
                            {
                                this.UpdateViewConnectionsTaskPane(false);
                            }
                            UpdateConnectionTaskPaneDirtyStatus();
                        }
                        else //New Connetion. Set it as default
                        {
                            // test connection before setting it as the defeault
                            CurrentConnection = form.Connection;
                            UpdateConnectionTaskPaneDirtyStatus();
                        }
                        RerunPipelineTaskPane rerunPipelineTaskPane = this.GetActiveRerunPipelineTaskPane(DocumentKey);
                        if (rerunPipelineTaskPane != null)
                        {
                            rerunPipelineTaskPane.UpdateConnectionsComboBox();
                        }
                    }
                    else
                    {
                        MessageBox.Show(Resources.CONNECTION_FAILED, Resources.GENEPATTREN_MSG_HEADER,
                            MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        /// <summary>
        /// Removes the connection.
        /// </summary>
        /// <param name="connectionName">Name of the connection.</param>
        /// <returns></returns>
        public bool RemoveConnection(string connectionName)
        {
            bool isRemoved = userData.Connections.Remove(connectionName);
            if (isRemoved)
            {
                this.SaveUserData();
                UpdateActiveConnectionList(connectionName);
                UpdateConnectionTaskPaneDirtyStatus();
            }
            return isRemoved;
        }

        /// <summary>
        /// Updates the active connection list.
        /// </summary>
        /// <param name="connectionName">Name of the connection.</param>
        private void UpdateActiveConnectionList(string connectionName)
        {
            IDictionaryEnumerator iter = _controlStates.GetEnumerator();
            while (iter.MoveNext())
            {
                if (string.Compare((iter.Value as ControlState).DropDownActiveConnection, connectionName) == 0)
                {
                    (iter.Value as ControlState).DropDownActiveConnection = string.Empty;
                }
            }
        }

        /// <summary>
        /// Updates the connection pane status.
        /// </summary>
        private void UpdateConnectionTaskPaneDirtyStatus()
        {
            IDictionaryEnumerator iter = _controlStates.GetEnumerator();
            while (iter.MoveNext())
            {
                if (string.Compare(iter.Key.ToString(), host.ActiveDocumentKey) == 0)
                {
                    (iter.Value as ControlState).IsConnectionPaneUpdated = true;
                }
                else
                {
                    (iter.Value as ControlState).IsConnectionPaneUpdated = false;
                }
            }
        }

        /// <summary>
        /// Updates the view connections task pane.
        /// </summary>
        /// <param name="updateTasksView">if set to <c>true</c> [update tasks view].</param>
        private void UpdateViewConnectionsTaskPane(bool updateTasksView)
        {
            ViewConnectionsUserControl viewConnectionsControl = GetViewConnectionsUserControl();
            if (viewConnectionsControl != null)
            {
                viewConnectionsControl.UpdateConnectionsList(updateTasksView);

            }
        }

        /// <summary>
        /// Updates the pane on connection changed.
        /// </summary>
        /// <param name="newConnection">The new connection.</param>
        private void UpdatePaneOnConnectionChanged(Connection newConnection)
        {
            string documentKey = DocumentKey;
            AssociatePipelineTaskPane associatePipelineTaskPane = GetPaneUserControl(TaskPaneKey.AssociatePipeline, documentKey);
            if (associatePipelineTaskPane != null)
            {
                associatePipelineTaskPane.UpdateConnection(newConnection);
            }

            AssociatePipelineTaskPane editPipelineTaskPane = GetPaneUserControl(TaskPaneKey.EditPipeline, documentKey);
            if (editPipelineTaskPane != null)
            {
                editPipelineTaskPane.UpdateConnection(newConnection);
            }

            AssociatePipelineTaskPane insertGenePatternlink = GetPaneUserControl(TaskPaneKey.InsertGenePatternLink, documentKey);
            if (insertGenePatternlink != null)
            {
                insertGenePatternlink.UpdateConnection(newConnection);
            }
        }

        /// <summary>
        /// Updates the view connections task pane.
        /// </summary>
        private void UpdateViewConnectionsTaskPane()
        {
            UpdateViewConnectionsTaskPane(true);
        }

        /// <summary>
        /// Gets the gene pattern server instance
        /// </summary>
        /// <returns>the instance</returns>
        public IGenePatternServer GetGenePatternServer()
        {
            IGenePatternServer genePatternServer = new GenePatternServer();

            genePatternServer.Connection = CurrentConnection;

            return genePatternServer;
        }

        /// <summary>
        /// Returns true if there is no active documenyt
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public bool ActiveDocumentIsNull
        {
            get
            {
                try
                {
                    return host.ActiveDocumentIsNull;
                }
                catch
                {
                    return true;
                }
            }
        }

        /// <summary>
        /// Returns true if the currently active document is protected
        /// </summary>
        public bool DocumentIsProtected
        {
            get
            {
                return host.CurrentDocumentProtected;
            }
        }

        /// <summary>
        /// Inserts the image.
        /// </summary>
        /// <param name="image">The image.</param>
        public void InsertImage(Image image)
        {
            try
            {
                host.InsertImage(image);
            }
            catch (InvalidOperationException)
            {
            }
        }

        /// <summary>
        /// Inserts the picture.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        public void InsertPicture(string filePath)
        {
            try
            {
                host.InsertPicture(filePath);
            }
            catch (InvalidOperationException)
            {
            }
        }


        /// <summary>
        /// Inserts and image into the document and then sets the thumbnail for the association to that image
        /// </summary>
        /// <param name="thumbnailImage">The thumbnail image.</param>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        public void InsertImageWithAssociation(Image thumbnailImage, PipelineAssociation pipelineAssociation)
        {
            if (pipelineAssociation == null)
                return;

            host.InsertImage(thumbnailImage);
            pipelineAssociation.ThumbnailImage = host.GetSelectedImageThumb;
        }

        /// <summary>
        /// Inserts the gene pattern link image.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        public Image InsertGenePatternLinkImage(bool showWizard, PipelineAssociation existingAssociationInsteadOfWizard)
        {
            CheckFirstTimeRunning();
            // if we are showing the wizard, the document can't be protected
            if (showWizard && CheckDocumentProtection())
            {
                return null;
            }

            List<PipelineAssociation> associations = null;

            // associate a pipeline with our new image

            if (showWizard)
            {
                associations = ShowAssociatePipelineWizard(true);
            }
            else
            {
                associations = new List<PipelineAssociation>();
                associations.Add(existingAssociationInsteadOfWizard);
            }
            if (associations != null && associations.Count > 0)
            {
                // insert (and select) image
                string assocName = associations[0].Name;
                Image image = FormatLinkImage(assocName);
                //Image image = Resources.icon_GPLINK;

                try
                {
                    updateAssociationList = false;
                    host.InsertImage(image);
                    DocumentImage docImage = host.GetSelectedImage();
                    Image thumb = host.GetSelectedImageThumb;
                    foreach (PipelineAssociation pa in associations)
                    {
                        if (docImage != null)
                            docImage.IsGenePatternLink = true;

                        pa.PipelineDocumentImage = docImage;
                        pa.ThumbnailImage = thumb;
                    }
                    updateAssociationList = true;
                    
                    // now that we assigned the proper docImage we can refresh the associations pane to get the details
                    UpdatePipelineAssociationsDisplay(docImage);
                }
                catch (InvalidOperationException ex)
                {
                    string clipboardError = Resources.CLIPBOARD_ERROR_MESSAGE;
                    MessageBox.Show(clipboardError, "Image Insert Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Trace.WriteLine("Clipboard error: " + ex.StackTrace);
                }
                catch (Exception)
                {
                    throw;
                }
                finally
                {
                    updateAssociationList = true;
                }
            }


            return host.GetSelectedImageThumb;
        }

        /// <summary>
        /// Inserts the gene pattern link from pane.
        /// </summary>
        /// <param name="pipelineAssociations">The pipeline associations.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="range">The range.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public void InsertGenePatternLinkFromPane(List<PipelineAssociation> pipelineAssociations,
            string documentKey, ImageRange range)
        {
            if (pipelineAssociations != null && pipelineAssociations.Count > 0)
            {
                // insert (and select) image
                string assocName = pipelineAssociations[0].Name;
                Image image = FormatLinkImage(assocName);

                try
                {
                    updateAssociationList = false;

                    ImageRange imageRange = host.InsertImage(image, documentKey, range);
                    DocumentImage docImage = host.GetSelectedImage(documentKey, imageRange);
                    Image thumb = GetSelectedImageThumb(image);

                    foreach (PipelineAssociation pa in pipelineAssociations)
                    {
                        if (docImage != null)
                            docImage.IsGenePatternLink = true;

                        pa.PipelineDocumentImage = docImage;
                        pa.ThumbnailImage = thumb;
                    }
                    updateAssociationList = true;

                    // now that we assigned the proper docImage we can refresh the associations pane to get the details
                    if (docImage != null)
                    {
                        UpdatePipelineAssociationsDisplay(docImage, documentKey);
                    }
                    
                }
                catch (InvalidOperationException ex)
                {
                    string clipboardError = Resources.CLIPBOARD_ERROR_MESSAGE;
                    MessageBox.Show(clipboardError, "Image Insert Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    Trace.WriteLine("Clipboard error: " + ex.StackTrace);
                }
                catch (Exception)
                {
                    throw;
                }
                finally
                {
                    updateAssociationList = true;
                }
            }
        }

        /// <summary>
        /// Toggles the document protection.
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public bool ToggleDocumentProtection()
        {
            CheckFirstTimeRunning();


            if (host.CurrentDocumentProtected)
            {
                // acquire password
                string password = String.Empty;

                using (EnterPasswordForm pf = new EnterPasswordForm())
                {
                    pf.ShowDialog();
                    password = pf.PasswordEntered;
                }
                return host.UnprotectDocument(password);
            }
            else
            {
                //Fix for BUG 1276
                //If we call DataStore.SerializeModel(..) after
                //protecting the document then this function will fail. 
                //So we are calling this function below, so that pipeline
                //related information will be seralized correctly.
                try
                {
                    SaveUserData();
                    DataStore.SerializeModel(ActiveModel);
                    // make sure this is the last line so we can remove traces of the add-in if there's nothing to serialize
                    CleanDataStore();
                }
                catch (COMException)
                {
                    // no worries, document was closing or not docx or something else we can ignore because it happened outside our code
                }
                catch (Exception ex)
                {
                    if (ex.Message.ToLower(CultureInfo.InvariantCulture).Contains("doc key was null"))
                    {
                        // couldn't generate doc key. ignore it.
                    }
                    else
                    {
                        MessageForm.ShowUnhandledExceptionMessage(ex);
                    }
                }

                // acquire password
                string password = String.Empty;

                using (PasswordForm pf = new PasswordForm())
                {
                    DialogResult res = pf.ShowDialog();
                    password = pf.PasswordEntered;
                    if (res == DialogResult.OK)
                    {
                        //generate a doc key before we protect it
                        return host.ProtectDocument(password);
                    }
                    else
                    {
                        return false;
                    }

                }
            }
        }

        /// <summary>
        /// Gets the associated pipelines for selected image.
        /// </summary>
        /// <returns>
        /// a <c>List&lt;PipelineAssociation&gt;</c> of the pipeline associations
        /// associated with the image
        /// </returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<PipelineAssociation> GetAssociatedPipelinesForSelectedImage()
        {
            DocumentImage documentImage = host.GetSelectedImage();
            return GetAssociatedPipelines(documentImage, ActiveModel);
        }


        /// <summary>
        /// Gets the associated pipelines for selected image.
        /// </summary>
        /// <param name="documentImage">The document image.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<PipelineAssociation> GetAssociatedPipelinesForSelectedImage(DocumentImage documentImage, string documentKey)
        {
            if (documentImage == null || string.IsNullOrEmpty(documentKey))
                return GetAssociatedPipelinesForSelectedImage();
           
            return GetAssociatedPipelines(documentImage, GetDocumentModel(documentKey));
        }

        /// <summary>
        /// Deletes the <c>PipelineAssociation</c> from the Model
        /// </summary>
        /// <param name="assoc"></param>
        public void DeletePipelineAssociation(PipelineAssociation assoc)
        {
            ActiveModel.DeletePipelineAssociation(assoc);
            UpdatePipelineAssociationsDisplay(host.GetSelectedImage());
            host.IsDocumentSaved = false;
        }

        /// <summary>
        /// Begins the model changes transaction.
        /// </summary>
        public void BeginModelChanges()
        {
            Debug.Assert(!ActiveModel.TransactionActive, "Transaction already active on Active Model");
            ActiveModel.BeginTransaction();
        }

        /// <summary>
        /// Begins the model changes.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        public void BeginModelChanges(string documentKey)
        {
            Debug.Assert(!GetDocumentModel(documentKey).TransactionActive, "Transaction already active on Active Model");
            GetDocumentModel(documentKey).BeginTransaction();
        }

        /// <summary>
        /// Rollbacks the model changes.
        /// </summary>
        public void RollbackModelChanges()
        {
            Model activeModel = models[host.ActiveDocumentKey];
            Debug.Assert(activeModel != null, "Active model is null or is'nt in a transaction and can't be rolled back");

            Debug.Assert(activeModel.TransactionActive, "No Transaction active on Active Model");
            models[host.ActiveDocumentKey] = activeModel.RollbackTransaction();
        }

        /// <summary>
        /// Rollbacks the model changes.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        public void RollbackModelChanges(string documentKey)
        {
            if (models.ContainsKey(documentKey))
            {
                Model activeModel = models[documentKey];
                Debug.Assert(activeModel != null, "Active model is null or is'nt in a transaction and can't be rolled back");
                Debug.Assert(activeModel.TransactionActive, "No Transaction active on Active Model");
                models[documentKey] = activeModel.RollbackTransaction();
            }

            SetJobProcessingStatus(documentKey, false);
        }

        /// <summary>
        /// Commits the model changes.
        /// </summary>
        public void CommitModelChanges()
        {
            Debug.Assert(ActiveModel.TransactionActive, "No Transaction active on Active Model");
            ActiveModel.CommitTransaction();
        }

        /// <summary>
        /// Commits the model changes.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        public void CommitModelChanges(string documentKey)
        {
            Debug.Assert(GetDocumentModel(documentKey).TransactionActive, "No Transaction active on Active Model");
            GetDocumentModel(documentKey).CommitTransaction();

            if (_documentJobProcessingStatus.ContainsKey(documentKey))
            {
                _documentJobProcessingStatus[documentKey].IsJobProcessingInProgress = false;
            }
        }

        /// <summary>
        /// Submits the job.
        /// </summary>
        /// <param name="job">The job.</param>
        /// <param name="connection">The connection to use for the rerun</param>
        /// <param name="registerNotifcation">if set to <c>true</c> register notifcation.</param>
        /// <param name="runInOriginalLocation">if set to <c>true</c> [run in original location].</param>
        /// <param name="emailAddress">The email address.</param>
        /// <returns>the id of the new job</returns>
        public int SubmitJob(Job job, Connection connection, bool registerNotifcation,
                           string emailAddress, bool runInOriginalLocation)
        {
            return SubmitJob(job, connection, registerNotifcation, emailAddress,
                             runInOriginalLocation, null);
        }

        /// <summary>
        /// Submits the job.
        /// </summary>
        /// <param name="job">The job.</param>
        /// <param name="connection">The connection to use for the rerun</param>
        /// <param name="registerNotifcation">if set to <c>true</c> register notifcation.</param>
        /// <param name="emailAddress">The email address.</param>
        /// <param name="runInOriginalLocation">if set to <c>true</c> [run in original location].</param>
        /// <param name="submitProgress">The submit progress.</param>
        /// <returns>the id of the new job</returns>
        public int SubmitJob(Job job, Connection connection, bool registerNotifcation,
                           string emailAddress, bool runInOriginalLocation,
                           SubmitProgress submitProgress)
        {

            int result = -1;

            CheckParameters(connection, job);
            FormatJobParameters(job.Parameters, string.Empty);

            IGenePatternServer genePatternServer = ServerBroker.GetGenePatternServerForConnection(connection);
            try
            {
                if (genePatternServer != null)
                {
                    Pipeline pipeline = ActiveModel.Pipelines[job.PipelineLsid];
                    InstallPipelines(genePatternServer, pipeline, runInOriginalLocation, submitProgress, job.StopPoint);
                    result = genePatternServer.RunPipeline(pipeline, job.Parameters, job.StopPoint);

                    RegisterForNotification(emailAddress, genePatternServer, registerNotifcation, result);


                }
            }
            catch (PipelineOrModuleNotInstalledException exception)
            {
                Trace.WriteLine("Unable to install Pipeline. Eror was : " + exception.Message, TraceLevel.Error.ToString());
                throw;
            }
            finally
            {
                #region Clear the DataFile, since the file size can be large
                genePatternServer = null;
                Type type;
                for (int i = 0; i < job.Parameters.Count; i++)
                {
                    type = job.Parameters[i].ParamValue.GetType();
                    if (type.Equals(typeof(InputFile)))
                    {
                        job.Parameters[i].ParamValue = job.Parameters[i].PreviousParamValue;
                    }
                }
                GC.Collect();
                #endregion
            }
            return result;
        }
        /// <summary>
        /// Submits the job.
        /// </summary>
        /// <param name="job">The job.</param>
        /// <param name="connection">The connection.</param>
        /// <param name="registerNotifcation">if set to <c>true</c> register notifcation.</param>
        /// <param name="emailAddress">The email address.</param>
        /// <param name="runInOriginalLocation">if set to <c>true</c> [run in original location].</param>
        /// <param name="submitProgress">The submit progress.</param>
        /// <param name="documentKey">Document Key To Get the document from which submit job was invoked</param>
        /// <returns>the id of the new job</returns>
        public int SubmitJob(Job job, Connection connection, bool registerNotifcation,
                    string emailAddress, bool runInOriginalLocation, SubmitProgress submitProgress, string documentKey)
        {
            int result = -1;

            CheckParameters(connection, job);
            FormatJobParameters(job.Parameters, documentKey);

            IGenePatternServer genePatternServer = ServerBroker.GetGenePatternServerForConnection(connection);
            try
            {
                if (genePatternServer != null)
                {
                    Pipeline pipeline = GetDocumentModel(documentKey).Pipelines[job.PipelineLsid];
                    InstallPipelines(genePatternServer, pipeline, runInOriginalLocation, submitProgress, job.StopPoint);
                    result = genePatternServer.RunPipeline(pipeline, job.Parameters, job.StopPoint);

                    RegisterForNotification(emailAddress, genePatternServer, registerNotifcation, result);

                }
            }
            catch (PipelineOrModuleNotInstalledException exception)
            {
                Trace.WriteLine("Unable to install Pipeline. Eror was : " + exception.Message, TraceLevel.Error.ToString());
                throw;
            }
            finally
            {
                #region Clear the DataFile, since the file size can be large
                genePatternServer = null;
                Type type;
                for (int i = 0; i < job.Parameters.Count; i++)
                {
                    type = job.Parameters[i].ParamValue.GetType();
                    if (type.Equals(typeof(InputFile)))
                    {
                        job.Parameters[i].ParamValue = job.Parameters[i].PreviousParamValue;
                    }
                }
                #endregion
            }
            return result;
        }

        /// <summary>
        /// Installs the pipelines.
        /// </summary>
        /// <param name="genePatternServer">The gene pattern server.</param>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="runInOriginalLocation">if set to <c>true</c> [run in original location].</param>
        /// <param name="submitProgress">The submit progress.</param>
        private void InstallPipelines(IGenePatternServer genePatternServer, Pipeline pipeline, 
                    bool runInOriginalLocation, SubmitProgress submitProgress, int stopPoint)
        {
            PipelineInstallationHelper pipelineInstallationHelper = new PipelineInstallationHelper();
            pipelineInstallationHelper.InstallPipelines(genePatternServer, pipeline, runInOriginalLocation,
                        submitProgress, this, stopPoint);
        }

        /// <summary>
        /// Registers for notification.
        /// </summary>
        /// <param name="emailAddress">The email address.</param>
        /// <param name="genePatternServer">The gene pattern server.</param>
        /// <param name="registerNotifcation">if set to <c>true</c> [register notifcation].</param>
        /// <param name="result">The result.</param>
        private static void RegisterForNotification(string emailAddress, IGenePatternServer genePatternServer, bool registerNotifcation, int result)
        {
            if (registerNotifcation && !string.IsNullOrEmpty(emailAddress))
            {
                genePatternServer.RegisterForJobNotification(emailAddress, result);
            }
        }

        /// <summary>
        /// Checks the parameters.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="job">The job.</param>
        private static void CheckParameters(Connection connection, Job job)
        {
            if (job == null)
                throw new ArgumentNullException("job");

            if (connection == null)
                throw new ArgumentNullException("connection");
        }

        /// <summary>
        /// Formats the job parameters.
        /// </summary>
        /// <param name="jobParameters">The job parameters.</param>
        /// <param name="documentKey">The document key.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void FormatJobParameters(List<JobParameter> jobParameters, string documentKey)
        {

            try
            {
                // if job parameter is a "file" then we have to convert
                // it to  "InputFile"
                for (int i = 0; i < jobParameters.Count; i++)
                {
                    Type type = jobParameters[i].ParamValue.GetType();

                    //File exists in Document
                    if (type.Equals(typeof(InternalFile)))
                    {
                        jobParameters[i].PreviousParamValue = jobParameters[i].ParamValue;
                        InternalFile internalFile = jobParameters[i].ParamValue as InternalFile;
                        InputFile inputFile = new InputFile();
                        inputFile.IsLinked = false;
                        inputFile.Name = internalFile.Filename;
                        inputFile.Path = internalFile.FullPath;

                        string temporaryFilePath = this.DataStore.GetFileName(internalFile.Guid, string.Empty, documentKey);
                        inputFile.Contents = "File?" + temporaryFilePath + "?Document";
                        jobParameters[i].ParamValue = inputFile;
                    }
                    //File exists in Hard Drive
                    else if (type.Equals(typeof(LocalFile)))
                    {
                        jobParameters[i].PreviousParamValue = jobParameters[i].ParamValue;
                        LocalFile localFile = jobParameters[i].ParamValue as LocalFile;
                        InputFile inputFile = new InputFile();
                        inputFile.IsLinked = false;
                        inputFile.Name = localFile.Filename;
                        inputFile.Path = localFile.FullPath;
                        jobParameters[i].ParamValue = inputFile;
                        inputFile.Contents = "File?" + localFile.FullPath;
                    }
                    //File is an URI
                    else if (type.Equals(typeof(SerializableUri)))
                    {
                        jobParameters[i].PreviousParamValue = jobParameters[i].ParamValue;
                        SerializableUri serializableUri = jobParameters[i].ParamValue as SerializableUri;
                        InputFile inputFile = new InputFile();
                        inputFile.IsLinked = true;
                        inputFile.Name = serializableUri.Uri.AbsolutePath;
                        inputFile.Path = serializableUri.Uri.AbsoluteUri;
                        jobParameters[i].ParamValue = inputFile;
                    }
                    else if (jobParameters[i].Optional)
                    {
                        jobParameters[i].PreviousParamValue = jobParameters[i].ParamValue;
                        //InputFile inputFile = new InputFile();
                        //inputFile.IsLinked = true;
                        //inputFile.Name = "";
                        //inputFile.Path = "";
                        //jobParameters[i].ParamValue = inputFile;
                    }
                }
            }
            catch (UriFormatException)
            {
                //MessageBox.Show(Resources.BAD_URI_ERROR, Resources.GENEPATTREN_MSG_HEADER,
                //    MessageBoxButtons.OK, MessageBoxIcon.Error);
                throw;
            }
            catch (UnauthorizedAccessException)
            {
                //MessageBox.Show(Resources.FILE_ACCESS_ERROR, Resources.GENEPATTREN_MSG_HEADER,
                //    MessageBoxButtons.OK, MessageBoxIcon.Error);
                throw;
            }
        }

        /// <summary>
        /// Adds the appendix entry.
        /// </summary>
        /// <param name="association">The association.</param>
        /// <param name="showParams">if set to <c>true</c> [show params].</param>
        /// <param name="comment">The comment.</param>
        /// <param name="resultsToEmbed">The results to embed.</param>
        /// <param name="isImage">if set to <c>true</c> [is image].</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object,System.Object)"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public string[] AddAppendixEntry(PipelineAssociation association, bool insertDataFiles, string comment,
            List<Result> resultsToEmbed, bool isImage, string documentKey)
        {
            LogTime("Began adding appendix entry at ");

            #region Form AppendixTitle

            string previousAssociationName = association.Name;
            if (previousAssociationName.EndsWith(" Rerun"))
            {
                previousAssociationName = previousAssociationName.Substring(0, previousAssociationName.Length - 6);
            }

            //"Rerun of {0} added at {1}"
            string title = String.Format(Resources.APPENDIX_TITLE, previousAssociationName);

            #endregion

            #region Form Parameter details to be into Appendix

            Pipeline pipeline = ActiveModel.Pipelines[association.PipelineLsid];
            List<AppendixParameters> appendixParameters = new List<AppendixParameters>();

            foreach (PipelineParameter pp in pipeline.Parameters)
            {
                foreach (JobParameter jobParameter in association.OriginalRerunParameters)
                {
                    if (jobParameter.Name == pp.Name)
                    {
                        Type type = jobParameter.ParamValue.GetType();
                        AppendixParameters appendixParameter =
                            new AppendixParameters(pp.ShortName, jobParameter.ParamValue.ToString());

                        if (insertDataFiles)
                        {
                            if (type.Equals(typeof(InternalFile)))
                            {
                                appendixParameter.IsDocumentFile = true;
                                InternalFile internalFile = jobParameter.ParamValue as InternalFile;
                                appendixParameter.FilePath = docStore.GetFileName(internalFile.Guid, internalFile.Filename, documentKey);
                                appendixParameter.IsInputFile = true;
                            }
                            else if (type.Equals(typeof(LocalFile)))
                            {
                                LocalFile localFile = jobParameter.ParamValue as LocalFile;
                                appendixParameter.FilePath = localFile.FullPath;
                                appendixParameter.IsInputFile = true;
                            }
                        }
                        appendixParameters.Add(appendixParameter);
                        break;
                    }
                }
            }

            #endregion

            #region Insert Result Into Appendix

            string[] resultsToEmbedArray = null;

            if (resultsToEmbed != null)
            {
                resultsToEmbedArray = new string[resultsToEmbed.Count];
                string tempDirectory = DefaultSetting.Path;//""; // can we just use the local path?  that would be sweet.

                for (int i = 0; i < resultsToEmbed.Count; i++)
                {
                    //write to temp directory, add filename to array
                    Result r = resultsToEmbed[i];
                    string tempPath = Path.Combine(tempDirectory, r.Filename);
                    using (FileStream fs = new FileStream(tempPath, FileMode.Create))
                    {
                        fs.Write(r.Content, 0, r.Content.Length);
                    }
                    resultsToEmbedArray[i] = tempPath;
                }
            }

            try
            {
                updateAssociationList = false;
                LogTime("Beginning host appendix entry addition ");

                host.AddAppendixEntry(title, comment, appendixParameters, resultsToEmbedArray,
                        association.PipelineDocumentImage, previousAssociationName, isImage);

                LogTime("Done host appendix entry addition ");
            }
            catch (Exception)
            {
                throw;
            }
            finally
            {
                updateAssociationList = true;
            }

            #endregion

            return resultsToEmbedArray;
        }

        /// <summary>
        /// Deletes appendix objects/images from document
        /// </summary>
        public void DeleteAppendix()
        {
            host.DeleteAppendixEntry();
        }

        /// <summary>
        /// Logs the time.
        /// </summary>
        /// <param name="description">The description.</param>
        private static void LogTime(string description)
        {
            Trace.WriteLine(description + " " + DateTime.Now);
        }

        /// <summary>
        /// Extracts the module.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <param name="path">The path.</param>
        public void ExtractModule(string lsid, string path, string documentKey)
        {
            // it might be stored as loose files, it might be a zip -- this is some extra work
            // but now we know it's a zip so we can unzip it
            using (MemoryStream zipStream = GetPipelineZip(lsid, documentKey))
            {
                ExtractModule(zipStream, path);
            }
        }

        /// <summary>
        /// Extracts the module.
        /// </summary>
        /// <param name="zipStream">The zip stream.</param>
        /// <param name="path">The path.</param>
        /// extracts all files for the given module to the given path
        public void ExtractModule(MemoryStream zipStream, string path)
        {
            //write a temp file from this stream 
            using (FileStream tempFileStream = new FileStream(path + "sourcefile.zip", FileMode.Create))
            {
                byte[] buffer = new byte[StreamHelper.BufferLength];
                StreamCopy(zipStream, tempFileStream, buffer);
                buffer = null;
            }

            using (ZipFile zipFile = new ZipFile(path + "sourcefile.zip"))
            {
                foreach (ZipEntry zipEntry in zipFile)
                {
                    if (!zipEntry.IsDirectory)
                    {
                        //only write files that don't already exist
                        if (!File.Exists(path + zipEntry.FileName))
                        {
                            using (FileStream outputFile = new FileStream(path + zipEntry.FileName, FileMode.Create))
                            {
                                zipEntry.Extract(outputFile);
                            }
                        }
                    }
                }
            }

            try
            {
                File.Delete(path + "sourcefile.zip");
            }
            catch (IOException)
            {
                // if we can't delete the temp source file, so be it.
            }
        }

        /// <summary>
        /// Scrolls to association.
        /// </summary>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        public void ScrollToAssociation(PipelineAssociation pipelineAssociation)
        {
            if (pipelineAssociation == null)
                return;

            host.ScrollToImage(pipelineAssociation.PipelineDocumentImage);
        }

        /// <summary>
        /// Scrolls to association.
        /// </summary>
        /// <param name="documentImage">The document image.</param>
        /// <param name="documentKey">The document key.</param>
        public void ScrollToImage(DocumentImage documentImage, string documentKey)
        {
            if (documentImage == null)
                return;

            DocumentImage tempDocumentImage = host.GetSelectedImage();
            if (tempDocumentImage != null)
            {
                string imageKey = tempDocumentImage.ImageKey;

                if (string.Compare(imageKey, documentImage.ImageKey, true) == 0)
                    return;
            }

            host.ScrollToImage(documentImage, documentKey);
        }

        /// <summary>
        /// Signals the rerun is present.
        /// </summary>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        public void SignalRerunIsPresent(PipelineAssociation pipelineAssociation)
        {
            AssociatedPipelinesUserControl control = GetActiveAssociatedPipelinesUserControl();
            if (control == null)
                return;

            if (control.InvokeRequired)
            {
                control.Invoke(new ControllerSignalRerunIsPresent(control.SignalRerunIsPresent), pipelineAssociation);
            }
            else
                control.SignalRerunIsPresent(pipelineAssociation);
        }

        /// <summary>
        /// Signal Rerun IsPresent Delegate
        /// </summary>
        public delegate void ControllerSignalRerunIsPresent(PipelineAssociation pipelineAssociation);

        /// <summary>
        /// Gets the document model.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        public Model GetDocumentModel(string documentKey)
        {
            Model model = null;
            if (models.ContainsKey(documentKey))
            {
                model = models[documentKey];
            }
            else
            {
                //Check to see if we have model stored in the Document
                model = docStore.DeserializeModel();

                //If no model exists, we create a new one
                if (model == null)
                {
                    model = new Model();
                }
                models.Add(documentKey, model);
            }
            return model;
        }

        /// <summary>
        /// Determines whether this instance can associate.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if this instance can associate; otherwise, <c>false</c>.
        /// </returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        public bool CanPipelineAssociate(bool isLinked)
        {
            CheckFirstTimeRunning();
            if (CheckDocumentProtection())
            {
                return false;
            }
            if (!isLinked)
            {
                DocumentImage documentImage = host.GetSelectedImage();
                if (documentImage == null)
                {
                    Trace.WriteLine("No image is selected to be associated to a pipeline", TraceLevel.Info.ToString());
                    MessageBox.Show(Resources.NO_IMAGE_SELECTED, Resources.NO_IMAGE_CAPTION,
                                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);

                    return false;
                }
                else if (documentImage.AppendixImage)
                {
                    MessageBox.Show(Resources.ASSOCITAION_WITH_RERUN_RESULT_NOT_ALLOWED, Resources.GENEPATTREN_MSG_HEADER,
                                  MessageBoxButtons.OK, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1);
                    return false;
                }
                
            }
            return true;
        }

        /// <summary>
        /// Determines whether this instance can rerun.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if this instance can rerun; otherwise, <c>false</c>.
        /// </returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        public bool CanPipelineRerun(bool isLinked)
        {
            CheckFirstTimeRunning();
            if (!isLinked)
            {
                DocumentImage documentImage = host.GetSelectedImage();
                if (documentImage == null)
                {
                    Trace.WriteLine(Resources.NO_IMAGE_SELECTED_FOR_RERUN, TraceLevel.Info.ToString());
                    MessageBox.Show(Resources.NO_IMAGE_SELECTED_FOR_RERUN, Resources.NO_IMAGE_CAPTION,
                                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);

                    return false;
                }
                if (documentImage.AppendixImage)
                {
                    MessageBox.Show(Resources.PIPELINE_RERUN_FROM_RERUN_RESULT_NOT_ALLOWED, Resources.GENEPATTREN_MSG_HEADER,
                    MessageBoxButtons.OK, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1);
                    return false;
                }
            }
            return true;
        }

        /// <summary>
        /// Gets the connection by document key.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        public Connection GetConnectionByDocumentKey(string documentKey)
        {
            string connectionName = string.Empty;

            if (_controlStates.ContainsKey(documentKey))
                connectionName = _controlStates[host.ActiveDocumentKey].DropDownActiveConnection;

            if (string.IsNullOrEmpty(connectionName))
            {
                connectionName = UserData.CurrentConnectionName;
            }

            if (!string.IsNullOrEmpty(connectionName) && UserData.Connections.ContainsKey(connectionName))
                return UserData.Connections[connectionName];
            else
                return null;
        }

        /// <summary>
        /// Switches to wizard UI.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        public bool SwitchToWizardUI()
        {
            if (AnyTaskPaneVisible())
            {
                DialogResult result = MessageBox.Show("Switching to wizard UI will stop the processing if any and will close all the opened task panes. Do you want to continue?",
                    Resources.GENEPATTREN_MSG_HEADER, MessageBoxButtons.OKCancel, MessageBoxIcon.Question);

                if (result == DialogResult.Cancel)
                    return false;
            }

            string documentKey = DocumentKey;

            RemovePane(TaskPaneKey.AssociatePipeline);
            RemovePane(TaskPaneKey.EditPipeline);
            RemovePane(TaskPaneKey.InsertGenePatternLink);
            RemovePane(TaskPaneKey.RerunPipeline);

            // Reset Document Job Execution Progress to false
            if (_documentJobProcessingStatus.ContainsKey(documentKey))
            {
                _documentJobProcessingStatus[documentKey].IsJobProcessingInProgress = false;
            }

            return true;
        }

        /// <summary>
        /// Removes the pane.
        /// </summary>
        /// <param name="paneKey">The pane key.</param>
        private void RemovePane(TaskPaneKey paneKey)
        {
            UserControl control = null;
            string title = userControlCreator.GetTitle(paneKey);
            control = taskPaneManager.GetTaskPaneUserControl(title);
            if (control != null)
            {
                if (paneKey == TaskPaneKey.RerunPipeline)
                {
                    (control as RerunPipelineTaskPane).CancelExecution();
                }
                else
                {
                    (control as AssociatePipelineTaskPane).CancelExecution(false);
                }
                taskPaneManager.RemoveTaskPane(title);
                if (paneKey != TaskPaneKey.EditPipeline)
                {
                    ribbonInvalidator.InvalidateControl(UserControlCreator.GetControlId(paneKey));
                }
            }
        }

        /// <summary>
        /// Determines whether [is task pane visible].
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if [is task pane visible]; otherwise, <c>false</c>.
        /// </returns>
        private bool AnyTaskPaneVisible()
        {

            string title = userControlCreator.GetTitle(TaskPaneKey.AssociatePipeline);
            if (taskPaneManager.IsTaskPaneVisible(title))
                return true;

            title = userControlCreator.GetTitle(TaskPaneKey.EditPipeline);
            if (taskPaneManager.IsTaskPaneVisible(title))
                return true;

            title = userControlCreator.GetTitle(TaskPaneKey.InsertGenePatternLink);
            if (taskPaneManager.IsTaskPaneVisible(title))
                return true;

            title = userControlCreator.GetTitle(TaskPaneKey.RerunPipeline);
            if (taskPaneManager.IsTaskPaneVisible(title))
                return true;

            return false;
        }

        /// <summary>
        /// Determines whether [is task pane visible] [the specified document key].
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <param name="taskPaneKey">The task pane key.</param>
        /// <returns>
        /// 	<c>true</c> if [is task pane visible] [the specified document key]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsTaskPaneVisible(string documentKey, TaskPaneKey taskPaneKey)
        {
            string title = userControlCreator.GetTitle(taskPaneKey);
            return taskPaneManager.IsTaskPaneVisible(title, documentKey);
        }

        /// <summary>
        /// Sets the button status.
        /// </summary>
        /// <param name="controlKey">The control key.</param>
        /// <param name="isPressed">if set to <c>true</c> [is pressed].</param>
        public void SetButtonStatus(ControlKey controlKey, bool isPressed)
        {
            if (_controlStates.ContainsKey(host.ActiveDocumentKey))
                _controlStates[host.ActiveDocumentKey].SetControlState(controlKey, isPressed);
            else
            {
                ControlState controlState = new ControlState();
                controlState.SetControlState(controlKey, isPressed);
                _controlStates.Add(host.ActiveDocumentKey, controlState);
            }
        }

        /// <summary>
        /// Gets the button status.
        /// </summary>
        /// <param name="controlKey">The control key.</param>
        /// <returns></returns>
        public bool GetButtonStatus(ControlKey controlKey)
        {
            if (_controlStates.ContainsKey(host.ActiveDocumentKey))
                return _controlStates[host.ActiveDocumentKey].GetControlState(controlKey);
            else
                return false;
        }

        /// <summary>
        /// Get the active rerunpipeline task pane
        /// </summary>
        /// <param name="documentKey"></param>
        /// <returns>active rerunpipeline task pane</returns>
        public RerunPipelineTaskPane GetActiveRerunPipelineTaskPane(string documentKey)
        {
            UserControl control = null;
            RerunPipelineTaskPane rerunPipelineTaskPane = null;
            string title = userControlCreator.GetTitle(TaskPaneKey.RerunPipeline);
            control = taskPaneManager.GetTaskPaneUserControl(title, documentKey);
            if (control != null)
                return control as RerunPipelineTaskPane;
            else
                return rerunPipelineTaskPane;
        }

        /// <summary>
        /// Cleans the data store.
        /// </summary>
        public void CleanDataStore()
        {
            ReadOnlyCollection<DocumentImage> docImages = host.GetAllImages();
            // go through the pipeline associations and make sure all the images exist
            // also log all their pipeline lsids for reference checking
            List<int> deletePAIds = new List<int>();
            List<string> deletePIds = new List<string>();
            List<string> lsids = new List<string>();
            foreach (PipelineAssociation pa in ActiveModel.PipelineAssociations.Values)
            {
                // does image exist in document still?
                if (docImages.Contains(pa.PipelineDocumentImage))
                {
                    if (!lsids.Contains(pa.PipelineLsid))
                    {
                        lsids.Add(pa.PipelineLsid);
                    }
                    lsids.AddRange(GetLsids(pa));
                }
                else
                {
                    deletePAIds.Add(pa.LocalId);
                }
            }
            //delete dead PAs
            foreach (int id in deletePAIds)
            {
                ActiveModel.PipelineAssociations.Remove(id);
            }
            foreach (Pipeline p in ActiveModel.Pipelines.Values)
            {
                if (lsids.Contains(p.Lsid))
                {
                    // add pipeline lsid to the don't-delete list
                    if (!p.IsLinked)
                    {
                        lsids.AddRange(GetLsids(p));
                    }
                }
                else // no one is using the pipeline
                {
                    deletePIds.Add(p.Lsid);
                }
            }
            foreach (string s in deletePIds)
            {
                ActiveModel.Pipelines.Remove(s);
            }
            // If there are no pipeline associataions 
            // remove model completeley
            if (ActiveModel.PipelineAssociations.Count == 0)
            {
                DataStore.RemoveAllContent();
            }
            else
            {
                // compress data store by deleting unused files
                DataStore.Compress(lsids);
            }
        }
        #endregion

        #region Event Handlers

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        internal void host_DocumentSaved(object sender, DocumentSaveEventArgs e)
        {

            try
            {
                if (IsJobRunning())
                {
                    e.CancelSaveDialog = true;
                    return;
                }

                SaveUserData();
                if (!host.CurrentDocumentProtected)
                {
                    DataStore.SerializeModel(ActiveModel);
                    // make sure this is the last line so we can remove traces of the add-in if there's nothing to serialize
                    CleanDataStore();
                }
            }
            catch (COMException)
            {
                // no worries, document was closing or not docx or something else we can ignore because it happened outside our code
            }
            catch (Exception ex)
            {
                if (ex.Message.ToLower(CultureInfo.InvariantCulture).Contains("doc key was null"))
                {
                    // couldn't generate doc key because the document was closing.  ignore it.
                }
                else
                {
                    MessageForm.ShowUnhandledExceptionMessage(ex);
                }
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void host_DocumentClose(object sender, EventArgs e)
        {
            // special case -- check for a document key existing  and a model to actually serialize before we do anything so we don't muck up an unchanged doc
            if (host.ActiveDocumentKeyExists && models.ContainsKey(host.ActiveDocumentKey))
            {
                // look for reruns that will be discarded
                bool rerunWillBeLost = false;
                foreach (PipelineAssociation association in ActiveModel.PipelineAssociations.Values)
                {
                    if (association.Rerun.Job.Id > -1 && !association.Rerun.InsertedToAppedix)
                    {
                        rerunWillBeLost = true;
                        break;
                    }
                }
                CancelCloseEventArgs cancelArgs = e as CancelCloseEventArgs;
                if (rerunWillBeLost && cancelArgs != null)
                {
                    string message = Resources.RERUN_DISCARD_WARNING;
                    string caption = Resources.RERUN_DISCARD_WARNING_CAPTION;
                    // do you want to close the document?
                    DialogResult result =
                        MessageBox.Show(message, caption, MessageBoxButtons.YesNo, MessageBoxIcon.Warning);

                    cancelArgs.Cancel = result == DialogResult.No;
                }
            }

            Trace.WriteLine("Document closing", TraceLevel.Info.ToString());
            ThreadPool.QueueUserWorkItem(new WaitCallback(CloseCallback), null);

            this.RemoveTaskPanes();

            if (host.IsDocumentSaved)
            {
                models.Remove(host.ActiveDocumentKey);
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void host_DocumentOpen(object sender, EventArgs e)
        {
            Trace.WriteLine("Document opening", TraceLevel.Info.ToString());

            //Clear a Model Dictionary
            IDictionaryEnumerator iter = models.GetEnumerator();
            while (iter.MoveNext())
            {
                string docKey = (string)iter.Key;
                if (!host.ModelExists(docKey))
                    models.Remove(docKey);
            }

            if (host.CurrentDocumentProtected)
            {
                if (DataStore.DeserializeModel() != null)
                {
                    MessageBox.Show(Resources.DOCUMENT_PROTECTED, Resources.GENEPATTREN_MSG_HEADER,
                        MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }

        }

        /// <summary>
        /// Handles the ImageDoubleClicked event of the host control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Microsoft.eResearch.ScientificAddIn.Common.SelectedImageEventArgs"/> instance containing the event data.</param>
        private void host_ImageDoubleClicked(object sender, SelectedImageEventArgs e)
        {
            DocumentImage selectedImage = e.SelectedImage;
            if (selectedImage != null)
            {
                bool isGenePatternLink = false;
                List<PipelineAssociation> associations = GetAssociatedPipelinesForSelectedImage(selectedImage, DocumentKey);
                foreach (PipelineAssociation association in associations)
                {
                    if (association != null && association.PipelineDocumentImage != null)
                    {
                        if (association.PipelineDocumentImage.IsGenePatternLink)
                        {
                            isGenePatternLink = true;
                            break;
                        }
                    }
                }
                if (isGenePatternLink)
                {
                    string title = userControlCreator.GetTitle(TaskPaneKey.ViewAssociations);
                    if(!taskPaneManager.IsTaskPaneVisible(title))
                    {
                        ViewAssociationsCommand viewAssociationCommand = new ViewAssociationsCommand(this);
                        viewAssociationCommand.Invoke(true);
                        e.IsTaskPaneVisible = true;
                    }
                }
            }
        }

        /// <summary>
        /// Handles the DocumentActivated event of the host control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void host_DocumentActivated(object sender, EventArgs e)
        {
            
            UpdateRibbonControlState();

            if (_controlStates.ContainsKey(host.ActiveDocumentKey))
            {
                if (!_controlStates[host.ActiveDocumentKey].IsConnectionPaneUpdated)
                {
                    UpdateViewConnectionsTaskPane();
                    RerunPipelineTaskPane rerunPipelineTaskPane = this.GetActiveRerunPipelineTaskPane(host.ActiveDocumentKey);
                    rerunPipelineTaskPane.UpdateConnectionsComboBox();
                    _controlStates[host.ActiveDocumentKey].IsConnectionPaneUpdated = true;

                }
            }
            else
            {
                ControlState controlState = new ControlState();
                controlState.IsConnectionPaneUpdated = true;
                _controlStates.Add(host.ActiveDocumentKey, controlState);
            }
        }

        /// <summary>
        /// Updates the state of the ribbon control.
        /// </summary>
        private void UpdateRibbonControlState()
        {
            string controlID = this.userControlCreator.GetControlId(ControlKey.ConnectionDropDown);
            this.ribbonInvalidator.InvalidateControl(controlID);

            controlID = userControlCreator.GetControlId(ControlKey.WizardFreeCheckBox);
            this.ribbonInvalidator.InvalidateControl(controlID);

            controlID = userControlCreator.GetControlId(ControlKey.ViewAssociationButton);
            this.ribbonInvalidator.InvalidateControl(controlID);

            controlID = userControlCreator.GetControlId(ControlKey.DashBoardButton);
            this.ribbonInvalidator.InvalidateControl(controlID);

            controlID = userControlCreator.GetControlId(ControlKey.AssociatePipelineButton);
            this.ribbonInvalidator.InvalidateControl(controlID);

            controlID = userControlCreator.GetControlId(ControlKey.InsertGenePatternLinkButton);
            this.ribbonInvalidator.InvalidateControl(controlID);

            controlID = userControlCreator.GetControlId(ControlKey.ProtectDocumentButton);
            this.ribbonInvalidator.InvalidateControl(controlID);

            controlID = userControlCreator.GetControlId(ControlKey.RerunPipelineButton);
            this.ribbonInvalidator.InvalidateControl(controlID);

            controlID = userControlCreator.GetControlId(ControlKey.ConnectionsButton);
            this.ribbonInvalidator.InvalidateControl(controlID);
        }

        /// <summary>
        /// If the document is protected, displays a warning message
        /// </summary>
        /// <returns>Whether or not the document is protect and has GP data</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        public bool CheckDocumentProtection()
        {
            if (host.CurrentDocumentProtected)
            {
                MessageBox.Show(Resources.DOCUMENT_PROTECTED_INVALID_OPERATION, Resources.GENEPATTREN_MSG_HEADER,
                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return true;
            }
            return false;
        }

        /// <summary>
        /// Closes the callback.
        /// </summary>
        /// <param name="state">The state.</param>
        private void CloseCallback(object state)
        {
            SaveUserData();
            RibbonInvalidator.InvalidateControl(userControlCreator.GetControlId(TaskPaneKey.ViewAssociations));
            RibbonInvalidator.InvalidateControl(userControlCreator.GetControlId(TaskPaneKey.Dashboard));
            RibbonInvalidator.InvalidateControl(userControlCreator.GetControlId(TaskPaneKey.Connections));
        }

        /// <summary>
        /// Saves the user data.
        /// </summary>
        private void SaveUserData()
        {
            Trace.WriteLine("Saving user data", TraceLevel.Info.ToString());
            if (userData != null)
                UserDataManager.Save(userData);
        }

        /// <summary>
        /// Handles the OnImageSelected event of the host control(which is word in our case).
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Microsoft.eResearch.ScientificAddIn.Common.SelectedImageEventArgs"/> instance containing the event data.</param>
        void host_ImageSelect(object sender, SelectedImageEventArgs e)
        {
            // updated the selected image
            DocumentImage selectedImage = e.SelectedImage;
            if (updateAssociationList && selectedImage != null)
            {
                UpdatePipelineAssociationsDisplay(selectedImage);
            }
        }

        /// <summary>
        /// Handles the ImageDeSelect event of the host control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void host_ImageDeSelect(object sender, EventArgs e)
        {
            DocumentImage temporaryImage = new DocumentImage();
            UpdatePipelineAssociationsDisplay(temporaryImage);
        }

        /// <summary>
        /// Updates the pipeline associations display.
        /// </summary>
        public void UpdatePipelineAssociationsDisplay()
        {
            DocumentImage documentImage = host.GetSelectedImage();
            if (documentImage != null)
                UpdatePipelineAssociationsDisplay(documentImage);
        }

        /// <summary>
        /// Updates the pipeline associations display.
        /// </summary>
        /// <param name="selectedImage">The selected image.</param>
        private void UpdatePipelineAssociationsDisplay(DocumentImage selectedImage)
        {
            List<PipelineAssociation> pipelineAssociationsList = null;
            pipelineAssociationsList = GetAssociatedPipelines(selectedImage, ActiveModel);
            
            if (pipelineAssociationsList != null && selectedImage != null)
            {
                DisplayPipelineAssociations(pipelineAssociationsList, selectedImage.AppendixImage);
            }
            else if (pipelineAssociationsList != null && selectedImage == null)
            {
                DisplayPipelineAssociations(pipelineAssociationsList, false);
            }
        }

        /// <summary>
        /// Updates the pipeline associations display.
        /// </summary>
        /// <param name="selectedImage">The selected image.</param>
        /// <param name="documentKey">The document key.</param>
        public void UpdatePipelineAssociationsDisplay(DocumentImage selectedImage, string documentKey)
        {
            List<PipelineAssociation> pipelineAssociationsList = null;
            pipelineAssociationsList = GetAssociatedPipelines(selectedImage, GetDocumentModel(documentKey));

            AssociatedPipelinesUserControl associatedPipelinesUserControl = GetActiveAssociatedPipelinesUserControl(documentKey);
            if (associatedPipelinesUserControl != null)
            {
                associatedPipelinesUserControl.DisplayPipelineAssociations(pipelineAssociationsList,
                    documentKey, selectedImage.AppendixImage);
            }

            UpdatePaneDocumentJobs(documentKey);

            RerunPipelineTaskPane rerunPipelineTaskPane = GetActiveRerunPipelineTaskPane(documentKey);
            if (rerunPipelineTaskPane != null)
            {
                rerunPipelineTaskPane.UpdateAssociations(selectedImage);
            }

            DashboardUserControl dash = GetActiveDashboardUserControl(documentKey);
            if (dash != null)
            {
                ReadOnlyCollection<PipelineAssociation> pipelineAssociationList = this.GetAllPipelineAssociations(documentKey);
                ICollection<ContentItem> contentItems = this.GetContentItems(pipelineAssociationList, documentKey);
                dash.DisplayPipelineAssociations(pipelineAssociationList, contentItems);
            }
        }

        /// <summary>
        /// Handles the TaskPaneVisibilityChanged event of the taskPaneManager control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Microsoft.eResearch.ScientificAddIn.Common.VisibilityChangedEventArgs"/> instance containing the event data.</param>
        private void taskPaneManager_TaskPaneVisibilityChanged(object sender, VisibilityChangedEventArgs e)
        {
            TaskPaneKey taskPaneKey = UserControlCreator.GetTaskPaneKey(e.TaskPaneTitle);
            if (ribbonInvalidator != null)
            {
                if (taskPaneKey != TaskPaneKey.EditPipeline)
                {
                    ribbonInvalidator.InvalidateControl(UserControlCreator.GetControlId(taskPaneKey));
                }
            }
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Downloads the and store input files.
        /// </summary>
        /// <param name="association">The association.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="genePatternServer">The gene pattern server.</param>
        /// <returns></returns>
        private List<string> DownloadAndStoreInputFiles(PipelineAssociation association, string documentKey,
            IGenePatternServer genePatternServer)
        {
            List<string> messages = new List<string>();
            Stream file;

            foreach (JobParameter jobParameter in association.Job.Parameters)
            {
                if (jobParameter.ParamValue is InternalFile)
                {
                    InternalFile internalFile = jobParameter.ParamValue as InternalFile;

                    string filename = null;
                    string fullFolder = Path.GetDirectoryName(internalFile.FullPath);
                    string folder = Path.GetFileName(fullFolder);

                    if (!string.IsNullOrEmpty(folder) && folder.ToLowerInvariant() == "attachments")
                    {
                        // the location of the file is one folder up from the 'tomcat' directory
                        // <install path>/temp/attachments
                        int index = internalFile.FullPath.LastIndexOf("..");
                        filename = @"..\" + internalFile.FullPath.Substring(index);
                    }
                    else if (!string.IsNullOrEmpty(folder) &&
                             !string.IsNullOrEmpty(Path.GetFileName(internalFile.FullPath)))
                    {
                        // the location of the file is in the temp/<folder> directort
                        filename = folder + Path.DirectorySeparatorChar + Path.GetFileName(internalFile.FullPath);

                        //fix for bug 1305
                        //The input parameters can be stored
                        //in one of the two path on Server
                        //[1]: GenePatternServer\Tomcat\temp
                        //[2]: GenePatternServer\temp\attachments

                        if (fullFolder.Contains("temp\\attachments"))
                        {
                            filename = "..\\..\\temp\\attachments\\" + filename;
                        }
                    }
                    if (filename != null)
                    {
                        if (genePatternServer == null)
                            genePatternServer = GetGenePatternServer();

                        file = genePatternServer.GetInputFile(filename);

                        InternalFile storedFile = docStore.GetHashKeyAndStoreFile(file, documentKey);
                        internalFile.Lsid = storedFile.Lsid;
                        internalFile.Length = storedFile.Length;
                        internalFile.Guid = storedFile.Guid;
                        Trace.WriteLine("Downloaded and stored file lsid: " + storedFile.Lsid);

                        file.Dispose();
                        file = null;

                        GC.Collect();
                        GC.WaitForPendingFinalizers();
                    }
                    else
                    {
                        messages.Add(
                            string.Format(CultureInfo.InvariantCulture, Resources.UNABLE_TO_RETRIEVE_INPUT_FILE,
                                          internalFile.Filename, jobParameter.Name));
                        jobParameter.ParamValue = null;
                    }
                }
            }

            if (genePatternServer != null)
                genePatternServer.Dispose();
            return messages;
        }

        /// <summary>
        /// Adds the pipeline files to zip stream.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="zipFile">The zip file.</param>
        private void AddPipelineFilesToZipStream(Pipeline pipeline, ZipFile zipFile)
        {
            // I'm not sure if this is a good check;  are manifest files required?  
            // In any case, pipeline.Manifest file has been null here before
            if (pipeline.ManifestFile != null)
            {
                AddFileToZip(zipFile, pipeline.ManifestFile);
            }
            foreach (InternalFile file in pipeline.Documentation)
            {
                AddFileToZip(zipFile, file);
            }
            foreach (InternalFile file in pipeline.Datafiles)
            {
                AddFileToZip(zipFile, file);
            }
        }

        /// <summary>
        /// Gets the lsids.
        /// </summary>
        /// <param name="pa">The pa.</param>
        /// <returns></returns>
        private List<string> GetLsids(PipelineAssociation pa)
        {
            List<string> lsids = new List<string>();
            // add lsids of input files
            foreach (JobParameter p in pa.Job.Parameters)
            {
                if (p.ParamValue is InternalFile)
                {
                    lsids.Add(((InternalFile)p.ParamValue).Lsid);
                }
            }
            foreach (InternalFile iF in pa.EmbeddedResults)
            {
                lsids.Add(iF.Lsid);
            }
            lsids.AddRange(GetLsids(ActiveModel.Pipelines[pa.PipelineLsid]));
            return lsids;
        }

        /// <summary>
        /// Gets the lsids.
        /// </summary>
        /// <param name="p">The p.</param>
        /// <returns></returns>
        private List<string> GetLsids(Pipeline p)
        {
            // build a list of all lsids that should have backing files
            List<string> lsids = new List<string>();

            if (!p.IsLinked)
            {
                string manifestLsid = p.ManifestFile.Lsid;
                if (!lsids.Contains(manifestLsid))
                {
                    lsids.Add(manifestLsid);
                }
                // get lsids for docs
                foreach (InternalFile internalFile in p.Documentation)
                {
                    string docLsid = internalFile.Lsid;
                    if (!lsids.Contains(docLsid))
                    {
                        lsids.Add(docLsid);
                    }
                }
                // get lsids for data files
                foreach (InternalFile internalFile in p.Datafiles)
                {
                    string docLsid = internalFile.Lsid;
                    if (!lsids.Contains(docLsid))
                    {
                        lsids.Add(docLsid);
                    }
                }
                // add lsid for pipeline
                if (!lsids.Contains(p.Lsid))
                {
                    lsids.Add(p.Lsid);
                }
                // add lsid for steps
                foreach (PipelineStep ps in p.Steps)
                {
                    if (ps.IsPipeline)
                    {
                        lsids.Add(ps.Lsid);
                        // recursively add lsids for child pipelines
                        lsids.AddRange(GetLsids(ActiveModel.Pipelines[ps.Lsid]));
                    }
                    else
                    {
                        // otherwise just add module lsid
                        if (!lsids.Contains(ps.Lsid))
                        {
                            lsids.Add(ps.Lsid);
                        }
                    }
                }
            }

            return lsids;
        }

        /// <summary>
        /// Adds the file to zip.
        /// </summary>
        /// <param name="zipfile">The zipfile.</param>
        /// <param name="file">The file.</param>
        private void AddFileToZip(ZipFile zipfile, InternalFile file)
        {
            byte[] buffer = new byte[StreamHelper.BufferLength];

            // write the internal file into the local dir
            using (FileStream fs = new FileStream(file.Filename, FileMode.OpenOrCreate))
            {
                using (MemoryStream theFile = DataStore.GetFile(file.Guid))
                {
                    StreamCopy(theFile, fs, buffer);
                }

            }

            if(zipfile.EntryFilenames.Contains(file.Filename) == false)
                zipfile.AddFile(file.Filename);
            buffer = null;
        }

        /// <summary>
        /// Adds the stream to zip.
        /// </summary>
        /// <param name="zipfile">The zipfile.</param>
        /// <param name="stream">The stream.</param>
        /// <param name="filename">The filename.</param>
        private static void AddStreamToZip(ZipFile zipfile, Stream stream, string filename)
        {
            // Make sure this zipfile doesn't already have this file in it...
            foreach (ZipEntry entry in zipfile)
            {
                if (entry.FileName.Equals(filename, StringComparison.InvariantCultureIgnoreCase))
                {
                    return;
                }
            }
            byte[] buffer = new byte[StreamHelper.BufferLength];
            // write the internal file into the local dir
            using (FileStream fs = new FileStream(filename, FileMode.Create))
            {
                StreamCopy(stream, fs, buffer);
            }

            zipfile.AddFile(filename);
            buffer = null;
        }

        /// <summary>
        /// Streams the copy.
        /// </summary>
        /// <param name="sourceStream">The source stream.</param>
        /// <param name="destStream">The dest stream.</param>
        /// <param name="buffer">The buffer.</param>
        private static void StreamCopy(Stream sourceStream, Stream destStream, byte[] buffer)
        {
            int offset = 0;
            while (sourceStream.Position < sourceStream.Length)
            {
                int bytesRead = sourceStream.Read(buffer, 0, buffer.Length);
                offset += bytesRead;
                destStream.Write(buffer, 0, bytesRead);
            }
        }

        /// <summary>
        /// Downloads the and store pipeline files.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="callback">The callback.</param>
        /// <param name="dontCallbackForSteps">if set to <c>true</c> [dont callback for steps].</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="genePatternServer">The gene pattern server.</param>
        private void DownloadAndStorePipelineFiles(Pipeline pipeline, DownloadProgress callback,
            bool dontCallbackForSteps, string documentKey, IGenePatternServer genePatternServer)
        {
            NotifyDownloadProgressListeners(callback, pipeline, true);

            ExtractAndStorePipelineFiles(pipeline, documentKey, genePatternServer);
            NotifyDownloadProgressListeners(callback, pipeline, false);
            DownloadProgress newCallback = dontCallbackForSteps ? null : callback;

            // Get the Current Model based on DocumentKey
            Model currentModel = new Model();
            if (string.IsNullOrEmpty(documentKey))
            {
                currentModel = ActiveModel;
            }
            else
            {
                currentModel = GetDocumentModel(documentKey);
            }


            // download files for subpipelines
            foreach (PipelineStep pipelineStep in pipeline.Steps)
            {
                if (pipelineStep.IsPipeline)
                {
                    Pipeline theStep = genePatternServer.GetPipelineByLsid(pipelineStep.Lsid);
                    DownloadAndStorePipelineFiles(theStep, newCallback, true, documentKey, genePatternServer);
                    currentModel.AddPipeline(theStep);
                }
                else //pipeline is module
                {
                    NotifyDownloadProgressListeners(newCallback, pipelineStep, true);
                    DownloadAndStoreModule(pipelineStep.Lsid, documentKey, genePatternServer);
                    NotifyDownloadProgressListeners(newCallback, pipelineStep, false);
                }
            }
        }

        /// <summary>
        /// Downloads the and store pipeline files.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="callback">The callback.</param>
        /// <param name="dontCallbackForSteps">if set to <c>true</c> [dont callback for steps].</param>
        private void DownloadAndStorePipelineFiles(Pipeline pipeline, DownloadProgress callback,
            bool dontCallbackForSteps)
        {
            NotifyDownloadProgressListeners(callback, pipeline, true);
            ExtractAndStorePipelineFiles(pipeline, null, null);
            NotifyDownloadProgressListeners(callback, pipeline, false);

            DownloadProgress newCallback = dontCallbackForSteps ? null : callback;

            // Get the Current Model based on DocumentKey
            Model currentModel = ActiveModel;

            // download files for subpipelines
            foreach (PipelineStep pipelineStep in pipeline.Steps)
            {
                if (pipelineStep.IsPipeline)
                {
                    Pipeline theStep = GetGenePatternServer().GetPipelineByLsid(pipelineStep.Lsid);
                    DownloadAndStorePipelineFiles(theStep, newCallback, true);
                    currentModel.AddPipeline(theStep);
                }
                else //pipeline is module
                {
                    NotifyDownloadProgressListeners(newCallback, pipelineStep, true);
                    DownloadAndStoreModule(pipelineStep.Lsid);
                    NotifyDownloadProgressListeners(newCallback, pipelineStep, false);
                }
            }
        }

        /// <summary>
        /// Notifies the download progress listeners.
        /// </summary>
        /// <param name="callback">The callback.</param>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="isStarting">if set to <c>true</c> [is starting].</param>
        private static void NotifyDownloadProgressListeners(DownloadProgress callback, Pipeline pipeline, bool isStarting)
        {
            if (callback != null) // notify start/end of download if we have listener
            {
                callback(pipeline.Name, pipeline.Lsid, isStarting);
            }
        }

        /// <summary>
        /// Notifies the download progress listeners.
        /// </summary>
        /// <param name="callback">The callback.</param>
        /// <param name="pipelineStep">The pipeline step.</param>
        /// <param name="isStarting">if set to <c>true</c> [is starting].</param>
        private static void NotifyDownloadProgressListeners(DownloadProgress callback, PipelineStep pipelineStep, bool isStarting)
        {
            if (callback != null) // notify start/end of download if we have listener
            {
                callback(pipelineStep.Name, pipelineStep.Lsid, isStarting);
            }
        }

        /// <summary>
        /// Downloads the and store module.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        private void DownloadAndStoreModule(string lsid)
        {
            using (MemoryStream moduleZip = CurrentGenePatternServer.DownloadTask(lsid, false))
            {
                Module theModule = new Module();
                theModule.Lsid = lsid;
                theModule.ZipFile = new InternalFile();
                theModule.ZipFile.Filename = lsid + ".zip";

                InternalFile storedFile = DataStore.StoreFile(moduleZip, lsid);
                theModule.ZipFile.Guid = storedFile.Guid;
                theModule.ZipFile.Length = storedFile.Length;

                ActiveModel.AddModule(theModule);

                theModule = null;
                storedFile = null;

                GC.Collect();
                GC.WaitForPendingFinalizers();
            }
        }

        /// <summary>
        /// Downloads the and store module.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="genePatternServer">The gene pattern server.</param>
        private void DownloadAndStoreModule(string lsid, string documentKey, IGenePatternServer genePatternServer)
        {
            using (MemoryStream moduleZip = genePatternServer.DownloadTask(lsid, false))
            {
                Module theModule = new Module();
                theModule.Lsid = lsid;
                theModule.ZipFile = new InternalFile();
                theModule.ZipFile.Filename = lsid + ".zip";

                InternalFile storedFile = DataStore.StoreFile(moduleZip, lsid, documentKey);
                theModule.ZipFile.Guid = storedFile.Guid;
                theModule.ZipFile.Length = storedFile.Length;

                GetDocumentModel(documentKey).AddModule(theModule);

                theModule = null;
                storedFile = null;

                GC.Collect();
                GC.WaitForPendingFinalizers();
            }
        }

        /// <summary>
        /// Assures the connection.
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        public bool AssureConnection()
        {
            string message;

            if (CurrentConnection == null || !Microsoft.eResearch.GenePattern.Server.GenePatternServer.TestConnection(CurrentConnection, false, out message))
            {
                Trace.WriteLine("Asking user to specify a connection", TraceLevel.Info.ToString());
                if (CurrentConnection != null)
                {
                    MessageBox.Show(Resources.CONNECTION_FAILED_CAPTION + ": " + CurrentConnection.Name, Resources.GENEPATTREN_MSG_HEADER,
                        MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
                ShowServerConnectionForm(null, false);
            }

            if (CurrentConnection == null)
            {
                Trace.WriteLine("No connection is configured", TraceLevel.Error.ToString());
                MessageBox.Show(Resources.NO_CONNECTION, Resources.NO_CONNECTION_CAPTION,
                                MessageBoxButtons.OK, MessageBoxIcon.Exclamation,
                                MessageBoxDefaultButton.Button1);
                return false;
            }

            return true;
        }

        /// <summary>
        /// Removes the task panes.
        /// </summary>
        private void RemoveTaskPanes()
        {
            Trace.WriteLine("Removing taskpanes", TraceLevel.Info.ToString());
            if (!IsWizardUI)
            {
                string documentKey = DocumentKey;

                RemovePane(TaskPaneKey.AssociatePipeline);
                RemovePane(TaskPaneKey.EditPipeline);
                RemovePane(TaskPaneKey.InsertGenePatternLink);
                RemovePane(TaskPaneKey.RerunPipeline);

                // Reset Document Job Execution Progress to false
                if (_documentJobProcessingStatus.ContainsKey(documentKey))
                {
                    _documentJobProcessingStatus[documentKey].IsJobProcessingInProgress = false;
                }
            }
            this.taskPaneManager.RemoveTaskPanes();
        }

        /// <summary>
        /// Extracts the and store pipeline files.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="genePatternServer">The gene pattern server.</param>
        private void ExtractAndStorePipelineFiles(Pipeline pipeline, string documentKey, IGenePatternServer genePatternServer)
        {
            if (genePatternServer == null)
                genePatternServer = CurrentGenePatternServer;

            using (MemoryStream pipelineStream = genePatternServer.DownloadTask(pipeline.Lsid, false))
            {
                PipelineZipExtractor pipelineZipExtractor = new PipelineZipExtractor(docStore);
                List<InternalFile> files = pipelineZipExtractor.GetFiles(pipelineStream, documentKey);
                Dictionary<string, InternalFile> dictionary = new Dictionary<string, InternalFile>();

                foreach (InternalFile file in files)
                {
                    dictionary.Add(file.Filename, file);
                }

                string manifestFilename = Properties.Settings.Default.manifestFilename;
                // remove manifest file if we have one and add to pipeline
                if (dictionary.ContainsKey(manifestFilename))
                {
                    pipeline.ManifestFile = dictionary[manifestFilename];
                    dictionary.Remove(manifestFilename);
                }

                // remove document filenames and add to pipeline
                pipeline.Documentation.Clear();
                foreach (string filename in pipeline.DocumentationFilenames)
                {
                    if (dictionary.ContainsKey(filename))
                    {
                        pipeline.Documentation.Add(dictionary[filename]);
                        dictionary.Remove(filename);
                    }
                }
                // dictionary now contains just the names of the datafiles
                foreach (InternalFile internalFile in dictionary.Values)
                {
                    pipeline.Datafiles.Add(internalFile);
                }

                pipelineZipExtractor = null;
                files = null;
                dictionary = null;
            }
        }

        /// <summary>
        /// Gets the file GUID.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <returns></returns>
        private Guid GetFileGuid(string lsid)
        {
            //TODO:  make this return an InternalFile.  Also, make it more obvious frm the name that it makes a server call.
            Guid fileGuid;
            //Check if LSID already exists 
            if (docStore.ContainsFile(lsid))
            {
                fileGuid = docStore.GetFileGuid(lsid);
            }
            else
            {
                using (MemoryStream memoryStream = CurrentGenePatternServer.DownloadTask(lsid, false))
                {
                    fileGuid = docStore.StoreFile(memoryStream, lsid).Guid;
                }
            }
            return fileGuid;
        }

        /// <summary>
        /// Gets the associated pipelines.
        /// </summary>
        /// <param name="image">The image.</param>
        /// <param name="model">The model.</param>
        /// <returns></returns>
        private static List<PipelineAssociation> GetAssociatedPipelines(DocumentImage image, Model model)
        {
            List<PipelineAssociation> pipelineAssociationsList = new List<PipelineAssociation>();
            if (image != null)
            {
                Dictionary<int, PipelineAssociation> pipelineAssociations = model.PipelineAssociations;
                foreach (KeyValuePair<int, PipelineAssociation> pipelineAssociation in pipelineAssociations)
                {
                    if (pipelineAssociation.Value.PipelineDocumentImage != null &&
                        pipelineAssociation.Value.PipelineDocumentImage.ImageKey.Equals(image.ImageKey))
                    {
                        if (image.AppendixImage)
                        {
                            continue;
                        }

                        pipelineAssociationsList.Add(pipelineAssociation.Value);
                    }
                }
            }

            return pipelineAssociationsList;
        }

        /// <summary>
        /// Displays the pipeline associations.
        /// </summary>
        /// <param name="pipelineAssociationsList">The pipeline associations list.</param>
        /// <param name="isAppendixImage">if set to <c>true</c> [is appendix image].</param>
        private void DisplayPipelineAssociations(List<PipelineAssociation> pipelineAssociationsList, bool isAppendixImage)
        {
            AssociatedPipelinesUserControl associatedPipelinesUserControl = GetActiveAssociatedPipelinesUserControl();
            if (associatedPipelinesUserControl != null)
            {
                associatedPipelinesUserControl.DisplayPipelineAssociations(pipelineAssociationsList,
                    string.Empty, isAppendixImage);
            }

            string documentKey = DocumentKey;
            UpdatePaneDocumentJobs(documentKey);

            RerunPipelineTaskPane rerunPipelineTaskPane = GetActiveRerunPipelineTaskPane(documentKey);
            if (rerunPipelineTaskPane != null)
            {
                rerunPipelineTaskPane.UpdateAssociations(pipelineAssociationsList);
            }

            DashboardUserControl dash = GetActiveDashboardUserControl();
            if (dash != null)
            {
                ReadOnlyCollection<PipelineAssociation> pipelineAssociationList = this.GetAllPipelineAssociations(documentKey);
                ICollection<ContentItem> contentItems = this.GetContentItems(pipelineAssociationList, documentKey);
                dash.DisplayPipelineAssociations(pipelineAssociationList, contentItems);
            }

        }

        /// <summary>
        /// Updates the pane document jobs.
        /// </summary>
        private void UpdatePaneDocumentJobs(string documentKey)
        {
            // Refresh Associate Pane
            AssociatePipelineTaskPane associatePipelinePane = GetPaneUserControl(TaskPaneKey.AssociatePipeline, documentKey);
            if (associatePipelinePane != null)
            {
                associatePipelinePane.RefreshDocumentJobs();
            }

            // Refresh Edit Pipeline Pane
            AssociatePipelineTaskPane editPipelinePane = GetPaneUserControl(TaskPaneKey.EditPipeline, documentKey);
            if (editPipelinePane != null)
            {
                editPipelinePane.RefreshDocumentJobs();
            }

            // Refresh Insert GenePattern Link Pane
            AssociatePipelineTaskPane insertGenePatternLinkPane = GetPaneUserControl(TaskPaneKey.InsertGenePatternLink, documentKey);
            if (insertGenePatternLinkPane != null)
            {
                insertGenePatternLinkPane.RefreshDocumentJobs();
            }
        }

        /// <summary>
        /// Gets the content items.
        /// </summary>
        /// <returns></returns>
        public ReadOnlyCollection<ContentItem> GetContentItems(ReadOnlyCollection<PipelineAssociation> pipelineAssociations, string documentKey)
        {
            if (string.IsNullOrEmpty(documentKey))
            {
                documentKey = DocumentKey;
            }

            List<ContentItem> result = new List<ContentItem>();
            Dictionary<string, ContentItem> lsidToContentItem = new Dictionary<string, ContentItem>();
            Model currentModel = GetDocumentModel(documentKey);

            foreach (PipelineAssociation pipelineAssociation in pipelineAssociations)
            {
                Pipeline pipeline = currentModel.Pipelines[pipelineAssociation.PipelineLsid];
                if (pipeline != null && !pipeline.IsLinked)
                {
                    ContentItem item = null;
                    lsidToContentItem.TryGetValue(pipeline.Lsid, out item);
                    if (item == null)
                    {
                        item = new ContentItem(pipeline.Name, GetPipelineSize(pipeline), ContentItemType.Pipeline);
                        lsidToContentItem.Add(pipeline.Lsid, item);
                        result.Add(item);
                    }

                    if (!item.BelongsTo.Contains(pipelineAssociation.Name))
                        item.BelongsTo.Add(pipelineAssociation.Name);

                    foreach (PipelineStep pipelineStep in pipeline.Steps)
                    {
                        ContentItem moduleItem = null;
                        ContentItemType contentItemType = ContentItemType.Module;
                        lsidToContentItem.TryGetValue(pipelineStep.Lsid, out moduleItem);
                        if (moduleItem == null)
                        {
                            long moduleSize = 0;
                            if (pipelineStep.IsPipeline)
                            {
                                moduleSize = GetPipelineSize(currentModel.Pipelines[pipelineStep.Lsid]);
                                contentItemType = ContentItemType.Pipeline;
                            }
                            else
                            {
                                moduleSize = currentModel.Modules[pipelineStep.Lsid].ZipFile.Length;
                                if (pipelineStep.Type == PipelineStepType.Module)
                                    contentItemType = ContentItemType.Module;
                                else
                                    contentItemType = ContentItemType.Vizualizer;
                            }
                            moduleItem = new ContentItem(pipelineStep.Name, moduleSize, contentItemType);
                            lsidToContentItem.Add(pipelineStep.Lsid, moduleItem);
                            result.Add(moduleItem);
                        }
                        if (!moduleItem.BelongsTo.Contains(pipelineAssociation.Name))
                            moduleItem.BelongsTo.Add(pipelineAssociation.Name);
                    }
                }

                foreach (JobParameter parameter in pipelineAssociation.Job.Parameters)
                {
                    InternalFile file = parameter.ParamValue as InternalFile;
                    if (file != null)
                    {
                        ContentItem fileItem = null;
                        lsidToContentItem.TryGetValue(file.Lsid, out fileItem);
                        if (fileItem == null)
                        {
                            fileItem = new ContentItem(file.Filename, file.Length, ContentItemType.InputFile);
                            fileItem.BelongsTo.Add(pipelineAssociation.Name);
                            result.Add(fileItem);
                            lsidToContentItem.Add(file.Lsid, fileItem);
                        }
                        else if (!fileItem.BelongsTo.Contains(pipelineAssociation.Name))
                            fileItem.BelongsTo.Add(pipelineAssociation.Name);
                    }
                }
                foreach (InternalFile file in pipelineAssociation.EmbeddedResults)
                {
                    ContentItem fileItem = null;
                    lsidToContentItem.TryGetValue(file.Lsid, out fileItem);
                    if (fileItem == null)
                    {
                        fileItem = new ContentItem(file.Filename, file.Length, ContentItemType.ResultFile);
                        fileItem.BelongsTo.Add(pipelineAssociation.Name);
                        result.Add(fileItem);
                        lsidToContentItem.Add(file.Lsid, fileItem);
                    }
                    else if (!fileItem.BelongsTo.Contains(pipelineAssociation.Name))
                        fileItem.BelongsTo.Add(pipelineAssociation.Name);
                }
            }
            return new ReadOnlyCollection<ContentItem>(result);
        }

        private static long GetPipelineSize(Pipeline pipeline)
        {
            long size = 0;
            size += pipeline.ManifestFile.Length;
            foreach (InternalFile file in pipeline.Documentation)
            {
                size += file.Length;
            }
            foreach (InternalFile file in pipeline.Datafiles)
            {
                size += file.Length;
            }
            return size;
        }

        /// <summary>
        /// Gets the active dashboard user control.
        /// </summary>
        /// <returns></returns>
        private DashboardUserControl GetActiveDashboardUserControl()
        {
            DashboardUserControl control = null;
            string title = userControlCreator.GetTitle(TaskPaneKey.Dashboard);
            if (taskPaneManager.TaskPaneExists(title))
            {
                control = taskPaneManager.GetTaskPaneUserControl(title) as DashboardUserControl;
            }

            return control;
        }

        /// <summary>
        /// Gets the active dashboard user control.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        private DashboardUserControl GetActiveDashboardUserControl(string documentKey)
        {
            UserControl control = null;
            DashboardUserControl dashboardUserControl = null;
            string title = userControlCreator.GetTitle(TaskPaneKey.Dashboard);
            control = taskPaneManager.GetTaskPaneUserControl(title, documentKey);
            if (control != null)
                return control as DashboardUserControl;
            else
                return dashboardUserControl;
        }

        /// <summary>
        /// Gets the view connections user control.
        /// </summary>
        /// <returns></returns>
        private ViewConnectionsUserControl GetViewConnectionsUserControl()
        {
            ViewConnectionsUserControl control = null;
            string title = userControlCreator.GetTitle(TaskPaneKey.Connections);
            if (taskPaneManager.TaskPaneExists(title))
            {
                control = taskPaneManager.GetTaskPaneUserControl(title) as ViewConnectionsUserControl;
            }

            return control;
        }

        /// <summary>
        /// Gets the pane associate pipeline user control.
        /// </summary>
        /// <returns></returns>
        private AssociatePipelineTaskPane GetPaneUserControl(TaskPaneKey paneKey, string documentKey)
        {
            AssociatePipelineTaskPane control = null;
            string title = userControlCreator.GetTitle(paneKey);
            if (taskPaneManager.TaskPaneExists(title, documentKey))
            {
                control = taskPaneManager.GetTaskPaneUserControl(title, documentKey) as AssociatePipelineTaskPane;
            }

            return control;
        }

        /// <summary>
        /// Gets the active associated pipelines user control.
        /// </summary>
        /// <returns></returns>
        public AssociatedPipelinesUserControl GetActiveAssociatedPipelinesUserControl()
        {
            AssociatedPipelinesUserControl control = null;
            string title = userControlCreator.GetTitle(TaskPaneKey.ViewAssociations);
            if (taskPaneManager.TaskPaneExists(title))
            {
                control = taskPaneManager.GetTaskPaneUserControl(title) as AssociatedPipelinesUserControl;
            }

            return control;
        }

        /// <summary>
        /// Gets the active associated pipelines user control.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        public AssociatedPipelinesUserControl GetActiveAssociatedPipelinesUserControl(string documentKey)
        {
            UserControl control = null;
            AssociatedPipelinesUserControl associatedPipelinesUserControl = null;
            string title = userControlCreator.GetTitle(TaskPaneKey.ViewAssociations);
            control = taskPaneManager.GetTaskPaneUserControl(title, documentKey);
            if (control != null)
                return control as AssociatedPipelinesUserControl;
            else
                return associatedPipelinesUserControl;

        }


        /// <summary>
        /// Gets the GP Link image [formatted with the passed text]
        /// Note:  text is now ignored due to change requests
        /// </summary>
        /// <param name="text"></param>
        /// <returns></returns>
        public static Image FormatLinkImage(string text)
        {
            int padding = 2;
            Font font = new Font("arial", 12, FontStyle.Bold, GraphicsUnit.Pixel);

            int extraWidth = 100;
            Image linkImage = Resources.icon_GPLINK;
            Image finalImage = new Bitmap(linkImage.Width + extraWidth + 2 * padding, linkImage.Height);

            Graphics g = Graphics.FromImage(finalImage);
            float width = GetTextWidth(font, g, text); ;
            if (width > extraWidth)
            {
                text += "...";
                do
                {
                    width = GetTextWidth(font, g, text);
                    if (width > extraWidth)
                    {
                        text = text.Substring(0, text.Length - 4);
                        text += "...";
                    }
                } while (width > extraWidth);
            }


            // shorten image for small text strings
            //extraWidth = (int)width;
            // change because text is ignored:
            extraWidth = 0;
            padding = 0;


            finalImage = new Bitmap(linkImage.Width + extraWidth + 2 * padding, linkImage.Height);
            g = Graphics.FromImage(finalImage);
            g.SmoothingMode = SmoothingMode.AntiAlias;
            Pen solidWhite = new Pen(new SolidBrush(Color.White), 20);

            Rectangle imageBounds = new Rectangle(new Point(0, 0), finalImage.Size);
            imageBounds.Width -= 1;
            imageBounds.Height -= 1;
            g.DrawRectangle(solidWhite, imageBounds);
            g.DrawImage(linkImage, 0, 0);

            RectangleF rectF = new RectangleF(linkImage.Width + padding, padding, finalImage.Width - linkImage.Width, finalImage.Height);

            rectF.Location = new PointF(rectF.Location.X - 1, rectF.Location.Y - 1);
            //g.DrawString(text, font, new SolidBrush(Color.Black), rectF);

            font.Dispose();
            solidWhite.Dispose();
            return finalImage;
        }

        /// <summary>
        /// Gets the width of the text.
        /// </summary>
        /// <param name="font">The font.</param>
        /// <param name="g">The g.</param>
        /// <param name="text">The text.</param>
        /// <returns></returns>
        private static float GetTextWidth(Font font, Graphics g, string text)
        {
            // measure predicted width of text.  note that the rectangle is overly wide
            SizeF size = g.MeasureString(text, font, new SizeF(1000, 1000));
            return size.Width;
        }

        /// <summary>
        /// Thumbnails the callback.
        /// </summary>
        /// <returns></returns>
        public bool ThumbnailCallback()
        {
            return false;
        }

        /// <summary>
        /// Determines whether [is job running].
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if [is job running]; otherwise, <c>false</c>.
        /// </returns>
        private bool IsJobRunning()
        {
            bool isJobRunning = false;
            bool isJobProgressCanceled = false;

            if (!IsWizardUI)
            {
                isJobProgressCanceled = CheckPaneJobProgress(TaskPaneKey.AssociatePipeline, out isJobRunning);
                if (isJobRunning)
                    return !isJobProgressCanceled;

                isJobProgressCanceled = CheckPaneJobProgress(TaskPaneKey.EditPipeline, out isJobRunning);
                if (isJobRunning)
                    return !isJobProgressCanceled;

                isJobProgressCanceled = CheckPaneJobProgress(TaskPaneKey.InsertGenePatternLink, out isJobRunning);
                if (isJobRunning)
                    return !isJobProgressCanceled;

                isJobProgressCanceled = CheckPaneJobProgress(TaskPaneKey.RerunPipeline, out isJobRunning);
                if (isJobRunning)
                    return !isJobProgressCanceled;
            }

            return isJobRunning;
        }

        /// <summary>
        /// Cancels the job processing.
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private static bool CancelJobProcessing()
        {
            DialogResult result = MessageBox.Show(Resources.JOB_PROCESSING_WARNING, Resources.GENEPATTREN_MSG_HEADER,
                MessageBoxButtons.YesNo, MessageBoxIcon.Warning);

            if (result == DialogResult.Yes)
                return true;

            return false;
        }

        /// <summary>
        /// Checks the pane job progress.
        /// </summary>
        /// <param name="taskPaneKey">The task pane key.</param>
        /// <param name="isJobRunning">if set to <c>true</c> [is job running].</param>
        /// <returns></returns>
        private bool CheckPaneJobProgress(TaskPaneKey taskPaneKey, out bool isJobRunning)
        {
            bool isJobProgressCanceled = false;
            isJobRunning = false;

            if (taskPaneKey == TaskPaneKey.RerunPipeline)
            {
                RerunPipelineTaskPane rerunPipelinePane = GetActiveRerunPipelineTaskPane(DocumentKey);
                if (rerunPipelinePane != null)
                {
                    isJobRunning = rerunPipelinePane.IsJobExecutionInProgress();
                    if (isJobRunning)
                    {
                        if (CancelJobProcessing())
                        {
                            rerunPipelinePane.CancelExecution();
                            isJobProgressCanceled = false;
                        }
                    }
                }
            }
            else
            {
                AssociatePipelineTaskPane associatePipelinePane = GetPaneUserControl(taskPaneKey, DocumentKey);
                if (associatePipelinePane != null)
                {
                    isJobRunning = associatePipelinePane.IsJobExecutionInProgress();
                    if (isJobRunning)
                    {
                        if (CancelJobProcessing())
                        {
                            associatePipelinePane.CancelExecution(true);
                            isJobProgressCanceled = false;
                        }
                    }
                }
            }
            return isJobProgressCanceled;
        }

        /// <summary>
        /// Determines whether this instance [can open model dialog].
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if this instance [can open model dialog]; otherwise, <c>false</c>.
        /// </returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        public bool CanOpenModelDialog()
        {
            IDictionaryEnumerator ie = _documentJobProcessingStatus.GetEnumerator();
            bool isJobRunning = false;
            while (ie.MoveNext())
            {
                if (((ie.Value) as DocumentJobProcessing).IsJobProcessingInProgress)
                {
                    isJobRunning = true;
                    break;
                }
            }

            if (isJobRunning)
            {
                DialogResult result = MessageBox.Show("Job processing is in progress, opening model dialog will stop the job processing in all the open documents.\nDo you want to continue?",
                      Resources.GENEPATTREN_MSG_HEADER, MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
                if (result == DialogResult.No)
                {
                    return true;
                }
            }
            return false;
        }

        /// <summary>
        /// Updates the name of the rerun result association.
        /// </summary>
        /// <param name="newAssociationName">New name of the association.</param>
        /// <param name="imageKey">The image key.</param>
        public void UpdateRerunResultAssociationName(string newAssociationName, string imageKey)
        {
            if (string.IsNullOrEmpty(imageKey))
            {
                DocumentImage selectedImage = host.GetSelectedImage();
                if (selectedImage == null)
                    return;
                imageKey = selectedImage.ImageKey;
            }
            //host.UpdateRerunResultAssociationName(newAssociationName, imageKey);
        }
        #endregion

        #region Image ThumbNail

        /// <summary>
        /// Gets the selected image thumb.
        /// </summary>
        /// <param name="image">The image.</param>
        /// <returns></returns>
        public Image GetSelectedImageThumb(Image image)
        {
            Image.GetThumbnailImageAbort dummyCallBack =
                new System.Drawing.Image.GetThumbnailImageAbort(ThumbnailCallback);

            int thumbSize = 64;
            Image thumbnail = GetProportionedThumbnail(image, thumbSize, dummyCallBack);

            return thumbnail;
        }

        /// <summary>
        /// Gets the proportioned thumbnail.
        /// </summary>
        /// <param name="img">The img.</param>
        /// <param name="maxPixels">The max pixels.</param>
        /// <param name="ThumbnailCallback">The thumbnail callback.</param>
        /// <returns></returns>
        private static Image GetProportionedThumbnail(Image img, int maxPixels, Image.GetThumbnailImageAbort ThumbnailCallback)
        {
            float scaleFactor = 1;
            if (img.Width > img.Height)
            {
                if (img.Width > maxPixels)
                {
                    scaleFactor = (float)maxPixels / (float)img.Width;
                }
            }
            else
            {
                if (img.Height > maxPixels)
                {
                    scaleFactor = (float)maxPixels / (float)img.Height;
                }
            }
            int newH = (int)(scaleFactor * img.Height);
            int newW = (int)(scaleFactor * img.Width);


            if (img.Width > img.Height && img.Width > maxPixels)
            {
                newW = maxPixels; // can't trust floating point stuff, something newH is maxPixels-1
            }
            else if (img.Height > maxPixels)
            {
                newH = maxPixels;
            }

            Image thumbnailImg = img.GetThumbnailImage(newW, newH, ThumbnailCallback, IntPtr.Zero);
            Bitmap thumbnail = new Bitmap(thumbnailImg);
            int thumbSize = maxPixels;
            // if it's not square, we need to add some whitespace padding since
            // the display will stretch it into a square

            Bitmap paddedThumb = new Bitmap(thumbSize, thumbSize);
            using (Graphics gr = Graphics.FromImage(paddedThumb))
            {
                gr.FillRectangle(Brushes.White, new Rectangle(0, 0, maxPixels, maxPixels));
                int x = (int)(((float)maxPixels / 2) - ((float)thumbnail.Width / 2));
                int y = (int)(((float)maxPixels / 2) - ((float)thumbnail.Height / 2));
                gr.DrawImage(thumbnail, x, y, thumbnail.Width, thumbnail.Height);
            }
            return paddedThumb;
        }

        #endregion
    }
}
