using System;
using System.Collections;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.ScientificAddIn.Common;
using Microsoft.Office.Core;
using Office = Microsoft.Office.Core;
using System.Collections.Generic;
using System.Runtime.InteropServices;

// 
namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// Wrapper class for IDatastorage object that uses CustomXMLParts from word. 
    /// </summary>
    public class DataStore : IDataStore
    {
        private IDocumentStorage _documentStorage;
        private IPackageHost _host;
        private const string MODEL_ID = "model";
        private const string FILES_ID = "files";
        private Dictionary<string, SerializableDictionary<string, InternalFile>> docStores = new Dictionary<string, SerializableDictionary<string, InternalFile>>();

        /// <summary>
        /// Initializes a new instance of the <see cref="DataStore"/> class.
        /// </summary>
        /// <param name="documentStorage">The data store.</param>
        public DataStore(IDocumentStorage documentStorage)
        {
            _documentStorage = documentStorage;
        }

        #region IDatastore members

        /// <summary>
        /// Stores the given file with lsid as key in a collection.
        /// </summary>
        /// <param name="file">The file.</param>
        /// <param name="lsid">The lsid.</param>
        /// <returns></returns>
        public InternalFile StoreFile(Stream file, string lsid)
        {
            SerializableDictionary<string, InternalFile> files = GetActiveDocStore();
            if (files.ContainsKey(lsid))
            {
                // already have stored this file, just return the guid
                Trace.WriteLine(String.Format("File is already stored:  returning file with lsid {0}", lsid));
                return files[lsid];
            }
            else
            {
                // store file and return new guid

                CustomXMLPart part = _documentStorage.GetCustomXmlPartById(lsid);
                StoreStream(part, file);

                InternalFile theFile = new InternalFile();
                // get guid
                theFile.Guid = new Guid(part.Id);

                theFile.Length = file.Length;
                theFile.Lsid = lsid;
                files.Add(lsid, theFile);

                Trace.WriteLine("Stored File with lsid: " + lsid);
                
                Marshal.ReleaseComObject(part);
                part = null;
                return theFile;
            }
        }

        /// <summary>
        /// Stores the file.
        /// </summary>
        /// <param name="file">The file.</param>
        /// <param name="lsid">The lsid.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        public InternalFile StoreFile(Stream file, string lsid, string documentKey)
        {
            SerializableDictionary<string, InternalFile> files = GetDocumentDataStore(documentKey);
            InternalFile theFile = new InternalFile();

            if (files.ContainsKey(lsid))
            {
                // already have stored this file, just return the guid
                Trace.WriteLine(String.Format("File is already stored:  returning file with lsid {0}", lsid));
                theFile = files[lsid];
                theFile.IsFileCreated = false;
            }
            else
            {
                // store file and return new guid

                CustomXMLPart part = _documentStorage.GetCustomXmlPartById(lsid, documentKey, _host);
                StoreStream(part, file);

                // get guid
                theFile.Guid = new Guid(part.Id);
                theFile.Length = file.Length;
                theFile.Lsid = lsid;
                files.Add(lsid, theFile);

                Trace.WriteLine("Stored File with lsid: " + lsid);
                Marshal.ReleaseComObject(part);
                part = null;
            }

            return theFile;
        }

        /// <summary>
        /// Gets the File GUID corresponding to the given Lsid .
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <returns>The GUID</returns>
        public Guid GetFileGuid(string lsid)
        {
            return GetActiveDocStore()[lsid].Guid;
        }

        /// <summary>
        /// Gets the hash key and store file.
        /// </summary>
        /// <param name="file">The file.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        public InternalFile GetHashKeyAndStoreFile(Stream file, string documentKey)
        {
            if (file == null)
                throw new ArgumentNullException("file");
            if (file.Length < DefaultSetting.LargeFileSize)
            {
                return StoreSmallFile(file, documentKey);
            }
            else
            {
                return StoreLargeFile(file, documentKey);
            }
        }

        /// <summary>
        /// Stores the small file.
        /// </summary>
        /// <param name="file">The file.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.Byte.ToString(System.String)")]
        private InternalFile StoreSmallFile(Stream file, string documentKey)
        {
            file.Position = 0;
            StringBuilder hashString = new StringBuilder();
            Crc32 crc32 = new Crc32();

            foreach (byte b in crc32.ComputeHash(file))
            {
                hashString.Append(b.ToString("x2").ToLower(CultureInfo.InvariantCulture));
            }


            // return the "lsid" of this file, which is actually the crc32
            InternalFile retFile;

            if (string.IsNullOrEmpty(documentKey))
                retFile = StoreFile(file, hashString.ToString());
            else
                retFile = StoreFile(file, hashString.ToString(), documentKey);

            retFile.Lsid = hashString.ToString();
            return retFile;
        }

        /// <summary>
        /// Stores the large file.
        /// </summary>
        /// <param name="file">The file.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        private InternalFile StoreLargeFile(Stream file, string documentKey)
        {
            /* if the file size is more than 5MB, the file will be divided into 5MB chunks and each fragments
             * will be stored in each custom xml part. These fragments are linked together by defining 'next'
             * attribute in <xmlpart> tag, which will point to the next part, and we will return handle of
             * first framnet from this function. */

            int fragmentCount = (int)file.Length / DefaultSetting.LargeFileSize;
            int leftOverFileSize = (int)file.Length % DefaultSetting.LargeFileSize;
            InternalFile retFile = null;
            string savedLsid = null;
            InternalFile tempFile = null;
            string hashString = string.Empty;

            for (int i = 0; i < fragmentCount; i++)
            {
                hashString = StoreChunkedData(file, documentKey, ref tempFile, DefaultSetting.LargeFileSize);
                
                if (!tempFile.IsFileCreated)
                {
                    return tempFile;
                }

                if (i == 0)
                {
                    retFile = tempFile;
                    retFile.Lsid = hashString;
                }
                else
                {
                    retFile.Length += tempFile.Length;
                }

                if (fragmentCount == 0 && leftOverFileSize == 0)
                {
                    _documentStorage.SetNextField(hashString, null, documentKey, _host);
                }
                else if (savedLsid != null)
                {
                    _documentStorage.SetNextField(savedLsid, hashString, documentKey, _host);
                    savedLsid = hashString;
                }
                else
                {
                    savedLsid = hashString;
                }

            }

            if (leftOverFileSize > 0)
            {
                hashString = StoreChunkedData(file, documentKey, ref tempFile, leftOverFileSize);
                _documentStorage.SetNextField(savedLsid, hashString, documentKey, _host);
                _documentStorage.SetNextField(hashString, null, documentKey, _host);
                retFile.Length += tempFile.Length;
            }

            return retFile;
        }

        /// <summary>
        /// Stores the chunked data.
        /// </summary>
        /// <param name="file">The file.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="tempFile">The temp file.</param>
        /// <param name="maxSize">Size of the max.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.Byte.ToString(System.String)")]
        private string StoreChunkedData(Stream file, string documentKey, ref InternalFile tempFile, int maxSize)
        {
            int readSize = 0;
            int totalReadSize = 0;
            byte[] buffer = new byte[StreamHelper.BufferLength];
            using (Stream tempStream = new MemoryStream())
            {
                do
                {
                    readSize = file.Read(buffer, 0, StreamHelper.BufferLength);
                    if (readSize <= 0)
                        break;
                    tempStream.Write(buffer, 0, readSize);
                    totalReadSize += readSize;

                } while (totalReadSize < (maxSize));

                Crc32 crc32 = new Crc32();
                tempStream.Position = 0;
                StringBuilder hashString = new StringBuilder();

                foreach (byte b in crc32.ComputeHash(tempStream))
                {
                    hashString.Append(b.ToString("x2").ToLower(CultureInfo.InvariantCulture));
                }

                if (string.IsNullOrEmpty(documentKey))
                    tempFile = StoreFile(tempStream, hashString.ToString());
                else
                    tempFile = StoreFile(tempStream, hashString.ToString(), documentKey);

                buffer = null;
                return hashString.ToString();
            }
        }

        /// <summary>
        /// Determines whether the data storage contains the file with the specified lsid.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <returns>
        /// 	<c>true</c> if the data storage contains the file for the given LSID; otherwise, <c>false</c>
        /// </returns>
        public bool ContainsFile(string lsid)
        {
            return GetActiveDocStore().ContainsKey(lsid);
        }

        /// <summary>
        /// Remove all data in the store that isn't being used by one of the passed lsids
        /// </summary>
        /// <param name="lsids">The list of lsids that shall remain in the document</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public void Compress(List<string> lsids)
        {
            SerializableDictionary<string, InternalFile> files = GetActiveDocStore();
            Trace.WriteLine("Compressing Data Store...");
            List<string> toRemove = new List<string>();

            #region Get Next Node Lsid

            List<string> nextLisdList = new List<string>();
            foreach(string lsid in lsids)
            {
                string nextlisd = lsid;
                
                do
                {
                    nextlisd = _documentStorage.GetNextGuid(nextlisd);
                    
                    if (string.IsNullOrEmpty(nextlisd))
                        break;

                    nextLisdList.Add(nextlisd);

                } while (!string.IsNullOrEmpty(nextlisd));
            }

            #endregion

            // File Key does not exists add it to remove list
            foreach (string lsid in files.Keys)
            {
                if (!lsids.Contains(lsid) && !nextLisdList.Contains(lsid))
                {
                    // remove data since it is no longer referenced
                    toRemove.Add(lsid);
                }
            }

            // delete the entry for the file and its backing xml part
            foreach (string lsid in toRemove)
            {
                Trace.WriteLine("Removing file:" + lsid);
                InternalFile file = files[lsid];
                files.Remove(lsid);
                _documentStorage.DeleteCustomXmlPartByGuid(file.Guid);
            }

            files = null;
        }

        /// <summary>
        /// Removes all content for this application
        /// </summary>
        public void RemoveAllContent()
        {
            GetActiveDocStore().Clear();
            _documentStorage.Reset();
        }

        /// <summary>
        /// Get a previously stored file from its guid
        /// </summary>
        /// <param name="guid"></param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
        public MemoryStream GetFile(Guid guid)
        {
            CustomXMLPart part = _documentStorage.GetCustomXmlPartByGuid(guid);
            if (part != null)
            {
                return new MemoryStream(GetBytes(part));
            }
            else
            {
                Trace.WriteLine("No file stored matching guid " + guid);
                throw new Exception("One or more pipeline files are missing. Please try associating pipeline again.");
            }
        }

        /// <summary>
        /// Gets the name of the file.
        /// </summary>
        /// <param name="guid">The GUID.</param>
        /// <param name="fileName">Name of the file. Send string.Empty to create temporary file</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
        public string GetFileName(Guid guid, string fileName, string documentKey)
        {
            string filePath = CreateTemporaryName(fileName);
            CustomXMLPart part;

            try
            {
                if(string.IsNullOrEmpty(documentKey))
                    part = _documentStorage.GetCustomXmlPartByGuid(guid);
                else
                    part = _documentStorage.GetCustomXmlPartByGuid(guid, documentKey, _host);

                if (part != null)
                {
                    using (FileStream fileStream = new FileStream(filePath, FileMode.OpenOrCreate, FileAccess.Write))
                    {
                        byte[] buffer = GetBytes(part);
                        fileStream.Write(buffer, 0, buffer.Length);
                        fileStream.Flush();
                        string nextguid = null;

                        do
                        {
                            nextguid = _documentStorage.GetNextGuid("{" + guid.ToString().ToUpper(CultureInfo.InvariantCulture) + "}",
                                documentKey, _host);
                            if (nextguid != null)
                            {
                                part = _documentStorage.GetCustomXmlPartByGuid(new Guid(nextguid), documentKey, _host);
                                buffer = GetBytes(part);
                                fileStream.Write(buffer, 0, buffer.Length);
                                buffer = null;
                                guid = new Guid(nextguid);
                            }

                        } while (nextguid != null);

                        Marshal.ReleaseComObject(part);
                        buffer = null;
                        part = null;
                    }
                }
                else
                {
                    throw new Exception("No file stored matching guid " + guid);
                }
            }
            catch (Exception)
            {
                // this can throw an exception, probably should be caught and retyped to something internal
                DeleteTemporaryFile(filePath);
                throw;
            }

            return filePath;
        }

        /// <summary>
        /// Creates the name of the temporary.
        /// </summary>
        /// <param name="fileName">Name of the file.</param>
        /// <returns></returns>
        private static string CreateTemporaryName(string fileName)
        {
            if (string.IsNullOrEmpty(fileName))
            {
                Guid guid = Guid.NewGuid();
                string filePath = DefaultSetting.Path + guid.ToString() + ".tmp";
                return filePath;
            }
            else
            {
                return DefaultSetting.Path + fileName;
            }
        }

        /// <summary>
        /// Deletes the temporary file.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        private static void DeleteTemporaryFile(string filePath)
        {
            if (File.Exists(filePath))
            {
                File.Delete(filePath);
            }
        }

        /// <summary>
        /// Serializes the model.
        /// </summary>
        /// <param name="model">The model.</param>
        public void SerializeModel(Model model)
        {
            using (MemoryStream ms = new MemoryStream())
            {
                Serialize(model, ms, MODEL_ID);
                SerializeFiles();
            }
        }

        /// <summary>
        /// Deserializes the model.
        /// </summary>
        /// <returns>The Model</returns>
        public Model DeserializeModel()
        {

            Model model;
            try
            {
                model = Deserialize<Model>(MODEL_ID);
                // restore files as well
                SerializableDictionary<string, InternalFile> newDict = Deserialize<SerializableDictionary<string, InternalFile>>(FILES_ID);
                if (newDict != null)
                {
                    SerializableDictionary<string, InternalFile> files = newDict;
                    string documentKey = _host.ActiveDocumentKey;

                    // if we already have files for this deserialized, use what is in memory instead
                    if (!docStores.ContainsKey(documentKey))
                    {
                        docStores.Add(documentKey, files);
                    }
                    files = null;
                }
                newDict = null;
            }
            catch (Exception)
            {
                throw;
            }

            return model;
        }

        #endregion

        #region Helper Functions

        /// <summary>
        /// Serializes the specified obj.
        /// </summary>
        /// <param name="obj">The obj.</param>
        /// <param name="stream">The stream.</param>
        /// <param name="part_id">The part_id.</param>
        private void Serialize(object obj, MemoryStream stream, string part_id)
        {
            Type t = obj.GetType();
            XmlSerializer xs = new XmlSerializer(t);
            xs.Serialize(stream, obj);
            CustomXMLPart part = _documentStorage.GetCustomXmlPartById(part_id);
            
            StoreXmlStream(part, stream);

            Marshal.ReleaseComObject(part);
            xs = null;
            part = null;
        }

        /// <summary>
        /// Deserializes the specified part_id.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="part_id">The part_id.</param>
        /// <returns></returns>
        private T Deserialize<T>(string part_id)
        {
            CustomXMLPart part = _documentStorage.GetCustomXmlPartById(part_id);
            using (Stream serialized = GetXmlStream(part))
            {
                Marshal.ReleaseComObject(part);

                if (serialized.Length > 0)
                {
                    XmlSerializer xs = new XmlSerializer(typeof(T));
                    return (T)xs.Deserialize(serialized);
                }
                else
                {
                    return default(T);
                }
            }
        }

        /// <summary>
        /// Stores the stream.
        /// </summary>
        /// <param name="part">The part.</param>
        /// <param name="dataStream">The data stream.</param>
        private void StoreStream(CustomXMLPart part, Stream dataStream)
        {
            byte[] data = new byte[dataStream.Length];
            dataStream.Position = 0;
            dataStream.Read(data, 0, data.Length);
            StoreString(part, Convert.ToBase64String(data));
            data = null;
            GC.Collect();
        }

        /// <summary>
        /// Serializes the files.
        /// </summary>
        private void SerializeFiles()
        {
            using (MemoryStream ms = new MemoryStream())
            {
                SerializableDictionary<string, InternalFile> files = GetActiveDocStore();
                Serialize(files, ms, FILES_ID);
            }
        }

        /// <summary>
        /// Stores the XML stream.
        /// </summary>
        /// <param name="part">The part.</param>
        /// <param name="ms">The ms.</param>
        private void StoreXmlStream(CustomXMLPart part, MemoryStream ms)
        {

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.LoadXml(Encoding.Default.GetString(ms.ToArray()));
            CustomXMLNode dataNode = part.SelectSingleNode("//data");

            if (dataNode != null)
            {
                dataNode.ReplaceChildSubtree(xmlDoc.ChildNodes[1].OuterXml, dataNode.FirstChild);
            }
            else
            {
                part.DocumentElement.AppendChildNode("data", part.DocumentElement.NamespaceURI,
                                                     MsoCustomXMLNodeType.msoCustomXMLNodeElement, "");
                dataNode = part.SelectSingleNode("//data");
                dataNode.AppendChildSubtree(xmlDoc.ChildNodes[1].OuterXml);
            }

            Marshal.ReleaseComObject(dataNode);
            xmlDoc = null;
            dataNode = null;
        }

        /// <summary>
        /// Gets the XML stream.
        /// </summary>
        /// <returns></returns>
        private Stream GetXmlStream(CustomXMLPart part)
        {
            CustomXMLNode dataNode = part.SelectSingleNode("//data");
            if (dataNode != null)
            {
                return new MemoryStream(Encoding.Default.GetBytes(dataNode.FirstChild.NodeValue));
            }
            else
            {
                return new MemoryStream();
            }
        }

        /// <summary>
        /// Stores the string.
        /// </summary>
        /// <param name="part">The part.</param>
        /// <param name="data">The data.</param>
        private void StoreString(CustomXMLPart part, string data)
        {
            CustomXMLNode dataNode = part.SelectSingleNode("//data");

            if (dataNode != null)
            {
                AssignData(data, dataNode, part);
                Marshal.ReleaseComObject(dataNode);
            }
            else
            {
                CreateDataNode(data, part);
            }
            
            dataNode = null;
            data = null;
        }

        /// <summary>
        /// Creates the data node.
        /// </summary>
        /// <param name="data">The data.</param>
        /// <param name="part">The part.</param>
        private void CreateDataNode(string data, CustomXMLPart part)
        {
            part.DocumentElement.AppendChildNode("data", part.DocumentElement.NamespaceURI, 
                MsoCustomXMLNodeType.msoCustomXMLNodeElement, null);
            CustomXMLNode dataNode = part.SelectSingleNode("//data");
            AddCdataNode(data, dataNode, part);

            Marshal.ReleaseComObject(dataNode);
            dataNode = null;
        }

        /// <summary>
        /// Assigns the data.
        /// </summary>
        /// <param name="data">The data.</param>
        /// <param name="dataNode">The data node.</param>
        /// <param name="part">The part.</param>
        private void AssignData(string data, CustomXMLNode dataNode, CustomXMLPart part)
        {
            IEnumerator nodes = dataNode.ChildNodes.GetEnumerator();
            nodes.MoveNext();//Go to the content node
            if (nodes.Current != null)
            {
                CustomXMLNode contentXmlNode;
                contentXmlNode = nodes.Current as CustomXMLNode;
                if ((contentXmlNode != null) && (contentXmlNode.NodeType == MsoCustomXMLNodeType.msoCustomXMLNodeCData))
                {
                    contentXmlNode.NodeValue = data;
                    Marshal.ReleaseComObject(contentXmlNode);
                }
                else
                {
                    Trace.WriteLine("Content node should be of Type 'CustomXMLNode' and mso Custom node type must be 'MsoCustomXMLNodeType.msoCustomXMLNodeCData'",
                                     TraceLevel.Error.ToString());
                }
                contentXmlNode = null;
            }
            else
            {
                //Create the content Node
                AddCdataNode(data, dataNode, part);
            }
            nodes = null;
        }

        /// <summary>
        /// Adds the cdata node.
        /// </summary>
        /// <param name="data">The data.</param>
        /// <param name="dataNode">The data node.</param>
        /// <param name="part">The part.</param>
        private void AddCdataNode(string data, CustomXMLNode dataNode, CustomXMLPart part)
        {
            dataNode.AppendChildNode("content", part.DocumentElement.NamespaceURI, MsoCustomXMLNodeType.msoCustomXMLNodeCData, data);

            dataNode = null;
            data = null;

            GC.Collect();
            GC.WaitForPendingFinalizers();
            GC.Collect();
        }

        /// <summary>
        /// Gets the bytes.
        /// </summary>
        /// <param name="part">The part.</param>
        /// <returns></returns>
        private byte[] GetBytes(CustomXMLPart part)
        {
            string data = GetString(part);
            return Convert.FromBase64String(data);
        }

        /// <summary>
        /// Gets the string.
        /// </summary>
        /// <param name="part">The part.</param>
        /// <returns></returns>
        private string GetString(CustomXMLPart part)
        {
            CustomXMLNode dataNode = part.SelectSingleNode("//data");
            if (dataNode != null)
            {
                return dataNode.Text;
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// Gets the active doc store.
        /// </summary>
        /// <returns></returns>
        private SerializableDictionary<string, InternalFile> GetActiveDocStore()
        {
            string docKey = _host.ActiveDocumentKey;
            if (docStores.ContainsKey(docKey))
                return docStores[docKey];
            else
            {
                SerializableDictionary<string, InternalFile> newFiles = new SerializableDictionary<string, InternalFile>();
                docStores.Add(docKey, newFiles);
                return newFiles;
            }
        }

        /// <summary>
        /// Gets the document data store.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        private SerializableDictionary<string, InternalFile> GetDocumentDataStore(string documentKey)
        {
            if (docStores.ContainsKey(documentKey))
                return docStores[documentKey];
            else
            {
                SerializableDictionary<string, InternalFile> newFiles = new SerializableDictionary<string, InternalFile>();
                docStores.Add(documentKey, newFiles);
                return newFiles;
            }
        }

        /// <summary>
        /// Sets the package host for the datastore.  You must call this before using the object.
        /// </summary>
        /// <param name="host"></param>
        public void SetPackageHost(IPackageHost host)
        {
            _host = host;
        }

        #endregion
    }
}
