using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;

namespace Microsoft.eResearch.GenePattern.Package
{
    public partial class DownloadPipelineFiles : Form
    {
        private int totalDownloads;
        private int downloadsComplete;
        private IController controller;
        private List<Pipeline> pipelines;
        private List<PipelineAssociation> associations;
        private bool isEditing;
        private string progressMessage = "";
        private Thread asyncDownload;
        private DateTime startTime;
        private bool isResultOK;

        /// <summary>
        /// Initializes a new instance of the <see cref="DownloadPipelineFiles"/> class.
        /// </summary>
        public DownloadPipelineFiles()
        {
            InitializeComponent();
            this.BackgroundImage = Resources.Clean;
            this.BackgroundImageLayout = ImageLayout.Stretch;
        }

        /// <summary>
        /// Set items needed for the work.
        /// </summary>
        /// <param name="controllerInstance">The controller instance.</param>
        /// <param name="pipelineList">The pipeline list.</param>
        /// <param name="associationList">The association list.</param>
        /// <param name="editing">if set to <c>true</c> [editing].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public void SetWork(IController controllerInstance, List<Pipeline> pipelineList, List<PipelineAssociation> associationList, bool editing)
        {
            this.controller = controllerInstance;
            this.isEditing = editing;
            this.pipelines = pipelineList;
            this.associations = associationList;
        }

        /// <summary>
        /// Handles the Shown event of the DownloadPipelineFiles control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void DownloadPipelineFiles_Shown(object sender, EventArgs e)
        {
            // kick off the background worker when form opens
            asyncDownload = new Thread(new ThreadStart(DownloadFiles));
            asyncDownload.Name = "Download Pipelines";
            asyncDownload.SetApartmentState(ApartmentState.STA);
            asyncDownload.Start();
            progressTimer.Start();
            downloadAnimationUserControl1.Start();
            startTime = DateTime.Now;

            //set total downloads by counting pipelines and steps

            foreach (Pipeline p in pipelines)
            {
                totalDownloads += CountPipelinePieces(p);
            }
        }

        /// <summary>
        /// Counts the pipeline pieces.
        /// </summary>
        /// <param name="p">The p.</param>
        /// <returns></returns>
        private static int CountPipelinePieces(Pipeline p)
        {
            int count = 1;
            if (!p.IsLinked)
            {
                count += p.Steps.Count;
            }
            return count;
        }

        /// <summary>
        /// Handles the Tick event of the progressTimer control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void progressTimer_Tick(object sender, EventArgs e)
        {
            Progress.Minimum = 0;
            Progress.Maximum = totalDownloads;

            if (downloadsComplete > Progress.Maximum)
            {
                Progress.Maximum = downloadsComplete;
            }
            Progress.Value = downloadsComplete;
            if (Messages.Text != progressMessage)
            {
                Messages.AppendText(progressMessage.Substring(Messages.Text.Length));
            }

            // keep window open for at least 1 seconds
            if (!asyncDownload.IsAlive && (DateTime.Now - startTime).Seconds > 2)
            {
                Close(); // close form when download finishes
            }
            else if (!asyncDownload.IsAlive)
            {
                // download is done but time not done -- show progress finished
                Progress.Value = Progress.Maximum;
            }
        }

        /// <summary>
        /// Downloads the files.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void DownloadFiles()
        {
            try
            {
                if (!isEditing)
                    controller.BeginModelChanges();
                List<string> messages = null;

                if (isEditing)
                {
                    messages = controller.UpdatePipelineAssociations(pipelines, associations, new DownloadProgress(UpdateDownload));
                    controller.UpdateRerunResultAssociationName(associations[0].Name, string.Empty);
                }
                else
                {
                    messages = controller.AddPipelineAssociations(pipelines, associations, new DownloadProgress(UpdateDownload));
                }
                controller.CommitModelChanges();

                if (messages != null && messages.Count > 0)
                {
                    string message = string.Join(Environment.NewLine + Environment.NewLine, messages.ToArray());
                    MessageBox.Show(message, Resources.ATTENTION, MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                isResultOK = true;
            }
            catch (ThreadAbortException)
            {
                // user clicked cancel
                if (!isEditing)
                    controller.RollbackModelChanges();

            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
                if (controller.ActiveModel.TransactionActive)
                {
                    controller.RollbackModelChanges();
                }
            }
        }

        /// <summary>
        /// Updates the download.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="lsid">The lsid.</param>
        /// <param name="isStarting">if set to <c>true</c> [is starting].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object)")]
        private void UpdateDownload(string name, string lsid, bool isStarting)
        {
            if (isStarting)
            {
                string startMessage = Resources.START_PROCESSING;
                progressMessage += String.Format(startMessage, name) + Environment.NewLine;
            }
            else
            {
                string finishedMessage = Resources.FINISH_PROCESSING;
                progressMessage += String.Format(finishedMessage, name) + Environment.NewLine;
                downloadsComplete++;
            }
        }

        /// <summary>
        /// Handles the Click event of the Cancel control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void Cancel_Click(object sender, EventArgs e)
        {
            try
            {
                if (asyncDownload.IsAlive)
                {
                    asyncDownload.Abort();
                }
                Close();
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the FormClosing event of the DownloadPipelineFiles control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        private void DownloadPipelineFiles_FormClosing(object sender, FormClosingEventArgs e)
        {
            downloadAnimationUserControl1.Stop();
            if (asyncDownload.IsAlive)
            {
                asyncDownload.Abort();
            }
            if (isResultOK)
                this.DialogResult = DialogResult.OK;
        }
    }
}
