using System;
using System.Diagnostics;
using System.Windows.Forms;
using Microsoft.eResearch.ScientificAddIn.Common;
using Microsoft.eResearch.GenePattern.Common;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// The starting point of the GenePattern package plug-in. This class should be
    /// instatiated by the Microsoft.eResearch.ScientificAddIn component
    /// </summary>
    public class GenePatternPackage : IScientificPackage, IUserControlCreator
    {
        private const string VIEW_ASSOCIATIONS_TOGGLE_BUTTON = "viewAssociationsToggleButton";
        private const string DASHBOARD_TOGGLE_BUTTON = "dashboardToggleButton";
        private const string VIEW_CONNECTIONS_TOGGLE_BUTTON = "viewConnectionsToggleButton";
        private const string CONNECTIONS_DROP_DOWN = "connectionsDropDown";
        private const string ASSOCIATE_PIPELINE_TOGGLE_BUTTON = "associatePipelineButton";
        private const string INSERT_GENEPATTERN_LINK_TOGGLE_BUTTON = "insertGenePatternLinkButton";
        private const string RERUN_PIPELINE_TOGGLE_BUTTON = "rerunPipelineButton";
        private const string WIZARD_FREE_CHECKBOX = "chkWizardFree";
        private const string PROTECT_DOCUMENT_TOGGLE_BUTTON = "protectDocumentButton";
        private readonly IController controller;

        /// <summary>
        /// Initializes a new instance of the <see cref="GenePatternPackage"/> class.
        /// </summary>
        public GenePatternPackage()
        {
            this.controller = new Controller(this);


        }

        #region IScientificPackage Members

        /// <summary>
        /// Gets a ribbon extension from the package.
        /// </summary>
        /// <returns>
        /// an object that should be of type Office.IRibbonExtensibility
        /// </returns>
        public object GetRibbon()
        {
            GenePatternRibbon ribbon = new GenePatternRibbon(controller);
            return ribbon;
        }


        /// <summary>
        /// Sets the task pane manager for this package. This controls
        /// all the task panes associated with the package
        /// </summary>
        /// <param name="taskPaneManager">The task pane manager.</param>
        public void SetTaskPaneManager(ITaskPaneManager taskPaneManager)
        {
            controller.TaskPaneManager = taskPaneManager;
        }

        /// <summary>
        /// Sets the document storage.
        /// </summary>
        /// <param name="documentStorage">The document storage.</param>
        public void SetDocumentStorage(IDocumentStorage documentStorage)
        {
            controller.DataStore = new DataStore(documentStorage);
        }

        /// <summary>
        /// Sets the package host.
        /// </summary>
        /// <param name="host">The host.</param>
        public void SetPackageHost(IPackageHost host)
        {
            controller.SetPackageHost(host);
        }

        #endregion

        #region IUserControlCreator Members

        /// <summary>
        /// Gets a user control matching the specified <c>TaskPaneKey</c>.
        /// </summary>
        /// <param name="taskPaneKey">The task pane key.</param>
        /// <returns>
        /// a <c>UserControl</c> is there is a match otherwise null
        /// </returns>
        public UserControl GetUserControl(TaskPaneKey taskPaneKey)
        {
            UserControl result = null;

            switch (taskPaneKey)
            {
                case TaskPaneKey.ViewAssociations:
                    AssociatedPipelinesUserControl associatedPipelinesUserControl = new AssociatedPipelinesUserControl();
                    associatedPipelinesUserControl.Controller = controller;
                    result = associatedPipelinesUserControl;
                    break;
                case TaskPaneKey.Connections:
                    ViewConnectionsUserControl connectionsUserControl = new ViewConnectionsUserControl();
                    connectionsUserControl.Controller = controller;
                    result = connectionsUserControl;
                    break;
                case TaskPaneKey.AssociatePipeline:
                    AssociatePipelineTaskPane asscoiatePipelineCommand = new AssociatePipelineTaskPane();
                    asscoiatePipelineCommand.Controller = controller;
                    asscoiatePipelineCommand.CurrentTaskPaneType = TaskPaneKey.AssociatePipeline;
                    result = asscoiatePipelineCommand;
                    break;
                case TaskPaneKey.InsertGenePatternLink:
                    AssociatePipelineTaskPane insertGenePatternLink = new AssociatePipelineTaskPane();
                    insertGenePatternLink.Controller = controller;
                    insertGenePatternLink.IsLinked = true;
                    insertGenePatternLink.CurrentTaskPaneType = TaskPaneKey.InsertGenePatternLink;
                    result = insertGenePatternLink;
                    break;
                case TaskPaneKey.RerunPipeline:
                    RerunPipelineTaskPane RerunPipelineTaskPaneUserControl = new RerunPipelineTaskPane();
                    RerunPipelineTaskPaneUserControl.SetController(controller);
                    result = RerunPipelineTaskPaneUserControl;
                    break;

            }

            return result;
        }

        /// <summary>
        /// Creates the edit user control.
        /// </summary>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        /// <returns></returns>
        public UserControl CreateEditUserControl(PipelineAssociation pipelineAssociation)
        {
            AssociatePipelineTaskPane editPipeline = new AssociatePipelineTaskPane();
            editPipeline.IsEditMode = true;
            editPipeline.Controller = controller;
            editPipeline.EditExistingPipeline(pipelineAssociation);
            editPipeline.CurrentTaskPaneType = TaskPaneKey.EditPipeline;
            return editPipeline;
        }

        /// <summary>
        /// Gets the title matching the specified <c>TaskPaneKey</c>.
        /// </summary>
        /// <param name="taskPaneKey">The task pane key.</param>
        /// <returns>a <c>string</c> with the matching title</returns>
        public string GetTitle(TaskPaneKey taskPaneKey)
        {
            string result;

            switch (taskPaneKey)
            {
                case TaskPaneKey.ViewAssociations:
                    result = Resources.VIEW_ASSOCIATIONS_TITLE;
                    break;
                case TaskPaneKey.Dashboard:
                    result = Resources.DASHBOARD_TITLE;
                    break;
                case TaskPaneKey.Connections:
                    result = Resources.VIEW_CONNECTIONS_TITLE;
                    break;
                case TaskPaneKey.AssociatePipeline:
                    result = Resources.ASSOCIATE_PIPELINE;
                    break;
                case TaskPaneKey.InsertGenePatternLink:
                    result = Resources.INSERT_GENEPATTERN_LINK;
                    break;
                case TaskPaneKey.EditPipeline:
                    result = Resources.EDIT_PIPELINE;
                    break;
                case TaskPaneKey.RerunPipeline:
                    result = Resources.RERUN_PIPELINE;
                    break;
                    
                default:
                    Trace.WriteLine("No title found for taskPaneKey: " + taskPaneKey.ToString(), TraceLevel.Error.ToString());
                    result = "Default title";
                    break;
            }

            return result;
        }

        /// <summary>
        /// Gets the task pane key.
        /// </summary>
        /// <param name="taskPaneTitleOrControlId">The task pane title.</param>
        /// <returns></returns>
        public TaskPaneKey GetTaskPaneKey(string taskPaneTitleOrControlId)
        {
            TaskPaneKey result = TaskPaneKey.Default;

            if (taskPaneTitleOrControlId == Resources.VIEW_ASSOCIATIONS_TITLE || taskPaneTitleOrControlId == VIEW_ASSOCIATIONS_TOGGLE_BUTTON)
            {
                result = TaskPaneKey.ViewAssociations;
            }
            else if (taskPaneTitleOrControlId == Resources.DASHBOARD_TITLE || taskPaneTitleOrControlId == DASHBOARD_TOGGLE_BUTTON)
            {
                result = TaskPaneKey.Dashboard;
            }
            else if (taskPaneTitleOrControlId == Resources.VIEW_CONNECTIONS_TITLE || taskPaneTitleOrControlId == VIEW_CONNECTIONS_TOGGLE_BUTTON)
            {
                result = TaskPaneKey.Connections;
            }
            else if (taskPaneTitleOrControlId == Resources.ASSOCIATE_PIPELINE || taskPaneTitleOrControlId == ASSOCIATE_PIPELINE_TOGGLE_BUTTON)
            {
                result = TaskPaneKey.AssociatePipeline;
            }
            else if (taskPaneTitleOrControlId == Resources.INSERT_GENEPATTERN_LINK || taskPaneTitleOrControlId == INSERT_GENEPATTERN_LINK_TOGGLE_BUTTON)
            {
                result = TaskPaneKey.InsertGenePatternLink;
            }
            else if (taskPaneTitleOrControlId == Resources.EDIT_PIPELINE)
            {
                result = TaskPaneKey.EditPipeline;
            }
            else if (taskPaneTitleOrControlId == Resources.RERUN_PIPELINE || taskPaneTitleOrControlId == RERUN_PIPELINE_TOGGLE_BUTTON)
            {
                result = TaskPaneKey.RerunPipeline;
            }
            else
            {
                Trace.WriteLine("No taskPaneKey found for title: " + taskPaneTitleOrControlId, TraceLevel.Error.ToString());
            }

            return result;
        }

        /// <summary>
        /// Gets the control id.
        /// </summary>
        /// <param name="taskPaneKey">The task pane key.</param>
        /// <returns></returns>
        public string GetControlId(TaskPaneKey taskPaneKey)
        {
            string result;

            switch (taskPaneKey)
            {
                case TaskPaneKey.ViewAssociations:
                    result = VIEW_ASSOCIATIONS_TOGGLE_BUTTON;
                    break;
                case TaskPaneKey.Dashboard:
                    result = DASHBOARD_TOGGLE_BUTTON;
                    break;
                case TaskPaneKey.Connections:
                    result = VIEW_CONNECTIONS_TOGGLE_BUTTON;
                    break;
                case TaskPaneKey.AssociatePipeline:
                    result = ASSOCIATE_PIPELINE_TOGGLE_BUTTON;
                    break;
                case TaskPaneKey.InsertGenePatternLink:
                    result = INSERT_GENEPATTERN_LINK_TOGGLE_BUTTON;
                    break;
                case TaskPaneKey.RerunPipeline:
                    result = RERUN_PIPELINE_TOGGLE_BUTTON;
                    break;
                default:
                    Trace.WriteLine("No controlId for taskPaneKey: " + taskPaneKey.ToString(), TraceLevel.Error.ToString());
                    result = "Default title";
                    break;
            }

            return result;
        }

        /// <summary>
        /// Gets the control id.
        /// </summary>
        /// <param name="controlKey">The control key.</param>
        /// <returns></returns>
        public string GetControlId(ControlKey controlKey)
        {
            string result = null;
            switch (controlKey)
            {
                case ControlKey.ConnectionDropDown:
                    result = CONNECTIONS_DROP_DOWN;
                    break;
                case ControlKey.WizardFreeCheckBox:
                    result = WIZARD_FREE_CHECKBOX;
                    break;
                case ControlKey.ViewAssociationButton:
                    result = VIEW_ASSOCIATIONS_TOGGLE_BUTTON;
                    break;
                case ControlKey.DashBoardButton:
                    result = DASHBOARD_TOGGLE_BUTTON;
                    break;
                case ControlKey.ConnectionsButton:
                    result = VIEW_CONNECTIONS_TOGGLE_BUTTON;
                    break;
                case ControlKey.AssociatePipelineButton:
                    result = ASSOCIATE_PIPELINE_TOGGLE_BUTTON;
                    break;
                case ControlKey.InsertGenePatternLinkButton:
                    result = INSERT_GENEPATTERN_LINK_TOGGLE_BUTTON;
                    break;
                case ControlKey.RerunPipelineButton:
                    result = RERUN_PIPELINE_TOGGLE_BUTTON;
                    break;
                case ControlKey.ProtectDocumentButton:
                    result = PROTECT_DOCUMENT_TOGGLE_BUTTON;
                    break;
            }
            return result;
        }

        #endregion
    }
}
