using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Reflection;
using System.Runtime.InteropServices;
using Microsoft.eResearch.GenePattern.Common;
using MSOffice = Microsoft.Office.Core;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// The ribbon extensibility of the GenePattern Package
    /// 
    /// Class must be declared as public on order for Word to be able to show the ribbon defined in here.
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Interoperability", "CA1409:ComVisibleTypesShouldBeCreatable"), ComVisible(true)]
    public class GenePatternRibbon : IRibbonInvalidator, MSOffice.IRibbonExtensibility
    {
        #region Fields

        private MSOffice.IRibbonUI _ribbonUI;
        private readonly IController controller;
        private List<Connection> connectionsList;
        private string connectionDropDownId;
        private bool invalidateRibbon = true;
        #endregion

        /// <summary>
        /// Initializes a new instance of the <see cref="GenePatternRibbon"/> class.
        /// </summary>
        internal GenePatternRibbon(IController controller)
        {
            if (controller == null)
                throw new ArgumentNullException("controller");

            this.controller = controller;
            controller.RibbonInvalidator = this;
        }

        #region IRibbonExtensibility Members

        /// <summary>
        /// Gets the custom UI.
        /// </summary>
        /// <param name="RibbonID">The ribbon ID.</param>
        /// <returns></returns>
        public string GetCustomUI(string RibbonID)
        {
            return Resources.GenePatternRibbon;
        }

        #endregion

        #region IRibbon Validator Members

        /// <summary>
        /// Invalidates the control.
        /// </summary>
        /// <param name="controlId">The control id.</param>
        public void InvalidateControl(string controlId)
        {
            try
            {
                _ribbonUI.InvalidateControl(controlId);
            }
            catch (COMException)
            {
                // these can happen on shutdown.  we don't care.
            }
        }

        #endregion

        #region Ribbon Callbacks

        /// <summary>
        /// Called when ribbon extension loads.
        /// </summary>
        /// <param name="ribbonUI">The ribbon UI.</param>
        public void OnLoad(MSOffice.IRibbonUI ribbonUI)
        {
            this._ribbonUI = ribbonUI;
        }

        /// <summary>
        /// Callback for getting the label for a ribbon control.
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        public string GetLabel(MSOffice.IRibbonControl ribbonControl)
        {
            string result = string.Empty;
            try
            {
                if (ribbonControl == null)
                {
                    Trace.WriteLine("ribbonControl is null", TraceLevel.Error.ToString());
                    throw new ArgumentNullException("ribbonControl");
                }

                string labelResourceName = ribbonControl.Id + "Label";
                result = GetResourceString(labelResourceName);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
            return result;
        }

        /// <summary>
        /// The callback for loading a named image for a ribbon control.
        /// </summary>
        /// <param name="imageName">Name of the image.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        public Bitmap LoadImage(string imageName)
        {
            Bitmap result = Resources.NotFound;
            try
            {
                PropertyInfo propertyInfo = typeof(Resources).GetProperty(imageName, BindingFlags.Static | BindingFlags.NonPublic);
                if (propertyInfo != null)
                {
                    result = propertyInfo.GetValue(null, null) as Bitmap;
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
            return result;
        }

        /// <summary>
        /// The callback for getting the screen tip for a ribbon control.
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        public string GetScreenTip(MSOffice.IRibbonControl ribbonControl)
        {
            string result = string.Empty;
            try
            {
                if (ribbonControl == null)
                {
                    throw new ArgumentNullException("ribbonControl");
                }
                string screenTipResourceName = ribbonControl.Id + "ScreenTip";
                result = GetResourceString(screenTipResourceName);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
            return result;
        }

        /// <summary>
        /// On action callback for the Dashboard toggle button.
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <param name="isPressed">if set to <c>true</c> [is pressed].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public void DashboardToggleButtonOnAction(MSOffice.IRibbonControl ribbonControl, bool isPressed)
        {
            try
            {
                if (ribbonControl == null)
                    throw new ArgumentNullException("ribbonControl");

                DashboardCommand command = new DashboardCommand(controller);
                command.Invoke(ribbonControl.Id, isPressed);
                controller.SetButtonStatus(ControlKey.DashBoardButton, isPressed);
                CheckForNeedToInvalidate(command, ribbonControl.Id);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// On action callback for the ViewsAssociations toggle button .
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <param name="isPressed">if set to <c>true</c> [is pressed].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public void ViewAssociationsToggleButtonOnAction(MSOffice.IRibbonControl ribbonControl, bool isPressed)
        {
            try
            {
                if (ribbonControl == null)
                    throw new ArgumentNullException("ribbonControl");

                ViewAssociationsCommand command = new ViewAssociationsCommand(controller);
                command.Invoke(isPressed);
                controller.SetButtonStatus(ControlKey.ViewAssociationButton, isPressed);
                CheckForNeedToInvalidate(command, ribbonControl.Id);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
                Trace.WriteLine("Exception displaying visualizers:" + ex.ToString());
            }
        }

        /// <summary>
        /// Deletes the gene pattern data button on action.
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public void DeleteGenePatternDataButtonOnAction(MSOffice.IRibbonControl ribbonControl)
        {
            try
            {
                if (ribbonControl == null)
                    throw new ArgumentNullException("ribbonControl");

                Command command = new DeleteGenePatternDataCommand(controller);
                command.Invoke(ribbonControl.Id);
                CheckForNeedToInvalidate(command, ribbonControl.Id);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// on action callback for  viewing the connectionsList callback
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <param name="isPressed">if set to <c>true</c> [is pressed].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public void ViewConnectionsToggleButtonOnAction(MSOffice.IRibbonControl ribbonControl, bool isPressed)
        {
            try
            {
                if (ribbonControl == null)
                    throw new ArgumentNullException("ribbonControl");

                ViewConnectionsCommand command = new ViewConnectionsCommand(controller);
                command.Invoke(isPressed);
                controller.SetButtonStatus(ControlKey.ConnectionsButton, isPressed);
                CheckForNeedToInvalidate(command, ribbonControl.Id);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Associates the pipeline button on action.
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <param name="isPressed">if set to <c>true</c> [is pressed].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public void AssociatePipelineButtonOnAction(MSOffice.IRibbonControl ribbonControl, bool isPressed)
        {
            try
            {
                if (ribbonControl == null)
                    throw new ArgumentNullException("ribbonControl");

                if (controller.IsWizardUI)
                {
                    Command command = new AssociatePipelineCommand(controller);
                    command.Invoke(ribbonControl.Id);
                    isPressed = false;
                    _ribbonUI.InvalidateControl(ribbonControl.Id);
                   
                }
                else
                {
                    PaneAsscoiatePipelineCommand associatePipelineCommand = new PaneAsscoiatePipelineCommand(controller);
                    if (associatePipelineCommand.Invoke(isPressed))
                    {
                        CheckForNeedToInvalidate(associatePipelineCommand, ribbonControl.Id);
                    }
                    else
                    {
                        isPressed = false;
                        _ribbonUI.InvalidateControl(ribbonControl.Id);
                    }
                }

                controller.SetButtonStatus(ControlKey.AssociatePipelineButton, isPressed);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Protects the document button on action.
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <param name="isPressed">if set to <c>true</c> [is pressed].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public void ProtectDocumentButtonOnAction(MSOffice.IRibbonControl ribbonControl, bool isPressed)
        {
            try
            {
                if (ribbonControl == null)
                    throw new ArgumentNullException("ribbonControl");

                Command command = new ProtectDocumentCommand(controller);
                command.Invoke(ribbonControl.Id);
                controller.SetButtonStatus(ControlKey.ProtectDocumentButton, isPressed);
                CheckForNeedToInvalidate(command, ribbonControl.Id);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Genes the pattern link button on action.
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <param name="isPressed">if set to <c>true</c> [is pressed].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public void GenePatternLinkButtonOnAction(MSOffice.IRibbonControl ribbonControl, bool isPressed)
        {
            try
            {
                if (ribbonControl == null)
                    throw new ArgumentNullException("ribbonControl");
                if (controller.IsWizardUI)
                {
                    Command command = new InsertGenePatternLinkCommand(controller);
                    command.Invoke(ribbonControl.Id);
                    isPressed = false;
                    _ribbonUI.InvalidateControl(ribbonControl.Id);
                }
                else
                {
                    PaneInsertGenePatternLinkedCommand insertGenePatternLinkedCommand =
                        new PaneInsertGenePatternLinkedCommand(controller);
                    if (insertGenePatternLinkedCommand.Invoke(isPressed))
                    {
                        CheckForNeedToInvalidate(insertGenePatternLinkedCommand, ribbonControl.Id);
                    }
                    else
                    {
                        isPressed = false;
                        _ribbonUI.InvalidateControl(ribbonControl.Id);
                    }
                }
                controller.SetButtonStatus(ControlKey.InsertGenePatternLinkButton, isPressed);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Reruns the pipeline button on action.
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <param name="isPressed">if set to <c>true</c> [is pressed].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public void RerunPipelineButtonOnAction(MSOffice.IRibbonControl ribbonControl, bool isPressed)
        {
            try
            {
                if (ribbonControl == null)
                    throw new ArgumentNullException("ribbonControl");

                if (controller.IsWizardUI)
                {
                    Command command = new RerunPipelineCommand(controller);
                    command.Invoke(ribbonControl.Id);
                    isPressed = false;
                    _ribbonUI.InvalidateControl(ribbonControl.Id);
                   
                }
                else
                {
                    PaneRenurPipelineCommand rerunPipelineCommand = new PaneRenurPipelineCommand(controller);
                    if (rerunPipelineCommand.Invoke(isPressed))
                    {
                        CheckForNeedToInvalidate(rerunPipelineCommand, ribbonControl.Id);
                    }
                    else
                    {
                        isPressed = false;
                        _ribbonUI.InvalidateControl(ribbonControl.Id);
                    }
                }

                controller.SetButtonStatus(ControlKey.RerunPipelineButton, isPressed);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Callback for determinig the pressed state of a ribbon toggle button.
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        public bool ToggleButtonGetPressed(MSOffice.IRibbonControl ribbonControl)
        {
            bool result = false;
            try
            {
                if (ribbonControl == null)
                {
                    Trace.WriteLine("ribbonControl is null", TraceLevel.Error.ToString());
                    throw new ArgumentNullException("ribbonControl");
                }

                if (!controller.ActiveDocumentIsNull)
                {
                    TaskPaneKey taskPaneKey = controller.UserControlCreator.GetTaskPaneKey(ribbonControl.Id);
                    result = controller.TaskPaneManager.IsTaskPaneVisible(controller.UserControlCreator.GetTitle(taskPaneKey));
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
            return result;
        }

        /// <summary>
        /// Documents the protected get pressed.
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        public bool DocumentProtectedGetPressed(MSOffice.IRibbonControl ribbonControl)
        {
            bool result = false;
            try
            {
                if (ribbonControl == null)
                {
                    Trace.WriteLine("ribbonControl is null", TraceLevel.Error.ToString());
                    throw new ArgumentNullException("ribbonControl");
                }

                result = controller.DocumentIsProtected;
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
            return result;
        }

        /// <summary>
        /// The onAction callback for the New connection button.
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public void NewConnectionButtonOnAction(MSOffice.IRibbonControl ribbonControl)
        {
            try
            {
                if (ribbonControl == null)
                    throw new ArgumentNullException("ribbonControl");

                Command command = new EditConnectionCommand(controller);
                command.Invoke(ribbonControl.Id);
                CheckForNeedToInvalidate(command, ribbonControl.Id);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Connectionses the drop down on action.
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <param name="selectedValue">The selected value.</param>
        /// <param name="selectedIndex">Index of the selected.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId = "selectedValue")]
        public void ConnectionsDropDownOnAction(MSOffice.IRibbonControl ribbonControl, string selectedValue, int selectedIndex)
        {
            try
            {
                if (ribbonControl == null)
                    throw new ArgumentNullException("ribbonControl");

                SetConnectionFormIndex(selectedIndex);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Sets the index of the connection form.
        /// </summary>
        /// <param name="selectedIndex">Index of the selected.</param>
        private void SetConnectionFormIndex(int selectedIndex)
        {
            Connection connection = this.connectionsList[selectedIndex];
            if (connection != null)
            {
                this.controller.UserData.CurrentConnectionName = connection.Name;
                this.controller.CurrentConnection = connection;
            }
        }

        /// <summary>
        /// Callback for the Shows the connectionsList button
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public int GetconnectionsDropDownItemCount(MSOffice.IRibbonControl ribbonControl)
        {
            int noOfItems = 0;
            try
            {
                if (ribbonControl == null)
                    throw new ArgumentNullException("ribbonControl");
                this.connectionDropDownId = ribbonControl.Id;

                // use a list to match index to connection as it will be used in the 
                // other callback for the connections drop-down list
                // as the Connections dictionary can't be index by integer
                this.connectionsList = new List<Connection>();
                foreach (KeyValuePair<string, Connection> connection in controller.UserData.Connections)
                {
                    this.connectionsList.Add(connection.Value);
                }

                noOfItems = this.connectionsList.Count;
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
            return noOfItems;
        }

        /// <summary>
        /// Callback caled for each item in drop down (This provides the index for the connection name)
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public int GetconnectionsDropDownSelectedItemIndex(MSOffice.IRibbonControl ribbonControl)
        {
            int index = 0;

            try
            {
                if (controller.DocumentCount == 0)
                    return index;

                if (ribbonControl == null)
                    throw new ArgumentNullException("ribbonControl");

                //Get the Active Connection from Stored location for current Document
                string connectionName = controller.ActiveConnectionForDocument;
                if (string.IsNullOrEmpty(connectionName))
                {
                    connectionName = controller.UserData.CurrentConnectionName;
                    controller.ActiveConnectionForDocument = connectionName;
                }

                if (string.IsNullOrEmpty(connectionName) && this.connectionsList.Count > 0)
                {
                    SetConnectionFormIndex(0);
                }
                else
                {
                    foreach (Connection connection in this.connectionsList)
                    {
                        if (connection.Name.Equals(connectionName))
                        {
                            break;
                        }
                        index++;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
            return index;
        }

        /// <summary>
        /// Callback for the Shows the connectionsList button
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <param name="index">The index.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public string GetconnectionsDropDownLabel(MSOffice.IRibbonControl ribbonControl, int index)
        {
            String label = String.Empty;
            try
            {
                if (ribbonControl == null)
                    throw new ArgumentNullException("ribbonControl");
                label = this.connectionsList[index].Name;
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }

            return label;
        }

        /// <summary>
        /// Called when [wizard free check changed].
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <param name="isChecked">if set to <c>true</c> [is checked].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public void OnWizardFreeCheckChanged(MSOffice.IRibbonControl ribbonControl, bool isChecked)
        {
            try
            {
                if (controller.DocumentCount == 0)
                    return;

                if (ribbonControl == null)
                    throw new ArgumentNullException("ribbonControl");

                controller.IsWizardUI = isChecked;
                if (isChecked)
                {
                    if (!controller.SwitchToWizardUI())
                    {
                        controller.IsWizardUI = true;
                        GetWizardCheckPressed(ribbonControl);
                        _ribbonUI.InvalidateControl(ribbonControl.Id);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Gets the wizard check pressed.
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId = "ribbonControl")]
        public bool GetWizardCheckPressed(MSOffice.IRibbonControl ribbonControl)
        {
            try
            {
                return controller.IsWizardUI;
            }
            catch (COMException) { }
            
            return false;
        }

        /// <summary>
        /// Called when [dialog open event].
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        /// <param name="cancelDefault">if set to <c>true</c> [cancel default].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId = "ribbonControl")]
        public void OnDialogOpenEvent(MSOffice.IRibbonControl ribbonControl, ref bool cancelDefault)
        {
            cancelDefault = controller.CanOpenModelDialog();
        }

        /// <summary>
        /// Gets the enabled.
        /// </summary>
        /// <param name="ribbonControl">The ribbon control.</param>
        public bool GetEnabled(MSOffice.IRibbonControl ribbonControl)
        {
            if (controller.DocumentCount > 0)
            {
                if (!invalidateRibbon)
                {
                    _ribbonUI.Invalidate();
                }
                invalidateRibbon = true;
                return true;
            }

            if (invalidateRibbon)
            {
                _ribbonUI.Invalidate();
                invalidateRibbon = false;
            }

            return false;
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Gets a named resource string.
        /// </summary>
        /// <param name="resourceName">Name of the resource.</param>
        /// <returns></returns>
        private static string GetResourceString(string resourceName)
        {
            string result = Resources.ResourceManager.GetString(resourceName);

            if (result == null)
            {
                Trace.WriteLine("Resource string not found for name = " + resourceName, TraceLevel.Error.ToString());
                result = resourceName;
            }

            return result;
        }

        /// <summary>
        /// Checks for need to invalidate the ribbon control.
        /// </summary>
        /// <param name="command">The command.</param>
        /// <param name="controlId">The control id.</param>
        private void CheckForNeedToInvalidate(Command command, string controlId)
        {
            if (command.NeedsInvalidate)
            {
                _ribbonUI.InvalidateControl(controlId);
            }
        }

        #endregion

        /// <summary>
        /// Gets the connection drop down ID.
        /// </summary>
        /// <value>The connection drop down ID.</value>
        public string ConnectionDropDownId
        {
            get { return connectionDropDownId; }
        }
    }
}
