using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.ScientificAddIn.Common;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// Callback used to signify that a file has been download.  Assumes whoever is handling the callback knows what the total expected number is.
    /// </summary>
    public delegate void DownloadProgress(string name, string lsid, bool isStarting);

    /// <summary>
    /// Delegate for Submit Job Progress
    /// </summary>
    public delegate void SubmitProgress(string name, bool isStarting);

    /// <summary>
    /// The controller of the GenePattern Package
    /// </summary>
    public interface IController
    {
        /// <summary>
        /// Gets or sets the task pane manager.
        /// </summary>
        /// <value>The task pane manager.</value>
        ITaskPaneManager TaskPaneManager { get; set; }

        /// <summary>
        /// Gets or sets the data store.
        /// </summary>
        /// <value>The data store.</value>
        IDataStore DataStore { get; set; }

        /// <summary>
        /// Gets the models.
        /// </summary>
        /// <value>The models.</value>
        Dictionary<string, Model> Models { get; }

        /// <summary>
        /// Gets or sets the ribbon invalidator.
        /// </summary>
        /// <value>The ribbon invalidator.</value>
        IRibbonInvalidator RibbonInvalidator { get; set; }

        /// <summary>
        /// Gets the connection.
        /// </summary>
        /// <value>The connection.</value>
        Connection CurrentConnection { get; set; }

        /// <summary>
        /// Gets the user data.
        /// </summary>
        /// <value>The user data.</value>
        UserData UserData
        {
            get;
        }

        /// <summary>
        /// Gets a value indicating whether active document is null.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if active document is null; otherwise, <c>false</c>.
        /// </value>
        bool ActiveDocumentIsNull { get; }

        /// <summary>
        /// Gets the gene pattern server instance
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        IGenePatternServer GetGenePatternServer();

        /// <summary>
        /// Shows the associate pipeline wizard.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        List<PipelineAssociation> ShowAssociatePipelineWizard(bool isLinked);

        /// <summary>
        /// Sets the package host.
        /// </summary>
        /// <param name="packageHost">The package host.</param>
        void SetPackageHost(IPackageHost packageHost);

        /// <summary>
        /// Updates the pipeline association.
        /// </summary>
        /// <param name="pipelines">The pipelines.</param>
        /// <param name="pipelineAssociations">The pipeline associations.</param>
        List<string> UpdatePipelineAssociations(List<Pipeline> pipelines, List<PipelineAssociation> pipelineAssociations);

        /// <summary>
        /// Updates the pipeline association.
        /// </summary>
        /// <param name="pipelines">The pipelines.</param>
        /// <param name="pipelineAssociations">The pipeline associations.</param>
        /// <param name="downloadProgressCallback">The callback for monitoring progress</param>
        List<string> UpdatePipelineAssociations(List<Pipeline> pipelines, List<PipelineAssociation> pipelineAssociations, DownloadProgress downloadProgressCallback);

        /// <summary>
        /// Updates the pipeline associations.
        /// </summary>
        /// <param name="pipelines">The pipelines.</param>
        /// <param name="pipelineAssociations">The pipeline associations.</param>
        /// <param name="downloadProgressCallback">The download progress callback.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="genePatternServer">The gene pattern server.</param>
        /// <returns></returns>
        List<string> UpdatePipelineAssociations(List<Pipeline> pipelines,
            List<PipelineAssociation> pipelineAssociations, DownloadProgress downloadProgressCallback,
            string documentKey, IGenePatternServer genePatternServer);

        /// <summary>
        /// The underlying model for the current document
        /// </summary>
        Model ActiveModel { get; }

        /// <summary>
        /// Gets the user control creator.
        /// </summary>
        /// <value>The user control creator.</value>
        IUserControlCreator UserControlCreator
        {
            get;
        }

        /// <summary>
        /// Checks the document protection.
        /// </summary>
        /// <returns></returns>
        bool CheckDocumentProtection();

        /// <summary>
        /// Gets the server broker.
        /// </summary>
        /// <value>The server broker.</value>
        IServerBroker ServerBroker { get; }

        /// <summary>
        /// Gets a value indicating whether [document is protected].
        /// </summary>
        /// <value><c>true</c> if [document is protected]; otherwise, <c>false</c>.</value>
        bool DocumentIsProtected { get; }

        /// <summary>
        /// Adds the pipeline association.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        List<string> AddPipelineAssociations(List<Pipeline> pipeline, List<PipelineAssociation> pipelineAssociation);

        /// <summary>
        /// Adds the pipeline associations.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        /// <param name="downloadProgressCallback">The download progress callback.</param>
        /// <returns></returns>
        List<string> AddPipelineAssociations(List<Pipeline> pipeline, List<PipelineAssociation> pipelineAssociation,
            DownloadProgress downloadProgressCallback);

        /// <summary>
        /// Adds the pipeline associations.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        /// <param name="downloadProgressCallback">The download progress callback.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="documentImage">The document image.</param>
        /// <param name="genePatternServer">The gene pattern server.</param>
        /// <returns></returns>
        List<string> AddPipelineAssociations(List<Pipeline> pipeline, List<PipelineAssociation> pipelineAssociation,
            DownloadProgress downloadProgressCallback, string documentKey, DocumentImage documentImage,
            IGenePatternServer genePatternServer);

        /// <summary>
        /// Gets the pipelineID files.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <returns>a <c>MemoryStream</c> representing the pipeline zip file</returns>
        MemoryStream GetPipelineZip(string lsid, string documentKey);

        /// <summary>
        /// Gets the associated pipelines for selected image.
        /// </summary>
        /// <returns>a <c>List&lt;PipelineAssociation&gt;</c> of the pipeline associations 
        /// associated with the image</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        List<PipelineAssociation> GetAssociatedPipelinesForSelectedImage();

        /// <summary>
        /// Get the active rerunpipeline task pane
        /// </summary>
        /// <param name="documentKey"></param>
        /// <returns></returns>
        RerunPipelineTaskPane GetActiveRerunPipelineTaskPane(string documentKey);


        /// <summary>
        /// Gets all pipeline associations in the model
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        ReadOnlyCollection<PipelineAssociation> GetAllPipelineAssociations();

        /// <summary>
        /// Deletes all pipeline associations in the model
        /// </summary>
        void DeleteAllPipelineassociations(ReadOnlyCollection<PipelineAssociation> pipelineassociations);

        /// <summary>
        /// Cleans the data store.
        /// </summary>
        void CleanDataStore();

        /// <summary>
        /// Removes the connection.
        /// </summary>
        /// <param name="connectionName">Name of the connection.</param>
        /// <returns></returns>
        bool RemoveConnection(string connectionName);

        /// <summary>
        /// Shows the server connection form.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="editMode">if set to <c>true</c> [edit mode].</param>
        void ShowServerConnectionForm(Connection connection, bool editMode);

        /// <summary>
        /// Shows the server connection form with GenePattern Public Server Settings.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="editMode">if set to <c>true</c> [edit mode].</param>
        void ShowGenePatternServerConnectionForm(Connection connection, bool editMode);

        /// <summary>
        /// Deletes the <c>PipelineAssociation</c> from the Model
        /// </summary>
        /// <param name="assoc"></param>
        void DeletePipelineAssociation(PipelineAssociation assoc);

        /// <summary>
        /// Inserts the gene pattern link image.
        /// </summary>
        Image InsertGenePatternLinkImage(bool showWizard, PipelineAssociation existingAssociationInsteadOfWizard);

        /// <summary>
        /// Toggles the document protection.
        /// </summary>
        /// <returns></returns>
        bool ToggleDocumentProtection();

        /// <summary>
        /// Begins the model changes transaction.
        /// </summary>
        void BeginModelChanges();

        /// <summary>
        /// Begins the model changes.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        void BeginModelChanges(string documentKey);


        /// <summary>
        /// Commits the model changes.
        /// </summary>
        void CommitModelChanges();

        /// <summary>
        /// Commits the model changes.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        void CommitModelChanges(string documentKey);

        /// <summary>
        /// Rollbacks the model changes.
        /// </summary>
        void RollbackModelChanges();

        /// <summary>
        /// Rollbacks the model changes.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        void RollbackModelChanges(string documentKey);

        /// <summary>
        /// Shows the rerun pipeline wizard.
        /// </summary>
        bool ShowRerunPipelineWizard();

        /// <summary>
        /// Submits the job.
        /// </summary>
        /// <param name="job">The job.</param>
        /// <param name="connection">The connection.</param>
        /// <param name="registerNotifcation">if set to <c>true</c> register notifcation.</param>
        /// <param name="emailAddress">The email address.</param>
        /// <param name="runInOriginalLocation">if set to <c>true</c> [run in original location].</param>
        /// <returns>the id of the new job</returns>
        int SubmitJob(Job job, Connection connection, bool registerNotifcation,
                        string emailAddress, bool runInOriginalLocation);

        /// <summary>
        /// Submits the job.
        /// </summary>
        /// <param name="job">The job.</param>
        /// <param name="connection">The connection.</param>
        /// <param name="registerNotifcation">if set to <c>true</c> register notifcation.</param>
        /// <param name="emailAddress">The email address.</param>
        /// <param name="runInOriginalLocation">if set to <c>true</c> [run in original location].</param>
        /// <param name="submitProgress">The submit progress.</param>
        /// <returns>the id of the new job</returns>
        int SubmitJob(Job job, Connection connection, bool registerNotifcation,
                    string emailAddress, bool runInOriginalLocation, SubmitProgress submitProgress);


        /// <summary>
        /// Submits the job.
        /// </summary>
        /// <param name="job">The job.</param>
        /// <param name="connection">The connection.</param>
        /// <param name="registerNotifcation">if set to <c>true</c> register notifcation.</param>
        /// <param name="emailAddress">The email address.</param>
        /// <param name="runInOriginalLocation">if set to <c>true</c> [run in original location].</param>
        /// <param name="submitProgress">The submit progress.</param>
        /// <param name="documentKey">Document Key To Get the document from which submit job was invoked</param>
        /// <returns>the id of the new job</returns>
        int SubmitJob(Job job, Connection connection, bool registerNotifcation,
                    string emailAddress, bool runInOriginalLocation, SubmitProgress submitProgress, string documentKey);

        /// <summary>
        /// Shift document focus to the appendix
        /// </summary>
        /// <param name="association">The association.</param>
        /// <param name="showParams">if set to <c>true</c> [show params].</param>
        /// <param name="comment">The comment.</param>
        /// <param name="resultsToEmbed">The results to embed.</param>
        /// <param name="isImage">if set to <c>true</c> [is image].</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        string[] AddAppendixEntry(PipelineAssociation association, bool showParams, string comment, 
            List<Result> resultsToEmbed, bool isImage, string documentKey);

        /// <summary>
        /// Deletes appendix objects/images from document
        /// </summary>
        void DeleteAppendix();

        /// <summary>
        /// Gets the pipeline zip.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <param name="includePipelineSteps">if set to <c>true</c> [include pipeline steps].</param>
        /// <returns></returns>
        MemoryStream GetPipelineZip(string lsid, bool includePipelineSteps);

        /// <summary>
        /// Extracts the module.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <param name="path">The path.</param>
        void ExtractModule(string lsid, string path, string documentKey);

        /// <summary>
        /// Extracts the module.
        /// </summary>
        /// <param name="zipStream">The zip stream.</param>
        /// <param name="path">The path.</param>
        void ExtractModule(MemoryStream zipStream, string path);

        /// <summary>
        /// Signals the rerun is present.
        /// </summary>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        void SignalRerunIsPresent(PipelineAssociation pipelineAssociation);

        /// <summary>
        /// Scrolls to association.
        /// </summary>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        void ScrollToAssociation(PipelineAssociation pipelineAssociation);

        /// <summary>
        /// Inserts the image with association.
        /// </summary>
        /// <param name="thumbnailImage">The thumbnail image.</param>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        void InsertImageWithAssociation(Image thumbnailImage, PipelineAssociation pipelineAssociation);

        /// <summary>
        /// Inserts the image.
        /// </summary>
        /// <param name="image">The image.</param>
        void InsertImage(Image image);

        /// <summary>
        /// Inserts the picture.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        void InsertPicture(string filePath);

        /// <summary>
        /// Gets the content items.
        /// </summary>
        /// <returns></returns>
        ReadOnlyCollection<ContentItem> GetContentItems(ReadOnlyCollection<PipelineAssociation> pipelineAssociations, string documentKey);

        /// <summary>
        /// Updates the pipeline associations display.
        /// </summary>
        void UpdatePipelineAssociationsDisplay();

        /// <summary>
        /// Assures the connection.
        /// </summary>
        /// <returns></returns>
        bool AssureConnection();

        /// <summary>
        /// Check to see if this is the first time to application has been run
        /// </summary>
        void CheckFirstTimeRunning();

        /// <summary>
        /// Gets or sets the active connection for document.
        /// </summary>
        /// <value>The active connection for document.</value>
        string ActiveConnectionForDocument { get; set; }

        /// <summary>
        /// Gets the document key.
        /// </summary>
        /// <value>The document key.</value>
        string DocumentKey { get;}

        /// <summary>
        /// Gets the document model.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        Model GetDocumentModel(string documentKey);

        /// <summary>
        /// Determines whether this instance [can pipeline associate] the specified is linked.
        /// </summary>
        /// <param name="isLinked">if set to <c>true</c> [is linked].</param>
        /// <returns>
        /// 	<c>true</c> if this instance [can pipeline associate] the specified is linked; otherwise, <c>false</c>.
        /// </returns>
        bool CanPipelineAssociate(bool isLinked);

        /// <summary>
        /// Inserts the gene pattern link from pane.
        /// </summary>
        /// <param name="pipelineAssociations">The pipeline associations.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="range">The range.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        void InsertGenePatternLinkFromPane(List<PipelineAssociation> pipelineAssociations, string documentKey, ImageRange range);

        /// <summary>
        /// Gets the range.
        /// </summary>
        /// <returns></returns>
        ImageRange GetRange { get;}

        /// <summary>
        /// Gets a value indicating whether this instance is wizard free UI.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is wizard free UI; otherwise, <c>false</c>.
        /// </value>
        bool IsWizardUI { get; set;}

        /// <summary>
        /// Gets a value indicating whether the job is failed or not.
        /// </summary>
        /// <value>
        /// 	<c>false</c> if the job failed; otherwise, <c>true</c>.
        /// </value>
        bool JobStatus { get;}


        /// <summary>
        /// Gets the connection by document key.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        Connection GetConnectionByDocumentKey(string documentKey);

        /// <summary>
        /// Gets the get document image.
        /// </summary>
        /// <value>The get document image.</value>
        DocumentImage GetDocumentImage { get;}

        /// <summary>
        /// Updates the pipeline associations display.
        /// </summary>
        /// <param name="selectedImage">The selected image.</param>
        /// <param name="documentKey">The document key.</param>
        void UpdatePipelineAssociationsDisplay(DocumentImage selectedImage, string documentKey);

        /// <summary>
        /// Switches to wizard UI.
        /// </summary>
        /// <returns></returns>
        bool SwitchToWizardUI();

        /// <summary>
        /// Sets the button status.
        /// </summary>
        /// <param name="controlKey">The control key.</param>
        /// <param name="isPressed">if set to <c>true</c> [is pressed].</param>
        void SetButtonStatus(ControlKey controlKey, bool isPressed);

        /// <summary>
        /// Gets the button status.
        /// </summary>
        /// <param name="controlKey">The control key.</param>
        /// <returns></returns>
        bool GetButtonStatus(ControlKey controlKey);

        /// <summary>
        /// Determines whether [is job processing in progress] [the specified document key].
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <returns>
        /// 	<c>true</c> if [is job processing in progress] [the specified document key]; otherwise, <c>false</c>.
        /// </returns>
        bool IsJobProcessingInProgress(string documentKey);

        /// <summary>
        /// Sets the job processing status.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <param name="status">if set to <c>true</c> [status].</param>
        void SetJobProcessingStatus(string documentKey, bool status);

        /// <summary>
        /// Determines whether [is task pane visible] [the specified document key].
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <param name="taskPaneKey">The task pane key.</param>
        /// <returns>
        /// 	<c>true</c> if [is task pane visible] [the specified document key]; otherwise, <c>false</c>.
        /// </returns>
        bool IsTaskPaneVisible(string documentKey, TaskPaneKey taskPaneKey);

        /// <summary>
        /// Determines whether this instance [can pipeline rerun] the specified is linked.
        /// </summary>
        /// <param name="isLinked">if set to <c>true</c> [is linked].</param>
        /// <returns>
        /// 	<c>true</c> if this instance [can pipeline rerun] the specified is linked; otherwise, <c>false</c>.
        /// </returns>
        bool CanPipelineRerun(bool isLinked);

        /// <summary>
        /// Determines whether this instance [can open model dialog].
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if this instance [can open model dialog]; otherwise, <c>false</c>.
        /// </returns>
        bool CanOpenModelDialog();

        /// <summary>
        /// Gets the document count.
        /// </summary>
        /// <value>The document count.</value>
        int DocumentCount { get; }

        /// <summary>
        /// Gets the associated pipelines for selected image.
        /// </summary>
        /// <param name="documentImage">The document image.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        List<PipelineAssociation> GetAssociatedPipelinesForSelectedImage(DocumentImage documentImage, string documentKey);

        /// <summary>
        /// Updates the name of the rerun result association.
        /// </summary>
        /// <param name="newAssociationName">New name of the association.</param>
        /// <param name="imageKey">The image key.</param>
        void UpdateRerunResultAssociationName(string newAssociationName, string imageKey);

        /// <summary>
        /// Scrolls to association.
        /// </summary>
        /// <param name="documentImage">The document image.</param>
        /// <param name="documentKey">The document key.</param>
        void ScrollToImage(DocumentImage documentImage, string documentKey);
    }
}
