using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Text;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using System.Threading;

namespace Microsoft.eResearch.GenePattern.Package
{
    public partial class InsertAppendixForm : Form
    {
        private string primaryResultName;
        private IList<Result> results;
        private IGenePatternServer server;
        private PipelineAssociation association;
        private Thread downloadThread;


        /// <summary>
        /// Initializes a new instance of the <see cref="InsertAppendixForm"/> class.
        /// </summary>
        /// <param name="gpserver">The gpserver.</param>
        /// <param name="selectedAssociation">The selected association.</param>
        public InsertAppendixForm(IGenePatternServer gpserver, PipelineAssociation selectedAssociation)
        {
            server = gpserver;
            association = selectedAssociation;
            InitializeComponent();
            BackgroundImage = Resources.Clean;
            BackgroundImageLayout = ImageLayout.Stretch;
            this.DialogResult = DialogResult.Cancel;
        }

        /// <summary>
        /// Sets the parameters visible.
        /// </summary>
        /// <param name="visible">if set to <c>true</c> [visible].</param>
        public void SetParametersVisible(bool visible)
        {
            this.chkInsertInputFiles.Visible = visible;
        }

        /// <summary>
        /// Gets or sets the annotation text.
        /// </summary>
        /// <value>The annotation text.</value>
        public string AnnotationText
        {
            get
            {
                return Annotation.Text;
            }
            set { Annotation.Text = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [show parameters checked].
        /// </summary>
        /// <value>
        /// 	<c>true</c> if [show parameters checked]; otherwise, <c>false</c>.
        /// </value>
        public bool ShowParametersChecked
        {
            get { return chkInsertInputFiles.Checked; }
            set { chkInsertInputFiles.Checked = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [embed all results].
        /// </summary>
        /// <value><c>true</c> if [embed all results]; otherwise, <c>false</c>.</value>
        public bool EmbedAllResults
        {
            get { return radioAllFiles.Checked; }
            set { radioAllFiles.Checked = value; }
        }

        /// <summary>
        /// Gets or sets the name of the primary result.
        /// </summary>
        /// <value>The name of the primary result.</value>
        public string PrimaryResultName
        {
            get { return primaryResultName; }
            set
            {
                primaryResultName = value;
                string shortResultName = primaryResultName;
                if (shortResultName.Length > 50)
                {
                    shortResultName = shortResultName.Substring(0, 50) + "...";
                }
                lblSelectedResultFile.Text = string.Format(CultureInfo.InvariantCulture,
                                             Resources.INSERT_RESULT_INTO_APPENDIX,
                                             shortResultName);
            }
        }

        /// <summary>
        /// Gets or sets the results.
        /// </summary>
        /// <value>The results.</value>
        public IList<Result> Results
        {
            get { return results; }
            set { results = value; }
        }

        /// <summary>
        /// Handles the Click event of the OK control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OK_Click(object sender, EventArgs e)
        {
            downloadThread = new Thread(new ThreadStart(DownloadResults));
            OkButton.Enabled = false;
            downloadResultsBar.Visible = true;
            downloadResultsLabel.Visible = true;

            downloadThread.Start();
            downloadTimer.Start();


        }

        /// <summary>
        /// Downloads the results.
        /// </summary>
        private void DownloadResults()
        {
            try
            {
                IList<Result> jobresults = null;
                if (EmbedAllResults)
                {
                    jobresults = server.GetJobResults(association.Rerun.Job.Id);
                }
                else
                {

                    // find the first shallow result with the filename
                    int sourceJobId = -1;
                    foreach (ShallowJobResult res in association.Rerun.Job.ShallowResults)
                    {
                        if (res.Filename == primaryResultName)
                        {
                            sourceJobId = res.SourceJobId;
                            break;
                        }
                    }

                    // if we found the source job, we can download it exactly
                    if (sourceJobId != -1)
                        jobresults = server.GetJobResults(sourceJobId, primaryResultName);
                    else // otherwise get all results
                        jobresults = server.GetJobResults(association.Rerun.Job.Id);
                }

                results = new List<Result>();
                foreach (Result r in jobresults)
                {
                    if (!r.Filename.ToLower(CultureInfo.InvariantCulture).EndsWith(SelectPrimaryResult.ExecutionLog))
                    {
                        results.Add(r);
                    }
                }

                jobresults.Clear();
                jobresults = null;
                GC.Collect();
            }
            catch (ThreadAbortException)
            {
                results = null;
            }
            //DialogResult = DialogResult.OK;
        }

        /// <summary>
        /// Handles the Tick event of the downloadTimer control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void downloadTimer_Tick(object sender, EventArgs e)
        {

            if (!downloadThread.IsAlive)
            {
                // download is done;  close form
                DialogResult = System.Windows.Forms.DialogResult.OK;
            }
        }

        /// <summary>
        /// Handles the Click event of the Cancel control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Cancel_Click(object sender, EventArgs e)
        {
            if (downloadThread != null && downloadThread.IsAlive)
            {
                downloadThread.Abort();
            }
        }

    }
}