using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace Microsoft.eResearch.GenePattern.Package
{
    public partial class MessageForm : Form
    {
        private bool isExpanded;
        private int detailPanelHeight;

        /// <summary>
        /// Shows the unhandled exception message.
        /// </summary>
        /// <param name="ex">The exception.</param>
        public static void ShowUnhandledExceptionMessage(Exception ex)
        {
            ShowUnhandledExceptionMessage(ex, true);
        }

        /// <summary>
        /// Shows the unhandled exception message.
        /// </summary>
        /// <param name="ex">The exception.</param>
        /// <param name="messageOnly">if set to <c>true</c> show only the message.</param>
        public static void ShowUnhandledExceptionMessage(Exception ex, bool messageOnly)
        {
            using (MessageForm form = new MessageForm(ex, messageOnly))
            {
                form.ShowDialog();
            }
        }

        /// <summary>
        /// Shows the unhandled exception message.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="details">The details.</param>
        public static void ShowUnhandledExceptionMessage(string message, string details)
        {
            using (MessageForm form = new MessageForm(message,details))
            {
                form.ShowDialog();
            }
        }

        private MessageForm(string message, string details)
        {
            InitializeForm();
            txtMessage.Text = message;
            DetailsButton.Visible = true;
            DetailTextBox.Text = details;
        }

        private MessageForm(Exception ex, bool showOnlyMessage)
        {
            InitializeForm();

            txtMessage.Text = ex.Message;

            if (showOnlyMessage)
            {
                DetailsButton.Visible = false;
            }
            else
            {
                StringWriter writer = new StringWriter(CultureInfo.InvariantCulture);
                WriteException(ex, writer);
                DetailTextBox.Text = ex.StackTrace;
            }

            WriteExceptionToTrace(ex);
        }

        private void InitializeForm()
        {
            InitializeComponent();

            this.BackgroundImage = Resources.Clean;
            this.BackgroundImageLayout = ImageLayout.Stretch;

            CollapseDetails();
        }

        private void CollapseDetails()
        {
            detailPanelHeight = DetailsPanel.Height;
            int newHeight = this.Height - this.detailPanelHeight;
            this.Height = newHeight > 0 ? newHeight : 0;
        }

        private void ExpandDetails()
        {
            DetailsPanel.Height = detailPanelHeight;
            this.Height = this.Height + this.detailPanelHeight;
        }

        private void WriteExceptionToTrace(Exception ex)
        {
            if (ex == null)
                return;

            WriteExceptionToTrace(ex.InnerException);
            Trace.WriteLine(ex.ToString(), TraceLevel.Error.ToString());

        }

        private void WriteException(Exception ex, TextWriter writer)
        {
            if (ex == null || writer == null)
                return;

            WriteException(ex.InnerException, writer);
            writer.WriteLine(ex);
        }

        private void DetailsButton_Click(object sender, EventArgs e)
        {
            if (isExpanded)
            {
                isExpanded = false;
                CollapseDetails();
                DetailsButton.Text = Resources.SHOW_DETAILS;
            }
            else
            {
                isExpanded = true;
                ExpandDetails();
                DetailsButton.Text = Resources.HIDE_DETAILS;
            }
            txtMessage.Invalidate();
        }
    }
}