using System;
using System.Collections.Generic;
using System.Diagnostics;
using Microsoft.eResearch.GenePattern.Common;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// Helper Class for Installing Pipeline on Server
    /// </summary>
    public class PipelineInstallationHelper
    {
        private IController _controller;
        internal void InstallPipelines(IGenePatternServer genePatternServer, Pipeline pipeline, bool runInOriginalLocation, 
                                        SubmitProgress submitProgress, IController controller, int stopPoint)
        {
            this._controller = controller;
            PipelineInstaller pipelineInstaller = SetUpPipelineInstaller(pipeline, runInOriginalLocation);
            Debug.Assert(pipelineInstaller!=null, "Installer cannot be null");
            if (pipelineInstaller != null)
            {
               pipelineInstaller.InstallPipelinesIfRequired(pipeline, genePatternServer, submitProgress, false, stopPoint);
            }
            else
            {
                Trace.WriteLine("Pipeline Installer is Null. Wont be able to install the pipelines", TraceLevel.Warning.ToString());
            }
            
        }

        private PipelineInstaller SetUpPipelineInstaller(Pipeline pipeline, bool runInOriginalLocation)
        {
            PipelineInstaller pipelineInstaller = null;
            Connection connection = null;
            string message = null;

            if (pipeline.IsLinked)
            {
                pipelineInstaller = GetLinkedPipelineInstaller(connection, message, pipeline, pipelineInstaller, runInOriginalLocation);
            }
            else //Not linked..So get the info from document
            {
                pipelineInstaller = new FromDocumentInstaller(this._controller);
            }
            return pipelineInstaller;
        }

        private PipelineInstaller GetLinkedPipelineInstaller(Connection connection, string message, Pipeline pipeline, PipelineInstaller pipelineInstaller, bool runInOriginalLocation)
        {
            //Means that we have to check if pipelines are installed at original server and run it
            if (runInOriginalLocation)
            {
                pipelineInstaller = new RemoteVerifyer(this._controller);
            }
            else // Run in specifed location
            {
                bool hasPipelines = true;
                //If we already have an connection to the linked server use it 
                connection = CheckExistingConnectionForMatch(pipeline);
                IGenePatternServer server;
                if (connection == null) //no matching connections.. set connection to linked Connection
                {
                    connection = pipeline.LinkConnection;
                    server = _controller.ServerBroker.GetTestedVerifiedServer(connection, string.Empty, true);
                }
                else
                {
                    server =  _controller.ServerBroker.GetGenePatternServerForConnection(connection);
                }

                hasPipelines = HasPipelinesInServer(pipeline, server);
                //Check to see if connection has pipelines installed 
                if (hasPipelines)
                {
                     pipelineInstaller = new FromServerInstaller(this._controller, server);
               
                }
                else // Does not have pipelines so have to prompt user to enter user name for downloading it from Linked server
                {
                    pipelineInstaller = GetPipelineInstaller(connection, message, pipelineInstaller);
                }
                
            }
            
            return pipelineInstaller;
        }

        private PipelineInstaller GetPipelineInstaller(Connection connection, string message, PipelineInstaller pipelineInstaller)
        {
            IGenePatternServer sourceServer = this._controller.ServerBroker.GetTestedVerifiedServer(connection, message);
            if (sourceServer != null)
            {
                pipelineInstaller = new FromServerInstaller(this._controller, sourceServer);
            }
            return pipelineInstaller;
        }

        private static bool HasPipelinesInServer(Pipeline pipeline, IGenePatternServer server)
        {
            bool hasPipelines = true;
            if (!server.IsInstalled(pipeline.Lsid))
            {
                hasPipelines = false;
                return hasPipelines;
            }

            foreach (PipelineStep pipelineStep in pipeline.Steps)
            {
                if (!server.IsInstalled(pipelineStep.Lsid))
                {
                    hasPipelines = false;
                    break;
                }
            }
            return hasPipelines;
        }

        private Connection CheckExistingConnectionForMatch(Pipeline pipeline)
        {
            Connection matchingConnection = null;
            //See if we have a connection for the same linked server and use it 
            //or Download pipelines from linked server to current connection
            foreach (KeyValuePair<string, Connection> connectionMapping in this._controller.UserData.Connections)
            {
                Connection  connection = connectionMapping.Value;
                if (connection.ServerName.Equals(pipeline.LinkConnection.ServerName) &&
                        (connection.Port.Equals(pipeline.LinkConnection.Port)))
                {
                    matchingConnection = connection;
                    break;
                }
            }
            return matchingConnection;
        }
    }
}
