using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;
using Microsoft.eResearch.GenePattern.Common;

namespace Microsoft.eResearch.GenePattern.Package
{
    internal abstract class PipelineInstaller
    {
        private IController controller;

        public PipelineInstaller(IController controller)
        {
            this.controller = controller;
        }

        protected abstract void InstallPipeline(IGenePatternServer destinationServer, string lsid);

        protected abstract void InstallModule(IGenePatternServer destinationServer, string lsid);

        /// <summary>
        /// Installs the pipelines if required.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="genePatternServer">The gene pattern server.</param>
        /// <param name="submitProgress">The submit progress.</param>
        /// <param name="dontCallbackForSteps">if set to <c>true</c> [dont callback for steps].</param>
        /// <param name="stopPoint">The stop point.</param>
        public void InstallPipelinesIfRequired(Pipeline pipeline, IGenePatternServer genePatternServer, 
                                                SubmitProgress submitProgress, bool dontCallbackForSteps, int stopPoint)
        {
            NotifySubmitProgressListeners(submitProgress, pipeline.Name, true);
            if (!genePatternServer.IsInstalled(pipeline.Lsid))
            {
                Trace.WriteLine("Installing Pipeline with ID : " + pipeline.Lsid, TraceLevel.Info.ToString());
                this.InstallPipeline(genePatternServer, pipeline.Lsid);
            }
            NotifySubmitProgressListeners(submitProgress, pipeline.Name, false);

            SubmitProgress newCallback = dontCallbackForSteps ? null : submitProgress;
            int index = 0;

            foreach (PipelineStep pipelineStep in pipeline.Steps)
            {
                if (pipelineStep.IsPipeline)
                {
                    Pipeline subPipeline = controller.ActiveModel.Pipelines[pipelineStep.Lsid];
                    InstallPipelinesIfRequired(subPipeline, genePatternServer, newCallback, true, subPipeline.Steps.Count);
                }
                else
                {
                    if (index <= stopPoint)
                        NotifySubmitProgressListeners(newCallback, pipelineStep.Name, true);

                    if (!genePatternServer.IsInstalled(pipelineStep.Lsid))
                    {
                        Trace.WriteLine("Installing Pipeline Step with ID : " + pipelineStep.Lsid,
                                        TraceLevel.Info.ToString());
                        InstallModule(genePatternServer, pipelineStep.Lsid);
                    }

                    if (index <= stopPoint)
                        NotifySubmitProgressListeners(newCallback, pipelineStep.Name, false);

                }

                index++;
            }
        }

        /// <summary>
        /// Notifies the submit progress listeners.
        /// </summary>
        /// <param name="callback">The callback.</param>
        /// <param name="name">The name.</param>
        /// <param name="isStarting">if set to <c>true</c> [is starting].</param>
        private static void NotifySubmitProgressListeners(SubmitProgress callback, string name, bool isStarting)
        {
            if (callback != null) // notify start/end of download if we have listener
            {
                callback(name, isStarting);
            }
        }

    }
}
