using System;
using System.Collections.Generic;
using System.Globalization;
using System.Threading;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using System.Drawing;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// Wizard for rerunning a pipeline
    /// </summary>
    public partial class RerunPipelineWizard : Form, IWizard
    {
        private RerunPipelineWizardModel rerunPipelineWizardModel;
        private int currentStep;
        private IntroPage introPage;
        private SelectPipelineAssociation selectPipelineAssociation;
        private EditParameters editParameters;
        private EditStopPoint editStopPoint;
        private ConfigureJob configureJob;
        private SummaryPage summaryPage;
        private readonly IController controller;
        private const int MAX_STEPS = 6;
        private Bitmap[] stepImages;
        private bool abort;
        private bool workComplete;

        /// <summary>
        /// Initializes a new instance of the <see cref="RerunPipelineWizard"/> class.
        /// </summary>
        /// <param name="controller">The controller.</param>
        /// <param name="pipelineAssociations">The pipeline associations.</param>
        /// <param name="selectedPipelineAssociation">The selected pipeline association.</param>
        public RerunPipelineWizard(IController controller,
                                    List<PipelineAssociation> pipelineAssociations,
                                    PipelineAssociation selectedPipelineAssociation)
        {
            if (controller == null)
                throw new ArgumentNullException("controller");

            if (pipelineAssociations == null)
                throw new ArgumentNullException("pipelineAssociations");

            InitializeComponent();

            this.controller = controller;
            rerunPipelineWizardModel = new RerunPipelineWizardModel();

            rerunPipelineWizardModel.PipelineAssociations = pipelineAssociations;
            foreach (PipelineAssociation association in pipelineAssociations)
            {
                if (!rerunPipelineWizardModel.Pipelines.ContainsKey(association.PipelineLsid))
                {
                    Pipeline pipeline = controller.ActiveModel.Pipelines[association.PipelineLsid];
                    rerunPipelineWizardModel.Pipelines.Add(pipeline.Lsid, pipeline);
                }
            }
            rerunPipelineWizardModel.SelectedPipelineAssociation = selectedPipelineAssociation;
            foreach (KeyValuePair<string, Connection> connections in this.controller.UserData.Connections)
            {

                rerunPipelineWizardModel.Connections.Add(connections.Value);
            }
            rerunPipelineWizardModel.SelectedConnection = controller.CurrentConnection;

            rerunPipelineWizardModel.SelectedConnection = controller.CurrentConnection;
            bool showWizardIntroPage = controller.UserData.ShowRerunWizardIntroPage;

            introPage = new IntroPage();
            introPage.Header = Resources.RERUN_PIPELINE_INTRO_HEADER;
            introPage.ShowIntroPage = showWizardIntroPage;
            introPage.ShowIntroPageChanged += new EventHandler<EventArgs>(introPage_ShowIntroPageChanged);
            selectPipelineAssociation = new SelectPipelineAssociation(rerunPipelineWizardModel);
            editParameters = new EditParameters(rerunPipelineWizardModel);
            editStopPoint = new EditStopPoint(rerunPipelineWizardModel);
            configureJob = new ConfigureJob(rerunPipelineWizardModel, this.controller);
            summaryPage = new SummaryPage(rerunPipelineWizardModel);

            stepImages = new Bitmap[]
                {
                    Resources.Rerun_Pipeline_Wizard_1,
                    Resources.Rerun_Pipeline_Wizard_2,
                    Resources.Rerun_Pipeline_Wizard_3,
                    Resources.Rerun_Pipeline_Wizard_4,
                    Resources.Rerun_Pipeline_Wizard_5,
                    Resources.Rerun_Pipeline_Wizard_6
                 };
            SetImages();

            currentStep = showWizardIntroPage ? 0 : 1;
            if (selectedPipelineAssociation != null)
            {
                currentStep = 2;
            }
        }

        void introPage_ShowIntroPageChanged(object sender, EventArgs e)
        {
            controller.UserData.ShowRerunWizardIntroPage = introPage.ShowIntroPage;
            UserDataManager.Save(controller.UserData);
        }

        private void SetImages()
        {
            this.BackgroundImage = Resources.Wizard_background;
            this.BackgroundImageLayout = ImageLayout.Stretch;
            TitleImagePictureBox.Image = Resources.Rerun_Wizard_Icon;
            TitlePictureBox.Image = Resources.Rerun_Wizard_Title;

            SetCurrentStepIndicationImage(currentStep);
        }

        private void SetCurrentStepIndicationImage(int step)
        {
            CurrentStepIndicationPanel.BackgroundImageLayout = ImageLayout.Stretch;
            CurrentStepIndicationPanel.BackgroundImage = stepImages[step];
        }

        private void UpdateCall(int delta)
        {
            WizardBaseControl oldControl = GetControlForStep(currentStep);
            if (oldControl.CanHide() || delta <= 0) // can always go backwards
            {
                currentStep += delta;
                WizardBaseControl newControl = GetControlForStep(currentStep);
                bool successfullyHidden = oldControl.OnHide(delta);
                if (successfullyHidden || delta < 0)
                {
                    ShowCurrentStep();
                    newControl.OnShown();
                }
                else
                {
                    // rollback
                    newControl = oldControl;
                    currentStep -= delta;
                }


                if (newControl.CanHide())
                {
                    this.Next.Enabled = currentStep < MAX_STEPS - 1;
                }
                else
                {
                    Next.Enabled = false;
                }
                this.Previous.Enabled = currentStep > 0;
            }
        }

        private void ShowCurrentStep()
        {
            UserControl control = GetControlForStep(currentStep);
            control.Dock = DockStyle.Fill;
            StepPanel.Controls.Clear();
            StepPanel.Controls.Add(control);
            SetCurrentStepIndicationImage(currentStep);
            UpdateFinishButton();
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
        private WizardBaseControl GetControlForStep(int step)
        {
            switch (step)
            {
                case 0:
                    return introPage;
                case 1:
                    return selectPipelineAssociation;
                case 2:
                    return editParameters;
                case 3:
                    return editStopPoint;
                case 4:
                    return configureJob;
                case 5:
                    return summaryPage;
                default:
                    throw new IndexOutOfRangeException(); // TODO:  what if step is out of bounds?

            }
        }

        private void RerunPipelineWizard_Load(object sender, EventArgs e)
        {
            int delta = 0;
            // if pipeline association already selected, 
            // verify rerun info overwrite
            if (currentStep == 2)
            {
                WizardBaseControl control = GetControlForStep(1);
                if (!control.OnHide(0))
                {
                    delta = -1;
                }
            }
            UpdateCall(delta);
        }

        internal void AbortForm(Exception ex)
        {
            MessageForm.ShowUnhandledExceptionMessage(ex);
            abort = true;
            this.DialogResult = DialogResult.Cancel;
            Close();
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void Finish_Click(object sender, EventArgs e)
        {
            try
            {
                // store data from last step
                GetControlForStep(currentStep).OnHide(0);

                SubmitJobProgress dialog = new SubmitJobProgress();
                dialog.SetWork(controller,
                                rerunPipelineWizardModel.Job,
                                rerunPipelineWizardModel.SelectedConnection.Clone() as Connection,
                                rerunPipelineWizardModel.SelectedPipelineAssociation.LocalId,
                                rerunPipelineWizardModel.DoEmailNotification,
                                rerunPipelineWizardModel.EmailAddress,
                                rerunPipelineWizardModel.RunInOriginalLocation,
                                rerunPipelineWizardModel.ShowVisualizers);
                DialogResult canceled = dialog.ShowDialog();
                MemoryManagement.FlushMemory();

                if (canceled != DialogResult.Cancel)
                {
                    workComplete = true;
                }
                else
                {
                    this.DialogResult = DialogResult.Cancel; // bubble the cancellation up
                }
            }

            catch (Exception ex)
            {
                AbortForm(ex);
            }
        }

        /// <summary>
        /// Handles the Click event of the Next control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Next_Click(object sender, EventArgs e)
        {
            PerformNextStep();
        }

        /// <summary>
        /// Performs the next step.
        /// </summary>
        public void PerformNextStep()
        {
            if (currentStep < MAX_STEPS)
            {
                UpdateCall(1);
            }
        }

        private void Previous_Click(object sender, EventArgs e)
        {
            if (currentStep > 0)
            {
                UpdateCall(-1);
            }

        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void RerunPipelineWizard_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (abort)
                return;
            if (!workComplete)   //Message box asking confirmation required while closing Rerun Dialog Box abruptly before Finish.
            {
                string message = Resources.CLOSE_RERUN_WARNING;
                string caption = Resources.CLOSE_RERUN_WARNING_CAPTION;
                DialogResult result =
                    MessageBox.Show(message, caption, MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                if (result == DialogResult.No)
                {
                    e.Cancel = true;
                }
            }
        }


        #region IWizard Members

        /// <summary>
        /// Updates the finish button.
        /// </summary>
        public void UpdateFinishButton()
        {
            bool canFinish = rerunPipelineWizardModel.Job != null;
            if (canFinish)
            {
                canFinish = (rerunPipelineWizardModel.Job.StopPoint != -1);
                canFinish = canFinish && (rerunPipelineWizardModel.SelectedConnection != null);
                canFinish = canFinish && (!string.IsNullOrEmpty(rerunPipelineWizardModel.Job.PipelineLsid));
                canFinish = canFinish && rerunPipelineWizardModel.AreParametersSet;
            }
            Finish.Enabled = canFinish;
        }

        /// <summary>
        /// Enables the next button.
        /// </summary>
        /// <param name="enabled">if set to <c>true</c> [enabled].</param>
        public void EnableNextButton(bool enabled)
        {
            if (currentStep < MAX_STEPS - 1)
            {
                Next.Enabled = enabled;
            }
        }

        #endregion
    }
}
