using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using Microsoft.eResearch.GenePattern.Common;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// Backing model for the rerun pipeline wizard
    /// </summary>
    public class RerunPipelineWizardModel : DomainBase, IWizardModel
    {
        private List<PipelineAssociation> pipelineAssociations = new List<PipelineAssociation>();
        private Job job;
        private Dictionary<string, Pipeline> pipelines = new Dictionary<string, Pipeline>();
        private PipelineAssociation selectedPipelineAssociation;
        private bool doEmailNotification;
        private string emailAddress;
        private List<Connection> connections = new List<Connection>();
        private Connection selectedConnection;
        private bool runInOriginalLocation;
        private bool showVisualizers = true;

        /// <summary>
        /// Gets the job.
        /// </summary>
        /// <value>The job.</value>
        public Job Job
        {
            get { return job; }
        }

        /// <summary>
        /// Adds the empty data.
        /// </summary>
        public void AddEmptyData()
        {
        }

        /// <summary>
        /// Gets or sets the pipeline associations.
        /// </summary>
        /// <value>The pipeline associations.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<PipelineAssociation> PipelineAssociations
        {
            get { return pipelineAssociations; }
            set { pipelineAssociations = value; }
        }

        /// <summary>
        /// Gets the pipelines.
        /// </summary>
        /// <value>The pipelines.</value>
        public Dictionary<string, Pipeline> Pipelines
        {
            get { return pipelines; }
        }

        /// <summary>
        /// Gets or sets the selected pipeline association.
        /// </summary>
        /// <value>The selected pipeline association.</value>
        public PipelineAssociation SelectedPipelineAssociation
        {
            get { return selectedPipelineAssociation; }
            set
            {
                selectedPipelineAssociation = value;
                UpdateJob();
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether to do email notification when job is finished
        /// </summary>
        /// <value><c>true</c> if email notification is requested; otherwise, <c>false</c>.</value>
        public bool DoEmailNotification
        {
            get { return doEmailNotification; }
            set { doEmailNotification = value; }
        }

        /// <summary>
        /// Gets or sets the email address.
        /// </summary>
        /// <value>The email address.</value>
        public string EmailAddress
        {
            get { return emailAddress; }
            set { emailAddress = value; }
        }

        /// <summary>
        /// Updates the job.
        /// </summary>
        private void UpdateJob()
        {
            if (selectedPipelineAssociation == null)
                job = null;
            else
            {
                job = selectedPipelineAssociation.Job.Clone() as Job;
                job.StopPoint = selectedPipelineAssociation.UserStopPoint;
                runInOriginalLocation = SelectedPipeline.IsLinked;
            }
        }

        /// <summary>
        /// Gets a value indicating whether all parameters are set.
        /// </summary>
        /// <value><c>true</c> if all parameters are set; otherwise, <c>false</c>.</value>
        public bool AreParametersSet
        {
            get
            {
                bool result = true;
                if (job != null)
                {
                    foreach (JobParameter parameter in job.Parameters)
                    {
                        if (!parameter.Optional)
                        {
                            if (parameter.ParamValue == null ||
                                string.IsNullOrEmpty(parameter.ParamValue.ToString()))
                            {
                                result = false;
                                break;
                            }
                        }
                    }
                }
                else
                {
                    result = false;
                }
                return result;
            }
        }

        /// <summary>
        /// Gets the connections.
        /// </summary>
        /// <value>The connections.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<Connection> Connections
        {
            get { return connections; }
        }

        /// <summary>
        /// Gets or sets the selected connection.
        /// </summary>
        /// <value>The selected connection.</value>
        public Connection SelectedConnection
        {
            get { return selectedConnection; }
            set { selectedConnection = value; }
        }

        /// <summary>
        /// Gets the selected pipeline.
        /// </summary>
        /// <value>The selected pipeline.</value>
        public Pipeline SelectedPipeline
        {
            get
            {
                if (job == null || string.IsNullOrEmpty(job.PipelineLsid))
                    return null;

                Pipeline pipeline = null;
                pipelines.TryGetValue(job.PipelineLsid, out pipeline);
                return pipeline;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [run in original location].
        /// </summary>
        /// <value>
        /// 	<c>true</c> if [run in original location]; otherwise, <c>false</c>.
        /// </value>
        public bool RunInOriginalLocation
        {
            get { return runInOriginalLocation; }
            set { runInOriginalLocation = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [show visualizers].
        /// </summary>
        /// <value><c>true</c> if [show visualizers]; otherwise, <c>false</c>.</value>
        public bool ShowVisualizers
        {
            get { return showVisualizers; }
            set { showVisualizers = value; }
        }

        /// <summary>
        /// Resets the parameters.
        /// </summary>
        internal void ResetParameters()
        {
            if (selectedPipelineAssociation != null)
            {
                if (job == null)
                {
                    UpdateJob();
                }
                else
                {
                    job.Parameters.Clear();
                    foreach (JobParameter jobParameter in selectedPipelineAssociation.Job.Parameters)
                    {
                        job.Parameters.Add(jobParameter.Clone() as JobParameter);
                    }
                }
            }
        }

        /// <summary>
        /// Resets the stop point.
        /// </summary>
        internal void ResetStopPoint()
        {
            if (selectedPipelineAssociation != null)
            {
                if (job == null)
                {
                    UpdateJob();
                }
                else
                {
                    job.StopPoint = selectedPipelineAssociation.UserStopPoint;
                }
            }
        }
    }
}
