using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;

namespace Microsoft.eResearch.GenePattern.Package
{
    public partial class SelectDocumentationForm : Form
    {
        private List<BaseFile> documentationFiles = new List<BaseFile>();
        private string basePath;
        private BaseFile selectedFile;
        private bool openWithDefaultApplication = true;

        /// <summary>
        /// Initializes a new instance of the <see cref="SelectDocumentationForm"/> class.
        /// </summary>
        public SelectDocumentationForm()
        {
            InitializeComponent();
            this.BackgroundImage = Resources.Clean;
            this.BackgroundImageLayout = ImageLayout.Stretch;
            DialogResult = System.Windows.Forms.DialogResult.Cancel;
            SetEnabledButtons(false);
        }

        /// <summary>
        /// Gets or sets the documentation files.
        /// </summary>
        /// <value>The documentation files.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1819:PropertiesShouldNotReturnArrays"), Browsable(false)]
        public BaseFile[] DocumentationFiles
        {
            get
            {
                return documentationFiles.ToArray();
            }
            set
            {
                documentationFiles = new List<BaseFile>();
                if (value != null)
                {
                    documentationFiles.AddRange(value);
                }
                UpdateCall();
            }
        }

        /// <summary>
        /// Gets or sets the base path for temp storage of documentaion files
        /// </summary>
        /// <value>The base path.</value>
        [Browsable(false)]
        public string BasePath
        {
            get { return basePath; }
            set { basePath = value; }
        }

        /// <summary>
        /// Gets the selected file.
        /// </summary>
        /// <value>The selected file.</value>
        public BaseFile SelectedFile
        {
            get { return selectedFile; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the documentation file shall be 
        /// opened with default application or saved.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if it it shall be opened with default application; to save <c>false</c>.
        /// </value>
        public bool OpenWithDefaultApplication
        {
            get { return openWithDefaultApplication; }
            set { openWithDefaultApplication = value; }
        }

        private void UpdateCall()
        {
            if (DesignMode || this.Enabled == false)
            {
                return;
            }

            listView1.Items.Clear();

            if (documentationFiles != null && documentationFiles.Count > 0)
            {
                foreach (BaseFile baseFile in documentationFiles)
                {
                    ListViewItem item = listView1.Items.Add(baseFile.Filename);

                    string path = string.Empty;
                    if (!basePath.EndsWith(Path.DirectorySeparatorChar.ToString()))
                    {
                        path = basePath + Path.DirectorySeparatorChar + baseFile.Filename;
                    }
                    else
                    {
                        path = basePath + baseFile.Filename;
                    }
                    FileInfo info = new FileInfo(path);
                    string extenstion = info.Extension;
                    if (!imageList1.Images.ContainsKey(extenstion))
                    {
                        Bitmap bitmap = DefaultApplicationIconHelper.GetSmallIcon(extenstion);
                        if (bitmap != null)
                        {
                            imageList1.Images.Add(extenstion, bitmap);
                        }
                    }
                    item.ImageKey = extenstion;
                    item.Tag = baseFile;
                }
                listView1.Items[0].Selected = true;
                listView1.Select();
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void listView1_DoubleClick(object sender, EventArgs e)
        {
            try
            {
                ExecuteCommand(true);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        private void ExecuteCommand(bool open)
        {
            if(listView1.SelectedItems.Count > 0)
            {
                ListViewItem item = listView1.SelectedItems[0];
                selectedFile = item.Tag as BaseFile;
                this.DialogResult = DialogResult.OK;

                openWithDefaultApplication = open;

                Close();
            }
        }
  
        private void listView1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if(listView1.SelectedItems.Count > 0)
            {
                SetEnabledButtons(true);
            }
            else
            {
                SetEnabledButtons(false);
            }
        }

        private void SetEnabledButtons(bool enabled)
        {
            OpenButton.Enabled = enabled;
            SaveButton.Enabled = enabled;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void OpenButton_Click(object sender, EventArgs e)
        {
            try
            {
                ExecuteCommand(true);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void SaveButton_Click(object sender, EventArgs e)
        {
            try
            {
                ExecuteCommand(false);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }      
    }
}