using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.GenePattern.Server;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// 
    /// </summary>
    internal sealed class ServerBroker : IServerBroker
    {
        private UserData userData;

        /// <summary>
        /// Initializes a new instance of the <see cref="ServerBroker"/> class.
        /// </summary>
        /// <param name="userData">The user data.</param>
        public ServerBroker(UserData userData)
        {
            this.userData = userData;
        }

        /// <summary>
        /// Gets the tested verified connection.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <returns></returns>
        public Connection GetTestedVerifiedConnection(Connection connection)
        {
            return GetTestedVerifiedConnection(connection, false);
        }

        /// <summary>
        /// Gets the tested verified connection.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="editPasswordOnly">if set to <c>true</c> [edit password only].</param>
        /// <returns></returns>
        public Connection GetTestedVerifiedConnection(Connection connection, bool editPasswordOnly)
        {
            IGenePatternServer genePatternServer = GetTestedVerifiedServer(connection, String.Empty, editPasswordOnly);
            Connection serverConnection = null;
            if (genePatternServer != null)
                serverConnection = genePatternServer.Connection;

            return serverConnection;
        }

        /// <summary>
        /// Gets an IGenePatternServer with connection that has been tested
        /// and whose credentials have been enterd.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="message">The message.</param>
        /// <returns>the server or null if user cancels</returns>
        public IGenePatternServer GetTestedVerifiedServer(Connection connection, string message)
        {
            return GetTestedVerifiedServer(connection, message, false);
        }

        /// <summary>
        /// Gets the tested verified server.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="message">The message.</param>
        /// <param name="editPasswordOnly">if set to <c>true</c> [edit password only].</param>
        /// <returns></returns>
        public IGenePatternServer GetTestedVerifiedServer(Connection connection, string message, bool editPasswordOnly)
        {
            bool keepTrying = true;
            IGenePatternServer genePatternServer = GetGenePatternServerForConnection(connection);
            while (keepTrying)
            {
                string test;
                if (Microsoft.eResearch.GenePattern.Server.GenePatternServer.TestConnection(genePatternServer.Connection, false, out test))
                {
                    bool isOK = VerifyUsernameAndPassword(genePatternServer, true, true, editPasswordOnly, message);
                    keepTrying = false;
                    if (!isOK)
                        genePatternServer = null;
                }
                else
                {
                    string errorMessage = "The connection failed. Click Retry to edit the connection or click Cancel to cancel the rerun";
                    string caption = "Connection failed!";
                    DialogResult dialogResult = MessageBox.Show(errorMessage, caption,
                        MessageBoxButtons.RetryCancel, MessageBoxIcon.Warning);

                    if (dialogResult == DialogResult.Retry)
                    {
                        bool isOK = VerifyUsernameAndPassword(genePatternServer, false, false, editPasswordOnly, message);
                        if (!isOK)
                        {
                            keepTrying = false;
                            genePatternServer = null;
                        }
                    }
                    else
                    {
                        keepTrying = false;
                        genePatternServer = null;
                    }
                }
            }
            return genePatternServer;
        }

        /// <summary>
        /// Gets the gene pattern server for connection.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <returns></returns>
        public IGenePatternServer GetGenePatternServerForConnection(Connection connection)
        {
            IGenePatternServer genePatternServer = new GenePatternServer();
            genePatternServer.Connection = connection;

            if (connection == null)
                return genePatternServer;

            // if the connection doesn't have a password, try to fill one in from our userdata

            if (String.IsNullOrEmpty(connection.Password))
            {
                foreach (Connection storedConn in userData.Connections.Values)
                {
                    if (storedConn.ServerName == connection.ServerName
                        && storedConn.Port == connection.Port
                        && storedConn.Username == connection.Username)
                    {
                        // already have stored creds for this connection
                        genePatternServer.Connection = storedConn;
                        break;
                    }
                }
            }

            return genePatternServer;
        }

        /// <summary>
        /// Verifies the username and password.
        /// </summary>
        /// <param name="genePatternServer">The gene pattern server.</param>
        /// <param name="verifyConnection">if set to <c>true</c> [verify connection].</param>
        /// <param name="verifyCredentialsOnly">if set to <c>true</c> [verify credentials only].</param>
        /// <param name="editPasswordOnly">if set to <c>true</c> [edit password only].</param>
        /// <param name="message">The message.</param>
        /// <returns></returns>
        private static bool VerifyUsernameAndPassword(IGenePatternServer genePatternServer,
                                                bool verifyConnection,
                                                bool verifyCredentialsOnly,
                                                bool editPasswordOnly,
                                                string message)
        {
            bool result = true;
            bool passwordRequired = false;

            if (verifyConnection)
            {
                ServiceInfo serviceInfo = genePatternServer.GetServiceInfo;
                passwordRequired = serviceInfo.RequiresPassword;
            }

            Connection connection = genePatternServer.Connection;

            if (!verifyCredentialsOnly ||
                string.IsNullOrEmpty(connection.Username) ||
                (passwordRequired && string.IsNullOrEmpty(connection.Password)))
            {
                PasswordRequiredStatus passwordRequiredStatus = passwordRequired
                                                                    ? PasswordRequiredStatus.Required
                                                                    : PasswordRequiredStatus.Unknown;
                if (connection != null && String.IsNullOrEmpty(message))
                {
                    message = ConstructMessage(connection.Username, passwordRequired, connection.Password);
                }
                using (ServerConnection serverConnectionForm =
                    new ServerConnection(connection, passwordRequiredStatus, verifyCredentialsOnly, message))
                {
                    // disable non
                    if (editPasswordOnly)
                    {
                        serverConnectionForm.DisableServerFields();
                    }
                    if (DialogResult.OK == serverConnectionForm.ShowDialog())
                    {
                        genePatternServer.Connection = serverConnectionForm.Connection;
                    }
                    else
                    {
                        result = false;
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// Constructs the message.
        /// </summary>
        /// <param name="userName">Name of the user.</param>
        /// <param name="passwordRequired">if set to <c>true</c> [password required].</param>
        /// <param name="password">The password.</param>
        /// <returns></returns>
        private static string ConstructMessage(string userName, bool passwordRequired, string password)
        {
            String message = String.Empty;
            StringBuilder stringBuilder = new StringBuilder();
            bool constructMessage = false;
            if (String.IsNullOrEmpty(userName))
            {
                stringBuilder.AppendLine("* User Name");
                constructMessage = true;
            }
            if (passwordRequired && String.IsNullOrEmpty(password))
            {
                stringBuilder.AppendLine("* Password");
                constructMessage = true;
            }

            if (constructMessage)
            {
                string header = "The following information must be provided to rerun the pipeline in the Linked Location"
                                + Environment.NewLine;

                stringBuilder = stringBuilder.Insert(0, header);
                message = stringBuilder.ToString();
            }

            return message;
        }
    }
}
