using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Globalization;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.GenePattern.Server;
using System.Text.RegularExpressions;
using System.Configuration;
using System.Xml;
using System.IO;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// Enum Password Required Status
    /// </summary>
    public enum PasswordRequiredStatus
    {
        /// <summary>
        /// {35A90EBF-F421-44A3-BE3A-47C72AFE47FE}
        /// </summary>
        Unknown, NotRequired, Required
    }

    /// <summary>
    /// A form for editing the properties of a <c>Connection</c>
    /// </summary>
    public partial class ServerConnection : Form
    {
        #region Fields

        private Connection originalConnection;
        private Connection connection;
        private bool cancelClosing;
        private PasswordRequiredStatus passwordRequiredStatus;
        private bool editCredentialsOnly;
        private const string defaultConnectionName = "Connection ";
        private const int MAX_PORT = 65535;
        private static int ConnectionNo = 1;
        private bool _IsConnectionSuccessful;
        string portNo, connectionName, userName, serverName, serverNameWithOutHttp;
        string GenePatternConnectionName, GenePatternServerUrl;
        int GenePatternPortNo;
        #endregion

        #region Properties

        /// <summary>
        /// Gets a value indicating whether this instance is connection successful.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is connection successful; otherwise, <c>false</c>.
        /// </value>
        public bool IsConnectionSuccessful
        {
            get { return _IsConnectionSuccessful; }
        }

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ServerConnection"/> class.
        /// </summary>
        /// <param name="originalConnection">The original connection.</param>
        public ServerConnection(Connection originalConnection)
            : this(originalConnection, PasswordRequiredStatus.Unknown, false)
        {

        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ServerConnection"/> class.
        /// </summary>
        /// <param name="originalConnection">The original connection.</param>
        /// <param name="passwordRequiredStatus">The password required status.</param>
        /// <param name="credentialsOnly">if set to <c>true</c> [credentials only].</param>
        public ServerConnection(Connection originalConnection,
                                PasswordRequiredStatus passwordRequiredStatus,
                                bool credentialsOnly)
            : this(originalConnection, passwordRequiredStatus, credentialsOnly, String.Empty)
        {

        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ServerConnection"/> class.
        /// </summary>
        /// <param name="originalConnection">The original connection.</param>
        /// <param name="passwordRequiredStatus">The password required status.</param>
        /// <param name="credentialsOnly">if set to <c>true</c> [credentials only].</param>
        /// <param name="message">The message.</param>
        public ServerConnection(Connection originalConnection,
                                PasswordRequiredStatus passwordRequiredStatus,
                                bool credentialsOnly,
                                string message)
        {
            InitializeComponent();
            //Always set it to be initially false

            //if (String.IsNullOrEmpty(message))
            //{
                int hieght = this.messagePanel.Height;
                this.messagePanel.Visible = false;
                this.Height = this.Height - hieght;
            //}
            //else
            //{
            //    txtMessageLabel.Text = message;
            //}
            DialogResult = DialogResult.Cancel;
            this.BackgroundImage = Resources.Clean;
            this.BackgroundImageLayout = ImageLayout.Stretch;

            this.originalConnection = originalConnection;
            if (originalConnection == null)
            {
                userName = Environment.UserName.ToLower(CultureInfo.InvariantCulture);
                connectionName = defaultConnectionName + ConnectionNo++;
                portNo = Resources.DEFAULT_PORT;
                serverName = Resources.DEFAULT_SERVER_NAME;
                originalConnection = new Connection(connectionName, serverName, userName,
                                                    string.Empty, int.Parse(portNo));
            }

            connection = originalConnection.Clone() as Connection;

            passwordTextBox.UseSystemPasswordChar = true;
            this.passwordRequiredStatus = passwordRequiredStatus;
            if (this.passwordRequiredStatus == PasswordRequiredStatus.NotRequired)
            {
                passwordTextBox.Text = Resources.NOT_REQUIRED;
                passwordTextBox.ReadOnly = true;
                passwordTextBox.UseSystemPasswordChar = false;
            }
            this.editCredentialsOnly = credentialsOnly;
            if (editCredentialsOnly)
            {
                serverNameTextBox.ReadOnly = true;
                portTextBox.ReadOnly = true;
            }

        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ServerConnection"/> class.
        /// </summary>
        public ServerConnection()
        {
            InitializeComponent();

            GetXMLValues();
            DialogResult = DialogResult.Cancel;
            this.BackgroundImage = Resources.Clean;
            this.BackgroundImageLayout = ImageLayout.Stretch;

            int hieght = this.messagePanel.Height;
            this.messagePanel.Visible = false;
            this.Height = this.Height - hieght;

            //portTextBox.Text = "0";
            portTextBox.Enabled = false;
            //serverNameTextBox.Text = "GenePattern.Broad.MIT.edu";
            serverNameTextBox.Enabled = false;
            //connectionNameTextBox.Text = "GenePattern Public Server";
            btnPGPS.Enabled = false;
            btnReset.Enabled = false;
            originalConnection = new Connection(GenePatternConnectionName, GenePatternServerUrl,
                                                string.Empty, string.Empty, GenePatternPortNo);
            connection = originalConnection.Clone() as Connection;
        }

        /// <summary>
        /// Gets the connection.
        /// </summary>
        /// <value>The connection.</value>
        public Connection Connection
        {
            get { return connection; }
        }

        /// <summary>
        /// Sets the message.
        /// </summary>
        /// <param name="message">The message.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        public void SetMessage(string message)
        {
            txtMessageLabel.Text = message;
        }

        #endregion

        #region Event handlers

        /// <summary>
        /// Handles the Load event of the ServerConnection control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ServerConnection_Load(object sender, EventArgs e)
        {
            connectionNameTextBox.Text = Connection.Name;
            serverNameTextBox.Text = Connection.CompleteServerName;
            portTextBox.Text = Connection.Port.ToString(CultureInfo.InvariantCulture);
            usernameTextBox.Text = Connection.Username;
            passwordTextBox.Text = Connection.Password;
        }

        /// <summary>
        /// Handles the Click event of the OK control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void OK_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            cancelClosing = true;

            ValidationResult result = IsValid();
            if (!result.IsValid)
            {
                string message = result.GetMessage();
                MessageBox.Show(this, message, Resources.GENEPATTREN_MSG_HEADER, MessageBoxButtons.OK,
                                MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);

                return;
            }

            string testMessage;
            _IsConnectionSuccessful = GenePatternServer.TestConnection(connection, true, out testMessage);
            if (_IsConnectionSuccessful)
            {
                cancelClosing = false;
                Close();
                return;
            }

            MessageBox.Show(this, Resources.TEST_FAILED, Resources.GENEPATTREN_MSG_HEADER, MessageBoxButtons.OK, MessageBoxIcon.Error,
                            MessageBoxDefaultButton.Button1);

            Cursor.Current = Cursors.Default;
        }

        /// <summary>
        /// Handles the Click event of the testConnection control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void testConnection_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            ValidationResult result = IsValid();
            if (!result.IsValid)
            {
                string message = result.GetMessage();

                MessageBox.Show(this, message, Resources.GENEPATTREN_MSG_HEADER, MessageBoxButtons.OK,
                                MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button1);
                return;
            }

            string resultMessage = Resources.TEST_FAILED;
            string testMessage;
            MessageBoxIcon messageIcon = MessageBoxIcon.Error;

            if (GenePatternServer.TestConnection(connection, true, out testMessage))
            {
                resultMessage = Resources.TEST_SUCCESSFUL;
                messageIcon = MessageBoxIcon.Information;
            }

            MessageBox.Show(this, resultMessage, Resources.GENEPATTREN_MSG_HEADER, MessageBoxButtons.OK, messageIcon,
                            MessageBoxDefaultButton.Button1);

            Cursor.Current = Cursors.Default;
        }

        /// <summary>
        /// Handles the Validating event of the serverNameTextBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
        private void serverNameTextBox_Validating(object sender, CancelEventArgs e)
        {
            Validate(sender as TextBox, new IsValidDelegate(IsServerNameValid));
        }

        /// <summary>
        /// Handles the TextChanged event of the serverNameTextBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void serverNameTextBox_TextChanged(object sender, EventArgs e)
        {
            Connection.CompleteServerName = serverNameTextBox.Text;
            string serverName = serverNameTextBox.Text.ToLower();
            serverName = serverName.Replace("http://", "");
            Connection.ServerName = serverName.Replace("https://", "");
        }

        /// <summary>
        /// Handles the FormClosing event of the ServerConnection control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void ServerConnection_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (DialogResult == DialogResult.Cancel)
            {
                if (HasChanged())
                {

                    DialogResult dialogResult = MessageBox.Show(this, Resources.DISCARD, Resources.DISCARD,
                                                                MessageBoxButtons.YesNo, MessageBoxIcon.Warning,
                                                                MessageBoxDefaultButton.Button2);

                    if (dialogResult.Equals(DialogResult.No))
                        cancelClosing = true;
                    else
                    {
                        cancelClosing = false;
                    }
                }

            }

            if (cancelClosing)
            {
                e.Cancel = true;
            }

            cancelClosing = false;
        }

        /// <summary>
        /// Handles the TextChanged event of the connectionNameTextBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void connectionNameTextBox_TextChanged(object sender, EventArgs e)
        {
            Connection.Name = connectionNameTextBox.Text;
        }

        /// <summary>
        /// Handles the Validating event of the connectionNameTextBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
        private void connectionNameTextBox_Validating(object sender, CancelEventArgs e)
        {
            Validate(sender as TextBox, new IsValidDelegate(IsConnectionNameValid));
        }

        /// <summary>
        /// Handles the Validating event of the usernameTextBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
        private void usernameTextBox_Validating(object sender, CancelEventArgs e)
        {
            Validate(sender as TextBox, new IsValidDelegate(IsUsernameValid));
        }

        /// <summary>
        /// Handles the TextChanged event of the usernameTextBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void usernameTextBox_TextChanged(object sender, EventArgs e)
        {
            Connection.Username = usernameTextBox.Text;
        }

        /// <summary>
        /// Handles the Validating event of the portTextBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
        private void portTextBox_Validating(object sender, CancelEventArgs e)
        {
            Validate(sender as TextBox, new IsValidDelegate(IsPortValid));
        }

        /// <summary>
        /// Handles the TextChanged event of the portTextBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void portTextBox_TextChanged(object sender, EventArgs e)
        {
            int result;

            bool success = int.TryParse(portTextBox.Text, out result);

            if (success)
            {
                Connection.Port = result;
            }
            else
            {
                Connection.Port = 0;
            }
        }

        /// <summary>
        /// Handles the TextChanged event of the passwordTextBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void passwordTextBox_TextChanged(object sender, EventArgs e)
        {
            Connection.Password = passwordTextBox.Text;
        }

        /// <summary>
        /// Handles the Validating event of the passwordTextBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.ComponentModel.CancelEventArgs"/> instance containing the event data.</param>
        private void passwordTextBox_Validating(object sender, CancelEventArgs e)
        {
            Validate(sender as TextBox, new IsValidDelegate(IsPasswordValid));
        }

        /// <summary>
        /// Handles the Click event of the btnPGPS control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.Int32.ToString")]
        private void btnPGPS_Click(object sender, EventArgs e)
        {
            GetXMLValues();

            usernameTextBox.Text = string.Empty;
            connectionNameTextBox.Text = GenePatternConnectionName;
            serverNameTextBox.Text = GenePatternServerUrl;
            portTextBox.Text = GenePatternPortNo.ToString();

            ValidateConnectionFields();
        }

        /// <summary>
        /// Validates the connection fields.
        /// </summary>
        private void ValidateConnectionFields()
        {
            Validate(connectionNameTextBox, new IsValidDelegate(IsConnectionNameValid));
            Validate(usernameTextBox, new IsValidDelegate(IsUsernameValid));
            Validate(serverNameTextBox, new IsValidDelegate(IsServerNameValid));
        }

        /// <summary>
        /// Gets the XML values.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.Int32.Parse(System.String)"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void GetXMLValues()
        {
            try
            {
                string filePath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData),
               "Infusion Development\\Microsoft Word Add-in for the GenePattern Reproducible Research Document");
                bool isFileExists = File.Exists(Path.Combine(filePath, "GenePatternPublicServer.xml"));
                if (isFileExists)
                {
                    filePath = Path.Combine(filePath, "GenePatternPublicServer.xml");
                    XmlDocument genePattenPublicServer = new XmlDocument();
                    genePattenPublicServer.Load(filePath);
                    XmlNodeList nodelist = genePattenPublicServer.SelectNodes("//appsettings/add");
                    foreach (XmlNode xnode in nodelist)
                    {
                        if (xnode.Attributes["Name"].Value == "ConnectionName")
                        {
                            GenePatternConnectionName = xnode.Attributes["value"].Value.Clone() as string;
                        }
                        else if (xnode.Attributes["Name"].Value == "ServerName")
                        {

                            GenePatternServerUrl = xnode.Attributes["value"].Value.Clone() as string;
                        }
                        else if (xnode.Attributes["Name"].Value == "PortNo")
                        {

                            GenePatternPortNo = int.Parse(xnode.Attributes["value"].Value.Clone() as string);
                        }
                    }
                }
                else
                {
                    GenePatternConnectionName = "GenePattern Public Server";
                    GenePatternServerUrl = "genepattern.broadinstitute.org";
                    GenePatternPortNo = 0;
                }
            }
            catch (Exception)
            {
                GenePatternConnectionName = "GenePattern Public Server";
                GenePatternServerUrl = "genepattern.broadinstitute.org";
                GenePatternPortNo = 0;
                portTextBox.Enabled = true;
                serverNameTextBox.Enabled = true;
            }
        }

        /// <summary>
        /// Handles the Click event of the btnReset control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnReset_Click(object sender, EventArgs e)
        {
            portTextBox.Text = Resources.DEFAULT_PORT;
            serverNameTextBox.Text = Resources.DEFAULT_SERVER_NAME;
            connectionNameTextBox.Text = connectionName;
            usernameTextBox.Text = Environment.UserName.ToLower(CultureInfo.InvariantCulture);
            ValidateConnectionFields();
        }

        #endregion

        #region Private methods

        /// <summary>
        /// Validates the specified text box.
        /// </summary>
        /// <param name="textBox">The text box.</param>
        /// <param name="isValidCall">The is valid call.</param>
        private void Validate(TextBox textBox, IsValidDelegate isValidCall)
        {
            ValidationResult result = isValidCall.Invoke();
            if (!result.IsValid)
            {
                errorProvider.SetError(textBox, result.GetMessage());
            }
            else
            {
                errorProvider.SetError(textBox, string.Empty);
            }
        }

        /// <summary>
        /// Determines whether this instance is valid.
        /// </summary>
        /// <returns></returns>
        private ValidationResult IsValid()
        {
            ValidationResult mainResult = IsServerNameValid();
            mainResult.Merge(IsConnectionNameValid());
            mainResult.Merge(IsUsernameValid());
            mainResult.Merge(IsPasswordValid());
            mainResult.Merge(IsPortValid());

            return mainResult;
        }

        // Function to Check for AlphaNumeric.
        /// <summary>
        /// Connections the name validation.
        /// </summary>
        /// <param name="strToCheck">The STR to check.</param>
        /// <returns></returns>
        private static bool ConnectionNameValidation(string strToCheck)
        {
            if (strToCheck.Length > 256)
                return false;

            if (!char.IsLetter(strToCheck[0]))
                return false;

            strToCheck = strToCheck.Replace(' ', 't');
            Regex regex = new Regex(@"\W");

            return !regex.IsMatch(strToCheck);
        }

        /// <summary>
        /// Determines whether [is connection name valid].
        /// </summary>
        /// <returns></returns>
        private ValidationResult IsConnectionNameValid()
        {
            // Validation for blank Connection Name.
            ValidationResult result = new ValidationResult(!string.IsNullOrEmpty(connectionNameTextBox.Text));
            if (!result.IsValid)
            {
                result.Messages.Add(Resources.CONNECTION_NAME_IS_REQUIRED);
                return result;
            }

            // Validation for Connection Name for AlphaNumeric Characters.
            result = new ValidationResult(ConnectionNameValidation(connectionNameTextBox.Text));
            if (!result.IsValid)
            {
                result.Messages.Add(Resources.CONNECTION_NAME_INVALID);
                return result;
            }

            return result;
        }

        /// <summary>
        /// Determines whether [is server name valid].
        /// </summary>
        /// <returns></returns>
        private ValidationResult IsServerNameValid()
        {
            ValidationResult result = new ValidationResult(!string.IsNullOrEmpty(serverNameTextBox.Text));
            if (!result.IsValid)
            {
                result.Messages.Add(Resources.SERVER_NAME_IS_REQUIRED);
            }
            return result;
        }

        /// <summary>
        /// Determines whether [is username valid].
        /// </summary>
        /// <returns></returns>
        private ValidationResult IsUsernameValid()
        {
            ValidationResult result = new ValidationResult(!string.IsNullOrEmpty(usernameTextBox.Text));
            if (!result.IsValid)
            {
                result.Messages.Add(Resources.USERNAME_IS_REQUIRED);
            }
            return result;
        }

        /// <summary>
        /// Determines whether [is password valid].
        /// </summary>
        /// <returns></returns>
        private ValidationResult IsPasswordValid()
        {
            ValidationResult result = null;
            if (passwordRequiredStatus == PasswordRequiredStatus.Required)
            {
                result = new ValidationResult(!string.IsNullOrEmpty(passwordTextBox.Text));
                if (!result.IsValid)
                {
                    result.Messages.Add(Resources.PASSWORD_IS_REQUIRED);
                }
            }
            else
                result = new ValidationResult(true);

            return result;
        }

        /// <summary>
        /// Determines whether [is port valid].
        /// </summary>
        /// <returns></returns>
        private ValidationResult IsPortValid()
        {
            bool isValid = true;

            if (!string.IsNullOrEmpty(portTextBox.Text))
            {
                int vessel;
                isValid = int.TryParse(portTextBox.Text, out vessel);
                isValid = isValid && (vessel >= 0 && vessel <= MAX_PORT);
            }

            ValidationResult result = new ValidationResult(isValid);
            if (!result.IsValid)
            {
                result.Messages.Add(Resources.PORT_MUST_BE_INTEGER);
            }
            return result;
        }

        /// <summary>
        /// Delegate Validation Result
        /// </summary>
        private delegate ValidationResult IsValidDelegate();

        /// <summary>
        /// Determines whether this instance has changed.
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if this instance has changed; otherwise, <c>false</c>.
        /// </returns>
        private bool HasChanged()
        {
            return !Connection.Equals(originalConnection);
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Disables the server fields.
        /// </summary>
        public void DisableServerFields()
        {
            this.connectionNameTextBox.Enabled = false;
            this.serverNameTextBox.Enabled = false;
            this.portTextBox.Enabled = false;
        }

        #endregion

    }
}