using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Globalization;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using Timer = System.Timers.Timer;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// Class Submit JobProgress Dialog
    /// </summary>
    public partial class SubmitJobProgress : Form
    {
        private IController controller;
        private Job job;
        private int pipelineAssociationId;
        private Connection connection;
        private int jobId = -1;
        private bool registerNotification;
        private bool runInOriginalLocation;
        private bool showVisualizers;
        private string emailAddress;
        private Thread asyncJob;
        private string progressMessage = "";
        private int totalTasks;
        private int tasksComplete;
        private bool done;
        private bool stateIsSwitched;

        /// <summary>
        /// Initializes a new instance of the <see cref="SubmitJobProgress"/> class.
        /// </summary>
        public SubmitJobProgress()
        {
            InitializeComponent();
            this.BackgroundImage = Resources.Clean;
            this.BackgroundImageLayout = ImageLayout.Stretch;
        }

        #region Events

        private void CloseButton_Click(object sender, EventArgs e)
        {
            if (!done)
            {
                AbortJob();
            }
            else
            {
                this.DialogResult = DialogResult.OK;
                this.Close();
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void SubmitJobProgress_Shown(object sender, EventArgs e)
        {
            try
            {
                asyncJob = new Thread(new ThreadStart(SubmitJob));
                asyncJob.Name = "Submit Job";
                //asyncJob.SetApartmentState(ApartmentState.STA);

                Pipeline pipeline = controller.ActiveModel.Pipelines[job.PipelineLsid];
                totalTasks += CountPipelinePieces(pipeline);

                progressTimer.Start();
                asyncJob.Start();
                downloadAnimationUserControl1.Start();
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
                Close();
            }
        }

        private void progressTimer_Tick(object sender, EventArgs e)
        {
            Progress.Minimum = 0;
            Progress.Maximum = totalTasks;

            if (tasksComplete > Progress.Maximum)
            {
                Progress.Maximum = tasksComplete;
            }
            Progress.Value = tasksComplete;
            if (ProgressTextBox.Text != progressMessage)
            {
                ProgressTextBox.AppendText(progressMessage.Substring(ProgressTextBox.Text.Length));
            }

            if (done && !stateIsSwitched)
            {
                SwitchToDoneState();
            }

            if (!asyncJob.IsAlive)
            {
                // download is done but time not done -- show progress finished
                Progress.Value = Progress.Maximum;
            }

            if (jobId > -1)
            {
                IdLabel.Text = jobId.ToString(CultureInfo.InvariantCulture);
            }
        }

        private void SubmitJobProgress_FormClosing(object sender, FormClosingEventArgs e)
        {
            downloadAnimationUserControl1.Stop();
            progressTimer.Stop();
            AbortJob();
        }

        #endregion

        /// <summary>
        /// Sets the work.
        /// </summary>
        /// <param name="controllerInstance">The controller instance.</param>
        /// <param name="jobInstance">The job instance.</param>
        /// <param name="connectionInstance">The connection instance.</param>
        /// <param name="associationId">The association id.</param>
        /// <param name="doRegisterNotification">if set to <c>true</c> register job completion notification.</param>
        /// <param name="theEmailAddress">The email address.</param>
        /// <param name="originalLocation">if set to <c>true</c> run the pipeline at its original location.</param>
        /// <param name="doShowVisualizers">if set to <c>true</c> [do show visualizers].</param>
        public void SetWork(IController controllerInstance, Job jobInstance,
                            Connection connectionInstance, int associationId,
                            bool doRegisterNotification, string theEmailAddress,
                            bool originalLocation, bool doShowVisualizers)
        {
            this.controller = controllerInstance;
            this.job = jobInstance;
            this.connection = connectionInstance;
            this.pipelineAssociationId = associationId;
            this.registerNotification = doRegisterNotification;
            this.emailAddress = theEmailAddress;
            this.runInOriginalLocation = originalLocation;
            this.showVisualizers = doShowVisualizers;
        }

        /// <summary>
        /// Submits the job.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void SubmitJob()
        {
            try
            {
                controller.BeginModelChanges();

                Connection verifiedConnection = controller.ServerBroker.GetTestedVerifiedConnection(connection, false);
                if (verifiedConnection != null)
                {
                    jobId = controller.SubmitJob(job, verifiedConnection, registerNotification,
                                                 emailAddress, runInOriginalLocation,
                                                 new SubmitProgress(UpdateProgress));

                    if (jobId != -1)
                    {
                        PipelineAssociation pipelineAssociation =
                            controller.ActiveModel.PipelineAssociations[pipelineAssociationId];
                        pipelineAssociation.Rerun = new Rerun(verifiedConnection);
                        pipelineAssociation.Rerun.Job.Id = jobId;
                        //pipelineAssociation.Rerun.SetConnection(verifiedConnection);
                        pipelineAssociation.Rerun.ShowVisualizers = showVisualizers;
                        pipelineAssociation.OriginalRerunParameters = job.Parameters;
                        controller.CommitModelChanges();

                        controller.SignalRerunIsPresent(pipelineAssociation);
                    }
                    else
                    {
                        controller.RollbackModelChanges();
                    }
                }
                else
                {
                    controller.RollbackModelChanges();
                }
                done = true;
            }
            catch (ThreadAbortException)
            {
                // user clicked cancel
                controller.RollbackModelChanges();
            }
            catch (PipelineOrModuleNotInstalledException ex)
            {
                controller.RollbackModelChanges();
                string message = "One or more modules required for rerunning the pipeline are "
                                 + Environment.NewLine
                                 + "missing in the referenced Server." + Environment.NewLine + Environment.NewLine
                                 + "Please see details below for more information."
                                 + Environment.NewLine + Environment.NewLine
                                 + "The rerun will be aborted.";
                MessageForm.ShowUnhandledExceptionMessage(message, ex.Message);
                this.AbortJob();
            }
            catch (Exception ex)
            {
                controller.RollbackModelChanges();
                MessageForm.ShowUnhandledExceptionMessage(ex);
                this.AbortJob();
            }
        }

        /// <summary>
        /// Aborts the job.
        /// </summary>
        private void AbortJob()
        {
            try
            {
                if (asyncJob.IsAlive)
                {
                    asyncJob.Abort();
                }
            }
            // The Threadabort exception is a special case. Even if we have catched Thread Abort exception, it is still rethrown by the CLR!
            // So we need to close the Progress form in the Finally block.
            catch (ThreadAbortException exception)
            {
                Trace.WriteLine("Submit Job Thread aborted successfully" + exception.Message, TraceLevel.Info.ToString());
            }
            finally
            {


                if (this.InvokeRequired)
                    this.BeginInvoke(new EventHandler(this.CloseForm));

                else
                    this.CloseForm(this, EventArgs.Empty);
                //this.DialogResult = DialogResult.OK;

            }

        }

        /// <summary>
        /// Closes the form.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="eventArgs">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void CloseForm(object sender, EventArgs eventArgs)
        {
            downloadAnimationUserControl1.Stop();
            progressTimer.Stop();
            this.Close();
        }

        /// <summary>
        /// Counts the pipeline pieces.
        /// </summary>
        /// <param name="p">The p.</param>
        /// <returns></returns>
        private static int CountPipelinePieces(Pipeline p)
        {
            int count = 1;
            count += p.Steps.Count;

            return count;
        }

        /// <summary>
        /// Updates the progress.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="isStarting">if set to <c>true</c> [is starting].</param>
        private void UpdateProgress(string name, bool isStarting)
        {
            if (isStarting)
            {
                string startMessage = Resources.START_PROCESSING;
                progressMessage += String.Format(CultureInfo.InvariantCulture, startMessage, name) + Environment.NewLine;
            }
            else
            {
                string finishedMessage = Resources.FINISH_PROCESSING;
                progressMessage += String.Format(CultureInfo.InvariantCulture, finishedMessage, name) + Environment.NewLine;
                tasksComplete++;
            }
        }

        /// <summary>
        /// Switches the state of to done.
        /// </summary>
        private void SwitchToDoneState()
        {
            CloseButton.Text = Resources.CLOSE;
            toolTip1.SetToolTip(CloseButton, "Close rerun dialog");

            stateIsSwitched = true;
            progressMessage += Resources.DONE + Environment.NewLine;
            downloadAnimationUserControl1.Stop();
            progressTimer.Enabled = false;
        }
    }
}
