using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Security;
using System.Threading;
using System.Windows.Forms;
using Infusion.GenePattern.Common;
using Infusion.ScientificAddIn.Common;
using Infusion.GenePattern.Server;
using System.Text;

namespace Infusion.GenePattern.Package
{
    public partial class AssociatePipelineTaskPane : UserControl
    {
        #region Variable Declaration

        private IController _controller;
        private SelectJobsUserControl _selectJobsUserControl;
        private StopPointUserControl _stopPointUserControl;
        private PrimaryResultUserControl _primaryResultUserControl;
        private AnnotationUserControl _annotationUserControl;
        private ShowParametersUserControl _showParametersUserControl;
        private ViewPipelinesUserControl _viewPipelinesUserControl;
        private string _documentKey;

        #endregion

        #region Associate Variable Declaration

        private int _totalDownloads;
        private int _downloadsComplete;
        private bool _isEditing;
        private string _progressMessage = string.Empty;
        private Thread _asyncDownload;
        private bool _isResultOk;
        private List<Pipeline> _pipelineList = new List<Pipeline>();
        private List<PipelineAssociation> _associationList = new List<PipelineAssociation>();
        private Job _selectedJob;
        private Pipeline _selectedPipeline;

        #endregion

        #region Public Methods

        /// <summary>
        /// Initializes a new instance of the <see cref="AssociatePipelineTaskPane"/> class.
        /// </summary>
        public AssociatePipelineTaskPane()
        {
            InitializeComponent();
            AddUserControls();
            lblStatus.Text = string.Empty;
            _selectJobsUserControl.JobsSelectedIndexChanged += new EventHandler<EventArgs>(OnJobsSelectedIndexChanged);
            _viewPipelinesUserControl.AssociationDeleteButtonClick += new EventHandler<EventArgs>(AssociationDeleteButtonClick);
            _annotationUserControl.AnnotationTextChanged += new EventHandler<EventArgs>(AnnotationTextChanged);
        }

        /// <summary>
        /// Sets the controller.
        /// </summary>
        /// <param name="controller">The controller.</param>
        public void SetController(IController controller)
        {
            this._controller = controller;
            this._documentKey = _controller.DocumentKey;
            UpdateServerJobs();
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Adds the user controls.
        /// </summary>
        private void AddUserControls()
        {
            _selectJobsUserControl = new SelectJobsUserControl();
            _selectJobsUserControl.Dock = DockStyle.Fill;
            splitContainer1.Panel1.Controls.Add(_selectJobsUserControl);

            _stopPointUserControl = new StopPointUserControl();
            _stopPointUserControl.Dock = DockStyle.Fill;
            splitContainer4.Panel1.Controls.Add(_stopPointUserControl);

            _primaryResultUserControl = new PrimaryResultUserControl();
            _primaryResultUserControl.Dock = DockStyle.Fill;
            splitContainer4.Panel2.Controls.Add(_primaryResultUserControl);

            _annotationUserControl = new AnnotationUserControl();
            _annotationUserControl.Dock = DockStyle.Fill;
            splitContainer2.Panel2.Controls.Add(_annotationUserControl);

            _showParametersUserControl = new ShowParametersUserControl();
            _showParametersUserControl.Dock = DockStyle.Fill;
            splitContainer6.Panel1.Controls.Add(_showParametersUserControl);

            _viewPipelinesUserControl = new ViewPipelinesUserControl();
            _viewPipelinesUserControl.Dock = DockStyle.Fill;
            splitContainer6.Panel2.Controls.Add(_viewPipelinesUserControl);
        }

        /// <summary>
        /// Updates the server jobs.
        /// </summary>
        private void UpdateServerJobs()
        {
            _selectJobsUserControl.InitialUpdate(_controller, _documentKey);
        }

        /// <summary>
        /// Clears the content of the step.
        /// </summary>
        private void ClearStepContent()
        {
            _stopPointUserControl.ClearStopPoint();
            _primaryResultUserControl.ClearPrimaryResult();
            _showParametersUserControl.ClearParameterList();
            _annotationUserControl.ClearAnnotation();
            ClearStatusText();
            SetButtonStatus(false);
        }

        /// <summary>
        /// Clears the status text.
        /// </summary>
        private void ClearStatusText()
        {
            lblStatus.Text = string.Empty;
        }

        /// <summary>
        /// Sets the button statu.
        /// </summary>
        /// <param name="isEnable">if set to <c>true</c> [is enable].</param>
        private void SetButtonStatus(bool isEnable)
        {
            btnAddAssociationToList.Enabled = isEnable;
            if (isEnable)
                btnAssociate.Enabled = isEnable;
            else
                btnAssociate.Enabled = _associationList.Count > 0 ? true : false;
        }

        /// <summary>
        /// Validates the selection.
        /// </summary>
        /// <returns></returns>
        private bool ValidateSelection()
        {
            bool isValide = true;

            if (_selectedJob == null)
                isValide = false;
            else if (_annotationUserControl.AssociationName.Length == 0)
                isValide = false;
            else if (_annotationUserControl.Annotation.Length == 0)
                isValide = false;
            else if (_primaryResultUserControl.PrimaryResultSelectedIndex < 0)
                isValide = false;
            else if (_stopPointUserControl.SelectedStopPoint < 0)
                isValide = false;

            return isValide;
        }

        #endregion

        #region Event Handler

        /// <summary>
        /// Called when [jobs selected index changed].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnJobsSelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                _selectedJob = _selectJobsUserControl.GetJob();
                if (_selectedJob != null)
                {
                    _selectedPipeline = _selectJobsUserControl.GetPipeline(_selectedJob.PipelineLsid, _documentKey);
                    _stopPointUserControl.UpdateStopPoint(_selectedPipeline);
                    _primaryResultUserControl.UpdatePrimaryResult(_selectedJob, _selectedPipeline);
                    _showParametersUserControl.UpdateShowParameterList(_selectedPipeline.Parameters, _selectedJob.Parameters);
                    bool isValide = ValidateSelection();
                    SetButtonStatus(isValide);
                }
                else
                {
                    ClearStepContent();
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the Click event of the btnAddAssociationToList control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnAddAssociationToList_Click(object sender, EventArgs e)
        {
            AddAssociation(true);
            lblStatus.Text = "Job added to Association list";
        }

        /// <summary>
        /// Annotations the text changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void AnnotationTextChanged(object sender, EventArgs e)
        {
            bool isValide = ValidateSelection();
            SetButtonStatus(isValide);
        }

        /// <summary>
        /// Associations the delete button click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void AssociationDeleteButtonClick(object sender, EventArgs e)
        {
            int index = (int)sender;
            _pipelineList.RemoveAt(index);
            _associationList.RemoveAt(index);
        }

        /// <summary>
        /// Handles the Click event of the btnAssociate control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnAssociate_Click(object sender, EventArgs e)
        {
            if (ValidateSelection())
            {
                if (!AddAssociation(false))
                    return;
            }

            // kick off the background worker when form opens
            _asyncDownload = new Thread(new ThreadStart(DownloadFiles));
            _asyncDownload.Name = "Download Pipelines";
            _asyncDownload.SetApartmentState(ApartmentState.STA);
            OnAssociationStart();

            _totalDownloads = 0;
            //set total downloads by counting pipelines and steps
            foreach (Pipeline p in _pipelineList)
            {
                _totalDownloads += CountPipelinePieces(p);
            }
        }

        /// <summary>
        /// Handles the Tick event of the timerProgress control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void timerProgress_Tick(object sender, EventArgs e)
        {
            progressBar.Minimum = 0;
            progressBar.Maximum = _totalDownloads;

            if (_downloadsComplete > progressBar.Maximum)
            {
                progressBar.Maximum = _downloadsComplete;
            }

            progressBar.Value = _downloadsComplete;
            lblStatus.Text = _progressMessage;

            if (!_asyncDownload.IsAlive)
            {
                OnAssociationCompletedOrCancel();
            }
        }

        /// <summary>
        /// Handles the Click event of the btnCancel control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            if (_asyncDownload.IsAlive && !_isResultOk)
            {
                _asyncDownload.Abort();
                _progressMessage = "Pipeline association canceled";
            }
        }

        #endregion

        #region Associate Pipeline

        /// <summary>
        /// Downloads the files.
        /// </summary>
        private void DownloadFiles()
        {
            try
            {
                _isResultOk = false;
                _controller.BeginModelChanges(_documentKey);
                List<string> messages = null;

                if (_isEditing)
                {
                    messages = _controller.UpdatePipelineAssociations(_pipelineList, _associationList,
                             new DownloadProgress(UpdateDownloadStatus));
                }
                else
                {
                    messages = _controller.AddPipelineAssociations(_pipelineList, _associationList,
                             new DownloadProgress(UpdateDownloadStatus), _documentKey);
                }
                _controller.CommitModelChanges(_documentKey);

                if (messages != null && messages.Count > 0)
                {
                    string message = string.Join(Environment.NewLine + Environment.NewLine, messages.ToArray());
                    MessageBox.Show(message, Resources.ATTENTION);
                }
                _isResultOk = true;
            }
            catch (ThreadAbortException)
            {
                // user clicked cancel
                _controller.RollbackModelChanges(_documentKey);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
                if (_controller.GetDocumentModel(_documentKey).TransactionActive)
                {
                    _controller.RollbackModelChanges(_documentKey);
                }
            }
        }

        /// <summary>
        /// Updates the download.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="lsid">The lsid.</param>
        /// <param name="isStarting">if set to <c>true</c> [is starting].</param>
        private void UpdateDownloadStatus(string name, string lsid, bool isStarting)
        {
            if (isStarting)
            {
                string startMessage = Resources.START_PROCESSING;
                _progressMessage = String.Format(startMessage, name);
            }
            else
            {
                string finishedMessage = Resources.FINISH_PROCESSING;
                _progressMessage = String.Format(finishedMessage, name);
                _downloadsComplete++;
            }
        }

        /// <summary>
        /// Counts the pipeline pieces.
        /// </summary>
        /// <param name="p">The p.</param>
        /// <returns></returns>
        private static int CountPipelinePieces(Pipeline p)
        {
            int count = 1;
            if (!p.IsLinked)
            {
                count += p.Steps.Count;
            }
            return count;
        }

        /// <summary>
        /// Adds the association.
        /// </summary>
        private PipelineAssociation CreatePipelineAssociation()
        {
            PipelineAssociation pipelineAssociation = new PipelineAssociation();
            pipelineAssociation.Job = _selectedJob;
            pipelineAssociation.LocalId = _selectedJob.Id;
            pipelineAssociation.PipelineLsid = _selectedJob.PipelineLsid;
            pipelineAssociation.UserStopPoint = _stopPointUserControl.SelectedStopPoint;
            pipelineAssociation.PrimaryResultName = _primaryResultUserControl.GetSelectedPrimaryResult;
            pipelineAssociation.Name = _annotationUserControl.AssociationName;
            pipelineAssociation.Annotation = _annotationUserControl.Annotation;
            return pipelineAssociation;
        }

        /// <summary>
        /// Adds the association to list.
        /// </summary>
        /// <param name="showMessage">if set to <c>true</c> [show message].</param>
        private bool AddAssociation(bool showMessage)
        {
            if (!(_isEditing))
            {
                if (_viewPipelinesUserControl.IsAssociationNameExist(_annotationUserControl.AssociationName,
                    _associationList))
                {
                    if (showMessage)
                    {
                        MessageBox.Show("This document already contains an association with the same name or you are trying to associate multiple pipelines with the same name, Please choose another name",
                            "GenePattern Add-In", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    }
                    return true;
                }
            }

            bool performOperation = CheckPipelineExists();
            if (performOperation)
            {
                _associationList.Add(CreatePipelineAssociation());
                _pipelineList.Add(_selectedPipeline);
                _viewPipelinesUserControl.UpdateAssociationList(_associationList, _pipelineList);
            }
            return performOperation;
        }

        /// <summary>
        /// Called when [association start].
        /// </summary>
        private void OnAssociationStart()
        {
            splitContainer1.Enabled = false;
            splitContainer5.Enabled = false;
            btnAddAssociationToList.Enabled = false;
            btnAssociate.Enabled = false;

            progressBar.Visible = true;
            downloadAnimationUserControl1.Visible = true;
            btnCancel.Visible = true;

            _asyncDownload.Start();
            timerProgress.Start();
            downloadAnimationUserControl1.Start();
        }

        /// <summary>
        /// Called when [association completed or cancel].
        /// </summary>
        private void OnAssociationCompletedOrCancel()
        {
            progressBar.Value = progressBar.Maximum;
            timerProgress.Stop();
            downloadAnimationUserControl1.Stop();

            if (_isResultOk)
            {
                lblStatus.Text = "Pipeline association succeeded";
                _associationList.Clear();
                _pipelineList.Clear();
                _viewPipelinesUserControl.ClearAssociations();
                _annotationUserControl.ClearAnnotation();
            }

            progressBar.Visible = false;
            downloadAnimationUserControl1.Visible = false;
            btnCancel.Visible = false;

            splitContainer1.Enabled = true;
            splitContainer5.Enabled = true;
            SetButtonStatus(true);
        }

        #endregion

        #region Check If Pipeline Exists

        /// <summary>
        /// Checks the pipeline exists.
        /// </summary>
        /// <returns></returns>
        public bool CheckPipelineExists()
        {
            Model model = _controller.GetDocumentModel(_documentKey);

            // check existence of pipeline
            if (model.Pipelines.ContainsKey(_selectedJob.PipelineLsid))
            {
                // is this pipeline already embedded AND we are linking?  warn about deletion
                Pipeline original = model.Pipelines[_selectedJob.PipelineLsid];

                if (!original.IsLinked && !_selectJobsUserControl.IsStorePipelineInDocument)
                {
                    if (WarnAlreadyStored())
                    {
                        return false; // cancel
                    }
                }
                else
                {
                    _selectedPipeline.IsLinked = !_selectJobsUserControl.IsStorePipelineInDocument;

                    // if this is linked and we have a current connection, we might be overwriting, so check
                    if (_selectedPipeline.IsLinked && _controller.CurrentConnection != null)
                    {
                        WarnOverWritingConnection(original.LinkConnection);
                    }
                    else if (!_selectedPipeline.IsLinked && original.IsLinked)
                    {
                        if (!_controller.AssureConnection())
                        {
                            return false;
                        }
                    }
                }
            }
            else
            {
                _selectedPipeline.IsLinked = !_selectJobsUserControl.IsStorePipelineInDocument;
                bool storeOldConnection = _selectedPipeline.IsLinked && _selectJobsUserControl.IsJobFromDocument ? true : false;
                StoreOldConnection(storeOldConnection);
            }
            return true;
        }

        /// <summary>
        /// Stores the old connection.
        /// </summary>
        private void StoreOldConnection(bool storeOldConnection)
        {
            //Store Old Connection. Else store current connection
            if (storeOldConnection)
            {
                // linked from a previous connection
                Pipeline thePipeline = _controller.GetDocumentModel(_documentKey).Pipelines[_selectedJob.PipelineLsid];
                _selectedPipeline.LinkConnection = thePipeline.LinkConnection;
                _selectedPipeline.LinkConnection.Password = null;
            }
            else
            {
                _selectedPipeline.LinkConnection = _controller.CurrentConnection.Clone() as Connection;
                //delete users creds for security reasons
                _selectedPipeline.LinkConnection.Password = String.Empty;
            }
        }

        /// <summary>
        /// Warns the over writing connection.
        /// </summary>
        /// <param name="linkConnection">The link connection.</param>
        private void WarnOverWritingConnection(Connection linkConnection)
        {
            // alert user that connection is being overwritten
            // compare the connection currently in the pipeline to the users current connection
            string oldUrl = linkConnection.ServerName + ":" + linkConnection.Port;
            string newUrl = _controller.CurrentConnection.ServerName + ":" + _controller.CurrentConnection.Port;

            if (string.Compare(oldUrl, newUrl, false) != 0)
            {
                string message = String.Format(
                                  "This pipeline was previously linked from a different server url ({0}).  Do you want to change its url to {1}?",
                                   oldUrl, newUrl);
                if (DialogResult.Yes == MessageBox.Show(message, "Update link location?", MessageBoxButtons.YesNo))
                {
                    StoreOldConnection(false);
                }
            }
        }

        /// <summary>
        /// Warns the user a linked pipeline is already embedded.  Returns true if they cancel the operation
        /// </summary>
        /// <returns></returns>
        private bool WarnAlreadyStored()
        {
            string formatString = Resources.REMOVE_EMBEDDED_PIPELINE_MESSAGE; // TODO localize
            string caption = "Pipeline already embedded";
            DialogResult result =
                MessageBox.Show(String.Format(formatString, _selectedPipeline.Name), caption,
                                MessageBoxButtons.YesNoCancel);

            _selectedPipeline.IsLinked = result == DialogResult.Yes;

            switch (result)
            {
                case DialogResult.Yes:
                    StoreOldConnection(false);
                    break;
                case DialogResult.Cancel:
                    return true;
                case DialogResult.No:
                    _selectJobsUserControl.IsStorePipelineInDocument = true;
                    break;
            }

            return false;
        }

        #endregion
    }
}
