using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Security;
using System.Threading;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.ScientificAddIn.Common;
using Microsoft.eResearch.GenePattern.Server;
using System.Text;
using System.Globalization;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// Delegate Export Pipeline
    /// </summary>
    public delegate void ExportPipeline(string filename, PipelineAssociation association);

    /// <summary>
    /// Delegate Download Job Results
    /// </summary>
    public delegate void DownloadJobResults(PipelineAssociation association);

    /// <summary>
    /// This user control is used for displaying the associated pipelines with an image
    /// </summary>
    public partial class AssociatedPipelinesUserControl : UserControl, ITaskPaneControl
    {
        private List<PipelineAssociation> pipelineAssociationList;
        private IController _controller;
        private Thread visualizersThread;
        private Pipeline visualizersThreadPipeline;
        private PipelineAssociation visualizersThreadPA;
        private bool doNotUpdateUI;
        private IDictionary<string, IList<Result>> cachedResults;
        private string documentKey;

        bool handlingRerun;
       
        /// <summary>
        /// Initializes a new instance of the <see cref="AssociatedPipelinesUserControl"/> class.
        /// </summary>
        public AssociatedPipelinesUserControl()
        {
            InitializeComponent();
            SetPipelineAssociationButtonsEnabled(false);
            ResizeTabs(tabControl1);
            ResizeTabs(PipelineTabControl);

            tabControl1.TabPages.Clear();
            tabControl1.TabPages.Add(DataTabPage);
            tabControl1.TabPages.Add(ImageTabPage);
            tabControl1.TabPages.Add(DetailTabPage);

            cachedResults = new Dictionary<string, IList<Result>>();
        }

        /// <summary>
        /// Sets the set controller.
        /// </summary>
        /// <value>The set controller.</value>
        public IController Controller
        {
            get
            {
                return _controller;
            }
            set
            {
                this._controller = value;
                documentKey = _controller.DocumentKey;
            }
        }

        /// <summary>
        /// Sets the dock position.
        /// </summary>
        /// <param name="dockPosition">The dock position.</param>
        public void SetDockPosition(TaskPaneDockPosition dockPosition)
        {
            if ((dockPosition & (TaskPaneDockPosition.Left | TaskPaneDockPosition.Right | TaskPaneDockPosition.Floating)) != 0)
            {
                SetVertical(true);
            }
            else
            {
                SetVertical(false);
            }
        }

        /// <summary>
        /// Sets the vertical.
        /// </summary>
        /// <param name="controlIsVertical">if set to <c>true</c> [control is vertical].</param>
        private void SetVertical(bool controlIsVertical)
        {
            this.SuspendLayout();

            // Reduce the splitter distances to avoid ending up with an invalid
            // value when changing the orientation of the split panel controls
            int third = 0;
            if (controlIsVertical)
            {
                splitContainer1.SplitterDistance = splitContainer1.MinimumSize.Height;
                splitContainer2.SplitterDistance = splitContainer2.MinimumSize.Height;
                third = this.Height / 3;
            }
            else
            {
                splitContainer1.SplitterDistance = splitContainer1.MinimumSize.Width;
                splitContainer2.SplitterDistance = splitContainer2.MinimumSize.Width;
                third = this.Width / 3;
            }

            // splitcontainter orientation is opposite of the control's orientation
            splitContainer1.Orientation = controlIsVertical ? Orientation.Horizontal : Orientation.Vertical;
            splitContainer2.Orientation = controlIsVertical ? Orientation.Horizontal : Orientation.Vertical;

            splitContainer1.SplitterDistance = third;
            splitContainer2.SplitterDistance = third;

            this.ResumeLayout();
        }

        /// <summary>
        /// Displays the pipeline associations.
        /// </summary>
        /// <param name="pipelineAssociations">The pipeline associations.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="isAppendixImage">if set to <c>true</c> [is appendix image].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public void DisplayPipelineAssociations(List<PipelineAssociation> pipelineAssociations,
            string documentKey, bool isAppendixImage)
        {
            this.pipelineAssociationList = pipelineAssociations;
            doNotUpdateUI = true;
            UpdatePipelineAssociationsList(documentKey);
            doNotUpdateUI = false;
            UpdatePipelineDetails();

            SetButtonStatus(isAppendixImage);
        }

        /// <summary>
        /// Sets the button status.
        /// </summary>
        /// <param name="isAppendixImage">if set to <c>true</c> [is appendix image].</param>
        private void SetButtonStatus(bool isAppendixImage)
        {
            if (isAppendixImage)
            {
                RunButton.Enabled = false;
                DeleteButton.Enabled = false;
                Edit.Enabled = false;
            }
        }

        /// <summary>
        /// Updates the pipeline associations list.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        private void UpdatePipelineAssociationsList(string documentKey)
        {
            this.pipelineAssociationsListView.Items.Clear();

            OriginalRadioButton.Checked = true;
            if (pipelineAssociationList == null || pipelineAssociationList.Count == 0)
            {
                SetPipelineAssociationButtonsEnabled(false);
                if (pipelineAssociationsListView.Columns.Count > 0)
                    pipelineAssociationsListView.Columns[0].Width = pipelineAssociationsListView.Width;
                pipelineAssociationsListView.Items.Add(Resources.NO_PIPELINE_ASSOCIATIONS_FOUND);
                SetPipelineAssociationButtonsEnabled(false);
            }
            else
            {
                SetColumnWidths();
                Model currentModel = null;

                if (string.IsNullOrEmpty(documentKey))
                    currentModel = _controller.ActiveModel;
                else
                    currentModel = _controller.GetDocumentModel(documentKey);

                foreach (PipelineAssociation association in pipelineAssociationList)
                {
                    if (currentModel.Pipelines.ContainsKey(association.PipelineLsid))
                    {
                        Pipeline pipeline = currentModel.Pipelines[association.PipelineLsid];

                        ListViewItem item = new ListViewItem();
                        item.Text = association.Name;
                        if (pipeline.IsLinked)
                        {
                            item.ImageKey = "linkedPipeline";
                        }
                        else
                        {
                            item.ImageKey = "pipeline";
                        }
                        item.SubItems.Add(pipeline.Name);
                        item.Tag = association;
                        this.pipelineAssociationsListView.Items.Add(item);
                    }
                }
                pipelineAssociationsListView.Items[0].Selected = true;
                SetPipelineAssociationButtonsEnabled(true);
            }
        }

        /// <summary>
        /// Sets the column widths.
        /// </summary>
        private void SetColumnWidths()
        {
            if (pipelineAssociationsListView.Columns.Count > 1)
            {
                pipelineAssociationsListView.Columns[0].Width = (int)33 * pipelineAssociationsListView.Width / 100;
                pipelineAssociationsListView.Columns[1].Width = (int)66 * pipelineAssociationsListView.Width / 100;
            }
        }

        /// <summary>
        /// Handles the Click event of the savePipelineAssociation control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void savePipelineAssociation_Click(object sender, System.EventArgs e)
        {
            try
            {
                Debug.Assert(_controller != null);
                PipelineAssociation selectedAssociation = GetSelectedAssociation();
                if (selectedAssociation != null)
                {
                    // get pipeline for this association because it might be linked
                    Pipeline pipeline = GetSelectedPipeline();
                    if (pipeline.IsLinked)
                    {
                        //TODO LOCALIZE
                        if (DialogResult.Cancel == MessageBox.Show(Resources.PIPELINE_EMBED_CONNECTION_MSG, 
                            Resources.DOWNLOAD_DOC_CAPTION, MessageBoxButtons.OKCancel, MessageBoxIcon.Question))
                        {
                            return; // early exit for user cancellation
                        }
                    }

                    SaveFileDialog dialog = new SaveFileDialog();
                    dialog.Filter = "Zip Files (*.zip)|*.zip|All files (*.*)|*.*";
                    dialog.FileName = pipeline.Name.Replace(':', '_') + ".zip";
                    DialogResult result = dialog.ShowDialog();
                    if (result == DialogResult.OK && dialog.FileName != null)
                    {
                        using (DownloadZipForm form = new DownloadZipForm())
                        {
                            form.SetWork(_controller, selectedAssociation.PipelineLsid, dialog.FileName);
                            form.ShowDialog();
                        }
                    }
                }
                else
                {
                    InformUserToSelectAnAssociation();
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Extracts the pipeline zip async.
        /// </summary>
        /// <param name="filename">The filename.</param>
        /// <param name="selectedAssociation">The selected association.</param>
        private void ExtractPipelineZipAsync(string filename, PipelineAssociation selectedAssociation)
        {
            using (MemoryStream file = _controller.GetPipelineZip(selectedAssociation.PipelineLsid, documentKey))
            {
                using (BinaryWriter bw = new BinaryWriter(new FileStream(filename, FileMode.Create)))
                {
                    bw.Write(file.ToArray());
                }
            }
        }

        /// <summary>
        /// Handles the Click event of the Edit control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void Edit_Click(object sender, EventArgs e)
        {
            if (_controller.CheckDocumentProtection())
            {
                return;
            }

            try
            {
                PipelineAssociation assoc = GetSelectedAssociation();
                if (assoc != null)
                {
                    if (_controller.IsWizardUI)
                    {
                        // build business objects to rerun the pipeline wizard
                        using (AssociatePipelineWizard wizard = new AssociatePipelineWizard(_controller, false))
                        {
                            wizard.EditExistingPipeline(assoc);
                            wizard.ShowDialog();
                            _controller.UpdatePipelineAssociationsDisplay();
                        }
                    }
                    else
                    {
                        PaneEditPipelineCommand editPipeline = new PaneEditPipelineCommand(_controller);
                        editPipeline.Invoke(assoc);
                    }
                }
                else
                {
                    InformUserToSelectAnAssociation();
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }
        /// <summary>
        /// Informs the user to select result.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private static void InformUserToSelectResult()
        {
            MessageBox.Show(Resources.NO_RESULT_SELECT,
                            Resources.NO_RESULT_SELECT_CAPTION,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Information,
                            MessageBoxDefaultButton.Button2);
        }

        /// <summary>
        /// Informs the user to select an association.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private static void InformUserToSelectAnAssociation()
        {

            MessageBox.Show(Resources.NO_PIPELINEASSOCIATION_SELECTED,
                            Resources.NO_PIPELINEASSOCIATION_SELECTED_CAPTION,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Information,
                            MessageBoxDefaultButton.Button2);
        }

        /// <summary>
        /// Informs the user of no documentation.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private static void InformUserOfNoDocumentation()
        {
            MessageBox.Show(Resources.NO_DOCUMENT_PRESENT,
                            Resources.NO_DOCUMENT_PRESENT_CAPTION,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Information,
                            MessageBoxDefaultButton.Button2);
        }

        /// <summary>
        /// Gets the selected association.
        /// </summary>
        /// <returns></returns>
        private PipelineAssociation GetSelectedAssociation()
        {
            if (pipelineAssociationsListView.SelectedItems.Count > 0)
                return pipelineAssociationsListView.SelectedItems[0].Tag as PipelineAssociation;
            else
                return null;
        }

        /// <summary>
        /// Shows the documentation file.
        /// </summary>
        /// <param name="baseFile">The base file.</param>
        /// <param name="openWithDeafultApplication">if set to <c>true</c> [open with deafult application].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void ShowDocumentationFile(BaseFile baseFile, bool openWithDeafultApplication)
        {
            Pipeline pipeline = GetSelectedPipeline();

            if (pipeline != null && pipeline.IsLinked)
            {
                string message = Resources.DOWNLOAD_DOCUMENTATION_FILE;
                string caption = Resources.DOWNLOAD_DOCUMENTATION_FILE_CAPTION;
                if (DialogResult.Cancel == MessageBox.Show(message, caption, MessageBoxButtons.OKCancel, MessageBoxIcon.Question))
                {
                    return; // early exit for user cancellation
                }
            }

            if (pipeline != null)
            {
                string path = UserDataManager.GetTempPath(string.Empty);

                path = AssertPathExists(baseFile, path);
                if (openWithDeafultApplication)
                {
                    ProcessStartInfo info = new ProcessStartInfo(path);
                    info.UseShellExecute = true;
                    info.ErrorDialog = true;
                    info.ErrorDialogParentHandle = this.Handle;
                    Process process = new Process();
                    process.StartInfo = info;
                    process.Start();
                    Trace.WriteLine("Started process for file " + baseFile.FullPath, TraceLevel.Info.ToString());
                    // TODO clean up temp files
                }
                else
                {
                    SaveDescriptionFileDialog.FileName = baseFile.Filename;
                    SaveDescriptionFileDialog.InitialDirectory =
                        Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
                    if (DialogResult.OK == SaveDescriptionFileDialog.ShowDialog())
                    {
                        File.Copy(path, SaveDescriptionFileDialog.FileName, true);
                    }
                }
            }
        }

        /// <summary>
        /// Asserts the path exists.
        /// </summary>
        /// <param name="baseFile">The base file.</param>
        /// <param name="basePath">The base path.</param>
        /// <returns></returns>
        private string AssertPathExists(BaseFile baseFile, string basePath)
        {
            string path = basePath;
            if (!Directory.Exists(path))
            {
                Directory.CreateDirectory(path);
            }
            path += Path.DirectorySeparatorChar.ToString() + baseFile.Filename;

            if (!File.Exists(path))
            {
                MemoryStream memoryStream = null;

                try
                {
                    InternalFile internalFile = baseFile as InternalFile;
                    if (internalFile != null)
                    {
                        memoryStream = _controller.DataStore.GetFile(internalFile.Guid);
                    }
                    else
                    {
                        Pipeline pipeline = GetSelectedPipeline();
                        IGenePatternServer genePatternServer = new GenePatternServer();
                        Connection connection = pipeline.LinkConnection.Clone() as Connection;

                        using (ServerConnection serverConnectionForm = new ServerConnection(pipeline.LinkConnection))
                        {
                            if (DialogResult.OK == serverConnectionForm.ShowDialog())
                            {
                                connection = serverConnectionForm.Connection;
                            }
                        }

                        genePatternServer.Connection = connection;
                        memoryStream = genePatternServer.GetDocumentationFile(pipeline.Lsid, baseFile.Filename);
                    }
                    using (FileStream fileStream = new FileStream(path, FileMode.CreateNew))
                    {
                        fileStream.Write(memoryStream.ToArray(), 0, (int)memoryStream.Length);
                    }
                }
                finally
                {
                    if (memoryStream != null)
                        memoryStream.Dispose();
                }

            }

            return path;
        }

        /// <summary>
        /// Handles the Click event of the DeleteButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void DeleteButton_Click(object sender, EventArgs e)
        {

            if (_controller.CheckDocumentProtection())
            {
                return;
            }

            try
            {
                PipelineAssociation assoc = GetSelectedAssociation();
                if (assoc != null)
                {
                    string text = Resources.DELETE_PIPELINE_ASSOCIATION_QUESTION;
                    string caption = Resources.DELETE_PIPELINE_ASSOCIATION_CAPTION;
                    if (DialogResult.Yes == MessageBox.Show(text, caption, MessageBoxButtons.YesNo,
                                                            MessageBoxIcon.Question, MessageBoxDefaultButton.Button2))
                    {
                        _controller.DeletePipelineAssociation(assoc);
                    }
                }
                else
                {
                    InformUserToSelectAnAssociation();
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the Click event of the RunButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void RunButton_Click(object sender, EventArgs e)
        {
            try
            {
                PipelineAssociation pipelineAssociation = GetSelectedAssociation();
                if (pipelineAssociation != null)
                {
                    if (pipelineAssociation.Rerun.Job.Id > -1)
                    {
                        // TODO resouce it!
                        string text =
                            "The selected pipeline association already has a rerun. Do you want to discard the exisitng rerun information?";
                        string caption = "Overwrite rerun?";
                        DialogResult result =
                            MessageBox.Show(text, caption, MessageBoxButtons.OKCancel, MessageBoxIcon.Question,
                                            MessageBoxDefaultButton.Button2);
                        if (result == DialogResult.Cancel)
                            return;
                    }

                    if (_controller.IsWizardUI)
                    {
                        using (RerunPipelineWizard rerunPipelineWizard = 
                            new RerunPipelineWizard(_controller, pipelineAssociationList, pipelineAssociation))
                        {
                            rerunPipelineWizard.ShowDialog();
                            pipelineAssociation = null;
                        }
                    }
                    else
                    {
                        PaneRenurPipelineCommand rerunPipelineCommand = new PaneRenurPipelineCommand(_controller);
                        rerunPipelineCommand.Invoke(true, pipelineAssociationsListView.SelectedItems[0].Index);
                    }
                }
                else
                {
                    InformUserToSelectAnAssociation();
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the pipelineAssociationsListView control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void pipelineAssociationsListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                PipelineAssociation pipelineAssociation = GetSelectedAssociation();
                doNotUpdateUI = true;

                if (pipelineAssociation != null && pipelineAssociation.Rerun.Job.Id > -1)
                {
                    RerunRadioButton.Checked = true;
                }
                else
                {
                    OriginalRadioButton.Checked = true;
                }

                doNotUpdateUI = false;
                UpdatePipelineDetails();
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Updates the pipeline details.
        /// </summary>
        private void UpdatePipelineDetails()
        {
            if (doNotUpdateUI)
            {
                return;
            }


            PipelineAssociation pipelineAssociation = GetSelectedAssociation();
            if (pipelineAssociation == null)
            {
                ClearPipelineDetails();
                return;
            }
            Pipeline pipeline = GetSelectedPipeline();
            if (pipeline == null)
            {
                ClearPipelineDetails();
                return;
            }

            Outputs.BeginUpdate();
            pipelineSteps1.PipelineName = pipeline.Name;

            List<PipelineChild> stepNames = new List<PipelineChild>(pipeline.Steps.Count);
            for (int i = 0; i < pipeline.Steps.Count; i++)
            {
                PipelineStep step = pipeline.Steps[i];
                stepNames.Add(new PipelineChild(step.Name, (int)step.Type));
            }
            pipelineSteps1.Children = stepNames.ToArray();
            pipelineSteps1.SelectedChildIndex = pipelineAssociation.UserStopPoint;

            AnnotationTextBox.Text = pipelineAssociation.Annotation;
            AnnotationTabPage.Enabled = true;

            bool rerunIsPresent = (pipelineAssociation.Rerun.Job.Id > -1);
            ShowOriginalToRerunSwitch(rerunIsPresent);

            // try to get job
            JobStatus status = JobStatus.Finished;
            if (rerunIsPresent && 
                pipelineAssociation.Rerun.Job.Status != JobStatus.Finished )
            {
                status = UpdateRerunJob(pipelineAssociation);
                RerunTimer.Enabled = (status == JobStatus.Processing || status == JobStatus.Pending);
                // start a timer to update the ui if the job isn't done
            }

            if (rerunIsPresent && RerunRadioButton.Checked)
            {
                if (!handlingRerun)
                    ShowRerunParametersAndData(pipelineAssociation, pipeline, status);
            }
            else
            {
                ShowOriginalParametersAndData(pipelineAssociation, pipeline);
            }
            Outputs.EndUpdate();

        }

        /// <summary>
        /// Shows the rerun parameters and data.
        /// </summary>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="status">The status.</param>
        private void ShowRerunParametersAndData(PipelineAssociation pipelineAssociation,
                                                Pipeline pipeline, JobStatus status)
        {
            Job rerunJob = pipelineAssociation.Rerun.Job;
            SetParameters(rerunJob, pipeline);

            switch (status)
            {

                case JobStatus.Finished:
                    {

                        List<PipelineChild> dataFilenames = new List<PipelineChild>();
                        int outputSelIndex = -1;
                        for (int i = 0; i < rerunJob.FileNames.Count; i++)
                        {
                            if (!rerunJob.FileNames[i].EndsWith(SelectPrimaryResult.ExecutionLog))
                            {
                                dataFilenames.Add(new PipelineChild(rerunJob.FileNames[i], 0));
                                if (rerunJob.FileNames[i] == pipelineAssociation.PrimaryResultName)
                                {
                                    outputSelIndex = dataFilenames.Count - 1;
                                }
                            }
                        }

                        Outputs.PipelineName = pipeline.Name;
                        Outputs.Children = dataFilenames.ToArray();
                        Outputs.SelectedChildIndex = outputSelIndex != -1 ? outputSelIndex : Outputs.Children.Length - 1; ;
                        Outputs.SelectedChildImageKey = "selectedResult";
                        Outputs.ReadOnly = false;
                        Outputs.OnlyShowMessage = false;
                        Outputs.Enabled = true;

                        string rerunKey = pipelineAssociation.Rerun.Connection.ServerName +
                                          pipelineAssociation.Rerun.Job.Id;
                        if (!cachedResults.ContainsKey(rerunKey))
                        {
                            imageResultsViewer.DisplayRerunResults(null);
                        }
                        else
                        {
                            imageResultsViewer.DisplayRerunResults(cachedResults[rerunKey]);
                        }


                        SetImageAndDataButtonPanelsVisible(true, true);
                        DataInsertIntoAppendixButton.Enabled = (rerunJob.ShallowResults.Count != 0);
                        DataSaveResultToDiskButton.Enabled = (rerunJob.ShallowResults.Count != 0);
                        ImageInsertIntoAppendixButton.Enabled = (imageResultsViewer.ResultsList.Items.Count > 0);
                        ImageSaveToDiskButton.Enabled = (imageResultsViewer.ResultsList.Items.Count > 0);
                        SetDataTabControl(this.Outputs);
                        SetImageTabControl(imageResultsViewer);
                        ShowViewersButton.Visible = pipeline.HasVisualizers && rerunJob.HasReachedVisualizers(pipeline);
                    }
                    break;
                case JobStatus.Pending:
                case JobStatus.Processing:
                    {
                        if (!(DataTabMainPanel.Controls[0] is JobProcessing))
                        {
                            UserControl jobproc = new JobProcessing();
                            jobproc.Dock = DockStyle.Fill;
                            SetDataTabControl(jobproc);
                        }
                        if (!(ImageTabMainPanel.Controls[0] is JobProcessing))
                        {
                            UserControl jobproc = new JobProcessing();
                            jobproc.Dock = DockStyle.Fill;
                            SetImageTabControl(jobproc);
                        }
                        SetImageAndDataButtonPanelsVisible(false, false);
                    }
                    break;
                case JobStatus.Failed:
                case JobStatus.Unknown:
                case JobStatus.Canceled:
                    PipelineChildren newPipelineChildren = new PipelineChildren();
                    newPipelineChildren.Dock = DockStyle.Fill;
                    SetDataTabControl(newPipelineChildren);

                    ImageResultsViewer newImageViewer = new ImageResultsViewer();
                    newImageViewer.Dock = DockStyle.Fill;
                    SetImageTabControl(newImageViewer);
                    break;
            }
        }

        /// <summary>
        /// Gets the rerun server.
        /// </summary>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1303:DoNotPassLiteralsAsLocalizedParameters", MessageId = "Microsoft.eResearch.GenePattern.Package.IServerBroker.GetTestedVerifiedServer(Microsoft.eResearch.GenePattern.Common.Connection,System.String)")]
        private IGenePatternServer GetRerunServer(PipelineAssociation pipelineAssociation)
        {
            if (pipelineAssociation.Rerun.Connection == null)
                pipelineAssociation.Rerun.Connection = Controller.GetConnectionByDocumentKey(this.documentKey);

            IGenePatternServer rerunServer = _controller.ServerBroker.GetGenePatternServerForConnection(pipelineAssociation.Rerun.Connection);
            if (pipelineAssociation.Rerun.PromptForReconnect && !rerunServer.Ping())
            {
                rerunServer =
                    _controller.ServerBroker.GetTestedVerifiedServer(pipelineAssociation.Rerun.Connection,
                                                                     "Enter credentials for rerun of \"" +
                                                                     pipelineAssociation.Name +
                                                                     "\", or cancel to not download results.");
                // if it's null, the reconnect was canceled.  don't ask again
                if (rerunServer == null)
                {
                    pipelineAssociation.Rerun.PromptForReconnect = false;
                }
            }
            return rerunServer;
        }

        /// <summary>
        /// Sets the image and data button panels visible.
        /// </summary>
        /// <param name="visible">if set to <c>true</c> [visible].</param>
        /// <param name="insertAppendixVisible">if set to <c>true</c> [insert appendix visible].</param>
        private void SetImageAndDataButtonPanelsVisible(bool visible, bool insertAppendixVisible)
        {
            DataButtonPanel.Visible = visible;
            ImageButtonPanel.Visible = visible;
            DataInsertIntoAppendixButton.Visible = visible && insertAppendixVisible;
            ImageInsertIntoAppendixButton.Visible = visible && insertAppendixVisible;

            DataSaveResultToDiskButton.Visible = visible && insertAppendixVisible;
            ImageSaveToDiskButton.Visible = visible && insertAppendixVisible;
        }

        /// <summary>
        /// Sets the data tab control.
        /// </summary>
        /// <param name="control">The control.</param>
        private void SetDataTabControl(Control control)
        {
            if (DataTabMainPanel.Controls.Count != 1 || DataTabMainPanel.Controls[0] != control)
            {
                DataTabMainPanel.Controls.Clear();
                DataTabMainPanel.Controls.Add(control);
            }
        }

        /// <summary>
        /// Sets the image tab control.
        /// </summary>
        /// <param name="control">The control.</param>
        private void SetImageTabControl(Control control)
        {
            if (ImageTabMainPanel.Controls.Count != 1 || ImageTabMainPanel.Controls[0] != control)
            {
                ImageTabMainPanel.Controls.Clear();
                ImageTabMainPanel.Controls.Add(control);
            }
        }

        /// <summary>
        /// Shows the original parameters and data.
        /// </summary>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        /// <param name="pipeline">The pipeline.</param>
        private void ShowOriginalParametersAndData(PipelineAssociation pipelineAssociation,
                                                    Pipeline pipeline)
        {
            bool hasNoResults = pipelineAssociation.EmbeddedResults == null ||
                              pipelineAssociation.EmbeddedResults.Count == 0;
            if (hasNoResults)
            {
                // comment this line out if toggling above section
                ShowJobResults(pipelineAssociation);
            }
            else
            {
                Outputs.PipelineName = pipeline.Name;
                Outputs.ReadOnly = false;
                List<PipelineChild> filenames = new List<PipelineChild>(pipelineAssociation.EmbeddedResults.Count);
                for (int i = 0; i < pipelineAssociation.EmbeddedResults.Count; i++)
                {
                    filenames.Add(new PipelineChild(pipelineAssociation.EmbeddedResults[i].Filename, 0));
                }
                Outputs.PipelineName = pipeline.Name;
                Outputs.Children = filenames.ToArray();
                Outputs.SelectedChildImageKey = "selectedResult";
                Outputs.OnlyShowMessage = false;
                Outputs.Enabled = true;
                imageResultsViewer.DisplayEmbeddedResults(pipelineAssociation, _controller.DataStore);
            }

            SetImageAndDataButtonPanelsVisible(true, false);
            SetParameters(pipelineAssociation.Job, pipeline);
            SetDataTabControl(this.Outputs);
            SetImageTabControl(this.imageResultsViewer);

            DataSaveResultToDiskButton.Enabled = !hasNoResults;
            ImageSaveToDiskButton.Enabled = !hasNoResults;
        }

        /// <summary>
        /// Shows the job results.
        /// </summary>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        private void ShowJobResults(PipelineAssociation pipelineAssociation)
        {
            Outputs.PipelineName = pipelineAssociation.Job.PipelineName;
            Outputs.ReadOnly = true;
            List<PipelineChild> filenames = new List<PipelineChild>();
            int? primaryResultIndex = null;
            int i = 0;
            foreach (string filename in pipelineAssociation.Job.FileNames)
            {
                if (!filename.EndsWith(SelectPrimaryResult.ExecutionLog))
                {
                    if (filename == pipelineAssociation.PrimaryResultName)
                    {
                        primaryResultIndex = filenames.Count;
                    }
                    filenames.Add(new PipelineChild(filename, 0));
                }
                i++;
            }
            Outputs.Children = filenames.ToArray();
            if (primaryResultIndex != null)
            {
                Outputs.SelectedChildIndex = primaryResultIndex.Value;
            }
            Outputs.SelectedChildImageKey = "selectedResult";
            Outputs.OnlyShowMessage = false;
            Outputs.Enabled = true;

            imageResultsViewer.DisplayOriginalResults(pipelineAssociation);
        }

        /// <summary>
        /// Shows the original to rerun switch.
        /// </summary>
        /// <param name="doShow">if set to <c>true</c> [do show].</param>
        private void ShowOriginalToRerunSwitch(bool doShow)
        {
            OriginalToRerunSwitchPanel.Visible = doShow;
        }

        /// <summary>
        /// Updates the rerun job.
        /// </summary>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private JobStatus UpdateRerunJob(PipelineAssociation pipelineAssociation)
        {
            Rerun rerun = pipelineAssociation.Rerun;

            IGenePatternServer genePatternServer = GetRerunServer(pipelineAssociation);
            if (genePatternServer == null)
            {
                return JobStatus.Unknown;
            }

            try
            {
                rerun.Connection = genePatternServer.Connection;
                Job rerunJob = genePatternServer.GetJobById(rerun.Job.Id);
                rerun.Job = rerunJob;
                JobStatus status = rerunJob.Status;

                switch (status)
                {
                    case JobStatus.Unknown:
                        if (rerun.PromptForDelete)
                        {
                            // job status is null = job is gone = prompt user about removing the rerun
                            // TODO connection
                            // TODO only once per run
                            string message =
                                "This pipeline association has been rerun, but the associated job is no longer on the server, or the job failed. Do you want to delete the reference to it?";
                            DialogResult result =
                                MessageBox.Show(message, Resources.GENEPATTREN_MSG_HEADER, MessageBoxButtons.YesNo, MessageBoxIcon.Warning);

                            if (result == DialogResult.Yes)
                            {
                                rerun = new Rerun();
                            }
                            else
                            {
                                // suppress next time it gets selected
                                rerun.PromptForDelete = false;
                            }
                        }
                        break;
                    case JobStatus.Finished:
                        {
                            lock (cachedResults)
                            {
                                string rerunKey = rerun.Connection.ServerName + rerun.Job.Id;
                                if (!cachedResults.ContainsKey(rerunKey))
                                {
                                    // we've never downloaded results for this job.  do it now.
                                    DownloadJobResults downloader =
                                       new DownloadJobResults(DownloadJobResultsAsync);
                                    cachedResults.Add(rerunKey, null);
                                    downloader.BeginInvoke(pipelineAssociation, null, null);
                                    status = JobStatus.Processing;
                                }
                                else if (cachedResults[rerunKey] == null)
                                {
                                    // haven't finished downloading results.  change status to "processing"
                                    status = JobStatus.Processing; // switch status while download finishes
                                }
                            }
                        }
                        break;
                }
                return status;
            }
            catch (Exception)
            {
                return JobStatus.Failed;
                //throw ex;
            }

        }

        /// <summary>
        /// Sets the parameters.
        /// </summary>
        /// <param name="job">The job.</param>
        /// <param name="pipeline">The pipeline.</param>
        private void SetParameters(Job job, Pipeline pipeline)
        {
            pipelineParameters1.SetParameters(pipeline.Parameters, job.Parameters);
        }

        /// <summary>
        /// Clears the pipeline details.
        /// </summary>
        private void ClearPipelineDetails()
        {
            pipelineSteps1.PipelineName = null;
            AnnotationTextBox.Text = string.Empty;
            AnnotationTabPage.Enabled = false;
            Outputs.PipelineName = null;
            Outputs.Children = null;
            Outputs.Enabled = false;
            Outputs.OnlyShowMessage = false;
            SetDataTabControl(Outputs);
            SetImageTabControl(imageResultsViewer);
            SetImageAndDataButtonPanelsVisible(false, false);
            ShowOriginalToRerunSwitch(false);
            pipelineParameters1.SetParameters(null, null);
            imageResultsViewer.ResultsList.Items.Clear();
        }

        /// <summary>
        /// Gets the selected pipeline.
        /// </summary>
        /// <returns></returns>
        private Pipeline GetSelectedPipeline()
        {
            Pipeline pipeline = null;
            PipelineAssociation pipelineAssociation = GetSelectedAssociation();
            if (pipelineAssociation != null)
            {
                if (_controller.GetDocumentModel(documentKey).Pipelines.ContainsKey(pipelineAssociation.PipelineLsid))
                {
                    pipeline = _controller.GetDocumentModel(documentKey).Pipelines[pipelineAssociation.PipelineLsid];
                }
                else
                {
                    Trace.WriteLine("PipelineName from Pipeline Association not found in model", TraceLevel.Error.ToString());
                    ClearPipelineDetails();
                }
            }
            return pipeline;
        }

        /// <summary>
        /// Sets the pipeline association buttons enabled.
        /// </summary>
        /// <param name="enabled">if set to <c>true</c> [enabled].</param>
        private void SetPipelineAssociationButtonsEnabled(bool enabled)
        {
            DeleteButton.Enabled = enabled;
            DescriptionButton.Enabled = enabled;
            Edit.Enabled = enabled;
            SaveButton.Enabled = enabled;
            RunButton.Enabled = enabled;
            DataInsertIntoAppendixButton.Enabled = enabled;
            DataSaveResultToDiskButton.Enabled = enabled;

        }

        /// <summary>
        /// Resizes the tabs.
        /// </summary>
        /// <param name="control">The control.</param>
        private static void ResizeTabs(TabControl control)
        {
            int width = control.Size.Width - (6 / control.TabCount) - 1;
            int nominalWidth = width / control.TabCount;
            int remainder;
            Math.DivRem(width, control.TabCount, out remainder);

            control.ItemSize = new Size((nominalWidth > 1 ? nominalWidth : 1), control.ItemSize.Height);
        }

        /// <summary>
        /// Handles the SizeChanged event of the AssociatedPipelinesUserControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void AssociatedPipelinesUserControl_SizeChanged(object sender, EventArgs e)
        {
            ResizeTabs(tabControl1);
            ResizeTabs(PipelineTabControl);
        }

        /// <summary>
        /// Handles the Click event of the descriptionButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void descriptionButton_Click(object sender, EventArgs e)
        {
            try
            {
                if (pipelineAssociationsListView.SelectedItems.Count == 0)
                {
                    InformUserToSelectAnAssociation();
                    return;
                }
                Pipeline pipeline = GetSelectedPipeline();
                if (pipeline == null)
                {
                    return;
                }
                if (pipeline.DocumentationFilenames == null || pipeline.DocumentationFilenames.Count == 0)
                {
                    InformUserOfNoDocumentation();
                    return;
                }

                if (pipeline.DocumentationFilenames.Count > 0
                    || (pipeline.IsLinked && pipeline.DocumentationFilenames.Count > 0))
                {
                    using (SelectDocumentationForm form = new SelectDocumentationForm())
                    {
                        form.Location = new Point(Cursor.Position.X - form.Width / 2, Cursor.Position.Y - 2);
                        string path = UserDataManager.GetTempPath(string.Empty);
                        form.BasePath = path;

                        if (pipeline.IsLinked)
                        {
                            BaseFile[] results = new BaseFile[pipeline.DocumentationFilenames.Count];
                            for (int i = 0; i < pipeline.DocumentationFilenames.Count; i++)
                            {
                                Result result = new Result();
                                result.Filename = pipeline.DocumentationFilenames[i];
                                results[i] = result;
                            }
                            form.DocumentationFiles = results;
                        }
                        else
                        {
                            form.DocumentationFiles = pipeline.Documentation.ToArray();
                        }

                        DialogResult dialogResult = form.ShowDialog();
                        if (dialogResult == DialogResult.OK
                            && form.SelectedFile != null)
                        {
                            BaseFile baseFile = form.SelectedFile;
                            if (baseFile != null)
                            {
                                ShowDocumentationFile(baseFile, form.OpenWithDefaultApplication);
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Saves the result.
        /// </summary>
        /// <param name="isImage">if set to <c>true</c> [is image].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void SaveResult(bool isImage)
        {
            try
            {
                string suggestedfilename;
                byte[] data = GetSelectedBytesAndFilename(isImage, out suggestedfilename);

                if (data != null)
                {
                    SaveFileDialog dialog = new SaveFileDialog();
                    dialog.FileName = suggestedfilename;
                    DialogResult result = dialog.ShowDialog(this);

                    if (dialog.FileName != null && result != DialogResult.Cancel)
                    {
                        using (FileStream fileStrean = new FileStream(dialog.FileName, FileMode.Create))
                        {
                            using (BinaryWriter bw = new BinaryWriter(fileStrean))
                            {
                                bw.Write(data);
                            }
                        }
                    }

                    dialog.Dispose();
                    data = null;
                }
                else
                {
                    InformUserToSelectResult();
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Gets the selected bytes and filename.
        /// </summary>
        /// <param name="isImage">if set to <c>true</c> [is image].</param>
        /// <param name="suggestedfilename">The suggestedfilename.</param>
        /// <returns></returns>
        private byte[] GetSelectedBytesAndFilename(bool isImage, out string suggestedfilename)
        {
            byte[] data = null;
            suggestedfilename = "";
            if (RerunRadioButton.Checked)
            {
                // export selected result
                Result selResult = DownloadSelectedRerunResult(isImage);
                if (selResult != null)
                {
                    data = selResult.Content;
                    suggestedfilename = selResult.Filename;

                    selResult.Content = null;
                    selResult = null;
                    GC.Collect();
                }
            }
            else
            {
                if (isImage)
                {
                    if (imageResultsViewer.ResultsList.SelectedItems.Count > 0)
                    {
                        Image selImage = imageResultsViewer.GetSelectedFullSizeImage();
                        using (MemoryStream imageStream = new MemoryStream())
                        {
                            selImage.Save(imageStream, ImageFormat.Jpeg);
                            data = imageStream.ToArray();
                            suggestedfilename = imageResultsViewer.ResultsList.SelectedItems[0].Text;
                        }
                    }
                }
                else
                {
                    if (Outputs.SelectedChildIndex > -1)
                    {
                        int index = Outputs.SelectedChildIndex;
                        string filename = Outputs.SelectedNode.Text;

                        for (int i = 0; i < GetSelectedAssociation().EmbeddedResults.Count; i++)
                        {
                            InternalFile embeddedResult = GetSelectedAssociation().EmbeddedResults[i];
                            if (embeddedResult.Filename.Equals(filename) && i >= index)
                            {
                                using (MemoryStream dataStream = _controller.DataStore.GetFile(embeddedResult.Guid))
                                {
                                    data = dataStream.ToArray();
                                    suggestedfilename = embeddedResult.Filename;
                                }
                            }
                        }
                    }
                }
            }
            return data;
        }

        /// <summary>
        /// Downloads the selected rerun result.
        /// </summary>
        /// <param name="isImage">if set to <c>true</c> [is image].</param>
        /// <returns></returns>
        private Result DownloadSelectedRerunResult(bool isImage)
        {
            string filename = null;

            if (isImage)
            {
                if (imageResultsViewer.ResultsList.SelectedItems.Count > 0)
                {
                    filename = imageResultsViewer.ResultsList.SelectedItems[0].Text;
                }

            }
            else
            {
                filename = GetSelectedFilename();
            }
            if (filename == null)
            {
                return null; // no filename, so don't do anything
            }

            IList<Result> results = DownloadRerunResults(filename);

            // find selected result
            Result selResult = null;
            for (int i = 0; i < results.Count; i++)
            {
                if (results[i].Filename == filename)
                {
                    selResult = results[i];
                }
            }

            results.Clear();
            results = null;
            GC.Collect();

            return selResult;
        }

        /// <summary>
        /// Downloads the rerun results.
        /// </summary>
        /// <param name="specificfilename">The specificfilename.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
        private IList<Result> DownloadRerunResults(string specificfilename)
        {
            PipelineAssociation selAssociation = GetSelectedAssociation();

            // find the first shallow result with the filename
            int sourceJobId = -1;
            foreach (ShallowJobResult res in selAssociation.Rerun.Job.ShallowResults)
            {
                if (res.Filename == specificfilename)
                {
                    sourceJobId = res.SourceJobId;
                    break;
                }
            }
            if (sourceJobId == -1)
            {
                throw new ApplicationException("Couldn't find the source job for specific file: " + specificfilename);
            }

            IGenePatternServer rerunServer =
                _controller.ServerBroker.GetGenePatternServerForConnection(selAssociation.Rerun.Connection);
            IList<Result> results = rerunServer.GetJobResults(sourceJobId, specificfilename);
            List<Result> finalresults = new List<Result>();
            foreach (Result r in results)
            {
                if (!r.Filename.ToLower(CultureInfo.InvariantCulture).EndsWith(SelectPrimaryResult.ExecutionLog))
                {
                    finalresults.Add(r);
                }
            }
            rerunServer.Dispose();
            results.Clear();
            results = null;
            GC.Collect();

            return finalresults;
        }

        /// <summary>
        /// Gets the selected filename.
        /// </summary>
        /// <returns></returns>
        private string GetSelectedFilename()
        {
            if (Outputs.SelectedChildIndex >= 0 && Outputs.SelectedChildIndex < Outputs.Children.Length)
            {

                return Outputs.Children[Outputs.SelectedChildIndex].Name;
            }
            else
            {
                return null;
            }
        }

        #region Embed Rerun Results

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private static List<Result> EmbedAllResults(IList<Result> Result, ref List<Result> resultsToEmbed)
        {
            bool isLargeFilePresent = false;
            List<string> zeroSizedFiles = new List<string>();
            StringBuilder message = new StringBuilder();
            message.Append(Resources.EMBED_ALL_RESULTS);

            foreach (Result r in Result)
            {
                // add the results to the datastore and the association.. if we aren't locked.
                if (r.Length == 0)
                {
                    zeroSizedFiles.Add(r.Filename);
                    continue;
                }
                else
                {
                    resultsToEmbed.Add(r);
                }

                //If the file result size exceeds 5 MB,warning will be displayed
                if (r.Length > DefaultSetting.LargeFileSize)
                {
                    isLargeFilePresent = true;
                }
            }

            //if the File size is zero,warning will be displayed.
            if (zeroSizedFiles.Count != 0)
            {
                StringBuilder messageText = new StringBuilder();
                messageText.Append(Resources.DONOT_EMBED_ZERO_RESULT);
                foreach (string file in zeroSizedFiles)
                {
                    messageText.Append("  \n");
                    messageText.Append(file);
                }
                MessageBox.Show(messageText.ToString(), Resources.GENEPATTREN_MSG_HEADER,
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
            }

            //If the size of result files exceeds 5 MB,warning will be displayed.
            if (isLargeFilePresent)
            {
                message.Append("  \n");
                for (int i = 0; i < resultsToEmbed.Count; i++)
                {
                    if (resultsToEmbed[i].IsLargeSize)
                    {
                        message.Append("  \n");
                        message.Append(resultsToEmbed[i].Filename);
                    }
                }

                DialogResult res = MessageBox.Show(message.ToString(),
                        Resources.GENEPATTREN_MSG_HEADER, MessageBoxButtons.YesNo, MessageBoxIcon.Warning);

                //If "No" is selected,large files will be removed from resultsToEmbed list.
                if (res == DialogResult.No)
                {
                    for (int i = 0; i < resultsToEmbed.Count; i++)
                    {
                        if (resultsToEmbed[i].IsLargeSize)
                        {
                            resultsToEmbed.RemoveAt(i);
                            i--;
                        }
                    }
                }
            }
            return resultsToEmbed;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private List<Result> EmbedSingleResult(IList<Result> Result, bool isImage, ref List<Result> resultsToEmbed)
        {
            string filename = GetFilenameFromImageOrResults(isImage);
            
            foreach (Result r in Result)
            {
                //if the File size is zero,warning will be displayed.
                if (r.Filename == filename)
                {
                    if (r.Length == 0)
                    {
                        StringBuilder messageText = new StringBuilder();
                        messageText.Append(Resources.DONOT_EMBED_ZERO_RESULT);
                        messageText.Append("  \n");
                        messageText.Append(r.Filename);
                        MessageBox.Show(messageText.ToString(), Resources.GENEPATTREN_MSG_HEADER,
                            MessageBoxButtons.OK, MessageBoxIcon.Information);
                        break;
                    }

                    if (r.Length > DefaultSetting.LargeFileSize)  //If the file result size exceeds 5 MB,warning will be displayed
                    {
                        DialogResult res = MessageBox.Show(Resources.EMBED_SINGLE_RESULT,
                            Resources.GENEPATTREN_MSG_HEADER, MessageBoxButtons.YesNo, MessageBoxIcon.Warning);

                        if (res == DialogResult.No)
                            break;
                    }

                    resultsToEmbed.Add(r);
                    break;
                }
            }
            return resultsToEmbed;
        }

        #endregion

        /// <summary>
        /// Inserts the into appendix.
        /// </summary>
        /// <param name="isImage">if set to <c>true</c> [is image].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void InsertIntoAppendix(bool isImage)
        {
            string[] resultsToEmbedArray = null;
            try
            {
                // create a new association
                PipelineAssociation pa = GetSelectedAssociation();
                PipelineAssociation newAssociation = (PipelineAssociation)pa.Clone();
                newAssociation.MakeShallowClone();

                IGenePatternServer rerunServer =
                _controller.ServerBroker.GetGenePatternServerForConnection(pa.Rerun.Connection);
                Job rerunJob = rerunServer.GetJobById(pa.Rerun.Job.Id);

                newAssociation.Job = rerunJob;
                newAssociation.Rerun = new Rerun(); // clear rerun info
                newAssociation.Name += " Rerun"; // TODO Resource it!!

                Pipeline pipeline = GetSelectedPipeline();
                List<PipelineAssociation> associations = new List<PipelineAssociation>();
                associations.Add(newAssociation);

                List<Pipeline> pipelines = new List<Pipeline>();
                pipelines.Add(pipeline);

                if (!isImage)
                {
                    newAssociation.PrimaryResultName = GetSelectedFilename();
                }
                else
                {
                    newAssociation.PrimaryResultName = imageResultsViewer.ResultsList.SelectedItems[0].Text;
                }

                bool insertDataFiles = false;
                bool canceled = false;
                bool embedAll = false;
                string comment = "";
                IList<Result> results = null;

                using (InsertAppendixForm appForm = new InsertAppendixForm(rerunServer, pa))
                {
                    appForm.SetParametersVisible(newAssociation.Job.Parameters.Count > 0);
                    appForm.PrimaryResultName = newAssociation.PrimaryResultName;
                    canceled = appForm.ShowDialog() == DialogResult.Cancel;
                    comment = appForm.AnnotationText;
                    insertDataFiles = appForm.ShowParametersChecked;
                    embedAll = appForm.EmbedAllResults;
                    results = appForm.Results;

                    appForm.Dispose();
                    rerunServer.Dispose();
                }

                if (!canceled)
                {
                    // add the rerun to the document
                    List<Result> resultsToEmbed = new List<Result>();

                    //code changes for issue 1323
                    if (embedAll)
                    {
                        resultsToEmbed = EmbedAllResults(results, ref resultsToEmbed); // Function call to embed all result files into document.
                    }
                    else
                    {
                        resultsToEmbed = EmbedSingleResult(results, isImage, ref resultsToEmbed); //Function call to embed Primary result file.
                    }

                    results.Clear();
                    results = null;
                    GC.Collect();

                    resultsToEmbedArray = _controller.AddAppendixEntry(newAssociation, insertDataFiles, comment,
                        resultsToEmbed, isImage, documentKey);

                    if (isImage && !embedAll) // protected image insert
                    {
                        _controller.InsertPicture(resultsToEmbedArray[0]);
                        pa.Rerun.InsertedToAppedix = true;
                    }
                    else
                    {
                        // protected file insert
                        pa.Rerun.InsertedToAppedix = true;
                    }
                    _controller.UpdatePipelineAssociationsDisplay();
                }
            }
            catch (ConnectionFailedException)
            {
                MessageBox.Show(Resources.RERUN_CONNECTION_ERROR, Resources.GENEPATTREN_MSG_HEADER,
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
            finally
            {
                #region Clear Temporary Files

                if (resultsToEmbedArray != null)
                {
                    if (resultsToEmbedArray != null)
                    {
                        foreach (string s in resultsToEmbedArray)
                        {
                            try
                            {
                                File.Delete(s);
                            }
                            catch (IOException) { }
                        }
                    }
                }

                #endregion
            }
        }

        /// <summary>
        /// Gets the filename from image or results.
        /// </summary>
        /// <param name="isImage">if set to <c>true</c> [is image].</param>
        /// <returns></returns>
        private string GetFilenameFromImageOrResults(bool isImage)
        {
            string filename = null;
            if (isImage)
            {
                if (imageResultsViewer.ResultsList.SelectedItems.Count > 0)
                {
                    filename = imageResultsViewer.ResultsList.SelectedItems[0].Text;
                }
            }
            else
            {
                filename = GetSelectedFilename();
            }
            return filename;
        }

        /// <summary>
        /// Periodically updates data for a rerun that isn't finished.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void RerunTimer_Tick(object sender, EventArgs e)
        {

            if (handlingRerun)
            {
                return;
            }
            handlingRerun = true;
            try
            {
                RerunTimer.Enabled = false; // pause the timer to verify connection and fetch data

                PipelineAssociation pipelineAssociation = GetSelectedAssociation();
                Pipeline pipeline = GetSelectedPipeline();
                if (pipelineAssociation != null && pipeline != null)
                {
                    // don't do the update if the rerun doesn't have a connection -- someone else will provide it!
                    if (pipelineAssociation.Rerun.Job.Id > -1 && !String.IsNullOrEmpty(pipelineAssociation.Rerun.Connection.Username))
                    {
                        JobStatus status = UpdateRerunJob(pipelineAssociation);
                        if (RerunRadioButton.Checked)
                        {
                            ShowRerunParametersAndData(pipelineAssociation, pipeline, status);
                        }
                        if (status == JobStatus.Finished)
                        {
                            // don't restart the timer since we got the data
                            if (pipelineAssociation.Rerun.ShowVisualizers)
                            {
                                DisplayVisualizers();
                                // don't show visualizers again
                                pipelineAssociation.Rerun.ShowVisualizers = false;
                            }
                        }
                        else if (status == JobStatus.Processing)
                        {
                            // restart the timer to wait for job completion
                            RerunTimer.Enabled = true;
                        }

                    }
                }
            }
            catch (Exception ex)
            {
                RerunTimer.Enabled = false;
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
            handlingRerun = false;
        }

        /// <summary>
        /// Displays all visualizers for a given pipeline association/pipeline
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void DisplayVisualizers()
        {
            try
            {
                if (visualizersThread == null || !visualizersThread.IsAlive)
                {
                    visualizersThreadPipeline = GetSelectedPipeline();
                    visualizersThreadPA = GetSelectedAssociation();
                    visualizersThread = new Thread(new ThreadStart(DisplayVisualizersAsync));
                    visualizersThread.Start();
                }
            }
            catch (Exception e)
            {
                MessageForm.ShowUnhandledExceptionMessage(e);
                Trace.WriteLine("Exception occured while displaying visualizers:" + e.ToString());
            }
        }

        /// <summary>
        /// Displays the visualizers async.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void DisplayVisualizersAsync()
        {
            Trace.WriteLine("Beginning to show visualizers:" + DateTime.Now);
            PipelineAssociation pipelineAssociation = visualizersThreadPA;
            Pipeline pipeline = visualizersThreadPipeline;
            try
            {
                int index = 0;
                int stopPoint = pipelineAssociation.Rerun.Job.StopPoint;
                // get a list of result filenames we need to show the visualizers
                List<ShallowJobResult> resultsNeededForVisualizers = new List<ShallowJobResult>();
                foreach (PipelineStep ps in pipeline.Steps)
                {
                    if (ps.VisualizerData != null && ps.VisualizerData.Count > 0 && index <= stopPoint)
                    {
                        List<ShallowJobResult> resultNames = ps.GetVisualizerFilenames(pipelineAssociation.Rerun.Job.ShallowResults);
                        resultsNeededForVisualizers.AddRange(resultNames);
                    }
                    index++;
                }

                Trace.WriteLine("Getting Job Results: " + DateTime.Now);
                IGenePatternServer rerunServer = _controller.ServerBroker.GetGenePatternServerForConnection(pipelineAssociation.Rerun.Connection);
                IList<Result> finalResults = rerunServer.GetJobResults(resultsNeededForVisualizers);

                Trace.WriteLine("Done Getting Job Results: " + DateTime.Now);
               
                index = 0;
                foreach (PipelineStep ps in pipeline.Steps)
                {
                    if (ps.VisualizerData != null && ps.VisualizerData.Count > 0 && index <= stopPoint)
                    {
                        // get results
                        IList<Result> vizResults = ps.GetVisualizerSourceResults(finalResults);
                        VisualizeResult(vizResults, ps, pipelineAssociation);
                    }
                    index++;
                }
            }
            catch (Win32Exception)
            {
                MessageBox.Show(
                    "One or more visualizers could not be launched. This may be because you do not have Java installed.",
                    Resources.GENEPATTREN_MSG_HEADER, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                if (ex != null)
                {
                    if (ex.Message.Contains("This command is not allowed because the document is locked"))
                    {
                        MessageBox.Show(Resources.VISUALIZER_PROTECTION_ERROR, Resources.GENEPATTREN_MSG_HEADER,
                            MessageBoxButtons.OK, MessageBoxIcon.Information);
                        return;
                    }
                }
                MessageForm.ShowUnhandledExceptionMessage(ex);
                Trace.WriteLine("Exception displaying visualizers:" + ex.ToString());
            }
            finally
            {
                pipelineAssociation = null;
                pipeline = null;
            }
            Trace.WriteLine("Done showing visualizers:" + DateTime.Now);
        }

        /// <summary>
        /// Handles the DoubleClick event of the Outputs control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Outputs_DoubleClick(object sender, EventArgs e)
        {
            OpenResultWithDefaultApplication(false);
        }

        /// <summary>
        /// Opens the result with default application.
        /// </summary>
        /// <param name="isImage">if set to <c>true</c> [is image].</param>
        private void OpenResultWithDefaultApplication(bool isImage)
        {
            string suggestedfilename;
            byte[] data = GetSelectedBytesAndFilename(isImage, out suggestedfilename);

            if (data != null)
            {
                // open with associated application
                string path = UserDataManager.GetTempPath(string.Empty);
                if (!Directory.Exists(path))
                {
                    Directory.CreateDirectory(path);
                }
                path += suggestedfilename;

                using (FileStream fileStream = new FileStream(path, FileMode.Create, FileAccess.ReadWrite))
                {
                    using (BinaryWriter bw = new BinaryWriter(fileStream))
                    {
                        bw.Write(data);
                    }
                }

                ProcessStartInfo info = new ProcessStartInfo(path);
                info.UseShellExecute = true;
                info.ErrorDialog = true;
                info.ErrorDialogParentHandle = this.Handle;

                Process process = new Process();
                process.StartInfo = info;
                process.Start();
                Trace.WriteLine("Started process for file " + path, TraceLevel.Info.ToString());
            }

            data = null;
            GC.Collect();
            GC.WaitForPendingFinalizers();
        }

        private string IsGTROptionalFile(string commandline, Rerun rerunJob)
        {
            foreach (JobParameter param in rerunJob.Job.Parameters)
            {
                if (param.Name.EndsWith(".gtr.file") && param.ParamValue.ToString().Length == 0)
                {
                    commandline = commandline.Replace("-g<gtr.file>", string.Empty);
                }
            }

            return commandline;
        }

        /// <summary>
        /// Visualizes the result.
        /// </summary>
        /// <param name="selResult">The sel result.</param>
        /// <param name="visualizer">The visualizer.</param>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        private void VisualizeResult(IList<Result> selResult, PipelineStep visualizer, PipelineAssociation pipelineAssociation)
        {
            if (visualizer.VisualizerData.Count != selResult.Count)
            {
                return;
            }

            string lsidpath = visualizer.Lsid.Replace(":", "_") + "\\";
            string tempPath = UserDataManager.GetTempPath(String.Empty);
            //string filename = GetSelectedFilename();

            if (!Directory.Exists(tempPath))
            {
                Directory.CreateDirectory(tempPath);
            }
            tempPath += lsidpath;

            if (!Directory.Exists(tempPath))
            {
                Directory.CreateDirectory(tempPath);
            }
            Trace.WriteLine("Begin installing supporting files: " + DateTime.Now);
            string commandLine = InstallSupportingFiles(visualizer, tempPath, pipelineAssociation);
            commandLine = IsGTROptionalFile(commandLine, pipelineAssociation.Rerun);
            Trace.WriteLine("Done installing supporting files: " + DateTime.Now);
            commandLine = commandLine.Replace("\\=", "="); // remove escaped equals signs
            commandLine = commandLine.Replace("<java_flags> ", "");
            commandLine = commandLine.Replace("<path.separator>", ";");
            commandLine = commandLine.Replace("<java>", "java");

            for (int i = 0; i < visualizer.VisualizerData.Count; i++)
            {
                commandLine =
                    commandLine.Replace("<" + visualizer.VisualizerData[i].FilenameVariable + ">",
                                        "\"" + tempPath + selResult[i].Filename + "\"");
            }

            // replacing lib dir is extra tricky since we need to change <libdir>filename to "<libdir>filename" thanks to
            // spaces in paths
            string libDirToken = "<libdir>";
            while (commandLine.Contains(libDirToken))
            {
                int beginIndex = commandLine.IndexOf(libDirToken);
                int pathSeparator = commandLine.IndexOf(";", beginIndex + libDirToken.Length);
                if (pathSeparator == -1) // no separator because it's the last one; looks for a space instead
                    pathSeparator = commandLine.IndexOf(" ", beginIndex + libDirToken.Length);
                string before = commandLine.Substring(0, beginIndex);
                string after = commandLine.Substring(pathSeparator);
                string middle = commandLine.Substring(beginIndex, pathSeparator - beginIndex);
                middle = middle.Replace(libDirToken, tempPath);
                commandLine = before + "\"" + middle + "\"" + after;
            }

            string appName = commandLine.Substring(0, commandLine.IndexOf(" "));
            commandLine = commandLine.Substring(commandLine.IndexOf(" ") + 1);
            commandLine = commandLine.Replace("<dataset.filename>", "");
            commandLine = commandLine.Replace("<dataset.file>", "");

            foreach (Result result in selResult)
            {
                string tempPathFilename = tempPath + result.Filename;
                using (FileStream fileStream = new FileStream(tempPathFilename, FileMode.Create, FileAccess.ReadWrite))
                {
                    using (BinaryWriter bw = new BinaryWriter(fileStream))
                    {
                        bw.Write(result.Content);
                    }
                }
            }

            ProcessStartInfo info = new ProcessStartInfo(appName, commandLine);
            info.UseShellExecute = false;
            info.CreateNoWindow = true;
            info.LoadUserProfile = true;
            Process process = new Process();
            process.StartInfo = info;
            process.Start();

        }

        /// <summary>
        /// Installs the supporting files.
        /// </summary>
        /// <param name="visualizer">The visualizer.</param>
        /// <param name="tempPath">The temp path.</param>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        /// <returns></returns>
        private string InstallSupportingFiles(PipelineStep visualizer, string tempPath, PipelineAssociation pipelineAssociation)
        {
            // if we already have the module, then extract
            if (_controller.GetDocumentModel(documentKey).Modules.ContainsKey(visualizer.Lsid))
            {
                Trace.WriteLine("Extracting module to temp path: " + visualizer.Lsid + " at " + DateTime.Now);
                _controller.ExtractModule(visualizer.Lsid, tempPath, documentKey);
                Trace.WriteLine("Done module to temp path: " + visualizer.Lsid + " at " + DateTime.Now);
            }
            else // otherwise download it
            {
                Trace.WriteLine("Downloading and Extracting module to temp path: " + visualizer.Lsid + " at " + DateTime.Now);
                Connection rerunLocation = pipelineAssociation.Rerun.Connection.Clone() as Connection;

                IGenePatternServer linkedServer = _controller.ServerBroker.GetGenePatternServerForConnection(rerunLocation);
                using (MemoryStream moduleZip = linkedServer.DownloadTask(visualizer.Lsid, false))
                {
                    _controller.ExtractModule(moduleZip, tempPath);
                }
                Trace.WriteLine("Done Downloading and Extracting module to temp path: " + visualizer.Lsid + " at " + DateTime.Now);
            }
            // open manifest and read commandLine arg
            // TODO pathseparator?
            try
            {
                using (FileStream manifestStream = new FileStream(tempPath + "\\manifest", FileMode.Open))
                {
                    using (StreamReader sr = new StreamReader(manifestStream))
                    {
                        string line = null;
                        while ((line = sr.ReadLine()) != null)
                        {
                            string beginDelim = "commandLine=";
                            if (line.StartsWith(beginDelim))
                            {
                                return line.Substring(beginDelim.Length);
                            }
                        }
                    }
                }
            }
            catch (FileNotFoundException)
            {
                Trace.WriteLine("No manifest found for locally installed module: " + visualizer.Name);
            }
            return null;
        }

        /// <summary>
        /// Handles the CheckedChanged event of the OriginalRadioButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OriginalRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            UpdatePipelineDetails();
        }

        /// <summary>
        /// Handles the Click event of the DataSaveResultToDisk control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void DataSaveResultToDisk_Click(object sender, EventArgs e)
        {
            SaveAll saveAll = new SaveAll();
            DialogResult dialogResult = saveAll.ShowDialog(this);
            switch (dialogResult)
            {
                case DialogResult.Yes:
                    SaveAllResult();
                    break;
                case DialogResult.No:
                    SaveResult(false);
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// Saves all result.
        /// </summary>
        private void SaveAllResult()
        {
            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
            DialogResult folderDialogResult = folderBrowserDialog.ShowDialog(this);
            if (folderDialogResult == DialogResult.OK)
            {
                SaveAllResults(folderBrowserDialog.SelectedPath);
            }
        }

        /// <summary>
        /// Saves all results.
        /// </summary>
        /// <param name="folderPath">The folder path.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void SaveAllResults(string folderPath)
        {
            try
            {
                string suggestedfilename = string.Empty;
                foreach (PipelineChild pipelineChild in Outputs.Children)
                {
                    suggestedfilename = pipelineChild.Name;
                    IList<Result> downloadResult = DownloadRerunResults(suggestedfilename);
                    byte[] data = downloadResult[0].Content;

                    if (data != null)
                    {
                        suggestedfilename = Path.Combine(folderPath, suggestedfilename);
                        if (!string.IsNullOrEmpty(suggestedfilename))
                        {
                            if (!File.Exists(suggestedfilename))
                            {
                                using (FileStream fileStrean = new FileStream(suggestedfilename, FileMode.Create))
                                {
                                    using (BinaryWriter bw = new BinaryWriter(fileStrean))
                                    {
                                        bw.Write(data);
                                    }
                                }
                            }
                            else
                            {
                                DialogResult question = MessageBox.Show("Do you want to overwrite " + " " + suggestedfilename,
                                                 "OverWrite", MessageBoxButtons.YesNo,
                                                 MessageBoxIcon.Question);
                                if (question == DialogResult.Yes)
                                {
                                    using (FileStream fileStrean = new FileStream(suggestedfilename, FileMode.Create))
                                    {
                                        using (BinaryWriter bw = new BinaryWriter(fileStrean))
                                        {
                                            bw.Write(data);
                                        }
                                    }
                                }
                            }
                        }
                        data = null;
                    }
                    else
                    {
                        InformUserToSelectResult();
                    }
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Signals the rerun is present.
        /// </summary>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        internal void SignalRerunIsPresent(PipelineAssociation pipelineAssociation)
        {
            foreach (ListViewItem item in pipelineAssociationsListView.Items)
            {
                PipelineAssociation association = item.Tag as PipelineAssociation;
                if (association != null && association.LocalId == pipelineAssociation.LocalId)
                {
                    item.Selected = true;
                    if (association.Job.Id != pipelineAssociation.Job.Id)
                    {
                        association.Rerun = pipelineAssociation.Rerun;
                    }

                    if (RerunRadioButton.Checked)
                    {
                        UpdatePipelineDetails();
                    }
                    else
                    {
                        RerunRadioButton.Checked = true;
                    }
                    tabControl1.SelectedTab = DataTabPage;
                    break;
                }
            }
        }

        /// <summary>
        /// Handles the Click event of the ImageSaveToDiskButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ImageSaveToDiskButton_Click(object sender, EventArgs e)
        {
            SaveResult(true);
        }

        /// <summary>
        /// Handles the Click event of the ImageInsertIntoAppendixButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ImageInsertIntoAppendixButton_Click(object sender, EventArgs e)
        {
            if (imageResultsViewer.ResultsList.SelectedItems.Count > 0)
            {
                InsertIntoAppendix(true);
                MemoryManagement.FlushMemory();
            }
        }

        /// <summary>
        /// Handles the Click event of the DataInsertIntoAppendixButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void DataInsertIntoAppendixButton_Click(object sender, EventArgs e)
        {
            if (Outputs.SelectedChildIndex >= 0)
            {
                InsertIntoAppendix(false);
                MemoryManagement.FlushMemory();
            }
        }

        /// <summary>
        /// Handles the Click event of the ShowViewersButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void ShowViewersButton_Click(object sender, EventArgs e)
        {
            try
            {
                PipelineAssociation pipelineAssociation = GetSelectedAssociation();
                DisplayVisualizers();
                pipelineAssociation.Rerun.ShowVisualizers = false;
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the DoubleClick event of the imageResultsViewer control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void imageResultsViewer_DoubleClick(object sender, EventArgs e)
        {
            if (imageResultsViewer.ResultsList.SelectedItems.Count > 0)
            {
                OpenResultWithDefaultApplication(true);
            }
        }

        /// <summary>
        /// Downloads the job results async.
        /// </summary>
        /// <param name="association">The association.</param>
        private void DownloadJobResultsAsync(PipelineAssociation association)
        {
            string rerunKey = association.Rerun.Connection.ServerName + association.Rerun.Job.Id;
            IList<Result> results = new List<Result>();

            try
            {
                IGenePatternServer server = GetRerunServer(association);
                if (server != null)
                {
                    results = server.GetJobResults(association.Rerun.Job.Id);
                    lock (cachedResults)
                    {
                        cachedResults[rerunKey] = results;
                        server.Dispose();
                        results = null;
                        GC.Collect();
                    }
                }
            }
            catch (Exception ex)
            {
                Result result = new Result();
                results.Add(result);
                cachedResults[rerunKey] = results;
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }
    }
}
