using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Globalization;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;

namespace Microsoft.eResearch.GenePattern.Package
{
    public partial class ConfigureJob : RerunPipelineWizardControl
    {
        private const string NEW_CONNECTION_PROMPT = "New Connection ...";
        private const string GENEPATTERN_PUBLIC_SERVER = "GenePattern Public Server ...";
        private const string EMPTY_CONNECTION_PROMPT = "Select Connection ....";
        Regex emailregex = new Regex("(?<user>[^@]+)@(?<host>.+)");
        private IController controller;

        /// <summary>
        /// Initializes a new instance of the <see cref="ConfigureJob"/> class.
        /// </summary>
        /// <param name="model">The model.</param>
        /// <param name="controller">The controller.</param>
        public ConfigureJob(RerunPipelineWizardModel model, IController controller)
            : base(model as IWizardModel)
        {
            this.controller = controller;
            InitializeComponent();
        }

        /// <summary>
        /// Returns true if the control can be hidden (advanced).  returns false
        /// if the control needs some correction or input.
        /// </summary>
        /// <returns></returns>
        public override bool CanHide()
        {
            if (EmailNotificationCheckBox.Checked)
            {
                //if (!EmailAddressIsValid(EmailAddressTextBox.Text))
                //    return false;
            }
            if (RerunPipelineWizardModel.SelectedConnection == null)
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// Invalids the connection.
        /// </summary>
        /// <returns></returns>
        private bool InvalidConnection()
        {
            bool invalidConnection = false;
            if(this.ChooseConnectionComboBox1.Visible)
            {

                object selectedValue = this.ChooseConnectionComboBox1.SelectedItem;
                if (selectedValue != null)
                {
                    if (selectedValue.Equals(NEW_CONNECTION_PROMPT) || selectedValue.Equals(EMPTY_CONNECTION_PROMPT))
                        invalidConnection = true;
                }
            }

            if (this.ChooseConnectionComboBox2.Visible)
            {
                object selectedValue = this.ChooseConnectionComboBox2.SelectedItem;
                if (selectedValue != null)
                {
                    if (selectedValue.Equals(NEW_CONNECTION_PROMPT) || selectedValue.Equals(EMPTY_CONNECTION_PROMPT))
                        invalidConnection = true;
                }
            }

            return invalidConnection;

        }

        /// <summary>
        /// Called when control is shown.
        /// </summary>
        public override void OnShown()
        {
            UpdateConnectionsComboBox();
            EmailNotificationCheckBox.Checked = RerunPipelineWizardModel.DoEmailNotification;
            EmailAddressTextBox.Enabled = EmailNotificationCheckBox.Checked;
            EmailAddressTextBox.Text = RerunPipelineWizardModel.EmailAddress;
            Pipeline pipeline = RerunPipelineWizardModel.SelectedPipeline;
            if (pipeline != null)
            {
                if (pipeline.IsLinked)
                {
                    ConnectionPanel.Visible = false;
                    RunLinkedPipelinePanel.Visible = true;
                    CurrentLocationRadioButton.Checked = true; // check the button now (not design time) to store the info
                    SetVisualizersMessageForDownload(true);
                }
                else
                {
                    ConnectionPanel.Visible = true;
                    RunLinkedPipelinePanel.Visible = false;
                    SetVisualizersMessageForDownload(false);
                }

                if (pipeline.LinkConnection != null)
                {
                    string serverAddress = pipeline.LinkConnection.ShortServerInformation;
                    CurrentLocationRadioButton.Text =
                        string.Format(CultureInfo.InvariantCulture, Resources.CURRENT_LOCATION_TEXT, serverAddress);
                }

                if (pipeline.HasVisualizers) //Visualizer is enabled only if the selected stopoint is HeatMapviewer              
                {
                    ShowResults.Enabled = true;
                    ShowResults.Visible = pipeline.HasVisualizers;
                    ShowResults.Checked = RerunPipelineWizardModel.ShowVisualizers;
                }
                else
                {
                    ShowResults.Enabled = false;
                    RerunPipelineWizardModel.ShowVisualizers = false;
                }
            }           
                        
        }

        /// <summary>
        /// Called when the control is hidden
        /// </summary>
        /// <param name="delta"></param>
        /// <returns>
        /// Returns false if the hide should be canceled
        /// </returns>
        public override bool OnHide(int delta)
        {
            SetConnection();
            return CanHide();
        }

        /// <summary>
        /// Emails the address is valid.
        /// </summary>
        /// <param name="emailAddress">The email address.</param>
        /// <returns></returns>
        public bool EmailAddressIsValid(string emailAddress)
        {
            if (string.IsNullOrEmpty(emailAddress))
                return false;

            Match m = emailregex.Match(emailAddress);
            return m.Success;
        }

        #region Event Handlers

        /// <summary>
        /// Handles the SelectedIndexChanged event of the ChooseConnectionComboBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void ChooseConnectionComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {

                ComboBox comboBox = sender as ComboBox;
               
                object selectedValue = comboBox.SelectedItem;
                if (!selectedValue.Equals(NEW_CONNECTION_PROMPT) && 
                    !selectedValue.Equals(EMPTY_CONNECTION_PROMPT) &&
                    !selectedValue.Equals(GENEPATTERN_PUBLIC_SERVER))
                {
                    Connection connection = comboBox.SelectedItem as Connection;
                    if (connection != null)
                    {
                        UpdateCurrentConnection(connection);
                    }
                }
                else if (selectedValue.Equals(NEW_CONNECTION_PROMPT))
                {
                    this.controller.ShowServerConnectionForm(null, false);
                    RemoveIndexChangedEventHandler();
                    this.UpdateConnectionsComboBox();
                    AddIndexChangedEventHandler();
                    Connection currentConnection = this.controller.CurrentConnection;
                    if(currentConnection != null)
                    {
                        this.UpdateCurrentConnection(currentConnection);
                    }
                }
                else if (selectedValue.Equals(GENEPATTERN_PUBLIC_SERVER))
                {
                    this.controller.ShowGenePatternServerConnectionForm(null, false);
                    RemoveIndexChangedEventHandler();
                    this.UpdateConnectionsComboBox();
                    AddIndexChangedEventHandler();
                    Connection currentConnection = this.controller.CurrentConnection;
                    if (currentConnection != null)
                    {
                        this.UpdateCurrentConnection(currentConnection);
                    }
                }
                else //Empty Connection Prompt.. Set current connection to null
                {
                    UpdateCurrentConnection(null);
                }
                
                
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the CheckedChanged event of the EmailNotificationCheckBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void EmailNotificationCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                EmailAddressTextBox.Enabled = EmailNotificationCheckBox.Checked;
                RerunPipelineWizardModel.DoEmailNotification = EmailNotificationCheckBox.Checked;
                UpdateNextButton();
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the CheckedChanged event of the CurrentLocationRadioButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void CurrentLocationRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                if (CurrentLocationRadioButton.Checked)
                {
                    SetPipelineLinkConnection();
                    RerunPipelineWizardModel.RunInOriginalLocation = true;
                    SetVisualizersMessageForDownload(true);
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the CheckedChanged event of the ShowResults control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ShowResults_CheckedChanged(object sender, EventArgs e)
        {
            RerunPipelineWizardModel.ShowVisualizers = ShowResults.Checked;
        }

        /// <summary>
        /// Handles the CheckedChanged event of the OwnConnectionRadioButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void OwnConnectionRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                if (OwnConnectionRadioButton.Checked)
                {
                    SetLinkedButOwnConnection();
                    SetVisualizersMessageForDownload(false);
                }
                else
                {
                    ChooseConnectionComboBox2.Enabled = false;
                }
                RerunPipelineWizardModel.RunInOriginalLocation = !OwnConnectionRadioButton.Checked;
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the TextChanged event of the EmailAddressTextBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void EmailAddressTextBox_TextChanged(object sender, EventArgs e)
        {
            try
            {
                RerunPipelineWizardModel.EmailAddress = EmailAddressTextBox.Text;
                base.UpdateNextButton();

            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        #endregion

        /// <summary>
        /// Updates the connections combo box.
        /// </summary>
        private void UpdateConnectionsComboBox()
        {
            InitializeComboBoxes();
            int index = 3; //as empty connection Prompt is 0, new connection prompt is 1
            int selectedIndex = 0;
            foreach (KeyValuePair<string, Connection> connectionMapping in this.controller.UserData.Connections)
            {
                Connection connection = connectionMapping.Value;
                ChooseConnectionComboBox1.Items.Add(connection);
                ChooseConnectionComboBox2.Items.Add(connection);
                if (connection.Equals(this.controller.CurrentConnection))
                {
                    selectedIndex = index;
                    this.RerunPipelineWizardModel.SelectedConnection = connection;
                }
                index++;
            }

            if (ChooseConnectionComboBox1.Items.Count > 0)
            {
                ChooseConnectionComboBox1.SelectedIndex = selectedIndex;
            }
            if (ChooseConnectionComboBox2.Items.Count > 0
                && ChooseConnectionComboBox2.SelectedItem == null)
            {
                ChooseConnectionComboBox2.SelectedIndex = selectedIndex;
            }
        }

        /// <summary>
        /// Initializes the combo boxes.
        /// </summary>
        private void InitializeComboBoxes()
        {
            ChooseConnectionComboBox1.Items.Clear();
            ChooseConnectionComboBox2.Items.Clear();

            this.ChooseConnectionComboBox1.Items.Add(EMPTY_CONNECTION_PROMPT);
            this.ChooseConnectionComboBox2.Items.Add(EMPTY_CONNECTION_PROMPT);
            this.ChooseConnectionComboBox1.Items.Add(NEW_CONNECTION_PROMPT);
            this.ChooseConnectionComboBox2.Items.Add(NEW_CONNECTION_PROMPT);
            this.ChooseConnectionComboBox1.Items.Add(GENEPATTERN_PUBLIC_SERVER);
            this.ChooseConnectionComboBox2.Items.Add(GENEPATTERN_PUBLIC_SERVER);
        }

        /// <summary>
        /// Sets the visualizers message for download.
        /// </summary>
        /// <param name="download">if set to <c>true</c> [download].</param>
        private void SetVisualizersMessageForDownload(bool download)
        {
            if (download)
                ShowResults.Text = Resources.SHOW_RESULTS_LINKED;
            else
                ShowResults.Text = Resources.SHOW_RESULTS_NOT_LINKED;
        }

        /// <summary>
        /// Sets the connection.
        /// </summary>
        private void SetConnection()
        {
            Pipeline pipeline = RerunPipelineWizardModel.SelectedPipeline;
            if (pipeline != null)
            {
                if (pipeline.IsLinked)
                {
                    if (CurrentLocationRadioButton.Checked)
                    {
                        SetPipelineLinkConnection();
                    }
                    else
                    {
                        SetLinkedButOwnConnection();
                    }
                }
                else
                {
                    UpdateCurrentConnection(ChooseConnectionComboBox1.SelectedItem as Connection);
                }
            }
        }

        /// <summary>
        /// Adds the index changed event handler.
        /// </summary>
        private void AddIndexChangedEventHandler()
        {
            this.ChooseConnectionComboBox1.SelectedIndexChanged += new EventHandler(this.ChooseConnectionComboBox_SelectedIndexChanged);
            this.ChooseConnectionComboBox2.SelectedIndexChanged += new EventHandler(this.ChooseConnectionComboBox_SelectedIndexChanged);
        }

        /// <summary>
        /// Removes the index changed event handler.
        /// </summary>
        private void RemoveIndexChangedEventHandler()
        {
            this.ChooseConnectionComboBox1.SelectedIndexChanged -= new EventHandler(this.ChooseConnectionComboBox_SelectedIndexChanged);
            this.ChooseConnectionComboBox2.SelectedIndexChanged -= new EventHandler(this.ChooseConnectionComboBox_SelectedIndexChanged);
        }

        /// <summary>
        /// Sets the pipeline link connection.
        /// </summary>
        private void SetPipelineLinkConnection()
        {
            Pipeline pipeline = RerunPipelineWizardModel.SelectedPipeline;
            if (pipeline != null && pipeline.IsLinked)
            {
                Connection connection = CheckExistingConnectionForMatch(pipeline);
                if(connection == null)
                    UpdateCurrentConnection(pipeline.LinkConnection); 
                else
                    UpdateCurrentConnection(connection); 
            }
        }

        /// <summary>
        /// Checks the existing connection for match.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        /// <returns></returns>
        private Connection CheckExistingConnectionForMatch(Pipeline pipeline)
        {
            Connection matchingConnection = null;
            //See if we have a connection for the same linked server and use it 
            //or Download pipelines from linked server to current connection
            foreach (KeyValuePair<string, Connection> connectionMapping in this.controller.UserData.Connections)
            {
                Connection connection = connectionMapping.Value;
                if (connection.ServerName.Equals(pipeline.LinkConnection.ServerName) &&
                        (connection.Port.Equals(pipeline.LinkConnection.Port)))
                {
                    matchingConnection = connection;
                    break;
                }
            }
            return matchingConnection;
        }

        /// <summary>
        /// Sets the linked but own connection.
        /// </summary>
        private void SetLinkedButOwnConnection()
        {
            Connection connection = ChooseConnectionComboBox2.SelectedItem as Connection;
            UpdateCurrentConnection(connection);
            ChooseConnectionComboBox2.Enabled = true;
        }

        /// <summary>
        /// Updates the current connection.
        /// </summary>
        /// <param name="connection">The connection.</param>
        private void UpdateCurrentConnection(Connection connection)
        {
            RerunPipelineWizardModel.SelectedConnection = connection;
            this.UpdateNextButton();
        }
    }
}
