using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.ScientificAddIn.Common;

namespace Microsoft.eResearch.GenePattern.Package
{
    public partial class DashboardUserControl : UserControl, ITaskPaneControl
    {
        private IController _controller;
        private List<PipelineAssociation> pipelineAssociationList;

        /// <summary>
        /// Initializes a new instance of the <see cref="DashboardUserControl"/> class.
        /// </summary>
        public DashboardUserControl()
        {
            InitializeComponent();
            pipelineAssociationList = new List<PipelineAssociation>();
        }

        /// <summary>
        /// Displays the pipeline associations.
        /// </summary>
        /// <param name="pipelineAssociations">The pipeline associations.</param>
        /// <param name="contentItems">The content items.</param>
        public void DisplayPipelineAssociations(ICollection<PipelineAssociation> pipelineAssociations, ICollection<ContentItem> contentItems)
        {
           Trace.WriteLine("Dashboard - Displaying Pipeline Associations");
            pipelineAssociationList.Clear();
            foreach (PipelineAssociation pa in pipelineAssociations)
            {
                if (!pipelineAssociationList.Contains(pa))
                {
                    pipelineAssociationList.Add(pa);
                }
            }
            UpdatePipelineAssociationsList();
            UpdateDocumentContents(contentItems);
            ScrollToImage.Enabled = pipelineAssociations.Count == 0 ? false : true;
        }

        /// <summary>
        /// Updates the document contents.
        /// </summary>
        /// <param name="contentItems">The content items.</param>
        private void UpdateDocumentContents(ICollection<ContentItem> contentItems)
        {
            if (contentItems == null)
            {
                documentContents1.ContentItems = null;
            }
            else
            {

                ContentItem[] items = new ContentItem[contentItems.Count];
                contentItems.CopyTo(items, 0);
                documentContents1.ContentItems = items;
            }
        }

        /// <summary>
        /// Updates the pipeline associations list.
        /// </summary>
        private void UpdatePipelineAssociationsList()
        {
           Trace.WriteLine("Dashboard - Updating Pipeline Associations List");
            AssociationImages.Items.Clear();
            Thumbnails.Images.Clear();

            foreach (PipelineAssociation pa in pipelineAssociationList)
            {
                AddAssociation(pa, pa.ThumbnailImage);
            }
        }

        /// <summary>
        /// Adds the association.
        /// </summary>
        /// <param name="association">The association.</param>
        /// <param name="thumbnail">The thumbnail.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.Int32.ToString")]
        private void AddAssociation(PipelineAssociation association, Image thumbnail)
        {
           Trace.WriteLine("Dashboard - Adding Association");
            foreach (ListViewItem item in AssociationImages.Items)
            {
                PipelineAssociation pa = item.Tag as PipelineAssociation;
                if (pa.PipelineDocumentImage.Equals(association.PipelineDocumentImage))
                {
                    // the image is already used, update its label
                    item.Text += ", " + association.Name;
                    item.ToolTipText = item.Text;
                    return;
                }
            }

            // peter's temp workaround
            if (thumbnail == null) return;

            // if we get here it's a new image
            Thumbnails.Images.Add(Thumbnails.Images.Count.ToString(), thumbnail);
            ListViewItem newItem = AssociationImages.Items.Add(association.Name);
            newItem.ImageKey = (Thumbnails.Images.Count - 1).ToString();
            newItem.Tag = association;
            newItem.ToolTipText = newItem.Text;
        }

        /// <summary>
        /// Sets the set controller.
        /// </summary>
        /// <value>The set controller.</value>
        public IController Controller
        {
            get
            {
                return _controller;
            }
            set
            {
                _controller = value;
            }
        }

        #region ITaskPaneControl Members

        /// <summary>
        /// Sets the dock position.
        /// </summary>
        /// <param name="dockPosition">The dock position.</param>
        public void SetDockPosition(TaskPaneDockPosition dockPosition)
        {
            if ((dockPosition & (TaskPaneDockPosition.Left | TaskPaneDockPosition.Right | TaskPaneDockPosition.Floating)) != 0)
            {
                SetVertical(true);
            }
            else
            {
                SetVertical(false);
            }
        }

        private void SetVertical(bool controlIsVertical)
        {
            this.SuspendLayout();
            int third;
            if (controlIsVertical)
            {
                third = this.Height / 3;
            }
            else
            {

                third = this.Width / 3;
            }

            // splitcontainter orientation is opposite of the control's orientation
            splitContainer1.Orientation = controlIsVertical ? Orientation.Horizontal : Orientation.Vertical;
            splitContainer1.SplitterDistance = third;


            this.ResumeLayout();
        }

        #endregion

        /// <summary>
        /// Handles the Click event of the ScrollToImage control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ScrollToImage_Click(object sender, EventArgs e)
        {
            if (AssociationImages.SelectedItems.Count > 0)
            {
                PipelineAssociation pa = AssociationImages.SelectedItems[0].Tag as PipelineAssociation;
                _controller.ScrollToAssociation(pa);
            }
        }

        /// <summary>
        /// Handles the DoubleClick event of the AssociationImages control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void AssociationImages_DoubleClick(object sender, EventArgs e)
        {
            if (AssociationImages.SelectedItems.Count > 0)
            {
                PipelineAssociation pa = AssociationImages.SelectedItems[0].Tag as PipelineAssociation;
                _controller.ScrollToAssociation(pa);
            }
        }
    }
}
