using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Reflection;
using System.Text;
using System.Windows.Forms;
using System.Globalization;

namespace Microsoft.eResearch.GenePattern.Package
{
    public partial class DocumentContents : UserControl
    {
        private List<ContentItem> contentItems = new List<ContentItem>();
        private NumberFormatInfo infoNoDecimals = CultureInfo.InvariantCulture.NumberFormat.Clone() as NumberFormatInfo;
        private NumberFormatInfo infoTwoDecimals = CultureInfo.InvariantCulture.NumberFormat.Clone() as NumberFormatInfo;

        /// <summary>
        /// Initializes a new instance of the <see cref="DocumentContents"/> class.
        /// </summary>
        public DocumentContents()
        {
            InitializeComponent();
            this.infoNoDecimals.NumberDecimalDigits = 0;
            this.infoTwoDecimals.NumberDecimalDigits = 2;

            UpdateCall();
        }

        /// <summary>
        /// Gets or sets the content items.
        /// </summary>
        /// <value>The content items.</value>
        [Browsable(false)]
        public ContentItem[] ContentItems
        {
            get { return contentItems.ToArray(); }
            set
            {
                contentItems.Clear();
                if(value != null)
                   contentItems.AddRange(value);
                UpdateCall();
            }
        }

        /// <summary>
        /// Updates the call.
        /// </summary>
        private void UpdateCall()
        {
            DocumentContentsListView.BeginUpdate();

            DocumentContentsListView.Items.Clear();

            long totalSize = 0;
            if (contentItems != null)
            {
                foreach (ContentItem contentItem in contentItems)
                {
                    ListViewItem listViewItem = DocumentContentsListView.Items.Add(contentItem.Name);
                    switch(contentItem.ContentItemType)
                    {
                        case ContentItemType.Module:
                            listViewItem.ImageKey = "module";
                            break;
                        case ContentItemType.Vizualizer:
                            listViewItem.ImageKey = "visualizer";
                            break;
                        case ContentItemType.Pipeline:
                            listViewItem.ImageKey = "pipeline";
                            break;
                        case ContentItemType.InputFile:
                            listViewItem.ImageKey = "inputFile";
                            break;
                        case ContentItemType.ResultFile:
                            listViewItem.ImageKey = "resultFile";
                            break;
                    }
                    listViewItem.SubItems.Add(string.Join(", ",contentItem.BelongsTo.ToArray()));

                    //fix for BUG 1212
                    //Covert size in bytes to KB
                    decimal kbSize = Math.Ceiling(Convert.ToDecimal(contentItem.Size) / Convert.ToDecimal(1024));
                    listViewItem.SubItems.Add(kbSize.ToString("N", infoNoDecimals));
                    totalSize += contentItem.Size;
                }
            }
            
            if(DocumentContentsListView.Items.Count == 0)
            {
                string message = Resources.DOCUMENT_EMPTY;
                DocumentContentsListView.Items.Add(message);
            }

            DocumentContentsListView.EndUpdate();

            if (totalSize > 0)
            {
                TotalSizeLabel.Visible = true;
                SizeLabel.Visible = true;

                //fix for BUG 1212
                //Convert size in bytes to MB
                double totalSizeMB = Convert.ToDouble(totalSize) / Convert.ToDouble(1024 * 1024);
                SizeLabel.Text = totalSizeMB.ToString("F", infoTwoDecimals) + " " + Resources.CONTENT_SIZE_UNIT;
            }
            else
            {
                TotalSizeLabel.Visible = false;
                SizeLabel.Visible = false;
            }

        }

        /// <summary>
        /// Handles the Click event of the AboutButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void AboutButton_Click(object sender, EventArgs e)
        {
            //changes made by Sukhman on 08/07/2008
            MessageBox.Show(Assembly.GetExecutingAssembly().FullName, "GenePattern Reproducible Research Document Add-In",
                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
    }
}
