using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.IO;
using System.Text;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// User control for showing the parameters of the pipeline association
    /// </summary>
    public partial class EditParameters : RerunPipelineWizardControl
    {
        /// <summary>
        /// used to store the job parameter of the last render
        /// for comparison, to avoid updating the tablelayoutpanel
        /// </summary>
        private JobParameterList lastJobParameterList;
        private string currentDirectory = string.Empty;
       

        /// <summary>
        /// Initializes a new instance of the <see cref="ShowParameters"/> class.
        /// </summary>
        /// <param name="model">The model.</param>
        public EditParameters(RerunPipelineWizardModel model)
            : base(model)
        {
            InitializeComponent();
        }

        private void UpdateCall()
        {
            Job job = RerunPipelineWizardModel.Job;
            JobParameterList currentJobParameterList = GetJobParameterList(job);

            if (JobParameterList.Equals(currentJobParameterList, lastJobParameterList))
                return;

            if (!string.IsNullOrEmpty(job.PipelineLsid))
            {
                Pipeline pipeline = RerunPipelineWizardModel.Pipelines[job.PipelineLsid];
                if (pipeline.Parameters.Count > 0)
                {
                    ResetButton.Visible = true;
                    PopulateParameters(job, pipeline);
                }
                else
                {
                    ResetButton.Visible = false;
                    ShowNoParameters();
                }
                SetLastJobParameterList(job);
            }
        }

        private void ShowNoParameters()
        {
            tableLayoutPanel1.SuspendLayout();
            tableLayoutPanel1.Controls.Clear();

            tableLayoutPanel1.RowCount = 1;
            Label labelControl = new Label();
            labelControl.Text = Resources.NO_PARAMETERS;
            tableLayoutPanel1.Controls.Add(labelControl, 0, 0);
            tableLayoutPanel1.SetColumnSpan(labelControl, 2);

            tableLayoutPanel1.ResumeLayout();
        }

        private void PopulateParameters(Job job, Pipeline pipeline)
        {
            tableLayoutPanel1.SuspendLayout();
            tableLayoutPanel1.Controls.Clear();
            tableLayoutPanel1.RowCount = pipeline.Parameters.Count;

            for (int j = 0; j < pipeline.Parameters.Count; j++)
            {
                PipelineParameter pipelineParameter = pipeline.Parameters[j];
                Label nameControl = new Label();
                tableLayoutPanel1.Controls.Add(nameControl, 0, j);
                nameControl.Left = 4;
                Padding padding = new Padding(nameControl.Margin.All);
                padding.Top = 8;
                nameControl.Margin = padding;
                nameControl.Anchor = AnchorStyles.Left | AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Bottom;
                nameControl.Text = pipelineParameter.Optional ? pipelineParameter.ShortName : pipelineParameter.ShortName + "*";
                nameControl.AutoSize = true;
                toolTip1.SetToolTip(nameControl, pipelineParameter.Description);

                Control valueControl = null;
                JobParameter jobParameter = job.GetParameter(pipelineParameter.Name);
                if (pipelineParameter.Type == ParameterType.String)
                {
                    TextBox textBox = new TextBox();
                    textBox.TextChanged += new EventHandler(valueControl_TextChanged);
                    if (jobParameter != null && jobParameter.ParamValue != null)
                    {
                        textBox.Text = jobParameter.ParamValue.ToString();
                    }
                    valueControl = textBox;
                    valueControl.Left = 4;
                    ConfigureControlLayout(pipelineParameter, valueControl);
                }

                if (pipelineParameter.Type == ParameterType.StringCollection)
                {
                    ComboBox comboBox = new ComboBox();
                    comboBox.SelectedIndexChanged += new EventHandler(comboBox_SelectedIndexChanged);
                    comboBox.DropDownStyle = ComboBoxStyle.DropDownList;
                    comboBox.Items.Clear();
                    int index = 0;
                    foreach (Choice choice in pipelineParameter.Choices)
                    {
                        comboBox.Items.Add(choice);

                        if (jobParameter != null 
                            && jobParameter.ParamValue != null 
                            && choice.Value == jobParameter.ParamValue.ToString())
                        {
                            comboBox.SelectedIndex = index;
                        }
                        index++;
                    }
                    comboBox.ValueMember = "Value";
                    comboBox.DisplayMember = "Text";
                    valueControl = comboBox;
                    ConfigureControlLayout(pipelineParameter, valueControl);
                }

                if (pipelineParameter.Type == ParameterType.InputFile)
                {
                    InputFileParameterUserControl fileParameterUserControl =
                        new InputFileParameterUserControl();
                    fileParameterUserControl.ControlId = j;
                    
                    fileParameterUserControl.TextChanged += new EventHandler(filenameChanged);
                    fileParameterUserControl.OpenFileDialogShown += new EventHandler<OpenFileDialogShownEventArgs>(OpenFileDialogShown);
                    fileParameterUserControl.OpenFileDialogClosed += new EventHandler<OpenFileDialogClosedEventArgs>(OpenFileDialogClosed);

                    if (jobParameter != null && jobParameter.ParamValue != null)
                    {
                        if (jobParameter.ParamValue is BaseFile)
                        {
                            fileParameterUserControl.IsFile = true;
                        }
                        else
                        {
                            fileParameterUserControl.IsFile = false;
                        }
                        fileParameterUserControl.Text = jobParameter.ParamValue.ToString();
                    }
                    valueControl = fileParameterUserControl;
                    ConfigureControlLayout(pipelineParameter, valueControl);
                }
                tableLayoutPanel1.Controls.Add(valueControl, 1, j);
            }
            tableLayoutPanel1.ResumeLayout();
        }

        private static void ConfigureControlLayout(PipelineParameter pipelineParameter, Control valueControl)
        {
            valueControl.Anchor = AnchorStyles.Left | AnchorStyles.Top | AnchorStyles.Right;
            valueControl.Tag = pipelineParameter;
        }

        private static JobParameterList GetJobParameterList(Job job)
        {
            JobParameterList result = new JobParameterList();
            if (job != null)
            {
                foreach (JobParameter jobParameter in job.Parameters)
                {
                    result.JobParameters.Add(jobParameter);
                }
            }
            return result;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void filenameChanged(object sender, EventArgs e)
        {
            try
            {
                InputFileParameterUserControl inputFileParameterUserControl = sender as InputFileParameterUserControl;
                if (inputFileParameterUserControl == null)
                    return;

                Job job = RerunPipelineWizardModel.Job;
                PipelineParameter pipelineParameter = inputFileParameterUserControl.Tag as PipelineParameter;
                if (pipelineParameter != null)
                {
                    JobParameter jobParameter = job.GetParameter(pipelineParameter.Name);
                    if (string.IsNullOrEmpty(inputFileParameterUserControl.Text))
                    {
                        jobParameter.ParamValue = null;
                    }
                    else
                    {
                        if (inputFileParameterUserControl.IsFile)
                        {
                            LocalFile localFile = new LocalFile();
                            localFile.FullPath = inputFileParameterUserControl.Text;
                            localFile.Filename = Path.GetFileName(localFile.FullPath);
                            jobParameter.ParamValue = localFile;
                        }
                        else
                        {
                            try
                            {
                                jobParameter.ParamValue = new SerializableUri(inputFileParameterUserControl.Text);
                            }
                            catch (UriFormatException)
                            {
                                jobParameter.ParamValue = inputFileParameterUserControl.Text;
                                MessageBox.Show("Invalid URI. Please enter valid URI.", Resources.GENEPATTREN_MSG_HEADER,
                                                                           MessageBoxButtons.OK, MessageBoxIcon.Information);
                            }
                        }
                    }
                }
                UpdateNextButton();
                SetLastJobParameterList(job);

            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        private void SetLastJobParameterList(Job job)
        {
            JobParameterList jobParameterList = GetJobParameterList(job);
            lastJobParameterList = jobParameterList.Clone() as JobParameterList;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        void comboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                ComboBox comboBox = sender as ComboBox;
                if (comboBox == null)
                    return;

                Job job = RerunPipelineWizardModel.Job;
                PipelineParameter pipelineParameter = comboBox.Tag as PipelineParameter;
                if (pipelineParameter != null)
                {
                    JobParameter jobParameter = job.GetParameter(pipelineParameter.Name);
                    if (comboBox.SelectedItem != null)
                    {
                        jobParameter.ParamValue = ((Choice)comboBox.SelectedItem).Value;
                    }
                    else
                    {
                        jobParameter.ParamValue = null;
                    }
                    UpdateNextButton();
                }
                SetLastJobParameterList(job);

            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        void valueControl_TextChanged(object sender, EventArgs e)
        {
            try
            {
                TextBox textBox = sender as TextBox;
                if (textBox == null)
                    return;

                Job job = RerunPipelineWizardModel.Job;
                PipelineParameter pipelineParameter = textBox.Tag as PipelineParameter;
                if (pipelineParameter != null)
                {
                    JobParameter jobParameter = job.GetParameter(pipelineParameter.Name);
                    jobParameter.ParamValue = textBox.Text;
                }
                UpdateNextButton();
                SetLastJobParameterList(job);

            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Called when control is shown.
        /// </summary>
        public override void OnShown()
        {
            UpdateCall();
        }

        /// <summary>
        /// Returns true if the control can be hidden (advanced).  returns false
        /// if the control needs some correction or input.
        /// </summary>
        /// <returns></returns>
        public override bool CanHide()
        {
            return RerunPipelineWizardModel.AreParametersSet;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void ResetButton_Click(object sender, EventArgs e)
        {
            try
            {
                RerunPipelineWizardModel.ResetParameters();
                UpdateCall();
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Opens the file dialog shown.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="Microsoft.eResearch.GenePattern.Common.OpenFileDialogShownEventArgs"/> instance containing the event data.</param>
        private void OpenFileDialogShown(object sender, OpenFileDialogShownEventArgs e)
        {
            e.IntialDirectory = currentDirectory;
        }

        /// <summary>
        /// Opens the file dialog closed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="Microsoft.eResearch.GenePattern.Common.OpenFileDialogClosedEventArgs"/> instance containing the event data.</param>
        private void OpenFileDialogClosed(object sender, OpenFileDialogClosedEventArgs e)
        {
            currentDirectory = e.CurrentDirectory;
        }
    }

    class JobParameterList : DomainBase
    {
        private List<JobParameter> jobParameters = new List<JobParameter>();

        public List<JobParameter> JobParameters
        {
            get { return jobParameters; }
        }
    }
}