using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using System.Globalization;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// Class Image Viewer
    /// </summary>
    public partial class ImageResultsViewer : UserControl
    {
        private static string[] imageThumbnails = { "gif", "jpg", "bmp", "png", "jpeg", "tiff", "tif", "eps" };
        private Dictionary<string, Image> fullsizeImages = new Dictionary<string, Image>();
        private ListViewItem selectedItem;

        /// <summary>
        /// Initializes a new instance of the <see cref="ImageResultsViewer"/> class.
        /// </summary>
        public ImageResultsViewer()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Displays the embedded results.
        /// </summary>
        /// <param name="association">The association.</param>
        /// <param name="dataStore">The data store.</param>
        public void DisplayEmbeddedResults(PipelineAssociation association, IDataStore dataStore)
        {
            IList<Result> results = new List<Result>();
            foreach (InternalFile file in association.EmbeddedResults)
            {
                Result r = new Result();
                if (IsImageFile(file.Filename))
                {
                    r.Filename = file.Filename;
                    r.Content = dataStore.GetFile(file.Guid).ToArray();

                    results.Add(r);
                }

            }
            ShowImagesFromResults(results);
        }

        /// <summary>
        /// Displays the rerun results.
        /// </summary>
        /// <param name="results">The results.</param>
        public void DisplayRerunResults(IList<Result> results)
        {
            if (results == null)
            {
                ResultsList.Enabled = false;
                return;
            }

            ResultsList.Enabled = true;

            ShowImagesFromResults(results);
        }

        /// <summary>
        /// Displays the original results.
        /// </summary>
        /// <param name="association">The association.</param>
        public void DisplayOriginalResults(PipelineAssociation association)
        {
            ShowImagesFromResults(association.Job.ShallowResults);
        }

        /// <summary>
        /// Shows the images from results.
        /// </summary>
        /// <param name="results">The results.</param>
        private void ShowImagesFromResults(IList<Result> results)
        {
            ResultsList.Items.Clear();
            ImageList.Images.Clear();
            fullsizeImages.Clear();
            int index = 1;
            Image image = null;
            Stream imageStream = null;

            foreach (Result result in results)
            {
                bool isImage = IsImageFile(result.Filename);

                // make a thumbnail or something here
                if (isImage)
                {
                    ListViewItem item = new ListViewItem(result.Filename);
                    item.Tag = result;
                    try
                    {
                        if (!string.Equals(Path.GetExtension(result.Filename), ".eps"))
                        {
                            imageStream = new MemoryStream(result.Content);
                            item.ImageKey = index.ToString();
                            image = Image.FromStream(imageStream);
                        }
                        else
                        {
                            item.ImageKey = index.ToString();
                            image = Resources.NoPreviewAvailable;
                        }
                        if (image != null)
                        {
                            fullsizeImages.Add(index.ToString(), image);
                            Image.GetThumbnailImageAbort dummyCallback = new Image.GetThumbnailImageAbort(ThumbnailCallback);
                            Image thumb = GetProportionedThumbnail(image, 64, dummyCallback);
                            ImageList.Images.Add(index.ToString(), thumb);
                            ResultsList.Items.Add(item);
                            index++;

                            #region Release Resources

                            if (image != null)
                                image.Dispose();

                            if (imageStream != null)
                                imageStream.Dispose();

                            #endregion
                        }
                        else
                        {
                            throw new Exception("image is null");
                        }
                    }
                    catch (Exception)
                    {
                        throw;
                    }
                }
            }
        }

        /// <summary>
        /// Shows the images from results.
        /// </summary>
        /// <param name="results">The results.</param>
        private void ShowImagesFromResults(IList<ShallowJobResult> results)
        {
            ResultsList.Items.Clear();
            ImageList.Images.Clear();
            fullsizeImages.Clear();
            int index = 1;
            foreach (ShallowJobResult result in results)
            {

                // make a thumbnail or something here
                if (result.ThumbnailImage != null)
                {
                    ResultsList.Enabled = true;
                    ListViewItem item = new ListViewItem(result.Filename);
                    item.Tag = result;

                    item.ImageKey = index.ToString();
                    item.ToolTipText = result.Filename;
                    Image image = result.ThumbnailImage;
                    fullsizeImages.Add(index.ToString(), image);

                    Image thumb = result.ThumbnailImage;
                    ImageList.Images.Add(index.ToString(), thumb);

                    ResultsList.Items.Add(item);
                    index++;

                }
            }
        }

        /// <summary>
        /// Determines whether [is image file] [the specified filename].
        /// </summary>
        /// <param name="filename">The filename.</param>
        /// <returns>
        /// 	<c>true</c> if [is image file] [the specified filename]; otherwise, <c>false</c>.
        /// </returns>
        public static bool IsImageFile(string filename)
        {
            if (filename == null)
            {
                return false;
            }
            // is the result an image?
            bool isImage = false;
            int firstDot = filename.LastIndexOf('.');
            if (firstDot > 0)
            {
                string extension = filename.Substring(firstDot + 1).ToLower(CultureInfo.InvariantCulture);
                foreach (string s in imageThumbnails)
                {
                    if (string.Compare(extension, s, true, CultureInfo.InvariantCulture) == 0)
                    {
                        isImage = true;
                    }
                }
                return isImage;
            }
            return false;
        }

        #region ImageManipulation
        /// <summary>
        ///  this is a duplicate of a function in ThisAddIn.cs.  Sigh :(
        /// </summary>        
        public static Image GetProportionedThumbnail(Image img, int maxPixels, Image.GetThumbnailImageAbort thumbnailCallback)
        {
            if (img == null)
                return null;

            float scaleFactor = 1;
            if (img.Width > img.Height)
            {
                if (img.Width > maxPixels)
                {
                    scaleFactor = (float)maxPixels / (float)img.Width;
                }
            }
            else
            {
                if (img.Height > maxPixels)
                {
                    scaleFactor = (float)maxPixels / (float)img.Height;
                }
            }
            int newH = (int)(scaleFactor * img.Height);
            int newW = (int)(scaleFactor * img.Width);

            //if the newW is zero after manipulation then we need to set its value to maxPixels
            // else img.GetThumbnailImage will through an expection for zero width
            if (newW == 0)
            {
                newW = maxPixels;
            }
            //if the newH is zero after manipulation then we need to set its value to maxPixels
            // else img.GetThumbnailImage will through an expection for zero height
            if (newH == 0)
            {
                newH = maxPixels;
            }

            if (img.Width > img.Height && img.Width > maxPixels)
            {
                newW = maxPixels; // can't trust floating point stuff, something newH is maxPixels-1
            }
            else if (img.Height > maxPixels)
            {
                newH = maxPixels;
            }

            Image thumbnailImg = img.GetThumbnailImage(newW, newH, thumbnailCallback, IntPtr.Zero);
            Bitmap thumbnail = new Bitmap(thumbnailImg);
            int thumbSize = maxPixels;

            // if it's not square, we need to add some whitespace padding since
            // the display will stretch it into a square
            if (thumbnail.Width < thumbSize || thumbnail.Height < thumbSize)
            {
                Bitmap paddedThumb = new Bitmap(thumbSize, thumbSize);
                using (Graphics g = Graphics.FromImage(paddedThumb))
                {
                    g.FillRectangle(Brushes.White, 0, 0, thumbSize, thumbSize);

                    int startX = thumbnailImg.Width >= thumbSize ? 0 : (thumbSize - thumbnailImg.Width) / 2;
                    int startY = thumbnailImg.Height >= thumbSize ? 0 : (thumbSize - thumbnailImg.Height) / 2;
                    g.DrawImage(thumbnailImg, startX, startY, thumbnail.Width, thumbnail.Height);
                    thumbnail = paddedThumb;
                }
            }
            return thumbnail;
        }
        /// <summary>
        /// Dummy method used in thumbnail creation
        /// </summary>
        /// <returns>True if the thumbnail should be canceled</returns>
        public bool ThumbnailCallback()
        {
            return false;
        }
        #endregion

        /// <summary>
        /// Gets the selected full size image.
        /// </summary>
        /// <returns></returns>
        internal Image GetSelectedFullSizeImage()
        {
            if (selectedItem != null)
            {
                return fullsizeImages[selectedItem.ImageKey];
            }
            return null;
        }

        /// <summary>
        /// Handles the DoubleClick event of the ResultsList control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ResultsList_DoubleClick(object sender, EventArgs e)
        {
            OnDoubleClick(e);
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the ResultsList control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ResultsList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (ResultsList.SelectedItems.Count > 0)
            {
                selectedItem = ResultsList.SelectedItems[0];
            }
        }
    }
}
