using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.IO;
using System.Text;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// User control for specifying a file name or a url
    /// </summary>
    public partial class InputFileParameterUserControl : UserControl
    {
        private string lastUrl;
        private string lastFileName;
        private string defaultDirectory;
        private int controlId = 0;
    
        /// <summary>
        /// Event Handler for Open File Dialog Shown
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
        public EventHandler<OpenFileDialogShownEventArgs> OpenFileDialogShown;

        /// <summary>
        /// Event Handler for Open File Dialog Closed
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
        public EventHandler<OpenFileDialogClosedEventArgs> OpenFileDialogClosed;

        /// <summary>
        /// Initializes a new instance of the <see cref="InputFileParameterUserControl"/> class.
        /// </summary>
        public InputFileParameterUserControl()
        {
            InitializeComponent();
            defaultDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
        }

        /// <summary>
        /// Gets or sets the control id.
        /// </summary>
        /// <value>The control id.</value>
        public int ControlId
        {
            get { return controlId; }
            set { controlId = value; }
        }

        /// <summary>
        /// </summary>
        /// <value></value>
        public override string Text
        {
            get
            {
                return FileName.Text;
            }
            set
            {
                FileName.Text = value;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating the specified text is a file or url.
        /// </summary>
        /// <value><c>true</c> if this instance is file; otherwise, <c>false</c> and thus a url.</value>
        public bool IsFile
        {
            get { return fileRadioButton.Checked; }
            set
            {
                fileRadioButton.Checked = value;
                urlRadioButton.Checked = !value;
            }
        }

        /// <summary>
        /// Handles the Click event of the BrowseButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void BrowseButton_Click(object sender, EventArgs e)
        {
            try
            {
                if (fileRadioButton.Checked)
                {
                    ChooseFile();
                }
                else
                {
                    SpecifyUrl();
                }

            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Specifies the URL.
        /// </summary>
        private void SpecifyUrl()
        {
            using (SpecifyUrlForm specifyUrlForm = new SpecifyUrlForm())
            {
                specifyUrlForm.GetServerUri = FileName.Text;
                if (specifyUrlForm.ShowDialog() == DialogResult.OK)
                {
                    FileName.Text = specifyUrlForm.GetServerUri;
                }
            }
        }

        /// <summary>
        /// Chooses the file.
        /// </summary>
        private void ChooseFile()
        {
            openFileDialog1.InitialDirectory = OnOpenFileDialogShown();
            if (DialogResult.OK == openFileDialog1.ShowDialog())
            {
                FileName.Text = openFileDialog1.FileName;
                OnOpenFileDialogClosed();
            }
        }

        /// <summary>
        /// Handles the CheckedChanged event of the fileRadioButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void fileRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                UpdateFilename();
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Updates the filename.
        /// </summary>
        private void UpdateFilename()
        {
            if (fileRadioButton.Checked)
            {
                lastUrl = FileName.Text;
                if (File.Exists(lastFileName))
                {
                    FileName.Text = lastFileName;
                }
                else
                {
                    FileName.Text = string.Empty;
                }
            }
            else
            {
                lastFileName = FileName.Text;
                if (Uri.IsWellFormedUriString(lastUrl, UriKind.Absolute))
                {
                    FileName.Text = lastUrl;
                }
                else
                {
                    FileName.Text = string.Empty;
                }

            }
        }

        /// <summary>
        /// Handles the TextChanged event of the FileName control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void FileName_TextChanged(object sender, EventArgs e)
        {
            try
            {
                if (FileName.Text.Length == 0)
                    toolTip1.SetToolTip(FileName, Resources.FILENAME_TEXTBOX_TOOLTIP);
                else
                    toolTip1.SetToolTip(FileName, FileName.Text);

                OnTextChanged(EventArgs.Empty);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Called when [open file dialog shown].
        /// </summary>
        /// <returns></returns>
        private String OnOpenFileDialogShown()
        {
            if (OpenFileDialogShown != null)
            {
                OpenFileDialogShownEventArgs eventErgs = new OpenFileDialogShownEventArgs();
                OpenFileDialogShown(this, eventErgs);
               
                if(string.IsNullOrEmpty(eventErgs.IntialDirectory))
                {
                    return defaultDirectory;
                }
                else
                {
                    return eventErgs.IntialDirectory;
                }
            }
            return string.Empty;
        }

        /// <summary>
        /// Called when [open file dialog closed].
        /// </summary>
        private void OnOpenFileDialogClosed()
        {
            if (OpenFileDialogClosed != null)
            {
                OpenFileDialogClosedEventArgs eventErgs = new OpenFileDialogClosedEventArgs();
                eventErgs.CurrentDirectory = Path.GetDirectoryName(openFileDialog1.FileName);
                OpenFileDialogClosed(this, eventErgs);
            }
        }

    }
}
