using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;

namespace Microsoft.eResearch.GenePattern.Package
{
    public partial class PipelineParameters : UserControl
    {
        private List<PipelineParameter> pipelineParameters;
        private Dictionary<string, JobParameter> name2jobParameterDictionary = new Dictionary<string, JobParameter>();

        /// <summary>
        /// Initializes a new instance of the <see cref="PipelineParameters"/> class.
        /// </summary>
        public PipelineParameters()
        {
            InitializeComponent();
            SetColumnWidths();
            UpdateCall();
        }

        /// <summary>
        /// Sets the parameters.
        /// </summary>
        /// <param name="pipelineParameterList">The pipeline parameter list.</param>
        /// <param name="jobParameters">The job parameters.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public void SetParameters(List<PipelineParameter> pipelineParameterList,
                                  List<JobParameter> jobParameters)
        {
            this.pipelineParameters = pipelineParameterList;
            this.name2jobParameterDictionary.Clear();
            if (jobParameters != null)
            {
                foreach (JobParameter jobParameter in jobParameters)
                {
                    name2jobParameterDictionary.Add(jobParameter.Name, jobParameter);
                }
            }
            UpdateCall();
        }

        /// <summary>
        /// Updates the call.
        /// </summary>
        private void UpdateCall()
        {
            ParametersListView.Items.Clear();

            if (pipelineParameters == null)
            {
                ParametersListView.Enabled = false;
                return;
            }
            else if(pipelineParameters.Count == 0)
            {
                ParametersListView.Enabled = true;
                if (ParametersListView.Columns.Count > 0)
                    ParametersListView.Columns[0].Width = ParametersListView.Width;
                ParametersListView.Items.Add(Resources.NO_PARAMETERS);
                return;
            }
            else
            {
                ParametersListView.Enabled = true;
                SetColumnWidths();
            }


            foreach (PipelineParameter pipelineParameter in pipelineParameters)
            {
                string parameterName = pipelineParameter.ShortName;

                ListViewItem item = new ListViewItem(parameterName, "parameter");
                item.ToolTipText = pipelineParameter.Description;
                ParametersListView.Items.Add(item);

                JobParameter jobParameter = null;
                name2jobParameterDictionary.TryGetValue(pipelineParameter.Name, out jobParameter);
                if (jobParameter != null && jobParameter.ParamValue != null)
                {
                    string paramValue = string.Empty;
                    if (pipelineParameter.Type == ParameterType.StringCollection)
                    {
                        Choice choice = pipelineParameter.GetChoice(jobParameter.ParamValue.ToString());
                        paramValue = choice.Text;
                    }
                    else if(pipelineParameter.Type == ParameterType.InputFile 
                        && jobParameter.ParamValue is InternalFile)
                    {
                        paramValue = ((InternalFile) jobParameter.ParamValue).Filename;
                        // make the value "pretty" if it has that axis junk in the filename
                        string attachmentName = ".att_";
                        if(paramValue.Contains(attachmentName))
                        {
                            int endOfAttachmentName = paramValue.LastIndexOf(".att_");
                            paramValue = paramValue.Substring(endOfAttachmentName + attachmentName.Length);
                        }
                    }
                    else
                    {
                        paramValue = jobParameter.ParamValue.ToString();
                    }
                        
                    item.SubItems.Add(paramValue);
                }
            }
        }

        /// <summary>
        /// Sets the column widths.
        /// </summary>
        private void SetColumnWidths()
        {
            if (ParametersListView.Columns.Count > 1)
            {
                ParametersListView.Columns[0].Width = (int) 33*ParametersListView.Width/100;
                ParametersListView.Columns[1].Width = (int) 66*ParametersListView.Width/100;
            }
        }

        /// <summary>
        /// Handles the ItemSelectionChanged event of the ParametersListView control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.ListViewItemSelectionChangedEventArgs"/> instance containing the event data.</param>
        private void ParametersListView_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            if (e.IsSelected)
            {
                e.Item.Selected = false;
            }
        }
    }
}
