using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Threading;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.GenePattern.Server;

namespace Microsoft.eResearch.GenePattern.Package
{
    // TODO: Localize
    /// <summary>
    /// The Jobs that are run in the Given Gene Pattern Server
    /// </summary>
    public partial class SelectJobFromServer : AssociatePipelineWizardControl
    {
        private IGenePatternServer genePatternServer;
        private ICollection<PipelineAssociation> downloadSearchResult;
        private ICollection<PipelineAssociation> documentSearchResult;
        private IController controller;
        private Thread downloadJobs;
        private string lastFilterString; // null so we have to do a query the first time

        /// <summary>
        /// Initializes a new instance of the <see cref="SelectJobFromServer"/> class.
        /// </summary>
        /// <param name="genePatternServer">The gene pattern server.</param>
        /// <param name="model">The model.</param>
        /// <param name="controller">The controller.</param>
        public SelectJobFromServer(IGenePatternServer genePatternServer, AssociatePipelineWizardModel model, IController controller)
            : base(model)
        {
            InitializeComponent();
            this.genePatternServer = genePatternServer;
            this.controller = controller;
        }

        /// <summary>
        /// Lists the jobs from document.
        /// </summary>
        /// <param name="forceUpdate">if set to <c>true</c> [force update].</param>
        private void ListJobsFromDocument(bool forceUpdate)
        {
            // only requery when search term changes
            string filterTerm = searchText.Text.Trim();
            if (lastFilterString != filterTerm || forceUpdate)
            {

                // get data from model to fake the search result and then update as normal
                List<int> docJobs = new List<int>();
                documentSearchResult = new List<PipelineAssociation>();
                // override job stop points with association stop points for consistency
                foreach (PipelineAssociation pa in controller.ActiveModel.PipelineAssociations.Values)
                {

                    PipelineAssociation paClone = (PipelineAssociation)pa.Clone();
                    paClone.MakeShallowClone();

                    paClone.Job = (Job)pa.Job.Clone(); // clone it so you don't change what is stored!
                    paClone.Job.StopPoint = paClone.UserStopPoint;
                    documentSearchResult.Add(paClone);
                    docJobs.Add(pa.Job.Id);

                }
                FilterAndDisplayJobs(true);
            }
            JobPanel.Controls.Clear();
            JobPanel.Controls.Add(Jobs);

        }

        /// <summary>
        /// Lists the jobs from server.
        /// </summary>
        /// <param name="forceUpdate">if set to <c>true</c> [force update].</param>
        private void ListJobsFromServer(bool forceUpdate)
        {
            if (downloadJobs == null)
            {
                // only requery when search term changes
                string filterTerm = searchText.Text.Trim();
                Cursor.Current = Cursors.WaitCursor;
                if (lastFilterString != filterTerm || forceUpdate)
                {
                    Jobs.Items.Clear();
                    // clear data and start the timer that will poll when the download finishes
                    downloadSearchResult = new List<PipelineAssociation>();
                    if (controller.AssureConnection())
                    {
                        // get jobs for the current user by default
                        downloadJobs = new Thread(new ThreadStart(DownloadJobs));
                        downloadJobs.Start();
                        JobsCompletedTimer.Start();
                        // show progress bar
                        JobPanel.Controls.Clear();
                        JobProcessing waitControl = new JobProcessing();
                        waitControl.Dock = DockStyle.Fill;
                        waitControl.messageLabel.Text = "Downloading Jobs...";
                        JobPanel.Controls.Add(waitControl);
                    }
                }
                Cursor.Current = Cursors.Default;
            }
        }

        /// <summary>
        /// Filters the and display jobs.
        /// </summary>
        /// <param name="useDocJobs">if set to <c>true</c> [use doc jobs].</param>
        private void FilterAndDisplayJobs(bool useDocJobs)
        {
            string filterTerm = searchText.Text.Trim();

            SetJobListColumns();
            Jobs.Items.Clear();

            ICollection<PipelineAssociation> searchResult = useDocJobs ? documentSearchResult : downloadSearchResult;
            if (searchResult.Count == 0)
            {
                ListViewItem noResults = Jobs.Items.Add("");
                noResults.ImageKey = "job";
                noResults.SubItems.Add("No jobs found on server for username.");
            }

            foreach (PipelineAssociation pa in searchResult)
            {
                if ((String.IsNullOrEmpty(filterTerm) || MatchesString(pa.Job, filterTerm)) && pa.Job != null)
                {
                    ListViewItem newItem = Jobs.Items.Add(pa.Job.Id.ToString(CultureInfo.InvariantCulture));
                    newItem.ImageKey = "job";
                    // if jobs are coming from the document, get name of association
                    // go through existing associations and find 
                    if (UseExisting.Checked)
                    {

                        newItem.SubItems.Add(pa.Name);
                    }

                    newItem.SubItems.Add(pa.Job.PipelineName);
                    newItem.Tag = pa.Job;

                    if (AssociatePipelineWizardModel.PipelineAssociation.Job != null &&
                        pa.Job.Id == AssociatePipelineWizardModel.PipelineAssociation.Job.Id)
                    {
                        newItem.Selected = true;
                    }
                }
            }

            if (Jobs.Items.Count == 0)
            {
                ListViewItem noResults = Jobs.Items.Add("");
                noResults.ImageKey = "job";
                noResults.SubItems.Add("No jobs on server matched the criteria.");
            }
            lastFilterString = filterTerm;
        }

        /// <summary>
        /// Sets the job list columns.
        /// </summary>
        private void SetJobListColumns()
        {
            Jobs.Columns.Clear();

            if (PickJob.Checked)
            {
                // add columns for server jobs
                Jobs.Columns.Add(Resources.ID, 53, HorizontalAlignment.Left);
                Jobs.Columns.Add(Resources.PIPELINE, 240, HorizontalAlignment.Left);
            }
            else
            {
                // add columns for document jobs
                Jobs.Columns.Add(Resources.ID, 53, HorizontalAlignment.Left);
                Jobs.Columns.Add(Resources.ASSOCIATION, 100, HorizontalAlignment.Left);
                Jobs.Columns.Add(Resources.PIPELINE, 240, HorizontalAlignment.Left);
            }
        }

        /// <summary>
        /// Matcheses the string.
        /// </summary>
        /// <param name="job">The job.</param>
        /// <param name="filterTerm">The filter term.</param>
        /// <returns></returns>
        private static bool MatchesString(Job job, string filterTerm)
        {
            try
            {
                if (job.Id.ToString(CultureInfo.InvariantCulture).Contains(filterTerm))
                {
                    return true;
                }
                if ((job.PipelineName != null &&
                    job.PipelineName.ToLower(CultureInfo.InvariantCulture).Contains(filterTerm.ToLower(CultureInfo.InvariantCulture))))
                {
                    return true;
                }
                return false;
            }
            catch (NullReferenceException)
            {
                // if any of this stuff is null, then it doesn't match.
                return false;
            }
        }

        /// <summary>
        /// Handles the Click event of the Search control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void Search_Click(object sender, EventArgs e)
        {
            try
            {
                // filter the list view
                if (PickJob.Checked)
                {
                    ListJobsFromServer(true);
                }
                else
                {
                    ListJobsFromDocument(true);
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Gets the Job object associated with the currently selected job in the UI
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        public Job GetSelectedJob()
        {
            if (Jobs.SelectedItems.Count > 0)
            {
                return Jobs.SelectedItems[0].Tag as Job;
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// Returns true if the control can be hidden (advanced).  returns false
        /// if the control needs some correction or input.
        /// </summary>
        /// <returns></returns>
        public override bool CanHide()
        {
            return GetSelectedJob() != null;
        }

        /// <summary>
        /// Called when control is shown.
        /// </summary>
        public override void OnShown()
        {
            int pipelineCount = controller.ActiveModel.PipelineAssociations.Count;

            JobSourcePanel.Visible = (pipelineCount > 0);
            UseExisting.Enabled = (pipelineCount > 0);
            
            if (!associatePipelineWizard.IsEditing)
            {
                PickJob.Checked = true;
                ListJobsFromServer(true);
            }
            else
            {
                UseExisting.Checked = true;
                ListJobsFromDocument(true);
            }
            UpdateNextButton();
        }

        /// <summary>
        /// Called when the control is hidden
        /// </summary>
        /// <param name="delta"></param>
        /// <returns>
        /// Returns false if the hide should be canceled
        /// </returns>
        public override bool OnHide(int delta)
        {
            // store selected pipeline
            AssociatePipelineWizardModel.PipelineAssociation.Job = GetSelectedJob();
            Debug.Assert(GetSelectedJob() != null || delta <= 0); // if we're going forward we need a job!
            if (AssociatePipelineWizardModel.PipelineAssociation.Job != null &&
                AssociatePipelineWizardModel.PipelineAssociation.Job.Id > -1)
            {
                if (PickJob.Checked)
                {
                    // get full data
                    AssociatePipelineWizardModel.PipelineAssociation.Job =
                        genePatternServer.GetJobById(AssociatePipelineWizardModel.PipelineAssociation.Job.Id);
                    // create new pipeline from selected job
                    AssociatePipelineWizardModel.Pipeline =
                        genePatternServer.GetPipelineByLsid(
                            AssociatePipelineWizardModel.PipelineAssociation.Job.PipelineLsid);
                }
                else
                {
                    // get pipeline from document
                    AssociatePipelineWizardModel.Pipeline =
                        controller.ActiveModel.Pipelines[
                            AssociatePipelineWizardModel.PipelineAssociation.Job.PipelineLsid].Clone() as Pipeline;
                }

                // check existence of pipeline
                if (controller.ActiveModel.Pipelines.ContainsKey(AssociatePipelineWizardModel.Pipeline.Lsid))
                {
                    // is this pipeline already embedded AND we are linking?  warn about deletion
                    Pipeline original = controller.ActiveModel.Pipelines[AssociatePipelineWizardModel.Pipeline.Lsid];

                    if (UseExisting.Checked)
                    {
                        if (!IsPipelineServerSame(original.LinkConnection))
                        {
                            return false; // cancel
                        }
                    }

                    if (!original.IsLinked && LinkPipeline.Checked)
                    {
                        bool cancelled = WarnAlreadyStored();
                        if (cancelled)
                        {
                            return false; // cancel
                        }
                    }
                    else if (original.IsLinked && StorePipeline.Checked)
                    {
                        if (WarnAlreadyLinkedPipeline())
                        {
                            return false; // cancel
                        }
                    }
                    else
                    {
                        AssociatePipelineWizardModel.Pipeline.IsLinked = LinkPipeline.Checked;

                        if (!AssociatePipelineWizardModel.Pipeline.IsLinked && original.IsLinked)
                        {
                            bool gotConnection = controller.AssureConnection();
                            if (!gotConnection)
                            {
                                return false;
                            }
                        }
                    }
                }
                else
                {
                    AssociatePipelineWizardModel.Pipeline.IsLinked = LinkPipeline.Checked;


                    if (AssociatePipelineWizardModel.Pipeline.IsLinked && !PickJob.Checked)
                    {
                        StoreOldConnection();
                    }
                    else
                    {
                        // store connection for future reference, even it's not linked
                        StoreCurrentConnection();
                    }
                }
            }

            return true;
        }

        /// <summary>
        /// Determines whether [is pipeline server same].
        /// </summary>
        /// <param name="linkedConnection">The linked connection.</param>
        /// <returns>
        /// 	<c>true</c> if [is pipeline server same]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsPipelineServerSame(Connection linkedConnection)
        {
            if (genePatternServer != null && genePatternServer.Connection != null)
            {
                if (linkedConnection.ServerUri != genePatternServer.Connection.ServerUri)
                {
                    string message = string.Format(Resources.CONNECT_TO_ORIGINAL_SERVER, linkedConnection.ServerName);
                    if (DialogResult.Cancel == MessageBox.Show(message, Resources.GENEPATTREN_MSG_HEADER,
                                    MessageBoxButtons.OKCancel, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1))
                    {
                        return false;
                    }
                }
            }
            return true;
        }

        /// <summary>
        /// Stores the old connection.
        /// </summary>
        private void StoreOldConnection()
        {
            // linked from a previous connection
            Pipeline thePipeline =
                controller.ActiveModel.Pipelines[
                    AssociatePipelineWizardModel.PipelineAssociation.Job.PipelineLsid];
            AssociatePipelineWizardModel.Pipeline.LinkConnection = thePipeline.LinkConnection;
            AssociatePipelineWizardModel.Pipeline.LinkConnection.Password = null;
        }

        /// <summary>
        /// Warns the overwriting connection.
        /// </summary>
        /// <param name="original">The original.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object,System.Object)")]
        private void WarnOverwritingConnection(Pipeline original)
        {
            if (controller.CurrentConnection == null)
            {

            }

            // alert user that connection is being overwritten
            // compare the connection currently in the pipeline to the users current connection
            if (original.LinkConnection.ServerUri != controller.CurrentConnection.ServerUri)
            {
                string oldUrl = original.LinkConnection.ServerName + ":" + original.LinkConnection.Port;
                string newUrl = controller.CurrentConnection.ServerName + ":" +
                                 controller.CurrentConnection.Port;
                string message =
                    String.Format(Resources.PIPELINE_LINKED_TO_DIFFERENT_SERVER, oldUrl, newUrl);

                if (DialogResult.Yes == MessageBox.Show(message, Resources.UPDATE_LINK_CAPTION,
                    MessageBoxButtons.YesNo, MessageBoxIcon.Warning))
                {
                    StoreCurrentConnection();
                }
            }
        }

        /// <summary>
        /// Warns the user a linked pipeline is already embedded.  Returns true if they cancel the operation
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object)")]
        private bool WarnAlreadyStored()
        {
            DialogResult result =
                MessageBox.Show(String.Format(Resources.REMOVE_EMBEDDED_PIPELINE_MESSAGE, AssociatePipelineWizardModel.Pipeline.Name),
                Resources.PIPELINE_EMBEDDED_CAPTION, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Warning);

            AssociatePipelineWizardModel.Pipeline.IsLinked = result == DialogResult.Yes;
            switch (result)
            {
                case DialogResult.Yes:
                    StoreCurrentConnection();
                    break;
                case DialogResult.Cancel:
                    return false;
                case DialogResult.No:
                    StorePipeline.Checked = true;
                    break;
            }
            return false;
        }

        /// <summary>
        /// Warns the user a linked pipeline is already embedded.  Returns true if they cancel the operation
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object)")]
        private bool WarnAlreadyLinkedPipeline()
        {
            DialogResult result =
                MessageBox.Show(String.Format(Resources.REMOVE_LINKED_PIPELINE, AssociatePipelineWizardModel.Pipeline.Name),
                Resources.PIPELINE_LINKED_CAPTION, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Warning);

            switch (result)
            {
                case DialogResult.Yes:
                    AssociatePipelineWizardModel.Pipeline.IsLinked = false;
                    break;
                case DialogResult.Cancel:
                    return true;
                case DialogResult.No:
                    LinkPipeline.Checked = true;
                    AssociatePipelineWizardModel.Pipeline.IsLinked = true;
                    break;
            }

            return false;
        }

        /// <summary>
        /// Stores the current connection.
        /// </summary>
        private void StoreCurrentConnection()
        {
            AssociatePipelineWizardModel.Pipeline.LinkConnection = controller.CurrentConnection.Clone() as Connection;
            //delete users creds for security reasons
            AssociatePipelineWizardModel.Pipeline.LinkConnection.Password = String.Empty;
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the Jobs control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void Jobs_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                Job selectedJob = GetSelectedJob();
                AssociatePipelineWizardModel.PipelineAssociation.UserStopPoint = selectedJob == null ? -1 : selectedJob.StopPoint;
                UpdateNextButton();
                if (selectedJob != null && controller.ActiveModel.Pipelines.ContainsKey(selectedJob.PipelineLsid))
                {
                    StorePipeline.Checked = !(controller.ActiveModel.Pipelines[selectedJob.PipelineLsid].IsLinked);
                    LinkPipeline.Checked = (controller.ActiveModel.Pipelines[selectedJob.PipelineLsid].IsLinked);
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Updates the next button.
        /// </summary>
        protected override void UpdateNextButton()
        {
            if (Parent != null)
            {
                AssociatePipelineWizard apw = Parent.Parent as AssociatePipelineWizard;
                if (apw != null)
                {
                    apw.EnableNextButton(GetSelectedJob() != null);
                }
            }
        }

        /// <summary>
        /// Handles the CheckedChanged event of the UseExisting control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void UseExisting_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                if (UseExisting.Checked)
                {
                    searchText.Clear();

                    // deselect other radio button
                    PickJob.Enabled = false;
                    PickJob.Checked = false;
                    PickJob.Enabled = true;

                    // populate job list from jobs already in document
                    ListJobsFromDocument(true);

                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the CheckedChanged event of the PickJob control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void PickJob_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                if (PickJob.Checked)
                {
                    searchText.Clear();

                    // deselect other radio button
                    if (UseExisting.Enabled)
                    {
                        UseExisting.Enabled = false;
                        UseExisting.Checked = false;
                        UseExisting.Enabled = true;
                    }
                    ListJobsFromServer(true);
                }
                else
                {

                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Downloads the jobs.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void DownloadJobs()
        {
            genePatternServer = controller.GetGenePatternServer();
            // get jobs for the current user by default
            try
            {
                IList<Job> jobs = genePatternServer.GetJobsByCurrentUser();
                if (PickJob.Checked) // this might not still be checked if the user clicked the other radio button!
                {
                    // wrap each job in a fake pipeline association to make the filteranddisplay() method easier
                    downloadSearchResult.Clear();
                    foreach (Job job in jobs)
                    {
                        PipelineAssociation pa = new PipelineAssociation();
                        pa.Job = job;
                        downloadSearchResult.Add(pa);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex, true);
            }
        }

        /// <summary>
        /// Handles the Tick event of the JobsCompletedTimer control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void JobsCompletedTimer_Tick(object sender, EventArgs e)
        {
            if (!downloadJobs.IsAlive && PickJob.Checked)
            {
                JobsCompletedTimer.Stop();
                FilterAndDisplayJobs(false);

                JobPanel.Controls.Clear();
                JobPanel.Controls.Add(Jobs);
                downloadJobs = null;
            }
        }

        /// <summary>
        /// Handles the MouseDoubleClick event of the Jobs control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void Jobs_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            try
            {
                PerformNextStep();
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }

        }
    }
}
