using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Infusion.GenePattern.Common;
using System.Threading;
using System.Globalization;

namespace Infusion.GenePattern.Package
{
    public partial class SelectJobsUserControl : UserControl
    {
        #region Variable Declaration

        private IController _controller;
        private ICollection<PipelineAssociation> _downloadSearchResult;
        private ICollection<PipelineAssociation> _documentSearchResult;
        private Thread _downloadJobs;
        private string _lastFilterString;
        private IGenePatternServer _genePatternServer;
   
        /// <summary>
        /// Occurs when [jobs selected index changed].
        /// </summary>
        public event EventHandler<EventArgs> JobsSelectedIndexChanged;
        
        #endregion

        #region Public Methods

        /// <summary>
        /// Initializes a new instance of the <see cref="SelectJobsUserControl"/> class.
        /// </summary>
        public SelectJobsUserControl()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Initials the update.
        /// </summary>
        /// <param name="controller">The controller.</param>
        /// <param name="documentKey">The document key.</param>
        public void InitialUpdate(IController controller, string documentKey)
        {
            this._controller = controller;
            _genePatternServer = controller.GetGenePatternServer();
            chkRetrieveJobFromDocument.Checked = controller.GetDocumentModel(documentKey).PipelineAssociations.Count > 0 ? true : false;

            if (chkRetrieveJobFromDocument.Checked)
            {
                ListJobsFromDocument(true);
            }
            else
            {
                ListJobsFromServer(true);
            }
        }
      
        /// <summary>
        /// Gets the job.
        /// </summary>
        /// <returns></returns>
        public Job GetJob()
        {
            if (Jobs.SelectedItems.Count > 0)
            {
                if (chkRetrieveJobFromDocument.Checked)
                {
                    return Jobs.SelectedItems[0].Tag as Job;
                }
                else
                {
                    return _genePatternServer.GetJobById(((Job)Jobs.SelectedItems[0].Tag).Id);
                }
            }
            return null;
        }

        /// <summary>
        /// Gets the pipeline.
        /// </summary>
        /// <param name="pipelineLsid">The pipeline lsid.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        public Pipeline GetPipeline(string pipelineLsid, string documentKey)
        {
            if (chkRetrieveJobFromDocument.Checked)
            {
                return _controller.GetDocumentModel(documentKey).Pipelines[pipelineLsid].Clone() as Pipeline;
            }
            else
            {
                return _genePatternServer.GetPipelineByLsid(pipelineLsid);
            }
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Lists the jobs from document.
        /// </summary>
        /// <param name="forceUpdate">if set to <c>true</c> [force update].</param>
        private void ListJobsFromDocument(bool forceUpdate)
        {
            // only requery when search term changes
            string filterTerm = txtSearch.Text.Trim();

            if (string.Compare(_lastFilterString, filterTerm, true) != 0 || forceUpdate)
            {
                // get data from model to fake the search result and then update as normal
                List<int> docJobs = new List<int>();
                _documentSearchResult = new List<PipelineAssociation>();
                PipelineAssociation paClone;

                foreach (PipelineAssociation pa in _controller.ActiveModel.PipelineAssociations.Values)
                {
                    paClone = (PipelineAssociation)pa.Clone();
                    paClone.MakeShallowClone();
                    paClone.Job = pa.Job.Clone() as Job; // clone it so you don't change what is stored!
                    paClone.Job.StopPoint = paClone.UserStopPoint;
                    _documentSearchResult.Add(paClone);
                    docJobs.Add(pa.Job.Id);
                }

                FilterAndDisplayJobs(true);
            }
            JobPanel.Controls.Clear();
            JobPanel.Controls.Add(Jobs);
        }

        /// <summary>
        /// Lists the jobs from server.
        /// </summary>
        /// <param name="forceUpdate">if set to <c>true</c> [force update].</param>
        private void ListJobsFromServer(bool forceUpdate)
        {
            if (_downloadJobs == null)
            {
                // only requery when search term changes
                string filterTerm = txtSearch.Text.Trim();
                Cursor.Current = Cursors.WaitCursor;
                if (string.Compare(_lastFilterString, filterTerm, true) != 0 || forceUpdate)
                {
                    Jobs.Items.Clear(); // clear data and start the timer that will poll when the download finishes
                    _downloadSearchResult = new List<PipelineAssociation>();

                    if (_controller.AssureConnection())
                    {
                        // get jobs for the current user by default
                        _downloadJobs = new Thread(new ThreadStart(DownloadJobs));
                        _downloadJobs.Start();
                        timerJobsCompleted.Start();
                        // show progress bar
                        JobPanel.Controls.Clear();
                        JobProcessing waitControl = new JobProcessing();
                        waitControl.Dock = DockStyle.Fill;
                        waitControl.messageLabel.Text = "Downloading Jobs...";
                        JobPanel.Controls.Add(waitControl);
                    }
                }
                Cursor.Current = Cursors.Default;
            }
        }

        /// <summary>
        /// Filters the and display jobs.
        /// </summary>
        /// <param name="useDocJobs">if set to <c>true</c> [use doc jobs].</param>
        private void FilterAndDisplayJobs(bool useDocJobs)
        {
            string filterTerm = txtSearch.Text.Trim();

            SetJobListColumns();
            Jobs.Items.Clear();

            ICollection<PipelineAssociation> searchResult = useDocJobs ? _documentSearchResult : _downloadSearchResult;

            if (searchResult.Count == 0)
            {
                ListViewItem noResults = Jobs.Items.Add("");
                noResults.ImageKey = "job";
                noResults.SubItems.Add("No jobs found on server for username.");
            }

            foreach (PipelineAssociation pa in searchResult)
            {
                if ((String.IsNullOrEmpty(filterTerm) || MatchesString(pa.Job, filterTerm)) && pa.Job != null)
                {
                    ListViewItem newItem = Jobs.Items.Add(pa.Job.Id.ToString(CultureInfo.InvariantCulture));
                    newItem.ImageKey = "job";
                    // if jobs are coming from the document, get name of association
                    // go through existing associations and find 
                    if (chkRetrieveJobFromDocument.Checked)
                    {
                        newItem.SubItems.Add(pa.Name);
                    }

                    newItem.SubItems.Add(pa.Job.PipelineName);
                    newItem.Tag = pa.Job;
                }
            }

            if (Jobs.Items.Count == 0)
            {
                ListViewItem noResults = Jobs.Items.Add("");
                noResults.ImageKey = "job";
                noResults.SubItems.Add("No jobs on server matched criteria.");
            }
            _lastFilterString = filterTerm;
        }

        /// <summary>
        /// Sets the job list columns.
        /// </summary>
        private void SetJobListColumns()
        {
            Jobs.Columns.Clear();

            if (chkRetrieveJobFromDocument.Checked)
            {
                // add columns for document jobs
                Jobs.Columns.Add("Id", 53, HorizontalAlignment.Left);
                Jobs.Columns.Add("Association", 100, HorizontalAlignment.Left);
                Jobs.Columns.Add("Pipeline", 240, HorizontalAlignment.Left);
            }
            else
            {
                // add columns for server jobs
                Jobs.Columns.Add("Id", 53, HorizontalAlignment.Left);
                Jobs.Columns.Add("Pipeline", 240, HorizontalAlignment.Left);
            }
        }

        /// <summary>
        /// Matcheses the string.
        /// </summary>
        /// <param name="job">The job.</param>
        /// <param name="filterTerm">The filter term.</param>
        /// <returns></returns>
        private bool MatchesString(Job job, string filterTerm)
        {
            try
            {
                if (job.Id.ToString(CultureInfo.InvariantCulture).Contains(filterTerm))
                {
                    return true;
                }
                else if ((job.PipelineName != null && job.PipelineName.ToLower().Contains(filterTerm.ToLower())))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                // if any of this stuff is null, then it doesn't match.
                return false;
            }
        }

        /// <summary>
        /// Downloads the jobs.
        /// </summary>
        private void DownloadJobs()
        {
            try
            {
                IList<Job> jobs = _genePatternServer.GetJobsByCurrentUser();
                if (!chkRetrieveJobFromDocument.Checked)
                {
                    // wrap each job in a fake pipeline association to make the filteranddisplay() method easier
                    _downloadSearchResult.Clear();
                    PipelineAssociation pipelineAssociation;
                    foreach (Job job in jobs)
                    {
                        pipelineAssociation = new PipelineAssociation();
                        pipelineAssociation.Job = job;
                        _downloadSearchResult.Add(pipelineAssociation);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex, true);
            }
        }

        #endregion

        #region Event Handler

        /// <summary>
        /// Handles the CheckedChanged event of the chkRetrieveJobFromDocument control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void chkRetrieveJobFromDocument_CheckedChanged(object sender, EventArgs e)
        {
            txtSearch.Clear();
            if (chkRetrieveJobFromDocument.Checked)
            {
                ListJobsFromDocument(true);
            }
            else
            {
                ListJobsFromServer(true);
            }
            OnJobsSelectedIndexChanged();
        }

        /// <summary>
        /// Handles the Click event of the btnSearch control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnSearch_Click(object sender, EventArgs e)
        {
            // filter the list view
            if (chkRetrieveJobFromDocument.Checked)
            {
                ListJobsFromDocument(true);
            }
            else
            {
                ListJobsFromServer(true);
            }
            OnJobsSelectedIndexChanged();
        }

        /// <summary>
        /// Handles the Tick event of the timerJobsCompleted control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void timerJobsCompleted_Tick(object sender, EventArgs e)
        {
            if (!_downloadJobs.IsAlive && !chkRetrieveJobFromDocument.Checked)
            {
                timerJobsCompleted.Stop();
                FilterAndDisplayJobs(false);

                JobPanel.Controls.Clear();
                JobPanel.Controls.Add(Jobs);
                _downloadJobs = null;
            }
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the Jobs control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Jobs_SelectedIndexChanged(object sender, EventArgs e)
        {
            OnJobsSelectedIndexChanged();
        }

        /// <summary>
        /// Called when [jobs selected index changed].
        /// </summary>
        private void OnJobsSelectedIndexChanged()
        {
            if (JobsSelectedIndexChanged != null)
            {
                JobsSelectedIndexChanged(this, EventArgs.Empty);
            }
        }

        #endregion

        # region Property

        /// <summary>
        /// Gets a value indicating whether [islinked pipeline].
        /// </summary>
        /// <value><c>true</c> if [islinked pipeline]; otherwise, <c>false</c>.</value>
        public bool IsStorePipelineInDocument
        {
            get { return chkStorePipelineInDocument.Checked; }
            set { chkStorePipelineInDocument.Checked = value; }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is job from document.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is job from document; otherwise, <c>false</c>.
        /// </value>
        public bool IsJobFromDocument
        {
            get { return chkRetrieveJobFromDocument.Checked; }
        }

        # endregion
      
    }
}
