using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.GenePattern.Server;


namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// User control for setting the primary result of a pipeline association
    /// </summary>
    public partial class SelectPrimaryResult : AssociatePipelineWizardControl
    {
        /// <summary>
        /// LOG FILE NAME
        /// </summary>
        public const string ExecutionLog = "execution_log.txt";

        /// <summary>
        /// Initializes a new instance of the <see cref="SelectPrimaryResult"/> class.
        /// </summary>
        /// <param name="model">The model.</param>
        public SelectPrimaryResult(AssociatePipelineWizardModel model)
            : base(model)
        {
            InitializeComponent();
            Outputs.SelectedIndexChanged += new EventHandler<EventArgs>(Outputs_SelectedIndexChanged);
            Outputs.NodeMouseDoubleClick += new EventHandler<EventArgs>(Outputs_NodeMouseDoubleClick);
        }

        /// <summary>
        /// Handles the NodeMouseDoubleClick event of the Outputs control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void Outputs_NodeMouseDoubleClick(object sender, EventArgs e)
        {
            try
            {
                PerformNextStep();
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the Outputs control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void Outputs_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                UpdateNextButton();
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Called when control is shown.
        /// </summary>
        public override void OnShown()
        {
            Outputs.PipelineName = AssociatePipelineWizardModel.Pipeline.Name;
            List<string> nameList = AssociatePipelineWizardModel.PipelineAssociation.Job.FileNames;
            List<PipelineChild> resultNames = new List<PipelineChild>(nameList.Count);
            int index = -1;
            for (int i = 0; i < nameList.Count; i++)
            {
                if (!nameList[i].EndsWith(ExecutionLog))
                {
                    string r = nameList[i];
                    resultNames.Add(new PipelineChild(r, 0));
                    if (AssociatePipelineWizardModel.PipelineAssociation.PrimaryResultName == r)
                    {
                        index = resultNames.Count - 1;
                    }
                }
            }
            Outputs.Children = resultNames.ToArray();
            Outputs.SelectedChildIndex = index != -1 ? index : Outputs.Children.Length - 1;
            UpdateNextButton();
        }

        /// <summary>
        /// Returns true if the control can be hidden (advanced).  returns false
        /// if the control needs some correction or input.
        /// </summary>
        /// <returns></returns>
        public override bool CanHide()
        {
            if (AssociatePipelineWizardModel.PipelineAssociation.Job.ShallowResults.Count > 0)
            {
                return Outputs.SelectedChildIndex > -1;
            }
            else
            {
                return true;
            }
        }

        /// <summary>
        /// Called when the control is hidden
        /// </summary>
        /// <returns>
        /// Returns false if the hide should be canceled
        /// </returns>
        public override bool OnHide(int delta)
        {
            List<string> nameList = AssociatePipelineWizardModel.PipelineAssociation.Job.FileNames;
            if (Outputs.SelectedChildIndex > -1 && Outputs.SelectedChildIndex < nameList.Count)
            {
                // some outputs were not selectable so we can't just index straight to the list
                int resultIndex = 0;
                int realIndex = 0;
                while (resultIndex < Outputs.SelectedChildIndex || nameList[realIndex].EndsWith(ExecutionLog))
                {
                    if (!nameList[realIndex].EndsWith(ExecutionLog))
                    {

                        resultIndex++;
                    }
                    realIndex++;
                }
                AssociatePipelineWizardModel.PipelineAssociation.PrimaryResultName = nameList[realIndex];

            }
            else
            {
                AssociatePipelineWizardModel.PipelineAssociation.PrimaryResultName = null;
            }
            return true;

        }
    }
}
