using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.GenePattern.Server;
using Microsoft.eResearch.ScientificAddIn.Common;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// Task pane for viewing, editing Server Connections.
    /// </summary>
    public partial class ViewConnectionsUserControl : UserControl, ITaskPaneControl
    {

        #region Fields

        private IController _controller;
        private ListViewColumnSorter connectionsListViewColumnSorter;
        private List<Task> tasks;
        private ListViewColumnSorter tasksListViewColumnSorter;
        private string errorMessage;
        private int nameColumnDefaultWidth;
        private string _documentKey;
        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ViewConnectionsUserControl"/> class.
        /// </summary>
        public ViewConnectionsUserControl()
        {
            InitializeComponent();
            tasksListViewColumnSorter = new ListViewColumnSorter();
            tasksListViewColumnSorter.Order = SortOrder.Ascending;
            this.tasksListView.ListViewItemSorter = tasksListViewColumnSorter;

            this.nameColumnDefaultWidth = this.tasksListView.Columns[0].Width;
            this.connectionsListViewColumnSorter = new ListViewColumnSorter();
            this.connectionsListViewColumnSorter.Order = SortOrder.Ascending;
            this.configuredConnectionsListView.ListViewItemSorter = connectionsListViewColumnSorter;

            this.jobProcessing1.Visible = false;
            this.jobProcessing1.SetProcessingDisplayMessage(Resources.RETRIEVING_PIPELINES); // TODO resource it


            this.backgroundWorker.DoWork += new System.ComponentModel.DoWorkEventHandler(backgroundWorker_DoWork);
            this.backgroundWorker.RunWorkerCompleted += new System.ComponentModel.RunWorkerCompletedEventHandler(backgroundWorker_RunWorkerCompleted);
        }

        #endregion

        #region Methods

        //  Public Methods (2)


        /// <summary>
        /// Gets or sets the controller.
        /// </summary>
        /// <value>The controller.</value>
        public IController Controller
        {
            get
            {
                return _controller;
            }
            set
            {
                if (value == null)
                    return;

                this._controller = value;
                _documentKey = _controller.DocumentKey;
                this.UpdateConnectionsList(true);
            }

        }

        /// <summary>
        /// Updates the connections list.
        /// </summary>
        public void UpdateConnectionsList(bool updateTasks)
        {
            UserData userData = this._controller.UserData;
            UpdateConfiguredConnections(userData);
            string currentConnection = this._controller.ActiveConnectionForDocument;//userData.CurrentConnectionName;
            if (string.IsNullOrEmpty(currentConnection))
            {
                currentConnection = userData.CurrentConnectionName;
            }
            if (updateTasks) //Update the tasks list only if required
            {
                // We update the tasks list after updating "configured connections" list view as 
                // there is a possibility that we might get an exception trying to get the 
                // tasks from the server
                UpdateTaskList(currentConnection);
            }
        }

        #region Event Handlers

        private void addConfigConnectionButton_Click(object sender, EventArgs e)
        {
            this._controller.ShowServerConnectionForm(null, false);
            // Select Current Connection in Connection Pane 
            if (null != this._controller.CurrentConnection)
            {
                SelectConfiguredConnectionsListItem(this._controller.CurrentConnection.Name);
            }
        }

        void backgroundWorker_DoWork(object sender, System.ComponentModel.DoWorkEventArgs e)
        {

            object argument = e.Argument;
            if (argument != null)
            {
                string currentConnection = argument.ToString();
                try
                {
                    this.tasks = new List<Task>();
                    if (!String.IsNullOrEmpty(currentConnection))
                        tasks = this._controller.GetGenePatternServer().GetAllTasks;
                }
                catch (ConnectionFailedException exception)
                {
                    this.errorMessage = "Error occured during retrieving tasks. Please check Configuration.";
                    if (this.InvokeRequired)
                        this.BeginInvoke(new EventHandler(this.ShowTaskretrievalErrorMessage));
                    else
                        this.ShowTaskretrievalErrorMessage(this, EventArgs.Empty);

                    Trace.WriteLine("Error occured during Retrieveing tasks for connection Name : " + currentConnection
                                    + " Exception details : " + exception.Message, TraceLevel.Warning.ToString());
                }
            }

        }

        void backgroundWorker_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            // If user control already is disposed 
            // when background worker is done
            if (IsDisposed)
                return;

            if (this.tasksListView.InvokeRequired)
                this.Invoke(new EventHandler(this.UpdateTaskListView));
            else
                this.UpdateTaskListView(this, EventArgs.Empty);

        }

        private void configuredConnectionsListView_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            // Determine if clicked column is already the column that is being sorted.
            if (e.Column == this.connectionsListViewColumnSorter.SortColumn)
            {
                // Reverse the current sort direction for this column.
                if (this.connectionsListViewColumnSorter.Order == SortOrder.Ascending)
                {
                    this.connectionsListViewColumnSorter.Order = SortOrder.Descending;
                }
                else
                {
                    this.connectionsListViewColumnSorter.Order = SortOrder.Ascending;
                }
            }
            else
            {
                // Set the column number that is to be sorted; default to ascending.
                this.connectionsListViewColumnSorter.SortColumn = e.Column;
                this.connectionsListViewColumnSorter.Order = SortOrder.Ascending;
            }

            // Perform the sort with these new sort options.
            this.configuredConnectionsListView.Sort();

        }

        private void defaultConnectionButton_Click(object sender, EventArgs e)
        {
            string connectionName = this.GetSelectedConnectionName();
            if (connectionName != null)
            {
                //Update the curent conenction ..this will also force the update of the list views through a call from
                // teh controller
                this._controller.CurrentConnection = this._controller.UserData.Connections[connectionName];
                SelectConfiguredConnectionsListItem(connectionName);
                InvalidateRibbon();

            }
            else //No item selected ..Prompt user
                InformUserToSelectAConnection();
        }

        private void editConfigConnectionButton_Click(object sender, System.EventArgs e)
        {
            string connectionName = GetSelectedConnectionName();
            if (connectionName != null)
            {
                Connection connection = this._controller.UserData.Connections[connectionName];
                this._controller.ShowServerConnectionForm(connection, true);
                // Select Current Connection in Connection Pane 
                SelectConfiguredConnectionsListItem(connectionName);
                InvalidateRibbon();
            }
            else //No item selected ..Prompt user
                InformUserToSelectAConnection();
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object)")]
        private void removeConfigConnectionButton_Click(object sender, EventArgs e)
        {
            string connectionName = GetSelectedConnectionName();
            if (connectionName != null)
            {
                string confirmationMessage = string.Format(Resources.REMOVE_CONNECTION_CONFIRMATION, connectionName);
                if (MessageBox.Show(this, confirmationMessage, Resources.GENEPATTREN_MSG_HEADER,
                    MessageBoxButtons.OKCancel, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.OK)
                {
                    RemoveConnection(connectionName);
                }
            }
            else //No item selected ..Prompt user
                InformUserToSelectAConnection();
        }

        private bool CheckForCurrentConnectionMatch(string connectionName)
        {
            bool result = false;
            string currentConnection = this._controller.UserData.CurrentConnectionName;
            if (!String.IsNullOrEmpty(currentConnection))
            {
                if (currentConnection.Equals(connectionName))
                {
                    this._controller.UserData.CurrentConnectionName = null;
                    result = true;
                }

            }
            return result;
        }

        private void tasksListView_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            // Determine if clicked column is already the column that is being sorted.
            if (e.Column == tasksListViewColumnSorter.SortColumn)
            {
                // Reverse the current sort direction for this column.
                if (tasksListViewColumnSorter.Order == SortOrder.Ascending)
                {
                    tasksListViewColumnSorter.Order = SortOrder.Descending;
                }
                else
                {
                    tasksListViewColumnSorter.Order = SortOrder.Ascending;
                }
            }
            else
            {
                // Set the column number that is to be sorted; default to ascending.
                tasksListViewColumnSorter.SortColumn = e.Column;
                tasksListViewColumnSorter.Order = SortOrder.Ascending;
            }

            // Perform the sort with these new sort options.
            this.tasksListView.Sort();

        }

        private void configuredConnectionsListView_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            bool enable = configuredConnectionsListView.SelectedItems.Count > 0 ? true : false;
            SetButtonsEnabled(enable);
            if (enable)
            {
                string connectionName = this.GetSelectedConnectionName();
                if (connectionName != null)
                {
                    if (string.Compare(this._controller.CurrentConnection.Name, connectionName, 0) == 0)
                    {
                        defaultConnectionButton.Enabled = false;
                    }
                }
            }
        }

        #endregion

        /// <summary>
        /// Enables the buttons.
        /// </summary>
        /// <param name="enabled">if set to <c>true</c> [enabled].</param>
        private void EnableButtons(bool enabled)
        {
            this.addConfigConnectionButton.Enabled = enabled;

            bool enableSubCondition = enabled && this.configuredConnectionsListView.SelectedItems.Count > 0;

            SetButtonsEnabled(enableSubCondition);
        }

        /// <summary>
        /// Removes the connection.
        /// </summary>
        /// <param name="connectionName">Name of the connection.</param>
        private void RemoveConnection(string connectionName)
        {
            if (connectionName != null)
            {
                bool isCurrentConnection = CheckForCurrentConnectionMatch(connectionName);
                bool isConnectionRemoved = _controller.RemoveConnection(connectionName);

                // if we deleted the current connection and we have other connections, set one of them as current
                if (isCurrentConnection)
                {
                    Dictionary<string, Connection>.Enumerator iter = _controller.UserData.Connections.GetEnumerator();
                    iter.MoveNext();
                    if (iter.Current.Value != null)
                    {
                        _controller.CurrentConnection = iter.Current.Value;
                    }
                }

                if (isConnectionRemoved) //see if connection was removed from userData.Connections
                {
                    // Update connections list
                    // If connection removed was current connection we update task list too
                    this.UpdateConnectionsList(isCurrentConnection);

                    Trace.WriteLine("Removed Connection : " + connectionName + " from Connections",
                                    TraceLevel.Info.ToString());

                    // Select Current Connection in Connection Pane 
                    if (null != _controller.CurrentConnection)
                    {
                        SelectConfiguredConnectionsListItem(this._controller.CurrentConnection.Name);
                    }
                    else
                    {
                        // Set CurrentConnection to null again for refreshing Pane
                        _controller.CurrentConnection = null;
                    }
                }
                else
                {
                    Trace.WriteLine("Unable to remove Connection : " + connectionName + " from Connections",
                                    TraceLevel.Warning.ToString());
                }

                InvalidateRibbon();

                RerunPipelineTaskPane rerunPipelineTaskPane = _controller.GetActiveRerunPipelineTaskPane(_documentKey);
                if (rerunPipelineTaskPane != null)
                {
                    rerunPipelineTaskPane.UpdateConnectionsComboBox();
                }
            }
            else //No item selected ..Prompt user
            {
                InformUserToSelectAConnection();
            }
        }

        /// <summary>
        /// Invalidates the ribbon.
        /// </summary>
        private void InvalidateRibbon()
        {
            string controlID = this._controller.UserControlCreator.GetControlId(ControlKey.ConnectionDropDown);
            this._controller.RibbonInvalidator.InvalidateControl(controlID);
        }

        /// <summary>
        /// Gets the name of the selected connection.
        /// </summary>
        /// <returns></returns>
        private string GetSelectedConnectionName()
        {
            string connectionName = null;
            if (this.configuredConnectionsListView.SelectedItems.Count > 0)
                connectionName = this.configuredConnectionsListView.SelectedItems[0].Name;
            return connectionName;
        }

        /// <summary>
        /// Informs the user to select A connection.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1303:DoNotPassLiteralsAsLocalizedParameters", MessageId = "System.Windows.Forms.MessageBox.Show(System.String,System.String,System.Windows.Forms.MessageBoxButtons,System.Windows.Forms.MessageBoxIcon,System.Windows.Forms.MessageBoxDefaultButton)")]
        private static void InformUserToSelectAConnection()
        {
            // TODO resource them
            string text = "Please select a connection from the list.";
            string caption = "No Connection selected!";
            MessageBox.Show(text,
                            caption,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Information,
                            MessageBoxDefaultButton.Button2);
        }

        /// <summary>
        /// Updates the configured connections.
        /// </summary>
        /// <param name="userData">The user data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.Int32.ToString")]
        private void UpdateConfiguredConnections(UserData userData)
        {
            this.configuredConnectionsListView.BeginUpdate();

            this.configuredConnectionsListView.Items.Clear();
            SerializableDictionary<string, Connection> connectionMappings = userData.Connections;

            string currentConnection = this._controller.ActiveConnectionForDocument;
            if (string.IsNullOrEmpty(currentConnection))
            {
                currentConnection = userData.CurrentConnectionName;
            }

            foreach (KeyValuePair<string, Connection> connectionMapping in connectionMappings)
            {
                Connection conn = connectionMapping.Value;
                int imageIndex = (conn.Name == currentConnection) ? 1 : 0;
                ListViewItem connectionItem = new ListViewItem(conn.Name, imageIndex);
                connectionItem.Name = conn.Name;
                
                if(string.IsNullOrEmpty(conn.CompleteServerName))
                    connectionItem.SubItems.Add(conn.ServerName);
                else
                    connectionItem.SubItems.Add(conn.CompleteServerName);

                connectionItem.SubItems.Add(conn.Username);
                connectionItem.SubItems.Add(conn.Port.ToString());
                this.configuredConnectionsListView.Items.Add(connectionItem);
            }

            SetButtonsEnabled(this.configuredConnectionsListView.SelectedItems.Count > 0);

            this.configuredConnectionsListView.EndUpdate();
        }

        /// <summary>
        /// Sets the buttons enabled.
        /// </summary>
        /// <param name="enabled">if set to <c>true</c> [enabled].</param>
        private void SetButtonsEnabled(bool enabled)
        {
            editConfigConnectionButton.Enabled = enabled;
            removeConfigConnectionButton.Enabled = enabled;
            defaultConnectionButton.Enabled = enabled;
        }

        /// <summary>
        /// Updates the task list.
        /// </summary>
        /// <param name="currentConnection">The current connection.</param>
        private void UpdateTaskList(string currentConnection)
        {
            this.currentConnectionTextBox.Text = currentConnection;
            this.tasksLabel.Text = "Pipelines at: " + currentConnection;
            this.tasks = new List<Task>();

            this.jobProcessing1.Visible = true;
            this.tasksListView.Visible = false;

            if (backgroundWorker.IsBusy)
            {
                this.backgroundWorker.CancelAsync();
                this.backgroundWorker.DoWork -= new DoWorkEventHandler(backgroundWorker_DoWork);
                this.backgroundWorker.RunWorkerCompleted -= new RunWorkerCompletedEventHandler(backgroundWorker_RunWorkerCompleted);
                this.backgroundWorker.Dispose();

                this.backgroundWorker = new BackgroundWorker();
                this.backgroundWorker.WorkerSupportsCancellation = true;
                this.backgroundWorker.DoWork += new DoWorkEventHandler(backgroundWorker_DoWork);
                this.backgroundWorker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(backgroundWorker_RunWorkerCompleted);
            }

            this.backgroundWorker.RunWorkerAsync(currentConnection);
        }

        /// <summary>
        /// Updates the task list view.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void UpdateTaskListView(object sender, EventArgs e)
        {
            try
            {
                if (String.IsNullOrEmpty(this.errorMessage))
                {
                    this.tasksListView.Columns[0].Width = this.nameColumnDefaultWidth;
                    this.tasksListView.BeginUpdate();
                    this.tasksListView.Items.Clear();
                    foreach (Task task in this.tasks)
                    {
                        ListViewItem item;
                        if (task.TaskType == TaskType.Pipeline)
                        {

                            item = new ListViewItem(task.Name, "pipeline");
                            item.SubItems.Add(task.TaskType.ToString());
                            item.ToolTipText = task.Name;
                            this.tasksListView.Items.Add(item);
                        }

                    }

                    this.tasksListView.EndUpdate();
                }
            }
            catch (Exception)
            {
                throw;
            }
            finally
            {
                this.errorMessage = null;
                this.jobProcessing1.Visible = false;
                this.tasksListView.Visible = true;
            }
        }

        /// <summary>
        /// Shows the taskretrieval error message.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ShowTaskretrievalErrorMessage(object sender, EventArgs e)
        {
            this.tasksListView.Items.Clear();
            ListViewItem errorItem = new ListViewItem();
            errorItem.ForeColor = Color.Red;
            errorItem.Font = new Font(this.tasksListView.Font, FontStyle.Italic);
            errorItem.Text = this.errorMessage;
            this.tasksListView.Items.Add(errorItem);
            if (tasksListView.Columns.Count > 0)
                this.tasksListView.Columns[0].Width = 250;
        }

        /// <summary>
        /// Sets the dock position.
        /// </summary>
        /// <param name="dockPosition">The dock position.</param>
        public void SetDockPosition(TaskPaneDockPosition dockPosition)
        {
            if ((dockPosition & (TaskPaneDockPosition.Left | TaskPaneDockPosition.Right | TaskPaneDockPosition.Floating)) != 0)
            {
                SetVertical(true);
            }
            else
            {
                SetVertical(false);
            }
        }

        /// <summary>
        /// Sets the vertical.
        /// </summary>
        /// <param name="controlIsVertical">if set to <c>true</c> [control is vertical].</param>
        private void SetVertical(bool controlIsVertical)
        {
            this.SuspendLayout();
            int third;
            if (controlIsVertical)
            {
                third = this.Height / 3;
            }
            else
            {

                third = this.Width / 3;
            }

            // splitcontainter orientation is opposite of the control's orientation
            splitContainer1.Orientation = controlIsVertical ? Orientation.Horizontal : Orientation.Vertical;
            splitContainer1.SplitterDistance = third;


            this.ResumeLayout();
        }

        /// <summary>
        /// Selects the configured connections list item.
        /// </summary>
        /// <param name="connectionName">Name of the connection.</param>
        private void SelectConfiguredConnectionsListItem(string connectionName)
        {
            int index = configuredConnectionsListView.Items.IndexOfKey(connectionName);
            if (index != -1)
            {
                configuredConnectionsListView.Items[index].Selected = true;
            }
        }

        #endregion

    }
}