using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Infusion.GenePattern.Common;

namespace Infusion.GenePattern.Package
{
    public partial class ViewPipelinesUserControl : UserControl
    {
        /// <summary>
        /// Occurs when [association delete button click].
        /// </summary>
        public event EventHandler<EventArgs> AssociationDeleteButtonClick;

        #region Public Methods

        /// <summary>
        /// Initializes a new instance of the <see cref="ViewPipelinesUserControl"/> class.
        /// </summary>
        public ViewPipelinesUserControl()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Updates the association list.
        /// </summary>
        /// <param name="isEdit">if set to <c>true</c> [is edit].</param>
        /// <param name="pipelineAssociationList">The pipeline association list.</param>
        /// <param name="pipelineList">The pipeline list.</param>
        public void UpdateAssociationList(List<PipelineAssociation> pipelineAssociationList,
                            List<Pipeline> pipelineList)
        {
            ClearAssociations();

            PipelineAssociation pa;
            Pipeline p;
            ListViewItem item;
            for (int index = 0; index < pipelineAssociationList.Count; index++)
            {
                pa = pipelineAssociationList[index];
                p = pipelineList[index];
                item = new ListViewItem(pa.Name);
                item.SubItems.Add(p.Name);
                item.SubItems.Add(pa.Annotation);
                item.Tag = index;
                item.ImageKey = p.IsLinked ? "linkedPipeline" : "pipeline";
                Associations.Items.Add(item);
            }
            if (Associations.SelectedItems.Count == 0)
                this.RemoveAssociation.Enabled = false;
        }


        /// <summary>
        /// Determines whether [is association name exist] [the specified asso pipeline name].
        /// </summary>
        /// <param name="AssoPipelineName">Name of the asso pipeline.</param>
        /// <param name="pipelineAssociationList">The pipeline association list.</param>
        /// <returns>
        /// 	<c>true</c> if [is association name exist] [the specified asso pipeline name]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsAssociationNameExist(String AssoPipelineName, List<PipelineAssociation> pipelineAssociationList)
        {
            bool alreadyExistingName = false;
            int count = pipelineAssociationList.Count;
            for (int i = 0; i < count; i++)
            {
                if (string.Compare(AssoPipelineName, pipelineAssociationList[i].Name, false) == 0)
                {
                    alreadyExistingName = true;
                    break;
                }
            }
            return alreadyExistingName;
        }

        /// <summary>
        /// Clears the associations.
        /// </summary>
        public void ClearAssociations()
        {
            Associations.Items.Clear();
        }
        #endregion

        #region Private Method

        /// <summary>
        /// Handles the Click event of the RemoveAssociation control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void RemoveAssociation_Click(object sender, EventArgs e)
        {
            try
            {
                if (Associations.SelectedItems.Count > 0)
                {
                    if (Associations.SelectedItems.Count - Associations.Items.Count == 0)
                    {
                        string message =
                            "Cannot delete all pipeline associations.  Use the previous button to go back and edit it, or cancel to not add any associations.";
                        string title = "Deleting last pipeline association";
                        MessageBox.Show(message, title);
                    }
                    else
                    {
                        foreach (ListViewItem item in Associations.SelectedItems)
                        {
                            if (AssociationDeleteButtonClick != null)
                            {
                                AssociationDeleteButtonClick(item.Tag, EventArgs.Empty);
                                Associations.Items.RemoveAt(item.Index);
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the Associations control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Associations_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.RemoveAssociation.Enabled = Associations.SelectedItems.Count > 0 ? true : false;
        }

        #endregion
    }
}
