using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Security;
using System.Threading;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.ScientificAddIn.Common;
using Microsoft.eResearch.GenePattern.Server;
using System.Text;
using System.Runtime.InteropServices;
using System.Globalization;
using System.Collections;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// User Control for Associating Pipeline
    /// </summary>
    public partial class AssociatePipelineTaskPane : UserControl, ITaskPaneControl
    {

        #region Variable Declaration

        private IController _controller;
        private string _documentKey;
        ImageRange _imageRange = new ImageRange();
        private int _jobSelectionIndex;
        private TaskPaneDockPosition previousDockPosition = TaskPaneDockPosition.Right;
        private TaskPaneKey _currentTaskPaneType;
        private bool useOriginalConnection;
        IGenePatternServer genePatternServer;

        #endregion

        #region Associate Pipeline Variable Declaration

        private int _totalDownloads;
        private int _downloadsComplete;
        private bool _isEditing;
        private bool _isLinked;
        private string _progressMessage = string.Empty;
        private Thread _asyncDownload;
        private JobStatus jobStaus = JobStatus.Unknown;
        private List<Pipeline> _pipelineList = new List<Pipeline>();
        private List<PipelineAssociation> _associationList = new List<PipelineAssociation>();
        private Job _selectedJob;
        private Pipeline _selectedPipeline;
        private string _editPipelineName = string.Empty;
        private DocumentImage _documentImage;

        #endregion

        #region Public Properties

        /// <summary>
        /// Gets or sets a value indicating whether this instance is linked.
        /// </summary>
        /// <value><c>true</c> if this instance is linked; otherwise, <c>false</c>.</value>
        public bool IsLinked
        {
            get { return _isLinked; }
            set { _isLinked = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is edit mode.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is edit mode; otherwise, <c>false</c>.
        /// </value>
        public bool IsEditMode
        {
            get { return _isEditing; }
            set { _isEditing = value; }
        }

        /// <summary>
        /// Used to set the task pane type
        /// </summary>
        public TaskPaneKey CurrentTaskPaneType
        {
            set
            {
                _currentTaskPaneType = value;
            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Initializes a new instance of the <see cref="AssociatePipelineTaskPane"/> class.
        /// </summary>
        public AssociatePipelineTaskPane()
        {
            InitializeComponent();
            txtStatusLabel.Text = string.Empty;
            selectJobsUserControl.JobsSelectedIndexChanged += new EventHandler<EventArgs>(OnJobsSelectedIndexChanged);
            annotationUserControl.AnnotationTextChanged += new EventHandler<EventArgs>(AnnotationTextChanged);
            tcStopPointAndPrimaryResult.SelectedIndex = 2;
        }

        /// <summary>
        /// Sets the controller.
        /// </summary>
        /// <value>The controller.</value>
        public IController Controller
        {
            get
            {
                return this._controller;
            }
            set
            {
                if (value == null)
                    return;

                this._controller = value;
                this._documentKey = _controller.DocumentKey;
                UpdateServerJobs();
            }
        }

        /// <summary>
        /// Edits the existing pipeline.
        /// </summary>
        /// <param name="pa">The pipeline association</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public void EditExistingPipeline(PipelineAssociation pa)
        {
            try
            {
                _pipelineList.Clear();
                _associationList.Clear();

                _pipelineList.Add(_controller.GetDocumentModel(_documentKey).Pipelines[pa.PipelineLsid]);
                _associationList.Add(pa);

                if (_pipelineList.Count > 0)
                    chkLinkPipeline.Checked = _pipelineList[0].IsLinked;

                selectJobsUserControl.SelectJob(pa.Job.Id);
                _editPipelineName = pa.Name;
                annotationUserControl.AssociationName = pa.Name;
                annotationUserControl.Annotation = pa.Annotation;
            }
            catch (Exception ex)
            {
                CancelExecution(true);
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Updates the connection.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public void UpdateConnection(Connection newConnection)
        {
            try
            {
                ClearStepContent();
                selectJobsUserControl.UpdateConnection(newConnection);
            }
            catch (ThreadAbortException) { }
            catch (Exception ex)
            {
                CancelExecution(true);
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Cancels the execution.
        /// </summary>
        public void CancelExecution(bool callAssociationCancel)
        {
            jobStaus = JobStatus.Canceled;

            if (_asyncDownload != null && _asyncDownload.IsAlive && jobStaus != JobStatus.Finished)
            {
                _controller.SetJobProcessingStatus(_documentKey, false);
                timerProgress.Stop();
                downloadAnimationUserControl1.Stop();
                _asyncDownload.Abort();
            }

            StepsAfterJobCancel(callAssociationCancel);
        }

        /// <summary>
        /// Stepses the after job cancel.
        /// </summary>
        /// <param name="CallAssociationCancel">if set to <c>true</c> [call association cancel].</param>
        private void StepsAfterJobCancel(bool CallAssociationCancel)
        {
            if (CallAssociationCancel)
            {
                OnAssociationCompletedOrCancel();
            }
        }

        /// <summary>
        /// Checks the pipeline exists.
        /// </summary>
        /// <returns></returns>
        public bool CheckPipelineExists()
        {
            useOriginalConnection = false;
            Model model = _controller.GetDocumentModel(_documentKey);

            // check existence of pipeline
            if (model.Pipelines.ContainsKey(_selectedJob.PipelineLsid))
            {
                // is this pipeline already embedded AND we are linking?  warn about deletion
                Pipeline original = model.Pipelines[_selectedJob.PipelineLsid];
                if(selectJobsUserControl.IsJobFromDocument)
                {
                    if (!IsPipelineServerSame())
                    {
                        return false; // cancel
                    }
                    useOriginalConnection = true;
                }
                
                
                if (!original.IsLinked && chkLinkPipeline.Checked)
                {
                    if (WarnAlreadyStored())
                    {
                        return false; // cancel
                    }
                }
                else if (original.IsLinked && chkStorePipeline.Checked)
                {
                    if (WarnAlreadyLinkedPipeline())
                    {
                        return false; // cancel
                    }
                }
                else
                {
                    _selectedPipeline.IsLinked = chkLinkPipeline.Checked;

                    if (!_selectedPipeline.IsLinked && original.IsLinked)
                    {
                        if (!_controller.AssureConnection())
                        {
                            return false;
                        }
                    }
                }
            }
            else
            {
                _selectedPipeline.IsLinked = chkLinkPipeline.Checked;
                bool storeOldConnection = _selectedPipeline.IsLinked && selectJobsUserControl.IsJobFromDocument ? true : false;
                StoreOldConnection(storeOldConnection);
            }
            return true;

        }

        /// <summary>
        /// Determines whether [is pipeline server same].
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if [is pipeline server same]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsPipelineServerSame()
        {
            if (selectJobsUserControl.GenePatternServer != null && selectJobsUserControl.GenePatternServer.Connection != null)
            {
                if (_selectedPipeline.LinkConnection.ServerUri != selectJobsUserControl.GenePatternServer.Connection.ServerUri)
                {
                    string message = string.Format(Resources.CONNECT_TO_ORIGINAL_SERVER, _selectedPipeline.LinkConnection.ServerName);
                    if (DialogResult.Cancel == MessageBox.Show(message, Resources.GENEPATTREN_MSG_HEADER,
                                    MessageBoxButtons.OKCancel, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button1))
                    {
                        return false;
                    }
                }
            }

            genePatternServer = _controller.ServerBroker.GetTestedVerifiedServer(_selectedPipeline.LinkConnection, string.Empty, true);
            if (genePatternServer == null)
            {
                return false;
            }
            return true;
        }

        /// <summary>
        /// Refreshes the document jobs.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public void RefreshDocumentJobs()
        {
            try
            {
                if (selectJobsUserControl.IsJobFromDocument && !IsJobExecutionInProgress())
                {
                    ClearStepContent();
                    if (!_isEditing)
                    {
                        _associationList.Clear();
                        _pipelineList.Clear();
                    }
                    selectJobsUserControl.RetrieveJobs();
                }
            }
            catch (Exception ex)
            {
                CancelExecution(true);
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Determines whether [is job execution in progress].
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if [is job execution in progress]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsJobExecutionInProgress()
        {
            if (_asyncDownload == null)
                return false;

            if (_controller.GetDocumentModel(_documentKey).TransactionActive ||
                _asyncDownload.ThreadState == System.Threading.ThreadState.Running)
            {
                return true;
            }
            return false;
        }
        
        /// <summary>
        /// Sets the annotation tab as default.
        /// </summary>
        public void SetDefaultValues()
        {
            tcStopPointAndPrimaryResult.SelectedIndex = 2;
            if (!IsJobExecutionInProgress())
            {
                if (!_isEditing)
                {
                    selectJobsUserControl.SetJobFromServer(true);
                }
                else
                {
                    selectJobsUserControl.IsJobFromDocument = true;
                }
            }
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Updates the server jobs.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void UpdateServerJobs()
        {
            try
            {
                selectJobsUserControl.InitialUpdate(_controller, _documentKey, _isEditing);
            }
            catch (Exception ex)
            {
                CancelExecution(true);
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Clears the content of the step.
        /// </summary>
        private void ClearStepContent()
        {
            stopPointUserControl.ClearStopPoint();
            primaryResultUserControl.ClearPrimaryResult();
            showParametersUserControl.ClearParameterList();
            if (!_isEditing)
            {
                ClearAnnotationText();
            }
            ClearStatusText();
            SetButtonStatus(false);

        }

        /// <summary>
        /// Clears the status text.
        /// </summary>
        private void ClearStatusText()
        {
            txtStatusLabel.Text = string.Empty;
        }

        /// <summary>
        /// Sets the button statu.
        /// </summary>
        /// <param name="isEnable">if set to <c>true</c> [is enable].</param>
        private void SetButtonStatus(bool isEnable)
        {
            bool enableAddToListButton = isEnable && !_isEditing;
            enableAddToListButton = enableAddToListButton && ValidateSelection();

            bool enableAssociateButton = isEnable || (_associationList.Count > 0 && !_isEditing) || ValidateSelection();
            btnAssociate.Enabled = enableAssociateButton;
        }

        /// <summary>
        /// Validates the selection.
        /// </summary>
        /// <returns></returns>
        private bool ValidateSelection()
        {
            bool isValide = true;

            if (_selectedJob == null)
                isValide = false;
            else if (annotationUserControl.AssociationName.Length == 0)
                isValide = false;
            else if (annotationUserControl.Annotation.Length == 0)
                isValide = false;
            else if (primaryResultUserControl.PrimaryResultSelectedIndex < 0)
                isValide = false;
            else if (stopPointUserControl.SelectedStopPoint < 0)
                isValide = false;

            return isValide;
        }

        /// <summary>
        /// Stores the old connection.
        /// </summary>
        private void  StoreOldConnection(bool storeOldConnection)
        {
            //Store Old Connection. Else store current connection
            if (storeOldConnection)
            {
                // linked from a previous connection
                Pipeline thePipeline = _controller.GetDocumentModel(_documentKey).Pipelines[_selectedJob.PipelineLsid];
                _selectedPipeline.LinkConnection = thePipeline.LinkConnection;
                _selectedPipeline.LinkConnection.Password = null;
            }
            else
            {
                _selectedPipeline.LinkConnection = _controller.CurrentConnection.Clone() as Connection;
                //delete users creds for security reasons
                _selectedPipeline.LinkConnection.Password = String.Empty;
            }
        }

        /// <summary>
        /// Warns the over writing connection.
        /// </summary>
        /// <param name="linkConnection">The link connection.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object,System.Object)")]
        private void WarnOverWritingConnection(Connection linkConnection)
        {
            // alert user that connection is being overwritten
            // compare the connection currently in the pipeline to the users current connection

            if (linkConnection.ServerUri != _controller.CurrentConnection.ServerUri)
            {
                string oldUrl = linkConnection.ServerName + ":" + linkConnection.Port;
                string newUrl = _controller.CurrentConnection.ServerName + ":" + _controller.CurrentConnection.Port;
                string message = String.Format(Resources.PIPELINE_LINKED_TO_DIFFERENT_SERVER, oldUrl, newUrl);

                if (DialogResult.Yes == MessageBox.Show(message, Resources.UPDATE_LINK_CAPTION,
                    MessageBoxButtons.YesNo, MessageBoxIcon.Warning))
                {
                    StoreOldConnection(false);
                }
            }
        }

        /// <summary>
        /// Warns the user a linked pipeline is already embedded.  Returns true if they cancel the operation
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object)")]
        private bool WarnAlreadyStored()
        {
            string formatString = Resources.REMOVE_EMBEDDED_PIPELINE_MESSAGE; // TODO localize
            DialogResult result =
                MessageBox.Show(String.Format(formatString, _selectedPipeline.Name), Resources.PIPELINE_EMBEDDED_CAPTION,
                                MessageBoxButtons.YesNoCancel, MessageBoxIcon.Warning);

            _selectedPipeline.IsLinked = result == DialogResult.Yes;

            switch (result)
            {
                case DialogResult.Yes:
                    StoreOldConnection(false);
                    break;
                case DialogResult.Cancel:
                    return true;
                case DialogResult.No:
                    chkStorePipeline.Checked = true;
                    break;
            }

            return false;
        }

        /// <summary>
        /// Warns the user a linked pipeline is already embedded.  Returns true if they cancel the operation
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object)")]
        private bool WarnAlreadyLinkedPipeline()
        {
            string formatString = Resources.REMOVE_LINKED_PIPELINE; // TODO localize
            DialogResult result =
                MessageBox.Show(String.Format(formatString, _selectedPipeline.Name), Resources.PIPELINE_LINKED_CAPTION,
                                MessageBoxButtons.YesNoCancel, MessageBoxIcon.Warning);

            switch (result)
            {
                case DialogResult.Yes:
                    _selectedPipeline.IsLinked = false;
                    break;
                case DialogResult.Cancel:
                    return true;
                case DialogResult.No:
                    chkLinkPipeline.Checked = false;
                    _selectedPipeline.IsLinked = true;
                    break;
            }

            return false;
        }

        /// <summary>
        /// Downloads the files.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void DownloadFiles()
        {
            try
            {

                _controller.BeginModelChanges(_documentKey);
                List<string> messages = null;


                if (_isEditing)
                {
                    messages = _controller.UpdatePipelineAssociations(_pipelineList, _associationList,
                             new DownloadProgress(UpdateDownloadStatus), _documentKey,
                             genePatternServer);
                    
                    if (_editPipelineName != _associationList[0].Name)
                    {
                        _controller.UpdateRerunResultAssociationName(_associationList[0].Name,
                            _documentImage.ImageKey);
                    }
                }
                else
                {
                    messages = _controller.AddPipelineAssociations(_pipelineList, _associationList,
                             new DownloadProgress(UpdateDownloadStatus), _documentKey, _documentImage,
                             genePatternServer);
                }
                _controller.CommitModelChanges(_documentKey);

                if (messages != null && messages.Count > 0)
                {
                    string message = string.Join(Environment.NewLine + Environment.NewLine, messages.ToArray());
                    if (message.Length > 0)
                    {
                        MessageBox.Show(message, Resources.ATTENTION, MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                }

                if (!_controller.JobStatus && !_isEditing)
                {
                    jobStaus = JobStatus.Failed;
                }
                else
                {
                    jobStaus = JobStatus.Finished;
                }
            }
            catch (ThreadAbortException)
            {
                // user clicked cancel
                jobStaus = JobStatus.Failed;
            }
            // NO need to handle the exception.
            catch (COMException)
            {
                jobStaus = JobStatus.Failed;
                _controller.SetJobProcessingStatus(_documentKey, false);
            }
            catch (Exception ex)
            {
                jobStaus = JobStatus.Failed;
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
            finally
            {
                if (jobStaus != JobStatus.Processing)
                {
                    if (_controller.GetDocumentModel(_documentKey).TransactionActive)
                    {
                        _controller.RollbackModelChanges(_documentKey);
                    }
                }
            }
        }

        /// <summary>
        /// Updates the download.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="lsid">The lsid.</param>
        /// <param name="isStarting">if set to <c>true</c> [is starting].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object)")]
        private void UpdateDownloadStatus(string name, string lsid, bool isStarting)
        {
            if (isStarting)
            {
                string startMessage = "Processing {0}...";//Resources.START_PROCESSING;
                _progressMessage = String.Format(startMessage, name);
            }
            else
            {
                string finishedMessage = "Processing {0}...";//Resources.FINISH_PROCESSING;
                _progressMessage = String.Format(finishedMessage, name);
                _downloadsComplete++;
            }
        }

        /// <summary>
        /// Counts the pipeline pieces.
        /// </summary>
        /// <param name="p">The p.</param>
        /// <returns></returns>
        private static int CountPipelinePieces(Pipeline p)
        {
            int count = 1;
            if (!p.IsLinked)
            {
                count += p.Steps.Count;
            }
            return count;
        }

        /// <summary>
        /// Adds the association.
        /// </summary>
        private PipelineAssociation CreatePipelineAssociation()
        {
            PipelineAssociation pipelineAssociation = new PipelineAssociation();
            pipelineAssociation.Job = _selectedJob;
            pipelineAssociation.UserStopPoint = stopPointUserControl.SelectedStopPoint;
            pipelineAssociation.PrimaryResultName = primaryResultUserControl.GetSelectedPrimaryResult;
            pipelineAssociation.Name = annotationUserControl.AssociationName;
            pipelineAssociation.Annotation = annotationUserControl.Annotation;
            return pipelineAssociation;
        }

        /// <summary>
        /// Adds the association to list.
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private bool AddAssociationToList()
        {
            if (!(_isEditing))
            {
                if (IsAssociationNameExist())
                {
                    MessageBox.Show(Resources.ASSOCIATION_ALREADY_EXISTS,
                        Resources.GENEPATTREN_MSG_HEADER, MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    return false;
                }
            }

            bool performOperation = CheckPipelineExists();
            if (performOperation)
            {
                if (_isEditing)
                {
                    _associationList[0].Job = _selectedJob.Clone() as Job;
                    _associationList[0].UserStopPoint = stopPointUserControl.SelectedStopPoint;
                    _associationList[0].PrimaryResultName = primaryResultUserControl.GetSelectedPrimaryResult;
                    _pipelineList[0] = _selectedPipeline.Clone() as Pipeline;
                }
                else
                {
                    _associationList = new List<PipelineAssociation>();
                    _pipelineList = new List<Pipeline>();
                    _associationList.Add(CreatePipelineAssociation());
                    _pipelineList.Add(_selectedPipeline);
                }
            }
            return performOperation;
        }

        /// <summary>
        /// Called when [association start].
        /// </summary>
        private void OnAssociationStart()
        {

            #region CheckConnection

            if(!useOriginalConnection)
            {
                genePatternServer = selectJobsUserControl.GenePatternServer;
            }

            #endregion

            splitContainerTop.Enabled = false;
            pnlStorePipeline.Enabled = false;
            btnAssociate.Enabled = false;

            progressBar.Visible = true;
            downloadAnimationUserControl1.Visible = true;
            btnCancel.Enabled = true;

            jobStaus = JobStatus.Unknown;
            _asyncDownload.Start();
            timerProgress.Start();
            downloadAnimationUserControl1.Start();
        }

        /// <summary>
        /// Called when [association completed or cancel].
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void OnAssociationCompletedOrCancel()
        {
            btnCancel.Enabled = false;
            timerProgress.Stop();
            downloadAnimationUserControl1.Stop();
            progressBar.Value = progressBar.Maximum;

            progressBar.Visible = false;
            downloadAnimationUserControl1.Visible = false;

            OnJobFinished();

            txtStatusLabel.Text = GetStatusMessage();
            if (jobStaus == JobStatus.Finished)
            {
                MessageBox.Show(this, txtStatusLabel.Text, "Status",
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                txtStatusLabel.Text = string.Empty;
            }
           
            splitContainerTop.Enabled = true;
            pnlStorePipeline.Enabled = true;
            _controller.ScrollToImage(_documentImage, _documentKey);

            string paneTitle = _controller.UserControlCreator.GetTitle(_currentTaskPaneType);
            _controller.TaskPaneManager.SetTaskPaneVisibility(paneTitle, false, _documentKey);
          
            MemoryManagement.FlushMemory();
        }

        /// <summary>
        /// Called when [job finished].
        /// </summary>
        private void OnJobFinished()
        {
            if (jobStaus == JobStatus.Finished)
            {
                selectJobsUserControl.IsJobFromDocument = true;

                if (IsLinked)
                {
                    _controller.InsertGenePatternLinkFromPane(_associationList, _documentKey, _imageRange);
                }
                else
                {
                    _controller.UpdatePipelineAssociationsDisplay(_documentImage, _documentKey);
                }

                if (_isEditing)
                {
                    selectJobsUserControl.SetSelectedIndex(_jobSelectionIndex);
                    _editPipelineName = _associationList[0].Name;
                }
                else
                {
                    _associationList.Clear();
                    _pipelineList.Clear();
                    ClearAnnotationText();
                }
            }

        }

        /// <summary>
        /// Resets the values.
        /// </summary>
        private void ResetValues()
        {
            _imageRange.Start = 0;
            _imageRange.End = 0;
            _totalDownloads = 0;
        }

        /// <summary>
        /// Gets the status message.
        /// </summary>
        /// <returns></returns>
        private string GetStatusMessage()
        {
            switch (jobStaus)
            {
                case JobStatus.Finished:
                    SetButtonStatus(false);
                    return Resources.JOB_COMPLETED;
                case JobStatus.Canceled:
                    SetButtonStatus(true);
                    return Resources.JOB_CANCELED;
                case JobStatus.Failed:
                    SetButtonStatus(true);
                    return Resources.JOB_FAILED;
                default:
                    return string.Empty;
            }
        }

        /// <summary>
        /// Starts the association.
        /// </summary>
        private void StartAssociation()
        {

            if (_isEditing)
            {
                _associationList[0].Name = annotationUserControl.AssociationName;
                _associationList[0].Annotation = annotationUserControl.Annotation;
            }

            // kick off the background worker when form opens
            _asyncDownload = new Thread(new ThreadStart(DownloadFiles));
            _asyncDownload.Name = "Download Pipelines";
            _asyncDownload.SetApartmentState(ApartmentState.STA);
            OnAssociationStart();


            //set total downloads by counting pipelines and steps
            foreach (Pipeline p in _pipelineList)
            {
                _totalDownloads += CountPipelinePieces(p);
            }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        private bool IsAssociationNameExist()
        {
            bool alreadyExistingName = false;

            IDictionaryEnumerator ie = _controller.GetDocumentModel(_documentKey).PipelineAssociations.GetEnumerator();

            string assoPipelineModelName = string.Empty;
            while (ie.MoveNext())
            {
                assoPipelineModelName = (ie.Value as PipelineAssociation).Name;
                if (assoPipelineModelName.Equals(annotationUserControl.AssociationName))
                {
                    alreadyExistingName = true;
                    break;
                }
            }

            return alreadyExistingName;
        }

       

        #endregion

        #region Event Handler

        /// <summary>
        /// Called when [jobs selected index changed].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void OnJobsSelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                _selectedJob = selectJobsUserControl.GetJob;
                if (_selectedJob != null)
                {
                    Cursor.Current = Cursors.WaitCursor;

                    _selectedPipeline = selectJobsUserControl.GetPipeline(_selectedJob.PipelineLsid, _documentKey);
                    _pipelineList.Clear();
                    _associationList.Clear();

                    primaryResultUserControl.UpdatePrimaryResult(_selectedJob, _selectedPipeline);
                    showParametersUserControl.UpdateShowParameterList(_selectedPipeline.Parameters, _selectedJob.Parameters);

                    if (_isEditing)
                    {
                        PipelineAssociation pipelineAssociation = _associationList[0].Clone() as PipelineAssociation;
                        pipelineAssociation.Job = _selectedJob;

                        if (selectJobsUserControl.IsJobFromDocument)
                        {
                            _jobSelectionIndex = selectJobsUserControl.GetSelectedIndex;
                        }
                        stopPointUserControl.UpdateStopPoint(_selectedPipeline, pipelineAssociation.UserStopPoint);
                    }
                    else
                    {
                        stopPointUserControl.UpdateStopPoint(_selectedPipeline, _selectedJob.StopPoint);
                    }

                    bool isValide = ValidateSelection();
                    SetButtonStatus(isValide);

                    Cursor.Current = Cursors.Default;
                }
                else
                {
                    ClearStepContent();
                }
            }
            catch (Exception ex)
            {
                CancelExecution(true);
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Clears the annotation text.
        /// </summary>
        private void ClearAnnotationText()
        {
            annotationUserControl.Annotation = string.Empty;
            annotationUserControl.AssociationName = string.Empty;
        }

        /// <summary>
        /// Annotations the text changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void AnnotationTextChanged(object sender, EventArgs e)
        {
            try
            {
                bool isValide = ValidateSelection();
                SetButtonStatus(isValide);
            }
            catch (Exception ex)
            {
                CancelExecution(true);
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Associations the delete button click.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        void AssociationDeleteButtonClick(object sender, EventArgs e)
        {
            try
            {
                int index = (int)sender;

                if (index != -1 && index < _associationList.Count)
                {
                    _pipelineList.RemoveAt(index);
                    _associationList.RemoveAt(index);
                }

                txtStatusLabel.Text = Resources.JOB_DELETED_FROM_ASSOCIATION_LIST;
                if (_associationList.Count == 0 && !ValidateSelection())
                {
                    btnAssociate.Enabled = false;
                }
            }
            catch (Exception ex)
            {
                CancelExecution(true);
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the Click event of the btnAssociate control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        private void btnAssociate_Click(object sender, EventArgs e)
        {
            try
            {
                if (_controller.CanPipelineAssociate(_isLinked) == false)
                    return;

                #region Check any other Job execution is in progress
                
                // Check if any other Transaction is Active on Current Model
                if (_controller.IsJobProcessingInProgress(_documentKey))
                {
                    MessageBox.Show(Resources.JOB_IN_PROGRESS, Resources.GENEPATTREN_MSG_HEADER,
                        MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }

                #endregion

                #region If Edit Mode Check if Edited Name is not already present

                if (_isEditing)
                {
                    _associationList[0].Name = _editPipelineName;

                    if (!annotationUserControl.AssociationName.Equals(_editPipelineName))
                    {
                        if (IsAssociationNameExist())
                        {
                            MessageBox.Show(Resources.ASSOCIATION_ALREADY_EXISTS, Resources.GENEPATTREN_MSG_HEADER,
                                MessageBoxButtons.OK, MessageBoxIcon.Warning);
                            return;
                        }
                    }
                }

                #endregion

                // Reset the Values
                ResetValues();

                #region Store Range/DocumentImage

                if (_isLinked)
                    _imageRange = _controller.GetRange;
                else
                    _documentImage = _controller.GetDocumentImage;

                #endregion

                #region Validate the Current Selection and Add to Associate list is not already present

                if (ValidateSelection())
                {
                    if (!AddAssociationToList())
                    {
                        return;
                    }
                }

                #endregion

                _controller.SetJobProcessingStatus(_documentKey, true);

                //Start the Association
                StartAssociation();

            }
            catch (Exception ex)
            {
                CancelExecution(true);
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the Tick event of the timerProgress control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void timerProgress_Tick(object sender, EventArgs e)
        {
            progressBar.Minimum = 0;
            progressBar.Maximum = _totalDownloads;

            if (_downloadsComplete > progressBar.Maximum)
            {
                progressBar.Maximum = _downloadsComplete;
            }

            progressBar.Value = _downloadsComplete;
            txtStatusLabel.Text = _progressMessage;

            if (jobStaus == JobStatus.Failed)
            {
                StepsAfterJobCancel(true);
                _controller.SetJobProcessingStatus(_documentKey, false);
                return;
            }

            if (!_asyncDownload.IsAlive)
            {
                OnAssociationCompletedOrCancel();
                _controller.SetJobProcessingStatus(_documentKey, false);
            }
        }

        /// <summary>
        /// Handles the Click event of the btnCancel control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void btnCancel_Click(object sender, EventArgs e)
        {
            try
            {
                btnCancel.Enabled = false;
                CancelExecution(true);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        #endregion

        #region ITaskPaneControl Members

        /// <summary>
        /// Sets the dock position.
        /// </summary>
        /// <param name="dockPosition">The dock position.</param>
        public void SetDockPosition(TaskPaneDockPosition dockPosition)
        {
            if ((dockPosition & (TaskPaneDockPosition.Left | TaskPaneDockPosition.Right | TaskPaneDockPosition.Floating)) != 0)
            {
                SetVertical(true);
            }
            else
            {
                SetVertical(false);
            }
            previousDockPosition = dockPosition;
        }

        /// <summary>
        /// Sets the vertical.
        /// </summary>
        /// <param name="controlIsVertical">if set to <c>true</c> [control is vertical].</param>
        private void SetVertical(bool controlIsVertical)
        {
            this.SuspendLayout();

            int initialSplitterDistance = controlIsVertical ==
                true ? splitContainerTop.MinimumSize.Height : splitContainerTop.MinimumSize.Width;

            splitContainerLowerBottom.SplitterDistance = initialSplitterDistance;
            splitContainerBottom.SplitterDistance = initialSplitterDistance;
            splitContainerTop.SplitterDistance = initialSplitterDistance;

            Orientation orientation = controlIsVertical ? Orientation.Horizontal : Orientation.Vertical;

            splitContainerTop.Orientation = orientation;
            splitContainerBottom.Orientation = orientation;
            splitContainerLowerBottom.Orientation = orientation;

            if (controlIsVertical)
            {
               
                splitContainerTop.SplitterDistance = (int)(this.Height * 0.44);
                splitContainerComplete.SplitterDistance = (int)(this.Height * 0.78);
                splitContainerLowerBottom.SplitterDistance = (int)(this.Height * 0.06);
                splitContainerBottom.SplitterDistance = (int)(this.Height * 0.08);
            }
            else
            {
                splitContainerComplete.FixedPanel = FixedPanel.None;
                int splitterDistance = (this.Width / 2);
                splitContainerTop.SplitterDistance = splitterDistance;
                splitContainerComplete.SplitterDistance = (int)(this.Height * 0.70);
                splitContainerBottom.SplitterDistance = (int)(this.Height * 0.16);
                splitContainerLowerBottom.SplitterDistance = (int)(this.Height * 0.10);

                if (previousDockPosition == TaskPaneDockPosition.Floating)
                {
                    splitContainerComplete.SplitterDistance = (int)(this.Height * 0.69);
                    splitContainerBottom.SplitterDistance = (int)(this.Height * 0.30);
                    splitContainerLowerBottom.SplitterDistance = (int)(this.Height * 0.22);
                }
            }

            this.ResumeLayout();
        }

        #endregion

    }
}
