using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using System.IO;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// 
    /// </summary>
    public partial class EditParametersUserControl : UserControl
    {
        /// <summary>
        /// used to store the job parameter of the last render
        /// for comparison, to avoid updating the tablelayoutpanel
        /// </summary>
        private JobParameterList lastJobParameterList;
        private Job currentJob;
        private Pipeline currentPipeline;
        private string currentDirectory = string.Empty;
        List<bool> isInputFilesPresent = new List<bool>();

        /// <summary>
        /// Gets the edited pipeline
        /// </summary>
        public Pipeline EditParameterPipeline
        {
            get { return currentPipeline; }
        }

        /// <summary>
        /// Gets the edited Job
        /// </summary>
        public Job EditParameterJob
        {
            get { return currentJob; }
        }

        /// <summary>
        /// Occurs when [Filename Changes].
        /// </summary>
        public event EventHandler<EventArgs> MakeRerunButtonEnable;

        /// <summary>
        /// Occurs when [Reset button is pressed].
        /// </summary>
        public event EventHandler<EventArgs> ResetParameter;

        /// <summary>
        /// Initializes a new instance of the <see cref="EditParametersUserControl"/> class.
        /// </summary>
        public EditParametersUserControl()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Gets a value indicating whether this instance is input files present.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is input files present; otherwise, <c>false</c>.
        /// </value>
        public bool IsInputFilesPresent
        {
            get { return !isInputFilesPresent.Contains(false); }
        }

        /// <summary>
        /// Updates the call.
        /// </summary>
        /// <param name="job">The job.</param>
        /// <param name="pipeline">The pipeline.</param>
        internal void UpdateCall(Job job, Pipeline pipeline)
        {
            currentJob = job;
            currentPipeline = pipeline;
            JobParameterList currentJobParameterList = GetJobParameterList(job);

            if (JobParameterList.Equals(currentJobParameterList, lastJobParameterList))
                return;

            if (!string.IsNullOrEmpty(job.PipelineLsid))
            {
                if (pipeline.Parameters.Count > 0)
                {
                    resetButton.Visible = true;
                    PopulateParameters(job, pipeline);
                }
                else
                {
                    resetButton.Visible = false;
                    ShowNoParameters();
                }
                SetLastJobParameterList(job);
            }
        }

        /// <summary>
        /// Shows the no parameters.
        /// </summary>
        private void ShowNoParameters()
        {
            tableLayoutPanel1.SuspendLayout();
            tableLayoutPanel1.Controls.Clear();

            tableLayoutPanel1.RowCount = 1;
            Label labelControl = new Label();
            labelControl.Text = Resources.NO_PARAMETERS;
            tableLayoutPanel1.Controls.Add(labelControl, 0, 0);
            tableLayoutPanel1.SetColumnSpan(labelControl, 2);

            tableLayoutPanel1.ResumeLayout();
        }

        /// <summary>
        /// Populates the parameters.
        /// </summary>
        /// <param name="job">The job.</param>
        /// <param name="pipeline">The pipeline.</param>
        private void PopulateParameters(Job job, Pipeline pipeline)
        {
            tableLayoutPanel1.SuspendLayout();
            tableLayoutPanel1.Controls.Clear();
            tableLayoutPanel1.RowCount = pipeline.Parameters.Count;
            isInputFilesPresent.Clear();
            bool isInputFile;

            for (int j = 0; j < pipeline.Parameters.Count; j++)
            {
                isInputFile = false;
                PipelineParameter pipelineParameter = pipeline.Parameters[j];
                Label nameControl = new Label();
                
                nameControl.Left = 4;
                Padding padding = new Padding(nameControl.Margin.All);
                padding.Top = 8;
                nameControl.Margin = padding;
                nameControl.Anchor = AnchorStyles.Left | AnchorStyles.Top | AnchorStyles.Right | AnchorStyles.Bottom;
                nameControl.Text = pipelineParameter.Optional ? pipelineParameter.ShortName : pipelineParameter.ShortName + "*";
                nameControl.AutoSize = true;

                Control valueControl = null;
                JobParameter jobParameter = job.GetParameter(pipelineParameter.Name);
                if (pipelineParameter.Type == ParameterType.String)
                {
                    TextBox textBox = new TextBox();
                    textBox.TextChanged += new EventHandler(valueControl_TextChanged);
                    if (jobParameter != null && jobParameter.ParamValue != null)
                    {
                        textBox.Text = jobParameter.ParamValue.ToString();
                    }
                    valueControl = textBox;
                    valueControl.Left = 4;
                    ConfigureControlLayout(pipelineParameter, valueControl);
                }

                else if (pipelineParameter.Type == ParameterType.StringCollection)
                {
                    ComboBox comboBox = new ComboBox();
                    comboBox.DropDownStyle = ComboBoxStyle.DropDownList;
                    comboBox.SelectedIndexChanged += new EventHandler(comboBox_SelectedIndexChanged);
                    comboBox.Items.Clear();
                    int index = 0;
                    foreach (Choice choice in pipelineParameter.Choices)
                    {
                        comboBox.Items.Add(choice);

                        if (jobParameter != null
                            && jobParameter.ParamValue != null
                            && choice.Value == jobParameter.ParamValue.ToString())
                        {
                            comboBox.SelectedIndex = index;
                        }
                        index++;
                    }
                    comboBox.ValueMember = "Value";
                    comboBox.DisplayMember = "Text";
                    valueControl = comboBox;
                    ConfigureControlLayout(pipelineParameter, valueControl);
                }

                else if (pipelineParameter.Type == ParameterType.InputFile)
                {
                    isInputFile = true;
                    InputFileParameterUserControl fileParameterUserControl =
                        new InputFileParameterUserControl();
                    fileParameterUserControl.ControlId = j;
                    
                    fileParameterUserControl.TextChanged += new EventHandler(filenameChanged);
                    fileParameterUserControl.OpenFileDialogShown += new EventHandler<OpenFileDialogShownEventArgs>(OpenFileDialogShown);
                    fileParameterUserControl.OpenFileDialogClosed += new EventHandler<OpenFileDialogClosedEventArgs>(OpenFileDialogClosed);

                    if (jobParameter != null && jobParameter.ParamValue != null)
                    {
                        if (jobParameter.ParamValue is BaseFile)
                        {
                            fileParameterUserControl.IsFile = true;
                        }
                        else
                        {
                            fileParameterUserControl.IsFile = false;
                        }

                        fileParameterUserControl.Text = jobParameter.ParamValue.ToString();
                        isInputFilesPresent.Add(true);
                    }
                    valueControl = fileParameterUserControl;
                    ConfigureControlLayout(pipelineParameter, valueControl);
                }

                #region Add Controls

                if (isInputFile)
                {
                    tableLayoutPanel1.Controls.Add(nameControl, 0, j);
                    tableLayoutPanel1.SetColumnSpan(nameControl, 2);
                    tableLayoutPanel1.Controls.Add(valueControl, 1, j);
                    tableLayoutPanel1.SetColumnSpan(valueControl, 2);
                }
                else
                {
                    tableLayoutPanel1.Controls.Add(nameControl, 0, j);
                    tableLayoutPanel1.Controls.Add(valueControl, 1, j);
                }

                #endregion
                
            }
            tableLayoutPanel1.ResumeLayout();
        }

        /// <summary>
        /// Configures the control layout.
        /// </summary>
        /// <param name="pipelineParameter">The pipeline parameter.</param>
        /// <param name="valueControl">The value control.</param>
        private static void ConfigureControlLayout(PipelineParameter pipelineParameter, Control valueControl)
        {
            valueControl.Anchor = AnchorStyles.Left | AnchorStyles.Top | AnchorStyles.Right;
            valueControl.Tag = pipelineParameter;
        }

        /// <summary>
        /// Gets the job parameter list.
        /// </summary>
        /// <param name="job">The job.</param>
        /// <returns></returns>
        private static JobParameterList GetJobParameterList(Job job)
        {
            JobParameterList result = new JobParameterList();
            if (job != null)
            {
                foreach (JobParameter jobParameter in job.Parameters)
                {
                    result.JobParameters.Add(jobParameter);
                }
            }
            return result;
        }

        /// <summary>
        /// Filenames the changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void filenameChanged(object sender, EventArgs e)
        {
            try
            {
                InputFileParameterUserControl inputFileParameterUserControl = sender as InputFileParameterUserControl;
                if (inputFileParameterUserControl == null)
                    return;

                Job job = currentJob;
                PipelineParameter pipelineParameter = inputFileParameterUserControl.Tag as PipelineParameter;
                if (pipelineParameter != null)
                {
                    JobParameter jobParameter = job.GetParameter(pipelineParameter.Name);
                    if (string.IsNullOrEmpty(inputFileParameterUserControl.Text))
                    {
                        jobParameter.ParamValue = null;
                        if (inputFileParameterUserControl.IsFile)
                        {
                            isInputFilesPresent[inputFileParameterUserControl.ControlId] = false;
                        }
                    }
                    else
                    {
                        
                        if (inputFileParameterUserControl.IsFile)
                        {
                            LocalFile localFile = new LocalFile();
                            localFile.FullPath = inputFileParameterUserControl.Text;
                            localFile.Filename = Path.GetFileName(localFile.FullPath);
                            jobParameter.ParamValue = localFile;
                            isInputFilesPresent[inputFileParameterUserControl.ControlId] = true;
                        }
                        else
                        {
                            try
                            {
                                jobParameter.ParamValue = new SerializableUri(inputFileParameterUserControl.Text);
                            }
                            catch (UriFormatException)
                            {
                                jobParameter.ParamValue = inputFileParameterUserControl.Text;
                                MessageBox.Show("Invalid URI. Please enter valid URI.", Resources.GENEPATTREN_MSG_HEADER,
                                                                           MessageBoxButtons.OK, MessageBoxIcon.Information);
                            }
                        }
                       
                    }

                    MakeRerunButtonEnable(IsInputFilesPresent, EventArgs.Empty);
                }
                SetLastJobParameterList(job);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Sets the last job parameter list.
        /// </summary>
        /// <param name="job">The job.</param>
        private void SetLastJobParameterList(Job job)
        {
            JobParameterList jobParameterList = GetJobParameterList(job);
            lastJobParameterList = jobParameterList.Clone() as JobParameterList;
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the comboBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        void comboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                ComboBox comboBox = sender as ComboBox;
                if (comboBox == null)
                    return;

                Job job = currentJob;
                PipelineParameter pipelineParameter = comboBox.Tag as PipelineParameter;
                if (pipelineParameter != null)
                {
                    JobParameter jobParameter = job.GetParameter(pipelineParameter.Name);
                    if (comboBox.SelectedItem != null)
                    {
                        jobParameter.ParamValue = ((Choice)comboBox.SelectedItem).Value;
                    }
                    else
                    {
                        jobParameter.ParamValue = null;
                    }
                }
                SetLastJobParameterList(job);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the TextChanged event of the valueControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        void valueControl_TextChanged(object sender, EventArgs e)
        {
            try
            {
                TextBox textBox = sender as TextBox;
                if (textBox == null)
                    return;

                Job job = currentJob;
                PipelineParameter pipelineParameter = textBox.Tag as PipelineParameter;
                if (pipelineParameter != null)
                {
                    JobParameter jobParameter = job.GetParameter(pipelineParameter.Name);
                    jobParameter.ParamValue = textBox.Text;
                }
                SetLastJobParameterList(job);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the Click event of the resetButton control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void resetButton_Click(object sender, EventArgs e)
        {
            try
            {
                ResetParameter(this, EventArgs.Empty);
                MakeRerunButtonEnable(true, EventArgs.Empty);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Clears the parameters.
        /// </summary>
        internal void ClearParameters()
        {
            tableLayoutPanel1.SuspendLayout();
            tableLayoutPanel1.Controls.Clear();
            tableLayoutPanel1.ResumeLayout();
            lastJobParameterList = null;
        }

        /// <summary>
        /// Opens the file dialog shown.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="Microsoft.eResearch.GenePattern.Common.OpenFileDialogShownEventArgs"/> instance containing the event data.</param>
        private void OpenFileDialogShown(object sender, OpenFileDialogShownEventArgs e)
        {
            e.IntialDirectory = currentDirectory;
        }

        /// <summary>
        /// Opens the file dialog closed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="Microsoft.eResearch.GenePattern.Common.OpenFileDialogClosedEventArgs"/> instance containing the event data.</param>
        private void OpenFileDialogClosed(object sender, OpenFileDialogClosedEventArgs e)
        {
            currentDirectory = e.CurrentDirectory;
        }
    }
}
