using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Package;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.ScientificAddIn.Common;
using System.Globalization;
using System.Threading;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace Microsoft.eResearch.GenePattern.Package
{
    public partial class RerunPipelineTaskPane : UserControl, ITaskPaneControl
    {
        #region Private Members

        private IController _controller;
        private string _documentKey;
        private Job _selectedJob;
        private Pipeline _selectedPipeline;
        private PipelineAssociation _selectedPipelineAssociation;
        private Connection preVerifiedConnection;
        private int jobId = -1;
        private Thread asyncJob;
        private string progressMessage = "";
        private int totalTasks;
        private int tasksComplete;
        private bool done, selectedPipelineSetRerun, serverSettingSetRerun, parameterSetRerun;
        private bool stateIsSwitched;
        private int _selectedPipelineAssociationLocalId;
        private bool doEmailNotification, showVisualizers;
        private string emailAddress;
        private TaskPaneDockPosition previousDockPosition = TaskPaneDockPosition.Right, currentDockPosition;
        private delegate void DelegateIntializeMembers();
        DelegateIntializeMembers delegateIntializeMembers;
        private DocumentImage documentImage;

        #endregion

        #region Public Methods

        /// <summary>
        /// Initializes a new instance of the <see cref="RerunPipelineTaskPane"/> class.
        /// </summary>
        public RerunPipelineTaskPane()
        {
            InitializeComponent();

            selectPipelineToRerunUserControl.AssociationSelectedIndexChanged += new EventHandler<EventArgs>(selectPipelineToRerunUserControl_AssociationSelectedIndexChanged);
            selectPipelineToRerunUserControl.AssociationCleared += new EventHandler<EventArgs>(selectPipelineToRerunUserControl_AssociationCleared);
            editParametersUserControl.MakeRerunButtonEnable += new EventHandler<EventArgs>(editParametersUserControl_MakeRerunButtonEnable);
            editParametersUserControl.ResetParameter += new EventHandler<EventArgs>(editParametersUserControl_ResetParameter);
            serveSettingUserControl.SetRerunButton += new EventHandler<EventArgs>(serveSettingUserControl_SetRerunButton);
            delegateIntializeMembers = new DelegateIntializeMembers(InitializeAbortedMembers);
        }

        /// <summary>
        /// Handles the SetRerunButton event of the serveSettingUserControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void serveSettingUserControl_SetRerunButton(object sender, EventArgs e)
        {
            if (sender is bool)
            {
                serverSettingSetRerun = (bool)sender;
                btnrerun.Enabled = selectedPipelineSetRerun && serverSettingSetRerun;
            }
        }

        /// <summary>
        /// Handles the AssociationCleared event of the selectPipelineToRerunUserControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void selectPipelineToRerunUserControl_AssociationCleared(object sender, EventArgs e)
        {
            stopPointUserControl.ClearStopPoint();
            editParametersUserControl.ClearParameters();
        }

        /// <summary>
        /// Handles the ResetParameter event of the editParametersUserControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void editParametersUserControl_ResetParameter(object sender, EventArgs e)
        {
            _selectedPipelineAssociation = selectPipelineToRerunUserControl.GetPipelineAssociation();

            if (_selectedPipelineAssociation != null)
            {
                _selectedJob = _selectedPipelineAssociation.Job.Clone() as Job;

                if (_selectedJob != null)
                {
                    btnrerun.Enabled = editParametersUserControl.IsInputFilesPresent;
                    _selectedPipeline = selectPipelineToRerunUserControl.GetPipeline(_selectedJob.PipelineLsid, _documentKey);
                    editParametersUserControl.UpdateCall(_selectedJob, _selectedPipeline);
                }
            }
        }

        /// <summary>
        /// Handles the MakeRerunButtonEnable event of the editParametersUserControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void editParametersUserControl_MakeRerunButtonEnable(object sender, EventArgs e)
        {
            if (sender is bool)
            {
                parameterSetRerun = (bool)sender;
                btnrerun.Enabled = selectedPipelineSetRerun && serverSettingSetRerun && parameterSetRerun;
            }
        }

        /// <summary>
        /// Handles the JobsSelectedIndexChanged event of the selectPipelineToRerunUserControl control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void selectPipelineToRerunUserControl_AssociationSelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                txtStatusLabel.Text = string.Empty;
                progressBar.Value = 0;
                progressBar.Visible = false;
                downloadAnimationUserControl1.Visible = false;
                _selectedPipelineAssociation = selectPipelineToRerunUserControl.GetPipelineAssociation();

                if (_selectedPipelineAssociation == null)
                {
                    _selectedJob = null;
                    ResetFields();
                    stopPointUserControl.ClearStopPoint();
                    editParametersUserControl.ClearParameters();
                }
                else if (_selectedPipelineAssociation != null)
                {
                    _selectedJob = _selectedPipelineAssociation.Job.Clone() as Job;
                    if (_selectedJob != null)
                    {
                        selectedPipelineSetRerun = true;
                        btnrerun.Enabled = selectedPipelineSetRerun && serverSettingSetRerun;
                        _selectedPipeline = selectPipelineToRerunUserControl.GetPipeline(_selectedJob.PipelineLsid, _documentKey);
                        stopPointUserControl.UpdateStopPoint(_selectedPipeline, _selectedPipelineAssociation.UserStopPoint);
                        editParametersUserControl.UpdateCall(_selectedJob, _selectedPipeline);
                        serveSettingUserControl.SetShowResult(_selectedPipeline.HasVisualizers);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Cancels the execution.
        /// </summary>
        public void CancelExecution()
        {
            _controller.SetJobProcessingStatus(_documentKey, false);
            if (!done)
            {
                AbortJob();
            }
        }

        /// <summary>
        /// Updates the associations.
        /// </summary>
        public void UpdateAssociations(DocumentImage selectedImage)
        {
            if (!IsJobExecutionInProgress())
            {
                txtStatusLabel.Text = string.Empty;
                selectPipelineToRerunUserControl.IntialUpdate(_controller, _documentKey, selectedImage);
                serveSettingUserControl.SetControll(_controller);
            }
        }

        /// <summary>
        /// Determines whether [is job execution in progress].
        /// </summary>
        /// <returns>
        /// 	<c>true</c> if [is job execution in progress]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsJobExecutionInProgress()
        {
            if (_controller.GetDocumentModel(_documentKey).TransactionActive)
            {
                if (asyncJob != null && asyncJob.ThreadState == System.Threading.ThreadState.Running)
                {
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Gets a value indicating whether all parameters are set.
        /// </summary>
        /// <value><c>true</c> if all parameters are set; otherwise, <c>false</c>.</value>
        private bool IsParametersSet
        {
            get
            {
                bool result = true;
                if (_selectedJob != null)
                {
                    foreach (JobParameter parameter in _selectedJob.Parameters)
                    {
                        if (!parameter.Optional)
                        {
                            if (parameter.ParamValue == null ||
                                string.IsNullOrEmpty(parameter.ParamValue.ToString()))
                            {
                                MessageBox.Show("Please enter input parameter.", Resources.GENEPATTREN_MSG_HEADER,
                                            MessageBoxButtons.OK, MessageBoxIcon.Information);
                                result = false;
                                break;
                            }
                        }
                    }
                }
                else
                {
                    result = false;
                }

                return result;
            }
        }

        /// <summary>
        /// Disables the split container.
        /// </summary>
        private void DisableSplitContainer()
        {
            splitContainerTop.Enabled = false;
        }

        /// <summary>
        /// Enables the split container.
        /// </summary>
        private void EnableSplitContainer()
        {
            splitContainerTop.Enabled = true;
        }

        /// <summary>
        /// Updates the pipeline assocations.
        /// </summary>
        private void UpdatePipelineAssocations()
        {
            selectPipelineToRerunUserControl.IntialUpdate(_controller, _documentKey, null);
            serveSettingUserControl.SetControll(_controller);
        }

        /// <summary>
        /// Submits the job.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void SubmitJob()
        {
            try
            {
                _controller.BeginModelChanges(_documentKey);
                Connection verifiedConnection = _controller.ServerBroker.GetTestedVerifiedConnection(preVerifiedConnection, false);
                if (verifiedConnection != null && _selectedJob != null)
                {
                    _selectedJob.StopPoint = stopPointUserControl.SelectedStopPoint;
                    jobId = _controller.SubmitJob(_selectedJob, verifiedConnection, doEmailNotification,
                                                  emailAddress, false, new SubmitProgress(UpdateProgressBar), _documentKey);
                    if (jobId != -1)
                    {
                        PipelineAssociation pipelineAssociation =
                            _controller.GetDocumentModel(_documentKey).PipelineAssociations[_selectedPipelineAssociationLocalId];
                        pipelineAssociation.Rerun = new Rerun(verifiedConnection);
                        pipelineAssociation.Rerun.Job.Id = jobId;
                        //pipelineAssociation.Rerun.SetConnection(verifiedConnection);
                        pipelineAssociation.Rerun.ShowVisualizers = showVisualizers;
                        pipelineAssociation.OriginalRerunParameters = _selectedJob.Parameters;
                        _controller.CommitModelChanges(_documentKey);
                        _controller.SignalRerunIsPresent(pipelineAssociation);
                        MemoryManagement.FlushMemory();

                    }
                    else
                    {
                        _controller.RollbackModelChanges(_documentKey);
                    }
                }
                else
                {
                    _controller.RollbackModelChanges(_documentKey);
                }
                done = true;
            }
            catch (ThreadAbortException)
            {
                // user clicked cancel
                InitializeMembers();
            }
            catch (PipelineOrModuleNotInstalledException ex)
            {
                string message = "One or more modules required for rerunning the pipeline are "
                                 + Environment.NewLine
                                 + "missing in the referenced Server." + Environment.NewLine + Environment.NewLine
                                 + "Please see details below for more information."
                                 + Environment.NewLine + Environment.NewLine
                                 + "The rerun will be aborted.";
                MessageForm.ShowUnhandledExceptionMessage(message, ex.Message);
                this.AbortJob();
            }
            // NO Need to handle the exception
            catch (COMException)
            {
                _controller.SetJobProcessingStatus(_documentKey, false);
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
                this.AbortJob();
            }
            finally
            {
                if (_controller.GetDocumentModel(_documentKey).TransactionActive)
                {
                    _controller.RollbackModelChanges(_documentKey);
                }
            }
        }

        /// <summary>
        /// Aborts the job.
        /// </summary>
        private void AbortJob()
        {
            try
            {
                if (asyncJob != null && asyncJob.IsAlive)
                {
                    asyncJob.Abort();
                }
            }
            // The Threadabort exception is a special case. Even if we have catched Thread Abort exception, it is still rethrown by the CLR!
            // So we need to close the progressBar form in the Finally block.
            catch (ThreadAbortException exception)
            {
                Trace.WriteLine("Submit Job Thread aborted successfully" + exception.Message, TraceLevel.Info.ToString());
            }
            finally
            {
                if (delegateIntializeMembers != null)
                {
                    this.Invoke(delegateIntializeMembers);
                }
            }
        }

        /// <summary>
        /// Initializes the aborted members.
        /// </summary>
        private void InitializeAbortedMembers()
        {
            btnCancel.Enabled = false;
            txtStatusLabel.Text = string.Empty;
            jobId = -1;
            progressBar.Visible = false;
            progressBar.Minimum = 0;
            done = false;
            stateIsSwitched = false;
            progressTimer.Stop();
            downloadAnimationUserControl1.Stop();
            btnCancel.Enabled = false;
            EnableSplitContainer();
            btnrerun.Enabled = true;
        }

        /// <summary>
        /// Initializes the members.
        /// </summary>
        private void InitializeMembers()
        {

            //txtStatusLabel.Text = string.Empty;
            jobId = -1;
            progressBar.Minimum = 0;
            done = false;
            stateIsSwitched = false;
            progressTimer.Stop();
            downloadAnimationUserControl1.Stop();
        }

        /// <summary>
        /// Counts the pipeline pieces.
        /// </summary>
        /// <param name="p">The p.</param>
        /// <returns></returns>
        private static int CountPipelinePieces(Pipeline p)
        {
            int count = 1;
            count += p.Steps.Count;

            return count;
        }

        /// <summary>
        /// Updateprogresses the bar.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="isStarting">if set to <c>true</c> [is starting].</param>
        private void UpdateProgressBar(string name, bool isStarting)
        {
            if (isStarting)
            {
                string startMessage = Resources.START_PROCESSING;
                progressMessage = String.Format(CultureInfo.InvariantCulture, startMessage, name);
            }
            else
            {
                string finishedMessage = Resources.FINISH_PROCESSING;
                progressMessage = String.Format(CultureInfo.InvariantCulture, finishedMessage, name);
                tasksComplete++;
            }
        }

        /// <summary>
        /// Switches the state of to done.
        /// </summary>
        private void SwitchToDoneState()
        {
            _controller.ScrollToImage(documentImage, _documentKey);
            _controller.UpdatePipelineAssociationsDisplay(documentImage, _documentKey);

            stateIsSwitched = true;
            downloadAnimationUserControl1.Stop();
            progressTimer.Enabled = false;
            btnCancel.Enabled = false;
            txtStatusLabel.Text = string.Empty;
            progressBar.Minimum = 0;
            EnableSplitContainer();
            this.btnrerun.Enabled = true;
        }

        #endregion

        #region Internal Methods

        /// <summary>
        /// Sets the controller.
        /// </summary>
        /// <param name="controller">The controller.</param>
        internal void SetController(IController controller)
        {
            this._controller = controller;
            this._documentKey = _controller.DocumentKey;
            UpdatePipelineAssocations();
        }

        /// <summary>
        /// Updates the connections combo box.
        /// </summary>
        internal void UpdateConnectionsComboBox()
        {
            serveSettingUserControl.UpdateConnectionsComboBox();
        }

        /// <summary>
        /// Updates the associations.
        /// </summary>
        /// <param name="pipelineAssociationsList">The pipeline associations list.</param>
        /// <param name="DocumentKey">The document key.</param>
        internal void UpdateAssociations(List<PipelineAssociation> pipelineAssociationsList)
        {
            if (!IsJobExecutionInProgress())
            {
                ResetFields();
                selectPipelineToRerunUserControl.ListJobFromDocument(pipelineAssociationsList, _documentKey);
            }
        }

        /// <summary>
        /// Resets the fields.
        /// </summary>
        private void ResetFields()
        {
            txtStatusLabel.Text = string.Empty;
            this.btnrerun.Enabled = false;
            selectedPipelineSetRerun = false;
        }

        /// <summary>
        /// Sets the index.
        /// </summary>
        /// <param name="index">The index.</param>
        internal void SetTheIndex(int index)
        {
            selectPipelineToRerunUserControl.SetIndex(index);
        }

        # endregion

        #region ITaskPaneControl Members

        /// <summary>
        /// Sets the dock position.
        /// </summary>
        /// <param name="dockPosition">The dock position.</param>
        public void SetDockPosition(TaskPaneDockPosition dockPosition)
        {
            try
            {
                currentDockPosition = dockPosition;
                if ((dockPosition & (TaskPaneDockPosition.Left | TaskPaneDockPosition.Right | TaskPaneDockPosition.Floating)) != 0)
                {
                    SetVertical(true);
                }
                else
                {
                    SetVertical(false);
                }
            }
            catch (Exception)
            {
                throw;
            }
            finally
            {
                previousDockPosition = dockPosition;
            }
        }

        /// <summary>
        /// Sets the vertical.
        /// </summary>
        /// <param name="controlIsVertical">if set to <c>true</c> [control is vertical].</param>
        private void SetVertical(bool controlIsVertical)
        {

            this.SuspendLayout();

            int initialSplitterDistance = controlIsVertical ==
                true ? splitContainerTop.MinimumSize.Height : splitContainerTop.MinimumSize.Width;

            splitContainerLowerBottom.SplitterDistance = initialSplitterDistance;
            splitContainerBottom.SplitterDistance = initialSplitterDistance;
            splitContainerMiddle.SplitterDistance = initialSplitterDistance;
            splitContainerTop.SplitterDistance = initialSplitterDistance;

            Orientation orientation = controlIsVertical ? Orientation.Horizontal : Orientation.Vertical;

            splitContainerTop.Orientation = orientation;
            splitContainerMiddle.Orientation = orientation;
            splitContainerBottom.Orientation = orientation;
            splitContainerLowerBottom.Orientation = orientation;

            if (controlIsVertical)
            {
                int splitterDistance = this.Height / 3;
                splitContainerTop.SplitterDistance = (int)(this.Height * 0.20);
                splitContainerMiddle.SplitterDistance = (int)(this.Height * 0.38);

                splitContainerLowerBottom.SplitterDistance = (int)(this.Height * 0.05);
                splitContainerBottom.SplitterDistance = (int)(this.Height * 0.05);

            }
            else
            {
                splitContainer3.FixedPanel = FixedPanel.None;
                int splitterDistance = this.Width / 3;
                splitContainerTop.SplitterDistance = splitterDistance;
                splitContainerMiddle.SplitterDistance = splitterDistance;

                splitContainerBottom.SplitterDistance = (int)(this.Height * 0.11);
                splitContainerLowerBottom.SplitterDistance = (int)(this.Height * 0.10);

                if (previousDockPosition == TaskPaneDockPosition.Floating)
                {
                    splitContainerBottom.SplitterDistance = (int)(this.Height * 0.28);
                    splitContainerLowerBottom.SplitterDistance = (int)(this.Height * 0.20);
                }
            }

            this.ResumeLayout();
        }

        #endregion

        #region Events

        /// <summary>
        /// Handles the Click event of the btnrerun control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void btnrerun_Click(object sender, EventArgs e)
        {
            try
            {
                if (_controller.CanPipelineRerun(false) == false || IsParametersSet == false)
                    return;

                // Check if any other Transaction is Active on Current Model
                if (_controller.IsJobProcessingInProgress(_documentKey))
                {
                    MessageBox.Show(Resources.JOB_IN_PROGRESS, Resources.GENEPATTREN_MSG_HEADER,
                        MessageBoxButtons.OK, MessageBoxIcon.Information);
                    return;
                }

                _controller.SetJobProcessingStatus(_documentKey, true);

                documentImage = _controller.GetDocumentImage;

                DisableSplitContainer();
                InitializeMembers();
                downloadAnimationUserControl1.Visible = true;
                progressBar.Visible = true;
                doEmailNotification = serveSettingUserControl.DoEmailNotification();
                emailAddress = serveSettingUserControl.GetEmailAddress();
                showVisualizers = serveSettingUserControl.ShowVisualizers();
                _selectedPipelineAssociationLocalId = _selectedPipelineAssociation.LocalId;
                preVerifiedConnection = serveSettingUserControl.GetConnection;
                asyncJob = new Thread(new ThreadStart(SubmitJob));
                asyncJob.Name = "Submit Job";
                asyncJob.SetApartmentState(ApartmentState.STA);

                Pipeline pipeline = _controller.GetDocumentModel(_documentKey).Pipelines[_selectedJob.PipelineLsid];
                totalTasks += CountPipelinePieces(pipeline);

                progressTimer.Start();
                asyncJob.Start();
                downloadAnimationUserControl1.Start();
                btnrerun.Enabled = false;
                btnCancel.Enabled = true;
            }
            catch (Exception ex)
            {
                downloadAnimationUserControl1.Visible = false;
                CancelExecution();
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the Tick event of the progressTimer control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void progressTimer_Tick(object sender, EventArgs e)
        {
            progressBar.Minimum = 0;
            progressBar.Maximum = totalTasks;

            if (tasksComplete > progressBar.Maximum)
            {
                progressBar.Maximum = tasksComplete;
                btnCancel.Enabled = false;
                txtStatusLabel.Text = string.Empty;
                EnableSplitContainer();
                this.btnrerun.Enabled = true;
            }
            progressBar.Value = tasksComplete;
            if (txtStatusLabel.Text != progressMessage)
            {
                txtStatusLabel.Text = progressMessage.ToString();
            }

            if (done && !stateIsSwitched)
            {
                SwitchToDoneState();
            }

            if (!asyncJob.IsAlive)
            {
                // download is done but time not done -- show progress finished
                progressBar.Value = progressBar.Maximum;
                _controller.SetJobProcessingStatus(_documentKey, false);
            }

            if (done && jobId > -1)
            {
                progressTimer.Stop();
                txtStatusLabel.Text = "Job Id Created : " + jobId.ToString(CultureInfo.InvariantCulture);
                progressBar.Value = progressBar.Maximum;
                downloadAnimationUserControl1.Stop();
                btnCancel.Enabled = false;
                EnableSplitContainer();
                btnrerun.Enabled = true;

                OnJobCompletion();
            }
        }

        /// <summary>
        /// Called when [job completion].
        /// </summary>
        private void OnJobCompletion()
        {
            string paneTitle = _controller.UserControlCreator.GetTitle(TaskPaneKey.RerunPipeline);
            _controller.TaskPaneManager.SetTaskPaneVisibility(paneTitle, false, _documentKey);

            if (_controller.IsTaskPaneVisible(_documentKey, TaskPaneKey.ViewAssociations) == false)
            {
                string message = string.Format(Resources.JOB_COMPLETION, jobId, preVerifiedConnection.Name);
                MessageBox.Show(message, Resources.GENEPATTREN_MSG_HEADER, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        /// <summary>
        /// Handles the Click event of the btnCancel control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void btnCancel_Click(object sender, EventArgs e)
        {
            try
            {
                btnCancel.Enabled = false;
                CancelExecution();
                downloadAnimationUserControl1.Visible = false;
                progressBar.Visible = false;
            }
            catch (Exception ex)
            {
                CancelExecution();
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
            finally
            {
                InitializeMembers();
                EnableSplitContainer();
                this.btnrerun.Enabled = true;
            }
        }

        /// <summary>
        /// Handles the EnabledChanged event of the btnrerun control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnrerun_EnabledChanged(object sender, EventArgs e)
        {
            if (btnrerun.Enabled)
            {
                progressBar.Visible = false;
                downloadAnimationUserControl1.Visible = false;
            }
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the tabControl1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void tabControl1_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                string currentTabPageName = ((TabControl)sender).SelectedTab.Text;
                preVerifiedConnection = serveSettingUserControl.GetConnection;
                doEmailNotification = serveSettingUserControl.DoEmailNotification();
                emailAddress = serveSettingUserControl.GetEmailAddress();
                showVisualizers = serveSettingUserControl.ShowVisualizers();


                _selectedPipelineAssociation = selectPipelineToRerunUserControl.GetPipelineAssociation();
                _selectedJob = null;
                if (_selectedPipelineAssociation != null)
                {
                    _selectedJob = _selectedPipelineAssociation.Job.Clone() as Job;
                }
                _selectedPipeline = null;
                if (_selectedJob != null)
                {
                    //this.btnrerun.Enabled = true;
                    _selectedPipeline = selectPipelineToRerunUserControl.GetPipeline(_selectedJob.PipelineLsid, _documentKey);
                }
                if (string.Equals(currentTabPageName, "Summary"))
                {
                    summaryPageUserControl.OnTabChanged(editParametersUserControl.EditParameterPipeline, editParametersUserControl.EditParameterJob,
                                                        doEmailNotification, showVisualizers,
                                                        emailAddress, preVerifiedConnection,
                                                        stopPointUserControl.SelectedStopPoint,
                                                        stopPointUserControl.SelectedStopPointName);
                }
            }
            catch (Exception)
            {
                throw;
            }
            finally
            {
                this.ResumeLayout();
            }
        }

        #endregion
    }
}
