using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using System.Threading;
using System.Globalization;

namespace Microsoft.eResearch.GenePattern.Package
{
    public partial class SelectJobsUserControl : UserControl
    {
        #region Variable Declaration

        private IController _controller;
        private ICollection<PipelineAssociation> _downloadSearchResult;
        private ICollection<PipelineAssociation> _documentSearchResult;
        private Thread _downloadJobs;
        private string _lastFilterString;
        private IGenePatternServer _genePatternServer;
        private string _documentKey = string.Empty;
        bool isFirstCreated = true;

        /// <summary>
        /// Occurs when [jobs selected index changed].
        /// </summary>
        public event EventHandler<EventArgs> JobsSelectedIndexChanged;

        #endregion

        #region Public Methods

        /// <summary>
        /// Initializes a new instance of the <see cref="SelectJobsUserControl"/> class.
        /// </summary>
        public SelectJobsUserControl()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Initials the update.
        /// </summary>
        /// <param name="controller">The controller.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="isEdit">if set to <c>true</c> [is edit].</param>
        public void InitialUpdate(IController controller, string documentKey, bool isEdit)
        {
            this._controller = controller;
            this._documentKey = documentKey;
            _genePatternServer = controller.GetGenePatternServer();
           
            if (isEdit)
                rbnDocument.Checked = controller.GetDocumentModel(documentKey).PipelineAssociations.Count > 0 ? true : false;
            else
                rbnServer.Checked = true;

            RetrieveJobs();
        }

        /// <summary>
        /// Retrieves the jobs.
        /// </summary>
        public void RetrieveJobs()
        {
            if (rbnDocument.Checked)
            {
                ListJobsFromDocument(true);
            }
            else
            {
                ListJobsFromServer(true);
            }
        }

        /// <summary>
        /// Gets the job.
        /// </summary>
        /// <returns></returns>
        public Job GetJob
        {
            get
            {
                if (Jobs.SelectedItems.Count > 0)
                {
                    if (rbnDocument.Checked)
                    {
                        return Jobs.SelectedItems[0].Tag as Job;
                    }
                    else
                    {
                        Job job = Jobs.SelectedItems[0].Tag as Job;
                        if (job != null)
                        {
                            return _genePatternServer.GetJobById(job.Id);
                        }
                    }
                }
                return null;
            }
        }

        /// <summary>
        /// Gets the pipeline.
        /// </summary>
        /// <param name="pipelineLsid">The pipeline lsid.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        public Pipeline GetPipeline(string pipelineLsid, string documentKey)
        {
            if (rbnDocument.Checked)
            {
                return _controller.GetDocumentModel(documentKey).Pipelines[pipelineLsid].Clone() as Pipeline;
            }
            else
            {
                return _genePatternServer.GetPipelineByLsid(pipelineLsid);
            }
        }

        /// <summary>
        /// Selects the job.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        public void SelectJob(int jobId)
        {
            for (int i = 0; i < Jobs.Items.Count; i++)
            {
                if (((Job)Jobs.Items[i].Tag).Id == jobId)
                {
                    Jobs.Items[i].Selected = true;
                }
            }
        }

        /// <summary>
        /// Updates the connection.
        /// </summary>
        /// <param name="newConnection">The new connection.</param>
        public void UpdateConnection(Connection newConnection)
        {
            if (_downloadJobs != null)
            {
                if (_downloadJobs.ThreadState == ThreadState.Running)
                {
                    _downloadJobs.Abort();
                }
            }
            _downloadJobs = null;
            _genePatternServer.Connection = newConnection;
            RetrieveJobs();
        }

        /// <summary>
        /// Gets or sets the index of the selected.
        /// </summary>
        /// <value>The index of the selected.</value>
        public int GetSelectedIndex
        {
            get
            {
                return Jobs.SelectedIndices[0];
            }
        }

        /// <summary>
        /// Sets the index of the selected.
        /// </summary>
        /// <param name="index">The index.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        public void SetSelectedIndex(int index)
        {
            if (index != -1 && index < Jobs.Items.Count)
            {
                Jobs.Items[index].Selected = true;
            }
        }

        /// <summary>
        /// Sets the job from server.
        /// </summary>
        /// <param name="value">if set to <c>true</c> [value].</param>
        public void SetJobFromServer(bool value)
        {
            rbnServer.Checked = value;
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Lists the jobs from document.
        /// </summary>
        /// <param name="forceUpdate">if set to <c>true</c> [force update].</param>
        private void ListJobsFromDocument(bool forceUpdate)
        {
            // only requery when search term changes
            string filterTerm = txtSearch.Text.Trim();

            if (string.Compare(_lastFilterString, filterTerm, true, CultureInfo.InvariantCulture) != 0 || forceUpdate)
            {
                // get data from model to fake the search result and then update as normal
                List<int> docJobs = new List<int>();
                _documentSearchResult = new List<PipelineAssociation>();
                PipelineAssociation paClone;

                foreach (PipelineAssociation pa in _controller.GetDocumentModel(_documentKey).PipelineAssociations.Values)
                {
                    paClone = (PipelineAssociation)pa.Clone();
                    paClone.MakeShallowClone();
                    paClone.Job = pa.Job.Clone() as Job; // clone it so you don't change what is stored!
                    paClone.Job.StopPoint = paClone.UserStopPoint;
                    _documentSearchResult.Add(paClone);
                    docJobs.Add(pa.Job.Id);
                }

                FilterAndDisplayJobs(true);
            }
            JobPanel.Controls.Clear();
            JobPanel.Controls.Add(Jobs);
        }

        /// <summary>
        /// Lists the jobs from server.
        /// </summary>
        /// <param name="forceUpdate">if set to <c>true</c> [force update].</param>
        private void ListJobsFromServer(bool forceUpdate)
        {
            if (_downloadJobs == null)
            {
                // only requery when search term changes
                string filterTerm = txtSearch.Text.Trim();
                Cursor.Current = Cursors.WaitCursor;

                if (string.Compare(_lastFilterString, filterTerm, true, CultureInfo.InvariantCulture) != 0 || forceUpdate)
                {
                    // clear data and start the timer that will poll when the download finishes
                    Jobs.Items.Clear();
                    _downloadSearchResult = new List<PipelineAssociation>();

                    // These Condition is used to display the Create Connection dialog only once when Task Pane is created.
                    #region Create Connection Dialog Conditions

                    if (_genePatternServer == null || (_genePatternServer.Connection == null && !isFirstCreated))
                    {
                        return;
                    }

                    if (isFirstCreated)
                        isFirstCreated = false;

                    #endregion

                    if (_controller.AssureConnection())
                    {
                        // get jobs for the current user by default
                        _downloadJobs = new Thread(new ThreadStart(DownloadJobs));
                        _downloadJobs.Start();
                        timerJobsCompleted.Start();

                        // show progress bar
                        JobPanel.Controls.Clear();
                        JobProcessing waitControl = new JobProcessing();
                        waitControl.Dock = DockStyle.Fill;
                        waitControl.messageLabel.Text = "Downloading Jobs...";
                        JobPanel.Controls.Add(waitControl);
                    }
                }
                Cursor.Current = Cursors.Default;
            }
        }

        /// <summary>
        /// Filters the and display jobs.
        /// </summary>
        /// <param name="useDocJobs">if set to <c>true</c> [use doc jobs].</param>
        private void FilterAndDisplayJobs(bool useDocJobs)
        {
            string filterTerm = txtSearch.Text.Trim();

            SetJobListColumns();
            Jobs.Items.Clear();

            ICollection<PipelineAssociation> searchResult = useDocJobs ? _documentSearchResult : _downloadSearchResult;

            if (searchResult.Count == 0)
            {
                ListViewItem noResults = Jobs.Items.Add("");
                noResults.ImageKey = "job";
                noResults.SubItems.Add("No jobs found on server for current user.");
            }

            foreach (PipelineAssociation pa in searchResult)
            {
                if ((String.IsNullOrEmpty(filterTerm) || MatchesString(pa.Job, filterTerm)) && pa.Job != null)
                {
                    ListViewItem newItem = Jobs.Items.Add(pa.Job.Id.ToString(CultureInfo.InvariantCulture));
                    newItem.ImageKey = "job";
                    // if jobs are coming from the document, get name of association
                    // go through existing associations and find 
                    if (rbnDocument.Checked)
                    {
                        newItem.SubItems.Add(pa.Name);
                    }

                    newItem.SubItems.Add(pa.Job.PipelineName);
                    newItem.Tag = pa.Job;
                }
            }

            if (Jobs.Items.Count == 0)
            {
                ListViewItem noResults = Jobs.Items.Add("");
                noResults.ImageKey = "job";
                noResults.SubItems.Add("No jobs on server matched the criteria.");
            }
            _lastFilterString = filterTerm;
        }

        /// <summary>
        /// Sets the job list columns.
        /// </summary>
        private void SetJobListColumns()
        {
            Jobs.Columns.Clear();

            if (rbnDocument.Checked)
            {
                // add columns for document jobs
                Jobs.Columns.Add(Resources.ID, 53, HorizontalAlignment.Left);
                Jobs.Columns.Add(Resources.ASSOCIATION, 100, HorizontalAlignment.Left);
                Jobs.Columns.Add(Resources.PIPELINE, 240, HorizontalAlignment.Left);
            }
            else
            {
                // add columns for server jobs
                Jobs.Columns.Add(Resources.ID, 53, HorizontalAlignment.Left);
                Jobs.Columns.Add(Resources.PIPELINE, 240, HorizontalAlignment.Left);
            }
        }

        /// <summary>
        /// Matcheses the string.
        /// </summary>
        /// <param name="job">The job.</param>
        /// <param name="filterTerm">The filter term.</param>
        /// <returns></returns>
        private static bool MatchesString(Job job, string filterTerm)
        {
            try
            {
                if (job.Id.ToString(CultureInfo.InvariantCulture).Contains(filterTerm))
                {
                    return true;
                }
                else if ((job.PipelineName != null &&
                    job.PipelineName.ToLower(CultureInfo.InvariantCulture).Contains(filterTerm.ToLower(CultureInfo.InvariantCulture))))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            catch (NullReferenceException)
            {
                // if any of this stuff is null, then it doesn't match.
                return false;
            }
        }

        /// <summary>
        /// Downloads the jobs.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void DownloadJobs()
        {
            try
            {
                if (_genePatternServer.Connection == null)
                {
                    _genePatternServer.Connection = _controller.GetConnectionByDocumentKey(_documentKey);
                }

                IList<Job> jobs = _genePatternServer.GetJobsByCurrentUser();
                if (rbnServer.Checked)
                {
                    // wrap each job in a fake pipeline association to make the filteranddisplay() method easier
                    _downloadSearchResult.Clear();
                    PipelineAssociation pipelineAssociation;
                    foreach (Job job in jobs)
                    {
                        pipelineAssociation = new PipelineAssociation();
                        pipelineAssociation.Job = job;
                        _downloadSearchResult.Add(pipelineAssociation);
                    }
                }
            }
            catch (ThreadAbortException) { }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex, true);
            }
        }

        #endregion

        #region Event Handler

        /// <summary>
        /// Handles the CheckedChanged event of the rbnDocument control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void rbnDocument_CheckedChanged(object sender, EventArgs e)
        {
            txtSearch.Clear();
            isFirstCreated = !rbnDocument.Checked;
            RetrieveJobs();
            OnJobsSelectedIndexChanged();
        }

        /// <summary>
        /// Handles the Click event of the btnSearch control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnSearch_Click(object sender, EventArgs e)
        {
            isFirstCreated = !rbnDocument.Checked;
            RetrieveJobs();
            OnJobsSelectedIndexChanged();
        }

        /// <summary>
        /// Handles the Tick event of the timerJobsCompleted control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void timerJobsCompleted_Tick(object sender, EventArgs e)
        {
            if (rbnServer.Checked)
            {
                if (_downloadJobs == null || !_downloadJobs.IsAlive)
                {
                    timerJobsCompleted.Stop();
                    FilterAndDisplayJobs(false);

                    JobPanel.Controls.Clear();
                    JobPanel.Controls.Add(Jobs);
                    _downloadJobs = null;
                }
            }
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the Jobs control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Jobs_SelectedIndexChanged(object sender, EventArgs e)
        {
            OnJobsSelectedIndexChanged();
        }

        /// <summary>
        /// Called when [jobs selected index changed].
        /// </summary>
        private void OnJobsSelectedIndexChanged()
        {
            if (JobsSelectedIndexChanged != null)
            {
                JobsSelectedIndexChanged(this, EventArgs.Empty);
            }
        }

        #endregion

        # region Property

        /// <summary>
        /// Gets or sets a value indicating whether this instance is job from document.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is job from document; otherwise, <c>false</c>.
        /// </value>
        public bool IsJobFromDocument
        {
            get
            {
                return rbnDocument.Checked;
            }
            set
            {
                rbnDocument.Checked = value;
            }
        }

        /// <summary>
        /// Gets or sets the gene pattern server.
        /// </summary>
        /// <value>The gene pattern server.</value>
        public IGenePatternServer GenePatternServer
        {
            get { return _genePatternServer; }
            set { _genePatternServer = value; }
        }

        # endregion
    }
}
