using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using System.Globalization;

namespace Microsoft.eResearch.GenePattern.Package
{
    public partial class SummaryPageUserControl : UserControl
    {
        # region Private Members

        private Job job;
        private bool doEmailNotification, showVisualizers;
        string emailAddress = string.Empty;
        private Pipeline pipeline;
        private Connection connection;
        private int stopPointIndex;
        private string stopPointName;
        # endregion

        # region Consutructor

        /// <summary>
        /// Initializes a new instance of the <see cref="SummaryPageUserControl"/> class.
        /// </summary>
        public SummaryPageUserControl()
        {
            InitializeComponent();
        }

        # endregion

        # region Internal Method

        /// <summary>
        /// Called when [tab changed].
        /// </summary>
        /// <param name="_selectedPipeline">The _selected pipeline.</param>
        /// <param name="_selectedJob">The _selected job.</param>
        /// <param name="_doEmailNotification">if set to <c>true</c> [_do email notification].</param>
        /// <param name="_showVisualizers">if set to <c>true</c> [_show visualizers].</param>
        /// <param name="_emailAddress">The _email address.</param>
        /// <param name="_connection">The _connection.</param>
        /// <param name="_stopPointIndex">Index of the _stop point.</param>
        /// <param name="_stopPointName">Name of the _stop point.</param>
        internal void OnTabChanged(Pipeline _selectedPipeline, Job _selectedJob,
                                   bool _doEmailNotification, bool _showVisualizers,
                                   string _emailAddress, Connection _connection, int _stopPointIndex, string _stopPointName)
        {
            doEmailNotification = _doEmailNotification;
            showVisualizers = _showVisualizers;
            emailAddress = _emailAddress;
            pipeline = _selectedPipeline;
            connection = _connection;
            stopPointName = _stopPointName;
            stopPointIndex = _stopPointIndex;

            if (_selectedJob != null)
            {
                job = _selectedJob.Clone() as Job;
            }
            else
            {
                job = null;
            }

            if (pipeline != null && job != null)
            {
                UpdateCall();
            }
            else
            {
                SummaryTextBox.Text = string.Empty;
            }
        }

        # endregion

        # region Private Methods

        /// <summary>
        /// Updates the call.
        /// </summary>
        private void UpdateCall()
        {
            SummaryTextBox.Text = string.Empty;
            StringBuilder builder = new StringBuilder();

            BuildPipelineInfo(builder);
            BuildEmailInfo(builder);
            BuildServerSummaryInfo(builder);
            BuildVisualizerInfo(builder);

            SummaryTextBox.Text = builder.ToString();

        }

        /// <summary>
        /// Builds the pipeline info.
        /// </summary>
        /// <param name="builder">The builder.</param>
        private void BuildPipelineInfo(StringBuilder builder)
        {
            //Pipeline pipeline = RerunPipelineWizardModel.SelectedPipeline;
            //Job job = RerunPipelineWizardModel.Job;
            string selectedPipelineText = Resources.SELECTED_PIPELINE_SUMMARY_TEXT;
            builder.AppendFormat(CultureInfo.InvariantCulture, selectedPipelineText, pipeline.Name).AppendLine();

            if (pipeline.Parameters.Count > 0)
            {
                builder.AppendLine();
                string parametersText = Resources.PARAMETERS_SUMMARY_TEXT;
                builder.AppendLine(parametersText);
                foreach (PipelineParameter pipelineParameter in pipeline.Parameters)
                {
                    JobParameter jobParameter = job.GetParameter(pipelineParameter.Name);
                    string paramValue = string.Empty;
                    if (jobParameter != null && jobParameter.ParamValue != null)
                    {
                        if (pipelineParameter.Type == ParameterType.StringCollection)
                        {
                            paramValue = pipelineParameter.GetChoice(jobParameter.ParamValue.ToString()).Text;
                        }
                        else
                        {
                            paramValue = jobParameter.ParamValue.ToString();
                        }
                    }
                    builder.AppendFormat(CultureInfo.InvariantCulture, "{0} = {1}", pipelineParameter.ShortName, paramValue)
                        .AppendLine();
                }
            }

            builder.AppendLine();
            string stopAtText = Resources.STOP_AT_STEP_SUMMARY_TEXT;
            builder.AppendFormat(CultureInfo.InvariantCulture, stopAtText, stopPointIndex + 1,
                                 stopPointName).AppendLine();
        }

        /// <summary>
        /// Builds the email info.
        /// </summary>
        /// <param name="builder">The builder.</param>
        private void BuildEmailInfo(StringBuilder builder)
        {
            if (doEmailNotification)
            {
                builder.AppendLine();
                string sendNotificationText = Resources.SEND_NOTIFICATION_SUMMARY_TEXT;
                builder.AppendFormat(CultureInfo.InvariantCulture, sendNotificationText,
                                     emailAddress).AppendLine();
            }
            builder.AppendLine();
        }

        /// <summary>
        /// Builds the server summary info.
        /// </summary>
        /// <param name="builder">The builder.</param>
        private void BuildServerSummaryInfo(StringBuilder builder)
        {
            string runAtText = Resources.RUN_PIPELINE_ON_SERVER_SUMMARY_TEXT;
            if (connection != null)
            {
                builder.AppendFormat(CultureInfo.InvariantCulture, runAtText, connection.Name);
                builder.AppendLine();
                builder.AppendLine();
            }
            else
            {
                builder.AppendFormat(CultureInfo.InvariantCulture, "Connection selected by user is null");
                builder.AppendLine();
                builder.AppendLine();
            }
        }

        /// <summary>
        /// Builds the visualizer info.
        /// </summary>
        /// <param name="builder">The builder.</param>
        private void BuildVisualizerInfo(StringBuilder builder)
        {
            if (pipeline.HasVisualizers)
            {
                string text = Resources.DO_SHOW_VISUALIZERS;
                if (!showVisualizers)
                {
                    text = Resources.DONT_SHOW_VISUALIZERS;
                }
                builder.AppendLine(text);
            }
        }

        # endregion
    }
}
