using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.eResearch.GenePattern.Common;
using System.Collections;

namespace Microsoft.eResearch.GenePattern.Package
{
    /// <summary>
    /// Class View Pipeline Association
    /// </summary>
    public partial class ViewPipelinesUserControl : UserControl
    {
        /// <summary>
        /// Occurs when [association delete button click].
        /// </summary>
        public event EventHandler<EventArgs> AssociationDeleteButtonClick;

        private bool _isEditPipeline;

        /// <summary>
        /// Sets a value indicating whether this instance is edit pipeline.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is edit pipeline; otherwise, <c>false</c>.
        /// </value>
        public bool IsEditPipeline
        {
            get { return _isEditPipeline; }
            set { _isEditPipeline = value; }
        }

        #region Public Methods

        /// <summary>
        /// Initializes a new instance of the <see cref="ViewPipelinesUserControl"/> class.
        /// </summary>
        public ViewPipelinesUserControl()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Updates the association list.
        /// </summary>
        /// <param name="pipelineAssociationList">The pipeline association list.</param>
        /// <param name="pipelineList">The pipeline list.</param>
        public void UpdateAssociationList(List<PipelineAssociation> pipelineAssociationList,
                            List<Pipeline> pipelineList)
        {
            ClearAssociations();

            PipelineAssociation pa;
            Pipeline p;
            ListViewItem item;
            for (int index = 0; index < pipelineAssociationList.Count; index++)
            {
                pa = pipelineAssociationList[index];
                p = pipelineList[index];
                item = new ListViewItem(pa.Name);
                item.SubItems.Add(p.Name);
                item.SubItems.Add(pa.Annotation);
                item.Tag = index;
                item.ImageKey = p.IsLinked ? "linkedPipeline" : "pipeline";
                Associations.Items.Add(item);
            }
            if (Associations.SelectedItems.Count == 0)
                this.RemoveAssociation.Enabled = false;
        }

        /// <summary>
        /// Updates the association list.
        /// </summary>
        /// <param name="pipelineAssociation">The pipeline association.</param>
        /// <param name="pipeline">The pipeline.</param>
        public void UpdateAssociationList(PipelineAssociation pipelineAssociation,
                            Pipeline pipeline)
        {
            ClearAssociations();

            if (pipeline == null || pipelineAssociation == null)
                return;

            ListViewItem item;

            item = new ListViewItem(pipelineAssociation.Name);
            item.SubItems.Add(pipeline.Name);
            item.SubItems.Add(pipelineAssociation.Annotation);
            item.Tag = 0;
            item.ImageKey = pipeline.IsLinked ? "linkedPipeline" : "pipeline";
            Associations.Items.Add(item);

            if (Associations.SelectedItems.Count == 0)
                this.RemoveAssociation.Enabled = false;
        }

        /// <summary>
        /// Determines whether [is association name exist] [the specified is editing].
        /// </summary>
        /// <param name="isEditing">if set to <c>true</c> [is editing].</param>
        /// <param name="assoPipelineName">Name of the asso pipeline.</param>
        /// <param name="editPipelineName">Name of the edit pipeline.</param>
        /// <param name="pipelineAssociationList">The pipeline association list.</param>
        /// <param name="model">The model.</param>
        /// <returns>
        /// 	<c>true</c> if [is association name exist] [the specified is editing]; otherwise, <c>false</c>.
        /// </returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public bool IsAssociationNameExist(bool isEditing, string assoPipelineName, string editPipelineName,
            List<PipelineAssociation> pipelineAssociationList, Model model)
        {
            bool alreadyExistingName = false;
            int count = pipelineAssociationList.Count;

            if (count != 0)
            {
                for (int i = 0; i < count; i++)
                {
                    if (assoPipelineName.Equals(pipelineAssociationList[i].Name))
                    {
                        alreadyExistingName = true;
                        return alreadyExistingName;
                    }
                }
            }

            IDictionaryEnumerator ie = model.PipelineAssociations.GetEnumerator();

            string assoPipelineModelName = string.Empty;
            while (ie.MoveNext())
            {
                assoPipelineModelName = (ie.Value as PipelineAssociation).Name;
                if (assoPipelineModelName.Equals(assoPipelineName))
                {
                    alreadyExistingName = true;
                    break;
                }
            }

            return alreadyExistingName;
        }

        /// <summary>
        /// Clears the associations.
        /// </summary>
        public void ClearAssociations()
        {
            Associations.Items.Clear();
        }

        /// <summary>
        /// Disables the buttons.
        /// </summary>
        public void DisableButtons()
        {
            RemoveAssociation.Enabled = false;
        }
        #endregion

        #region Private Method

        /// <summary>
        /// Handles the Click event of the RemoveAssociation control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void RemoveAssociation_Click(object sender, EventArgs e)
        {
            try
            {
                if (Associations.SelectedItems.Count > 0)
                {
                    foreach (ListViewItem item in Associations.SelectedItems)
                    {
                        if (AssociationDeleteButtonClick != null)
                        {
                            AssociationDeleteButtonClick(item.Tag, EventArgs.Empty);
                            Associations.Items.RemoveAt(item.Index);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageForm.ShowUnhandledExceptionMessage(ex);
            }
        }

        /// <summary>
        /// Handles the SelectedIndexChanged event of the Associations control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Associations_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.RemoveAssociation.Enabled = Associations.SelectedItems.Count > 0 && !_isEditPipeline ? true : false;
        }

        #endregion
    }
}
