using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Xml;
using System.Xml.XPath;

namespace Microsoft.eResearch.GenePattern.Server
{
    /// <summary>
    /// Converts the given soapRequest containing "handler" elements of type DataHandler 
    /// to the corresponding Java Type
    /// </summary>
    public sealed class DataHandlerConverter
    {
        private static XmlNamespaceManager namespaceManager;
        private XmlDocument xmlDocument = new XmlDocument();
        private const string xsiPrefix = "xsi";
        private const string xsiNamespaceURI = "http://www.w3.org/2001/XMLSchema-instance";
        private const string soapPrefix = "soap";
        private const string soapNamespaceURI = "http://schemas.xmlsoap.org/soap/envelope/";
        private const string apacheNamespaceURI = "http://xml.apache.org/xml-soap";

        /// <summary>
        /// Converts the input Stream to a stream that is expected by the AXIS web service
        /// and also fills in the file Data along with thier ID in a dictionary
        /// </summary>
        /// <param name="inputStream">The input stream.</param>
        /// <param name="contentIdBinarydataDictionary">The contentIdBinarydataDictionary.</param>
        /// <returns>
        /// A Memory Stream with the dataHandler element reformatted the way expected by Java
        /// </returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1021:AvoidOutParameters", MessageId = "1#")]
        public MemoryStream ConvertStream(Stream inputStream, out Dictionary<string, string> contentIdBinarydataDictionary)
        {
            contentIdBinarydataDictionary = new Dictionary<string, string>();
            xmlDocument.Load(inputStream);
            MemoryStream outputStream = ConvertDataHandlerNodes(contentIdBinarydataDictionary);
           
            xmlDocument = null;
            
            return outputStream;
        }

        /// <summary>
        /// Converts the data handler nodes.
        /// </summary>
        /// <param name="mappings">The mappings.</param>
        /// <returns></returns>
        private MemoryStream ConvertDataHandlerNodes(Dictionary<string, string> mappings)
        {
            MemoryStream outputStream = new MemoryStream();
            namespaceManager = GetNamespaceManager();
            //Get all handler elemnts

            try
            {
                FormatDataHandlerElements(mappings);
            }
            catch (XPathException e)
            {
                Trace.WriteLine("Excetion finding nodes." + e.Message, TraceLevel.Info.ToString());
            }

            xmlDocument.Save(outputStream);
            outputStream.Position = 0;
            return outputStream;
        }

        /// <summary>
        /// Removes the data handler node.
        /// </summary>
        /// <param name="dataHandlerNode">The data handler node.</param>
        private void RemoveDataHandlerNode(XmlNode dataHandlerNode)
        {
            XmlNode bodyNode = xmlDocument.SelectSingleNode(@"/soap:Envelope/soap:Body", namespaceManager);
            bodyNode.RemoveChild(dataHandlerNode);
        }

        /// <summary>
        /// Formats the data handler elements.
        /// </summary>
        /// <param name="mappings">The mappings.</param>
        private void FormatDataHandlerElements(Dictionary<string, string> mappings)
        {
            string xpathQuery = "soap:Envelope/soap:Body//*[contains(@xsi:type, ':DataHandler')]";
            XmlNodeList allDatahandlerNodes = xmlDocument.SelectNodes(xpathQuery, namespaceManager);
            XmlAttribute id;
            XmlNode dataHandlerReferenceNode;
            XmlAttribute href;

            foreach (XmlNode dataHandlerNode in allDatahandlerNodes)
            {

                id = dataHandlerNode.Attributes["id"];
                string idValue = "#" + id.Value;

                string query = "soap:Envelope/soap:Body//*[@href='" + idValue + "']";
                dataHandlerReferenceNode = xmlDocument.SelectSingleNode(query, namespaceManager);

                //construct the required attributes
                ConstructTypeAttribute(dataHandlerReferenceNode);
                ConstructNamespaceMappingAttribute(dataHandlerReferenceNode);

                href = dataHandlerReferenceNode.Attributes["href"];
                string contentID = "cid:" + Guid.NewGuid();
                href.Value = contentID;

                //Get the file Data and also remove the File node
                mappings.Add(contentID, GetFileData(dataHandlerNode));

            }
           
            foreach (XmlNode dataHandlerNode in allDatahandlerNodes)
            {
                RemoveDataHandlerNode(dataHandlerNode);
            }

            dataHandlerReferenceNode = null;
            allDatahandlerNodes = null;
            href = null;
        }

        /// <summary>
        /// Constructs the namespace mapping attribute.
        /// </summary>
        /// <param name="handlerNode">The handler node.</param>
        private static void ConstructNamespaceMappingAttribute(XmlNode handlerNode)
        {
            XmlAttribute n1Attribute =
                handlerNode.OwnerDocument.CreateAttribute("xmlns:ns1", "http://www.w3.org/2000/xmlns/");
            n1Attribute.Value = apacheNamespaceURI;
            handlerNode.Attributes.Append(n1Attribute);
        }

        /// <summary>
        /// Constructs the type attribute.
        /// </summary>
        /// <param name="handlerNode">The handler node.</param>
        private static void ConstructTypeAttribute(XmlNode handlerNode)
        {
            XmlAttribute typeAttribute =
                handlerNode.OwnerDocument.CreateAttribute("xsi", "type", xsiNamespaceURI);
            typeAttribute.Value = "ns1:DataHandler";
            handlerNode.Attributes.Append(typeAttribute);

        }

        /// <summary>
        /// Gets the file data.
        /// </summary>
        /// <param name="dataHandlerNode">The data handler node.</param>
        /// <returns></returns>
        private static string GetFileData(XmlNode dataHandlerNode)
        {
            string fileData = null;
            if (dataHandlerNode != null)
            {
                XmlNode fileNode = dataHandlerNode.SelectSingleNode("./File", namespaceManager);
                fileData = fileNode.InnerText;
                dataHandlerNode.RemoveChild(fileNode);
                fileNode = null;
            }
            return fileData;
        }

        /// <summary>
        /// Gets the namespace manager.
        /// </summary>
        /// <returns></returns>
        private XmlNamespaceManager GetNamespaceManager()
        {
            namespaceManager = new XmlNamespaceManager(xmlDocument.NameTable);
            namespaceManager.AddNamespace(xsiPrefix, xsiNamespaceURI);
            namespaceManager.AddNamespace(soapPrefix, soapNamespaceURI);
            return namespaceManager;
        }
    }
}
