using System;
using System.Collections.Generic;
using System.Web.Services.Protocols;
using System.IO;
using Microsoft.eResearch.GenePattern.Server.SoapExtensions;

namespace Microsoft.eResearch.GenePattern.Server
{
    /// <summary>
    ///  This extension allows us to interpret the web service request is sent to the client,
    ///  and construct the request in the structure expected by the Java Client. By default, 
    ///  the Java client expects a apachesoap:DataHandler elemnt in the web request. 
    ///  We change the XMl request sent out in the "AfterSerialize" stage to convert the
    ///  Dot Net Data HAndler type into an apachesoap:DatHandler type.
    ///  
    /// </summary>
    public class DataHandlerSoapExtension : SoapExtension, IDisposable
    {
        private bool disposed;

        protected Stream stream;      // original stream that is sent out to server 
        protected Stream tempStream; //temp stream, where request is written to bt .NET

        // reader and writer are used to copy streams
        protected TextReader reader;
        protected TextWriter writer;

        /// <summary>
        /// When overridden in a derived class, allows a SOAP extension to initialize data specific to an XML Web service method using an attribute applied to the XML Web service method at a one time performance cost.
        /// </summary>
        /// <param name="methodInfo">A <see cref="T:System.Web.Services.Protocols.LogicalMethodInfo"></see> representing the specific function prototype for the XML Web service method to which the SOAP extension is applied.</param>
        /// <param name="attribute">The <see cref="T:System.Web.Services.Protocols.SoapExtensionAttribute"></see> applied to the XML Web service method.</param>
        /// <returns>
        /// The <see cref="T:System.Object"></see> that the SOAP extension initializes for caching.
        /// </returns>
        public override object GetInitializer(LogicalMethodInfo methodInfo,
                                              SoapExtensionAttribute attribute)
        {
            return null;
        }


        /// <summary>
        /// When overridden in a derived class, allows a SOAP extension to initialize data specific to a class implementing an XML Web service at a one time performance cost.
        /// </summary>
        /// <param name="serviceType">The type of the class implementing the XML Web service to which the SOAP extension is applied.</param>
        /// <returns>
        /// The <see cref="T:System.Object"></see> that the SOAP extension initializes for caching.
        /// </returns>
        public override object GetInitializer(System.Type serviceType)
        {
            return null;
        }

        /// <summary>
        /// When overridden in a derived class, allows a SOAP extension to initialize itself using the data cached in the <see cref="M:System.Web.Services.Protocols.SoapExtension.GetInitializer(System.Web.Services.Protocols.LogicalMethodInfo,System.Web.Services.Protocols.SoapExtensionAttribute)"></see> method.
        /// </summary>
        /// <param name="initializer">The <see cref="T:System.Object"></see> returned from <see cref="M:System.Web.Services.Protocols.SoapExtension.GetInitializer(System.Web.Services.Protocols.LogicalMethodInfo,System.Web.Services.Protocols.SoapExtensionAttribute)"></see> cached by ASP.NET.</param>
        public override void Initialize(object initializer)
        {
        }

        //============================================================
        // Override: ChainStream
        //============================================================
        ///<summary>
        ///  ChainStream allows us to save a reference to the Stream representing
        ///  the SOAP request or SOAP response so we can manipulate the request
        ///  (or response) stream.
        ///  We need to save the reference to the stream during ChainStream,
        ///  because the stream is not writable during subsequent calls to
        ///  ProcessMessage.
        /// 
        /// The flow of control is:
        /// 1) ChainStream 
        ///      provides access to the incoming serialized stream buffer
        /// 2) ProcessMessage.BeforeSerialize
        ///      Set the content type to Multipart/related
        /// 3) ProcessMessage.AfterSerialize
        ///      convert the request to be a MIME request, containing the apache:soapHandler type
        /// 4) Web Method is executed
        /// 5) ChainStream
        ///      provides access to the outgoing serialized stream buffer
        /// 6) ProcessMessage.BeforeDeserialize 
        ///      not used
        /// 7) ProcessMessage.AfterDeserialize
        ///      not used
        ///</summary>
        //============================================================
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
        public override Stream ChainStream(Stream chainStream)
        {
            if (chainStream == null)
                throw new ArgumentNullException("stream");

            switch (chainStream.GetType().ToString())
            {
                case "System.Net.ConnectStream":
                    return chainStream;
                // Stream that is obtained after the BeforeSerialize stage is obtained here.
                // By returning a MemoryStream(tempStream), we tell .NET to writethe request to this stream.
                // The original stream(stream) is not readable after "AfterSerialize" stage.
                // So we read from the tempStream, do modifcations, and write to "stream"
                case "System.Web.Services.Protocols.SoapExtensionStream":
                    this.stream = chainStream;
                    tempStream = new MemoryStream();
                    return tempStream;
                default:
                    return chainStream;
            }
        }

        //============================================================
        // Override: ProcessMessage
        //============================================================
        ///<summary>
        ///  ProcessMessage gets control at for stages during the 
        ///  processing of a SOAP request:
        ///  1) BeforeSerialize - before the web method results are 
        ///     serialized for return to the caller via HTTP
        ///  2) AfterSerialize - after the web method results are serialized
        ///     for return to the caller via HTTP
        ///  3) BeforeDeserialize: as the serialized request is received
        ///     via HTTP - before is is deserialized for invocation of the
        ///     web method
        ///  4) AfterDeserialize - after the request is serialized but
        ///     before the web method is invoked
        ///     
        ///</summary>
        //============================================================
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
        public override void ProcessMessage(SoapMessage message)
        {
            switch (message.Stage)
            {
                case SoapMessageStage.BeforeDeserialize:
                    break;

                case SoapMessageStage.AfterDeserialize:
                    break;

                case SoapMessageStage.BeforeSerialize:
                    //We need to set this to multipart, as we are retuning a multipart. The default(text/xml) does not help us

                    message.ContentType = "multipart/related; ";
                    break;

                case SoapMessageStage.AfterSerialize:
                    AfterSerialize();
                    break;

                default:
                    throw new Exception(Resource1.INVALID_STAGE_MESSAGEPROCESSING);
            }
        }

        /// <summary>
        /// Afters the serialize.
        /// </summary>
        private void AfterSerialize()
        {
            tempStream.Position = 0;
            DataHandlerConverter converter = new DataHandlerConverter();
            Dictionary<string, string> contentIdBinaryDataDictionary;
            MemoryStream convertedStream = converter.ConvertStream(tempStream, out contentIdBinaryDataDictionary);

            Mime mimeMultipart = ConstructMimeMessage(contentIdBinaryDataDictionary, convertedStream);
           
            converter = null;
            convertedStream = null;
            contentIdBinaryDataDictionary = null;

            string mimeMessage = mimeMultipart.Content;
            writer = new StreamWriter(stream);
            writer.Write(mimeMessage);
            writer.Flush();
            
            mimeMessage = null;
            mimeMultipart = null;
        }

        private Mime ConstructMimeMessage(Dictionary<string, string> contentIdBinaryDataDictionary, MemoryStream convertedStream)
        {
            //Construct the main MIME part (this is the one to which the xML soap request 
            // and binary MIME parst are added)
            Mime mimeMultipart = new Mime();

            //Construct a MIME for the soap request and add it to the multipart MIME
            Mime xmlMime = ConstructSoapRequestMime(convertedStream);
            mimeMultipart.AppendMime(xmlMime);

            //Add all the binary data referenced by the 'href' attribute of DataHandler elements
            // as seprate MIME parts
            AddBinaryDataAsMime(contentIdBinaryDataDictionary, mimeMultipart);

            return mimeMultipart;
        }

        private Mime ConstructSoapRequestMime(MemoryStream convertedStream)
        {

            //Construct an MIME message containing the XML Part
            Mime xmlMime = new Mime();
            reader = new StreamReader(convertedStream);
            string soaprequest = reader.ReadToEnd();
            xmlMime.SetXmlMessage(soaprequest);
            xmlMime.Charset = "utf-8";
            xmlMime.Encoding = "binary";

            return xmlMime;
        }

        private static void AddBinaryDataAsMime(Dictionary<string, string> contentIdBinaryDataDictionary, Mime mimeMultipart)
        {
            // Foreach value in the dictionary, construct a MIME message with binaryData 
            // and a Content-ID header containg the ContentID
            foreach (KeyValuePair<string, string> keyValuePair in contentIdBinaryDataDictionary)
            {
                Mime binaryDataMime = new Mime();
                binaryDataMime.Headers.Add("Content-Id", keyValuePair.Key);
                binaryDataMime.Headers.Add("Content-Type", "application/octet-stream");
                binaryDataMime.Headers.Add("content-transfer-encoding", "base64");
                binaryDataMime.SetBinaryMessage(keyValuePair.Value);
                mimeMultipart.AppendMime(binaryDataMime);
            }
        }


        #region IDisposable Members

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Disposes the specified the privately held resources.
        /// </summary>
        /// <param name="disposing">if set to <c>true</c> disposes resources.</param>
        private void Dispose(bool disposing)
        {
            // Check to see if Dispose has already been called.
            if (!this.disposed)
            {
                // If disposing equals true, dispose all managed 
                // and unmanaged resources.
                if (disposing)
                {
                    stream.Dispose();
                    tempStream.Dispose();
                    writer.Dispose();
                    reader.Dispose();
                }
            }
            disposed = true;
        }
        #endregion

    }

    /// <summary>
    /// DataHandler for SoapExtension Attribute
    /// </summary>
    [AttributeUsage(AttributeTargets.Method)]
    public sealed class DataHandlerSoapExtensionAttribute : SoapExtensionAttribute
    {
        private int priority;

        ///<summary>
        ///    Soap Extension Attribute ExtensionType property (override)
        ///</summary>
        public override Type ExtensionType
        {
            get { return typeof(DataHandlerSoapExtension); }
        }

        ///<summary>
        ///    Soap Extension Attribute Priority property (override)
        ///</summary>
        public override int Priority
        {
            get { return priority; }
            set { priority = value; }
        }
    }

}
