using System.Diagnostics;
using System.IO;
using System.Xml;

namespace Microsoft.eResearch.GenePattern.Server
{
    /// <summary>
    /// Converts a Map defined as such by 
    /// 
    /// </summary>
    public class DotNetMapToJavaMapConverter
    {
        private XmlDocument xmlDocument;
        private static XmlNamespaceManager namespaceManager;

        private const string xsiPrefix = "xsi";
        private const string xsiNamespaceURI = "http://www.w3.org/2001/XMLSchema-instance";
        private const string soapPrefix = "soap";
        private const string soapNamespaceURI = "http://schemas.xmlsoap.org/soap/envelope/";
        private const string apacheSoapPrefix = "apacheSoap";
        private const string apacheSoapNamespaceURI = "http://xml.apache.org/xml-soap";

        /// <summary>
        /// Converts the given stream to a stream that has the Dot Net map items formatted to match the Java Map Items.
        /// </summary>
        /// <param name="oldStream">The old stream.</param>
        /// <returns>The stream with the Map Items configured to the Java Map format</returns>
        public MemoryStream ConvertStream(Stream oldStream)
        {
            MemoryStream newStream = new MemoryStream();
            xmlDocument = new XmlDocument();
            xmlDocument.Load(oldStream);
            namespaceManager = GetNamespaceManager();
            ConvertDotNetMapToJavaMap();
            xmlDocument.Save(newStream);
            newStream.Position = 0;

            xmlDocument = null;
            oldStream = null;
            
            return newStream;
        }

        /// <summary>
        /// Converts the dot net map to java map.
        /// </summary>
        private void ConvertDotNetMapToJavaMap()
        {
            //Get all map elements
            string mapNodesQuery = "soap:Envelope//*[contains(@xsi:type,':Map')]";
            XmlNodeList mapNodes = this.xmlDocument.SelectNodes(mapNodesQuery, namespaceManager);
            XmlNode soapBodyNode;
            XmlNodeList itemNodes;

            foreach (XmlNode mapNode in mapNodes)
            {
                soapBodyNode = this.xmlDocument.SelectSingleNode("/soap:Envelope/soap:Body", namespaceManager);
                
                Debug.Assert(soapBodyNode != null);
                
                itemNodes = mapNode.SelectNodes("./item", namespaceManager);
                ConstructJavaMapItems(itemNodes, mapNode, soapBodyNode);
                RemoveChildItemNodes(itemNodes, mapNode);
            }

            mapNodes = null;
            soapBodyNode = null;
            itemNodes = null;
        }

        /// <summary>
        /// Constructs the java map items.
        /// </summary>
        /// <param name="itemNodes">The item nodes.</param>
        /// <param name="mapNode">The map node.</param>
        /// <param name="soapBodyNode">The SOAP body node.</param>
        private void ConstructJavaMapItems(XmlNodeList itemNodes, XmlNode mapNode, XmlNode soapBodyNode)
        {
            // get all the map Item Array nodes referenced by item nodes in this collection
            // and get the Key and Value elements and convert them to the dot net equivalent
            XmlNode mapItemArrayNode;
            XmlNodeList arrayItemNode;

            foreach (XmlNode itemNode in itemNodes)
            {
                string itemNodeID = GetNodeId(itemNode);
                string idQuery = "soap:Envelope//*[@id ='" + itemNodeID + "']";
                
                mapItemArrayNode = this.xmlDocument.SelectSingleNode(idQuery, namespaceManager);
                arrayItemNode = mapItemArrayNode.SelectNodes("./Item", namespaceManager);
                ConvertMapItemToMap(arrayItemNode, mapNode, soapBodyNode);
                
                //Remove the array child nodes of mapItemArrayNode
                RemoveChildItemNodes(arrayItemNode, mapItemArrayNode);
                soapBodyNode.RemoveChild(mapItemArrayNode);
            }

            mapItemArrayNode = null;
            arrayItemNode = null;
        }

        /// <summary>
        /// Converts the map item to map.
        /// </summary>
        /// <param name="arrayItemNode">The array item node.</param>
        /// <param name="mapNode">The map node.</param>
        /// <param name="soapBodyNode">The SOAP body node.</param>
        private void ConvertMapItemToMap(XmlNodeList arrayItemNode, XmlNode mapNode, XmlNode soapBodyNode)
        {
            XmlNode mapItemNode;
            XmlNode keyNode;
            XmlNode valueNode;
            XmlNode itemNode;

            foreach (XmlNode ItemNode in arrayItemNode)
            {
                string ItemNodeID = GetNodeId(ItemNode);
                string idQuery = "soap:Envelope//*[@id ='" + ItemNodeID + "']";

                mapItemNode = this.xmlDocument.SelectSingleNode(idQuery, namespaceManager);
                keyNode = mapItemNode.SelectSingleNode("./key");
                valueNode = mapItemNode.SelectSingleNode("./value");
                itemNode = mapNode.OwnerDocument.CreateElement("item");
                itemNode.AppendChild(keyNode);
                itemNode.AppendChild(valueNode);

                mapNode.AppendChild(itemNode);
                soapBodyNode.RemoveChild(mapItemNode);
            }

            mapItemNode = null;
            keyNode = null;
            valueNode = null;
            itemNode = null;
        }

        /// <summary>
        /// Removes the child item nodes.
        /// </summary>
        /// <param name="childNodes">The child nodes.</param>
        /// <param name="parentNode">The parent node.</param>
        private static void RemoveChildItemNodes(XmlNodeList childNodes, XmlNode parentNode)
        {
            foreach (XmlNode ItemNode in childNodes)
            {
                parentNode.RemoveChild(ItemNode);
            }
        }

        /// <summary>
        /// Gets the node id.
        /// </summary>
        /// <param name="itemNode">The item node.</param>
        /// <returns></returns>
        private static string GetNodeId(XmlNode itemNode)
        {
            XmlNode hrefNode = itemNode.Attributes["href"];
            string id = hrefNode.Value.Replace("#", string.Empty);

            return id;
        }

        /// <summary>
        /// Gets the namespace manager.
        /// </summary>
        /// <returns></returns>
        private XmlNamespaceManager GetNamespaceManager()
        {
            namespaceManager = new XmlNamespaceManager(xmlDocument.NameTable);
            namespaceManager.AddNamespace(xsiPrefix, xsiNamespaceURI);
            namespaceManager.AddNamespace(soapPrefix, soapNamespaceURI);
            namespaceManager.AddNamespace(apacheSoapPrefix, apacheSoapNamespaceURI);
            return namespaceManager;
        }
    }
}
