using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Text;
using System.Web.Services.Protocols;
using System.Xml;

namespace Microsoft.eResearch.GenePattern.Server
{
    /// <summary>
    /// This class is used as an attribute for Web Service methods
    /// to rewrite the values of int[] elements to be inline in the 
    /// xml element that represents the array element instead of being
    /// reference by id to another element.
    /// </summary>
    public class IntArrayRewrite : StreamRewriteExtension
    {
        string elementName;


        /// <summary>
        /// Fetches the ArrayElementName property as configured for an attribute declaration
        /// </summary>
        /// <param name="methodInfo">A <see cref="T:System.Web.Services.Protocols.LogicalMethodInfo"></see> representing the specific function prototype for the XML Web service method to which the SOAP extension is applied.</param>
        /// <param name="attribute">The <see cref="T:System.Web.Services.Protocols.SoapExtensionAttribute"></see> applied to the XML Web service method.</param>
        /// <returns>
        /// The value of <see cref="T:System.Object"></see> the ArrayElementName configuration parameter.
        /// </returns>
        public override object GetInitializer(LogicalMethodInfo methodInfo, SoapExtensionAttribute attribute)
        {
            if (attribute == null)
                throw new ArgumentNullException("attribute");

            return ((IntArrayRewriteAttribute)attribute).ArrayElementName;
        }

        /// <summary>
        /// Extracts the ArrayElementName from the initialzer object (as created in the GetInitializer method)
        /// </summary>
        /// <param name="initializer">The <see cref="T:System.Object"></see> returned from <see cref="M:System.Web.Services.Protocols.SoapExtension.GetInitializer(System.Web.Services.Protocols.LogicalMethodInfo,System.Web.Services.Protocols.SoapExtensionAttribute)"></see> cached by ASP.NET.</param>
        public override void Initialize(object initializer)
        {
            elementName = (string)initializer;
        }

        /// <summary>
        /// Rewrites the stream by inlining integer values for the elements named
        /// like the ArrayElementName property (having an href attribute)
        /// </summary>
        /// <param name="message"></param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object,System.Object)")]
        protected override void BeforeDeserialize(SoapMessage message)
        {
            XmlDocument document = new XmlDocument();

            using (MemoryStream memStream = GetMemoryStream(oldStream))
            {
                document.Load(memStream);
                memStream.Position = 0;
            }

            XmlNamespaceManager namespaceManager = new XmlNamespaceManager(document.NameTable);
            namespaceManager.AddNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance");
            namespaceManager.AddNamespace("xsd", "http://www.w3.org/2001/XMLSchema");
            namespaceManager.AddNamespace("soap", "http://schemas.xmlsoap.org/soap/envelope/");

            Dictionary<string, XmlElement> arrayElementDictionary = new Dictionary<string, XmlElement>();
            
            XmlNodeList arrayElementNodes = document.SelectNodes("//" + elementName + "[@href]", namespaceManager);
            foreach (XmlElement arrayElement in arrayElementNodes)
            {
                string hrefId = arrayElement.GetAttribute("href");
                string id = hrefId.Substring(hrefId.LastIndexOf('#') + 1);

                arrayElementDictionary.Add(id, arrayElement);
            }

            string xPathTemplate = "//multiRef[@id='{0}']";

            foreach (KeyValuePair<string, XmlElement> keyValuePair in arrayElementDictionary)
            {
                XmlElement valueElement = document.SelectSingleNode(string.Format(xPathTemplate, keyValuePair.Key, CultureInfo.InvariantCulture)) as XmlElement;
                if(valueElement != null)
                {
                    keyValuePair.Value.InnerText = valueElement.InnerText;
                    valueElement.ParentNode.RemoveChild(valueElement);
                    keyValuePair.Value.RemoveAttribute("href");
                }                
            }

            StreamWriter sw = new StreamWriter(newStream, Encoding.Default);

            sw.WriteLine(document.OuterXml);
            sw.Flush();
            newStream.Flush();
            newStream.SetLength(newStream.Position);
            newStream.Position = 0;

        }
    }

    /// <summary>
    /// Configures the type IntArrayRewrite as an Attribute
    /// </summary>
    [AttributeUsage(AttributeTargets.Method)]
    public sealed class IntArrayRewriteAttribute : SoapExtensionAttribute
    {
        private int priority;
        private string arrayElementName;

        /// <summary>
        /// Gets or sets the name of the array element of the int[] in the soap message.
        /// </summary>
        /// <value>The name of the array element.</value>
        public string ArrayElementName
        {
            get { return arrayElementName; }
            set { arrayElementName = value; }
        }

        ///<summary>
        ///    Soap Extension Attribute ExtensionType property (override)
        ///</summary>
        public override Type ExtensionType
        {
            get { return typeof(IntArrayRewrite); }
        }

        ///<summary>
        ///    Soap Extension Attribute Priority property (override)
        ///</summary>
        public override int Priority
        {
            get { return priority; }
            set { priority = value; }
        }
    }

}
