using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Xml;

namespace Microsoft.eResearch.GenePattern.Server
{
    /// <summary>
    /// Class that converts Java Map XML from an Apache Web Service to its .NET form
    /// </summary>
    public sealed class JavaMapToDotNetMapConverter
    {
        private const string xsiPrefix = "xsi";
        private const string xsiNamespaceURI = "http://www.w3.org/2001/XMLSchema-instance";
        private const string soapPrefix = "soap";
        private const string soapNamespaceURI = "http://schemas.xmlsoap.org/soap/envelope/";
        private const string mapItemElementName = "mapItem";

        /// <summary>
        /// Converts the java map to dot net map.
        /// </summary>
        /// <param name="originalStream">The original stream.</param>
        /// <param name="destinationStream">The destination stream.</param>
        public void Convert(Stream originalStream, Stream destinationStream)
        {
            //MemoryStream destinationStream = new MemoryStream(); 
            XmlDocument xmlDocument = new XmlDocument();

            try
            {
                xmlDocument.Load(originalStream);
            }
            catch(XmlException)
            {
                // sometimes the stream is truncated.  i don't know why.
            }
            
            XmlNamespaceManager namespaceManager = GetNamespaceManager(xmlDocument);

            //Get all elemnts with Map Attribute
            string xpathQuery = "soap:Envelope//*[contains(@xsi:type, ':Map')]";
            
            XmlNodeList mapItems = xmlDocument.SelectNodes(xpathQuery, namespaceManager);
            FormatJavaMapToDotNetMap(mapItems, namespaceManager);

            xmlDocument.Save(destinationStream);

            xmlDocument = null;
        }

        /// <summary>
        /// Formats the java map to dot net map.
        /// </summary>
        /// <param name="mapItems">The map items.</param>
        /// <param name="namespaceManager">The namespace manager.</param>
        private static void FormatJavaMapToDotNetMap(XmlNodeList mapItems, XmlNamespaceManager namespaceManager)
        {
            XmlNodeList items;
            List<XmlNode> mapItemsList;

            foreach (XmlNode mapItemAxis in mapItems)
            {
                //Get the items
                items = mapItemAxis.SelectNodes("./item", namespaceManager);
                mapItemsList = ConstructMapItemsList(items);
                RemoveOldItemNodes(items, mapItemAxis);
                ConstructMapItemArray(mapItemAxis, mapItemsList);
            }

            items = null;
            mapItemsList = null;
        }

        /// <summary>
        /// Constructs the map item array.
        /// </summary>
        /// <param name="mapItemAxis">The map item axis.</param>
        /// <param name="mapItemsList">The map items list.</param>
        private static void ConstructMapItemArray(XmlNode mapItemAxis, List<XmlNode> mapItemsList)
        {
            string apacheNs = "apacheNs";
            string itemArrayElementName = "item";
            
            XmlElement itemDotNet = mapItemAxis.OwnerDocument.CreateElement(itemArrayElementName);

            //Create the attributes that identify it as an array
            XmlNode arrayTypeAttribute = AddAttribute(mapItemAxis, "soapenc", "arrayType", "http://schemas.xmlsoap.org/soap/encoding/", apacheNs + ":mapItem[" + mapItemsList.Count + "]");
            itemDotNet.Attributes.Append(arrayTypeAttribute as XmlAttribute);

            XmlNode apacheNsAttribute = AddAttribute(mapItemAxis, "xmlns", apacheNs, "http://www.w3.org/2000/xmlns/","http://xml.apache.org/xml-soap");   
            itemDotNet.Attributes.Append(apacheNsAttribute as XmlAttribute);

            //Add the MapItems to the Array
            foreach (XmlNode mapItem in mapItemsList)
            {
                itemDotNet.AppendChild(mapItem);
            }
            mapItemAxis.AppendChild(itemDotNet);

            itemDotNet = null;
            arrayTypeAttribute = null;
            apacheNsAttribute = null;
        }

        /// <summary>
        /// Adds the attribute.
        /// </summary>
        /// <param name="element">The element.</param>
        /// <param name="prefix">The prefix.</param>
        /// <param name="localName">Name of the local.</param>
        /// <param name="namespaceURI">The namespace URI.</param>
        /// <param name="value">The value.</param>
        /// <returns></returns>
        private static XmlNode AddAttribute(XmlNode element,string prefix, string localName, string namespaceURI,  string value)
        {
            XmlNode arrayTypeAttribute =
                element.OwnerDocument.CreateAttribute(prefix,localName, namespaceURI);
            
            arrayTypeAttribute.Value = value;
            return arrayTypeAttribute;
        }

        /// <summary>
        /// Removes the old item nodes.
        /// </summary>
        /// <param name="items">The items.</param>
        /// <param name="mapItemAxis">The map item axis.</param>
        private static void RemoveOldItemNodes(XmlNodeList items, XmlNode mapItemAxis)
        {
            //Remove the old item elements
            foreach (XmlNode itemNode in items)
            {
                mapItemAxis.RemoveChild(itemNode);
            }
        }

        /// <summary>
        /// Constructs the map items list.
        /// </summary>
        /// <param name="items">The items.</param>
        /// <returns></returns>
        private static List<XmlNode> ConstructMapItemsList(XmlNodeList items)
        {
            List<XmlNode> mapItemsList = new List<XmlNode>();
            XmlNode mapItemDotNet;
            XmlNodeList children;
            IEnumerator enumerator;
            XmlNode childNode;

            foreach (XmlNode item in items)
            {
                mapItemDotNet = item.OwnerDocument.CreateNode(XmlNodeType.Element, mapItemElementName, String.Empty);
                children = item.ChildNodes;
                
                enumerator = children.GetEnumerator();
                while(enumerator.MoveNext())
                {
                    childNode = enumerator.Current as XmlNode;
                    if (childNode != null) mapItemDotNet.AppendChild(childNode.Clone());
                }

                mapItemsList.Add(mapItemDotNet);
            }

            mapItemDotNet = null;
            childNode = null;
            children = null;
            enumerator = null;

            return mapItemsList;
        }

        /// <summary>
        /// Gets the namespace manager.
        /// </summary>
        /// <param name="xmlDocument">The XML document.</param>
        /// <returns></returns>
        private static XmlNamespaceManager GetNamespaceManager(XmlDocument xmlDocument)
        {
            XmlNamespaceManager namespaceManager = new XmlNamespaceManager(xmlDocument.NameTable);
            namespaceManager.AddNamespace(xsiPrefix, xsiNamespaceURI);
            namespaceManager.AddNamespace(soapPrefix, soapNamespaceURI);
            return namespaceManager;
        }
    }
    
}
