using System;
using System.Collections.Generic;
using System.Text;

namespace Microsoft.eResearch.GenePattern.Server.SoapExtensions
{
    /// <summary>
    /// Represents the Content type of a MIME message
    /// </summary>
    public enum ContentType
    {
        /// <summary>
        /// Represents Text xontent
        /// </summary>
        Text,
        /// <summary>
        /// Represents XML content
        /// </summary>
        Xml,
        /// <summary>
        /// Represents a multi part content
        /// </summary>
        Multipart,
        /// <summary>
        /// Represents an Ocet Stream
        /// </summary>
        OcetStream
        
    }


    /// <summary>
    /// Used for constructing a MIME message
    /// </summary>
    public class Mime
    {
        #region Fields

        private const string headerSeparator = ":";
        private const string encodingHeader = "content-transfer-encoding";
        private const string boundaryIdentifier = "--";


        private StringBuilder contents;
        private ContentType contentType;
        private string message;
        private string charset;
        private string encoding;
        private string boundary;
        private bool isMultipart;
        private List<Mime> multipartList;
        private IDictionary<string, string> headers;

        #endregion

        /// <summary>
        /// Gets a value indicating whether this instance is multipart.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is multipart; otherwise, <c>false</c>.
        /// </value>
        public bool IsMultipart
        {
            get { return isMultipart; }
        }

        /// <summary>
        /// Gets or sets the multipart list.
        /// </summary>
        /// <value>The multipart list.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<Mime> MultipartList
        {
            get { return multipartList; }
        }

        /// <summary>
        /// Gets the boundary identifier.
        /// </summary>
        /// <value>The boundary identifier.</value>
        public string BoundaryIdentifier
        {
            get { return boundaryIdentifier; }
        } 

        /// <summary>
        /// Initializes a new instance of the <see cref="Mime"/> class.
        /// </summary>
        public Mime()
        {
            this.contents = new StringBuilder();
            
        }

        #region Properties

        /// <summary>
        /// Gets or sets the boundary.
        /// </summary>
        /// <value>The boundary.</value>
        public string Boundary
        {
            get { return boundary; }
            set { boundary = value; }
        }

        /// <summary>
        /// Gets the content.
        /// </summary>
        /// <value>The content.</value>
        public string  Content
        {
            get
            {
                ConstructContent();
                return this.contents.ToString();
            }

        }

        /// <summary>
        /// Gets or sets the type of the content.
        /// </summary>
        /// <value>The type of the content.</value>
        public ContentType ContentType
        {
            get { return contentType; }
            set
            {
                contentType = value;
            }
        }

        /// <summary>
        /// Gets or sets the charset.
        /// </summary>
        /// <value>The charset.</value>
        public string Charset
        {
            get { return charset; }
            set { charset = value; }
        }

        /// <summary>
        /// Gets or sets the encoding.
        /// </summary>
        /// <value>The encoding.</value>
        public string Encoding
        {
            get { return encoding; }
            set { encoding = value; }
        }

        /// <summary>
        /// Gets the headers.
        /// </summary>
        /// <value>The headers.</value>
        public IDictionary<string, string> Headers
        {
            get
            {
                if (headers == null) headers = new Dictionary<string, string>();
                return headers;
            }
        }

        /// <summary>
        /// Clears the content.
        /// </summary>
        public void ClearContent()
        {
            this.contents = new StringBuilder();
        }

        #endregion

        #region Methods

        #region Public

        /// <summary>
        /// Appends the MIME.
        /// </summary>
        /// <param name="mime">The MIME.</param>
        public void AppendMime(Mime mime)
        {
            this.contentType = SoapExtensions.ContentType.Multipart;
            this.isMultipart = true;

            if (String.IsNullOrEmpty(this.boundary)) this.boundary = Guid.NewGuid().ToString();
            if(this.multipartList == null) this.multipartList = new List<Mime>();
            
            this.multipartList.Add(mime);
        }

        /// <summary>
        /// Sets the binary message.
        /// </summary>
        /// <param name="bytes">The bytes.</param>
        public void SetBinaryMessage(string bytes)
        {
            //string data = Convert.ToBase64String(bytes, Base64FormattingOptions.InsertLineBreaks);
            this.message = bytes;
            
        }


        /// <summary>
        /// Sets the text message.
        /// </summary>
        /// <param name="textMessage">The text message.</param>
        public void SetTextMessage(string textMessage)
        {  
            this.message = textMessage;
        }

        /// <summary>
        /// Sets the XML message.
        /// </summary>
        /// <param name="xmlMessage">The XML message.</param>
        public void SetXmlMessage(string xmlMessage)
        {
            this.ContentType = SoapExtensions.ContentType.Xml;
            this.message = xmlMessage;
        }

        #endregion

        #region Private

        private void AddBoundary()
        {
            this.contents.AppendLine();
            if(String.IsNullOrEmpty(this.boundary))
            {
                this.boundary = Guid.NewGuid().ToString();
            }
            
            this.contents.AppendLine(boundaryIdentifier + this.boundary);
        }

        private void StartBoundarySeparator()
        {
            this.contents.Append("boundary=\"");
            this.contents.Append(this.boundary);
            this.contents.AppendLine("\"");
        }

        private void AddCharacterSet()
        {
            if (!String.IsNullOrEmpty(this.charset))
            {
                this.contents.Append(" charset=\"");
                this.contents.Append(this.charset);
                this.contents.AppendLine("\"");
            }
        }

        private void AddContentType(ContentType messageContentType)
        {
            string contentTypedata = null;
            
            switch(messageContentType)
            {
                case(SoapExtensions.ContentType.Xml):
                    {
                        contentTypedata = "text/xml;";
                        break;
                    }
                case (SoapExtensions.ContentType.Multipart):
                    {
                        contentTypedata = "multipart/related;";
                        break;
                    }
              
            }

            if (!String.IsNullOrEmpty(contentTypedata))
            {
                this.contents.Append("content-type: ");
                this.contents.AppendLine(contentTypedata);
                this.contents.Append("\t");
            }
        }

        private void AddEncoding()
        {
            if (!String.IsNullOrEmpty(this.encoding))
            {
                this.contents.Append(encodingHeader + headerSeparator);
                this.contents.AppendLine(this.encoding);
            }
        }

        private void AddHeaders()
        {
            if (this.headers != null)
            {
                foreach (KeyValuePair<string, string> header in this.headers)
                {
                    this.contents.Append(header.Key);
                    this.contents.AppendLine(headerSeparator + header.Value);
                }
            }
          
        }

        private void AddMessage()
        {
            this.contents.AppendLine();
            this.contents.Append(message);
        }

        private void AddMultiparts()
        {
            AddBoundary();
            foreach (Mime mime in this.multipartList)
            {
                this.contents.Append(mime.Content);
                this.AddBoundary();
            }
            this.contents.Insert(this.contents.Length - 2, boundaryIdentifier);
            this.contents.AppendLine();
        }


        /// <summary>
        /// Constructs the content.
        /// </summary>
        private void ConstructContent()
        {
            this.contents = new StringBuilder();
            this.AddContentType(ContentType);
                
            //If it is  multipartList message we have to 
            // start teh boundary identifier
            if (this.isMultipart)
            {
                StartBoundarySeparator();
            }
            else
            {
                AddHeaders();
                AddCharacterSet();
                AddEncoding();
            }
              
            this.AddMessage();
            //Add multiparts if it is a multipart message
            if(this.isMultipart) AddMultiparts();
        }

        /// <summary>
        /// Gets the content start.
        /// </summary>
        /// <value>The get content start.</value>
        /// <returns></returns>
        public string GetContentStart
        {
            get
            {
                this.contents = new StringBuilder();
                this.AddContentType(ContentType);

                //If it is  multipartList message we have to 
                // start teh boundary identifier
                if (this.isMultipart)
                {
                    StartBoundarySeparator();
                }
                else
                {
                    AddHeaders();
                    AddCharacterSet();
                    AddEncoding();
                }

                this.contents.AppendLine();
                return contents.ToString();
            }
        }

        /// <summary>
        /// Gets the content message.
        /// </summary>
        /// <returns></returns>
        public string GetContentMessage
        {
            get
            {
                return this.message;
            }
        }

        /// <summary>
        /// Gets the multiparts.
        /// </summary>
        /// <value>The get multiparts.</value>
        /// <returns></returns>
        public string GetMultiparts
        {
            get
            {
                this.contents = new StringBuilder();
                AddBoundary();
                foreach (Mime mime in this.multipartList)
                {
                    this.contents.Append(mime.Content);
                    this.AddBoundary();
                }
                this.contents.Insert(this.contents.Length - 2, boundaryIdentifier);
                this.contents.AppendLine();
                return contents.ToString();
            }
        }

        /// <summary>
        /// Gets the content.
        /// </summary>
        /// <value>The content of the get strutured.</value>
        /// <returns></returns>
        public string GetStruturedContent
        {
            get
            {
                this.contents = new StringBuilder();
                this.AddContentType(ContentType);

                //If it is  multipartList message we have to 
                // start teh boundary identifier
                if (this.isMultipart)
                {
                    StartBoundarySeparator();
                }
                else
                {
                    AddHeaders();
                    AddCharacterSet();
                    AddEncoding();
                }

                this.AddMessage();

                return contents.ToString();
            }
        }

        /// <summary>
        /// Gets the boundary.
        /// </summary>
        /// <value>The get content boundary.</value>
        /// <returns></returns>
        public string GetContentBoundary
        {
            get
            {
                this.contents = new StringBuilder();
                this.contents.AppendLine();
                if (String.IsNullOrEmpty(this.boundary))
                {
                    this.boundary = Guid.NewGuid().ToString();
                }

                this.contents.AppendLine(boundaryIdentifier + this.boundary);
                return contents.ToString();
            }
        }

        #endregion

        #endregion
    }
}
