using System;
using System.Collections.Generic;
using System.IO;
using System.Web.Services.Protocols;
using Microsoft.eResearch.GenePattern.Server.SoapExtensions;

namespace Microsoft.eResearch.GenePattern.Server
{
    //============================================================
    // MapDataHandlerExtension Soap Extension
    //============================================================
    ///<summary>
    ///     The MapDataHandlerExtension class is a SOAP extension that allows
    ///     us to convert the Map returned in the repsonse to the "Map"
    ///     type expected by .Net and also the Data Handler elements.
    ///     
    ///     The resulting SOAP message with Map items mapped
    ///     is then substituted for the original unserialized request
    ///     during the ProcessMessage.
    ///     
    ///     The 'SubmitJob' web method is decorated with the attribute:
    ///     [MapDataHandlerAttribute]
    ///     to enable this SOAP extension.
    ///     
    ///</summary>
    public class MapDataHandlerExtension : StreamRewriteExtension
    {

        private bool disposed;
        protected Stream stream;      // original stream that is sent out to server 
        protected Stream tempStream; //temp stream, where request is written to bt .NET

        #region Overriden Methods

        /// <summary>
        /// When overridden in a derived class, allows a SOAP extension to initialize data specific to an XML Web service method using an attribute applied to the XML Web service method at a one time performance cost.
        /// </summary>
        /// <param name="methodInfo">A <see cref="T:System.Web.Services.Protocols.LogicalMethodInfo"></see> representing the specific function prototype for the XML Web service method to which the SOAP extension is applied.</param>
        /// <param name="attribute">The <see cref="T:System.Web.Services.Protocols.SoapExtensionAttribute"></see> applied to the XML Web service method.</param>
        /// <returns>
        /// The <see cref="T:System.Object"></see> that the SOAP extension initializes for caching.
        /// </returns>
        public override object GetInitializer(LogicalMethodInfo methodInfo,
                                              SoapExtensionAttribute attribute)
        {
            return null;
        }


        /// <summary>
        /// When overridden in a derived class, allows a SOAP extension to initialize data specific to a class implementing an XML Web service at a one time performance cost.
        /// </summary>
        /// <param name="serviceType">The type of the class implementing the XML Web service to which the SOAP extension is applied.</param>
        /// <returns>
        /// The <see cref="T:System.Object"></see> that the SOAP extension initializes for caching.
        /// </returns>
        public override object GetInitializer(Type serviceType)
        {
            return null;
        }

        /// <summary>
        /// When overridden in a derived class, allows a SOAP extension to initialize itself using the data cached in the <see cref="M:System.Web.Services.Protocols.SoapExtension.GetInitializer(System.Web.Services.Protocols.LogicalMethodInfo,System.Web.Services.Protocols.SoapExtensionAttribute)"></see> method.
        /// </summary>
        /// <param name="initializer">The <see cref="T:System.Object"></see> returned from <see cref="M:System.Web.Services.Protocols.SoapExtension.GetInitializer(System.Web.Services.Protocols.LogicalMethodInfo,System.Web.Services.Protocols.SoapExtensionAttribute)"></see> cached by ASP.NET.</param>
        public override void Initialize(object initializer)
        {
        }

        //============================================================
        // Override: ChainStream
        //============================================================
        ///<summary>
        ///  ChainStream allows us to save a reference to the Stream representing
        ///  the SOAP request or SOAP response so we can manipulate the request
        ///  (or response) stream.
        ///  We need to save the reference to the stream during ChainStream,
        ///  because the stream is not writable during subsequent calls to
        ///  ProcessMessage.
        /// 
        /// The flow of control is:
        /// 1) ChainStream 
        ///      provides access to the incoming serialized stream buffer
        /// 2) ProcessMessage.BeforeSerialize
        ///      Set the content type to Multipart/related
        /// 3) ProcessMessage.AfterSerialize
        ///      convert the request to be a MIME request, containing the apache:soapHandler type
        /// 4) Web Method is executed
        /// 5) ChainStream
        ///      provides access to the outgoing serialized stream buffer
        /// 6) ProcessMessage.BeforeDeserialize 
        ///      not used
        /// 7) ProcessMessage.AfterDeserialize
        ///      not used
        ///</summary>
        //============================================================
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
        public override Stream ChainStream(Stream chainStream)
        {
            if (chainStream == null)
                throw new ArgumentNullException("stream");

            switch (chainStream.GetType().ToString())
            {
                case "System.Net.ConnectStream":
                    return chainStream;
                // Stream that is obtained after the BeforeSerialize stage is obtained here.
                // By returning a MemoryStream(tempStream), we tell .NET to write the request to this stream.
                // The original stream(stream) is not readable after "AfterSerialize" stage.
                // So we read from the tempStream, do modifcations, and write to "stream"
                case "System.Web.Services.Protocols.SoapExtensionStream":
                    this.stream = chainStream;
                    tempStream = new MemoryStream();
                    return tempStream;
                default:
                    return chainStream;
            }
        }

        //============================================================
        // Override: ProcessMessage
        //============================================================
        ///<summary>
        ///  ProcessMessage gets control at for stages during the 
        ///  processing of a SOAP request:
        ///  1) BeforeSerialize - before the web method results are 
        ///     serialized for return to the caller via HTTP
        ///  2) AfterSerialize - after the web method results are serialized
        ///     for return to the caller via HTTP
        ///  3) BeforeDeserialize: as the serialized request is received
        ///     via HTTP - before is is deserialized for invocation of the
        ///     web method
        ///  4) AfterDeserialize - after the request is serialized but
        ///     before the web method is invoked
        ///     
        ///</summary>
        //============================================================
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
        public override void ProcessMessage(SoapMessage message)
        {
            switch (message.Stage)
            {
                case SoapMessageStage.BeforeDeserialize:
                    break;

                case SoapMessageStage.AfterDeserialize:
                    break;

                case SoapMessageStage.BeforeSerialize:
                    //We need to set this to multipart, as we are retuning a multipart. The default(text/xml) does not help us

                    message.ContentType = "multipart/related; ";
                    break;

                case SoapMessageStage.AfterSerialize:
                    AfterSerialize();
                    break;

                default:
                    throw new Exception(Resource1.INVALID_STAGE_MESSAGEPROCESSING);
            }
        }

        /// <summary>
        /// Handles the needed processing in the AfterSerialize stage
        /// </summary>
        /// <param name="message">The message.</param>
        private void AfterSerialize()
        {
            tempStream.Position = 0;

            DotNetMapToJavaMapConverter dotNetMapToJavaMapConverter = new DotNetMapToJavaMapConverter();
            MemoryStream outputStream = dotNetMapToJavaMapConverter.ConvertStream(tempStream);
            outputStream.Flush();


            DataHandlerConverter converter = new DataHandlerConverter();
            Dictionary<string, string> contentIdBinaryDataDictionary;
            MemoryStream convertedStream = converter.ConvertStream(outputStream, out contentIdBinaryDataDictionary);
            outputStream = null;

            convertedStream.Position = 0;
            Mime mimeMultipart = ConstructMimeMessage(contentIdBinaryDataDictionary, convertedStream);
            contentIdBinaryDataDictionary = null;
            convertedStream = null;
            GC.Collect();

            WriteMimeMessage(mimeMultipart);
            
            mimeMultipart = null;
            GC.Collect();
        }

        #endregion

        #region Write Mime Message To Stream

        /// <summary>
        /// Writes the MIME message.
        /// </summary>
        /// <param name="mimeMultipart">The MIME multipart.</param>
        private void WriteMimeMessage(Mime mimeMultipart)
        {
            writer = new StreamWriter(stream);
            writer.Write(mimeMultipart.GetStruturedContent);
            
            if (mimeMultipart.IsMultipart)
            {
                for (int i = 0; i < mimeMultipart.MultipartList.Count; i++)
                {
                    writer.Write(mimeMultipart.GetContentBoundary);
                    WriteMultiPartContent(mimeMultipart.MultipartList[i]);

                    if (i == mimeMultipart.MultipartList.Count - 1)
                    {
                        string boundary = mimeMultipart.GetContentBoundary;
                        writer.Write(boundary.Insert(boundary.Length - 2, mimeMultipart.BoundaryIdentifier));
                        writer.WriteLine();
                    }
                    else
                        writer.Write(mimeMultipart.GetContentBoundary);

                    writer.Flush();
                }
            }

            writer.Flush();
            writer.Dispose();
            writer = null;
        }

        /// <summary>
        /// Writes the content of the multi part.
        /// </summary>
        /// <param name="mimeMultipart">The MIME multipart.</param>
        private void WriteMultiPartContent(Mime mimeMultipart)
        {
            writer.Write(mimeMultipart.GetContentStart);
            string message = mimeMultipart.GetContentMessage;
         
            if (message.StartsWith("File"))
                SetBinaryData(message);
            else
                writer.Write(message);
            
            if (mimeMultipart.IsMultipart)
            {
                writer.Write(mimeMultipart.GetMultiparts);
            }
        }

        /// <summary>
        /// Sets the binary data.
        /// </summary>
        /// <param name="message">The message.</param>
        private void SetBinaryData(string message)
        {
            string[] splitData = message.Split(new char[] { '?' }, StringSplitOptions.RemoveEmptyEntries);
            bool isDataFromDocument = splitData.Length > 2 ? true : false;
            string filePath = splitData[1];
            splitData = null;

            if (File.Exists(filePath))
            {
                using (FileStream input = new FileStream(filePath, FileMode.Open, FileAccess.Read))
                {
                    byte[] buffer = new byte[(int)input.Length];
                    input.Read(buffer, 0, buffer.Length);
                    input.Flush();
                    writer.Write(Convert.ToBase64String(buffer));
                    buffer = null;
                    GC.Collect();
                }
                if (isDataFromDocument)
                {
                    File.Delete(filePath);
                }
            }
        }

        #endregion

        #region Construct MIME Message

        /// <summary>
        /// Constructs the MIME message.
        /// </summary>
        /// <param name="contentIdBinaryDataDictionary">The content id binary data dictionary.</param>
        /// <param name="convertedStream">The converted stream.</param>
        /// <returns></returns>
        private Mime ConstructMimeMessage(Dictionary<string, string> contentIdBinaryDataDictionary, MemoryStream convertedStream)
        {
            //Construct the main MIME part (this is the one to which the xML soap request 
            // and binary MIME parst are added)
            Mime mimeMultipart = ConstructMimeMultipart();

            //Construct a MIME for the soap request and add it to the multipart MIME
            Mime xmlMime = ConstructSoapRequestMime(convertedStream);
            mimeMultipart.AppendMime(xmlMime);

            //Add all the binary data referenced by the 'href' attribute of DataHandler elements
            // as seprate MIME parts
            AddBinaryDataAsMime(contentIdBinaryDataDictionary, mimeMultipart);

            return mimeMultipart;
        }

        /// <summary>
        /// Constructs the SOAP request MIME.
        /// </summary>
        /// <param name="convertedStream">The converted stream.</param>
        /// <returns></returns>
        private Mime ConstructSoapRequestMime(MemoryStream convertedStream)
        {

            //Construct an MIME message containing the XML Part
            Mime xmlMime = new Mime();
            reader = new StreamReader(convertedStream);
            string soaprequest = reader.ReadToEnd();
            xmlMime.SetXmlMessage(soaprequest);
            xmlMime.Charset = "utf-8";
            xmlMime.Encoding = "binary";

            return xmlMime;
        }

        /// <summary>
        /// Constructs the MIME multipart.
        /// </summary>
        /// <returns></returns>
        private static Mime ConstructMimeMultipart()
        {
            Mime mimeMultipart = new Mime();
            return mimeMultipart;
        }

        /// <summary>
        /// Adds the binary data as MIME.
        /// </summary>
        /// <param name="contentIdBinaryDataDictionary">The content id binary data dictionary.</param>
        /// <param name="mimeMultipart">The MIME multipart.</param>
        private static void AddBinaryDataAsMime(Dictionary<string, string> contentIdBinaryDataDictionary, Mime mimeMultipart)
        {
            // Foreach value in the dictionary, construct a MIME message with binaryData 
            // and a Content-ID header containg the ContentID
            foreach (KeyValuePair<string, string> keyValuePair in contentIdBinaryDataDictionary)
            {
                Mime binaryDataMime = new Mime();

                binaryDataMime.Headers.Add("Content-Id", keyValuePair.Key);
                binaryDataMime.Headers.Add("Content-Type", "application/octet-stream");
                binaryDataMime.Headers.Add("content-transfer-encoding", "base64");

                binaryDataMime.SetBinaryMessage(keyValuePair.Value);
                mimeMultipart.AppendMime(binaryDataMime);
            }
        }

        #endregion

        #region IDisposable Members

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Disposes the specified the privately held resources.
        /// </summary>
        /// <param name="disposing">if set to <c>true</c> disposes resources.</param>
        private void Dispose(bool disposing)
        {
            // Check to see if Dispose has already been called.
            if (!this.disposed)
            {
                // If disposing equals true, dispose all managed 
                // and unmanaged resources.
                if (disposing)
                {
                    stream.Dispose();
                    tempStream.Dispose();
                    writer.Dispose();
                    reader.Dispose();
                }
            }
            disposed = true;
        }
        #endregion

    }

    //============================================================
    // MapDataHandlerAttribute Class
    //============================================================
    ///<summary>
    ///    Defines a SoapExtensionAttribute for our SOAP Extension so that
    ///    any Web Service method can be decorated as follows -
    ///        [MapDataHandlerAttribute]
    ///    to enable this SOAP Extension.
    ///</summary>
    //============================================================
    [AttributeUsage(AttributeTargets.Method)]
    public sealed class MapDataHandlerAttribute : SoapExtensionAttribute
    {
        private int priority;

        ///<summary>
        ///    Soap Extension Attribute ExtensionType property (override)
        ///</summary>
        public override Type ExtensionType
        {
            get { return typeof(MapDataHandlerExtension); }
        }

        ///<summary>
        ///    Soap Extension Attribute Priority property (override)
        ///</summary>
        public override int Priority
        {
            get { return priority; }
            set { priority = value; }
        }
    }

}
