using System;
using System.Web.Services.Protocols;

namespace Microsoft.eResearch.GenePattern.Server
{
    //============================================================
    // MapItemExtension Soap Extension
    //============================================================
    ///<summary>
    ///     The MapItemExtension class is a SOAP extension that allows
    ///     us to convert the Map returned in the repsonse to the "Map"
    ///     type expected by .Net.
    ///     
    ///     The resulting SOAP message with Map items mapped
    ///     is then substituted for the original unserialized request
    ///     during the ProcessMessage.
    ///     
    ///     The 'GetTask' web method is decorated with the attribute:
    ///     [MapItemAttribute]
    ///     to enable this SOAP extension.
    ///     
    ///</summary>
    public class MapItemExtension : StreamRewriteExtension
    {
        /// <summary>
        /// This method converts the Map items defined in the web service Response(from Axis that uses Javas Map) to the 
        /// Map type generated by .NET from the AXIS WSDL
        /// The format returned by AXIS is :
        /// <code>
        ///  <multiRef id="id2799" 
		///	  soapenc:root="0" 
		///	  soapenv:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" 
		///	  xsi:type="ns1283:Map" 
		///	  xmlns:ns1283="http://xml.apache.org/xmlsoap" 
		///	  xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/">
		///    <item >
		///	    <key xsi:type="soapenc:string">fileFormat</key>
		///	    <value xsi:type="soapenc:string">cls</value>
		///    </item>
		///    <item>
		///	    <key xsi:type="soapenc:string">MODE</key>
		///	    <value xsi:type="soapenc:string">IN</value>
		///    </item>
	    /// </multiRef>
        /// </summary>
        /// <param name="message">The Message Passed</param>
        /// </code>
        /// 
        /// The format returned after conversion is 
        /// <code>
        ///  <multiRef id="id2799" soapenc:root="0" soapenv:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xsi:type="ns1283:Map" xmlns:ns1283="http://xml.apache.org/xmlsoap" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/">
        ///  <item soapenc:arrayType="apacheNs:mapItem[7]" 
		///        xmlns:apacheNs="http://xml.apache.org/xml-soap">
        ///     <mapItem>
        ///         <key xsi:type="soapenc:string">fileFormat</key>
        ///         <value xsi:type="soapenc:string">cls</value>
        ///       </mapItem>
        ///       <mapItem>
        ///         <key xsi:type="soapenc:string">MODE</key>
        ///         <value xsi:type="soapenc:string">IN</value>
        ///       </mapItem>
        /// </item>
        /// </multiRef>
        /// </code>
        protected override void BeforeDeserialize(SoapMessage message)
        {
            JavaMapToDotNetMapConverter converter = new JavaMapToDotNetMapConverter();
            //newStream = converter.Convert(oldStream);
            converter.Convert(oldStream,newStream);
            newStream.SetLength(newStream.Position);
            newStream.Flush();
            newStream.Position = 0;
            converter = null;
        }

  }
        //============================================================
        // MapItemAttribute Class
        //============================================================
        ///<summary>
        ///    Defines a SoapExtensionAttribute for our SOAP Extension so that
        ///    any Web Service method can be decorated as follows -
        ///        [MapItemAttribute]
        ///    to enable this SOAP Extension.
        ///</summary>
        //============================================================
        [AttributeUsage(AttributeTargets.Method)]
        public sealed class MapItemAttribute : SoapExtensionAttribute
        {
            private int priority;

            ///<summary>
            ///    Soap Extension Attribute ExtensionType property (override)
            ///</summary>
            public override Type ExtensionType
            {
                get { return typeof(MapItemExtension); }
            }

            ///<summary>
            ///    Soap Extension Attribute Priority property (override)
            ///</summary>
            public override int Priority
            {
                get { return priority; }
                set { priority = value; }
            }
        }
    
}
