using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.IO;
using System.Xml;
using System.Configuration;
using Microsoft.eResearch.GenePattern.Common;
using System.Globalization;

namespace Microsoft.eResearch.GenePattern.Server
{
    //============================================================
    // PublishViaMIME Soap Extension
    //============================================================
    ///<summary>
    ///     The PublishViaMIME class is a SOAP extension that allows
    ///     the 'Publish' web method to be invoked either via a standard
    ///     SOAP envelope (posted to the web service via HTTP), or via
    ///     a SOAP envelope that is the root of a multi part MIME message
    ///     (also posted to the web service via HTTP)
    ///     
    ///     The SOAP extension determines the message type by examining
    ///     the Content-Type header of the message.
    ///     If the Content-Type is "Multipart/Related", the SOAP envelope
    ///     is extracted (by finding the message part referenced by the
    ///     "start=" value in the Content-Type header), and all content
    ///     references within the SOAP message (href:cid="xxxx") are 
    ///     resolved.
    ///     
    ///     The resulting SOAP message with resolved content references
    ///     is then substituted for the original unserialized request
    ///     during the ProcessMessage.BeforeDeserialize stage, the
    ///     Content-Type is reset to text/xml, and normal web method
    ///     processing occurs.
    ///     
    ///     The 'Publish' web method is decorated with the attribute:
    ///     [PublishViaMIME]
    ///     to enable this SOAP extension.
    ///     
    ///     This is a very simple (and not robust) implementation of one
    ///     approach described in "SOAP Messages with Attachments":
    ///     http://msdn.microsoft.com/library/en-us/dnsoapsp/html/soapattachspec.asp
    ///</summary>
    ///
    ///<remarks>
    ///<author name="Paula Paul"
    ///        company="Paul Software, Inc."
    ///        email="Paula@PaulSoftware.com"> 
    ///</author>
    ///
    ///<changelog>
    ///  <change     date= "07/05/2001"   name="Paula Paul">
    ///    <tracking build=""             issueIDs=""></tracking>
    ///    <item summary=  "created class"></item>  
    ///    <item summary=  "created Content-Type handling logic"></item>
    ///  </change>
    ///  <change     date= "07/11/2001"   name="Paula Paul">
    ///    <tracking build=""             issueIDs=""></tracking>
    ///    <item summary=  "first pass with document substitution"></item>                
    ///  </change>
    ///  <change     date= "07/16/2001"   name="Paula Paul">
    ///    <tracking build=""             issueIDs=""></tracking>
    ///    <item summary=  "finished comments and code cleanup"></item>
    ///  </change>
    ///</changelog>
    ///</remarks>
    //============================================================== 
    public class MultipartRewriteExtension : StreamRewriteExtension
    {
        // TODO implement Dipose


        string MIMEBoundary = "";
        string startContentID = "";
        string SOAPenvelope;
        private const string CONTENT_ID = "Content-Id: ";
        private const string ENVELOPE_END = "</soapenv:Envelope>";
        private const string CONTENT_START_TOKEN = "start=\"";
        private const string BOUNDARY_START_TOKEN = "boundary=";

        //============================================================
        // Override: ProcessMessage
        //============================================================
        ///<summary>
        ///  ProcessMessage gets control at for stages during the 
        ///  processing of a SOAP request:
        ///  1) BeforeDeserialize: as the serialized request is received
        ///     via HTTP - before is is deserialized for invocation of the
        ///     web method
        ///  2) AfterDeserialize - after the request is serialized but
        ///     before the web method is invoked
        ///  3) BeforeSerialize - before the web method results are 
        ///     serialized for return to the caller via HTTP
        ///  4) AfterSerialize - after the web method results are serialized
        ///     for return to the caller via HTTP
        ///     
        ///  For PublishViaMIME, we are only interested in the BeforeDeserialize
        ///  stage.  We'll examine the incoming stream; if it is a multi part
        ///  MIME message we'll extract the SOAP envelope from the message,
        ///  resolve the references to other MIME parts (content), and replace
        ///  the request stream with the resolved SOAP envelope.
        ///</summary>
        //============================================================
        protected override void BeforeDeserialize(SoapMessage message)
        {
            if (IsMultiPartMIME(message.ContentType))  // handle multi part MIME 
            {
                // we need to make a memstream since the current stream isn't seekable
                using (MemoryStream memStream = GetMemoryStream(oldStream))
                {
                    //extract the SOAP envelope from the MIME Message
                    SOAPenvelope = GetSOAPEnvelope(memStream);
                    //extract the encoded content from the MIME message and
                    // insert the encoded content into the SOAP message
                    ResolveContentRefs(memStream);
                    //replace the original (MIME) stream with the SOAP message
                    ReplaceMessageStream(newStream);
                    //make sure we reset the request's content type, since it
                    //is no longer in "Multipart/Related" form
                    message.ContentType = "text/xml";

                    oldStream.Dispose();
                }
            }
            else //if not a multi part message, pass the original stream through
            {
                reader = new StreamReader(oldStream);
                writer = new StreamWriter(newStream);
                writer.WriteLine(reader.ReadToEnd());
                writer.Flush();
            }
        }

        //============================================================
        // isMultiPartMIME
        //============================================================
        ///<summary>
        ///    Given the Content-Type string from the header of the request,
        ///    determine if the request is in multi part MIME format.
        ///    If so, save the pertinent information from the Content-Type
        ///    string in member variables:
        ///      MIMEBoundary = the string that separates one content item
        ///                     from another
        ///      startContentID = the Content-ID of the 'root' content item
        ///                     (the root content item must be the SOAP message)
        ///                     
        ///    Sample Content-Type header string:
        ///    Multipart/Related; boundary=MIME_boundary; 
        ///               type=text/xml; start=..soapenvelope.xml.. 
        ///</summary>
        ///<param name='contentType'>The Content-Type header of the message.
        ///</param>
        ///<returns>(boolean) true if the Content-Type header indicates this is
        ///    a "Multipart/Related" (MIME) format message.  False otherwise.
        ///</returns>
        //============================================================
        private bool IsMultiPartMIME(string contentType)
        {
            bool results = false;
            String[] contentTypeItems = contentType.Split(new Char[] { ';' });
            foreach (String item in contentTypeItems)
            {
                String trimItem = item.Trim();
                if (string.Compare(trimItem, "MULTIPART/RELATED", true, CultureInfo.InvariantCulture) == 0)
                {
                    results = true;
                }

                if (trimItem.IndexOf(BOUNDARY_START_TOKEN) >= 0) //MIMEBoundary member variable
                {
                    MIMEBoundary =
                      trimItem.Substring(trimItem.IndexOf(BOUNDARY_START_TOKEN) + 9).Trim();
                    // trim trailing quote
                    MIMEBoundary = MIMEBoundary.Substring(0, MIMEBoundary.Length - 1);
                }

                if (trimItem.IndexOf(CONTENT_START_TOKEN) >= 0)   //startContentID member variable
                {
                    startContentID = (CONTENT_ID +
                      trimItem.Substring(trimItem.IndexOf(CONTENT_START_TOKEN) + 7)).Trim();
                    // trim trailing quote
                    startContentID = startContentID.Substring(0, startContentID.Length - 1);
                }
            }
            return results;
        }

        //============================================================
        // getSOAPEnvelope
        //============================================================
        ///<summary>
        ///    Given the HTTP request stream in multipart/related format,
        ///    extract the root content item, which contains the SOAP envelope
        ///    (with unresolved content references).   
        ///</summary>
        ///<param name='input'>The request stream.
        ///</param>
        ///<returns>(string) A string containing the SOAP envelope text, including
        ///    any unresolved content references to other parts of the multi part
        ///    message.
        ///</returns>
        //============================================================
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Reliability", "CA2000:DisposeObjectsBeforeLosingScope")]
        private string GetSOAPEnvelope(Stream input)
        {
            StringBuilder envelope = new StringBuilder();

            StreamReader sr = new StreamReader(input, System.Text.Encoding.Default, true);
            input.Position = 0;

            // Read through the (non null) lines of the incoming message
            // looking for the ContentID header equal to the start content id
            // the start content id identifies the soap envelope content item
            // NOTE - this logic assumes that the Content-ID header occurs right
            // before the encoded content.
            for (String Line = sr.ReadLine(); Line != null; Line = sr.ReadLine())
            {
                if (Line.IndexOf(startContentID) >= 0) //found the SOAP envelope
                {
                    bool foundEndOfEnvelope = false;
                    String envLine = sr.ReadLine();
                    while (envLine != null && !foundEndOfEnvelope)
                    {
                        envelope.Append(envLine); //TODO avoid string concat

                        if (envLine.IndexOf(ENVELOPE_END) >= 0) // todo
                        {
                            foundEndOfEnvelope = true;
                        }
                        envLine = sr.ReadLine();
                    }
                    break;
                }
            }

            sr.DiscardBufferedData();
            return envelope.ToString();
        }

        //============================================================
        // resolveContentRefs
        //============================================================
        ///<summary>
        ///    Given the HTTP request stream in multipart/related format,
        ///    locate all the content items (other than the root
        ///    SOAP envelope), and replace content references in the SOAP
        ///    envelope with the matching content items.
        ///    NOTE - this logic assumes that a Content-ID header
        ///    immediately precedes the encoded content!
        ///</summary>
        ///<param name='MIMEstream'>The request stream.  Also depends on the 
        ///    SOAPEnvelope member variable, which is populated at this point.
        ///</param>
        ///<returns>(void) 
        ///    Thows exceptions for content items without corresponding
        ///    references in the SOAP envelope, and for references in the SOAP
        ///    envelope without corresponding content items in the stream.
        ///</returns>
        //============================================================
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
        private void ResolveContentRefs(MemoryStream MIMEstream)
        {
            string contID;
            MIMEstream.Position = 0;
            BinaryAndStringReader br = new BinaryAndStringReader(MIMEstream);
            List<byte> binaryData = new List<byte>();
            // the delimiter separates binary content from the package
            const string delimiter = "\n------=";
            string Line = null;

            // Read through the request stream, looking for the Content-ID headers
            // ignore the content of the root item (the soap envelope content)
            for (Line = br.ReadLine(); (Line != null); Line = br.ReadLine())
            {
                // process every Content-ID *other* than the start Content-ID
                if ((Line.IndexOf(CONTENT_ID) >= 0) &&
                    (Line.IndexOf(startContentID) < 0))
                {
                    //save the Content-ID value, and the base64 encoded content
                    contID = Line.Substring(Line.IndexOf(CONTENT_ID) + 12).Trim();

                    //remove the starting and ending "<" and ">"
                    if (contID.StartsWith("<")) contID = contID.Substring(1);
                    if (contID.EndsWith(">")) contID = contID.Substring(0, contID.Length - 1);
                    //get the encoded content immediately following the Content-ID header
                    //(until the next MIME Boundary is reached)

                    br.ReadLine();

                    //extract binary data until we hit the ending delimiter

                    while (!ByteListEndWith(binaryData, delimiter) &&
                           br.BaseStream.Position <= br.BaseStream.Length - 1)
                    {
                        binaryData.Add(br.ReadByte());
                    }

                    // remove trailing "------=" if we didn't hit the end of the stream
                    if (br.BaseStream.Position <= br.BaseStream.Length - 1)
                    {
                        binaryData.RemoveRange(binaryData.Count - delimiter.Length, delimiter.Length);
                    }

                    // remove trailing carriage return if any
                    char lastChar = (char)binaryData[binaryData.Count - 1];
                    if (lastChar == '\r')
                    {
                        binaryData.RemoveRange(binaryData.Count - 1, 1);
                    }

                    InsertContent(contID, binaryData);

                    // this shouldn't need do anything, it should go out of scope!
                    binaryData.Clear();

                }
            }

            binaryData = null;

            // now that we've processed all the MIME 'attachments', look through
            // the envelope and make sure there are no remaining unresolved href's
            if (SOAPenvelope.ToString().IndexOf("href=\"cid:") >= 0)
            {
                throw new Exception("Envelope contains unresolved content references");
            }
            MIMEstream.Position = 0;  //reset the stream position
        }

        //============================================================
        // insertContent
        //============================================================
        /// <summary>
        /// Given the contentID and encoded content from an item in
        /// the multi part request stream, update the SOAPEnvelope.
        /// Insert the encoded content at the point of the matching
        /// reference attribute (href="cid:...") in the SOAP envelope.
        /// Reference attribute example:
        /// <Data href="cid:MIMEtesttext.txt@paulsoftware.com"></Data>
        /// </summary>
        /// <param name="contentID">The content ID of the item from the request
        /// stream.</param>
        /// <param name="binaryData">The data.</param>
        //============================================================
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2200:RethrowToPreserveStackDetails")]
        private void InsertContent(string contentID, List<byte> binaryData)
        {
            Trace.WriteLine("Beginning to rewrite SOAP Response");
            try
            {
                //string envelope = 
                int indexInsideNodeToReplace = SOAPenvelope.IndexOf("href=\"cid:" + contentID);
                Debug.Assert(indexInsideNodeToReplace < SOAPenvelope.Length);

                if (string.IsNullOrEmpty(SOAPenvelope))
                    return;

                int nodeBegin = SOAPenvelope.Substring(0, indexInsideNodeToReplace).LastIndexOf("<");
                int nodeEnd = SOAPenvelope.IndexOf(">", indexInsideNodeToReplace);
                int nodeClosingTagBegin = SOAPenvelope.IndexOf("<", nodeEnd);

                // cut up the envelope into [before][node][nodecontent(removed)][after]
                string textBefore = SOAPenvelope.Substring(0, nodeBegin);
                string node = SOAPenvelope.Substring(nodeBegin, nodeEnd - nodeBegin + 1);
                string textAfter = SOAPenvelope.Substring(nodeClosingTagBegin);

                // save some memory here
                SOAPenvelope = "";
                Trace.WriteLine("Successfully separated Soap envelope");


                // change the xsi:type and href attributes of the node
                XmlDocument nodeDoc = new XmlDocument();
                string nodename = node.Substring(1, node.IndexOf(" "));
                bool addCloseTag = !node.EndsWith("/>");
                string closeTag = "</" + nodename + ">";
                if (addCloseTag)
                    node += closeTag;
                nodeDoc.LoadXml(node.Replace("xsi:type", "xsitype"));

                nodeDoc.DocumentElement.Attributes["xsitype"].Value = "xsd:string";
                // remove the href now that we used it
                nodeDoc.DocumentElement.Attributes["href"].Value = "";
                nodeDoc.DocumentElement.Attributes.Remove(nodeDoc.DocumentElement.Attributes["href"]);
                node = nodeDoc.InnerXml.Replace("xsitype", "xsi:type");

                // strip closing tag
                if (addCloseTag)
                {
                    node = node.Substring(0, node.Length - closeTag.Length);
                }
                else
                {
                    // strip ending "/>" since we have content in the node now
                    node = node.Substring(0, node.Length - 2) + ">";
                }
                string contentToInsert = CacheToDisk(binaryData);

                Trace.WriteLine("Successfully altered XML, about to allocate a string builder of length " + (textBefore.Length + node.Length + textAfter.Length + contentToInsert.Length));
                StringBuilder rebuilder =
                   new StringBuilder(textBefore.Length + node.Length + textAfter.Length + contentToInsert.Length);
                rebuilder.Append(textBefore);
                rebuilder.Append(node);
                rebuilder.Append(contentToInsert);
                if (!addCloseTag)
                {
                    rebuilder.Append(closeTag);
                }
                rebuilder.Append(textAfter);

                SOAPenvelope = rebuilder.ToString();
                
                nodeDoc = null;
                rebuilder = null;
            }
            catch (Exception e)
            {
                Trace.WriteLine("Exception rewriting SOAP Response, probably fatal: " + e.Message + "\n\n" + e.StackTrace);
                throw e;
            }
            Trace.WriteLine("Done to rewriting SOAP Response");

        }

        /// <summary>
        /// Given a base64 string, if it is above a given length, write it to a temp file and return the filename
        /// else, don't change it
        /// </summary>
        /// <param name="bytes"></param>
        /// <returns></returns>
        private static string CacheToDisk(List<byte> bytes)
        {
            Guid tempFileGuid = Guid.NewGuid();
            string tempFilePath = DefaultSetting.Path + tempFileGuid.ToString() + ".tmp";
           
            using (FileStream fs = new FileStream(tempFilePath, FileMode.Create))
            {
                fs.Write(bytes.ToArray(), 0, bytes.Count);
            }
            Trace.WriteLine("Stored file at " + tempFilePath);
            return tempFilePath;
        }

        //============================================================
        // replaceMessageStream
        //============================================================
        ///<summary>
        ///    replaces a stream with a new stream containing the text found
        ///    in the SOAPEnvelope member variable.
        ///</summary>
        ///<param name='target'>The target stream.
        ///</param>
        ///<returns>(void) 
        ///</returns>
        //============================================================
        private void ReplaceMessageStream(Stream target)
        {
            target.Position = 0;

            // write the SOAPenvelope member variable contents to the target stream
            writer = new StreamWriter(target);
            writer.WriteLine(SOAPenvelope);
            writer.Flush();
            target.Flush();
            
            // tidy up the target stream
            target.SetLength(target.Position);
            target.Position = 0;
        }

        /// <summary>
        /// Searches the document recursively until it finds a node with the given attribute value
        /// </summary>
        /// <param name="doc"></param>
        /// <param name="attributeXPath"></param>
        /// <param name="contentID"></param>
        /// <returns></returns>
        static XmlNode GetContentNode(XmlNode doc, string attributeXPath, string contentID)
        {
            // there's probably a way to do this with xpath.  but this works.
            foreach (XmlNode node in doc.ChildNodes)
            {
                XmlNode href = node.SelectSingleNode(attributeXPath);
                if (href != null && href.Value == (contentID))
                {
                    return node;
                }
                else
                {
                    XmlNode content = GetContentNode(node, attributeXPath, contentID);
                    if (content != null)
                    {
                        return content;
                    }
                }
            }
            return null;
        }

        private static bool ByteListEndWith(List<byte> bytes, string match)
        {
            if (bytes.Count < match.Length)
            {
                return false;
            }
            else
            {
                int matchCount = 0;
                bool matched = true;

                while (matched && matchCount < match.Length)
                {
                    int byteIndex = bytes.Count - matchCount - 1;
                    int stringIndex = match.Length - matchCount - 1;
                    if (bytes[byteIndex] == (byte)match.ToCharArray()[stringIndex])
                    {
                        matchCount++;
                    }
                    else
                    {
                        matched = false;
                        break;
                    }
                }
                return matched;
            }
        }
    }

    //============================================================
    // PublishViaMIMEAttribute Class
    //============================================================
    ///<summary>
    ///    Defines a SoapExtensionAttribute for our SOAP Extension so that
    ///    any Web Service method can be decorated as follows -
    ///        [PublishViaMIMEAttribute]
    ///    to enable this SOAP Extension.
    ///</summary>
    //============================================================
    [AttributeUsage(AttributeTargets.Method)]
    public sealed class MultipartRewriteExtensionAttribute : SoapExtensionAttribute
    {
        private int priority;

        ///<summary>
        ///    Soap Extension Attribute ExtensionType property (override)
        ///</summary>
        public override Type ExtensionType
        {
            get { return typeof(MultipartRewriteExtension); }
        }

        ///<summary>
        ///    Soap Extension Attribute Priority property (override)
        ///</summary>
        public override int Priority
        {
            get { return priority; }
            set { priority = value; }
        }
    }
}
