using System;
using System.Web.Services.Protocols;
using System.IO;
using Microsoft.eResearch.GenePattern.Common;

namespace Microsoft.eResearch.GenePattern.Server
{
    //============================================================
    // PublishViaMIME Soap Extension
    //============================================================
    ///<summary>
    ///     The PublishViaMIME class is a SOAP extension that allows
    ///     the 'Publish' web method to be invoked either via a standard
    ///     SOAP envelope (posted to the web service via HTTP), or via
    ///     a SOAP envelope that is the root of a multi part MIME message
    ///     (also posted to the web service via HTTP)
    ///     
    ///     The SOAP extension determines the message type by examining
    ///     the Content-Type header of the message.
    ///     If the Content-Type is "Multipart/Related", the SOAP envelope
    ///     is extracted (by finding the message part referenced by the
    ///     "start=" value in the Content-Type header), and all content
    ///     references within the SOAP message (href:cid="xxxx") are 
    ///     resolved.
    ///     
    ///     The resulting SOAP message with resolved content references
    ///     is then substituted for the original unserialized request
    ///     during the ProcessMessage.BeforeDeserialize stage, the
    ///     Content-Type is reset to text/xml, and normal web method
    ///     processing occurs.
    ///     
    ///     The 'Publish' web method is decorated with the attribute:
    ///     [PublishViaMIME]
    ///     to enable this SOAP extension.
    ///     
    ///     This is a very simple (and not robust) implementation of one
    ///     approach described in "SOAP Messages with Attachments":
    ///     http://msdn.microsoft.com/library/en-us/dnsoapsp/html/soapattachspec.asp
    ///</summary>
    ///
    ///<remarks>
    ///<author name="Paula Paul"
    ///        company="Paul Software, Inc."
    ///        email="Paula@PaulSoftware.com"> 
    ///</author>
    ///
    ///<changelog>
    ///  <change     date= "07/05/2001"   name="Paula Paul">
    ///    <tracking build=""             issueIDs=""></tracking>
    ///    <item summary=  "created class"></item>  
    ///    <item summary=  "created Content-Type handling logic"></item>
    ///  </change>
    ///  <change     date= "07/11/2001"   name="Paula Paul">
    ///    <tracking build=""             issueIDs=""></tracking>
    ///    <item summary=  "first pass with document substitution"></item>                
    ///  </change>
    ///  <change     date= "07/16/2001"   name="Paula Paul">
    ///    <tracking build=""             issueIDs=""></tracking>
    ///    <item summary=  "finished comments and code cleanup"></item>
    ///  </change>
    ///</changelog>
    ///</remarks>
    //============================================================== 
    public abstract class StreamRewriteExtension : SoapExtension, IDisposable
    {
        private bool disposed;

        /// <summary>
        ///  reference to original stream
        /// </summary>
        protected Stream oldStream;
        /// <summary>
        ///  reference to new stream
        /// </summary>
        protected Stream newStream;
        /// <summary>
        /// reader used to copy streams
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
        protected TextReader reader;     // reader and writer are used to copy streams
        /// <summary>
        /// writer used to copy streams
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1051:DoNotDeclareVisibleInstanceFields")]
        protected TextWriter writer;

        /// <summary>
        /// When overridden in a derived class, allows a SOAP extension to initialize data specific to an XML Web service method using an attribute applied to the XML Web service method at a one time performance cost.
        /// </summary>
        /// <param name="methodInfo">A <see cref="T:System.Web.Services.Protocols.LogicalMethodInfo"></see> representing the specific function prototype for the XML Web service method to which the SOAP extension is applied.</param>
        /// <param name="attribute">The <see cref="T:System.Web.Services.Protocols.SoapExtensionAttribute"></see> applied to the XML Web service method.</param>
        /// <returns>
        /// The <see cref="T:System.Object"></see> that the SOAP extension initializes for caching.
        /// </returns>
        public override object GetInitializer(LogicalMethodInfo methodInfo,
                                              SoapExtensionAttribute attribute)
        {
            return null;
        }


        /// <summary>
        /// When overridden in a derived class, allows a SOAP extension to initialize data specific to a class implementing an XML Web service at a one time performance cost.
        /// </summary>
        /// <param name="serviceType">The type of the class implementing the XML Web service to which the SOAP extension is applied.</param>
        /// <returns>
        /// The <see cref="T:System.Object"></see> that the SOAP extension initializes for caching.
        /// </returns>
        public override object GetInitializer(System.Type serviceType)
        {
            return null;
        }

        /// <summary>
        /// When overridden in a derived class, allows a SOAP extension to initialize itself using the data cached in the <see cref="M:System.Web.Services.Protocols.SoapExtension.GetInitializer(System.Web.Services.Protocols.LogicalMethodInfo,System.Web.Services.Protocols.SoapExtensionAttribute)"></see> method.
        /// </summary>
        /// <param name="initializer">The <see cref="T:System.Object"></see> returned from <see cref="M:System.Web.Services.Protocols.SoapExtension.GetInitializer(System.Web.Services.Protocols.LogicalMethodInfo,System.Web.Services.Protocols.SoapExtensionAttribute)"></see> cached by ASP.NET.</param>
        public override void Initialize(object initializer)
        {
        }

        //============================================================
        // Override: ChainStream
        //============================================================
        ///<summary>
        ///  ChainStream allows us to save a reference to the Stream representing
        ///  the SOAP request or SOAP response so we can manipulate the request
        ///  (or response) stream.
        ///  We need to save the reference to the stream during ChainStream,
        ///  because the stream is not writable during subsequent calls to
        ///  ProcessMessage.
        /// 
        /// The flow of control is:
        /// 1) ChainStream 
        ///      provides access to the incoming serialized stream buffer
        /// 2) ProcessMessage.BeforeDeserialize 
        ///      replace the original MIME input with the SOAP message extracted
        ///      from the multi part message, with resolved content references
        /// 3) ProcessMessage.AfterDeserialize
        ///      not used
        /// 4) Web Method is executed
        /// 5) ChainStream
        ///      provides access to the outgoing serialized stream buffer
        /// 6) ProcessMessage.BeforeSerialize
        ///      not used
        /// 7) ProcessMessage.AfterSerialize
        ///      not used
        ///</summary>
        //============================================================
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
        public override Stream ChainStream(Stream stream)
         {
            if (stream == null)
                throw new ArgumentNullException("stream");

            //save a reference to the input stream and the return stream as local 
            //member variables.  The return stream will be modified in the 
            //ProcessMessage.BeforeDeserialize stage to replace the original input
            //stream with a modified SOAP message stream.
            switch (stream.GetType().ToString())
            {
                case "System.Net.SyncMemoryStream": // we want to modify the input 
                    //using (FileStream fileStream = new FileStream(@"c:\temp\syncstream.txt", FileMode.Create))
                    //{
                    //    long position = stream.Position;
                    //    byte[] bytes = ((MemoryStream)stream).ToArray();
                    //    fileStream.Write(bytes, 0, bytes.Length);
                    //}
                    oldStream = stream;
                    newStream = new MemoryStream();
                    return newStream;
                case "System.Net.ConnectStream":
                    oldStream = stream;
                    newStream = new MemoryStream();
                    return newStream;
                // and avoid messing with the output
                case "System.Web.Services.Protocols.SoapExtensionStream":
                    return stream;
                default:
                    throw new Exception(Resource1.UNKNOWN_STREAM_TYPE); //TODO put text in resource file
            }
        }

        //============================================================
        // Override: ProcessMessage
        //============================================================
        ///<summary>
        ///  ProcessMessage gets control at for stages during the 
        ///  processing of a SOAP request:
        ///  1) BeforeDeserialize: as the serialized request is received
        ///     via HTTP - before is is deserialized for invocation of the
        ///     web method
        ///  2) AfterDeserialize - after the request is serialized but
        ///     before the web method is invoked
        ///  3) BeforeSerialize - before the web method results are 
        ///     serialized for return to the caller via HTTP
        ///  4) AfterSerialize - after the web method results are serialized
        ///     for return to the caller via HTTP
        ///     
        ///</summary>
        //============================================================
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1062:ValidateArgumentsOfPublicMethods"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
        public override void ProcessMessage(SoapMessage message)
        {
            switch (message.Stage)
            {
                case SoapMessageStage.BeforeDeserialize:
                    // pass streams straight through in base class.  override this method for more complex rewriting behavior.
                    BeforeDeserialize(message);
                    break;

                case SoapMessageStage.AfterDeserialize:
                    break;

                case SoapMessageStage.BeforeSerialize:
                    break;

                case SoapMessageStage.AfterSerialize:
                    AfterSerialize(message);
                    break;

                default:
                    throw new Exception(Resource1.INVALID_STAGE_MESSAGEPROCESSING);
            }
        }



        /// <summary>
        /// Handles the needed processing in the AfterSerialize stage
        /// </summary>
        /// <param name="message">The message.</param>
        protected virtual void AfterSerialize(SoapMessage message)
        {
            //do nothing by default
        }


        /// <summary>
        /// This method performs the simplest possible operation for the before deserialize stage:
        /// copying from the old stream directly to the new stream without any rewriting
        /// </summary>
        protected virtual void BeforeDeserialize(SoapMessage message)
        {
            newStream.Position = 0;
            reader = new StreamReader(oldStream);
            writer = new StreamWriter(newStream);
            
            string data = reader.ReadToEnd();
           
            writer.WriteLine(data);
            writer.Flush();
            
            newStream.Flush();
            newStream.SetLength(newStream.Position);
            newStream.Position = 0;
        }

        /// <summary>
        /// Gets a memory stream with the contents of the input stream
        /// </summary>
        /// <param name="input">The input.</param>
        /// <returns>a memory stream with the contents of the input stream</returns>
        protected static MemoryStream GetMemoryStream(Stream input)
        {
            return StreamHelper.GetMemoryStream(input);
        }

        #region IDisposable Members

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Disposes the specified the privately held resources.
        /// </summary>
        /// <param name="disposing">if set to <c>true</c> disposes resources.</param>
        private void Dispose(bool disposing)
        {
            // Check to see if Dispose has already been called.
            if (!this.disposed)
            {
                // If disposing equals true, dispose all managed 
                // and unmanaged resources.
                if (disposing)
                {
                    oldStream.Dispose();
                    newStream.Dispose();
                    writer.Dispose();
                    reader.Dispose();
                }
            }
            disposed = true;
        }

        #endregion
    }

}
