using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Net;
using System.Text;
using System.Threading;
using System.Web.Services.Protocols;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.GenePattern.Server.Admin;
using Microsoft.eResearch.GenePattern.Server.Analysis;
using Microsoft.eResearch.GenePattern.Server.Properties;
using Microsoft.eResearch.GenePattern.Server.TaskIntegrator;
using ParameterInfo = Microsoft.eResearch.GenePattern.Server.Analysis.ParameterInfo;
using Task = Microsoft.eResearch.GenePattern.Server.TaskIntegrator;
using TaskInfo = Microsoft.eResearch.GenePattern.Server.Admin.TaskInfo;
using System.Diagnostics;
using DataHandler = Microsoft.eResearch.GenePattern.Server.TaskIntegrator.DataHandler;
using Map = Microsoft.eResearch.GenePattern.Server.Analysis.Map;
using mapItem = Microsoft.eResearch.GenePattern.Server.Analysis.mapItem;

namespace Microsoft.eResearch.GenePattern.Server
{
    /// <summary>
    /// The services from GenePatter server accessible in the GenePattern package
    /// </summary>
    public class GenePatternServer : IGenePatternServer, IDisposable
    {
        #region Fields

        private bool disposed;

        private AdminServiceService adminService;
        private AnalysisService analysisService;
        private TaskIntegratorService taskService;

        private string adminServicePath;
        private string analysisServicePath;
        private string taskServicePath;

        private Connection connection;
        private const string CONNECTION_SUCCESS = "OK";
        private const string NULL_URL = "http://0.0.0.0";
        private const int ACCESS_PUBLIC = 1;
        private const int ACCESS_PRIVATE = 2;
        int jodIdSwitch;

        #endregion

        #region Contructors

        /// <summary>
        /// Exposes methods of the GenePattern server
        /// </summary>
        public GenePatternServer()
        {
            this.adminService = new AdminServiceService();
            this.analysisService = new AnalysisService();
            this.taskService = new TaskIntegratorService();
            this.adminServicePath = Settings.Default.AdminWebServicePath;
            this.analysisServicePath = Settings.Default.AnalysisWebServicePath;
            this.taskServicePath = Settings.Default.TaskWebServicePath;

            SetUpWebServices();
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Gets the top level job from job info.
        /// </summary>
        /// <param name="jobInfo">The job info.</param>
        /// <returns></returns>
        public Job GetTopLevelJobFromJobInfo(JobInfo jobInfo)
        {
            return GetTopLevelJobFromJobInfo(jobInfo, new Dictionary<string, Pipeline>());
        }

        /// <summary>
        /// Gets the top level job from job info.
        /// </summary>
        /// <param name="jobInfo">The job info.</param>
        /// <param name="storedPipelines">The stored pipelines.</param>
        /// <returns></returns>
        public Job GetTopLevelJobFromJobInfo(JobInfo jobInfo, Dictionary<string, Pipeline> storedPipelines)
        {
            bool stopPointIsDefined = false;

            Job job = new Job();
            job.Id = jobInfo.jobNumber;
            job.PipelineLsid = jobInfo.taskLSID;
            job.PipelineName = jobInfo.taskName;
            job.Status = ConvertToJobStatus(jobInfo.status);

            job.DateSubmitted = jobInfo.dateSubmitted.HasValue ? jobInfo.dateSubmitted.Value : new DateTime();
            job.DateCompleted = jobInfo.dateCompleted.HasValue ? jobInfo.dateCompleted.Value : new DateTime();

            if (jobInfo.parameterInfoArray != null)
            {
                int outputCount = 1;
                foreach (ParameterInfo parameterInfo in jobInfo.parameterInfoArray)
                {
                    if (parameterInfo.outputFile)
                    {
                        Debug.Assert(parameterInfo.name.IndexOf('/') == -1);
                        ShallowJobResult result = new ShallowJobResult();
                        result.Filename = parameterInfo.name;
                        result.SourceJobId = job.Id;
                        result.SourceStep = -1;
                        result.SourceIndex = outputCount;
                        outputCount++;
                        job.ShallowResults.Add(result);
                    }
                    else if (parameterInfo.name == Properties.Resources.STOP_POINT_PARAMETER_NAME)
                    {
                        int vessel = 0;
                        if (int.TryParse(parameterInfo.value, out vessel))
                        {
                            job.StopPoint = vessel - 1;
                            stopPointIsDefined = true;
                        }
                    }
                    else
                    {
                        JobParameter param = ParameterHelper.GetJobParameter(parameterInfo);
                        try
                        {
                            if (parameterInfo.inputFile)
                            {
                                string uri = parameterInfo.value;
                                // flip slashes in case it's a linux machine!
                                //uri = uri.Replace("/", "\\");
                                if (uri.StartsWith("/"))
                                {
                                    uri = "file:/" + uri;
                                }

                                if (uri.Length != 0)
                                {
                                    Uri path = new Uri(uri);
                                    if (path.IsFile)
                                    {
                                        InternalFile internalFile = new InternalFile();
                                        internalFile.FullPath = parameterInfo.value;
                                        internalFile.Filename = Path.GetFileName(parameterInfo.value);
                                        param.ParamValue = internalFile;
                                    }
                                }
                            }
                        }
                        catch (Exception)
                        {
                            throw;
                        }
                        job.Parameters.Add(param);
                    }
                }
            }

            // if stop point was not set by parameter, set it to be the last step
            Pipeline pipeline = null;
            if (storedPipelines.ContainsKey(job.PipelineLsid))
            {
                pipeline = storedPipelines[job.PipelineLsid];
            }
            else
            {
                pipeline = GetPipelineByLsid(job.PipelineLsid);
                storedPipelines.Add(job.PipelineLsid, pipeline);
            }
            if (pipeline == null)
            {
                return null;
                // if the pipeline behind the job is gone, don't return the job
            }
            else
            {
                if (!stopPointIsDefined)
                {
                    int noSteps = pipeline.Steps.Count;
                    if (noSteps > 0)
                    {
                        job.StopPoint = noSteps - 1;
                    }
                }
                CheckJobParameters(job, pipeline);
                return job;
            }


        }

        /// <summary>
        /// Checks the job parameters.
        /// </summary>
        /// <param name="job">The job.</param>
        /// <param name="pipeline">The pipeline.</param>
        private static void CheckJobParameters(Job job, Pipeline pipeline)
        {
            //Check to make sure that "parameterInfo.inputFile" is set properly for Job Parameter
            // If the file specified in the Job Parameter is a URI, by default the 
            // parameterInfo.InputFile returns FALSE instead of True and so ParamValue of Job is set to a string
            // We neeed to change the paramValue to be of type "InputFile"
            foreach (PipelineParameter pipelineParameter in pipeline.Parameters)
            {
                if (pipelineParameter.Type == ParameterType.InputFile)
                {
                    JobParameter jobParameter = job.GetParameter(pipelineParameter.Name);
                    if (jobParameter != null)
                    {
                        InternalFile jobParameterInfoFile = jobParameter.ParamValue as InternalFile;
                        if (jobParameterInfoFile == null)
                        {
                            string paramValue = Convert.ToString(jobParameter.ParamValue);
                            if (paramValue.Length == 0)
                            {
                                jobParameter.ParamValue = string.Empty;
                            }
                            else
                            {
                                SerializableUri uri = new SerializableUri(paramValue);
                                jobParameter.ParamValue = uri;
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Tests the connection.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="testCredentials">if set to <c>true</c> [test credentials].</param>
        /// <param name="message">The message.</param>
        /// <returns></returns>
        public static bool TestConnection(Connection conn, bool testCredentials, out string message)
        {
            message = null;
            bool success = false;
            try
            {
                IGenePatternServer genePatternServer = new GenePatternServer();
                genePatternServer.Connection = conn;
                success = genePatternServer.TestConnection();
                conn.ServerVersion = genePatternServer.GetServiceInfo.Version;
                if (testCredentials)
                    success = success && genePatternServer.Ping();
            }
            catch (Exception e)
            {
                success = false;
                message = e.Message;
            }
            return success;
        }

        #endregion

        #region IGenePatternServer Members

        /// <summary>
        /// Gets the name of the stop point parameter.
        /// </summary>
        /// <value>The name of the stop point parameter.</value>
        public string StopPointParameterName
        {
            get { return Properties.Resources.STOP_POINT_PARAMETER_NAME; }
        }

        /// <summary>
        /// Gets or sets the connection.
        /// </summary>
        /// <value>The connection.</value>
        public Connection Connection
        {
            get
            {
                return connection;
            }
            set
            {
                this.connection = value;
                SetUpWebServices();
            }
        }

        /// <summary>
        /// Gets the job by id.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <returns>the matching <c>Job</c></returns>
        public Job GetJobById(int jobId)
        {
            VerifyConnection();
            Job job = null;
            try
            {
                JobInfo jobInfo = analysisService.getJob(jobId);

                if (jobInfo == null)
                {
                    Trace.WriteLine("No job found for job id = " + jobId, TraceLevel.Info.ToString());
                    return job;
                }

                job = GetTopLevelJobFromJobInfo(jobInfo);
                if (job == null)
                {
                    throw new Exception("Pipeline or Job no longer exists on server."); 
                }

                int[] childJobs = analysisService.getChildren(jobId);

                int jobCount = 1;
                foreach (int childJobId in childJobs)
                {
                    JobInfo childJob = analysisService.getJob(childJobId);

                    if (childJob != null && childJob.parameterInfoArray != null)
                    {
                        int outputIndex = 1;
                        foreach (ParameterInfo parameterInfo in childJob.parameterInfoArray)
                        {
                            if (parameterInfo.outputFile)
                            {
                                Debug.Assert(parameterInfo.name.IndexOf('/') == -1);
                                ShallowJobResult result = new ShallowJobResult();
                                result.Filename = parameterInfo.name;
                                result.SourceJobId = childJob.jobNumber;
                                result.SourceStep = jobCount;
                                result.SourceIndex = outputIndex;
                                job.ShallowResults.Add(result);
                                outputIndex++;
                            }
                        }
                    }
                    jobCount++;
                }
            }
            catch (WebException e)
            {
                Trace.WriteLine(e, TraceLevel.Error.ToString());
                throw new ConnectionFailedException(Resources.UNABLE_TO_CONNECT, e);
            }

            return job;
        }

        /// <summary>
        /// Gets the pipeline by lsid.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <returns>
        /// the matching <c>Pipeline</c> or null if no match
        /// </returns>
        public Pipeline GetPipelineByLsid(string lsid)
        {
            if (lsid == null)
                throw new ArgumentNullException("lsid");

            VerifyConnection();

            Pipeline result = null;

            try
            {
                TaskInfo taskInfo = adminService.getTask(lsid);
                if (taskInfo == null)
                {
                    Trace.WriteLine("Task info was null for lsid" + lsid);
                    return null;
                }

                if (taskInfo.pipeline)
                {
                    result = new Pipeline();
                    result.Name = taskInfo.name;
                    result.Lsid = taskInfo.lsid;

                    string[] documentationFilenames = GetDocumentation(lsid);
                    if (documentationFilenames != null)
                    {
                        result.DocumentationFilenames.AddRange(documentationFilenames);
                    }

                    Dictionary<string, string> attributes = MapHelper.GetDictionaryFromMap(taskInfo.attributes);

                    if (attributes.ContainsKey("serializedModel"))
                    {
                        string serializedModel = attributes["serializedModel"];
                        PipelineStepHelper.ExtractFromModel(serializedModel, result, this);
                    }

                    if (taskInfo.parameterInfoArray != null)
                    {
                        foreach (Admin.ParameterInfo parameterInfo in taskInfo.parameterInfoArray)
                        {
                            if (!parameterInfo.outputFile)
                            {
                                result.Parameters.Add(ParameterHelper.GetPipelineParameter(parameterInfo));
                            }
                        }
                    }
                }
            }
            catch (WebException e)
            {
                Trace.WriteLine(e, TraceLevel.Error.ToString());
                throw new ConnectionFailedException(Resources.UNABLE_TO_CONNECT, e);
            }

            return result;
        }

        /// <summary>
        /// Runs the pipeline.
        /// </summary>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="parameters">The parameter list.</param>
        /// <param name="stopPoint">The stop point.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.Int32.ToString")]
        public int RunPipeline(Pipeline pipeline, IList<JobParameter> parameters, int stopPoint)
        {
            if (pipeline == null)
                throw new ArgumentNullException("pipeline");

            if (parameters == null)
                throw new ArgumentNullException("parameters");

            VerifyConnection();
            Map files;
            List<ParameterInfo> parameterInfoList = null;
            ParameterInfo stopPointparameterInfo;
            ParameterInfo[] parameterInfos;
            JobInfo jobInfo;

            try
            {
                Admin.TaskInfo taskInfo = adminService.getTask(pipeline.Lsid);
                Dictionary<string, JobParameter> jobParametersDictionary =
                                    ParameterInfoHelper.CreateJobParametersMapping(parameters);

                parameterInfoList =
                        ParameterInfoHelper.CreateParameterInfoList(jobParametersDictionary, pipeline, out files);
                // HACK ALERT!!  GP Server counts pipelines from 1, not 0, so add 1 to stop point!
                stopPointparameterInfo =
                    ParameterInfoHelper.CreateStopPointParamterInfo("PIPELINE_ARG.StopAfterTask", (stopPoint + 1).ToString());
                parameterInfoList.Add(stopPointparameterInfo);
                parameterInfos = parameterInfoList.ToArray();
                jobInfo = analysisService.submitJob(taskInfo.ID, parameterInfos, files);

                int jobNumber = jobInfo.jobNumber;
                Trace.WriteLine("Job submitted for rerun with job number " + jobNumber);
                return jobNumber;

            }
            catch (Exception e)
            {
                if (parameterInfoList == null)
                    throw new ApplicationException("Rerun failed. One or more input parameters is invalid.");

                Trace.WriteLine("Exception: " + e, TraceLevel.Error.ToString());
                throw;
            }
            finally
            {
                files = null;
                parameterInfoList = null;
                stopPointparameterInfo = null;
                parameterInfos = null;
                GC.Collect();
            }
        }

        /// <summary>
        /// Checks the job status.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <returns>
        /// a <c>JobStatus</c> if job existis otherwise null
        /// </returns>
        public JobStatus CheckJobStatus(int jobId)
        {
            VerifyConnection();

            JobStatus result;

            try
            {
                JobInfo jobInfo = analysisService.checkStatus(jobId);
                result = ConvertToJobStatus(jobInfo.status);
            }
            catch (SoapException)
            {
                result = JobStatus.Unknown;
            }
            catch (WebException e)
            {
                Trace.WriteLine("Exception: " + e, TraceLevel.Error.ToString());
                throw new ConnectionFailedException(Resources.UNABLE_TO_CONNECT, e);
            }

            return result;

        }

        /// <summary>
        /// Gets the job results.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <returns>
        /// an <c>ICollection</c> of <c>Result</c>s from the matching job
        /// </returns>
        public IList<Result> GetJobResults(int jobId)
        {
            return GetJobResults(jobId, null, -1);
        }

        /// <summary>
        /// Gets the job results.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <param name="pipelineConnection">The pipeline connection.</param>
        /// <returns></returns>
        public IList<Result> GetJobResults(int jobId, Connection pipelineConnection)
        {
            this.analysisService = new AnalysisService();

            this.connection = pipelineConnection;
            SetUpWebServices();
            return GetJobResults(jobId, null, -1);
        }

        /// <summary>
        /// Registers for notification of job completion
        /// </summary>
        /// <param name="emailAddress">The email address.</param>
        /// <param name="jobId">The job id.</param>
        public void RegisterForJobNotification(string emailAddress, int jobId)
        {
            VerifyConnection();

            try
            {
                WebClientHelper emailNotification = new WebClientHelper(connection.ServerUri.AbsoluteUri,
                    connection.Username, connection.Password, connection.ServerVersion);
                emailNotification.RegisterForEmailNotification(emailAddress, jobId);
                emailNotification.Dispose();
            }
            catch (WebException e)
            {
                Trace.WriteLine("Exception: " + e, TraceLevel.Error.ToString());
                throw new ConnectionFailedException(Resources.UNABLE_TO_CONNECT, e);
            }
            catch (Exception e)
            {
                Trace.WriteLine("e = {0}" + e, TraceLevel.Error.ToString());
                throw;
            }
        }

        /// <summary>
        /// Gets the service info.
        /// </summary>
        /// <value></value>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.Convert.ToBoolean(System.String)")]
        public ServiceInfo GetServiceInfo
        {
            get
            {
                VerifyConnection();

                ServiceInfo result = new ServiceInfo();
                try
                {
                    Admin.Map map = adminService.getServiceInfo();

                    foreach (Admin.mapItem item in map.item)
                    {
                        if (0 == string.Compare(item.key.ToString(), "genepattern.version", true, CultureInfo.InvariantCulture))
                        {
                            result.Version = item.value.ToString();
                        }
                        if (0 == string.Compare(item.key.ToString(), "lsid.authority", true, CultureInfo.InvariantCulture))
                        {
                            result.LsidAuthority = item.value.ToString();
                        }
                        if (0 == string.Compare(item.key.ToString(), "require.password", true, CultureInfo.InvariantCulture))
                        {
                            result.RequiresPassword = Convert.ToBoolean(item.value.ToString());
                        }
                    }
                }
                catch (WebException exception)
                {
                    throw new ConnectionFailedException(Resources.UNABLE_TO_CONNECT, exception);
                }
                return result;
            }
        }

        /// <summary>
        /// Tests the connection to the server.
        /// </summary>
        /// <returns>true if connection </returns>
        public bool TestConnection()
        {
            VerifyConnection();
            return true;
        }

        /// <summary>
        /// Pings the server.
        /// </summary>
        /// <returns>returns true if connection and credentials are correct, otherwise false</returns>
        public bool Ping()
        {
            string message = null;
            try
            {
                message = this.analysisService.ping();
            }
            catch (WebException)
            {
                //throw new ConnectionFailedException(Resources.UNABLE_TO_CONNECT, exception);

                return false;
            }
            catch (SoapException)
            {
                return false;
            }
            if (message.Equals(CONNECTION_SUCCESS))
                return true;
            return false;
        }

        /// <summary>
        /// Gets all tasks.
        /// </summary>
        /// <value></value>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists")]
        public List<Common.Task> GetAllTasks
        {
            get
            {
                VerifyConnection();
                List<Common.Task> taskList = new List<Common.Task>();
                try
                {
                    Trace.WriteLine("Preparing to get tasks from server...");
                    Analysis.TaskInfo[] tasks = this.analysisService.getTasks();
                    Trace.WriteLine("Got tasks from server (" + tasks.Length + ")");
                    foreach (Analysis.TaskInfo task in tasks)
                    {
                        Common.Task t = new Common.Task();
                        t.Name = task.name;
                        t.TaskType = (task.pipeline) ? TaskType.Pipeline : t.TaskType = TaskType.Module;
                        taskList.Add(t);
                    }
                }
                catch (Exception exception)
                {
                    Trace.WriteLine("Exception occured while trying to retrieve tasks. Exception Stack Trace : " + exception.StackTrace,
                                    TraceLevel.Error.ToString());
                    throw new ConnectionFailedException("Unable to connect to GenePattern server.");
                }
                return taskList;
            }
        }

        /// <summary>
        /// Determines whether the specified pipeline or module is installed.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <returns>
        /// 	<c>true</c> if a module or pipeline with the specified lsid is installed; otherwise, <c>false</c>.
        /// </returns>
        public bool IsInstalled(string lsid)
        {
            VerifyConnection();

            bool result = false;
            try
            {
                TaskInfo taskInfo = adminService.getTask(lsid);
                if (taskInfo != null)
                {
                    result = true;
                }
            }
            catch (WebException e)
            {
                Trace.WriteLine("Exception: " + e, TraceLevel.Error.ToString());
                throw new ConnectionFailedException(Resources.UNABLE_TO_CONNECT, e);
            }

            return result;
        }

        /// <summary>
        /// Gets the jobs by user of teh username of the current connection.
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object,System.Object)")]
        public IList<Job> GetJobsByCurrentUser()
        {
            VerifyConnection();

            IList<Job> jobList = new List<Job>();

            try
            {
                string username = connection.Username;
                JobInfo[] jobInfos = analysisService.getJobs(username, -1, int.MaxValue, false); //maxjobnumber(i.e int.MaxValue) has been changed to -1 which means to start at the current maximum job num in the database while retuning jobs.
                Trace.WriteLine(
                    String.Format("Downloading jobs for user: {0}.  {1} jobs received.", username, jobInfos.Length));
                Dictionary<string, Pipeline> pipelineCache = new Dictionary<string, Pipeline>();
                foreach (JobInfo jobInfo in jobInfos)
                {
                    Job newJob = null;
                    newJob = GetTopLevelJobFromJobInfo(jobInfo, pipelineCache);

                    if (newJob != null && newJob.Status == JobStatus.Finished)
                    {
                        jobList.Add(newJob);
                    }
                }
            }
            catch (WebException e)
            {
                Trace.WriteLine(e, TraceLevel.Error.ToString());
                throw new ConnectionFailedException(Resources.UNABLE_TO_CONNECT, e);
            }

            return jobList;
        }

        /// <summary>
        /// Downloads the task (A Pipeline or a Module)
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <param name="recursive">Whether or not to download child modules as well</param>
        /// <returns>
        /// a <c>MemoryStream</c> hosting the pipeline zip file
        /// </returns>
        public MemoryStream DownloadTask(string lsid, bool recursive)
        {
            VerifyConnection();

            string result = null;

            try
            {
                result = taskService.exportToZip(lsid, recursive);
            }
            catch (WebException e)
            {
                Trace.WriteLine(e, TraceLevel.Error.ToString());
                throw new ConnectionFailedException(Resources.UNABLE_TO_CONNECT, e);
            }
            byte[] data = GetBytesFromWebServiceCall(result);
            result = null; // trying to free memory since result is possibly HUGE and this has out of mem errors in debug

            GC.Collect();
            GC.WaitForPendingFinalizers();

            return new MemoryStream(data);
        }

        /// <summary>
        /// Returns a byte array from a web service call.  Checks to see if the actual data was replaced with a filename,
        /// and re-inserts the data if that was the case
        /// </summary>
        /// <param name="result"></param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private static byte[] GetBytesFromWebServiceCall(string result)
        {
            try
            {
                if (File.Exists(result))
                {
                    using (FileStream fs = new FileStream(result, FileMode.Open, FileAccess.Read, FileShare.Read))
                    {
                        byte[] ret = new byte[fs.Length];
                        fs.Read(ret, 0, ret.Length);
                        return ret;
                    }
                }
                else
                {
                    return Convert.FromBase64String(result);
                }
            }
            catch (Exception)
            {
                throw;
            }
            finally
            {
                if (File.Exists(result))
                {
                    File.Delete(result);
                }
            }
        }

        /// <summary>
        /// Gets the child job ids of job with the specified id.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <returns>
        /// an <c>int[]</c> with ids of the child jobs
        /// </returns>
        public int[] GetChildren(int jobId)
        {
            VerifyConnection();

            int[] children = null;

            try
            {
                children = analysisService.getChildren(jobId);
            }
            catch (WebException e)
            {
                Trace.WriteLine(e, TraceLevel.Error.ToString());
                throw new ConnectionFailedException(Resources.UNABLE_TO_CONNECT, e);
            }

            return children;
        }

        /// <summary>
        /// Determines wheter the module with the specified lsid is a pipline
        /// </summary>
        /// <param name="lsid">the lsid of the module</param>
        /// <returns></returns>
        public bool IsPipeline(string lsid)
        {
            if (lsid == null)
                throw new ArgumentNullException("lsid");

            VerifyConnection();

            bool result = false;

            try
            {
                TaskInfo taskInfo = adminService.getTask(lsid);
                if (taskInfo != null)
                {
                    if (taskInfo.pipeline)
                    {
                        result = true;
                    }
                }
            }
            catch (WebException e)
            {
                Trace.WriteLine(e, TraceLevel.Error.ToString());
                throw new ConnectionFailedException(Resources.UNABLE_TO_CONNECT, e);
            }

            return result;
        }

        /// <summary>
        /// Installs the pipeline.
        /// </summary>
        /// <param name="data">The data.</param>
        /// <returns>
        /// The lsid of the installed pipeline
        /// </returns>
        public string InstallPipeline(byte[] data)
        {
            Trace.WriteLine("Installing pipeline ...", TraceLevel.Info.ToString());
            DataHandler dataHandler = new DataHandler();
            dataHandler.File = Convert.ToBase64String(data);
            string identifier = this.taskService.importZip(dataHandler, ACCESS_PRIVATE);
            Trace.WriteLine("identifier for Installed Pipeline= {0}" + identifier, TraceLevel.Info.ToString());
            return identifier;
        }

        /// <summary>
        /// Gets the documentation.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <returns></returns>
        public string[] GetDocumentation(string lsid)
        {
            VerifyConnection();

            string[] filenames = null;

            try
            {
                filenames = taskService.getDocFileNames(lsid);
            }
            catch (WebException we)
            {
                Trace.WriteLine(we, TraceLevel.Error.ToString());
                throw new ConnectionFailedException(Resources.UNABLE_TO_CONNECT, we);
            }

            return filenames;
        }

        /// <summary>
        /// Gets the input file.
        /// </summary>
        /// <param name="filename">The filename.</param>
        /// <returns>a <c>Stream</c> representing the file</returns>
        public Stream GetInputFile(string filename)
        {
            if (filename == null)
                throw new ArgumentNullException("filename");

            string urlformat = connection.ServerUri + "gp/getFile.jsp?file={0}&task=";

            //reverse slashes in filename since we're on the web now
            filename = filename.Replace('\\', '/');
            string url = String.Format(CultureInfo.InvariantCulture, urlformat, filename);
            return DownloadFileFromUrl(url);
        }

        /// <summary>
        /// Gets the documentation file.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <param name="filename">The filename.</param>
        /// <returns>
        /// a <c>MemoryStream</c> representing the file
        /// </returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object,System.Object)")]
        public MemoryStream GetDocumentationFile(string lsid, string filename)
        {
            string url = connection.ServerUri + string.Format("gp/getTaskDoc.jsp?name={0}&file={1}", lsid, filename);

            return DownloadFileFromUrl(url);
        }

        #endregion

        #region IDisposable Members

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Gets the job results.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <param name="specificFilename">The specific filename.</param>
        /// <returns>
        /// an <c>ICollection</c> of <c>Result</c>s from the matching job
        /// </returns>
        public IList<Result> GetJobResults(int jobId, string specificFilename)
        {
            return GetJobResults(jobId, specificFilename, -1);
        }

        /// <summary>
        /// Gets the job results.
        /// </summary>
        /// <param name="shallowResults">The shallow results.</param>
        /// <returns></returns>
        public IList<Result> GetJobResults(IList<ShallowJobResult> shallowResults)
        {
            List<Result> results = new List<Result>();
            foreach (ShallowJobResult res in shallowResults)
            {
                IList<Result> jobResults = GetJobResults(res.SourceJobId, res.Filename);
                foreach (Result r in jobResults)
                {
                    r.SourceStep = res.SourceStep; // since we asked for the child job, need to set the step index from the requested data
                    r.OutputIndex = res.SourceIndex;
                }
                results.AddRange(jobResults);
            }
            return results;
        }

        /// <summary>
        /// Gets the job results.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <param name="limitToFilename">The limit to filename.</param>
        /// <param name="jobIndex">Index of the job.</param>
        /// <returns></returns>
        private IList<Result> GetJobResults(int jobId, string limitToFilename, int jobIndex)
        {
            VerifyConnection();
            int[] childJobIds = new int[0];
            if (analysisService.getChildren(jobId).Length > 0)
            {
                jodIdSwitch = jobId;
                childJobIds = analysisService.getChildren(jobId);
            }
            else
            {
                childJobIds = analysisService.getChildren(jodIdSwitch);
            }
            List<Result> results = new List<Result>();
            try
            {
                JobInfo jobInfo = new JobInfo();
                //bug fix for 1091
                try
                {// make sure the job actually has results before we do this!
                    jobInfo = analysisService.getJob(jobId);
                }
                catch (Exception ex)
                {
                    if (ex.Message.Contains("JobIDNotFound"))
                    {
                        throw new Exception("Job with id " + jobId + " no longer exists on server");
                    }
                    else
                    {
                        throw ex;
                    }
                }
                int outFileCount = 0;
                foreach (ParameterInfo p in jobInfo.parameterInfoArray)
                {
                    outFileCount += p.outputFile ? 1 : 0;
                }

                if (outFileCount > 0)
                {
                    Trace.WriteLine("Getting job results for job: " + jobId);
                    Analysis.FileWrapper[] fileWrappers = null;
                    if (limitToFilename != null)
                    {
                        fileWrappers = analysisService.getResultFiles(jobId, new string[] { limitToFilename });
                    }

                    else
                    {
                        try
                        {
                            //Check whether the job is present in the server.
                            IList<Job> jobs = GetJobsByCurrentUser();
                            bool entered = false;
                            if (analysisService.getChildren(jobId).Length > 0)
                            {
                                foreach (Job job in jobs)
                                {
                                    if (job.Id == jobId)
                                    {
                                        fileWrappers = analysisService.getResultFiles(jobId);
                                        childJobIds = analysisService.getChildren(jobId);
                                        entered = true;
                                    }
                                }
                            }

                            else
                            {
                                foreach (int childJobId in childJobIds)
                                {
                                    if (childJobId == jobId)
                                    {
                                        fileWrappers = analysisService.getResultFiles(jobId);
                                        entered = true;
                                    }
                                }
                            }

                            if (!entered)
                            {
                                throw new Exception("Job with id " + jobId + " no longer exists on server");
                            }
                        }
                        catch (ThreadAbortException ex)
                        {
                            throw ex;
                        }
                        catch (Exception ex)
                        {
                            if (ex.Message.Contains("no longer exists on server"))
                            {
                                if (jobInfo != null)
                                    throw new Exception("Pipeline (" + jobInfo.taskName + ") no longer exists on server");
                                else
                                    throw new Exception("Job with id " + jobId + " no longer exists on server");
                            }
                            else
                            {
                                throw ex;
                            }
                        }
                    }
                    Trace.WriteLine(string.Format("Successfully got {0} job results for job", fileWrappers));
                    int fileCount = 1;
                    foreach (FileWrapper wrapper in fileWrappers)
                    {
                        Result result = new Result();
                        result.Filename = wrapper.filename;
                        result.Length = wrapper.length;
                        result.LastModified = new DateTime(wrapper.lastModified);
                        result.Content = GetBytesFromWebServiceCall(wrapper.dataHandler);
                        // store exact info on where the result came from so we can find a visualizer later
                        result.OutputIndex = fileCount;
                        result.SourceStep = jobIndex;
                        results.Add(result);
                        fileCount++;
                    }
                }
                if (limitToFilename == null)
                {
                    int[] childJobs = analysisService.getChildren(jobId);
                    int jobCount = 1;
                    foreach (int childJobId in childJobs)
                    {
                        results.AddRange(GetJobResults(childJobId, null, jobCount));
                        jobCount++;
                    }
                }
            }
            catch (WebException e)
            {
                Trace.WriteLine("Exception: " + e, TraceLevel.Error.ToString());
                throw new ConnectionFailedException(Resources.UNABLE_TO_CONNECT, e);
            }

            return results;
        }

        /// <summary>
        /// Downloads the file from URL.
        /// </summary>
        /// <param name="uri">The URI.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object,System.Object)")]
        private MemoryStream DownloadFileFromUrl(string uri)
        {

            try
            {
                WebClientHelper downloadFile = new WebClientHelper(connection.ServerUri.AbsoluteUri,
                    connection.Username, connection.Password, connection.ServerVersion);
                using (Stream responseStream = downloadFile.DownloadFileFromServer(uri))
                {
                    downloadFile.Dispose();
                    return StreamHelper.GetMemoryStream(responseStream);
                }
            }
            catch (WebException e)
            {
                Trace.WriteLine("Exception: " + e, TraceLevel.Error.ToString());
                string additionalInformation = string.Empty;
                if (!string.IsNullOrEmpty(e.Message))
                {
                    additionalInformation = " : " + e.Message;
                }
                throw new ConnectionFailedException(Resources.UNABLE_TO_CONNECT + additionalInformation, e);
            }
            catch (Exception e)
            {
                Trace.WriteLine("e = {0}" + e, TraceLevel.Error.ToString());
                throw;
            }
        }

        /// <summary>
        /// Converts to job status.
        /// </summary>
        /// <param name="status">The status.</param>
        /// <returns></returns>
        private static JobStatus ConvertToJobStatus(string status)
        {
            JobStatus result;
            switch (status)
            {
                case "Pending":
                    result = JobStatus.Pending;
                    break;

                case "Processing":
                    result = JobStatus.Processing;
                    break;

                case "Finished":
                    result = JobStatus.Finished;
                    break;
                default:
                    result = JobStatus.Unknown;
                    break;
            }
            return result;
        }

        /// <summary>
        /// Sets the web service paths.
        /// </summary>
        private void SetWebServicePaths()
        {
            //If connection is null set the URL to be the NULL_URL
            if (this.connection == null)
            {
                this.adminService.Url = NULL_URL;
                this.analysisService.Url = NULL_URL;
                this.taskService.Url = NULL_URL;
            }
            else
            {
                Uri serverUri = this.Connection.ServerUri;
                this.adminService.Url = serverUri + this.adminServicePath;
                this.analysisService.Url = serverUri + this.analysisServicePath;
                this.taskService.Url = serverUri + this.taskServicePath;
            }
        }

        /// <summary>
        /// Sets up web services.
        /// </summary>
        private void SetUpWebServices()
        {
            this.SetWebServicePaths();
            this.SetCredentials(this.analysisService);
            this.SetCredentials(this.adminService);
            this.SetCredentials(this.taskService);
        }

        /// <summary>
        /// Sets the credentials.
        /// </summary>
        /// <param name="service">The service.</param>
        private void SetCredentials(SoapHttpClientProtocol service)
        {
            service.UseDefaultCredentials = false;
            service.PreAuthenticate = true;
            service.Credentials = null;

            if (connection != null)
            {
                CredentialCache MyCredentialCache = new System.Net.CredentialCache();
                MyCredentialCache.Add(connection.ServerUri,
                                      "Basic",
                                      new System.Net.NetworkCredential(connection.Username, connection.Password));

                service.Credentials = MyCredentialCache;
            }
        }

        /// <summary>
        /// Verifies the connection.
        /// </summary>
        private void VerifyConnection()
        {
            if (this.connection == null || this.adminService.Url == NULL_URL
                || this.analysisService.Url == NULL_URL || this.taskService.Url == NULL_URL)
            {
                throw new ConnectionFailedException(Resources.CONNECTION_NOT_CONFIGURED, null);
            }
        }

        /// <summary>
        /// Disposes the specified the privately held resources.
        /// </summary>
        /// <param name="disposing">if set to <c>true</c> disposes resources.</param>
        private void Dispose(bool disposing)
        {
            // Check to see if Dispose has already been called.
            if (!this.disposed)
            {
                // If disposing equals true, dispose all managed 
                // and unmanaged resources.
                if (disposing)
                {
                    adminService.Dispose();
                    analysisService.Dispose();
                    taskService.Dispose();
                }
            }
            disposed = true;
        }

        #endregion
    }
}
