using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.GenePattern.Server.Admin;

namespace Microsoft.eResearch.GenePattern.Server
{
    /// <summary>
    /// Helper class for transforming server parameter info
    /// to instances internal classes
    /// </summary>
    public sealed class ParameterHelper
    {
        private ParameterHelper()
        {}

        /// <summary>
        /// Gets a job parameter from server representation.
        /// </summary>
        /// <param name="info">The info.</param>
        /// <returns></returns>
        public static JobParameter GetJobParameter(Analysis.ParameterInfo info)
        {
            if (info == null)
                throw new ArgumentNullException("info");

            JobParameter jobParameter = new JobParameter();
            jobParameter.Name = info.name;
            jobParameter.ParamValue = info.value;
            jobParameter.Optional = info.optional;

            return jobParameter;    
        }

        /// <summary>
        /// Gets the pipeline parameter from server representation.
        /// </summary>
        /// <param name="info">The info.</param>
        /// <returns></returns>
        public static PipelineParameter GetPipelineParameter(Admin.ParameterInfo info)
        {
            PipelineParameter pipelineParameter = new PipelineParameter();
            pipelineParameter.Name = info.name;
            pipelineParameter.Optional = info.optional;

            Dictionary<string, string> parameterAttirbutes = MapHelper.GetDictionaryFromMap(info.attributes);
            if (parameterAttirbutes.ContainsKey("altName") && !string.IsNullOrEmpty(parameterAttirbutes["altName"].Trim()))
            {
                pipelineParameter.ShortName = parameterAttirbutes["altName"].Replace(".", " ");
            }
            else if (!string.IsNullOrEmpty(info.name))
            {
                string[] parts = info.name.Split(new char[] {'.'});
                if(parts.Length > 1)
                {
                    string[] newParts = new string[parts.Length-1];
                    Array.Copy(parts, 1, newParts, 0, parts.Length - 1);
                    pipelineParameter.ShortName = string.Join(" ", newParts);
                }
                else
                {
                    pipelineParameter.ShortName = info.name;
                }
            }
            else
            {
                pipelineParameter.ShortName = info.name;
            }

            if (parameterAttirbutes.ContainsKey("altDescription") && !string.IsNullOrEmpty(parameterAttirbutes["altDescription"].Trim()))
            {
                pipelineParameter.Description = parameterAttirbutes["altDescription"];
            }
            else
            {
                pipelineParameter.Description = info.description;
            }

            if (info.inputFile)
            {
                pipelineParameter.Type = ParameterType.InputFile;
            }
            else
            {
                if (info.choices != null && info.choices.item != null && info.choices.item.Length > 0)
                {
                    pipelineParameter.Type = ParameterType.StringCollection;

                    foreach (mapItem item in info.choices.item)
                    {
                        pipelineParameter.Choices.Add(new Choice(item.value.ToString(), item.key.ToString()));
                    }
                }
                else
                {
                    pipelineParameter.Type = ParameterType.String;
                }
            }
            return pipelineParameter;
        }
    }
}
