using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using Microsoft.eResearch.GenePattern.Common;
using Microsoft.eResearch.GenePattern.Server.Analysis;

namespace Microsoft.eResearch.GenePattern.Server
{
    /// <summary>
    /// Parameters Info Helper Class
    /// </summary>
    public static class ParameterInfoHelper
    {

        #region  Methods (7)

        //  Public Methods (3)

        /// <summary>
        /// Creates the job parameters mapping with teh job Name as key and the Job object as the value.
        /// </summary>
        /// <param name="parameters">The parameters containing the Job Parameter.</param>
        /// <returns>A Dictionary with mapping of Job parameter names to Job Objects</returns>
        public static Dictionary<string, JobParameter> CreateJobParametersMapping(IList<JobParameter> parameters)
        {
            Dictionary<string, JobParameter> dictionary = new Dictionary<string, JobParameter>();
            foreach (JobParameter jobParameter in parameters)
            {
                dictionary.Add(jobParameter.Name, jobParameter);
            }
            return dictionary;
        }

        /// <summary>
        /// Creates the parameter info list.
        /// </summary>
        /// <param name="jobParameters">The job parameters.</param>
        /// <param name="pipeline">The pipeline.</param>
        /// <param name="files">The files.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1002:DoNotExposeGenericLists"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1021:AvoidOutParameters", MessageId = "2#")]
        public static List<ParameterInfo> CreateParameterInfoList(Dictionary<string, JobParameter> jobParameters,
                                                               Pipeline pipeline,
                                                               out Map files)
        {
            List<ParameterInfo> parameterInfoList = new List<ParameterInfo>();
            files = new Map();
            List<mapItem> fileMapItems = new List<mapItem>();
            foreach (PipelineParameter parameter in pipeline.Parameters)
            {
                ParameterInfo parameterInfo = new ParameterInfo();
                parameterInfo.name = parameter.Name;
                parameterInfo.outputFile = false;
                parameterInfo.password = false;
                Map attributes = null;
                switch (parameter.Type)
                {
                    case ParameterType.Number:
                    case ParameterType.String:
                    case ParameterType.StringCollection:
                        parameterInfo.inputFile = false;
                        parameterInfo.value = jobParameters[parameter.Name].ParamValue.ToString();
                        break;
                    case ParameterType.InputFile:
                        parameterInfo.inputFile = true;
                        InputFile inputFile = jobParameters[parameter.Name].ParamValue as InputFile;
                        Debug.Assert(inputFile != null, "Input File cannot be null.");
                        attributes = ConstructMapItem(fileMapItems, inputFile, parameterInfo, jobParameters[parameter.Name]);
                        break;
                }

                Map choices = new Map();
                choices.item = ConvertChoicesToMapArray(parameter.Choices);
                parameterInfo.choices = choices;
                parameterInfo.attributes = attributes;
                parameterInfo.optional = jobParameters[parameter.Name].Optional;

                parameterInfoList.Add(parameterInfo);
            }

            files.item = fileMapItems.ToArray();

            return parameterInfoList;
        }

        /// <summary>
        /// Creates the stop point paramter info.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="value">The value.</param>
        /// <returns></returns>
        public static ParameterInfo CreateStopPointParamterInfo(string name, string value)
        {
            ParameterInfo parameterInfo = new ParameterInfo();
            parameterInfo.name = name;
            parameterInfo.outputFile = false;
            parameterInfo.password = false;
            parameterInfo.inputFile = false;
            parameterInfo.description = String.Empty;
            parameterInfo.label = String.Empty;
            parameterInfo.value = value;
            parameterInfo.optional = false;
            Map choices = ConstructJavaMap(value, value);
            parameterInfo.choices = choices;


            return parameterInfo;
        }

        //  Private Methods (4)

        private static Map ConstructFileAttributes(string filePath, bool optional)
        {
            Map attributes = new Map();
            List<mapItem> mapItems = new List<mapItem>();
            mapItem file = new mapItem();
            file.key = "client_filename";
            // TODO : Have to get the file name from the JobParameter class
            file.value = filePath;

            mapItem type = new mapItem();
            type.key = "TYPE";
            type.value = "FILE";

            mapItem mode = new mapItem();
            mode.key = "MODE";
            mode.value = "IN";

            mapItems.Add(file);
            mapItems.Add(type);
            mapItems.Add(mode);

            if (optional)
            {
                mapItem opt = new mapItem();
                opt.key = "optional";
                opt.value = "on";
                mapItems.Add(opt);
            }

            attributes.item = mapItems.ToArray();
            return attributes;

        }

        private static Map ConstructJavaMap(string key, string value)
        {
            Map map = new Map();
            List<mapItem> mapItems = new List<mapItem>();
            mapItem mapItem = new mapItem();
            mapItem.key = key;

            mapItem.value = value;

            mapItems.Add(mapItem);

            map.item = mapItems.ToArray();
            return map;

        }

        private static Map ConstructJavaMap(string key, string value, bool optional)
        {
            Map map = new Map();
            List<mapItem> mapItems = new List<mapItem>();

            mapItem mapItem = new mapItem();
            mapItem.key = key;
            mapItem.value = value;
            mapItems.Add(mapItem);

            if (optional)
            {
                mapItem opt = new mapItem();
                opt.key = "optional";
                opt.value = "on";
                mapItems.Add(opt);
            }

            map.item = mapItems.ToArray();
            return map;
        }



        private static Map ConstructMapItem(List<mapItem> fileMapItems, InputFile inputFile,
                                              ParameterInfo parameterInfo, JobParameter jobParameter)
        {
            if (inputFile == null)
            {
                inputFile = new InputFile();
                inputFile.IsLinked = true;
                inputFile.Name = "";
                inputFile.Path = "";
            }

            Map attributes = null;
            if (inputFile.IsLinked)
            {
                //Treat like a string input
                parameterInfo.inputFile = false;
                attributes = ConstructJavaMap("client_filename", inputFile.Path, jobParameter.Optional);
                parameterInfo.value = inputFile.Path;
                parameterInfo.description = string.Empty;
                parameterInfo.label = String.Empty;

            }
            else
            {
                // Set file value to be data handler
                mapItem file = new mapItem();

                // check for naming collisions on files -- cant have duplicate keys in the map!
                bool nameCollision = true;
                string mapItemKey = inputFile.Name;
                while (nameCollision)
                {
                    nameCollision = false;
                    foreach (mapItem item in fileMapItems)
                    {
                        if (item.key.Equals(mapItemKey))
                        {
                            nameCollision = true;
                            int lastDot = mapItemKey.LastIndexOf(".");
                            string ext = "";
                            string filename = mapItemKey;
                            if (lastDot != -1)
                            {
                                ext = mapItemKey.Substring(lastDot + 1);
                                filename = mapItemKey.Substring(0, lastDot);
                            }

                            mapItemKey = filename + "_1." + ext;
                            break;
                        }
                    }
                }

                file.key = mapItemKey;
                DataHandler dataHandler = new DataHandler();
                dataHandler.File = inputFile.Contents;
                file.value = dataHandler;
                attributes = ConstructFileAttributes(inputFile.Path, jobParameter.Optional);
                parameterInfo.value = mapItemKey;
                fileMapItems.Add(file);
            }

            return attributes;
        }

        private static mapItem[] ConvertChoicesToMapArray(List<Choice> choices)
        {
            List<mapItem> mapItems = new List<mapItem>();
            foreach (Choice choice in choices)
            {
                Analysis.mapItem choiceMapItem = new mapItem();
                choiceMapItem.key = choice.Text;
                choiceMapItem.value = choice.Value;
                mapItems.Add(choiceMapItem);

            }

            return mapItems.ToArray();
        }

        #endregion

    }

}
