using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Microsoft.eResearch.GenePattern.Common;
using System.IO;
using System.Reflection;
using Microsoft.eResearch.GenePattern.Server.Properties;

namespace Microsoft.eResearch.GenePattern.Server
{
    /// <summary>
    /// Helper class for extracting information 
    /// about steps of a pipeline from its serialized
    /// state
    /// </summary>
    public sealed class PipelineStepHelper
    {
        private static Dictionary<string,bool> isPipelineCache = new Dictionary<string, bool>();
        private static Dictionary<string, string> fileMapping = null;
       
        private PipelineStepHelper()
        {
        }

        /// <summary>
        /// Extracts pipeline steps from the serialized model,
        /// fetches them from the server
        /// and adds them to the target pipeline
        /// </summary>
        /// <param name="serializedModel">The serialized model.</param>
        /// <param name="target">The target.</param>
        /// <param name="genePatternServer">The gene pattern server.</param>
        public static void ExtractFromModel(string serializedModel, Pipeline target, IGenePatternServer genePatternServer)
        {
            if (serializedModel == null)
                throw new ArgumentNullException("serializedModel");
            
            if (target == null)
                throw new ArgumentNullException("target");

            if (fileMapping == null)
                LoadFileMapping();

            XmlDocument document = new XmlDocument();
            document.LoadXml(serializedModel);

            XmlNodeList nodes = document.SelectNodes("//*[name(.)='task' or name(.)='visualizer']");
            string beginDelim = "gpUseResult(";
            string endDelim = ")";
            char[] delims = { ',', '\'' };

            foreach(XmlElement element in nodes)
            {
                PipelineStep pipelineStep = new PipelineStep();
                pipelineStep.Name = element.GetAttribute("name").Replace("'", string.Empty);
                pipelineStep.Lsid = element.GetAttribute("LSID");
                PipelineStepType stepType = PipelineStepType.Module;
                
                if(element.Name == "visualizer")
                {
                    stepType = PipelineStepType.Visualizer;

                    // get location of files that can be visualized with this module
                    foreach(XmlAttribute attribute in element.Attributes)
                    {

                        if (attribute.Value.StartsWith(beginDelim))
                        {
                            VisualizerData vizData = new VisualizerData();
                            int end = attribute.Value.IndexOf(endDelim);
                            string[] tokens = attribute.Value.Substring(beginDelim.Length, end-beginDelim.Length+1).Split(delims);
                            int sourceStep = -1;
                            int resultIndex = -1;
                            int.TryParse(tokens[0], out sourceStep);
                            bool isNumeric = int.TryParse(tokens[2], out resultIndex);
                            if(!isNumeric)
                            {
                                if (fileMapping.ContainsKey(tokens[2]))
                                {
                                    vizData.SourceFileExtension = fileMapping[tokens[2]];
                                }
                                else
                                {
                                    vizData.SourceFileExtension = tokens[2];
                                }
                            }
                            vizData.UseExtension = !isNumeric;

                            vizData.SourceFileStep = sourceStep;
                            vizData.SourceFileIndex = resultIndex;
                            vizData.FilenameVariable = attribute.Name;
                            pipelineStep.VisualizerData.Add(vizData);
                        }
                    }
                }
                if (isPipelineCache.ContainsKey(pipelineStep.Lsid))
                {
                    bool isPipeline = isPipelineCache[pipelineStep.Lsid];
                    if (isPipeline)
                    {
                        stepType = PipelineStepType.Pipeline;
                    }
                }
                else
                {
                    bool isPipeline = genePatternServer.IsPipeline(pipelineStep.Lsid);
                    if (!isPipelineCache.ContainsKey(pipelineStep.Lsid))
                    {
                        isPipelineCache.Add(pipelineStep.Lsid, isPipeline);
                    }
                    if (isPipeline)
                    {
                        stepType = PipelineStepType.Pipeline;
                    }
                }
                pipelineStep.Type = stepType;

                target.Steps.Add(pipelineStep);
            }
        }

        public static void LoadFileMapping()
        {
            try
            {
                fileMapping = new Dictionary<string, string>();
               
                XmlDocument xmlDocFileMapping = new XmlDocument();
                xmlDocFileMapping.LoadXml(Resources.FileExtensionMapping);
                XmlNodeList nodelist = xmlDocFileMapping.SelectNodes("//FileMapping/add");

                string name = string.Empty;
                foreach (XmlNode xnode in nodelist)
                {
                    name = xnode.Attributes["Name"].Value;
                    if (!fileMapping.ContainsKey(name))
                    {
                        fileMapping.Add(name, xnode.Attributes["value"].Value);
                    }
                }
            }
            catch (Exception ex)
            {
            }
        }
    }
}
