using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using System.Web;
using System.Globalization;

namespace Microsoft.eResearch.GenePattern.Server
{
    class WebClientHelper : IDisposable
    {
        public enum LoginStatus
        {
            SUCCESS,
            INVALID, //incorrect username and/or password
            IO_EXCEPTION,  //problem connecting to the server
        }

        #region Variable Declaration

        const string LOGIN_PAGE = "/gp/pages/login.jsf";
        const string HOME_PAGE = "/gp/pages/index.jsf";
        const string SERVER_VERSION_3_0 = "3.0";

        private string _serverUrl = string.Empty;
        private string _userName = string.Empty;
        private string _password = string.Empty;
        private string _serverVersion = string.Empty;
        HttpWebResponse response;
        HttpWebRequest request;
        
        #endregion

        #region Private Method

        /// <summary>
        /// Initializes a new instance of the <see cref="WebClientHelper"/> class.
        /// </summary>
        /// <param name="serverUrl">The server URL.</param>
        /// <param name="userName">Name of the user.</param>
        /// <param name="password">The password.</param>
        /// <param name="serverVersion">The server version.</param>
        public WebClientHelper(string serverUrl, string userName, string password, string serverVersion)
        {
            this._serverUrl = serverUrl;
            this._userName = userName;
            this._password = password;
            this._serverVersion = serverVersion;
        }

        /// <summary>
        /// Logins to server.
        /// </summary>
        /// <param name="cookie">The cookie.</param>
        /// <returns></returns>
        private LoginStatus LoginToServer(out string cookie)
        {
            cookie = string.Empty;
            try
            {
                // Get Response of Home Page
                response = (HttpWebResponse)request.GetResponse();

                if (!LOGIN_PAGE.Equals(response.ResponseUri.AbsolutePath))
                {
                    return LoginStatus.SUCCESS;
                }

                // Get the View State Value of Login Page
                string viewStateValue = ParseLoginForm();
                UpdateServerUrl();

                // Store Cookies in temporary variable for further use
                cookie = response.Headers["Set-Cookie"];
                request = PerformLoginOperation(viewStateValue, cookie);
                return ValidateLogin(cookie);
            }
            catch (Exception)
            {
                throw;
            }
            finally
            {
                if (response != null)
                {
                    response.Close();
                }
            }
        }

        /// <summary>
        /// Performs the login operation.
        /// </summary>
        /// <param name="viewStateValue">The view state value.</param>
        /// <param name="cookie">The cookie.</param>
        /// <returns></returns>
        private HttpWebRequest PerformLoginOperation(string viewStateValue, string cookie)
        {
            // Get POST content
            string postData = GetPostData(viewStateValue);
            byte[] byteArray = Encoding.ASCII.GetBytes(postData);

            // Create Web Request for Login Page
            request = (HttpWebRequest)WebRequest.Create(_serverUrl + LOGIN_PAGE);
            request.AllowAutoRedirect = false;
            request.ContentType = "application/x-www-form-urlencoded";
            request.Method = "POST";
            request.Headers["Cookie"] = cookie;

            Stream requestStream = request.GetRequestStream();
            requestStream.Write(byteArray, 0, byteArray.Length);
            requestStream.Flush();
            requestStream.Close();

            request.GetResponse();
            return request;
        }

        /// <summary>
        /// Gets the post data.
        /// </summary>
        /// <param name="viewStateValue">The view state value.</param>
        /// <returns></returns>
        private string GetPostData(string viewStateValue)
        {
            StringBuilder postData = new StringBuilder();
            postData.Append(HttpUtility.UrlEncodeUnicode("loginForm"));
            postData.Append("=");
            postData.Append(HttpUtility.UrlEncodeUnicode("loginForm"));

            postData.Append("&");
            postData.Append(HttpUtility.UrlEncodeUnicode("javax.faces.ViewState"));
            postData.Append("=");
            postData.Append(HttpUtility.UrlEncodeUnicode(viewStateValue));

            postData.Append("&");
            postData.Append(HttpUtility.UrlEncodeUnicode("username"));
            postData.Append("=");
            postData.Append(HttpUtility.UrlEncodeUnicode(_userName));

            if (!string.IsNullOrEmpty(_password))
            {
                postData.Append("&");
                postData.Append(HttpUtility.UrlEncodeUnicode("loginForm:password"));
                postData.Append("=");
                postData.Append(HttpUtility.UrlEncodeUnicode(_password));
            }

            postData.Append("&");
            postData.Append(HttpUtility.UrlEncodeUnicode("loginForm:signIn"));
            postData.Append("=");
            postData.Append(HttpUtility.UrlEncodeUnicode("Sign In"));
            return postData.ToString();
        }

        /// <summary>
        /// Parses the login form.
        /// </summary>
        /// <returns></returns>
        private string ParseLoginForm()
        {
            string matchViewState = "name=\"javax.faces.ViewState\" id=\"javax.faces.ViewState\" value=\"";
            string returnValue = string.Empty;
            string loginResponse = string.Empty;
            
            using (StreamReader streamReader = new StreamReader(response.GetResponseStream()))
            {
                loginResponse = streamReader.ReadToEnd();
            }

            int viewStateStartIndex = loginResponse.IndexOf(matchViewState);

            if (viewStateStartIndex == -1)
            {
                return null;
            }

            loginResponse = loginResponse.Substring(viewStateStartIndex + matchViewState.Length);
            returnValue = loginResponse.Substring(0, loginResponse.IndexOf("\" />"));
            return returnValue;
        }

        /// <summary>
        /// Updates the server URL.
        /// </summary>
        private void UpdateServerUrl()
        {
            Uri uri = response.ResponseUri;
            int port = uri.Port;
            String portStr = port > 0 ? ":" + port : "";
            this._serverUrl = uri.Scheme + "://" + uri.Host + portStr;
        }

        /// <summary>
        /// Validates the login.
        /// </summary>
        /// <param name="cookie">The cookie.</param>
        /// <returns></returns>
        private LoginStatus ValidateLogin(string cookie)
        {
            try
            {
                // Create WebRequest for Login Page
                request = (HttpWebRequest)WebRequest.Create(_serverUrl + HOME_PAGE);
                request.AllowAutoRedirect = false;
                request.Method = "GET";
                request.Headers["Cookie"] = cookie;
                response = request.GetResponse() as HttpWebResponse;

                switch (response.ResponseUri.AbsolutePath)
                {
                    case HOME_PAGE:
                        return LoginStatus.SUCCESS;
                    case LOGIN_PAGE:
                        return LoginStatus.INVALID;
                    default:
                        return LoginStatus.IO_EXCEPTION;
                }
            }
            catch (Exception)
            {
                throw;
            }
            finally
            {
                if (response != null)
                {
                    response.Close();
                }
            }
        }

        /// <summary>
        /// Gets the content body.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <param name="emailAddress">The email address.</param>
        /// <returns></returns>
        private string GetContentBody(int jobId, string emailAddress)
        {
            StringBuilder body = new StringBuilder();
            body.Append("jobID=");
            body.Append(jobId);
            body.Append("&cmd=notifyEmailJobCompletion");
            body.Append("&userID=");
            body.Append(_userName);
            body.Append("&userEmail=");
            body.Append(HttpUtility.UrlEncodeUnicode(emailAddress));
            return body.ToString();
        }

        #endregion

        #region Public Method

        /// <summary>
        /// Registers for email notification.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <param name="emailAddress">The email address.</param>
        public void RegisterForEmailNotification(string emailAddress, int jobId)
        {
            if (string.Compare(_serverVersion, SERVER_VERSION_3_0, true, CultureInfo.InvariantCulture) == 0)
            {
                RegisterForEmailNotification3_0(emailAddress, jobId);
                return;
            }

            request = (HttpWebRequest)WebRequest.CreateDefault(new Uri(_serverUrl + HOME_PAGE));

            LoginStatus state = LoginStatus.INVALID;
            string cookie = string.Empty;
            state = LoginToServer(out cookie);

            if (LoginStatus.SUCCESS != state)
            {
                return;
            }

            UTF8Encoding encoding = new UTF8Encoding();
            string body = GetContentBody(jobId, emailAddress);
            byte[] bytes = encoding.GetBytes(body);

            // Create WebRquest for Notify Job Completion Page
            request = (HttpWebRequest)WebRequest.Create(_serverUrl + "/gp/notifyJobCompletion.ajax");
            request.Method = "POST";
            request.AllowAutoRedirect = false;
            request.ContentType = "application/x-www-form-urlencoded";
            request.Headers["Cookie"] = cookie;

            using (Stream requestStream = request.GetRequestStream())
            {
                requestStream.Write(bytes, 0, bytes.Length);
            }

            request.GetResponse();
        }

        /// <summary>
        /// Registers for email notification.
        /// </summary>
        /// <param name="jobId">The job id.</param>
        /// <param name="emailAddress">The email address.</param>
        public void RegisterForEmailNotification3_0(string emailAddress, int jobId)
        {
            UTF8Encoding encoding = new UTF8Encoding();
            string body = GetContentBody(jobId, emailAddress);
            byte[] bytes = encoding.GetBytes(body);

            // Create WebRquest for Notify Job Completion Page
            request = (HttpWebRequest)WebRequest.Create(_serverUrl + "/gp/notifyJobCompletion.ajax");
            request.Method = "POST";
            request.AllowAutoRedirect = false;
            request.ContentType = "application/x-www-form-urlencoded";
            request.Headers["Cookie"] = "userid=" + _userName + "; Path=/gp";

            using (Stream requestStream = request.GetRequestStream())
            {
                requestStream.Write(bytes, 0, bytes.Length);
            }

            request.GetResponse();
        }

        /// <summary>
        /// Downloads the file from server.
        /// </summary>
        /// <param name="url">The URL.</param>
        /// <returns></returns>
        public Stream DownloadFileFromServer(string url)
        {
            if (string.Compare(_serverVersion, SERVER_VERSION_3_0, true, CultureInfo.InvariantCulture) == 0)
            {
                return DownloadFileFromServer3_0(url);
            }

            //Login into the Gene Pattern Server.
            request = (HttpWebRequest)WebRequest.CreateDefault(new Uri(_serverUrl + HOME_PAGE));

            LoginStatus state = LoginStatus.INVALID;
            string cookie = string.Empty;
            state = LoginToServer(out cookie);

            if (LoginStatus.SUCCESS != state)
            {
                return null;
            }

            // Create WebRquest for Download File 
            request = (HttpWebRequest)WebRequest.Create(new Uri(url));

            request.Method = "GET";
            request.AllowAutoRedirect = false;
            request.ContentType = "application/x-www-form-urlencoded";
            request.Headers["Cookie"] = cookie;
            //if (url.Contains("temp/attachments"))
            //{
            //    request.Headers["Cookie"] = GetUserCredentialCookie();
            //}
            response = request.GetResponse() as HttpWebResponse;
            
            // There is chance to make redirected request for the actual resource
            // in this case the 'Location' attribute of response header will contains the 
            // actual location.
            if (response.StatusCode == HttpStatusCode.Found)
            {
                request = (HttpWebRequest)WebRequest.Create(new Uri(response.Headers["Location"]));

                request.Method = "GET";
                request.AllowAutoRedirect = false;
                request.ContentType = "application/x-www-form-urlencoded";
                request.Headers["Cookie"] = cookie;
                response = request.GetResponse() as HttpWebResponse;
            }

            Stream returnStream = response.GetResponseStream();
            return returnStream;
        }

        /// <summary>
        /// Downloads the file from server3_0.
        /// </summary>
        /// <param name="url">The URL.</param>
        /// <returns></returns>
        public Stream DownloadFileFromServer3_0(string url)
        {
            request = (HttpWebRequest)WebRequest.Create(url);
            request.AllowAutoRedirect = false;
            request.ContentType = "application/x-www-form-urlencoded";
            request.Headers["Cookie"] = GetUserCredentialCookie();
            response = (HttpWebResponse)request.GetResponse();
            return response.GetResponseStream();
        }

        /// <summary>
        /// Gets the user credential cookie.
        /// </summary>
        /// <returns></returns>
        public string GetUserCredentialCookie()
        {
            StringBuilder credential = new StringBuilder();
            credential.Append("userid=");
            credential.Append(_userName);
            
            if(string.IsNullOrEmpty(_password))
            {
                credential.Append(" pwd=");
                credential.Append(_password);
            }

            credential.Append("; Path=/gp");
            return credential.ToString();
        }

        #endregion

        #region IDisposable

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources
        /// </summary>
        /// <param name="disposedStatus"><c>true</c> to release both managed and unmanaged resources; <c>false</c> to release only unmanaged resources.</param>
        protected virtual void Dispose(bool disposedStatus)
        {
            if (disposedStatus)
            {
                response = null;
                request = null;
                GC.Collect();
            }
        }

        #endregion
    }
}
