using System;
using System.Diagnostics;
using System.Xml;
using System.Drawing;


namespace Microsoft.eResearch.ScientificAddIn.Common
{
    /// <summary>
    /// Abstraction of an image in Word 2007
    /// </summary>
    public class DocumentImage : ICloneable
    {
        private string imageKey;
        private string rerunAssocName;
        private bool appendixImage;
        private bool isGenePatternLink;

        /// <summary>
        /// Gets or sets a value indicating whether this instance is gene pattern link.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is gene pattern link; otherwise, <c>false</c>.
        /// </value>
        public bool IsGenePatternLink
        {
            get { return isGenePatternLink; }
            set { isGenePatternLink = value; }
        }


        /// <summary>
        /// Initializes a new instance of the <see cref="DocumentImage"/> class.
        /// </summary>
        public DocumentImage()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DocumentImage"/> class.
        /// </summary>
        /// <param name="imageKey">The image key.</param>
        public DocumentImage(string imageKey)
        {
            ParseDocumentImageAttributes(imageKey);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DocumentImage"/> class.
        /// </summary>
        /// <param name="documentImage">The document image.</param>
        protected internal DocumentImage(DocumentImage documentImage)
            : this(documentImage.imageKey)
        {
        }

        /// <summary>
        /// Gets or sets the image key. This is used for identifying the image.
        /// </summary>
        /// <value>The image key.</value>
        public string ImageKey
        {
            get { return imageKey; }
            set { imageKey = value; }
        }

        /// <summary>
        /// Gets or sets the AppendixImage. This is used to distinguish embedded objects(embedded using insert into appendix) from other images.
        /// </summary>
        /// <value>The image key.</value>
        public bool AppendixImage
        {
            get { return appendixImage; }
            set { appendixImage = value; }
        }

        /// <summary>
        /// Determines whether the specified <see cref="T:System.Object"></see> is equal to the current <see cref="T:System.Object"></see>.
        /// </summary>
        /// <param name="obj">The <see cref="T:System.Object"></see> to compare with the current <see cref="T:System.Object"></see>.</param>
        /// <returns>
        /// true if the specified <see cref="T:System.Object"></see> is equal to the current <see cref="T:System.Object"></see>; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            DocumentImage documentImage = obj as DocumentImage;
            if (documentImage == null)
                return false;

            return imageKey == documentImage.imageKey;
        }

        /// <summary>
        /// Serves as a hash function for a particular type. <see cref="M:System.Object.GetHashCode"></see> is suitable for use in hashing algorithms and data structures like a hash table.
        /// </summary>
        /// <returns>
        /// A hash code for the current <see cref="T:System.Object"></see>.
        /// </returns>
        public override int GetHashCode()
        {
            return base.GetHashCode();
        }

        /// <summary>
        /// Returns true if the imageKey already contains a guid
        /// </summary>
        public bool HasGuid
        {
            get
            {
                if (imageKey == null)
                    return false;

                // find first occurrence of "<" and assume everything after that is xml
                int index = imageKey.IndexOf("<");
                if (index != -1)
                {
                    string xml = imageKey.Substring(index);
                    XmlDocument node = new XmlDocument();
                    node.LoadXml(xml);

                    // looking for something like this:  <Guid>[text]</Guid>
                    if (node.DocumentElement.Name == "Guid")
                    {
                        return !string.IsNullOrEmpty(node.InnerText.Trim());
                    }
                }
                return false;
            }

        }

        /// <summary>
        /// Creates a guid so that this alt text is guaranteed to be a unique identifier
        /// </summary>
        public void CreateGuid()
        {
            Debug.Assert(!HasGuid);

            if (imageKey != null)
            {
                int index = imageKey.IndexOf("<");
                // strip off anything that might look like xml
                if (index != -1)
                {
                    imageKey = imageKey.Substring(0, index);
                }
            }
            XmlDocument node = new XmlDocument();
            node.LoadXml("<Guid></Guid>");
            Guid guid = Guid.NewGuid();
            node.DocumentElement.InnerText = guid.ToString();

            imageKey = node.OuterXml;
        }

        /// <summary>
        /// Parses the document image attributes.
        /// </summary>
        /// <param name="alterNativeText">The alter native text.</param>
        private void ParseDocumentImageAttributes(string alterNativeText)
        {
            if (alterNativeText == null)
                return;

            int index = alterNativeText.IndexOf("<");

            if (index != -1)
            {
                string xml = alterNativeText.Substring(index);

                XmlDocument doc = new XmlDocument();
                doc.LoadXml("<items/>");

                XmlDocumentFragment oneNode = doc.CreateDocumentFragment();
                oneNode.InnerXml = xml;
                doc.DocumentElement.AppendChild(oneNode);

                imageKey = doc.FirstChild.FirstChild.OuterXml;

                if (doc.FirstChild.ChildNodes.Count > 1)
                {
                    appendixImage = doc.FirstChild.ChildNodes[1].InnerText == "Y" ? true : false;
                }
            }
        }

        /// <summary>
        /// Gets or sets the name of the rerun assoc.
        /// </summary>
        /// <value>The name of the rerun assoc.</value>
        public string RerunAssocName
        {
            get { return rerunAssocName; }
            set { rerunAssocName = value; }
        }

        #region ICloneable Members

        /// <summary>
        /// Creates a new object that is a copy of the current instance.
        /// </summary>
        /// <returns>
        /// A new object that is a copy of this instance.
        /// </returns>
        public object Clone()
        {
            return new DocumentImage(this);
        }

        #endregion

    }
}
