using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing;
using Microsoft.Office.Interop.Word;

namespace Microsoft.eResearch.ScientificAddIn.Common
{
    /// <summary>
    /// This interface is implemented by the Word Add-in and provides 
    /// components using the interface to react to certain word and for 
    /// accessing the information related to Images in the document
    /// </summary>
    public interface IPackageHost
    {
        #region Events

        /// <summary>
        /// An event that fires when a document is saved by the user. This event is needed
        /// to write in-memory data (representing custom XML parts) to the word document
        /// </summary>
        event EventHandler<DocumentSaveEventArgs> DocumentSave;


        /// <summary>
        /// An event that fires when a document is about to be closed by the user. 
        /// </summary>
        event EventHandler<CancelCloseEventArgs> DocumentClose;

        /// <summary>
        /// An event that fires when a document is opened by the user. 
        /// </summary>
        event EventHandler DocumentOpen;

        /// <summary>
        /// This event is fired whenever the user selects an image in the Word application.
        /// </summary>
        event EventHandler<SelectedImageEventArgs> ImageSelect;

        /// <summary>
        /// This event is fired whenever the user double click an image in the Word application.
        /// </summary>
        event EventHandler<SelectedImageEventArgs> ImageDoubleClicked;

        /// <summary>
        /// Occurs when [image deselect].
        /// </summary>
        event EventHandler ImageDeselect;

        /// <summary>
        /// Occurs when [selection changed].
        /// </summary>
        event EventHandler SelectionChanged;

        /// <summary>
        /// Occurs when [document activated].
        /// </summary>
        event EventHandler DocumentActivated;

        #endregion

        #region Properties

        /// <summary>
        /// Gets the active document key.
        /// </summary>
        /// <value>The active document key.</value>
        string ActiveDocumentKey
        {
            get;
        }

        /// <summary>
        /// Gets a value indicating whether [active document is null].
        /// </summary>
        /// <value>
        /// 	<c>true</c> if [active document is null]; otherwise, <c>false</c>.
        /// </value>
        bool ActiveDocumentIsNull
        {
            get;
        }

        /// <summary>
        /// Gets a value indicating whether [active document key exists].
        /// </summary>
        /// <value>
        /// 	<c>true</c> if [active document key exists]; otherwise, <c>false</c>.
        /// </value>
        bool ActiveDocumentKeyExists
        {
            get;
        }
        #endregion

        #region Methods

        /// <summary>
        /// Gets the selected image in Word
        /// </summary>
        /// <returns>The selected <c>DocumentImage</c></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        DocumentImage GetSelectedImage();

        /// <summary>
        /// returns a thumbnail of the currently selected image
        /// </summary>
        /// <returns></returns>
        Image GetSelectedImageThumb { get;}

        /// <summary>
        /// Gets the image from doc image.
        /// </summary>
        /// <param name="docImage">The doc image.</param>
        /// <returns></returns>
        Image GetImageFromDocImage(DocumentImage docImage);

        /// <summary>
        /// Makes the document dirty.
        /// </summary>
        void MakeDocumentDirty();

        /// <summary>
        /// Insert an image at the current cursor location in the host
        /// </summary>
        /// <param name="image"></param>
        void InsertImage(System.Drawing.Image image);

        /// <summary>
        /// Inserts the picture.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        void InsertPicture(string filePath);

        /// <summary>
        /// Scrolls to image.
        /// </summary>
        /// <param name="documentImage">The document image.</param>
        void ScrollToImage(DocumentImage documentImage);

        /// <summary>
        /// Scrolls to image passed to the function in the window (word doc) corresponding to the
        /// window key passed.
        /// </summary>
        /// <param name="documentImage">The document image.</param>
        /// <param name="documentKey">The document key.</param>
        void ScrollToImage(DocumentImage documentImage, string documentKey);

        /// <summary>
        /// Scrolls to the end of the document
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="content">The content.</param>
        /// <param name="?">The ?.</param>
        /// <param name="filesToEmbed">The files to embed.</param>
        /// <param name="originalImage">The original image.</param>
        /// <param name="previousAssocName">Name of the previous assoc.</param>
        /// <param name="isImage">if set to <c>true</c> [is image].</param>
        void AddAppendixEntry(string title, string content, List<AppendixParameters> appendixParameters, 
            string[] filesToEmbed, DocumentImage originalImage, string previousAssocName, bool isImage);

        /// <summary>
        /// Deletes all appendix objects from the document.
        /// </summary>
        void DeleteAppendixEntry();

        /// <summary>
        /// Gets all images.
        /// </summary>
        /// <value>The get all images.</value>
        /// <returns></returns>
        /// [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        ReadOnlyCollection<DocumentImage> GetAllImages();

        ReadOnlyCollection<DocumentImage> GetAllImages(string documentKey);

        /// <summary>
        /// Deletes the selected image.
        /// </summary>
        void DeleteSelectedImage();

        /// <summary>
        /// Gets a value indicating whether [current document protected].
        /// </summary>
        /// <value>
        /// 	<c>true</c> if [current document protected]; otherwise, <c>false</c>.
        /// </value>
        bool CurrentDocumentProtected { get; }

        /// <summary>
        /// Protects the document.
        /// </summary>
        /// <param name="password">The password.</param>
        /// <returns></returns>
        bool ProtectDocument(string password);

        /// <summary>
        /// Unprotects the document.
        /// </summary>
        /// <param name="password">The password.</param>
        /// <returns></returns>
        bool UnprotectDocument(string password);

        /// <summary>
        /// Models the exists.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns></returns>
        bool ModelExists(string key);

        /// <summary>
        /// Gets or sets value indicating whether this instance is document saved.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is document saved; otherwise, <c>false</c>.
        /// </value>
        bool IsDocumentSaved { get; set; }

        /// <summary>
        /// Sets the document saved.
        /// </summary>
        /// <param name="saved">if set to <c>true</c> [saved].</param>
        /// <param name="documentKey">The document key.</param>
        void SetDocumentSaved(bool saved, string documentKey);

        /// <summary>
        /// Gets the document by document key.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        Document GetDocumentByDocumentKey(string documentKey);

        /// <summary>
        /// Gets the selected image.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <param name="imageRange">The image range.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        DocumentImage GetSelectedImage(string documentKey, ImageRange imageRange);

        /// <summary>
        /// Gets the selected image.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <param name="imageKey">The image key.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        Image GetSelectedImage(string documentKey, string imageKey);

        /// <summary>
        /// Inserts the image.
        /// </summary>
        /// <param name="image">The image.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="range">The range.</param>
        /// <returns></returns>
        ImageRange InsertImage(System.Drawing.Image image, string documentKey, ImageRange range);

        /// <summary>
        /// Gets the range.
        /// </summary>
        /// <value>The get range.</value>
        /// <returns></returns>
        ImageRange GetRange { get;}

        /// <summary>
        /// Gets the document count.
        /// </summary>
        /// <value>The document count.</value>
        int DocumentCount { get; }

        /// <summary>
        /// Updates the name of the rerun result association.
        /// </summary>
        /// <param name="newAssociationName">New name of the association.</param>
        /// <param name="imageKey">The image key.</param>
        void UpdateRerunResultAssociationName(string newAssociationName, string imageKey);

        #endregion
    }
}
