using System;
using System.Windows.Forms;

namespace Microsoft.eResearch.ScientificAddIn.Common
{
    /// <summary>
    /// This interface helps manage the task panels in the word documents. 
    /// </summary>
    public interface ITaskPaneManager
    {
        /// <summary>
        /// The event fires when a custom task pane in Word changes visibility
        /// </summary>
        event EventHandler<VisibilityChangedEventArgs> TaskPaneVisibilityChanged;

        /// <summary>
        /// The event fires when a custom task pane in Word changes visibility
        /// </summary>
        event EventHandler<DockPositionChangedEventArgs> TaskPaneDockPositionChanged;

        /// <summary>
        /// Determines whether the task pane exists or not.
        /// </summary>
        /// <param name="title">The task pane title.</param>
        /// <returns><c>true</c> if the custom task pane is initialized, otherwise <c>false</c></returns>
        bool TaskPaneExists(string title);

        /// <summary>
        /// Creates a custom task pane.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="userControl">The user control.</param>
        void CreateTaskPane(string title, UserControl userControl);

        /// <summary>
        /// Determines whether a task pane is visible or not. A non-existing task pane always yields <c>false</c>.
        /// </summary>
        /// <param name="title">The task pane key.</param>
        /// <returns>
        /// 	<c>true</c> if task pane is visible, otherwise <c>false</c>.
        /// </returns>
        bool IsTaskPaneVisible(string title);

        /// <summary>
        /// Sets the task pane visibility. Creates the task pane if it does not exist.
        /// </summary>
        /// <param name="title">The task pane title.</param>
        /// <param name="isVisible">if set to <c>true</c> task pane is made visible, otherwise hidden.</param>
        void SetTaskPaneVisibility(string title, bool isVisible);

        /// <summary>
        /// Removes the task pane associated with the task pane key.
        /// </summary>
        void RemoveTaskPanes();

        /// <summary>
        /// Removes the task pane.
        /// </summary>
        /// <param name="title">The title.</param>
        void RemoveTaskPane(string title);

        /// <summary>
        /// Gets the task pane user control.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <returns></returns>
        UserControl GetTaskPaneUserControl(string title);

        /// <summary>
        /// Gets the task pane user control.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        UserControl GetTaskPaneUserControl(string title, string documentKey);

        /// <summary>
        /// Determines whether [is task pane visible] [the specified title].
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns>
        /// 	<c>true</c> if [is task pane visible] [the specified title]; otherwise, <c>false</c>.
        /// </returns>
        bool IsTaskPaneVisible(string title, string documentKey);

        /// <summary>
        /// Sets the task pane visibility.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="isVisible">if set to <c>true</c> [is visible].</param>
        /// <param name="documentKey">The document key.</param>
        void SetTaskPaneVisibility(string title, bool isVisible, string documentKey);

        /// <summary>
        /// Tasks the pane exists.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        bool TaskPaneExists(string title, string documentKey);
    }
}
