using System;
using System.Diagnostics;
using System.Globalization;
using Microsoft.eResearch.ScientificAddIn.Common;
using Microsoft.Office.Core;
using Word = Microsoft.Office.Interop.Word;
using System.Xml;
using System.Runtime.InteropServices;

namespace Microsoft.eResearch.ScientificAddIn
{
    /// <summary>
    /// Creates and deletes custom xml parts
    /// </summary>
    class CustomXmlPartDocStorage : IDocumentStorage
    {
        private readonly Word.Application wordApp;
        private static string CATALOG_NAMESPACE = "Microsoft.eResearch.ScientificAddIn.CustomXmlPartDocStorage";
        public CustomXmlPartDocStorage(Word.Application wordApp)
        {
            this.wordApp = wordApp;
        }

        #region IDocumentStorage Members

        /// <summary>
        /// Gets or creates a custom xml part with the given id
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns></returns>
        public CustomXMLPart GetCustomXmlPartById(string id)
        {
            // get catalog with listing of already created parts
            CustomXMLPart catalog = GetCatalog();
            CustomXMLNode root = catalog.DocumentElement;
            string storedGuid = GetStoredGuid(id, root);
            CustomXMLPart newPart = null;
            
            //add a new entry for this id
            if (storedGuid == null)
            {
                newPart = CreateNewPart(id, root);
            }
            else
            {
                // retrieve existing
                newPart = wordApp.ActiveDocument.CustomXMLParts.SelectByID(storedGuid);

                Debug.Assert(newPart != null);
                if (newPart == null) // this shouldn't happen, but make a new part so we don't crash
                {
                    foreach (CustomXMLPart part in wordApp.ActiveDocument.CustomXMLParts)
                    {
                        string guid = part.Id;
                        if (guid == storedGuid)
                        {
                            newPart = part;
                        }
                    }
                    if (newPart == null)
                    {
                        newPart = CreateNewPart(id, root);
                    }
                }
            }

            Marshal.ReleaseComObject(catalog);
            Marshal.ReleaseComObject(root);

            catalog = null;
            root = null;
            return newPart;
        }

        /// <summary>
        /// Gets the custom XML part by id.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="documentKey"></param>
        /// <param name="host">The host.</param>
        /// <returns></returns>
        public CustomXMLPart GetCustomXmlPartById(string id, string documentKey, IPackageHost host)
        {
            // get catalog with listing of already created parts
            Word.Document document = host.GetDocumentByDocumentKey(documentKey);
            CustomXMLPart catalog = GetCatalog(document);
            CustomXMLNode root = catalog.DocumentElement;
            string storedGuid = GetStoredGuid(id, root);
            CustomXMLPart newPart = null;
            
            //add a new entry for this id
            if (storedGuid == null)
            {
                newPart = CreateNewPart(id, root, document);
            }
            else
            {
                // retrieve existing
                newPart = document.CustomXMLParts.SelectByID(storedGuid);
                Debug.Assert(newPart != null);

                if (newPart == null) // this shouldn't happen, but make a new part so we don't crash
                {
                    foreach (CustomXMLPart part in document.CustomXMLParts)
                    {
                        string guid = part.Id;
                        if (guid == storedGuid)
                        {
                            newPart = part;
                        }
                    }
                    if (newPart == null)
                    {
                        newPart = CreateNewPart(id, root, document);
                    }
                }
            }

            Marshal.ReleaseComObject(catalog);
            Marshal.ReleaseComObject(root);
            catalog = null;
            root = null;
            return newPart;
        }

        /// <summary>
        /// Creates the new part.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="catalogRoot">The catalog root.</param>
        /// <param name="document">The document.</param>
        /// <returns></returns>
        private CustomXMLPart CreateNewPart(string id, CustomXMLNode catalogRoot, Word.Document document)
        {
            Object missing = Type.Missing;
            CustomXMLPart newPart = document.CustomXMLParts.Add("<root></root>", missing);
            catalogRoot.AppendChildSubtree("<xmlpart id=\""+id+"\" guid=\"" + newPart.Id + "\" />");
            return newPart;
        }

        /// <summary>
        /// Creates the new part.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="catalogRoot">The catalog root.</param>
        /// <returns></returns>
        private CustomXMLPart CreateNewPart(string id, CustomXMLNode catalogRoot)
        {
            Object missing = Type.Missing;
            CustomXMLPart newPart = wordApp.ActiveDocument.CustomXMLParts.Add("<root></root>", missing);
            catalogRoot.AppendChildSubtree("<xmlpart id=\"" + id + "\" guid=\"" + newPart.Id + "\" />");
            return newPart;
        }

        /// <summary>
        /// Gets the stored GUID.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <param name="root">The root.</param>
        /// <returns></returns>
        private static string GetStoredGuid(string id, CustomXMLNode root)
        {
            CustomXMLNodes nodes = root.SelectNodes("//xmlpart");
            string storedGuid = null;
            CustomXMLNode idNode = null;
           
            foreach(CustomXMLNode node in nodes)
            {
                idNode = node.SelectSingleNode("@id");
                if(idNode.NodeValue == id)
                {
                    storedGuid = node.SelectSingleNode("@guid").NodeValue;
                    break;
                }
            }
            
            if(nodes != null)
                Marshal.ReleaseComObject(nodes);

            if (idNode != null)
                Marshal.ReleaseComObject(idNode);
            
            nodes = null;
            idNode = null;

            return storedGuid;
        }

        /// <summary>
        /// Gets or creates the catalog that stores the mapping of ids to guids for custom xml parts
        /// </summary>
        /// <returns></returns>
        private CustomXMLPart GetCatalog()
        {
            // get or 
            Object missing = Type.Missing;
            CustomXMLParts parts = wordApp.ActiveDocument.CustomXMLParts.SelectByNamespace(CATALOG_NAMESPACE);
            CustomXMLPart catPart;
            
            if (parts.Count == 0) // no one has accessed the catalog yet
            {
                catPart = parts.Add("<catalog xmlns=\""+CATALOG_NAMESPACE+"\"></catalog>",  missing);
            }
            else
            {
                System.Collections.IEnumerator iter = parts.GetEnumerator();
                iter.MoveNext();
                catPart = (CustomXMLPart)iter.Current;
            }

            if (parts != null)
                Marshal.ReleaseComObject(parts);

            return catPart;
        }

        /// <summary>
        /// Gets the catalog.
        /// </summary>
        /// <param name="document">The document.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        private CustomXMLPart GetCatalog(Word.Document document)
        {
            CustomXMLParts parts = document.CustomXMLParts.SelectByNamespace(CATALOG_NAMESPACE);
            CustomXMLPart catPart;

            if (parts.Count == 0) // no one has accessed the catalog yet
            {
                // get or 
                Object missing = Type.Missing;
                catPart = parts.Add("<catalog xmlns=\"" + CATALOG_NAMESPACE + "\"></catalog>", missing);
            }
            else
            {
                System.Collections.IEnumerator iter = parts.GetEnumerator();
                iter.MoveNext();
                catPart = (CustomXMLPart)iter.Current;
            }

            if (parts != null)
                Marshal.ReleaseComObject(parts);

            return catPart;
        }

        /// <summary>
        /// Gets the XML part with the passed guid, or null if it doesn't exist.
        /// </summary>
        /// <param name="guid">The GUID.</param>
        /// <returns></returns>
        public CustomXMLPart GetCustomXmlPartByGuid(Guid guid)
        {
            string strGuid = "{" + guid.ToString().ToUpper(CultureInfo.InvariantCulture) + "}";
            return wordApp.ActiveDocument.CustomXMLParts.SelectByID(strGuid);
        }

        /// <summary>
        /// Gets the XML part with the passed guid, or null if it doesn't exist.
        /// </summary>
        /// <param name="guid">The GUID.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="host">The host.</param>
        /// <returns></returns>
        public CustomXMLPart GetCustomXmlPartByGuid(Guid guid, string documentKey, IPackageHost host)
        {
            string strGuid = "{" + guid.ToString().ToUpper(CultureInfo.InvariantCulture) + "}";
            Word.Document document = host.GetDocumentByDocumentKey(documentKey);
            return document.CustomXMLParts.SelectByID(strGuid);
        }

        /// <summary>
        /// Deletes a customxmlpart and removes it from the catalog
        /// </summary>
        /// <param name="id">The id.</param>
        public void DeleteCustomXmlPartById(string id)
        {
            CustomXMLNode root = GetCatalog().DocumentElement;
            string storedGuid = GetStoredGuid(id, root);

            DeleteCustomXmlPartByGuid(new Guid(storedGuid));

            // remove entry from catalog that references this guid
            CustomXMLNodes nodes = root.SelectNodes("//xmlpart");
            CustomXMLNode idNode;
            foreach (CustomXMLNode node in nodes)
            {
                idNode = node.SelectSingleNode("@id");
                if (idNode.NodeValue == id)
                {
                    root.RemoveChild(node);
                }
            }


            root = null;
            nodes = null;
            idNode = null;
        }

        /// <summary>
        /// Deletes a customxmlpart and removes it from the catalog
        /// </summary>
        /// <param name="guid">The GUID.</param>
        public void DeleteCustomXmlPartByGuid(Guid guid)
        {
            string strGuid = "{" + guid.ToString().ToUpper(CultureInfo.InvariantCulture) + "}";
            CustomXMLPart toDelete = wordApp.ActiveDocument.CustomXMLParts.SelectByID(strGuid);
            if (toDelete != null)
            {
                toDelete.Delete();
            }

            // remove entry from catalog that references this guid
            CustomXMLNode root = GetCatalog().DocumentElement;
            CustomXMLNodes nodes = root.SelectNodes("//xmlpart");
            CustomXMLNode idNode;

            foreach (CustomXMLNode node in nodes)
            {
                idNode = node.SelectSingleNode("@guid");
                if (idNode.NodeValue == strGuid)
                {
                    root.RemoveChild(node);
                }
            }

            root = null;
            nodes = null;
            idNode = null;
            toDelete = null;
        }

        /// <summary>
        /// Remove all data and the catalog.
        /// </summary>
        public void Reset()
        {
            CustomXMLNode root = GetCatalog().DocumentElement;
            CustomXMLNodes nodes = root.SelectNodes("//xmlpart");
            CustomXMLNode guidNode;
            CustomXMLPart part;

            // delete all data
            foreach (CustomXMLNode node in nodes)
            {
                guidNode = node.SelectSingleNode("@guid");
                string strGuid = guidNode.NodeValue;
                part = wordApp.ActiveDocument.CustomXMLParts.SelectByID(strGuid);
                
                if (part != null)
                {
                    part.Delete();
                }
            }

            root = null;
            nodes = null;
            guidNode = null;
            part = null;

            // delete catalog
            GetCatalog().Delete();
        }

        /// <summary>
        /// Sets the next field.
        /// </summary>
        /// <param name="lsid">The lsid.</param>
        /// <param name="next">The next.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="host">The host.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1062:ValidateArgumentsOfPublicMethods")]
        public void SetNextField(string lsid, string next, string documentKey, IPackageHost host)
        {
            Word.Document document;

            if (string.IsNullOrEmpty(documentKey))
                document = wordApp.ActiveDocument;
            else
                document = host.GetDocumentByDocumentKey(documentKey);

            CustomXMLPart catalogNode = GetCatalog(document);
            CustomXMLNode root = catalogNode.DocumentElement;
            CustomXMLNodes xmlNodes = root.SelectNodes("//xmlpart");
            CustomXMLNode attributeNode;

            foreach (CustomXMLNode node in xmlNodes)
            {
                attributeNode = node.SelectSingleNode("@id");
                if (attributeNode.NodeValue == lsid)
                {
                    string xml;

                    if (next != null)
                        xml = "<xmlpart id=\"" + node.SelectSingleNode("@id").NodeValue + "\" guid=\"" + node.SelectSingleNode("@guid").NodeValue + "\" next=\"" + next + "\" />";
                    else
                        xml = "<xmlpart id=\"" + node.SelectSingleNode("@id").NodeValue + "\" guid=\"" + node.SelectSingleNode("@guid").NodeValue + "\" />";

                    root.RemoveChild(node);
                    root.AppendChildSubtree(xml);
                    break;
                }
            }

            Marshal.ReleaseComObject(catalogNode);
            Marshal.ReleaseComObject(root);
            Marshal.ReleaseComObject(xmlNodes);

            catalogNode = null;
            root = null;
            xmlNodes = null;
            attributeNode = null;
        }

        /// <summary>
        /// Gets the next GUID.
        /// </summary>
        /// <param name="guidCurrent">The GUID current.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="host">The host.</param>
        /// <returns></returns>
        public string GetNextGuid(string guidCurrent, string documentKey, IPackageHost host)
        {
            Word.Document document;

            if (string.IsNullOrEmpty(documentKey))
                document = wordApp.ActiveDocument;
            else
                document = host.GetDocumentByDocumentKey(documentKey);

            CustomXMLPart catalogPart = GetCatalog(document);
            CustomXMLNode root = catalogPart.DocumentElement;
            CustomXMLNodes nodes = root.SelectNodes("//xmlpart");

            CustomXMLNode guidNode;
            CustomXMLNode nextNode;
            CustomXMLNode lsidNode;

            foreach (CustomXMLNode node in nodes)
            {
                guidNode = node.SelectSingleNode("@guid");

                // If Current is equal to Guid, then proceed further 
                if (string.Compare(guidNode.NodeValue, guidCurrent, false, CultureInfo.InvariantCulture) == 0)
                {
                    // Get Next Node 
                    nextNode = node.SelectSingleNode("@next");

                    // If Next Node or it's value is null, then return
                    if (nextNode == null || nextNode.NodeValue == null)
                    {
                        return null;
                    }

                    foreach (CustomXMLNode nodeId in nodes)
                    {
                        lsidNode = nodeId.SelectSingleNode("@id");

                        if ((lsidNode != null) && (string.Compare(lsidNode.NodeValue, nextNode.NodeValue, false, CultureInfo.InvariantCulture) == 0))
                        {
                            return nodeId.SelectSingleNode("@guid").NodeValue;
                        }
                    }
                    break;
                }
            }

            Marshal.ReleaseComObject(catalogPart);
            Marshal.ReleaseComObject(root);
            Marshal.ReleaseComObject(nodes);

            catalogPart = null;
            root = null;
            nodes = null;
            guidNode = null;
            nextNode = null;
            lsidNode = null;

            return null;
        }

        /// <summary>
        /// Gets the next GUID.
        /// </summary>
        /// <param name="lsidCurrent">The lsid current.</param>
        /// <returns></returns>
        public string GetNextGuid(string lsidCurrent)
        {

            CustomXMLPart catalogPart = GetCatalog();
            CustomXMLNode root = catalogPart.DocumentElement;
            CustomXMLNodes nodes = root.SelectNodes("//xmlpart");

            CustomXMLNode nextNode;
            CustomXMLNode lsidNode;
            string nextLsid = string.Empty;

            foreach (CustomXMLNode node in nodes)
            {
                lsidNode = node.SelectSingleNode("@id");

                // If Current is equal to Guid, then proceed further 
                if (string.Compare(lsidNode.NodeValue, lsidCurrent, false, CultureInfo.InvariantCulture) == 0)
                {
                    // Get Next Node 
                    
                    nextNode = node.SelectSingleNode("@next");
                    if (nextNode != null)
                        nextLsid = nextNode.NodeValue;

                    break;
                }
            }

            Marshal.ReleaseComObject(catalogPart);
            Marshal.ReleaseComObject(root);
            Marshal.ReleaseComObject(nodes);

            catalogPart = null;
            root = null;
            nodes = null;
            nextNode = null;
            lsidNode = null;

            return nextLsid;
        }

        #endregion
    }
}
