using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows.Forms;
using Microsoft.Office.Core;
using Microsoft.eResearch.ScientificAddIn.Common;
using CustomTaskPane=Microsoft.Office.Tools.CustomTaskPane;

namespace Microsoft.eResearch.ScientificAddIn
{
    /// <summary>
    /// Manages the task panes in the add in
    /// </summary>
    public class TaskPaneManager : ITaskPaneManager, IDisposable
    {
        private bool disposed;
        private readonly IWindowContainer windowContainer;

        /// <summary>
        /// Initializes a new instance of the <see cref="TaskPaneManager"/> class.
        /// </summary>
        /// <param name="windowContainer">The window container.</param>
        public TaskPaneManager(IWindowContainer windowContainer)
        {
            this.windowContainer = windowContainer;
        }

        /// <summary>
        /// Gets the task pane.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <returns></returns>
        private CustomTaskPane GetTaskPane(string title)
        {
            return windowContainer.GetTaskPane(title);
        }

        /// <summary>
        /// Initializes the task pane.
        /// </summary>
        /// <param name="userControl">The user control.</param>
        /// <param name="title">The title.</param>
        /// <returns></returns>
        private CustomTaskPane InitializeTaskPane(UserControl userControl, string title)
        {
            if (title == null) title = "NO TITLE";
            CustomTaskPane taskPane = windowContainer.AddTaskPane(userControl, title);
            taskPane.Width = Properties.Settings.Default.DefaultTaskPaneWidth;
            taskPane.DockPosition = MsoCTPDockPosition.msoCTPDockPositionRight;
            
            taskPane.Visible = false;
            taskPane.VisibleChanged += new EventHandler(taskPane_VisibleChanged);
            taskPane.DockPositionChanged += new EventHandler(taskPane_DockPositionChanged);

            return taskPane;
        }

        /// <summary>
        /// Handles the DockPositionChanged event of the taskPane control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void taskPane_DockPositionChanged(object sender, EventArgs e)
        {
            CustomTaskPane taskPane = sender as CustomTaskPane;
            OnTaskPaneDockPositionChanged(taskPane);
        }

        /// <summary>
        /// Called when [task pane dock position changed].
        /// </summary>
        /// <param name="taskPane">The task pane.</param>
        private void OnTaskPaneDockPositionChanged(CustomTaskPane taskPane)
        {
            if (TaskPaneDockPositionChanged != null)
            {
                TaskPaneDockPosition dockPosition = TaskPaneDockPosition.Floating;
                switch(taskPane.DockPosition)
                {
                    case MsoCTPDockPosition.msoCTPDockPositionBottom:
                        dockPosition = TaskPaneDockPosition.Bottom;
                        break;
                    case MsoCTPDockPosition.msoCTPDockPositionTop:
                        dockPosition = TaskPaneDockPosition.Top;
                        break;
                    case MsoCTPDockPosition.msoCTPDockPositionRight:
                        dockPosition = TaskPaneDockPosition.Right;
                        break;
                    case MsoCTPDockPosition.msoCTPDockPositionLeft:
                        dockPosition = TaskPaneDockPosition.Left;
                        break;
                    case MsoCTPDockPosition.msoCTPDockPositionFloating:
                        dockPosition = TaskPaneDockPosition.Floating;
                        break;
                }
                TaskPaneDockPositionChanged(taskPane, new DockPositionChangedEventArgs(dockPosition, taskPane.Title));
            }
        }

        /// <summary>
        /// Handles the VisibleChanged event of the taskPane control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void taskPane_VisibleChanged(object sender, EventArgs e)
        {
            CustomTaskPane taskPane = sender as CustomTaskPane;
            OnTaskPaneVisibilityChanged(taskPane);
        }

        /// <summary>
        /// Called when [task pane visibility changed].
        /// </summary>
        /// <param name="taskPane">The task pane.</param>
        private void OnTaskPaneVisibilityChanged(CustomTaskPane taskPane)
        {
            if (TaskPaneVisibilityChanged != null)
            {
                TaskPaneVisibilityChanged(taskPane, new VisibilityChangedEventArgs(taskPane.Visible, taskPane.Title));
            }
        }

        #region ITaskPaneManager Members

        /// <summary>
        /// Fires when the visibility of a custom task pane changes
        /// </summary>
        public event EventHandler<VisibilityChangedEventArgs> TaskPaneVisibilityChanged;

        /// <summary>
        /// Fires when the dock position of a custom task pane changes
        /// </summary>
        public event EventHandler<DockPositionChangedEventArgs> TaskPaneDockPositionChanged;

        /// <summary>
        /// Sets the task pane visibility. Creates the task pane if it does not exist.
        /// </summary>
        /// <param name="title">The task pane title.</param>
        /// <param name="isVisible">if set to <c>true</c> task pane is made visible, otherwise hidden.</param>
        public void SetTaskPaneVisibility(string title, bool isVisible)
        {            
            CustomTaskPane pane = GetTaskPane(title);
            if (pane != null)
            {
                pane.Visible = isVisible;
            }
            else
            {
 
                Trace.WriteLine("taskePaneWrapper is null, " + title, TraceLevel.Warning.ToString()); 
            }
        }


        /// <summary>
        /// Sets the task pane visibility.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="isVisible">if set to <c>true</c> [is visible].</param>
        /// <param name="documentKey">The document key.</param>
        public void SetTaskPaneVisibility(string title, bool isVisible, string documentKey)
        {
            CustomTaskPane pane = windowContainer.GetTaskPane(title, documentKey);
            if (pane != null)
            {
                pane.Visible = isVisible;
            }
            else
            {

                Trace.WriteLine("taskePaneWrapper is null, " + title, TraceLevel.Warning.ToString());
            }
        }

        /// <summary>
        /// Creates a custom task pane.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="userControl">The user control.</param>
        public void CreateTaskPane(string title, UserControl userControl)
        {
            if (userControl == null)
                throw new ArgumentNullException("userControl");

            InitializeTaskPane(userControl, title);
        }

        private delegate void RemoveTaskPanesDelegate();

        /// <summary>
        /// Removes the task pane.
        /// </summary>
        public void RemoveTaskPanes()
        {
            //RemoveTaskPanesDelegate removeTaskPanesDelegate = new RemoveTaskPanesDelegate(windowContainer.RemoveTaskPanes);
            //removeTaskPanesDelegate.BeginInvoke(null, null);

            windowContainer.RemoveTaskPanes();            
        }

        /// <summary>
        /// Removes the task pane.
        /// </summary>
        /// <param name="title">The title.</param>
        public void RemoveTaskPane(string title)
        {
            windowContainer.RemoveTaskPane(title);
        }

        /// <summary>
        /// Determines whether [is task pane visible] for [the specified task pane key].
        /// </summary>
        /// <param name="title">The task pane key.</param>
        /// <returns>
        /// 	<c>true</c> if [is task pane visible] for [the specified task pane key]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsTaskPaneVisible(string title)
        {
            bool result = false;

            CustomTaskPane taskPane = GetTaskPane(title);
            if (taskPane != null)
            {
                result = taskPane.Visible;
            }
            return result;
        }


        /// <summary>
        /// Determines whether [is task pane visible] [the specified title].
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        public bool IsTaskPaneVisible(string title, string documentKey)
        {
            bool result = false;
            CustomTaskPane customTaskPane = windowContainer.GetTaskPane(title, documentKey);
            if (customTaskPane != null)
            {
                result = customTaskPane.Visible;
            }
            return result;
        }

        /// <summary>
        /// Gets the task pane user control.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <returns></returns>
        public UserControl GetTaskPaneUserControl(string title)
        {
            CustomTaskPane customTaskPane = GetTaskPane(title);
            if (customTaskPane != null)
                return customTaskPane.Control;
            else
                return null;
        }

        /// <summary>
        /// Gets the task pane user control.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        public UserControl GetTaskPaneUserControl(string title, string documentKey)
        {
            CustomTaskPane customTaskPane = windowContainer.GetTaskPane(title, documentKey);
            if (customTaskPane != null)
                return customTaskPane.Control;
            else
                return null;
        }


        #endregion

        #region IDisposable Members

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Returns true if a task pane with the given title exists
        /// </summary>
        /// <param name="title"></param>
        /// <returns></returns>
        public bool TaskPaneExists(string title)
        {
            bool result = false;
            CustomTaskPane taskPane = windowContainer.GetTaskPane(title);
            if (taskPane != null)
            {
                result = true;    
            }
            return result;
        }

        /// <summary>
        /// Returns true if a task pane with the given title exists
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        public bool TaskPaneExists(string title, string documentKey)
        {
            bool result = false;
            CustomTaskPane taskPane = windowContainer.GetTaskPane(title, documentKey);
            if (taskPane != null)
            {
                result = true;
            }
            return result;
        }

        /// <summary>
        /// Disposes the specified the privately held resources.
        /// </summary>
        /// <param name="disposing">if set to <c>true</c> disposes resources.</param>
        private void Dispose(bool disposing)
        {
            // Check to see if Dispose has already been called.
            if (!this.disposed)
            {
                // If disposing equals true, dispose all managed 
                // and unmanaged resources.
                if (disposing)
                {
                    //foreach (CustomTaskPane customTaskPane in taskPaneDictionary.Values)
                    //{
                    //    customTaskPane.Dispose();
                    //}   
                }
            }
            disposed = true;
        }

        #endregion

    }
}
