using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows.Forms;
using Microsoft.eResearch.ScientificAddIn.Common;
using Microsoft.Office.Core;
using Microsoft.Office.Interop.Word;
using Word = Microsoft.Office.Interop.Word;
using Office = Microsoft.Office.Core;
using System.Reflection;
using CustomTaskPane = Microsoft.Office.Tools.CustomTaskPane;
using Font = Microsoft.Office.Interop.Word.Font;
using Rectangle = System.Drawing.Rectangle;
using Shape = Microsoft.Office.Core.Shape;
using System.Collections;
using System.Xml;
using System.Globalization;

namespace Microsoft.eResearch.ScientificAddIn
{
    /// <summary>
    /// The class is the actual add-in being loaded by Word 2007. It also the gateway
    /// for a IScentificPackage implementation to access functions in Word by abstraction.
    /// </summary>
    public partial class ThisAddIn : IWindowContainer, IDisposable, IPackageHost
    {
        #region Fields

        private bool disposed;

        private Microsoft.Office.Core.IRibbonExtensibility ribbon;
        private TaskPaneManager taskPaneManager;
        private IDocumentStorage docStorage;
        private Dictionary<int, string> windowKeyDictionary = new Dictionary<int, string>();
        private Dictionary<object, string> documentKeyDictionary = new Dictionary<object, string>();
        private static int docCounter = 1;
        private bool imageInCurrentSelection;
        private IList<Guid> currentGuids;

        const string CUSTOM_XML_NAMESPACE = "Microsoft.eResearch.ScientificAddIn";
        private bool windowSelectionChanging; // this is used to keep multiple window selection change events from firing inside each other
        private bool isImageDoubleClicked;
        private bool isFirstTimeAfterDoubleClick;
        private DocumentImage documentImage;

        #endregion

        /// <summary>
        /// Returns an object that extends a feature in the 2007 Microsoft Office system.
        /// </summary>
        /// <param name="serviceGuid">A <see cref="T:System.Guid"></see> that identifies an extensibility interface that is supported by applications in the 2007 Microsoft Office system.</param>
        /// <returns>
        /// An object that implements the extensibility interface that is identified by serviceGuid.
        /// </returns>
        protected override object RequestService(Guid serviceGuid)
        {

            if (serviceGuid == typeof(Microsoft.Office.Core.IRibbonExtensibility).GUID)
            {
                Debug.WriteLine("RequestService for Ribbon");
                if (ribbon == null)
                    ribbon = GetRibbonFromPackage();
                return ribbon;
            }

            return base.RequestService(serviceGuid);
        }

        /// <summary>
        /// Handles the Startup event of the ThisAddIn control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ThisAddIn_Startup(object sender, System.EventArgs e)
        {
            Debug.WriteLine("ThisAddIn_Startup");


            Application.WindowSelectionChange += new ApplicationEvents4_WindowSelectionChangeEventHandler(Application_WindowSelectionChange);
            Application.DocumentBeforeSave += new ApplicationEvents4_DocumentBeforeSaveEventHandler(Application_DocumentBeforeSave);

            Application.DocumentBeforeClose += new ApplicationEvents4_DocumentBeforeCloseEventHandler(Application_DocumentBeforeClose);
            Application.DocumentOpen += new ApplicationEvents4_DocumentOpenEventHandler(Application_DocumentOpen);
            Application.WindowActivate += new ApplicationEvents4_WindowActivateEventHandler(Application_WindowActivate);
            Application.WindowBeforeDoubleClick += new ApplicationEvents4_WindowBeforeDoubleClickEventHandler(Application_WindowBeforeDoubleClick);
        }

        /// <summary>
        /// Application_s the window before double click.
        /// </summary>
        /// <param name="Sel">The sel.</param>
        /// <param name="Cancel">if set to <c>true</c> [cancel].</param>
        void Application_WindowBeforeDoubleClick(Selection Sel, ref bool Cancel)
        {
            try
            {
                isImageDoubleClicked = true;
                documentImage = GetSelectedImage();

                if (documentImage != null)
                {
                    isFirstTimeAfterDoubleClick = OnImageDoubleClick(documentImage);
                }
            }
            catch (Exception e)
            {
                Trace.WriteLine("Excpetion: " + e, TraceLevel.Error.ToString());
            }
            finally
            {
                isImageDoubleClicked = false;
            }
        }

        /// <summary>
        /// Application_s the window activate.
        /// </summary>
        /// <param name="Doc">The doc.</param>
        /// <param name="Wn">The wn.</param>
        void Application_WindowActivate(Document Doc, Window Wn)
        {
            this.OnDocumentActivate();
        }

        /// <summary>
        /// Application_s the document before close.
        /// </summary>
        /// <param name="Doc">The doc.</param>
        /// <param name="Cancel">if set to <c>true</c> [cancel].</param>
        void Application_DocumentBeforeClose(Document Doc, ref bool Cancel)
        {
            this.OnDocumentClose(ref Cancel);
        }

        /// <summary>
        /// Application_s the document open.
        /// </summary>
        /// <param name="Doc">The doc.</param>
        void Application_DocumentOpen(Document Doc)
        {
            this.OnDocumentOpen();
        }

        /// <summary>
        /// Handles the Shutdown event of the ThisAddIn control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ThisAddIn_Shutdown(object sender, System.EventArgs e)
        {
            Debug.WriteLine("ThisAddIn_Shutdown");

            CleanUpActivity();
        }

        /// <summary>
        /// Cleans up activity.
        /// </summary>
        private static void CleanUpActivity()
        {
            #region Delete temporary files from Temporary Internet Folder

            string folderPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.InternetCache),
                                             "Content.Word");
            string[] temporaryfiles = Directory.GetFiles(folderPath, "*.tmp");
            if (temporaryfiles != null)
            {
                foreach (string file in temporaryfiles)
                {
                    try
                    {
                        File.Delete(file);
                    }
                    catch (IOException) { }
                }
            }

            folderPath = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            temporaryfiles = Directory.GetFiles(folderPath, "*.tmp");
            if (temporaryfiles != null)
            {
                foreach (string file in temporaryfiles)
                {
                    try
                    {
                        if (!Directory.Exists(file))
                            File.Delete(file);
                    }
                    catch (IOException) { }
                }
            }

            #endregion
        }

        /// <summary>
        /// Application_s the window selection change.
        /// </summary>
        /// <param name="Sel">The sel.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        void Application_WindowSelectionChange(Selection Sel)
        {
            if (windowSelectionChanging || isImageDoubleClicked)
            {
                return;
            }
            else if (isFirstTimeAfterDoubleClick)
            {
                isFirstTimeAfterDoubleClick = false;
                if (documentImage != null)
                {
                    ScrollToImage(documentImage);
                    SendKeys.Send("%(g)");
                    SendKeys.Send("{ESC} ");
                }
                return;
            }
            
            windowSelectionChanging = true;
            try
            {
                OnSelectionChanged();

                DocumentImage selectedImage = GetSelectedImage();
                if (selectedImage != null)
                {
                    this.OnImageSelect(selectedImage);
                }

                if (imageInCurrentSelection && selectedImage == null)
                {
                    this.OnImageDeSelect();
                }
                imageInCurrentSelection = selectedImage != null;
            }
            catch (Exception e)
            {
                Trace.WriteLine("Excpetion: " + e, TraceLevel.Error.ToString());
                //TODO MessageBox.Show(e.Message);
            }
            finally
            {
                windowSelectionChanging = false;
            }
        }

        /// <summary>
        /// Application_s the document before save.
        /// </summary>
        /// <param name="Doc">The doc.</param>
        /// <param name="SaveAsUI">if set to <c>true</c> [save as UI].</param>
        /// <param name="Cancel">if set to <c>true</c> [cancel].</param>
        void Application_DocumentBeforeSave(Document Doc, ref bool SaveAsUI, ref bool Cancel)
        {
            this.OnDocumentSave(ref Cancel);
        }

        /// <summary>
        /// Gets the ribbon from package.
        /// </summary>
        /// <returns></returns>
        private Microsoft.Office.Core.IRibbonExtensibility GetRibbonFromPackage()
        {
            Microsoft.Office.Core.IRibbonExtensibility tempRibbon = null;

            IScientificPackage package = GetPackage();
            if (package != null)
            {
                tempRibbon = package.GetRibbon() as Microsoft.Office.Core.IRibbonExtensibility;
            }
            else
            {
                Trace.WriteLine("GetRibbonFromPackage: package was null", TraceLevel.Error.ToString());
                Debug.Assert(false);
            }

            return tempRibbon;
        }

        /// <summary>
        /// Instantiates and returns an implementation of a package from configuration
        /// </summary>
        /// <returns>the new instance of a IScientificPackage</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.String.Format(System.String,System.Object,System.Object)"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private IScientificPackage GetPackage()
        {
            IScientificPackage package = null;
            Assembly packageAssembly = null;

            try
            {
                string assembly = Properties.Settings.Default.PlugInAssembly;
                string packageType = Properties.Settings.Default.PlugInType;
                string path = Properties.Settings.Default.PlugInAssemblyPath;

                bool assemblyLoaded = false;
                try
                {
                    packageAssembly = Assembly.Load(assembly);
                    package = packageAssembly.CreateInstance(packageType) as IScientificPackage;
                    assemblyLoaded = true;
                }
                catch (Exception ex)
                {
                    Trace.WriteLine("Unable to assembly " + assembly + ". Exception: " + ex, TraceLevel.Warning.ToString());
                }

                if (!assemblyLoaded)
                {
                    if (string.IsNullOrEmpty(path))
                    {
                        Trace.WriteLine("setting PlugInAssemblyPath was null or empty", TraceLevel.Error.ToString());
                    }
                    // build path to assembly directory 
                    if (!Path.IsPathRooted(path))
                    {
                        string currentDir = Path.GetDirectoryName(Assembly.GetExecutingAssembly().CodeBase);
                        // strip "file:/" from the front of the currentDir
                        currentDir = currentDir.Substring(6);
                        path = Path.Combine(currentDir, path);
                    }

                    if (path.EndsWith(".dll"))
                    {
                        Debug.Assert(System.IO.File.Exists(path), "Assembly path doesn't exist");
                        // it's a single dll
                        packageAssembly = Assembly.LoadFrom(path);
                        packageAssembly.GetTypes();
                        package = packageAssembly.CreateInstance(packageType) as IScientificPackage;
                    }
                    else
                    {
                        Debug.Assert(System.IO.Directory.Exists(path), "Assembly path doesn't exist");
                        // it's a directory
                        string[] files = System.IO.Directory.GetFiles(path, "*.dll");
                        foreach (string file in files)
                        {
                            Assembly asm = Assembly.LoadFrom(file);
                            // look for the class we are supposed to create
                            foreach (Type t in asm.GetTypes())
                            {
                                if (t.ToString() == packageType)
                                {
                                    package = asm.CreateInstance(packageType) as IScientificPackage;
                                }
                            }
                        }
                    }
                }

                if (package != null)
                {
                    // bind interfaces to package
                    taskPaneManager = new TaskPaneManager(this);
                    package.SetTaskPaneManager(taskPaneManager);

                    docStorage = new CustomXmlPartDocStorage(Application);
                    package.SetDocumentStorage(docStorage);

                    package.SetPackageHost(this);
                }
                else
                {
                    Debug.Assert(false, String.Format("Can't create package {0} from any of the .dlls that were loaded from {1}", packageType, path));
                    Trace.WriteLine("Unable to create IScientificPackage from assembly specified by " + path, TraceLevel.Error.ToString());
                }
            }
            catch (Exception e)
            {
                Trace.WriteLine("Exception in GetPackage" + e, TraceLevel.Error.ToString());
                Debug.Assert(false);
            }
            return package;
        }

        /// <summary>
        /// Called when [document save].
        /// </summary>
        private void OnDocumentSave(ref bool Cancel)
        {
            if (this.DocumentSave != null)
            {
                DocumentSaveEventArgs saveEventArgs = new DocumentSaveEventArgs();
                this.DocumentSave(this, saveEventArgs);
                Cancel = saveEventArgs.CancelSaveDialog;
            }
        }

        /// <summary>
        /// Called when [document activate].
        /// </summary>
        private void OnDocumentActivate()
        {
            if (this.DocumentActivated != null)
            {
                this.DocumentActivated(this, EventArgs.Empty);
            }
            // remove our guid and re-add after save?
            //this.ClearCustomGuidFromXmlPart(Application.ActiveDocument);
        }

        /// <summary>
        /// Called when [selection changed].
        /// </summary>
        private void OnSelectionChanged()
        {
            if (this.SelectionChanged != null)
            {
                SelectionChanged(this, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Called when [image select].
        /// </summary>
        /// <param name="documentImage">The document image.</param>
        private void OnImageSelect(DocumentImage documentImage)
        {
            if (this.ImageSelect != null)
            {
                SelectedImageEventArgs eventArgs = new SelectedImageEventArgs(documentImage);
                this.ImageSelect(this, eventArgs);
            }
        }

        /// <summary>
        /// Called when [image de select].
        /// </summary>
        private void OnImageDeSelect()
        {
            if (this.ImageDeselect != null)
            {
                this.ImageDeselect(this, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Called when [image double clicked].
        /// </summary>
        /// <param name="documentImage">The document image.</param>
        /// <returns></returns>
        private bool OnImageDoubleClick(DocumentImage documentImage)
        {
            if (this.ImageDoubleClicked != null)
            {
                SelectedImageEventArgs eventArgs = new SelectedImageEventArgs(documentImage);
                this.ImageDoubleClicked(this, eventArgs);
                return eventArgs.IsTaskPaneVisible;
            }
            return false;
        }

        /// <summary>
        /// Called when [document close].
        /// </summary>
        /// <param name="Cancel">if set to <c>true</c> [cancel].</param>
        private void OnDocumentClose(ref bool Cancel)
        {
            if (this.DocumentClose != null)
            {
                CancelCloseEventArgs args = new CancelCloseEventArgs();
                this.DocumentClose(this, args);
                Cancel = args.Cancel;
            }
        }

        /// <summary>
        /// Called when [document open].
        /// </summary>
        private void OnDocumentOpen()
        {
            // delete document key so we regenerate later
            Word.Document curDoc = Application.ActiveDocument;
            ClearCustomGuidFromXmlPart(curDoc);

            if (this.DocumentOpen != null)
            {
                this.DocumentOpen(this, EventArgs.Empty);
            }
        }

        #region VSTO generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisAddIn_Startup);
            this.Shutdown += new System.EventHandler(ThisAddIn_Shutdown);
        }

        #endregion

        #region IWindowContainer Members

        /// <summary>
        /// Gets the key of the currently active window in Word 2007
        /// </summary>
        /// <value>The current window key.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public string CurrentWindowKey
        {
            get
            {
                string result;
                Window window = null;

                try
                {
                    window = Application.ActiveWindow;
                }
                catch (Exception e)
                {
                    Trace.WriteLine("Exception: " + e, TraceLevel.Error.ToString());
                }

                if (window == null)
                {
                    Trace.WriteLine("There is no active window", TraceLevel.Info.ToString());
                    return null;
                }
                if (windowKeyDictionary.ContainsKey(window.Index))
                {
                    result = windowKeyDictionary[window.Index];
                }
                else
                {
                    result = window.Index.ToString(System.Globalization.CultureInfo.InvariantCulture);
                    windowKeyDictionary.Add(window.Index, result);
                }
                return result;
            }
        }

        /// <summary>
        /// Adds a new task pane to the current active window in Word 2007
        /// </summary>
        /// <param name="userControl">The user control to be displayed in the CustomTaskPane</param>
        /// <param name="title">The title of the CustomTaskPane.</param>
        /// <returns>A wrapper for the new CustomTaskPane</returns>
        public CustomTaskPane AddTaskPane(UserControl userControl, string title)
        {
            CustomTaskPane taskPane = null;

            Window window = Application.ActiveWindow;
            if (window != null)
            {
                taskPane = CustomTaskPanes.Add(userControl, title, window);
            }
            return taskPane;
        }

        /// <summary>
        /// Removes the task panes for the active document/window.
        /// </summary>
        public void RemoveTaskPanes()
        {
            List<CustomTaskPane> toRemove = new List<CustomTaskPane>();

            Window window = null;
            try
            {
                window = Application.ActiveWindow;


                if (window != null)
                {
                    foreach (CustomTaskPane customTaskPane in CustomTaskPanes)
                    {
                        if (customTaskPane.Window == window)
                        {
                            toRemove.Add(customTaskPane);
                        }
                    }
                    foreach (CustomTaskPane customTaskPane in toRemove)
                    {
                        RemoveTaskPane(customTaskPane);
                    }
                }
            }
            catch (COMException)
            {
            }
        }

        /// <summary>
        /// Removes the task pane.
        /// </summary>
        /// <param name="title">The title.</param>
        public void RemoveTaskPane(string title)
        {
            Window window = null;
            try
            {
                window = Application.ActiveWindow;
                if (window != null)
                {
                    foreach (CustomTaskPane customTaskPane in CustomTaskPanes)
                    {
                        if (customTaskPane.Window == window && customTaskPane.Title == title)
                        {
                            RemoveTaskPane(customTaskPane);
                            break;
                        }
                    }
                }
            }
            catch (COMException)
            {
            }
        }

        /// <summary>
        /// Removes the task pane.
        /// </summary>
        /// <param name="customTaskPane">The custom task pane.</param>
        private void RemoveTaskPane(CustomTaskPane customTaskPane)
        {
            UserControl userControl = customTaskPane.Control;
            if (userControl != null)
            {
                if (userControl.InvokeRequired)
                {
                    userControl.Invoke(new DiposeDelegate(this.DisposeUserControl), new object[] { userControl });
                }
            }

            bool wasPaneRemoved = CustomTaskPanes.Remove(customTaskPane);
            if (!wasPaneRemoved)
            {
                Trace.WriteLine("Unable to remove Custom Task Pane : " + customTaskPane.Title, TraceLevel.Warning.ToString());
            }
            else
            {
                Trace.WriteLine("Custom Task Pane removed", TraceLevel.Info.ToString());

            }
        }

        delegate void DiposeDelegate(UserControl userControl);

        /// <summary>
        /// Disposes the user control.
        /// </summary>
        /// <param name="userControl">The user control.</param>
        private void DisposeUserControl(UserControl userControl)
        {
            if (userControl != null && !userControl.IsDisposed)
            {
                userControl.Dispose();
            }
        }

        /// <summary>
        /// Gets task pane with the given title
        /// </summary>
        /// <param name="title"></param>
        /// <returns></returns>
        public CustomTaskPane GetTaskPane(string title)
        {
            CustomTaskPane result = null;
            Document document = null;

            if (Application.Documents.Count > 0)
                document = Application.ActiveDocument;

            if (document != null)
            {
                Window window = document.ActiveWindow;

                if (window != null)
                {
                    foreach (CustomTaskPane customTaskPane in CustomTaskPanes)
                    {
                        if (customTaskPane.Window == window && customTaskPane.Title == title)
                        {
                            result = customTaskPane;
                            break;
                        }
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// Gets the task pane.
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        public CustomTaskPane GetTaskPane(string title, string documentKey)
        {
            CustomTaskPane result = null;
            Document document = GetDocumentByDocumentKey(documentKey);

            if (document != null)
            {
                Window window = document.ActiveWindow;

                if (window != null)
                {
                    foreach (CustomTaskPane customTaskPane in CustomTaskPanes)
                    {
                        if (customTaskPane.Window == window && customTaskPane.Title == title)
                        {
                            result = customTaskPane;
                            break;
                        }
                    }
                }
            }
            return result;
        }

        #endregion

        #region IDisposable Members

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Disposes the specified the privately held resources.
        /// </summary>
        /// <param name="disposing">if set to <c>true</c> disposes resources.</param>
        private void Dispose(bool disposing)
        {
            // Check to see if Dispose has already been called.
            if (!this.disposed)
            {
                // If disposing equals true, dispose all managed 
                // and unmanaged resources.
                if (disposing)
                {
                    CustomTaskPanes.Dispose();
                    taskPaneManager.Dispose();
                }
            }
            disposed = true;
        }

        #endregion

        #region IPackageHost Members

        /// <summary>
        /// Gets a value indicating whether [active document key exists].
        /// </summary>
        /// <value>
        /// 	<c>true</c> if [active document key exists]; otherwise, <c>false</c>.
        /// </value>
        public bool ActiveDocumentKeyExists
        {
            get
            {
                Word.Document curDoc = Application.ActiveDocument;
                if (curDoc != null)
                {
                    return GuidFromCustomXmlPart(curDoc, false) != null;
                }
                return false;
            }
        }
        /// <summary>
        /// Gets the active document key in the Word application.
        /// </summary>
        /// <value>The active document key.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1305:SpecifyIFormatProvider", MessageId = "System.Int32.ToString")]
        public string ActiveDocumentKey
        {
            get
            {
                Word.Document curDoc = Application.ActiveDocument;

                Guid? uniqueKeyNullable = GuidFromCustomXmlPart(curDoc, true);

                if (uniqueKeyNullable == null)
                {
                    throw new Exception("Doc key was null even though we asked for one to be generated!");
                }
                Guid uniqueKey = uniqueKeyNullable.Value;
                if (documentKeyDictionary.ContainsKey(uniqueKey))
                {
                    return documentKeyDictionary[uniqueKey];
                }
                else
                {
                    string docCount = docCounter.ToString();
                    documentKeyDictionary.Add(uniqueKey, docCount);
                    docCounter++;
                    return docCount;
                }
            }
        }

        /// <summary>
        /// Gets a value indicating whether [active document is null].
        /// </summary>
        /// <value>
        /// 	<c>true</c> if [active document is null]; otherwise, <c>false</c>.
        /// </value>
        public bool ActiveDocumentIsNull
        {
            get { return Application.ActiveDocument == null; }
        }

        /// <summary>
        /// Insert an image into the document at the cursor location
        /// </summary>
        /// <param name="image"></param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1303:DoNotPassLiteralsAsLocalizedParameters", MessageId = "Microsoft.Office.Interop.Word.Range.InsertBefore(System.String)"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1303:DoNotPassLiteralsAsLocalizedParameters", MessageId = "System.InvalidOperationException.#ctor(System.String,System.Exception)")]
        public void InsertImage(System.Drawing.Image image)
        {
            if (image == null)
            {
                throw new ArgumentNullException("image");
            }

            // insert a space before the image
            Range rng = Application.Selection.Range;
            rng.Text = " ";
            object endOfDoc = WdUnits.wdCharacter;
            object moveToEnd = WdMovementType.wdMove;

            try
            {
                Clipboard.SetImage(image);
                rng.Paste();
                ClearClipBoard();
                rng.Select();
                foreach (InlineShape pasted in Application.Selection.InlineShapes)
                {
                    pasted.LockAspectRatio = MsoTriState.msoFalse;
                    if (pasted.Width > 400)
                    {
                        float resizeFactor = 400 / pasted.Width;
                        pasted.Width = pasted.Width * resizeFactor;
                    }
                    if (pasted.Height > 650)
                    {
                        float resizeFactor = 650 / pasted.Height;
                        pasted.Height = pasted.Height * resizeFactor;
                    }
                }
                moveToEnd = WdMovementType.wdExtend;
                rng.EndOf(ref endOfDoc, ref moveToEnd);
                rng.Select();
                Thread.Sleep(200); // needed for app not to hang
                rng.InsertBefore(" ");
                
            }
            catch (ExternalException ex)
            {
                Trace.WriteLine("External exception during insert image:" + ex.StackTrace);
                throw new InvalidOperationException("Failure during clipboard operations.", ex);
            }
        }

        /// <summary>
        /// Inserts the picture.
        /// </summary>
        /// <param name="filePath">The file path.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1303:DoNotPassLiteralsAsLocalizedParameters", MessageId = "Microsoft.Office.Interop.Word.Range.InsertBefore(System.String)"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1303:DoNotPassLiteralsAsLocalizedParameters", MessageId = "System.InvalidOperationException.#ctor(System.String,System.Exception)")]
        public void InsertPicture(string filePath)
        {
            // insert a space before the image
            Range rng = Application.Selection.Range;
            rng.InsertBefore(" ");
            object endOfDoc = WdUnits.wdCharacter;
            object moveToEnd = WdMovementType.wdMove;

            try
            {
                object imgRange = rng;
                string imgLocation = filePath;
                object trueField = true;
                object falseFeild = false;

                InlineShape insertedPicture = Application.ActiveDocument.InlineShapes.AddPicture(imgLocation, ref falseFeild, ref trueField, ref imgRange);
                insertedPicture.LockAspectRatio = MsoTriState.msoFalse;

                if (insertedPicture.Width > 375)
                {
                    float resizeFactor = 375 / insertedPicture.Width;
                    insertedPicture.Width = insertedPicture.Width * resizeFactor;
                }
                if (insertedPicture.Height > 575)
                {
                    float resizeFactor = 575 / insertedPicture.Height;
                    insertedPicture.Height = insertedPicture.Height * resizeFactor;
                }

                if (insertedPicture.Width < 25)
                {
                    insertedPicture.Width = 300;
                }

                moveToEnd = WdMovementType.wdExtend;
                rng.EndOf(ref endOfDoc, ref moveToEnd);
                rng.Select();
                Thread.Sleep(200); // needed for app not to hang
               
            }
            catch (ExternalException ex)
            {
                Trace.WriteLine("External exception during insert image:" + ex.StackTrace);
                throw new InvalidOperationException("Failure during clipboard operations.", ex);
            }
            //mark.Delete();
        }

        /// <summary>
        /// Inserts the image.
        /// </summary>
        /// <param name="image">The image.</param>
        /// <param name="documentKey">The document key.</param>
        /// <param name="range">The range.</param>
        /// <returns></returns>
        public ImageRange InsertImage(System.Drawing.Image image, string documentKey, ImageRange range)
        {
            if (image == null)
            {
                throw new ArgumentNullException("image");
            }

            object endOfDoc = WdUnits.wdCharacter;
            object moveToEnd = WdMovementType.wdMove;
            object start = range.Start;
            object end = range.End;

            // insert a space before the image
            Document document = GetDocumentByDocumentKey(documentKey);
            Range rng = document.Range(ref start, ref end);
            rng.Text = " ";
            try
            {
                Clipboard.SetImage(image);
                rng.Paste();
                ClearClipBoard();
                rng.Select();
                foreach (InlineShape pasted in document.InlineShapes)
                {
                    if (!string.IsNullOrEmpty(pasted.AlternativeText))
                    {
                        pasted.LockAspectRatio = MsoTriState.msoFalse;
                        if (pasted.Width > 400)
                        {
                            float resizeFactor = 400 / pasted.Width;
                            pasted.Width = pasted.Width * resizeFactor;
                        }
                        if (pasted.Height > 650)
                        {
                            float resizeFactor = 650 / pasted.Height;
                            pasted.Height = pasted.Height * resizeFactor;
                        }
                    }
                }
                moveToEnd = WdMovementType.wdExtend;
                rng.EndOf(ref endOfDoc, ref moveToEnd);
                rng.Select();
                Thread.Sleep(200); // needed for app not to hang
                rng.InsertBefore(" ");
                return new ImageRange(rng.Start + 1, rng.End);
            }
            catch (ExternalException ex)
            {
                Trace.WriteLine("External exception during insert image:" + ex.StackTrace);
                throw new InvalidOperationException("Failure during clipboard operations.", ex);
            }
        }

        private Guid? GuidFromCustomXmlPart(Document curDoc, bool createIfMissing)
        {
            try
            {
                CustomXMLParts namespaceParts = curDoc.CustomXMLParts.SelectByNamespace(CUSTOM_XML_NAMESPACE);
                if (namespaceParts.Count == 0 && createIfMissing) // have never queried this document, make a new guid
                {
                    Guid newKey = Guid.NewGuid();
                    namespaceParts.Add("<documentKey xmlns=\"" + CUSTOM_XML_NAMESPACE + "\">" + newKey + "</documentKey>",
                                       missing);
                    return newKey;
                }
                else if (namespaceParts.Count > 0) // retrieve existing guid
                {
                    System.Collections.IEnumerator iter = namespaceParts.GetEnumerator();
                    iter.MoveNext();
                    CustomXMLPart part = (CustomXMLPart)iter.Current;
                    Guid result = new Guid(part.DocumentElement.Text);
                    return result;
                }
                else
                {
                    return null;
                }
            }
            catch (COMException)
            {
                return null;
            }
        }

        // clear the guid from the document so it has to be regenerated, only if it will conflict with a current guid
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        private void ClearCustomGuidFromXmlPart(Document curDoc)
        {
            // get the guids of other open docs 
            currentGuids = new List<Guid>();
            foreach (Document d in Application.Documents)
            {
                if (d != Application.ActiveDocument)
                {
                    Guid? docGuid = GuidFromCustomXmlPart(d, false);
                    if (docGuid != null)
                    {
                        currentGuids.Add(docGuid.Value);
                    }
                }
            }
            Guid? myGuid = GuidFromCustomXmlPart(Application.ActiveDocument, false);
            // if my guid conflicts with a known document, delete mine so it gets created later
            if (myGuid != null && currentGuids.Contains(myGuid.Value))
            {
                CustomXMLParts namespaceParts = curDoc.CustomXMLParts.SelectByNamespace(CUSTOM_XML_NAMESPACE);
                if (namespaceParts.Count > 0)
                {
                    foreach (CustomXMLPart part in namespaceParts)
                    {
                        part.Delete();
                    }
                }
            }
        }

        /// <summary>
        /// Event raised before user saves the document
        /// </summary>
        public event EventHandler<DocumentSaveEventArgs> DocumentSave;

        /// <summary>
        /// Occurs when [document activated].
        /// </summary>
        public event EventHandler DocumentActivated;

        /// <summary>
        /// Event raised before user closes the document
        /// </summary>
        public event EventHandler<CancelCloseEventArgs> DocumentClose;

        /// <summary>
        /// Event raised after user opens a document
        /// </summary>
        public event EventHandler DocumentOpen;

        /// <summary>
        /// Event raised when an Image is selected by the user
        /// </summary>
        public event EventHandler<SelectedImageEventArgs> ImageSelect;

        /// <summary>
        /// Event raised when an Image is doubled clicked by user
        /// </summary>
        public event EventHandler<SelectedImageEventArgs> ImageDoubleClicked;

        /// <summary>
        /// Event raised when [image deselect].
        /// </summary>
        public event EventHandler ImageDeselect;

        /// <summary>
        /// Event raised when [selection changed].
        /// </summary>
        public event EventHandler SelectionChanged;

        /// <summary>
        /// Gets the selected image in Word
        /// </summary>
        /// <returns>The selected <c>DocumentImage</c></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public DocumentImage GetSelectedImage()
        {
            Selection Sel = Application.Selection;

            try
            {
                // ways to trl: alternative text?
                if (Sel.InlineShapes.Count > 0)
                {
                    foreach (InlineShape inlineShape in Sel.InlineShapes)
                    {
                        DocumentImage docImage = CreateDocImage(inlineShape);
                        return docImage;
                    }
                    return null;
                }
                else if (Sel.ShapeRange.Count > 0)
                {
                    foreach (Word.Shape shape in Sel.ShapeRange)
                    {
                        DocumentImage docImage = CreateDocImage(shape);
                        return docImage;
                    }
                    return null;
                }
                else
                {
                    return null;
                }
            }
            catch (Exception e)
            {
                Trace.WriteLine("Exception: " + e, TraceLevel.Error.ToString());
                //TODO MessageBox.Show(e.Message);
                return null;
            }
        }

        /// <summary>
        /// Gets the selected image.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <param name="imageRange">The image range.</param>
        /// <returns></returns>
        public DocumentImage GetSelectedImage(string documentKey, ImageRange imageRange)
        {
            Document document = GetDocumentByDocumentKey(documentKey);

            try
            {
                // ways to trl: alternative text?
                if (document.InlineShapes.Count > 0)
                {
                    foreach (InlineShape inlineShape in document.InlineShapes)
                    {
                        if (inlineShape.Range.Start == imageRange.Start &&
                            inlineShape.Range.End == imageRange.End)
                        {
                            DocumentImage docImage = CreateDocImage(inlineShape);
                            return docImage;
                        }
                       
                    }

                    foreach (InlineShape inlineShape in document.InlineShapes)
                    {
                        if (inlineShape.Range.End == imageRange.End)
                        {
                            DocumentImage docImage = CreateDocImage(inlineShape);
                            return docImage;
                        }
                    }

                    foreach (InlineShape inlineShape in document.InlineShapes)
                    {
                        if (inlineShape.Range.Start == imageRange.Start)
                        {
                            DocumentImage docImage = CreateDocImage(inlineShape);
                            return docImage;
                        }
                    }
                    return null;
                }
                else if (document.Shapes.Count > 0)
                {
                    foreach (Word.Shape shape in document.Shapes)
                    {
                        if (string.IsNullOrEmpty(shape.AlternativeText))
                        {
                            DocumentImage docImage = CreateDocImage(shape, documentKey);
                            return docImage;
                        }
                    }
                    return null;
                }
                else
                {
                    return null;
                }
            }
            catch (Exception e)
            {
                Trace.WriteLine("Exception: " + e, TraceLevel.Error.ToString());
                //TODO MessageBox.Show(e.Message);
                return null;
            }
        }

        /// <summary>
        /// Gets the selected image.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <param name="imageKey">The image key.</param>
        /// <returns></returns>
        public Image GetSelectedImage(string documentKey, string imageKey)
        {
            Document document = GetDocumentByDocumentKey(documentKey);

            try
            {
                DocumentImage docImage;
                foreach (InlineShape inlineShape in document.InlineShapes)
                {
                    docImage = CreateDocImage(inlineShape);
                    if (docImage != null)
                    {
                        if (string.Compare(docImage.ImageKey, imageKey, true) == 0)
                        {
                            return GetImageFromInlineShape(inlineShape);
                        }
                    }
                }

                foreach (Word.Shape shape in document.Shapes)
                {
                    docImage = CreateDocImage(shape);
                    if (docImage != null)
                    {
                        if (string.Compare(docImage.ImageKey, imageKey, true) == 0)
                        {
                            return GetImageFromShape(shape);
                        }
                    }
                }
                return null;
            }
            catch (Exception e)
            {
                Trace.WriteLine("Exception: " + e, TraceLevel.Error.ToString());
                //TODO MessageBox.Show(e.Message);
                return null;
            }
        }

        /// <summary>
        /// Gets the get selected image thumb.
        /// </summary>
        /// <value>The get selected image thumb.</value>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public Image GetSelectedImageThumb
        {
            get
            {
                Selection Sel = Application.Selection;
                try
                {
                    Image img = null;
                    // ways to track inline shapes: alternative text?
                    if (Sel.InlineShapes.Count > 0)
                    {

                        foreach (InlineShape inlineShape in Sel.InlineShapes)
                        {
                            if (inlineShape.Type == WdInlineShapeType.wdInlineShapePicture)
                            {
                                img = GetImageFromInlineShape(inlineShape);
                            }
                        }
                    }
                    else if (Sel.ShapeRange.Count > 0)
                    {
                        foreach (Word.Shape shape in Sel.ShapeRange)
                        {
                            if (shape.Type == MsoShapeType.msoPicture)
                            {
                                img = GetImageFromShape(shape);
                            }
                        }
                    }
                    if (img != null)
                    {
                        Image.GetThumbnailImageAbort dummyCallBack =
                            new System.Drawing.Image.GetThumbnailImageAbort(ThumbnailCallback);

                        int thumbSize = 64;
                        Image thumbnail = GetProportionedThumbnail(img, thumbSize, dummyCallBack);


                        return thumbnail;
                    }
                    else
                    {
                        return null;
                    }
                }
                catch (Exception e)
                {
                    Trace.WriteLine("Exception: " + e, TraceLevel.Error.ToString());
                    //TODO MessageBox.Show(e.Message);
                    return null;
                }
            }
        }

        /// <summary>
        /// Gets the image from shape.
        /// </summary>
        /// <param name="shape">The shape.</param>
        /// <returns></returns>
        private Image GetImageFromShape(Word.Shape shape)
        {
            //Image img;
            //Object dummy = new Object();
            //shape.Select(ref dummy);
            //Application.Selection.CopyAsPicture();
            //img = Clipboard.GetImage();
            //Clipboard.Clear();
            //return img;
            return null;
        }

        /// <summary>
        /// Gets the image from inline shape.
        /// </summary>
        /// <param name="inlineShape">The inline shape.</param>
        /// <returns></returns>
        private Image GetImageFromInlineShape(InlineShape inlineShape)
        {

            bool prevWindowSelectionStatus = windowSelectionChanging;
            windowSelectionChanging = true;// && isImageDoubleClicked == false;
            Image img = null;
            try
            {
                if (!CurrentDocumentProtected)
                {
                    try
                    {
                        inlineShape.Range.CopyAsPicture();
                        if (Clipboard.ContainsImage())
                        {
                            img = Clipboard.GetImage();
                            ClearClipBoard();
                        }
                    }
                    catch (COMException) { }
                    catch (ExternalException ex)
                    {
                        Trace.WriteLine("Exception: " + ex, TraceLevel.Error.ToString());
                    }
                }
            }
            catch (Exception)
            {
                throw;
            }
            finally
            {
                if (windowSelectionChanging)
                {
                    windowSelectionChanging = prevWindowSelectionStatus;
                }
            }
            return img;
        }

        private static void ClearClipBoard()
        {
            try
            {
                Clipboard.Clear();
            }
            catch (ExternalException ex) 
            {
                Trace.WriteLine("Exception: " + ex, TraceLevel.Error.ToString());
            }
        }

        /// <summary>
        /// Gets the proportioned thumbnail.
        /// </summary>
        /// <param name="img">The img.</param>
        /// <param name="maxPixels">The max pixels.</param>
        /// <param name="ThumbnailCallback">The thumbnail callback.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        private Image GetProportionedThumbnail(Image img, int maxPixels, Image.GetThumbnailImageAbort ThumbnailCallback)
        {
            float scaleFactor = 1;
            if (img.Width > img.Height)
            {
                if (img.Width > maxPixels)
                {
                    scaleFactor = (float)maxPixels / (float)img.Width;
                }
            }
            else
            {
                if (img.Height > maxPixels)
                {
                    scaleFactor = (float)maxPixels / (float)img.Height;
                }
            }
            int newH = (int)(scaleFactor * img.Height);
            int newW = (int)(scaleFactor * img.Width);


            if (img.Width > img.Height && img.Width > maxPixels)
            {
                newW = maxPixels; // can't trust floating point stuff, something newH is maxPixels-1
            }
            else if (img.Height > maxPixels)
            {
                newH = maxPixels;
            }

            Image thumbnailImg = img.GetThumbnailImage(newW, newH, ThumbnailCallback, IntPtr.Zero);
            Bitmap thumbnail = new Bitmap(thumbnailImg);
            int thumbSize = maxPixels;
            // if it's not square, we need to add some whitespace padding since
            // the display will stretch it into a square

            Bitmap paddedThumb = new Bitmap(thumbSize, thumbSize);
            using (Graphics gr = Graphics.FromImage(paddedThumb))
            {
                gr.FillRectangle(Brushes.White, new Rectangle(0, 0, maxPixels, maxPixels));
                int x = (int)(((float)maxPixels / 2) - ((float)thumbnail.Width / 2));
                int y = (int)(((float)maxPixels / 2) - ((float)thumbnail.Height / 2));
                gr.DrawImage(thumbnail, x, y, thumbnail.Width, thumbnail.Height);
            }
            return paddedThumb;
        }

        /// <summary>
        /// Scrolls to heading.
        /// </summary>
        /// <param name="windowKey">The window key.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId = "windowKey")]
        public void ScrollToHeading(string windowKey)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Creates the heading.
        /// </summary>
        /// <param name="text">The text.</param>
        /// <param name="styleName">Name of the style.</param>
        /// <param name="windowKey">The window key.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId = "windowKey"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId = "text"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId = "styleName")]
        public void CreateHeading(string text, string styleName, string windowKey)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Scrolls to image passed to the function in the window (word doc) corresponding to the
        /// window key passed.
        /// </summary>
        /// <param name="documentImage">The document image.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1062:ValidateArgumentsOfPublicMethods")]
        public void ScrollToImage(DocumentImage documentImage)
        {

            foreach (InlineShape inlineShape in Application.ActiveDocument.InlineShapes)
            {
                if (documentImage.Equals(CreateDocImage(inlineShape)))
                {
                    inlineShape.Select();
                    return;
                }
            }
            foreach (Word.Shape shape in Application.ActiveDocument.Shapes)
            {
                if (documentImage.Equals(CreateDocImage(shape)))
                {
                    object dummy = new object();
                    shape.Select(ref dummy);
                    return;
                }
            }
        }

        /// <summary>
        /// Scrolls to image passed to the function in the window (word doc) corresponding to the
        /// window key passed.
        /// </summary>
        /// <param name="documentImage">The document image.</param>
        /// <param name="documentKey">The document key.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1062:ValidateArgumentsOfPublicMethods")]
        public void ScrollToImage(DocumentImage documentImage, string documentKey)
        {
            Document document = GetDocumentByDocumentKey(documentKey);

            foreach (InlineShape inlineShape in document.InlineShapes)
            {
                if (documentImage.Equals(CreateDocImage(inlineShape)))
                {
                    inlineShape.Select();
                    imageInCurrentSelection = true;
                    return;
                }
            }
            foreach (Word.Shape shape in document.Shapes)
            {
                if (documentImage.Equals(CreateDocImage(shape)))
                {
                    object dummy = new object();
                    shape.Select(ref dummy);
                    imageInCurrentSelection = true;
                    return;
                }
            }
        }

        /// <summary>
        /// Scrolls to the end of the document
        /// </summary>
        /// <param name="title">The title.</param>
        /// <param name="content">The content.</param>
        /// <param name="appendixParameters"></param>
        /// <param name="filesToEmbed">The files to embed.</param>
        /// <param name="originalImage">The original image.</param>
        /// <param name="previousAssocName">Name of the previous assoc.</param>
        /// <param name="isImage">if set to <c>true</c> [is image].</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1303:DoNotPassLiteralsAsLocalizedParameters", MessageId = "Microsoft.Office.Interop.Word.Selection.TypeText(System.String)"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1062:ValidateArgumentsOfPublicMethods"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
        public void AddAppendixEntry(string title, string content, List<AppendixParameters> appendixParameters,
            string[] filesToEmbed, DocumentImage originalImage, string previousAssocName, bool isImage)
        {
            Range range = null;

            if (CurrentDocumentProtected)
            {
                MoveToStartOfDoc();

                object editorId = Word.WdEditorType.wdEditorEveryone;
                Range nextRange = null;
                int rangeTrials = 10; // this seems to infinite loop, so bail out until we can fix it better...PETER
                do
                {

                    nextRange = Application.Selection.GoToEditableRange(ref editorId);
                    if (nextRange != null)
                    {
                        range = nextRange;
                        rangeTrials--;
                    }
                }
                while (nextRange != null && rangeTrials > 0);

                if (range == null)
                    throw new Exception("No unprotected section found");

                range.Select();
            }
            else
            {
                MoveToEndOfDoc();
            }
            MoveAppSelectionToItsEnd();
            range = Application.Selection.Range;
            if (string.IsNullOrEmpty(range.Text))
            {
                range.Text = "\n";
                range.Select();
            }
            MoveAppSelectionToItsEnd();
            Application.Selection.TypeText("\n");

            bool hasParameters = appendixParameters.Count > 0;

            #region OleObject Parameters & Alternative Text

            object trueObject = true;
            object falseObject = false;

            object package = "Package";
            object index = missing;
            object iconFilename = missing;

            string alternativeText = "<AppendixObject>Y</AppendixObject>";

            #endregion

            #region Add Appendix Title

            Application.Selection.Text = title;
            Application.Selection.Range.Font.Size = 12;
            Application.Selection.Range.Bold = 1;
            Application.Selection.ParagraphFormat.SpaceAfter = 1;
            MoveAppSelectionToItsEnd();
            MoveToEndOfDoc();

            #endregion

            #region Add Time Stamp

            Application.Selection.Text = string.Format("Added at {0}", DateTime.Now);
            Application.Selection.Range.Underline = WdUnderline.wdUnderlineNone;
            Application.Selection.ParagraphFormat.SpaceAfter = 1;
            MoveToEndOfDoc();
            Application.Selection.TypeText("\n");
            Application.Selection.ParagraphFormat.SpaceAfter = 1;
            MoveToEndOfDoc();

            #endregion

            #region Add Comment
           
            if (!string.IsNullOrEmpty(content))
            {
                Application.Selection.TypeText("\n");
                Application.Selection.Text =  content;
                Application.Selection.Range.Underline = WdUnderline.wdUnderlineNone;
                Application.Selection.ParagraphFormat.SpaceAfter = 1;
                MoveToEndOfDoc();
                Application.Selection.TypeText("\n");
                Application.Selection.ParagraphFormat.SpaceAfter = 1;
                MoveToEndOfDoc();
            }

            #endregion

            #region Add parameters if they exist

            if (hasParameters)
            {
                Application.Selection.TypeText("\n");
                Application.Selection.Text = "Pipeline parameters:";
                Application.Selection.Range.Underline = WdUnderline.wdUnderlineSingle;
                Application.Selection.ParagraphFormat.SpaceAfter = 1;
                MoveToEndOfDoc();

                bool isInputFilePresent = false;
                foreach (AppendixParameters appendixParameter in appendixParameters)
                {
                    Application.Selection.TypeText("\n");
                    Application.Selection.Text = string.Format("{0}: {1}", appendixParameter.Name, appendixParameter.Value);
                    Application.Selection.Range.Underline = WdUnderline.wdUnderlineNone;
                    Application.Selection.Range.Font.Size = 11;
                    Application.Selection.ParagraphFormat.SpaceAfter = 1;
                    MoveToEndOfDoc();
                    Application.Selection.TypeText("\n");
                    MoveToEndOfDoc();

                    if (appendixParameter.IsInputFile)
                    {
                        isInputFilePresent = true;
                        object filename = appendixParameter.FilePath;
                        object tag = Path.GetFileName(appendixParameter.Value);
                        InlineShape shape = Application.Selection.Range.InlineShapes.AddOLEObject(ref package, ref filename, ref falseObject,
                                                                     ref trueObject,
                                                                     ref iconFilename, ref index, ref tag, ref missing);
                        shape.AlternativeText = CreateNewGuidXml() + alternativeText;

                        #region Delete Temporary File

                        if (appendixParameter.IsDocumentFile)
                        {
                            try
                            {
                                File.Delete(appendixParameter.FilePath);
                            }
                            catch (IOException) { }
                        }

                        #endregion
                    }
                }

                if (isInputFilePresent)
                {
                    Application.Selection.TypeText("\n");
                }
                Application.Selection.TypeText("\n");
                MoveToEndOfDoc();
            }

            #endregion

            #region Add results

            if (!hasParameters)
            {
                Application.Selection.TypeText("\n");
            }
            Application.Selection.Text = "Inserted results:";
            Application.Selection.Range.Font.Size = 11;
            Application.Selection.Range.Underline = WdUnderline.wdUnderlineSingle;
            MoveAppSelectionToItsEnd();
            Application.Selection.TypeText("\n");
            Application.Selection.Range.Underline = WdUnderline.wdUnderlineNone;

            if (filesToEmbed != null && (!isImage || filesToEmbed.Length > 1))
            {
                
                foreach (string file in filesToEmbed)
                {
                    // jump through hoops for word api
                    object filename = file;
                    object tag = Path.GetFileName(file);
                    InlineShape shape = Application.Selection.Range.InlineShapes.AddOLEObject(ref package, ref filename, ref falseObject,
                                                                     ref trueObject,
                                                                     ref iconFilename, ref index, ref tag, ref missing);

                    shape.AlternativeText = CreateNewGuidXml() + alternativeText;
                    shape.Select();
                }

                //If the user choose to insert image result, then remove the 
                //focus from the last inserted icon, so that insertion of 
                //image will not overwrite any of the icon.
                if (isImage)
                {
                    MoveAppSelectionToItsEnd();
                }
            }

            #endregion
        }

        /// <summary>
        /// Creates the GUID.
        /// </summary>
        private static string CreateNewGuidXml()
        {
            Guid guid = Guid.NewGuid();
            return string.Format("<Guid>{0}</Guid>", guid.ToString());
        }

        /// <summary>
        /// //Deletes all the Genepattern related embedded objects/images.
        /// </summary>
        public void DeleteAppendixEntry()
        {
            foreach (InlineShape inlineShape in Application.ActiveDocument.InlineShapes)
            {
                if (inlineShape.AlternativeText.Contains("Y"))
                {
                    inlineShape.Delete();
                }
            }
        }

        /// <summary>
        /// Makes the document dirty.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public void MakeDocumentDirty()
        {
            Selection Sel = Application.Selection;

            try
            {
                // ways to track inline shapes: alternative text?
                if (Sel.InlineShapes.Count > 0)
                {
                    foreach (InlineShape inlineShape in Sel.InlineShapes)
                    {
                        string prevName = inlineShape.AlternativeText;
                        inlineShape.AlternativeText = "@";
                        inlineShape.AlternativeText = prevName;
                    }

                }
                else if (Sel.ShapeRange.Count > 0)
                {
                    foreach (Word.Shape shape in Sel.ShapeRange)
                    {
                        string prevName = shape.Name;
                        shape.Name = "@";
                        shape.Name = prevName;
                    }
                }
            }
            catch (Exception e)
            {
                Trace.WriteLine("Exception: " + e, TraceLevel.Error.ToString());
            }
        }

        /// <summary>
        /// Moves the app selection to its end.
        /// </summary>
        private void MoveAppSelectionToItsEnd()
        {
            object unit = WdUnits.wdParagraph;
            int no = Application.Selection.Paragraphs.Count;
            if (no > 0)
            {
                object noObject = no - 1;
                Application.Selection.MoveStart(ref unit, ref noObject);
            }
            unit = WdUnits.wdLine;

            object extend = WdMovementType.wdMove;
            Application.Selection.EndOf(ref unit, ref extend);
            //unit = WdUnits.wdTable;
            //Application.Selection.EndOf(ref unit, ref extend);
            //Application.Selection.TypeText("x");
        }

        /// <summary>
        /// Gets all images.
        /// </summary>
        /// <returns></returns>
        public System.Collections.ObjectModel.ReadOnlyCollection<DocumentImage> GetAllImages()
        {
            List<DocumentImage> images = new List<DocumentImage>();
            foreach (InlineShape inlineShape in Application.ActiveDocument.InlineShapes)
            {
                images.Add(CreateDocImage(inlineShape));
            }
            foreach (Word.Shape shape in Application.ActiveDocument.Shapes)
            {
                images.Add(CreateDocImage(shape));
            }
            return new System.Collections.ObjectModel.ReadOnlyCollection<DocumentImage>(images);
        }

        /// <summary>
        /// Gets all images.
        /// </summary>
        /// <returns></returns>
        public System.Collections.ObjectModel.ReadOnlyCollection<DocumentImage> GetAllImages(string documentKey)
        {
            Document document = GetDocumentByDocumentKey(documentKey);

            List<DocumentImage> images = new List<DocumentImage>();
            foreach (InlineShape inlineShape in document.InlineShapes)
            {
                images.Add(CreateDocImage(inlineShape));
            }
            foreach (Word.Shape shape in document.Shapes)
            {
                images.Add(CreateDocImage(shape));
            }
            return new System.Collections.ObjectModel.ReadOnlyCollection<DocumentImage>(images);
        }

        /// <summary>
        /// Updates the name of the rerun result association.
        /// </summary>
        /// <param name="newAssociationName">New name of the association.</param>
        /// <param name="imageKey">The image key.</param>
        public void UpdateRerunResultAssociationName(string newAssociationName, string imageKey)
        {
            DocumentImage image;
            InlineShape shape;
            string rerunAssociationName = string.Format("<RerunAssocName>{0}</RerunAssocName>", newAssociationName);
            string objectType = "<AppendixObject>Y</AppendixObject>";
            string alternativeText = imageKey + rerunAssociationName + objectType;

            for (int i = 1; i <= Application.ActiveDocument.InlineShapes.Count; i++)
            {
                shape = Application.ActiveDocument.InlineShapes[i];

                if (shape != null)
                {
                    if ((shape.AlternativeText != null) && (shape.AlternativeText.Contains("RerunAssocName")))
                    {
                        image = CreateDocImage(shape);
                        if (image != null)
                        {
                            if (image.ImageKey == imageKey)
                            {
                                shape.AlternativeText = alternativeText;
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Deletes the currently selected Image, if there is one.
        /// </summary>
        public void DeleteSelectedImage()
        {
            //Debug.Assert(Application.Selection.InlineShapes.Count+Application.Selection.ShapeRange.Count == 1);

            // using foreach loops because they are less hassle than iterators.
            foreach (InlineShape shape in Application.Selection.InlineShapes)
            {
                shape.Delete();
            }
            foreach (Word.Shape shape in Application.Selection.ShapeRange)
            {
                shape.Delete();
            }
        }

        #endregion

        /// <summary>
        /// Creates the doc image.
        /// </summary>
        /// <param name="inlineShape">The inline shape.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        private DocumentImage CreateDocImage(InlineShape inlineShape)
        {
            DocumentImage docImage = new DocumentImage(inlineShape.AlternativeText);
            if (!docImage.HasGuid)
            {
                docImage.CreateGuid();
                //update the alt text with our guid
                inlineShape.AlternativeText = docImage.ImageKey;
            }
            return docImage;
        }

        /// <summary>
        /// Creates the doc image.
        /// </summary>
        /// <param name="shape">The shape.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        private DocumentImage CreateDocImage(Word.Shape shape)
        {
            bool saved = Application.ActiveDocument.Saved;
            DocumentImage docImage = new DocumentImage(shape.AlternativeText);
            if (!docImage.HasGuid)
            {
                docImage.CreateGuid();
                //update the alt text with our guid
                shape.AlternativeText = docImage.ImageKey;
            }
            Application.ActiveDocument.Saved = saved;
            return docImage;
        }

        /// <summary>
        /// Creates the doc image.
        /// </summary>
        /// <param name="shape">The shape.</param>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1822:MarkMembersAsStatic")]
        private DocumentImage CreateDocImage(Word.Shape shape, string documentKey)
        {
            bool saved = GetDocumentByDocumentKey(documentKey).Saved;
            DocumentImage docImage = new DocumentImage(shape.AlternativeText);
            if (!docImage.HasGuid)
            {
                docImage.CreateGuid();
                //update the alt text with our guid
                shape.AlternativeText = docImage.ImageKey;
            }
            SetDocumentSaved(saved, documentKey);
            return docImage;
        }

        /// <summary>
        /// Dummy method used in thumbnail creation
        /// </summary>
        /// <returns>True if the thumbnail should be canceled</returns>
        public bool ThumbnailCallback()
        {
            return false;
        }

        /// <summary>
        /// Gets the image from doc image.
        /// </summary>
        /// <param name="docImage">The doc image.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1062:ValidateArgumentsOfPublicMethods")]
        public Image GetImageFromDocImage(DocumentImage docImage)
        {
            Image img = null;
            foreach (InlineShape inlineShape in Application.ActiveDocument.InlineShapes)
            {
                if (docImage.Equals(CreateDocImage(inlineShape)))
                {
                    img = GetImageFromInlineShape(inlineShape);
                }
            }
            foreach (Word.Shape shape in Application.ActiveDocument.Shapes)
            {
                if (docImage.Equals(CreateDocImage(shape)))
                {
                    img = GetImageFromShape(shape);
                }
            }
            if (img != null)
            {
                Image.GetThumbnailImageAbort dummyCallBack =
                        new System.Drawing.Image.GetThumbnailImageAbort(ThumbnailCallback);

                int thumbSize = 64;
                Image thumbnail = GetProportionedThumbnail(img, thumbSize, dummyCallBack);
                return thumbnail;
            }
            else
            {
                return null;
            }
        }

        /// <summary>
        /// Gets a value indicating whether [current document protected].
        /// </summary>
        /// <value>
        /// 	<c>true</c> if [current document protected]; otherwise, <c>false</c>.
        /// </value>
        public bool CurrentDocumentProtected
        {
            get
            {
                try
                {
                    return Application.ActiveDocument.ProtectionType != WdProtectionType.wdNoProtection;
                }
                catch (COMException)
                {
                    // no document is open
                    return false;
                }
            }
        }

        /// <summary>
        /// Protects the document.
        /// </summary>
        /// <param name="password">The password.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1303:DoNotPassLiteralsAsLocalizedParameters", MessageId = "Microsoft.Office.Interop.Word.Range.InsertAfter(System.String)")]
        public bool ProtectDocument(string password)
        {
            if (this.ActiveDocumentIsNull)
                return false;

            try
            {
                // move to end of document
                object endOfDoc = WdUnits.wdStory;
                object moveToEnd = WdMovementType.wdMove;
                Application.Selection.EndOf(ref endOfDoc, ref moveToEnd);
                Application.Selection.Range.InsertAfter("\n");
                Application.Selection.EndOf(ref endOfDoc, ref moveToEnd);
                endOfDoc = WdUnits.wdTable;
                Application.Selection.EndOf(ref endOfDoc, ref moveToEnd);

                Range range = Application.Selection.Range;
                object noReset = false;
                object passwordString = password;
                object useIRM = false;
                object enforceStyleLock = false;

                this.Application.ActiveDocument.Protect(Word.WdProtectionType.wdAllowOnlyReading,
                    ref noReset, ref passwordString, ref useIRM, ref enforceStyleLock);

                object editorID = Word.WdEditorType.wdEditorEveryone;
                range.Editors.Add(ref editorID);

                return true;
            }
            // what happens if already protected? can we catch something?
            catch (Exception)
            {
                throw;
            }
        }

        /// <summary>
        /// Moves to end of doc.
        /// </summary>
        private void MoveToEndOfDoc()
        {
            // move to end of document
            object endOfDoc = WdUnits.wdStory;
            object moveToEnd = WdMovementType.wdMove;
            Application.Selection.EndOf(ref endOfDoc, ref moveToEnd);
            endOfDoc = WdUnits.wdTable;
            Application.Selection.EndOf(ref endOfDoc, ref moveToEnd);
        }

        /// <summary>
        /// Moves to start of doc.
        /// </summary>
        private void MoveToStartOfDoc()
        {
            // move to end of document
            object unit = WdUnits.wdStory;
            object moveType = WdMovementType.wdMove;
            Application.Selection.StartOf(ref unit, ref moveType);
            unit = WdUnits.wdTable;
            Application.Selection.StartOf(ref unit, ref moveType);
        }

        /// <summary>
        /// Unprotects the document.
        /// </summary>
        /// <param name="password">The password.</param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1303:DoNotPassLiteralsAsLocalizedParameters", MessageId = "System.Windows.Forms.MessageBox.Show(System.String,System.String,System.Windows.Forms.MessageBoxButtons,System.Windows.Forms.MessageBoxIcon)"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1300:SpecifyMessageBoxOptions")]
        public bool UnprotectDocument(string password)
        {
            if (ActiveDocumentIsNull)
                return false;

            try
            {
                object passwordString = password;
                object editorID = Word.WdEditorType.wdEditorEveryone;
                Editor editor = Application.Selection.Range.Editors.Item(ref editorID);


                this.Application.ActiveDocument.Unprotect(ref passwordString);

                if (editor != null)
                {
                    editor.Delete();
                }


                return true;
            }
            // what happens if already protected? can we catch something?
            catch (COMException ex)
            {
                if (ex.Message.Contains("password"))
                {
                    MessageBox.Show("Incorrect password.", "Document Protection", MessageBoxButtons.OK,
                                    MessageBoxIcon.Exclamation);
                    return false;
                }
                else if (ex.Message.Contains("The requested member of the collection does not exist.") && ex.ErrorCode.Equals(-2146822347))
                {
                    object passwordString = password;
                    this.Application.ActiveDocument.Unprotect(ref passwordString);
                    return true;
                }

                else
                {
                    throw;
                }
            }
            catch (Exception)
            {
                throw;
            }
        }

        /// <summary>
        /// Determines whether [is model key exists] [the specified key].
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns>
        /// 	<c>true</c> if [is model key exists] [the specified key]; otherwise, <c>false</c>.
        /// </returns>
        public bool ModelExists(string key)
        {
            if (currentGuids != null)
            {
                IDictionaryEnumerator iter = documentKeyDictionary.GetEnumerator();
                while (iter.MoveNext())
                {
                    if (string.Compare((string)iter.Value, key, true, CultureInfo.InvariantCulture) == 0)
                    {
                        return currentGuids.Contains(new Guid(iter.Key.ToString()));
                    }
                }

            }
            return false;
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is document saved.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is document saved; otherwise, <c>false</c>.
        /// </value>
        public bool IsDocumentSaved
        {
            get { return Application.ActiveDocument.Saved; }
            set { Application.ActiveDocument.Saved = value; }
        }

        /// <summary>
        /// Sets the document saved.
        /// </summary>
        /// <param name="saved">if set to <c>true</c> [saved].</param>
        /// <param name="documentKey">The document key.</param>
        public void SetDocumentSaved(bool saved, string documentKey)
        {
            GetDocumentByDocumentKey(documentKey).Saved = saved;
        }

        /// <summary>
        /// Gets the document using document key.
        /// </summary>
        /// <param name="documentKey">The document key.</param>
        /// <returns></returns>
        public Document GetDocumentByDocumentKey(string documentKey)
        {
            IDictionaryEnumerator ie = documentKeyDictionary.GetEnumerator();
            Guid documentGuid = new Guid();
            while (ie.MoveNext())
            {
                if (string.Compare(documentKey, (string)ie.Value, true, CultureInfo.InvariantCulture) == 0)
                {
                    documentGuid = new Guid(ie.Key.ToString());
                    break;
                }
            }

            foreach (Document document in Application.Documents)
            {
                Guid? docGuid = GuidFromCustomXmlPart(document, false);
                if (docGuid == documentGuid)
                {
                    return document;
                }
            }
            return Application.ActiveDocument;
        }

        /// <summary>
        /// Gets the range.
        /// </summary>
        /// <returns></returns>
        public ImageRange GetRange
        {
            get
            {
                Range rng = Application.Selection.Range;
                ImageRange range = new ImageRange();
                range.Start = rng.Start;
                range.End = rng.End;
                return range;
            }
        }

        /// <summary>
        /// Gets the document count.
        /// </summary>
        /// <value>The document count.</value>
        public int DocumentCount
        {
            get { return Application.Documents.Count; }
        }
    }
}
